package ext {

[Native("Ext.Assert", require)]
/**
 * This class provides help value testing methods useful for diagnostics. These are often
 * used in <code>debugHooks</code>:
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Class', {
 *
 *      debugHooks: {
 *          method: function (a) {
 *              <a href="#!/api/Ext.Assert-method-truthy" rel="Ext.Assert-method-truthy" class="docClass">Ext.Assert.truthy</a>(a, 'Expected "a" to be truthy');
 *          },
 *
 *          foo: function (object) {
 *              Ext.Assert.isFunctionProp(object, 'doSomething');
 *          }
 *      }
 *  });
 * </code></pre>
 *
 * <p><strong>NOTE:</strong> This class is entirely removed in production builds so all uses of it should
 * be either in <code>debug</code> conditional comments or <code>debugHooks</code>.</p>
 *
 * <p>The following type detection methods from the <code>Ext</code> object are wrapped as assertions
 * by this class:</p>
 *
 * <ul>
 * <li><code>isEmpty</code></li>
 * <li><code>isArray</code></li>
 * <li><code>isDate</code></li>
 * <li><code>isObject</code></li>
 * <li><code>isSimpleObject</code></li>
 * <li><code>isPrimitive</code></li>
 * <li><code>isFunction</code></li>
 * <li><code>isNumber</code></li>
 * <li><code>isNumeric</code></li>
 * <li><code>isString</code></li>
 * <li><code>isBoolean</code></li>
 * <li><code>isElement</code></li>
 * <li><code>isTextNode</code></li>
 * <li><code>isDefined</code></li>
 * <li><code>isIterable</code></li>
 * </ul>
 *
 *
 * <p>These appear both their exact name and with a "Prop" suffix for checking a property on
 * an object. For example, these are almost identical:</p>
 *
 * <pre><code> Ext.Assert.isFunction(object.foo);
 *
 *  Ext.Assert.isFunctionProp(object, 'foo');
 * </code></pre>
 *
 * <p>The difference is the default error message generated is better in the second use case
 * than the first.</p>
 *
 * <p>The above list are also expanded for "Not" flavors (and "Not...Prop"):</p>
 *
 * <ul>
 * <li><code>isNotEmpty</code></li>
 * <li><code>isNotArray</code></li>
 * <li><code>isNotDate</code></li>
 * <li><code>isNotObject</code></li>
 * <li><code>isNotSimpleObject</code></li>
 * <li><code>isNotPrimitive</code></li>
 * <li><code>isNotFunction</code></li>
 * <li><code>isNotNumber</code></li>
 * <li><code>isNotNumeric</code></li>
 * <li><code>isNotString</code></li>
 * <li><code>isNotBoolean</code></li>
 * <li><code>isNotElement</code></li>
 * <li><code>isNotTextNode</code></li>
 * <li><code>isNotDefined</code></li>
 * <li><code>isNotIterable</code></li>
 * </ul>
 */
public class Assert {
  /**
   * Checks that the first argument is falsey and throws an <code>Error</code> if it is not.
   */
  public native function falsey(b:Object, msg:Object):void;

  /**
   * Checks that the first argument is falsey and throws an <code>Error</code> if it is not.
   */
  public native function falseyProp(object:Object, property:Object):void;

  /**
   * Checks that the first argument is truthy and throws an <code>Error</code> if it is not.
   */
  public native function truthy(b:Object, msg:Object):void;

  /**
   * Checks that the first argument is truthy and throws an <code>Error</code> if it is not.
   */
  public native function truthyProp(object:Object, property:Object):void;
}
}