package ext {
import ext.container.Container;
import ext.form.field.IField;

[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.Editor", require)]
/**
 * Fires after a change has been made to the field, but before the change is reflected in the underlying
 * field.  Saving the change to the field can be canceled by returning false from the handler of this event.
 * Note that if the value has not changed and ignoreNoChange = true, the editing will still end but this
 * event will not fire since no edit actually occurred.
 * @eventType ext.events.Editor_value_startValueEvent.BEFORE_COMPLETE
 */
[Event(name = 'onBeforeComplete', type = 'ext.events.Editor_value_startValueEvent')]
/**
 * Fires when editing is initiated, but before the value changes.  Editing can be canceled by returning
 * false from the handler of this event.
 * @eventType ext.events.Editor_boundEl_valueEvent.BEFORE_START_EDIT
 */
[Event(name = 'onBeforeStartEdit', type = 'ext.events.Editor_boundEl_valueEvent')]
/**
 * Fires after editing has been canceled and the editor's value has been reset.
 * @eventType ext.events.Editor_value_startValueEvent.CANCEL_EDIT
 */
[Event(name = 'onCancelEdit', type = 'ext.events.Editor_value_startValueEvent')]
/**
 * Fires after editing is complete and any changed value has been written to the underlying field.
 * @eventType ext.events.Editor_value_startValueEvent.COMPLETE
 */
[Event(name = 'onComplete', type = 'ext.events.Editor_value_startValueEvent')]
/**
 * Fires when any key related to navigation (arrows, tab, enter, esc, etc.) is pressed.  You can check
 * <a href="#!/api/Ext.event.Event-method-getKey" rel="Ext.event.Event-method-getKey" class="docClass">Ext.event.Event.getKey</a> to determine which key was pressed.
 * @eventType ext.events.Editor_field_eventEvent.SPECIAL_KEY
 */
[Event(name = 'onSpecialKey', type = 'ext.events.Editor_field_eventEvent')]
/**
 * Fires when this editor is displayed
 * @eventType ext.events.Editor_boundEl_valueEvent.START_EDIT
 */
[Event(name = 'onStartEdit', type = 'ext.events.Editor_boundEl_valueEvent')]
/**
 * The Editor class is used to provide inline editing for elements on the page. The editor
 * is backed by a <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> that will be displayed to edit the underlying content.
 * The editor is a floating Component, when the editor is shown it is automatically aligned to
 * display over the top of the bound element it is editing. The Editor contains several options
 * for how to handle key presses:
 *
 * <ul>
 * <li><a href="#!/api/Ext.Editor-cfg-completeOnEnter" rel="Ext.Editor-cfg-completeOnEnter" class="docClass">completeOnEnter</a></li>
 * <li><a href="#!/api/Ext.Editor-cfg-cancelOnEsc" rel="Ext.Editor-cfg-cancelOnEsc" class="docClass">cancelOnEsc</a></li>
 * <li><a href="#!/api/Ext.Editor-cfg-swallowKeys" rel="Ext.Editor-cfg-swallowKeys" class="docClass">swallowKeys</a></li>
 * </ul>
 *
 *
 * <p>It also has options for how to use the value once the editor has been activated:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.Editor-cfg-revertInvalid" rel="Ext.Editor-cfg-revertInvalid" class="docClass">revertInvalid</a></li>
 * <li><a href="#!/api/Ext.Editor-cfg-ignoreNoChange" rel="Ext.Editor-cfg-ignoreNoChange" class="docClass">ignoreNoChange</a></li>
 * <li><a href="#!/api/Ext.Editor-cfg-updateEl" rel="Ext.Editor-cfg-updateEl" class="docClass">updateEl</a></li>
 * </ul>
 *
 *
 * <p>Sample usage:</p>
 *
 * <pre class='inline-example '><code>var form = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     width: 380,
 *     height: 400,
 *     title: 'User Details',
 *     bodyPadding: 10,
 *     items: [{
 *         html: 'Double-Click on the header title, this, or the field label to edit',
 *         height:30
 *     },{
 *         fieldLabel: 'First Name',
 *         name: 'firstname',
 *         xtype: 'textfield'
 *     }]
 * });
 *
 * var editor = new <a href="#!/api/Ext.Editor" rel="Ext.Editor" class="docClass">Ext.Editor</a>({
 *     // update the innerHTML of the bound element
 *     // when editing completes
 *     updateEl: true,
 *     alignment: 'l-l',
 *     autoSize: {
 *         width: 'boundEl'
 *     },
 *     field: {
 *         xtype: 'textfield'
 *     }
 * });
 *
 * form.header.getTitle().textEl.on('dblclick', function(e, t) {
 *     editor.startEdit(t);
 * });
 *
 * form.getTargetEl().on('dblclick', function(e, t) {
 *     editor.startEdit(t);
 *     // Manually focus, since clicking on the label will focus the text field
 *     editor.field.focus(50, true);
 * });
 * </code></pre>
 *
 * <p><p><img src="" alt="Ext.Editor component" width="" height=""/></p></p>
 */
public class Editor extends ext.container.Container {
  /**
   * @param config @inheritDoc
   */
  public function Editor(config:ext.Editor = null) {
    super();
  }

  [ExtConfig]
  /**
   * True to constrain the editor to the viewport
   * @default false
   */
  override public native function get constrain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set constrain(value:Boolean):void;

  [ExtConfig]
  /**
   * "sides" for sides/bottom only, "frame" for 4-way shadow, and "drop" for bottom-right shadow.
   * @default 'frame'
   */
  override public native function get shadow():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set shadow(value:*):void;

  /**
   * True if this editor is currently active.
   * @default false
   */
  public native function get editing():Boolean;

  [ExtConfig]
  /**
   * The position to align to (see <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">Ext.util.Positionable.alignTo</a> for more details).
   * @default 'c-c?'
   */
  public native function get alignment():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set alignment(value:String):void;

  [ExtConfig]
  /**
   * True to <a href="#!/api/Ext.Editor-method-completeEdit" rel="Ext.Editor-method-completeEdit" class="docClass">complete the editing process</a> if in edit mode when focus exits from this Editor's hierarchy.
   * @default true
   */
  public native function get allowBlur():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowBlur(value:Boolean):void;

  [ExtConfig]
  /**
   * True for the editor to automatically adopt the size of the underlying field. Otherwise, an object
   * can be passed to indicate where to get each dimension. The available properties are 'boundEl' and
   * 'field'. If a dimension is not specified, it will use the underlying height/width specified on
   * the editor object.
   * Examples:
   *
   * <pre><code>autoSize: true // The editor will be sized to the height/width of the field
   *
   * height: 21,
   * autoSize: {
   *     width: 'boundEl' // The width will be determined by the width of the boundEl, the height from the editor (21)
   * }
   *
   * autoSize: {
   *     width: 'field', // Width from the field
   *     height: 'boundEl' // Height from the boundEl
   * }
   * </code></pre>
   */
  public native function get autoSize():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoSize(value:*):void;

  [ExtConfig]
  /**
   * True to cancel the edit when the escape key is pressed.
   * @default true
   */
  public native function get cancelOnEsc():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cancelOnEsc(value:Boolean):void;

  [ExtConfig]
  /**
   * True to complete the edit when the enter key is pressed.
   * @default true
   */
  public native function get completeOnEnter():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set completeOnEnter(value:Boolean):void;

  [ExtConfig]
  /**
   * The Field object (or descendant) or config object for field
   */
  public native function get field():ext.form.field.IField;

  [ExtConfig]
  /**
   * @private
   */
  public native function set field(value:ext.form.field.IField):void;

  [ExtConfig]
  /**
   * False to keep the bound element visible while the editor is displayed
   * @default true
   */
  public native function get hideEl():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideEl(value:Boolean):void;

  [ExtConfig]
  /**
   * True to skip the edit completion process (no save, no events fired) if the user completes an edit and
   * the value has not changed.  Applies only to string values - edits for other data types
   * will never be ignored.
   * @default false
   */
  public native function get ignoreNoChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreNoChange(value:Boolean):void;

  [ExtConfig]
  /**
   * The offsets to use when aligning (see <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">Ext.util.Positionable.alignTo</a> for more details.
   * @default [0, 0]
   */
  public native function get offsets():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set offsets(value:Array):void;

  [ExtConfig]
  /**
   * An element to render to.
   * @default document.body
   */
  public native function get parentEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set parentEl(value:*):void;

  [ExtConfig]
  /**
   * True to automatically revert the field value and cancel the edit when the user completes an edit and the field
   * validation fails
   * @default true
   */
  public native function get revertInvalid():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set revertInvalid(value:Boolean):void;

  [ExtConfig]
  /**
   * Handle the keydown/keypress events so they don't propagate
   * @default true
   */
  public native function get swallowKeys():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set swallowKeys(value:Boolean):void;

  [ExtConfig]
  /**
   * True to update the innerHTML of the bound element when the update completes
   * @default false
   */
  public native function get updateEl():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set updateEl(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The data value of the underlying field
   * @default ''
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Object):void;

  /**
   * Cancels the editing process and hides the editor without persisting any changes.  The field value will be
   * reverted to the original starting value.
   * @param remainVisible Override the default behavior and keep the editor visible after cancel
   * @default false
   */
  public native function cancelEdit(remainVisible:Boolean = false):void;

  /**
   * Ends the editing process, persists the changed value to the underlying field, and hides the editor.
   * @param remainVisible Override the default behavior and keep the editor visible after edit
   * @default false
   */
  public native function completeEdit(remainVisible:Boolean = false):void;

  /**
   * Gets the data value of the editor
   * @return The data value
   */
  public native function getValue():*;

  /**
   * Realigns the editor to the bound field based on the current alignment config value.
   * @param autoSize True to size the field to the dimensions of the bound element.
   */
  public native function realign(autoSize:Boolean = false):void;

  /**
   * Sets the data value of the editor
   * @param value Any valid value supported by the underlying field
   */
  public native function setValue(value:*):void;

  /**
   * Starts the editing process and shows the editor.
   * @param el The element to edit
   * @param value A value to initialize the editor with. If a value is not provided, it defaults
   * to the innerHTML of el.
   */
  public native function startEdit(el:*, value:String = null):void;
}
}