package ext {
import ext.window.MessageBoxWindow;

[Uses("ext.window.MessageBoxWindow")]
[Uses("ext.window.Window")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.MessageBox", require = "Ext.window.MessageBox")]
/**
 * Utility class for generating different styles of message boxes.  The singleton instance, <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.MessageBox</a>
 * alias <code><a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a></code> can also be used.
 *
 * <p>Note that a MessageBox is asynchronous.  Unlike a regular JavaScript <code>alert</code> (which will halt
 * browser execution), showing a MessageBox will not cause the code to stop.  For this reason, if you have code
 * that should only run <em>after</em> some user feedback from the MessageBox, you must use a callback function
 * (see the <code>function</code> parameter for <a href="#!/api/Ext.MessageBox-method-show" rel="Ext.MessageBox-method-show" class="docClass">show</a> for more details).</p>
 *
 * <p>Basic alert</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Status', 'Changes saved successfully.');
 * </code></pre>
 *
 * <p>Prompt for user data and process the result using a callback</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.MessageBox-method-prompt" rel="Ext.MessageBox-method-prompt" class="docClass">Ext.Msg.prompt</a>('Name', 'Please enter your name:', function(btn, text){
 *     if (btn == 'ok'){
 *         // process text value and close...
 *     }
 * });
 * </code></pre>
 *
 * <p>Show a dialog using config options</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.MessageBox-event-show" rel="Ext.MessageBox-event-show" class="docClass">Ext.Msg.show</a>({
 *     title:'Save Changes?',
 *     message: 'You are closing a tab that has unsaved changes. Would you like to save your changes?',
 *     buttons: <a href="#!/api/Ext.MessageBox-property-YESNOCANCEL" rel="Ext.MessageBox-property-YESNOCANCEL" class="docClass">Ext.Msg.YESNOCANCEL</a>,
 *     icon: <a href="#!/api/Ext.MessageBox-property-QUESTION" rel="Ext.MessageBox-property-QUESTION" class="docClass">Ext.Msg.QUESTION</a>,
 *     fn: function(btn) {
 *         if (btn === 'yes') {
 *             console.log('Yes pressed');
 *         } else if (btn === 'no') {
 *             console.log('No pressed');
 *         } else {
 *             console.log('Cancel pressed');
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <p>Showing <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a> while it's already shown will cause the visible instance to be
 * overwritten with the newly passed config.  While this may be the desired outcome, you
 * can also create a new MessageBox that can exist alongside the <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a>
 * singleton instance.</p>
 *
 * <pre class='inline-example '><code>var myMsg = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.MessageBox" rel="Ext.window.MessageBox" class="docClass">Ext.window.MessageBox</a>', {
 *     // set closeAction to 'destroy' if this instance is not
 *     // intended to be reused by the application
 *     closeAction: 'destroy'
 * }).show({
 *     title: 'Custom MessageBox Instance',
 *     message: 'I can exist along with <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a>'
 * });
 *
 * <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Overlapping', '<a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a> instance');
 * </code></pre>
 */
public const MessageBox:ext.window.MessageBoxWindow;
}