package ext {

[Uses("ext.Widget")]
[Uses("ext.ProgressBase")]
[Uses("ext.Base")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.Progress", require)]
/**
 * A simple progress bar widget.
 *
 * <p>You are responsible for showing, updating (via <a href="#!/api/Ext.Progress-method-setValue" rel="Ext.Progress-method-setValue" class="docClass">setValue</a>) and clearing the
 * progress bar as needed from your own code. This method is most appropriate when you
 * want to show progress throughout an operation that has predictable points of interest
 * at which you can update the control.</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name', 'progress'],
 *     data: [
 *         { name: 'Lisa', progress: .159 },
 *         { name: 'Bart', progress: .216 },
 *         { name: 'Homer', progress: .55 },
 *         { name: 'Maggie', progress: .167 },
 *         { name: 'Marge', progress: .145 }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: store,
 *     columns: [
 *         { text: 'Name',  dataIndex: 'name' },
 *         {
 *             text: 'Progress',
 *             xtype: 'widgetcolumn',
 *             width: 120,
 *             dataIndex: 'progress',
 *             widget: {
 *                 xtype: 'progress'
 *             }
 *         }
 *     ],
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class Progress extends ext.Widget implements ext.IProgressBase {
  /**
   * @param config @inheritDoc
   */
  public function Progress(config:ext.Progress = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The base CSS class to apply to the progress bar's wrapper element.
   * @default 'x-progress'
   * @see #getBaseCls()
   * @see #setBaseCls()
   */
  override public native function get baseCls():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set baseCls(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specify as <code>true</code> to have this progress bar animate to new extent when updated.
   * @default false
   * @see #getAnimate()
   * @see #setAnimate()
   */
  public native function get animated():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set animate(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The background text
   * @see #getText()
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A template used to create this ProgressBar's background text given two values:
   *
   * <p>   `value  ' - The raw progress value between 0 and 1
   *    'percent' - The value as a percentage between 0 and 100</p>
   * @see #getTextTpl()
   * @see #setTextTpl()
   */
  public native function get textTpl():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set textTpl(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A floating point value between 0 and 1 (e.g., .5)
   * @default 0
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Number):void;

  /**
   * Returns the value of <a href="#!/api/Ext.Progress-cfg-animate" rel="Ext.Progress-cfg-animate" class="docClass">animate</a>.
   */
  public native function getAnimate():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.Progress-cfg-text" rel="Ext.Progress-cfg-text" class="docClass">text</a>.
   */
  public native function getText():String;

  /**
   * Returns the value of <a href="#!/api/Ext.ProgressBase-cfg-textTpl" rel="Ext.ProgressBase-cfg-textTpl" class="docClass">textTpl</a>.
   */
  public native function getTextTpl():*;

  /**
   * Returns the value of <a href="#!/api/Ext.ProgressBase-cfg-value" rel="Ext.ProgressBase-cfg-value" class="docClass">value</a>.
   */
  public native function getValue():Number;

  /**
   * Sets the value of <a href="#!/api/Ext.Progress-cfg-animate" rel="Ext.Progress-cfg-animate" class="docClass">animate</a>.
   * @param animate The new value.
   */
  public native function setAnimate(animate:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Progress-cfg-text" rel="Ext.Progress-cfg-text" class="docClass">text</a>.
   * @param text The new value.
   */
  public native function setText(text:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.ProgressBase-cfg-textTpl" rel="Ext.ProgressBase-cfg-textTpl" class="docClass">textTpl</a>.
   * @param textTpl The new value.
   */
  public native function setTextTpl(textTpl:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.ProgressBase-cfg-value" rel="Ext.ProgressBase-cfg-value" class="docClass">value</a>.
   * @param value The new value.
   */
  public native function setValue(value:Number):void;
}
}