package ext {

[Uses("ext.Component")]
[Uses("ext.ProgressBase")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.ProgressBar", require)]
/**
 * Fires after each update interval
 * @eventType ext.events.ProgressBar_value_textEvent.UPDATE
 */
[Event(name = 'onUpdate', type = 'ext.events.ProgressBar_value_textEvent')]
/**
 * An updateable progress bar component. The progress bar supports two different modes: manual and automatic.
 *
 * <p>In manual mode, you are responsible for showing, updating (via <a href="#!/api/Ext.ProgressBar-method-updateProgress" rel="Ext.ProgressBar-method-updateProgress" class="docClass">updateProgress</a>) and clearing the progress bar
 * as needed from your own code. This method is most appropriate when you want to show progress throughout an operation
 * that has predictable points of interest at which you can update the control.</p>
 *
 * <p>In automatic mode, you simply call <a href="#!/api/Ext.ProgressBar-method-wait" rel="Ext.ProgressBar-method-wait" class="docClass">wait</a> and let the progress bar run indefinitely, only clearing it once the
 * operation is complete. You can optionally have the progress bar wait for a specific amount of time and then clear
 * itself. Automatic mode is most appropriate for timed operations or asynchronous operations in which you have no need
 * for indicating intermediate progress.</p>
 *
 * <pre class='inline-example '><code>var p = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.ProgressBar" rel="Ext.ProgressBar" class="docClass">Ext.ProgressBar</a>', {
 *    renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *    width: 300
 * });
 *
 * // Wait for 5 seconds, then update the status el (progress bar will auto-reset)
 * p.wait({
 *     interval: 500, //bar will move fast!
 *     duration: 50000,
 *     increment: 15,
 *     text: 'Updating...',
 *     scope: this,
 *     fn: function(){
 *         p.updateText('Done!');
 *     }
 * });
 * </code></pre>
 */
public class ProgressBar extends ext.Component implements ext.IProgressBase {
  /**
   * @param config @inheritDoc
   */
  public function ProgressBar(config:ext.ProgressBar = null) {
    super();
  }

  [ExtConfig]
  /**
   * The base CSS class to apply to the progress bar's wrapper element.
   * @default 'x-progress'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The progress bar element's id (defaults to an auto-generated id)
   * @see #getId()
   */
  override public native function get id():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set id(value:String):void;

  [ExtConfig]
  /**
   * True to animate the progress bar during transitions, or an animation configuration
   * (see the <a href="#!/api/Ext.ProgressBar-method-animate" rel="Ext.ProgressBar-method-animate" class="docClass">animate</a> method for details).
   * @default false
   */
  public native function get animated():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animated(value:*):void;

  [ExtConfig]
  /**
   * The text shown in the progress bar.
   * @default ''
   */
  public native function get text():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  /**
   * The element to render the progress text to (defaults to the progress bar's internal text element)
   */
  public native function get textEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set textEl(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A template used to create this ProgressBar's background text given two values:
   *
   * <p>   `value  ' - The raw progress value between 0 and 1
   *    'percent' - The value as a percentage between 0 and 100</p>
   * @see #getTextTpl()
   * @see #setTextTpl()
   */
  public native function get textTpl():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set textTpl(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A floating point value between 0 and 1 (e.g., .5)
   * @default 0
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Number):void;

  /**
   * Returns the value of <a href="#!/api/Ext.ProgressBase-cfg-textTpl" rel="Ext.ProgressBase-cfg-textTpl" class="docClass">textTpl</a>.
   */
  public native function getTextTpl():*;

  /**
   * Returns the value of <a href="#!/api/Ext.ProgressBase-cfg-value" rel="Ext.ProgressBase-cfg-value" class="docClass">value</a>.
   */
  public native function getValue():Number;

  /**
   * Returns true if the progress bar is currently in a <a href="#!/api/Ext.ProgressBar-method-wait" rel="Ext.ProgressBar-method-wait" class="docClass">wait</a> operation
   * @return True if waiting, else false
   */
  public native function isWaiting():Boolean;

  /**
   * Resets the progress bar value to 0 and text to empty string. If hide = true, the progress bar will also be hidden
   * (using the <a href="#!/api/Ext.ProgressBar-cfg-hideMode" rel="Ext.ProgressBar-cfg-hideMode" class="docClass">hideMode</a> property internally).
   * @param hide True to hide the progress bar.
   * @default false
   * @return this
   */
  public native function reset(hide:Boolean = false):ext.ProgressBar;

  /**
   * Sets the value of <a href="#!/api/Ext.ProgressBase-cfg-textTpl" rel="Ext.ProgressBase-cfg-textTpl" class="docClass">textTpl</a>.
   * @param textTpl The new value.
   */
  public native function setTextTpl(textTpl:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.ProgressBase-cfg-value" rel="Ext.ProgressBase-cfg-value" class="docClass">value</a>.
   * @param value The new value.
   */
  public native function setValue(value:Number):void;

  /**
   * Updates the progress bar value, and optionally its text.
   *
   * <p>If the text argument is not specified, then the <a href="#!/api/Ext.ProgressBar-cfg-textTpl" rel="Ext.ProgressBar-cfg-textTpl" class="docClass">textTpl</a> will be used to generate the text.
   * If there is no <code>textTpl</code>, any existing text value will be unchanged. To blank out existing text, pass <code>""</code>.</p>
   *
   * <p>Note that even if the progress bar value exceeds 1, it will never automatically reset --
   * you are responsible for determining when the progress is complete and
   * calling <a href="#!/api/Ext.ProgressBar-method-reset" rel="Ext.ProgressBar-method-reset" class="docClass">reset</a> to clear and/or hide the control.</p>
   * @param value A floating point value between 0 and 1 (e.g., .5)
   * @default 0
   * @param text The string to display in the progress text element
   * @default ''
   * @param animate Whether to animate the transition of the progress bar. If this value is not
   * specified, the default for the class is used
   * @default false
   * @return this
   */
  public native function updateProgress(value:Number = 0, text:String = '', animate:Boolean = false):ext.ProgressBar;

  /**
   * Updates the progress bar text. If specified, textEl will be updated, otherwise the progress bar itself will
   * display the updated text.
   * @param text The string to display in the progress text element
   * @default ''
   * @return this
   */
  public native function updateText(text:String = ''):ext.ProgressBar;

  /**
   * Initiates an auto-updating progress bar. A duration can be specified, in which case the progress bar will
   * automatically reset after a fixed amount of time and optionally call a callback function if specified. If no
   * duration is passed in, then the progress bar will run indefinitely and must be manually cleared by calling
   * <a href="#!/api/Ext.ProgressBar-method-reset" rel="Ext.ProgressBar-method-reset" class="docClass">reset</a>.
   *
   * <p>Example usage:</p>
   *
   * <pre><code>var p = new <a href="#!/api/Ext.ProgressBar" rel="Ext.ProgressBar" class="docClass">Ext.ProgressBar</a>({
   *    renderTo: 'my-el'
   * });
   *
   * //Wait for 5 seconds, then update the status el (progress bar will auto-reset)
   * var p = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.ProgressBar" rel="Ext.ProgressBar" class="docClass">Ext.ProgressBar</a>', {
   *    renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *    width: 300
   * });
   *
   * //Wait for 5 seconds, then update the status el (progress bar will auto-reset)
   * p.wait({
   *    interval: 500, //bar will move fast!
   *    duration: 50000,
   *    increment: 15,
   *    text: 'Updating...',
   *    scope: this,
   *    fn: function(){
   *       p.updateText('Done!');
   *    }
   * });
   *
   * //Or update indefinitely until some async action completes, then reset manually
   * p.wait();
   * myAction.on('complete', function(){
   *     p.reset();
   *     p.updateText('Done!');
   * });
   * </code></pre>
   * @param config Configuration options
   * @param {Number} config.duration The length of time in milliseconds that the progress bar should
   * run before resetting itself (defaults to undefined, in which case it will run indefinitely
   * until reset is called)
   * @param {Number} config.interval The length of time in milliseconds between each progress update
   * (defaults to 1000 ms)
   * @param {Boolean} config.animate Whether to animate the transition of the progress bar. If this
   * value is not specified, the default for the class is used.
   * @param {Number} config.increment The number of progress update segments to display within the
   * progress bar (defaults to 10).  If the bar reaches the end and is still updating, it will
   * automatically wrap back to the beginning.
   * @param {String} config.text Optional text to display in the progress bar element (defaults to '').
   * @param {Function} config.fn A callback function to execute after the progress bar finishes auto-
   * updating.  The function will be called with no arguments.  This function will be ignored if
   * duration is not specified since in that case the progress bar can only be stopped programmatically,
   * so any required function should be called by the same code after it resets the progress bar.
   * @param {Object} config.scope The scope that is passed to the callback function (only applies when
   * duration and fn are both passed).
   * @return this
   */
  public native function wait(config:Object = null):ext.ProgressBar;
}
}