package ext {

[Uses("ext.Base")]
[Native]
/**
 * Provides a registry of all Components (instances of <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> or any subclass
 * thereof) on a page so that they can be easily accessed by <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">component</a>
 * <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a> (see <a href="#!/api/Ext.ComponentManager-method-get" rel="Ext.ComponentManager-method-get" class="docClass">get</a>, or the convenience method
 * <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>).
 *
 * <p>This object also provides a registry of available Component <em>classes</em> indexed by a
 * mnemonic code known as the Component's <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a>. The <code>xtype</code>
 * provides a way to avoid instantiating child Components when creating a full, nested
 * config object for a complete Ext page.</p>
 *
 * <p>A child Component may be specified simply as a <em>config object</em> as long as the correct
 * <code><a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a></code> is specified so that if and when the Component
 * needs rendering, the correct type can be looked up for lazy instantiation.</p>
 * <p>Type of singleton ComponentManager.</p>
 * @see ext.#ComponentManager ext.ComponentManager
 */
public class SComponentManager extends ext.Base {
  /**
   * Creates a new Component from the specified config object using the config object's
   * <code>xtype</code> to determine the class to instantiate.
   * @param config A configuration object for the Component you wish to create.
   * @param defaultType The <code>xtype</code> to use if the config object does not
   * contain a <code>xtype</code>. (Optional if the config contains a <code>xtype</code>).
   * @return The newly instantiated Component.
   */
  public native function create(config:Object, defaultType:String = null):ext.Component;

  /**
   * Executes the specified function once for each item in the collection.
   * @param fn The function to execute.
   * @param {String} fn.key The key of the item
   * @param {Number} fn.value The value of the item
   * @param {Number} fn.length The total number of items in the collection <strong> Removed
   * in 5.0 </strong>
   * @param {Boolean} fn.return False to cease iteration.
   * @param scope The scope to execute in. Defaults to <code>this</code>.
   */
  public native function each(fn:Function, scope:Object):void;

  /**
   * Returns an item by id.
   * @param id The id of the item
   * @return The item, undefined if not found.
   */
  public native function get(id:String):ext.Component;

  /**
   * Returns an array of all components
   */
  public native function getAll():Array;

  /**
   * Gets the number of items in the collection.
   * @return The number of items in the collection.
   */
  public native function getCount():Number;

  [Deprecated(since = "5.0")]
  /**
   * Checks if an item is registered.
   * @param component The mnemonic string by which the class may be looked up.
   * @return Whether the type is registered.
   */
  public native function isRegistered(component:String):Boolean;

  /**
   * Registers a function that will be called (a single time) when an item with the specified id is added to the manager.
   * This will happen on instantiation.
   * @param id The item id
   * @param fn The callback function. Called with a single parameter, the item.
   * @param scope The scope ('this' reference) in which the callback is executed.
   * Defaults to the item.
   */
  public native function onAvailable(id:String, fn:Function, scope:Object = null):void;

  [Deprecated(since = "5.0")]
  /**
   * Registers a new item constructor, keyed by a type key.
   * @param type The mnemonic string by which the class may be looked up.
   * @param cls The new instance class.
   */
  public native function registerType(type:String, cls:Function):void;
}
}