package ext {
import ext.env.OS;

[Uses("ext.env.OS")]
[Native]
/**
 * Provides useful information about the current operating system environment.
 *
 * <p>Example:</p>
 *
 * <pre><code>if (Ext.os.is.Windows) {
 *     // Windows specific code here
 * }
 *
 * if (Ext.os.is.iOS) {
 *     // iPad, iPod, iPhone, etc.
 * }
 *
 * console.log("Version " + <a href="#!/api/Ext.os-property-version" rel="Ext.os-property-version" class="docClass">Ext.os.version</a>);
 * </code></pre>
 *
 * <p>For a full list of supported values, refer to the <a href="#!/api/Ext.os-method-is" rel="Ext.os-method-is" class="docClass">is</a> property/method.</p>
 * <p>Type of singleton os.</p>
 * @see ext.#os ext.os
 */
public class Sos extends ext.env.OS {
  /**
   * The generic type of the current device.
   *
   * <p>Possible values:</p>
   *
   * <ul>
   * <li>Phone</li>
   * <li>Tablet</li>
   * <li>Desktop</li>
   * </ul>
   *
   *
   * <p>For testing purposes the deviceType can be overridden by adding
   * a deviceType parameter to the URL of the page, like so:</p>
   *
   * <pre><code>http://localhost/mypage.html?deviceType=Tablet
   * </code></pre>
   */
  public native function get deviceType():String;

  /**
   * @private
   */
  public native function set deviceType(value:String):void;

  /**
   * The full name of the current operating system. Possible values are:
   *
   * <ul>
   * <li>iOS</li>
   * <li>Android</li>
   * <li>WebOS</li>
   * <li>BlackBerry,</li>
   * <li>MacOS</li>
   * <li>Windows</li>
   * <li>Linux</li>
   * <li>Other</li>
   * </ul>
   * @default null
   */
  public native function get name():String;

  /**
   * Refer to <a href="#!/api/Ext.Version" rel="Ext.Version" class="docClass">Ext.Version</a>
   * @default null
   */
  public native function get version():ext.Version;

  /**
   * A "hybrid" property, can be either accessed as a method call, i.e:
   *
   * <pre><code>if (<a href="#!/api/Ext.os-method-is" rel="Ext.os-method-is" class="docClass">Ext.os.is</a>('Android')) {
   *     // ...
   * }
   * </code></pre>
   *
   * <p>or as an object with boolean properties, i.e:</p>
   *
   * <pre><code>if (Ext.os.is.Android) {
   *     // ...
   * }
   * </code></pre>
   *
   * <p>Versions can be conveniently checked as well. For example:</p>
   *
   * <pre><code>if (Ext.os.is.Android2) {
   *     // Equivalent to (Ext.os.is.Android &amp;&amp; Ext.os.version.equals(2))
   * }
   *
   * if (Ext.os.is.iOS32) {
   *     // Equivalent to (Ext.os.is.iOS &amp;&amp; Ext.os.version.equals(3.2))
   * }
   * </code></pre>
   *
   * <p>Note that only <a href="#!/api/Ext.Version-method-getMajor" rel="Ext.Version-method-getMajor" class="docClass">major component</a> and <a href="#!/api/Ext.Version-method-getShortVersion" rel="Ext.Version-method-getShortVersion" class="docClass">simplified</a>
   * value of the version are available via direct property checking. Supported values are:</p>
   *
   * <ul>
   * <li>iOS</li>
   * <li>iPad</li>
   * <li>iPhone</li>
   * <li>iPhone5 (also true for 4in iPods).</li>
   * <li>iPod</li>
   * <li>Android</li>
   * <li>WebOS</li>
   * <li>BlackBerry</li>
   * <li>Bada</li>
   * <li>MacOS</li>
   * <li>Windows</li>
   * <li>Linux</li>
   * <li>Other</li>
   * </ul>
   * @param name The OS name to check.
   */
  public native function matches(name:String):Boolean;
}
}