package ext.app.bind {

[Uses("ext.app.bind.BaseBinding")]
[Uses("ext.util.Schedulable")]
[Uses("ext.Base")]
[Native("Ext.app.bind.Binding", require)]
/**
 * This class is created to manage a direct bind.  <code><a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a></code> returns this from
 * its <a href="#!/api/Ext.app.ViewModel-method-bind" rel="Ext.app.ViewModel-method-bind" class="docClass">bind</a> method.
 */
public class Binding extends ext.app.bind.BaseBinding {
  /**
   * @param config @inheritDoc
   */
  public function Binding(config:ext.app.bind.Binding = null) {
    super();
  }

  [ExtConfig]
  /**
   * Normally a binding is only notified of changes to its bound property, but if that
   * property is an object it is sometimes helpful to be notified of changes to its
   * properties. To receive notifications of changes to all properties of a bound object,
   * set this to <code>true</code>.
   * @default false
   * @since 5.0.0
   */
  public native function get deep():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set deep(value:Boolean):void;

  /**
   * Destroys this binding. No further calls will be made to the callback method. No
   * methods should be called on this binding after calling this method.
   * @since 5.0.0
   */
  override public native function destroy(...params):void;

  /**
   * Binds to the <code>validation</code> association for the bound property. For example, when a
   * binding is bound to something like this:
   *
   * <pre><code> var binding = viewModel.bind('{theUser.name}', ...);
   * </code></pre>
   *
   * <p>The validation status for the "name" property can be requested like so:</p>
   *
   * <pre><code> var validationBinding = binding.bindValidation(fn, scope);
   * </code></pre>
   *
   * <p>Calling this method in the above example would be equivalent to the following bind:</p>
   *
   * <pre><code> var validationBinding = viewModel.bind('{theUser.validation.name}', fn, scope);
   * </code></pre>
   *
   * <p>The primary reason to use this method is in cases where the original bind expression
   * is not known.</p>
   *
   * <p>For example, this method is used by <code><a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a></code> when given the
   * <code><a href="#!/api/Ext.Component-cfg-modelValidation" rel="Ext.Component-cfg-modelValidation" class="docClass">modelValidation</a></code> config is set. As such it
   * not common for users to need to call this method.</p>
   * @since 5.0.0
   * @param callback The function to call when the validation changes.
   * @param scope The scope on which to call the <code>callback</code>.
   * @return A binding to the validation of the bound property.
   */
  public native function bindValidation(callback:Function, scope:Object = null):ext.app.bind.Binding;

  /**
   * Returns the diagnostic name for this binding.
   * @since 5.0.0
   */
  public native function getFullName():String;

  /**
   * Returns the current value of the bound property. If this binding <code>isLoading</code> this
   * value will be <code>undefined</code>.
   * @since 5.0.0
   * @return The value of the bound property.
   */
  public native function getValue():*;

  /**
   * Returns <code>true</code> if the bound property is loading. In the general case this means
   * that the value is just not available yet. In specific cases, when the bound property
   * is an <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code> it means that a request to the server is in progress to get
   * the record. For an <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code> it means that
   * <code><a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a></code> has been called on the store but it is
   * still in progress.
   * @since 5.0.0
   */
  public native function isLoading():Boolean;

  /**
   * This method returns <code>true</code> if this binding can only be read. If this method returns
   * <code>false</code> then the binding can be set using <code>setValue</code> (meaning this binding can be
   * a two-way binding).
   * @since 5.0.0
   */
  public native function isReadOnly():Boolean;

  /**
   * Tells the bound property to refresh itself. This has meaning when the bound property
   * is something like an <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code> and an <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code> but does nothing in
   * most cases.
   * @since 5.0.0
   */
  public native function refresh():void;

  /**
   * Sets the value of the bound property. This will throw an error in debug mode if
   * this binding <code>isReadOnly</code>.
   * @since 5.0.0
   * @param value The new value.
   */
  public native function setValue(value:*):void;
}
}