package ext.button {
import ext.Component;
import ext.Template;
import ext.event.Event;
import ext.menu.Menu;
import ext.mixin.IQueryable;

[Uses("ext.Component")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.button.Button", require)]
/**
 * Fires before the 'pressed' state of this button changes (only if enableToggle = true)
 * If a handler returns <code>false</code>, the toggle is vetoed.
 * @eventType ext.button.events.Button_pressedEvent.BEFORE_TOGGLE
 */
[Event(name = 'onBeforeToggle', type = 'ext.button.events.Button_pressedEvent')]
/**
 * Fires when this button is clicked, before the configured <a href="#!/api/Ext.button.Button-cfg-handler" rel="Ext.button.Button-cfg-handler" class="docClass">handler</a> is invoked. Execution of the
 * <a href="#!/api/Ext.button.Button-cfg-handler" rel="Ext.button.Button-cfg-handler" class="docClass">handler</a> may be vetoed by returning <code>false</code> to this event.
 * @eventType ext.button.events.Button_eEvent.CLICK
 */
[Event(name = 'onClick', type = 'ext.button.events.Button_eEvent')]
/**
 * Fired when the button's glyph is changed by the <a href="#!/api/Ext.button.Button-method-setGlyph" rel="Ext.button.Button-method-setGlyph" class="docClass">setGlyph</a> method.
 * @eventType ext.button.events.Button_newGlyph_oldGlyphEvent.GLYPH_CHANGE
 */
[Event(name = 'onGlyphChange', type = 'ext.button.events.Button_newGlyph_oldGlyphEvent')]
/**
 * Fired when the button's icon is changed by the <a href="#!/api/Ext.button.Button-method-setIcon" rel="Ext.button.Button-method-setIcon" class="docClass">setIcon</a> or <a href="#!/api/Ext.button.Button-method-setIconCls" rel="Ext.button.Button-method-setIconCls" class="docClass">setIconCls</a> methods.
 * @eventType ext.button.events.Button_oldIcon_newIconEvent.ICON_CHANGE
 */
[Event(name = 'onIconChange', type = 'ext.button.events.Button_oldIcon_newIconEvent')]
/**
 * If this button has a menu, this event fires when it is hidden
 * @eventType ext.button.events.Button_menuEvent.MENU_HIDE
 */
[Event(name = 'onMenuHide', type = 'ext.button.events.Button_menuEvent')]
/**
 * If this button has a menu, this event fires when it is shown
 * @eventType ext.button.events.Button_menuEvent.MENU_SHOW
 */
[Event(name = 'onMenuShow', type = 'ext.button.events.Button_menuEvent')]
/**
 * If this button has a menu, this event fires when the mouse leaves the menu triggering element
 * @eventType ext.button.events.Button_menu_eEvent.MENU_TRIGGER_OUT
 */
[Event(name = 'onMenuTriggerOut', type = 'ext.button.events.Button_menu_eEvent')]
/**
 * If this button has a menu, this event fires when the mouse enters the menu triggering element
 * @eventType ext.button.events.Button_menu_eEvent.MENU_TRIGGER_OVER
 */
[Event(name = 'onMenuTriggerOver', type = 'ext.button.events.Button_menu_eEvent')]
/**
 * Fires when the mouse exits the button
 * @eventType ext.button.events.Button_eEvent.MOUSE_OUT
 */
[Event(name = 'onMouseOut', type = 'ext.button.events.Button_eEvent')]
/**
 * Fires when the mouse hovers over the button
 * @eventType ext.button.events.Button_eEvent.MOUSE_OVER
 */
[Event(name = 'onMouseOver', type = 'ext.button.events.Button_eEvent')]
/**
 * Fired when the button's text is changed by the <a href="#!/api/Ext.button.Button-method-setText" rel="Ext.button.Button-method-setText" class="docClass">setText</a> method.
 * @eventType ext.button.events.Button_oldText_newTextEvent.TEXT_CHANGE
 */
[Event(name = 'onTextChange', type = 'ext.button.events.Button_oldText_newTextEvent')]
/**
 * Fires when the 'pressed' state of this button changes (only if enableToggle = true)
 * @eventType ext.button.events.Button_pressedEvent.TOGGLE
 */
[Event(name = 'onToggle', type = 'ext.button.events.Button_pressedEvent')]
/**
 * Create simple buttons with this component. Customizations include <a href="#!/api/Ext.button.Button-cfg-iconAlign" rel="Ext.button.Button-cfg-iconAlign" class="docClass">aligned</a>
 * <a href="#!/api/Ext.button.Button-cfg-iconCls" rel="Ext.button.Button-cfg-iconCls" class="docClass">icons</a>, <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">dropdown menus</a>, <a href="#!/api/Ext.button.Button-cfg-tooltip" rel="Ext.button.Button-cfg-tooltip" class="docClass">tooltips</a>
 * and <a href="#!/api/Ext.button.Button-cfg-scale" rel="Ext.button.Button-cfg-scale" class="docClass">sizing options</a>. Specify a <a href="#!/api/Ext.button.Button-cfg-handler" rel="Ext.button.Button-cfg-handler" class="docClass">handler</a> to run code when
 * a user clicks the button, or use <a href="#!/api/Ext.button.Button-cfg-listeners" rel="Ext.button.Button-cfg-listeners" class="docClass">listeners</a> for other events such as
 * <a href="#!/api/Ext.button.Button-event-mouseover" rel="Ext.button.Button-event-mouseover" class="docClass">mouseover</a>. Example usage:
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *     text: 'Click me',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     handler: function() {
 *         alert('You clicked the button!');
 *     }
 * });
 * </code></pre>
 *
 * <p>The <a href="#!/api/Ext.button.Button-cfg-handler" rel="Ext.button.Button-cfg-handler" class="docClass">handler</a> configuration can also be updated dynamically using the <a href="#!/api/Ext.button.Button-method-setHandler" rel="Ext.button.Button-method-setHandler" class="docClass">setHandler</a>
 * method.  Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *     text    : 'Dynamic Handler Button',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     handler : function() {
 *         // this button will spit out a different number every time you click it.
 *         // so firstly we must check if that number is already set:
 *         if (this.clickCount) {
 *             // looks like the property is already set, so lets just add 1 to that number and alert the user
 *             this.clickCount++;
 *             alert('You have clicked the button "' + this.clickCount + '" times.\n\nTry clicking it again..');
 *         } else {
 *             // if the clickCount property is not set, we will set it and alert the user
 *             this.clickCount = 1;
 *             alert('You just clicked the button for the first time!\n\nTry pressing it again..');
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <p>A button within a container:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.Container</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items   : [
 *         {
 *             xtype: 'button',
 *             text : 'My Button'
 *         }
 *     ]
 * });
 * </code></pre>
 *
 * <p>A useful option of Button is the <a href="#!/api/Ext.button.Button-cfg-scale" rel="Ext.button.Button-cfg-scale" class="docClass">scale</a> configuration. This configuration has three different options:</p>
 *
 * <ul>
 * <li><code>'small'</code></li>
 * <li><code>'medium'</code></li>
 * <li><code>'large'</code></li>
 * </ul>
 *
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *     renderTo: document.body,
 *     text    : 'Click me',
 *     scale   : 'large'
 * });
 * </code></pre>
 *
 * <p>Buttons can also be toggled. To enable this, you simple set the <a href="#!/api/Ext.button.Button-cfg-enableToggle" rel="Ext.button.Button-cfg-enableToggle" class="docClass">enableToggle</a> property to <code>true</code>.
 * Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     text: 'Click Me',
 *     enableToggle: true
 * });
 * </code></pre>
 *
 * <p>You can assign a menu to a button by using the <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a> configuration. This standard configuration
 * can either be a reference to a <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">menu</a> object, a <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">menu</a> id or a
 * <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">menu</a> config blob. When assigning a menu to a button, an arrow is automatically
 * added to the button.  You can change the alignment of the arrow using the <a href="#!/api/Ext.button.Button-cfg-arrowAlign" rel="Ext.button.Button-cfg-arrowAlign" class="docClass">arrowAlign</a> configuration
 * on button.  Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *     text      : 'Menu button',
 *     renderTo  : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     arrowAlign: 'bottom',
 *     menu      : [
 *         {text: 'Item 1'},
 *         {text: 'Item 2'},
 *         {text: 'Item 3'},
 *         {text: 'Item 4'}
 *     ]
 * });
 * </code></pre>
 *
 * <p>Using listeners, you can easily listen to events fired by any component, using the <a href="#!/api/Ext.button.Button-cfg-listeners" rel="Ext.button.Button-cfg-listeners" class="docClass">listeners</a>
 * configuration or using the <a href="#!/api/Ext.button.Button-method-addListener" rel="Ext.button.Button-method-addListener" class="docClass">addListener</a> method.  Button has a variety of different listeners:</p>
 *
 * <ul>
 * <li><code>click</code></li>
 * <li><code>toggle</code></li>
 * <li><code>mouseover</code></li>
 * <li><code>mouseout</code></li>
 * <li><code>mouseshow</code></li>
 * <li><code>menuhide</code></li>
 * <li><code>menutriggerover</code></li>
 * <li><code>menutriggerout</code></li>
 * </ul>
 *
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *     text     : 'Button',
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     listeners: {
 *         click: function() {
 *             // this == the button, as we are in the local scope
 *             this.setText('I was clicked!');
 *         },
 *         mouseover: function() {
 *             // set a new config which says we moused over, if not already set
 *             if (!this.mousedOver) {
 *                 this.mousedOver = true;
 *                 alert('You moused over a button!\n\nI wont do this again.');
 *             }
 *         }
 *     }
 * });
 * </code></pre>
 */
public class Button extends ext.Component implements ext.mixin.IQueryable {
  /**
   * @param config @inheritDoc
   */
  public function Button(config:ext.button.Button = null) {
    super();
  }

  [ExtConfig]
  /**
   * The base CSS class to add to all buttons.
   * @default 'x-btn'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  /**
   * A CSS class string to apply to the button's main element.
   */
  override public native function get cls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set cls(value:String):void;

  [ExtConfig]
  /**
   * True if this button is disabled.
   * @default false
   * @see #setDisabled()
   */
  override public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * True if this button is hidden.
   * @default false
   * @see #setHidden()
   */
  override public native function get hidden():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set hidden(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum width for this button (used to give a set of buttons a common width).
   * See also <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>.<a href="#!/api/Ext.panel.Panel-cfg-minButtonWidth" rel="Ext.panel.Panel-cfg-minButtonWidth" class="docClass">minButtonWidth</a>.
   * @see #getMinWidth()
   * @see #setMinWidth()
   */
  override public native function get minWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set minWidth(value:Number):void;

  [ExtConfig]
  /**
   * Sets a DOM tabIndex for this button. tabIndex may be set to <code>-1</code> in order to remove
   * the button from the tab rotation.
   * @default 0
   * @see #getTabIndex()
   * @see #setTabIndex()
   */
  override public native function get tabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tabIndex(value:Number):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Button, or subclass thereof.
   * @default true
   */
  public native function get isButton():Boolean;

  /**
   * @private
   */
  public native function set isButton(value:Boolean):void;

  /**
   * A <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Template</a> used to create the Button's DOM structure.
   *
   * <p>Instances, or subclasses which need a different DOM structure may provide a different template layout in
   * conjunction with an implementation of <a href="#!/api/Ext.button.Button-method-getTemplateArgs" rel="Ext.button.Button-method-getTemplateArgs" class="docClass">getTemplateArgs</a>.</p>
   */
  public native function get template():ext.Template;

  /**
   * @private
   */
  public native function set template(value:ext.Template):void;

  [ExtConfig]
  /**
   * False to not allow a pressed Button to be depressed. Only valid when <a href="#!/api/Ext.button.Button-cfg-enableToggle" rel="Ext.button.Button-cfg-enableToggle" class="docClass">enableToggle</a> is true.
   * @default true
   */
  public native function get allowDepress():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowDepress(value:Boolean):void;

  [ExtConfig]
  /**
   * The side of the Button box to render the arrow if the button has an associated <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a>. Two
   * values are allowed:
   *
   * <ul>
   * <li>'right'</li>
   * <li>'bottom'</li>
   * </ul>
   * @default 'right'
   */
  public native function get arrowAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set arrowAlign(value:String):void;

  [ExtConfig]
  /**
   * The className used for the inner arrow element if the button has a menu.
   * @default 'arrow'
   */
  public native function get arrowCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set arrowCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>false</code> to hide the button arrow.  Only applicable for <a href="#!/api/Ext.button.Split" rel="Ext.button.Split" class="docClass">Split Buttons</a> and buttons configured with a <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a>.
   * @default true
   * @see #getArrowVisible()
   * @see #setArrowVisible()
   */
  public native function get arrowVisible():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set arrowVisible(value:Boolean):void;

  [ExtConfig]
  /**
   * An object literal of parameters to pass to the url when the <a href="#!/api/Ext.button.Button-cfg-href" rel="Ext.button.Button-cfg-href" class="docClass">href</a> property is specified.
   */
  public native function get baseParams():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseParams(value:Object):void;

  [ExtConfig]
  /**
   * The DOM event that will fire the handler of the button. This can be any valid event name (dblclick, contextmenu).
   * @default 'click'
   */
  public native function get clickEvent():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clickEvent(value:String):void;

  [ExtConfig]
  /**
   * Whether or not to destroy any associated menu when this button is destroyed.
   * In addition, a value of <code>true</code> for this config will destroy the currently bound menu when a new
   * menu is set in <a href="#!/api/Ext.button.Button-method-setMenu" rel="Ext.button.Button-method-setMenu" class="docClass">setMenu</a> unless overridden by that method's destroyMenu function argument.
   * @default true
   */
  public native function get destroyMenu():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set destroyMenu(value:Boolean):void;

  [ExtConfig]
  /**
   * True to enable pressed/not pressed toggling. If a <a href="#!/api/Ext.button.Button-cfg-toggleGroup" rel="Ext.button.Button-cfg-toggleGroup" class="docClass">toggleGroup</a> is specified, this
   * option will be set to true.
   * @default false
   */
  public native function get enableToggle():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableToggle(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getGlyph()
   * @see #setGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  /**
   * False to disable visual cues on mouseover, mouseout and mousedown.
   * @default true
   */
  public native function get handleMouseEvents():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handleMouseEvents(value:Boolean):void;

  [ExtConfig]
  /**
   * A function called when the button is clicked (can be used instead of click event).
   *
   * <p>See also <a href="#!/api/Ext.button.Button-cfg-clickEvent" rel="Ext.button.Button-cfg-clickEvent" class="docClass">clickEvent</a></p>
   * @see #setHandler()
   */
  public native function get handler():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * The URL to open when the button is clicked. Specifying this config causes the Button to be
   * rendered with the specified URL as the <code>href</code> attribute of its <code>&lt;a&gt;</code> Element.
   *
   * <p>This is better than specifying a click handler of</p>
   *
   * <pre><code>function() { window.location = "http://www.sencha.com" }
   * </code></pre>
   *
   * <p>because the UI will provide meaningful hints to the user as to what to expect upon clicking
   * the button, and will also allow the user to open in a new tab or window, bookmark or drag the URL, or directly save
   * the URL stream to disk.</p>
   *
   * <p>See also the <a href="#!/api/Ext.button.Button-cfg-hrefTarget" rel="Ext.button.Button-cfg-hrefTarget" class="docClass">hrefTarget</a> config.</p>
   * @see #setHref()
   */
  public native function get href():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set href(value:String):void;

  [ExtConfig]
  /**
   * The target attribute to use for the underlying anchor. Only used if the <a href="#!/api/Ext.button.Button-cfg-href" rel="Ext.button.Button-cfg-href" class="docClass">href</a>
   * property is specified.
   * @default "_blank"
   */
  public native function get hrefTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hrefTarget(value:String):void;

  [ExtConfig]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.button.Button-cfg-iconCls" rel="Ext.button.Button-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * @see #setIcon()
   */
  public native function get icon():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The side of the Button box to render the icon. Four values are allowed:
   *
   * <ul>
   * <li>'top'</li>
   * <li>'right'</li>
   * <li>'bottom'</li>
   * <li>'left'</li>
   * </ul>
   * @default 'left'
   * @see #getIconAlign()
   * @see #setIconAlign()
   */
  public native function get iconAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconAlign(value:String):void;

  [ExtConfig]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #setIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Menu</a> object associated with this Button when configured with the <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a> config
   * option.
   * @see #getMenu()
   */
  public native function get menu():ext.menu.Menu;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menu(value:ext.menu.Menu):void;

  [ExtConfig]
  /**
   * The position to align the menu to (see <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">Ext.util.Positionable.alignTo</a> for more details).
   * @default 'tl-bl?'
   */
  public native function get menuAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuAlign(value:String):void;

  [ExtConfig]
  /**
   * If used in a <a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Toolbar</a>, the text to be used if this item is shown in the overflow menu.
   * See also <a href="#!/api/Ext.toolbar.Item" rel="Ext.toolbar.Item" class="docClass">Ext.toolbar.Item</a>.<code><a href="#!/api/Ext.toolbar.Item-cfg-overflowText" rel="Ext.toolbar.Item-cfg-overflowText" class="docClass">overflowText</a></code>.
   */
  public native function get overflowText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overflowText(value:String):void;

  [ExtConfig]
  /**
   * An object literal of parameters to pass to the url when the <a href="#!/api/Ext.button.Button-cfg-href" rel="Ext.button.Button-cfg-href" class="docClass">href</a> property is specified. Any params
   * override <a href="#!/api/Ext.button.Button-cfg-baseParams" rel="Ext.button.Button-cfg-baseParams" class="docClass">baseParams</a>. New params can be set using the <a href="#!/api/Ext.button.Button-method-setParams" rel="Ext.button.Button-method-setParams" class="docClass">setParams</a> method.
   * @see #setParams()
   */
  public native function get params():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set params(value:Object):void;

  [ExtConfig]
  /**
   * True if this button is pressed (only if enableToggle = true).
   * @default false
   * @see #setPressed()
   */
  public native function get pressed():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set pressed(value:Boolean):void;

  [ExtConfig]
  /**
   * Is set to <code>true</code> to prevent the default action when the <a href="#!/api/Ext.button.Button-cfg-clickEvent" rel="Ext.button.Button-cfg-clickEvent" class="docClass">clickEvent</a> is processed.
   * This provides focus control for clicks and stops scrolling on some devices when using the keyboard
   * to simulate clicks. Set this to <code>false</code> if you need to listen directly to element events (for
   * example, to use <code>window.open()</code> in response to a click).
   * @default true
   */
  public native function get preventDefault():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventDefault(value:Boolean):void;

  [ExtConfig]
  /**
   * True to repeat fire the click event while the mouse is down. This can also be a
   * <a href="#!/api/Ext.util.ClickRepeater" rel="Ext.util.ClickRepeater" class="docClass">ClickRepeater</a> config object.
   * @default false
   */
  public native function get repeat():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set repeat(value:*):void;

  [ExtConfig]
  /**
   * The size of the Button. Three values are allowed:
   *
   * <ul>
   * <li>'small' - Results in the button element being 16px high.</li>
   * <li>'medium' - Results in the button element being 24px high.</li>
   * <li>'large' - Results in the button element being 32px high.</li>
   * </ul>
   * @default 'small'
   * @see #setScale()
   */
  public native function get scale():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set scale(value:String):void;

  [ExtConfig]
  /**
   * The scope (<strong>this</strong> reference) in which the <code><a href="#!/api/Ext.button.Button-cfg-handler" rel="Ext.button.Button-cfg-handler" class="docClass">handler</a></code> and <code><a href="#!/api/Ext.button.Button-cfg-toggleHandler" rel="Ext.button.Button-cfg-toggleHandler" class="docClass">toggleHandler</a></code> is executed.
   * Defaults to this Button.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * True to force an attached <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a> with no items to be shown when clicking
   * this button. By default, the menu will not show if it is empty.
   * @default false
   */
  public native function get showEmptyMenu():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showEmptyMenu(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The button text to be used as innerHTML (html tags are accepted).
   * @see #getText()
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The text alignment for this button (center, left, right).
   * @default 'center'
   * @see #getTextAlign()
   * @see #setTextAlign()
   */
  public native function get textAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set textAlign(value:String):void;

  [ExtConfig]
  /**
   * The group this toggle button is a member of (only 1 per group can be pressed). If a toggleGroup
   * is specified, the <a href="#!/api/Ext.button.Button-cfg-enableToggle" rel="Ext.button.Button-cfg-enableToggle" class="docClass">enableToggle</a> configuration will automatically be set to true.
   */
  public native function get toggleGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set toggleGroup(value:String):void;

  [ExtConfig]
  /**
   * Function called when a Button with <a href="#!/api/Ext.button.Button-cfg-enableToggle" rel="Ext.button.Button-cfg-enableToggle" class="docClass">enableToggle</a> set to true is clicked.
   * <ul>
   *   <li><code>button</code> : ext.button.Button
   * This button.
   *   </li>
   *   <li><code>state</code> : Boolean
   * The next state of the Button, true means pressed.
   * &#64;controllable
   *   </li>
   * </ul>
   */
  public native function get toggleHandler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set toggleHandler(value:*):void;

  [ExtConfig]
  /**
   * The tooltip for the button - can be a string to be used as innerHTML (html tags are accepted) or
   * QuickTips config object.
   * @see #setTooltip()
   */
  public native function get tooltip():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tooltip(value:*):void;

  [ExtConfig]
  /**
   * The type of tooltip to use. Either 'qtip' for QuickTips or 'title' for title attribute.
   * @default 'qtip'
   */
  public native function get tooltipType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tooltipType(value:String):void;

  [ExtConfig]
  /**
   * The value of this button.  Only applicable when used as an item of a <a href="#!/api/Ext.button.Segmented" rel="Ext.button.Segmented" class="docClass">Segmented Button</a>.
   */
  public native function get value():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set value(value:*):void;

  /**
   * Retrieves the first direct child of this container which matches the passed selector or component.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector. If no selector is
   * specified, the first child will be returned.
   * @return The matching child <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function child(selector:* = undefined):ext.Component;

  /**
   * Programmatically activate the button.
   * @param e Optional event to process.
   */
  public native function click(e:ext.event.Event = null):void;

  /**
   * Retrieves the first descendant of this container which matches the passed selector.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. If no selector is
   * specified, the first child will be returned.
   * @return The matching descendant <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function down(selector:* = undefined):ext.Component;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Button-cfg-arrowVisible" rel="Ext.button.Button-cfg-arrowVisible" class="docClass">arrowVisible</a>.
   */
  public native function getArrowVisible():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Button-cfg-glyph" rel="Ext.button.Button-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function getGlyph():*;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Button-cfg-iconAlign" rel="Ext.button.Button-cfg-iconAlign" class="docClass">iconAlign</a>.
   */
  public native function getIconAlign():String;

  /**
   * Get the <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a> for this button.
   * @return The menu. <code>null</code> if no menu is configured.
   */
  public native function getMenu():ext.menu.Menu;

  /**
   * This method returns an object which provides substitution parameters for the <a href="#!/api/Ext.button.Button-cfg-renderTpl" rel="Ext.button.Button-cfg-renderTpl" class="docClass">XTemplate</a> used to
   * create this Button's DOM structure.
   *
   * <p>Instances or subclasses which use a different Template to create a different DOM structure may need to provide
   * their own implementation of this method.</p>
   * @return Substitution data for a Template. The default implementation which provides data for the default
   * <a href="#!/api/Ext.button.Button-property-template" rel="Ext.button.Button-property-template" class="docClass">template</a> returns an Object containing the following properties:
   * <ul>
   *   <li><code>innerCls</code> : String
   * A CSS class to apply to the button's text element.
   *   </li>
   *   <li><code>splitCls</code> : String
   * A CSS class to determine the presence and position of an arrow icon.
   * (<code>'x-btn-arrow'</code> or <code>'x-btn-arrow-bottom'</code> or <code>''</code>)
   *   </li>
   *   <li><code>iconUrl</code> : String
   * The url for the button icon.
   *   </li>
   *   <li><code>iconCls</code> : String
   * The CSS class for the button icon.
   *   </li>
   *   <li><code>glyph</code> : String
   * The glyph to use as the button icon.
   *   </li>
   *   <li><code>glyphCls</code> : String
   * The CSS class to use for the glyph element.
   *   </li>
   *   <li><code>glyphFontFamily</code> : String
   * The CSS font-family to use for the glyph element.
   *   </li>
   *   <li><code>text</code> : String
   * The <a href="#!/api/Ext.button.Button-cfg-text" rel="Ext.button.Button-cfg-text" class="docClass">text</a> to display ion the Button.
   *   </li>
   * </ul>
   */
  public native function getTemplateArgs():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Button-cfg-text" rel="Ext.button.Button-cfg-text" class="docClass">text</a>.
   */
  public native function getText():String;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Button-cfg-textAlign" rel="Ext.button.Button-cfg-textAlign" class="docClass">textAlign</a>.
   */
  public native function getTextAlign():String;

  /**
   * Returns true if the button has a menu and it is visible
   */
  public native function hasVisibleMenu():Boolean;

  /**
   * Hides this button's menu (if it has one)
   * @return this
   */
  public native function hideMenu():void;

  /**
   * Retrieves all descendant components which match the passed selector.
   * Executes an <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a> using this container as its root.
   * @param selector Selector complying to an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector.
   * If no selector is specified all items will be returned.
   * @return Components which matched the selector
   */
  public native function query(selector:String = null):Array;

  /**
   * Retrieves all descendant components which match the passed function.
   * The function should return false for components that are to be
   * excluded from the selection.
   * @param fn The matcher function. It will be called with a single argument,
   * the component being tested.
   * @param scope The scope in which to run the function. If not specified,
   * it will default to the active component.
   * @return Components matched by the passed function
   */
  public native function queryBy(fn:Function, scope:Object = null):Array;

  /**
   * Finds a component at any level under this container matching the id/itemId.
   * This is a shorthand for calling ct.down('#' + id);
   * @param id The id to find
   * @return The matching id, null if not found
   */
  public native function queryById(id:String):ext.Component;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Button-cfg-arrowVisible" rel="Ext.button.Button-cfg-arrowVisible" class="docClass">arrowVisible</a>.
   * @param arrowVisible The new value.
   */
  public native function setArrowVisible(arrowVisible:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Button-cfg-glyph" rel="Ext.button.Button-cfg-glyph" class="docClass">glyph</a>.
   * @param glyph The new value.
   */
  public native function setGlyph(glyph:*):void;

  /**
   * Assigns this Button's click handler
   * @param handler The function to call when the button is clicked
   * @param scope The scope (<code>this</code> reference) in which the handler function is executed.
   * Defaults to this Button.
   * @return this
   */
  public native function setHandler(handler:Function, scope:Object = null):ext.button.Button;

  /**
   * Sets the href of the embedded anchor element to the passed URL.
   *
   * <p>Also appends any configured <a href="#!/api/Ext.button.Button-cfg-baseParams" rel="Ext.button.Button-cfg-baseParams" class="docClass">baseParams</a> and parameters set through <a href="#!/api/Ext.button.Button-method-setParams" rel="Ext.button.Button-method-setParams" class="docClass">setParams</a>.</p>
   * @param href The URL to set in the anchor element.
   */
  public native function setHref(href:String):void;

  /**
   * Sets the background image (inline style) of the button. This method also changes the value of the <a href="#!/api/Ext.button.Button-cfg-icon" rel="Ext.button.Button-cfg-icon" class="docClass">icon</a>
   * config internally.
   * @param icon The path to an image to display in the button
   * @return this
   */
  public native function setIcon(icon:String):ext.button.Button;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Button-cfg-iconAlign" rel="Ext.button.Button-cfg-iconAlign" class="docClass">iconAlign</a>.
   * @param iconAlign The new value.
   */
  public native function setIconAlign(iconAlign:String):void;

  /**
   * Sets the CSS class that provides a background image to use as the button's icon. This method also changes the
   * value of the <a href="#!/api/Ext.button.Button-cfg-iconCls" rel="Ext.button.Button-cfg-iconCls" class="docClass">iconCls</a> config internally.
   * @param cls The CSS class providing the icon image
   * @return this
   */
  public native function setIconCls(cls:String):ext.button.Button;

  /**
   * Sets a new menu for this button. Pass a falsy value to unset the current menu.
   * To destroy the previous menu for this button, explicitly pass <code>false</code> as the second argument. If this is not set, the destroy will depend on the
   * value of <a href="#!/api/Ext.button.Button-cfg-destroyMenu" rel="Ext.button.Button-cfg-destroyMenu" class="docClass">destroyMenu</a>.
   * @param menu Accepts a menu component, a menu id or a menu config.
   * @param destroyMenu By default, will destroy the previous set menu and remove it from the menu manager. Pass <code>false</code> to prevent the destroy.
   */
  public native function setMenu(menu:*, destroyMenu:Boolean):void;

  /**
   * Sets the href of the link dynamically according to the params passed, and any <a href="#!/api/Ext.button.Button-cfg-baseParams" rel="Ext.button.Button-cfg-baseParams" class="docClass">baseParams</a> configured.
   *
   * <pre><code>var button = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
   *     renderTo   : document.body,
   *     text       : 'Open',
   *     href       : 'http://www.sencha.com',
   *     baseParams : {
   *         foo : 'bar'
   *     }
   * });
   *
   * button.setParams({
   *     company : 'Sencha'
   * });
   * </code></pre>
   *
   * <p>When clicked, this button will open a new window with the url http://www.sencha.com/?foo=bar&amp;company=Sencha because
   * the button was configured with the <a href="#!/api/Ext.button.Button-cfg-baseParams" rel="Ext.button.Button-cfg-baseParams" class="docClass">baseParams</a> to have <code>foo</code> = <code>'bar'</code>
   * and then used <a href="#!/api/Ext.button.Button-method-setParams" rel="Ext.button.Button-method-setParams" class="docClass">setParams</a> to set the <code>company</code> parameter to <code>'Sencha'</code>.</p>
   *
   * <p><strong>Only valid if the Button was originally configured with a <a href="#!/api/Ext.button.Button-cfg-href" rel="Ext.button.Button-cfg-href" class="docClass">href</a></strong></p>
   * @param params Parameters to use in the href URL.
   */
  public native function setParams(params:Object):void;

  /**
   * Sets the <code>pressed</code> state of this button.
   * @param pressed Pass <code>false</code> to clear the <code>pressed</code> state.
   * @default true
   * @return this
   */
  public native function setPressed(pressed:Boolean = true):ext.button.Button;

  /**
   * Method to change the scale of the button. See <a href="#!/api/Ext.button.Button-cfg-scale" rel="Ext.button.Button-cfg-scale" class="docClass">scale</a> for allowed configurations.
   * @param scale The scale to change to.
   */
  public native function setScale(scale:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Button-cfg-text" rel="Ext.button.Button-cfg-text" class="docClass">text</a>.
   * @param text The new value.
   */
  public native function setText(text:String):ext.button.Button;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Button-cfg-textAlign" rel="Ext.button.Button-cfg-textAlign" class="docClass">textAlign</a>.
   * @param textAlign The new value.
   */
  public native function setTextAlign(textAlign:String):void;

  /**
   * Sets the tooltip for this Button.
   * @param tooltip This may be:
   * <ul>
   * <li><strong>String</strong> : A string to be used as innerHTML (html tags are accepted) to show in a tooltip</li>
   * <li><strong>Object</strong> : A configuration object for <a href="#!/api/Ext.tip.QuickTipManager-method-register" rel="Ext.tip.QuickTipManager-method-register" class="docClass">Ext.tip.QuickTipManager.register</a>.</li>
   * </ul>
   * @return this
   */
  public native function setTooltip(tooltip:*):ext.button.Button;

  /**
   * Shows this button's menu (if it has one)
   * @param clickEvent (private)
   */
  public native function showMenu(clickEvent:Object):void;

  /**
   * If a state it passed, it becomes the pressed state otherwise the current state is toggled.
   * @param state Force a particular state
   * @param suppressEvent True to stop events being fired when calling this method.
   * @default false
   * @return this
   */
  public native function toggle(state:Boolean = false, suppressEvent:Boolean = false):ext.button.Button;

  /**
   * Traverses the tree rooted at this node in post-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>after</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPostOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Traverses the tree rooted at this node in pre-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>before</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPreOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Returns true if the passed event's x/y coordinates are within the trigger region
   */
  protected native function isWithinTrigger(e:ext.event.Event):void;
}
}