package ext.button {

[Uses("ext.button.Button")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.button.Split", require)]
/**
 * Fires when this button's arrow is clicked.
 * @eventType ext.button.events.SplitButton_eEvent.ARROW_CLICK
 */
[Event(name = 'onArrowClick', type = 'ext.button.events.SplitButton_eEvent')]
/**
 * A split button that provides a built-in dropdown arrow that can fire an event separately from the default click event
 * of the button. Typically this would be used to display a dropdown menu that provides additional options to the
 * primary button action, but any custom handler can provide the arrowclick implementation.  Example usage:
 *
 * <pre class='inline-example '><code>// display a dropdown menu:
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Split" rel="Ext.button.Split" class="docClass">Ext.button.Split</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     text: 'Options',
 *     // handle a click on the button itself
 *     handler: function() {
 *         alert("The button was clicked");
 *     },
 *     menu: new <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>({
 *         items: [
 *             // these will render as dropdown menu items when the arrow is clicked:
 *             {text: 'Item 1', handler: function(){ alert("Item 1 clicked"); }},
 *             {text: 'Item 2', handler: function(){ alert("Item 2 clicked"); }}
 *         ]
 *     })
 * });
 * </code></pre>
 *
 * <p>Instead of showing a menu, you can provide any type of custom functionality you want when the dropdown
 * arrow is clicked:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Split" rel="Ext.button.Split" class="docClass">Ext.button.Split</a>', {
 *     renderTo: 'button-ct',
 *     text: 'Options',
 *     handler: optionsHandler,
 *     arrowHandler: myCustomHandler
 * });
 * </code></pre>
 */
public class SplitButton extends ext.button.Button {
  /**
   * @param config @inheritDoc
   */
  public function SplitButton(config:ext.button.SplitButton = null) {
    super();
  }

  [ExtConfig]
  /**
   * A function called when the arrow button is clicked (can be used instead of click event)
   * <ul>
   *   <li><code>this</code> : ext.button.SplitButton</li>
   *   <li><code>e</code> : js.Event
   * The click event.
   * &#64;controllable
   *   </li>
   * </ul>
   * @see #setArrowHandler()
   */
  public native function get arrowHandler():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set arrowHandler(value:*):void;

  [ExtConfig]
  /**
   * The title attribute of the arrow.
   */
  public native function get arrowTooltip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set arrowTooltip(value:String):void;

  /**
   * Sets this button's arrow click handler.
   * @param handler The function to call when the arrow is clicked.
   * @param scope Scope for the function passed above.
   */
  public native function setArrowHandler(handler:Function, scope:Object = null):void;
}
}