package ext.data {
import ext.util.Collection;

[Uses("ext.data.AbstractStore")]
[Uses("ext.data.LocalStore")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.ChainedStore", require)]
/**
 * A chained store is a store that is a "view" of an existing store. The data comes from the
 * <a href="#!/api/Ext.data.ChainedStore-cfg-source" rel="Ext.data.ChainedStore-cfg-source" class="docClass">source</a>, however this view of the store may be sorted &amp; filtered independently without
 * having any impact on the <a href="#!/api/Ext.data.ChainedStore-cfg-source" rel="Ext.data.ChainedStore-cfg-source" class="docClass">source</a> store.
 */
public class ChainedStore extends ext.data.AbstractStore implements ext.data.ILocalStore {
  /**
   * @param config @inheritDoc
   */
  public function ChainedStore(config:ext.data.ChainedStore = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The backing data source for this chained store. Either a store instance
   * or the id of an existing store.
   * @see #getSource()
   * @see #setSource()
   */
  public native function get source():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set source(value:*):void;

  /**
   * Get the model used for this store.
   * @return The model
   */
  public native function getModel():ext.data.Model;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ChainedStore-cfg-source" rel="Ext.data.ChainedStore-cfg-source" class="docClass">source</a>.
   */
  public native function getSource():*;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ChainedStore-cfg-source" rel="Ext.data.ChainedStore-cfg-source" class="docClass">source</a>.
   * @param source The new value.
   */
  public native function setSource(source:*):void;

  /**
   * @inheritDoc
   */
  public native function add(...model):Array;

  /**
   * @inheritDoc
   */
  public native function aggregate(fn:Function, scope:Object = null, grouped:Boolean = false, field:String = null):Object;

  /**
   * @inheritDoc
   */
  public native function average(field:String, grouped:Boolean = false):Object;

  /**
   * @inheritDoc
   */
  public native function collect(property:String, includeOptions:Object = null):Array;

  /**
   * @inheritDoc
   */
  public native function count(grouped:Boolean = false):Number;

  /**
   * @inheritDoc
   */
  public native function each(fn:Function, scope:Object = null, includeOptions:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function first(grouped:Boolean = false):*;

  /**
   * @inheritDoc
   */
  public native function indexOf(record:ext.data.Model):Number;

  /**
   * @inheritDoc
   */
  public native function indexOfId(id:String):Number;

  /**
   * @inheritDoc
   */
  public native function insert(index:Number, records:*):Array;

  /**
   * @inheritDoc
   */
  public native function last(grouped:Boolean = false):*;

  /**
   * @inheritDoc
   */
  public native function max(field:String, grouped:Boolean = false):Object;

  /**
   * @inheritDoc
   */
  public native function min(field:String, grouped:Boolean = false):Object;

  /**
   * @inheritDoc
   */
  public native function query(property:String, value:*, anyMatch:Boolean = false, caseSensitive:Boolean = false, exactMatch:Boolean = false):ext.util.Collection;

  /**
   * @inheritDoc
   */
  public native function queryBy(fn:Function, scope:Object = null):ext.util.Collection;

  /**
   * @inheritDoc
   */
  public native function sum(field:String, grouped:Boolean = false):Number;
}
}