package ext.data {

[Uses("ext.data.SJsonP")]
[Uses("ext.Base")]
[Native("Ext.data.JsonP", require)]
/**
 * This class is used to create JSONP requests. JSONP is a mechanism that allows for making
 * requests for data cross domain. JSONP is basically a <code>&lt;script&gt;</code> node with the source of the url executing
 * a function that was created by <a href="#!/api/Ext.data.JsonP" rel="Ext.data.JsonP" class="docClass">Ext.data.JsonP</a>. Once the resource has loaded, the <code>&lt;script&gt;</code> node will be destroyed.
 *
 * <p>If you have a request such as:</p>
 *
 * <pre><code><a href="#!/api/Ext.data.JsonP-method-request" rel="Ext.data.JsonP-method-request" class="docClass">Ext.data.JsonP.request</a>({
 *     url : 'foo.php'
 * });
 * </code></pre>
 *
 * <p><a href="#!/api/Ext.data.JsonP" rel="Ext.data.JsonP" class="docClass">Ext.data.JsonP</a> will create a <code>&lt;script&gt;</code> node in the <code>&lt;head&gt;</code> with the <code>src</code> attribute pointing to
 * <code>foo.php?callback=Ext.data.JsonP.callback1</code>. The <code>foo.php</code> script will have to detect the <code>callback</code> URL parameter
 * and return valid JavaScript:</p>
 *
 * <pre><code>Ext.data.JsonP.callback1({"foo":"bar"});
 * </code></pre>
 *
 * <p>A simple PHP example would look like:</p>
 *
 * <pre><code>&lt;?php
 *
 * $data = array('foo' =&gt; 'bar');
 *
 * if (!empty($_REQUEST['callback'])) {
 *     header('Content-Type: application/javascript');
 *     echo $_REQUEST['callback'] . '(';
 * }
 *
 * echo json_encode($data);
 *
 * if (!empty($_REQUEST['callback']) {
 *     echo ');';
 * }
 *
 * ?&gt;
 * </code></pre>
 *
 * <p>More information is available <a href="http://en.wikipedia.org/wiki/JSONP">here</a>. You can also use <a href="http://www.jsonplint.com">JSONPLint</a> to test your JSONP.</p>
 */
public const JsonP:ext.data.SJsonP;
}