package ext.data {

[Uses("ext.data.Store")]
[Uses("ext.data.LocalStore")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.data.ProxyStore")]
[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.JsonPStore", require)]
/**
 * Small helper class to make creating <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>s from different domain JSON data easier.
 * A JsonPStore will be automatically configured with a <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">Ext.data.reader.Json</a> and a <a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">JsonPProxy</a>.
 *
 *
 * <p>A store configuration would be something like:
 * <pre><code>var store = new <a href="#!/api/Ext.data.JsonPStore" rel="Ext.data.JsonPStore" class="docClass">Ext.data.JsonPStore</a>({
 *     // store configs
 *     storeId: 'myStore',
 *
 *     // proxy configs
 *     url: 'get-images.php',
 *
 *     // reader configs
 *     root: 'images',
 *     fields: ['name', 'url', {name:'size', type: 'float'}, {name:'lastmod', type:'date'}]
 * });
 * </code></pre></p>
 *
 *
 * <p>This store is configured to consume a returned object of the form:
 * <pre><code>stcCallback({
 *     images: [
 *         {name: 'Image one', url:'/GetImage.php?id=1', size:46.5, lastmod: new Date(2007, 10, 29)},
 *         {name: 'Image Two', url:'/GetImage.php?id=2', size:43.2, lastmod: new Date(2007, 10, 30)}
 *     ]
 * })
 * </code></pre>
 * <p>Where stcCallback is the callback name passed in the request to the remote domain. See <a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">JsonPProxy</a>
 * for details of how this works.</p>
 * An object literal of this form could also be used as the <a href="#!/api/Ext.data.JsonPStore-cfg-data" rel="Ext.data.JsonPStore-cfg-data" class="docClass">data</a> config option.</p>
 */
public class JsonPStore extends ext.data.Store {
  /**
   * @param config @inheritDoc
   */
  public function JsonPStore(config:ext.data.JsonPStore = null) {
    super();
  }
}
}