package ext.data {
import ext.Base;
import ext.data.operation.Operation;
import ext.data.proxy.DataProxy;

[Uses("ext.Base")]
[Native("Ext.data.Request", require)]
/**
 * Simple class that represents a Request that will be made by any <a href="#!/api/Ext.data.proxy.Server" rel="Ext.data.proxy.Server" class="docClass">Ext.data.proxy.Server</a> subclass.
 * All this class does is standardize the representation of a Request as used by any ServerProxy subclass,
 * it does not contain any actual logic or perform the request itself.
 */
public class Request extends ext.Base {
  /**
   * Creates the Request object.
   * @param config Config object.
   */
  public function Request(config:ext.data.Request = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the action this Request represents. Usually one of 'create', 'read', 'update' or 'destroy'.
   * @see #getAction()
   * @see #setAction()
   */
  public native function get action():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set action(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to request binary data from the server.  This feature requires
   * the use of a binary reader such as AMF Reader
   * @default false
   * @see #getBinary()
   * @see #setBinary()
   */
  public native function get binary():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set binary(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Some requests (like JsonP) want to send an additional key that contains
   * the name of the callback function.
   * @see #getCallbackKey()
   * @see #setCallbackKey()
   */
  public native function get callbackKey():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set callbackKey(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Whether or not to disable caching for this request.
   * @default false
   * @see #getDisableCaching()
   * @see #setDisableCaching()
   */
  public native function get disableCaching():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disableCaching(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Some requests (like XMLHttpRequests) want to send additional server headers.
   * This configuration can be set for those types of requests.
   * @default {}
   * @see #getHeaders()
   * @see #setHeaders()
   */
  public native function get headers():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set headers(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This is used by some write actions to attach data to the request without encoding it
   * as a parameter.
   * @see #getJsonData()
   * @see #setJsonData()
   */
  public native function get jsonData():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set jsonData(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The HTTP method to use on this Request. Should be one of 'GET', 'POST', 'PUT' or 'DELETE'.
   * @default 'GET'
   * @see #getMethod()
   * @see #setMethod()
   */
  public native function get method():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set method(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The operation this request belongs to.
   * @see #getOperation()
   * @see #setOperation()
   */
  public native function get operation():ext.data.operation.Operation;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set operation(value:ext.data.operation.Operation):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * HTTP request params. The Proxy and its Writer have access to and can modify this object.
   * @see #getParams()
   * @see #setParams()
   */
  public native function get params():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set params(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Most oData feeds require basic HTTP authentication. This configuration allows
   * you to specify the password.
   * @see #getPassword()
   * @see #setPassword()
   */
  public native function get password():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set password(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The proxy this request belongs to.
   * @see #getProxy()
   * @see #setProxy()
   */
  public native function get proxy():ext.data.proxy.DataProxy;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set proxy(value:ext.data.proxy.DataProxy):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The url to access on this Request.
   * @see #getUrl()
   * @see #setUrl()
   */
  public native function get url():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set url(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Most oData feeds require basic HTTP authentication. This configuration allows
   * you to specify the username.
   * @see #getUsername()
   * @see #setUsername()
   */
  public native function get username():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set username(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This field is necessary when using cross-origin resource sharing.
   * @default false
   * @see #getWithCredentials()
   * @see #setWithCredentials()
   */
  public native function get withCredentials():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set withCredentials(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This is used by some write actions to attach data to the request without encoding it
   * as a parameter, but instead sending it as XML.
   * @see #getXmlData()
   * @see #setXmlData()
   */
  public native function get xmlData():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set xmlData(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-action" rel="Ext.data.Request-cfg-action" class="docClass">action</a>.
   */
  public native function getAction():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-binary" rel="Ext.data.Request-cfg-binary" class="docClass">binary</a>.
   */
  public native function getBinary():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-callbackKey" rel="Ext.data.Request-cfg-callbackKey" class="docClass">callbackKey</a>.
   */
  public native function getCallbackKey():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-disableCaching" rel="Ext.data.Request-cfg-disableCaching" class="docClass">disableCaching</a>.
   */
  public native function getDisableCaching():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-headers" rel="Ext.data.Request-cfg-headers" class="docClass">headers</a>.
   */
  public native function getHeaders():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-jsonData" rel="Ext.data.Request-cfg-jsonData" class="docClass">jsonData</a>.
   */
  public native function getJsonData():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-method" rel="Ext.data.Request-cfg-method" class="docClass">method</a>.
   */
  public native function getMethod():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-operation" rel="Ext.data.Request-cfg-operation" class="docClass">operation</a>.
   */
  public native function getOperation():ext.data.operation.Operation;

  /**
   * Gets a single param from the <a href="#!/api/Ext.data.Request-cfg-params" rel="Ext.data.Request-cfg-params" class="docClass">params</a>.
   * @param key The key for the param.
   * @return The value for the param. <code>undefined</code> if it does not exist.
   */
  public native function getParam(key:String):Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-params" rel="Ext.data.Request-cfg-params" class="docClass">params</a>.
   */
  public native function getParams():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-password" rel="Ext.data.Request-cfg-password" class="docClass">password</a>.
   */
  public native function getPassword():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-proxy" rel="Ext.data.Request-cfg-proxy" class="docClass">proxy</a>.
   */
  public native function getProxy():ext.data.proxy.DataProxy;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-url" rel="Ext.data.Request-cfg-url" class="docClass">url</a>.
   */
  public native function getUrl():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-username" rel="Ext.data.Request-cfg-username" class="docClass">username</a>.
   */
  public native function getUsername():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-withCredentials" rel="Ext.data.Request-cfg-withCredentials" class="docClass">withCredentials</a>.
   */
  public native function getWithCredentials():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Request-cfg-xmlData" rel="Ext.data.Request-cfg-xmlData" class="docClass">xmlData</a>.
   */
  public native function getXmlData():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-action" rel="Ext.data.Request-cfg-action" class="docClass">action</a>.
   * @param action The new value.
   */
  public native function setAction(action:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-binary" rel="Ext.data.Request-cfg-binary" class="docClass">binary</a>.
   * @param binary The new value.
   */
  public native function setBinary(binary:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-callbackKey" rel="Ext.data.Request-cfg-callbackKey" class="docClass">callbackKey</a>.
   * @param callbackKey The new value.
   */
  public native function setCallbackKey(callbackKey:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-disableCaching" rel="Ext.data.Request-cfg-disableCaching" class="docClass">disableCaching</a>.
   * @param disableCaching The new value.
   */
  public native function setDisableCaching(disableCaching:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-headers" rel="Ext.data.Request-cfg-headers" class="docClass">headers</a>.
   * @param headers The new value.
   */
  public native function setHeaders(headers:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-jsonData" rel="Ext.data.Request-cfg-jsonData" class="docClass">jsonData</a>.
   * @param jsonData The new value.
   */
  public native function setJsonData(jsonData:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-method" rel="Ext.data.Request-cfg-method" class="docClass">method</a>.
   * @param method The new value.
   */
  public native function setMethod(method:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-operation" rel="Ext.data.Request-cfg-operation" class="docClass">operation</a>.
   * @param operation The new value.
   */
  public native function setOperation(operation:ext.data.operation.Operation):void;

  /**
   * Sets a single param value in the <a href="#!/api/Ext.data.Request-cfg-params" rel="Ext.data.Request-cfg-params" class="docClass">params</a>.
   * @param key The key to set.
   * @param value The value to set.
   */
  public native function setParam(key:String, value:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-params" rel="Ext.data.Request-cfg-params" class="docClass">params</a>.
   * @param params The new value.
   */
  public native function setParams(params:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-password" rel="Ext.data.Request-cfg-password" class="docClass">password</a>.
   * @param password The new value.
   */
  public native function setPassword(password:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-proxy" rel="Ext.data.Request-cfg-proxy" class="docClass">proxy</a>.
   * @param proxy The new value.
   */
  public native function setProxy(proxy:ext.data.proxy.DataProxy):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-url" rel="Ext.data.Request-cfg-url" class="docClass">url</a>.
   * @param url The new value.
   */
  public native function setUrl(url:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-username" rel="Ext.data.Request-cfg-username" class="docClass">username</a>.
   * @param username The new value.
   */
  public native function setUsername(username:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-withCredentials" rel="Ext.data.Request-cfg-withCredentials" class="docClass">withCredentials</a>.
   * @param withCredentials The new value.
   */
  public native function setWithCredentials(withCredentials:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Request-cfg-xmlData" rel="Ext.data.Request-cfg-xmlData" class="docClass">xmlData</a>.
   * @param xmlData The new value.
   */
  public native function setXmlData(xmlData:Object):void;
}
}