package ext.data {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.data.ResultSet", require)]
/**
 * Simple wrapper class that represents a set of records returned by a Proxy.
 */
public class ResultSet extends ext.Base {
  /**
   * Creates the resultSet
   * @param config Config object.
   */
  public function ResultSet(config:ext.data.ResultSet = null) {
    super();
  }

  /**
   * Identifies this class as a result set.
   * @default true
   */
  public native function get isResultSet():Boolean;

  /**
   * @private
   */
  public native function set isResultSet(value:Boolean):void;

  [ExtConfig]
  /**
   * The number of records in this ResultSet. Note that total may differ from this number.
   * @see #setCount()
   */
  public native function get count():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set count(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True if the records have already been loaded. This is only meaningful when dealing with
   * SQL-backed proxies.
   * @default true
   * @see #getLoaded()
   * @see #setLoaded()
   */
  public native function get loaded():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set loaded(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The message that was read in from the data
   * @see #getMessage()
   * @see #setMessage()
   */
  public native function get message():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set message(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The metadata object from a server sourced JSON data packet.
   * @see #getMetadata()
   * @see #setMetadata()
   */
  public native function get metadata():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set metadata(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The array of record instances or record config objects.
   * @see #getRecords()
   * @see #setRecords()
   */
  public native function get records():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set records(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True if the ResultSet loaded successfully, false if any errors were encountered.
   * @default false
   * @see #getSuccess()
   * @see #setSuccess()
   */
  public native function get success():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set success(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The total number of records reported by the data source. This ResultSet may form a subset of
   * those records (see <a href="#!/api/Ext.data.ResultSet-cfg-count" rel="Ext.data.ResultSet-cfg-count" class="docClass">count</a>).
   * @see #getTotal()
   * @see #setTotal()
   */
  public native function get total():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set total(value:Number):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ResultSet-cfg-loaded" rel="Ext.data.ResultSet-cfg-loaded" class="docClass">loaded</a>.
   */
  public native function getLoaded():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ResultSet-cfg-message" rel="Ext.data.ResultSet-cfg-message" class="docClass">message</a>.
   */
  public native function getMessage():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ResultSet-cfg-metadata" rel="Ext.data.ResultSet-cfg-metadata" class="docClass">metadata</a>.
   */
  public native function getMetadata():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ResultSet-cfg-records" rel="Ext.data.ResultSet-cfg-records" class="docClass">records</a>.
   */
  public native function getRecords():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ResultSet-cfg-success" rel="Ext.data.ResultSet-cfg-success" class="docClass">success</a>.
   */
  public native function getSuccess():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ResultSet-cfg-total" rel="Ext.data.ResultSet-cfg-total" class="docClass">total</a>.
   */
  public native function getTotal():Number;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-count" rel="Ext.data.ResultSet-cfg-count" class="docClass">count</a>.
   * @param count The new value.
   */
  public native function setCount(count:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-loaded" rel="Ext.data.ResultSet-cfg-loaded" class="docClass">loaded</a>.
   * @param loaded The new value.
   */
  public native function setLoaded(loaded:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-message" rel="Ext.data.ResultSet-cfg-message" class="docClass">message</a>.
   * @param message The new value.
   */
  public native function setMessage(message:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-metadata" rel="Ext.data.ResultSet-cfg-metadata" class="docClass">metadata</a>.
   * @param metadata The new value.
   */
  public native function setMetadata(metadata:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-records" rel="Ext.data.ResultSet-cfg-records" class="docClass">records</a>.
   * @param records The new value.
   */
  public native function setRecords(records:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-success" rel="Ext.data.ResultSet-cfg-success" class="docClass">success</a>.
   * @param success The new value.
   */
  public native function setSuccess(success:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ResultSet-cfg-total" rel="Ext.data.ResultSet-cfg-total" class="docClass">total</a>.
   * @param total The new value.
   */
  public native function setTotal(total:Number):void;
}
}