package ext.data {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * This class defines a series of static methods that are used on a
 * <a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Ext.data.Field</a> for performing sorting. The methods cast the
 * underlying values into a data type that is appropriate for sorting on
 * that particular field.  If a <a href="#!/api/Ext.data.field.Field-property-type" rel="Ext.data.field.Field-property-type" class="docClass">Ext.data.Field.type</a> is specified,
 * the sortType will be set to a sane default if the sortType is not
 * explicitly defined on the field. The sortType will make any necessary
 * modifications to the value and return it.
 *
 * <ul>
 * <li><strong><code>asText</code></strong> - Removes any tags and converts the value to a string</li>
 * <li><strong><code>asUCText</code></strong> - Removes any tags and converts the value to an uppercase string</li>
 * <li><strong><code>asUCText</code></strong> - Converts the value to an uppercase string</li>
 * <li><strong><code>asDate</code></strong> - Converts the value into Unix epoch time</li>
 * <li><strong><code>asFloat</code></strong> - Converts the value to a floating point number</li>
 * <li><strong><code>asInt</code></strong> - Converts the value to an integer number</li>
 * </ul>
 *
 *
 * <p>It is also possible to create a custom sortType that can be used throughout
 * an application.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a>(<a href="#!/api/Ext.data.SortTypes" rel="Ext.data.SortTypes" class="docClass">Ext.data.SortTypes</a>, {
 *      asPerson: function(person){
 *          // expects an object with a first and last name property
 *          return person.lastName.toUpperCase() + person.firstName.toLowerCase();
 *      }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Employee', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *      fields: [{
 *          name: 'person',
 *          sortType: 'asPerson'
 *      }, {
 *          name: 'salary',
 *          type: 'float' // sortType set to asFloat
 *      }]
 *  });
 * </code></pre>
 * <p>Type of singleton SortTypes.</p>
 * @see ext.data.#SortTypes ext.data.SortTypes
 */
public class SSortTypes extends ext.Base {
  /**
   * The regular expression used to strip commas
   * @default /,/g
   */
  public native function get stripCommasRe():RegExp;

  /**
   * @private
   */
  public native function set stripCommasRe(value:RegExp):void;

  /**
   * The regular expression used to strip tags
   * @default /<\/?[^>]+>/gi
   */
  public native function get stripTagsRE():RegExp;

  /**
   * @private
   */
  public native function set stripTagsRE(value:RegExp):void;

  /**
   * Date sorting
   * @param s The value being converted
   * @return The comparison value
   */
  public native function asDate(s:Object):Number;

  /**
   * Float sorting
   * @param s The value being converted
   * @return The comparison value
   */
  public native function asFloat(s:Object):Number;

  /**
   * Integer sorting
   * @param s The value being converted
   * @return The comparison value
   */
  public native function asInt(s:Object):Number;

  /**
   * Strips all HTML tags to sort on text only
   * @param s The value being converted
   * @return The comparison value
   */
  public native function asText(s:Object):String;

  /**
   * Case insensitive string
   * @param s The value being converted
   * @return The comparison value
   */
  public native function asUCString(s:Object):String;

  /**
   * Strips all HTML tags to sort on text only - Case insensitive
   * @param s The value being converted
   * @return The comparison value
   */
  public native function asUCText(s:Object):String;

  /**
   * Default sort that does nothing
   * @param s The value being converted
   * @return The comparison value
   */
  public native function none(s:Object):Object;
}
}