package ext.data {
import ext.Component;
import ext.data.operation.DestroyOperation;
import ext.mixin.IQueryable;
import ext.tree.TreePanel;

[Uses("ext.data.Model")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.data.NodeInterface")]
[Uses("ext.Base")]
[Native("Ext.data.TreeModel", require)]
/**
 * This class is used as a base class from which to derive Models used in Trees.
 */
public class TreeModel extends ext.data.Model implements ext.mixin.IQueryable, ext.data.NodeInterface {
  /**
   * @param config @inheritDoc
   */
  public function TreeModel(data:Object, session:ext.data.Session = null) {
    super(null);
  }

  [ExtConfig]
  /**
   * The class name of child nodes to create when reading child nodes from
   * raw data. By default the type configured into the TreeStore is used.
   *
   * <p>This is one way of creating heterogeneous nodes in a tree.</p>
   *
   * <p>To do this through data types passed from the server, use the <a href="#!/api/Ext.data.reader.Reader-cfg-typeProperty" rel="Ext.data.reader.Reader-cfg-typeProperty" class="docClass">Ext.data.reader.Reader.typeProperty</a>.</p>
   *
   * <p>for example in the case of a hidden root node, you'd use the default type at level zero. See <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">TreePanel</a>'s
   * documentation for an example.</p>
   *
   * <p><em>Important</em>
   * If you are using this declaration on your tree models, and have a <a href="#!/api/Ext.tree.Panel-cfg-rootVisible" rel="Ext.tree.Panel-cfg-rootVisible" class="docClass">hidden root node</a>, you
   * MUST create a special root model definition which declares the type of its children.</p>
   *
   * <p>If you allow the TreeStore to create a root node of the same type as the first level of <em>visible</em> nodes
   * then the reader will atempt to read the wrong type of child node for the root.</p>
   *
   * <p>Example:</p>
   *
   * <p>   <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.World', {
   *        childType: 'Territory'
   *    });</p>
   *
   * <p>   ...</p>
   *
   * <p>   store: {
   *        id: 'myTreeStore',
   *        model: 'myApp.World' // The hidden root will know to create 'Territory' type children.
   *    }</p>
   *
   * <p>If the root is hidden, and the first level of visible nodes are going to be the <code>myApp.Territory</code> class,
   * then the hidden root must not be of the <code>myApp.Territory</code> class. Otherwise, it would try to read in the
   * territory data as its childType - most likely 'Country'.</p>
   */
  public native function get childType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set childType(value:String):void;

  /**
   * @inheritDoc
   */
  public native function get childNodes():Array;

  /**
   * @private
   */
  public native function set childNodes(value:Array):void;

  /**
   * @inheritDoc
   */
  public native function get firstChild():ext.data.NodeInterface;

  /**
   * @private
   */
  public native function set firstChild(value:ext.data.NodeInterface):void;

  /**
   * @inheritDoc
   */
  public native function get glyph():String;

  /**
   * @private
   */
  public native function set glyph(value:String):void;

  /**
   * @inheritDoc
   */
  public native function get isNode():Boolean;

  /**
   * @private
   */
  public native function set isNode(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get lastChild():ext.data.NodeInterface;

  /**
   * @private
   */
  public native function set lastChild(value:ext.data.NodeInterface):void;

  /**
   * @inheritDoc
   */
  public native function get nextSibling():ext.data.NodeInterface;

  /**
   * @private
   */
  public native function set nextSibling(value:ext.data.NodeInterface):void;

  /**
   * @inheritDoc
   */
  public native function get parentNode():ext.data.NodeInterface;

  /**
   * @private
   */
  public native function set parentNode(value:ext.data.NodeInterface):void;

  /**
   * @inheritDoc
   */
  public native function get previousSibling():ext.data.NodeInterface;

  /**
   * @private
   */
  public native function set previousSibling(value:ext.data.NodeInterface):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get allowDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowDrag(value:Boolean):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get allowDrop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowDrop(value:Boolean):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get checked():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checked(value:Boolean):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get children():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set children(value:Array):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get cls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cls(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get expandable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandable(value:Boolean):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get expanded():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expanded(value:Boolean):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get href():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set href(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get hrefTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hrefTarget(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get icon():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get iconCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get leaf():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set leaf(value:Boolean):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get qshowDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set qshowDelay(value:Number):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get qtip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set qtip(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get qtitle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set qtitle(value:String):void;

  [ExtConfig]
  /**
   * @inheritDoc
   */
  public native function get text():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set text(value:String):void;

  /**
   * @inheritDoc
   */
  override public native function erase(options:Object = null):ext.data.operation.DestroyOperation;

  /**
   * @inheritDoc
   */
  override public native function isLoading():Boolean;

  /**
   * Retrieves the first direct child of this container which matches the passed selector or component.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector. If no selector is
   * specified, the first child will be returned.
   * @return The matching child <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function child(selector:* = undefined):ext.Component;

  /**
   * Retrieves the first descendant of this container which matches the passed selector.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. If no selector is
   * specified, the first child will be returned.
   * @return The matching descendant <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function down(selector:* = undefined):ext.Component;

  /**
   * Retrieves all descendant components which match the passed selector.
   * Executes an <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a> using this container as its root.
   * @param selector Selector complying to an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector.
   * If no selector is specified all items will be returned.
   * @return Components which matched the selector
   */
  public native function query(selector:String = null):Array;

  /**
   * Retrieves all descendant components which match the passed function.
   * The function should return false for components that are to be
   * excluded from the selection.
   * @param fn The matcher function. It will be called with a single argument,
   * the component being tested.
   * @param scope The scope in which to run the function. If not specified,
   * it will default to the active component.
   * @return Components matched by the passed function
   */
  public native function queryBy(fn:Function, scope:Object = null):Array;

  /**
   * Finds a component at any level under this container matching the id/itemId.
   * This is a shorthand for calling ct.down('#' + id);
   * @param id The id to find
   * @return The matching id, null if not found
   */
  public native function queryById(id:String):ext.Component;

  /**
   * Traverses the tree rooted at this node in post-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>after</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPostOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Traverses the tree rooted at this node in pre-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>before</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPreOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * @inheritDoc
   */
  public native function appendChild(node:*, suppressEvents:Boolean = false, commit:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function bubble(fn:Function, scope:Object = null, args:Array = null):void;

  /**
   * @inheritDoc
   */
  public native function cascade(spec:Object):void;

  /**
   * @inheritDoc
   */
  public native function collapse(recursive:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function collapseChildren(recursive:Function = null, callback:Function = null, scope:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function contains(node:ext.data.NodeInterface):Boolean;

  /**
   * @inheritDoc
   */
  public native function copyNode(id:String = null, session:Object = null, deep:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function createNode(node:Object):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function eachChild(fn:Function, scope:Object = null, args:Array = null):void;

  /**
   * @inheritDoc
   */
  public native function expand(recursive:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function expandChildren(recursive:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function findChild(attribute:String, value:Object, deep:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function findChildBy(fn:Function, scope:Object = null, deep:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function fireEvent(eventName:String, ...args):Boolean;

  /**
   * @inheritDoc
   */
  public native function getChildAt(index:Number):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function getDepth():Number;

  /**
   * @inheritDoc
   */
  public native function getOwnerTree():ext.tree.TreePanel;

  /**
   * @inheritDoc
   */
  public native function getPath(field:String = null, separator:String = '/'):String;

  /**
   * @inheritDoc
   */
  public native function getTreeStore():ext.data.TreeStore;

  /**
   * @inheritDoc
   */
  public native function hasChildNodes():Boolean;

  /**
   * @inheritDoc
   */
  public native function indexOf(node:ext.data.NodeInterface):Number;

  /**
   * @inheritDoc
   */
  public native function indexOfId(id:String):Number;

  /**
   * @inheritDoc
   */
  public native function insertBefore(node:*, refNode:ext.data.NodeInterface):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function insertChild(index:Number, node:*):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function isAncestor(node:ext.data.NodeInterface):Boolean;

  /**
   * @inheritDoc
   */
  public native function isBranchLoaded():Boolean;

  /**
   * @inheritDoc
   */
  public native function isExpandable():Boolean;

  /**
   * @inheritDoc
   */
  public native function isExpanded():Boolean;

  /**
   * @inheritDoc
   */
  public native function isFirst():Boolean;

  /**
   * @inheritDoc
   */
  public native function isLast():Boolean;

  /**
   * @inheritDoc
   */
  public native function isLeaf():Boolean;

  /**
   * @inheritDoc
   */
  public native function isLoaded():Boolean;

  /**
   * @inheritDoc
   */
  public native function isRoot():Boolean;

  /**
   * @inheritDoc
   */
  public native function isVisible():Boolean;

  /**
   * @inheritDoc
   */
  public native function onRegisterTreeNode(treeStore:ext.data.TreeStore):void;

  /**
   * @inheritDoc
   */
  public native function onUnregisterTreeNode(treeStore:ext.data.TreeStore):void;

  /**
   * @inheritDoc
   */
  public native function remove(erase:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function removeAll(erase:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function removeChild(node:ext.data.NodeInterface, erase:Boolean = false):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function replaceChild(newChild:ext.data.NodeInterface, oldChild:ext.data.NodeInterface):ext.data.NodeInterface;

  /**
   * @inheritDoc
   */
  public native function serialize(writerParam:Object):void;

  /**
   * @inheritDoc
   */
  public native function sort(sortFn:Function = null, recursive:Boolean = false, suppressEvent:Boolean = false):void;

  [ArrayElementType("String")]
  /**
   * @inheritDoc
   */
  public native function updateInfo(commit:Boolean, info:Object):Array;
}
}