package ext.data {

[Uses("ext.data.Store")]
[Uses("ext.data.LocalStore")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.data.ProxyStore")]
[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.TreeStore", require)]
/**
 * Fires whenever the store reads data from a remote data source.
 * @eventType ext.data.events.TreeStore_records_successful_operation_nodeEvent.LOAD
 */
[Event(name = 'onLoad', type = 'ext.data.events.TreeStore_records_successful_operation_nodeEvent')]
/**
 * Fires when a new child node is appended
 * @eventType ext.data.events.TreeStore_node_indexEvent.NODE_APPEND
 */
[Event(name = 'onNodeAppend', type = 'ext.data.events.TreeStore_node_indexEvent')]
/**
 * Fires before a new child is appended, return false to cancel the append.
 * @eventType ext.data.events.TreeStore_nodeEvent.NODE_BEFORE_APPEND
 */
[Event(name = 'onNodeBeforeAppend', type = 'ext.data.events.TreeStore_nodeEvent')]
/**
 * Fires before this node is collapsed.
 * @eventType ext.data.events.TreeStoreEvent.NODE_BEFORE_COLLAPSE
 */
[Event(name = 'onNodeBeforeCollapse', type = 'ext.data.events.TreeStoreEvent')]
/**
 * Fires before this node is expanded.
 * @eventType ext.data.events.TreeStoreEvent.NODE_BEFORE_EXPAND
 */
[Event(name = 'onNodeBeforeExpand', type = 'ext.data.events.TreeStoreEvent')]
/**
 * Fires before a new child is inserted, return false to cancel the insert.
 * @eventType ext.data.events.TreeStore_node_refNodeEvent.NODE_BEFORE_INSERT
 */
[Event(name = 'onNodeBeforeInsert', type = 'ext.data.events.TreeStore_node_refNodeEvent')]
/**
 * Fires before this node is moved to a new location in the tree. Return false to cancel the move.
 * @eventType ext.data.events.TreeStore_oldParent_newParent_indexEvent.NODE_BEFORE_MOVE
 */
[Event(name = 'onNodeBeforeMove', type = 'ext.data.events.TreeStore_oldParent_newParent_indexEvent')]
/**
 * Fires before a child is removed, return false to cancel the remove.
 * @eventType ext.data.events.TreeStore_node_isMoveEvent.NODE_BEFORE_REMOVE
 */
[Event(name = 'onNodeBeforeRemove', type = 'ext.data.events.TreeStore_node_isMoveEvent')]
/**
 * Fires when this node is collapsed.
 * @eventType ext.data.events.TreeStoreEvent.NODE_COLLAPSE
 */
[Event(name = 'onNodeCollapse', type = 'ext.data.events.TreeStoreEvent')]
/**
 * Fires when this node is expanded.
 * @eventType ext.data.events.TreeStoreEvent.NODE_EXPAND
 */
[Event(name = 'onNodeExpand', type = 'ext.data.events.TreeStoreEvent')]
/**
 * Fires when a new child node is inserted.
 * @eventType ext.data.events.TreeStore_node_refNodeEvent.NODE_INSERT
 */
[Event(name = 'onNodeInsert', type = 'ext.data.events.TreeStore_node_refNodeEvent')]
/**
 * Fires when this node is moved to a new location in the tree
 * @eventType ext.data.events.TreeStore_oldParent_newParent_indexEvent.NODE_MOVE
 */
[Event(name = 'onNodeMove', type = 'ext.data.events.TreeStore_oldParent_newParent_indexEvent')]
/**
 * Fires when a child node is removed
 * @eventType ext.data.events.TreeStore_node_isMove_contextEvent.NODE_REMOVE
 */
[Event(name = 'onNodeRemove', type = 'ext.data.events.TreeStore_node_isMove_contextEvent')]
/**
 * Fires when this node's childNodes are sorted.
 * @eventType ext.data.events.TreeStore_childNodesEvent.NODE_SORT
 */
[Event(name = 'onNodeSort', type = 'ext.data.events.TreeStore_childNodesEvent')]
/**
 * Fires any time the tree's root node changes.
 * @eventType ext.data.events.TreeStore_newRoot_oldRootEvent.ROOT_CHANGE
 */
[Event(name = 'onRootChange', type = 'ext.data.events.TreeStore_newRoot_oldRootEvent')]
/**
 * The TreeStore is a store implementation that owns the <a href="#!/api/Ext.data.TreeStore-cfg-root" rel="Ext.data.TreeStore-cfg-root" class="docClass">root node</a> of
 * a tree, and provides methods to load either local or remote data as child nodes of the root
 * and any descendant non-leaf node.
 *
 * <p>The TreeStore must be used as the store of a <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">tree panel</a>.</p>
 *
 * <p>This class also relays many node events from the underlying node structure.</p>
 *
 * <h1>Using Models</h1>
 *
 * <p>If no Model is specified, an implicit model will be created that extends <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>.
 * The standard Tree fields will also be copied onto the Model for maintaining their state. These fields are listed
 * in the <a href="#!/api/Ext.data.NodeInterface" rel="Ext.data.NodeInterface" class="docClass">Ext.data.NodeInterface</a> documentation.</p>
 *
 * <h1>Reading Nested Data</h1>
 *
 * <p>For the tree to read nested data, the <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> must be configured with a root property,
 * so the reader can find nested data for each node (if a root is not specified, it will default to
 * 'children'). This will tell the tree to look for any nested tree nodes by the same keyword, i.e., 'children'.
 * If a root is specified in the config make sure that any nested nodes with children have the same name.</p>
 *
 * <p><strong>Note:</strong> Setting <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootProperty" rel="Ext.data.TreeStore-cfg-defaultRootProperty" class="docClass">defaultRootProperty</a> accomplishes the same thing.</p>
 *
 * <h1>rootProperty as a Function</h1>
 *
 * <p>You can pass a function as the data reader's rootProperty when the tree's dataset has
 * mixed root properties. Child nodes can then be programmatically determined at read time.</p>
 *
 * <p>For example, the child nodes may be passed via the 'children' property
 * name, though you may have a top-level root property of 'items'.</p>
 *
 * <p>See <a href="#!/api/Ext.data.reader.Reader-cfg-rootProperty" rel="Ext.data.reader.Reader-cfg-rootProperty" class="docClass">rootProperty</a> for more information.</p>
 *
 * <h1>Filtering</h1>
 *
 * <p>Filtering of nodes in a TreeStore is hierarchically top down by default. This means that if a non-leaf node does not
 * pass the filter, then it, and all its descendants are filtered <em>out</em> of the store.</p>
 *
 * <p>To reverse this, so that any node which passes the filter causes all its ancestors to be visible, configure
 * the <code>TreeStore</code> with '<a href="#!/api/Ext.data.TreeStore-cfg-filterer" rel="Ext.data.TreeStore-cfg-filterer" class="docClass">filterer: 'bottomup'</a>`</p>
 *
 * <p>You may also programatically filter individual tree nodes by setting their <code>'visible'</code> field.</p>
 *
 * <p>Setting this to <code>false</code> filters the node out so that it will not appear in the UI. Setting it to <code>true</code>
 * filters the node in.</p>
 *
 * <p>Note that if performing several filter operations, it is best to <a href="#!/api/Ext.data.TreeStore-method-suspendEvents" rel="Ext.data.TreeStore-method-suspendEvents" class="docClass">suspendEvents</a>
 * on the store first, and when all nodes have been modified, <a href="#!/api/Ext.data.TreeStore-method-resumeEvents" rel="Ext.data.TreeStore-method-resumeEvents" class="docClass">resumeEvents</a> and fire the
 * <a href="#!/api/Ext.data.TreeStore-event-refresh" rel="Ext.data.TreeStore-event-refresh" class="docClass">refresh</a> event on the store.</p>
 */
public class TreeStore extends ext.data.Store {
  /**
   * @param config @inheritDoc
   */
  public function TreeStore(config:ext.data.TreeStore = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <code>true</code>, when a node is reloaded, any records in the <a href="#!/api/Ext.data.TreeStore-property-removed" rel="Ext.data.TreeStore-property-removed" class="docClass">removed</a> record collection that were previously descendants of the node being reloaded will be cleared from the <a href="#!/api/Ext.data.TreeStore-property-removed" rel="Ext.data.TreeStore-property-removed" class="docClass">removed</a> collection.
   * Only applicable if <a href="#!/api/Ext.data.TreeStore-cfg-clearOnLoad" rel="Ext.data.TreeStore-cfg-clearOnLoad" class="docClass">clearOnLoad</a> is <code>true</code>.
   * @default true
   * @see #getClearRemovedOnLoad()
   * @see #setClearRemovedOnLoad()
   */
  override public native function get clearRemovedOnLoad():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set clearRemovedOnLoad(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated TreeStore, or subclass thereof.
   * @default true
   */
  public native function get isTreeStore():Boolean;

  /**
   * @private
   */
  public native function set isTreeStore(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Remove previously existing child nodes before loading.
   * @default true
   * @see #getClearOnLoad()
   * @see #setClearOnLoad()
   */
  public native function get clearOnLoad():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set clearOnLoad(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default root id.
   * @default "root"
   * @see #getDefaultRootId()
   * @see #setDefaultRootId()
   */
  public native function get defaultRootId():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultRootId(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @default "children"
   * @see #getDefaultRootProperty()
   * @see #setDefaultRootProperty()
   */
  public native function get defaultRootProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultRootProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default root text (if not specified)
   * @default "Root"
   * @see #getDefaultRootText()
   * @see #setDefaultRootText()
   */
  public native function get defaultRootText():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultRootText(value:String):void;

  [ExtConfig]
  /**
   * The order in which to prioritize how filters are applied to nodes.
   *
   * <p>The default, <code>'topdown'</code> means that if a parent node does <em>not</em> pass the filter, then the branch
   * ends there, and no descendant nodes are filtered in, even if they would pass the filter.</p>
   *
   * <p>By specifying <code>'bottomup'</code>, if a leaf node passes the filter, then all its ancestor nodes are filtered
   * in to allow it to be visible.</p>
   * @default topdown
   */
  public native function get filterer():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set filterer(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to true to automatically prepend a leaf sorter.
   * @default false
   * @see #getFolderSort()
   * @see #setFolderSort()
   */
  public native function get folderSort():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set folderSort(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true to prevent child nodes from being loaded until the the node is
   * expanded or loaded explicitly.
   * @default false
   */
  public native function get lazyFill():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lazyFill(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the parameter sent to the server which contains the identifier of the node.
   * @default "node"
   * @see #getNodeParam()
   * @see #setNodeParam()
   */
  public native function get nodeParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set nodeParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config allows node data to be returned from the server in linear format without having to structure it into <code>children</code>
   * arrays.
   *
   * <p>This property specifies which property name in the raw node data yields the id of the parent node.</p>
   *
   * <p>For example the following data would be read into a geographic tree by configuring the TreeStore with <code>parentIdProperty: 'parentId'</code>.
   * The node data contains an upward link to a parent node.</p>
   *
   * <pre><code>data: [{
   *     name: 'North America',
   *     id: 'NA'
   * }, {
   *     name: 'Unites States',
   *     id: 'USA',
   *     parentId: 'NA'
   * }, {
   *     name: 'Redwood City',
   *     leaf: true,
   *     parentId: 'USA'
   * }, {
   *     name: 'Frederick, MD',
   *     leaf: true,
   *     parentId: 'USA'
   * }]
   * </code></pre>
   * @see #getParentIdProperty()
   * @see #setParentIdProperty()
   */
  public native function get parentIdProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set parentIdProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The root node for this store. For example:
   *
   * <pre><code>root: {
   *     expanded: true,
   *     text: "My Root",
   *     children: [
   *         { text: "Child 1", leaf: true },
   *         { text: "Child 2", expanded: true, children: [
   *             { text: "GrandChild", leaf: true }
   *         ] }
   *     ]
   * }
   * </code></pre>
   *
   * <p>Setting the <code>root</code> config option is the same as calling <a href="#!/api/Ext.data.TreeStore-method-setRootNode" rel="Ext.data.TreeStore-method-setRootNode" class="docClass">setRootNode</a>.</p>
   *
   * <p>It's important to note that setting expanded to true on the root node will cause
   * the tree store to attempt to load.  This will occur regardless the value of
   * <a href="#!/api/Ext.data.ProxyStore-cfg-autoLoad" rel="Ext.data.ProxyStore-cfg-autoLoad" class="docClass">autoLoad</a>. If you you do not want the store
   * to load on instantiation, ensure expanded is false and load the store when you're ready.</p>
   * @see #getRoot()
   * @see #setRoot()
   */
  public native function get root():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set root(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>false</code> to not include the root node in this Stores collection.
   * @default false
   * @see #getRootVisible()
   * @see #setRootVisible()
   */
  public native function get rootVisible():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rootVisible(value:Boolean):void;

  /**
   * Collects unique values for a particular dataIndex from this store.
   * @param dataIndex The property to collect
   * @param options An object which contains options which modify how the store is traversed.
   * @param {Boolean} [options.allowNull] Pass true to allow null, undefined or empty string values.
   * @param {Boolean} [options.filtered] Pass <code>true</code> to collect from all records, even ones which are filtered.
   * @param {Boolean} [options.collapsed] Pass <code>true</code> to include nodes which are descendants of collapsed nodes.
   * <p>Note that the <code>filtered</code> option can also be passed as a separate parameter for
   * compatibility with previous versions.</p>
   * @return An array of the unique values
   */
  override public native function collect(dataIndex:String, options:Object = null):Array;

  /**
   * Calls the specified function for each <a href="#!/api/Ext.data.NodeInterface" rel="Ext.data.NodeInterface" class="docClass">node</a> in the store.
   *
   * <p>When store is filtered, only loops over the filtered records unless the <code>bypassFilters</code> parameter is <code>true</code>.</p>
   * @param fn The function to call. The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Record</a> is passed as the first parameter.
   * Returning <code>false</code> aborts and exits the iteration.
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * Defaults to the current <a href="#!/api/Ext.data.NodeInterface" rel="Ext.data.NodeInterface" class="docClass">node</a> in the iteration.
   * @param includeOptions An object which contains options which modify how the store is traversed.
   * @param {Boolean} [includeOptions.filtered] Pass <code>true</code> to include filtered out nodes in the iteration.
   * @param {Boolean} [includeOptions.collapsed] Pass <code>true</code> to include nodes which are descendants of collapsed nodes.
   * <p>Note that the <code>filtered</code> option can also be passed as a separate parameter for
   * compatibility with previous versions.</p>
   */
  override public native function each(fn:Function, scope:Object = null, includeOptions:Object = null):void;

  /**
   * Marks this store as needing a load. When the current executing event handler exits,
   * this store will send a request to load using its configured <a href="#!/api/Ext.data.TreeStore-cfg-proxy" rel="Ext.data.TreeStore-cfg-proxy" class="docClass">proxy</a>.
   *
   * <p><strong>Be aware that it is not usually valid for a developer to call this method on a TreeStore.</strong></p>
   *
   * <p>TreeStore loads are triggered by a load request from an existing <a href="#!/api/Ext.data.NodeInterface" rel="Ext.data.NodeInterface" class="docClass">tree node</a>,
   * when the node is expanding, and it has no locally defined children in its data.</p>
   *
   * <p><em>Note:</em> Even for synchronous Proxy types such as <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">memory proxy</a>,
   * the result will <em>NOT</em> be available in the following line of code. You must use a callback
   * in the load options, or a <a href="#!/api/Ext.data.TreeStore-event-load" rel="Ext.data.TreeStore-event-load" class="docClass">load listener</a>.</p>
   * @param options This is passed into the <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a>
   * object that is created and then sent to the proxy's <a href="#!/api/Ext.data.proxy.Proxy-method-read" rel="Ext.data.proxy.Proxy-method-read" class="docClass">Ext.data.proxy.Proxy.read</a> function.
   * In addition to the options listed below, this object may contain properties to configure the
   * <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a>.
   * @param {Function} [options.callback] A function which is called when the response arrives.
   * @param {Array} options.callback.records Array of records.
   * @param {ext.data.operation.Operation} options.callback.operation The Operation itself.
   * @param {Boolean} options.callback.success <code>true</code> when operation completed successfully.
   * @param {Boolean} [options.addRecords] Specify as <code>true</code> to <em>add</em> the incoming records rather than the
   * default which is to have the incoming records <em>replace</em> the existing store contents.
   * @default false
   */
  override public native function load(options:Object = null):ext.data.Store;

  /**
   * Reloads the root node of this store.
   * @param options A config object which contains options which may override the options passed to the previous load call. See the
   * <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> method for valid configs.
   */
  override public native function reload(options:Object = null):void;

  /**
   * Finds the first matching node in the tree by a specific field value regardless of visibility
   * due to collapsed states; all nodes present in the tree structure are searched.
   * @param fieldName The name of the Record field to test.
   * @param value Either a string that the field value
   * should begin with, or a RegExp to test against the field.
   * @param anyMatch False to match any part of the string, not just
   * the beginning.
   * @default true
   * @param caseSensitive True for case sensitive comparison
   * @default false
   * @param exactMatch True to force exact match (^ and $ characters
   * added to the regex). Ignored if <code>anyMatch</code> is <code>true</code>.
   * @default false
   * @return The matched node or null
   */
  public native function findNode(fieldName:String, value:*, anyMatch:Boolean = true, caseSensitive:Boolean = false, exactMatch:Boolean = false):ext.data.NodeInterface;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-clearOnLoad" rel="Ext.data.TreeStore-cfg-clearOnLoad" class="docClass">clearOnLoad</a>.
   */
  public native function getClearOnLoad():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootId" rel="Ext.data.TreeStore-cfg-defaultRootId" class="docClass">defaultRootId</a>.
   */
  public native function getDefaultRootId():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootProperty" rel="Ext.data.TreeStore-cfg-defaultRootProperty" class="docClass">defaultRootProperty</a>.
   */
  public native function getDefaultRootProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootText" rel="Ext.data.TreeStore-cfg-defaultRootText" class="docClass">defaultRootText</a>.
   */
  public native function getDefaultRootText():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-folderSort" rel="Ext.data.TreeStore-cfg-folderSort" class="docClass">folderSort</a>.
   */
  public native function getFolderSort():Boolean;

  /**
   * Returns the record node by id regardless of visibility due to collapsed states;
   * all nodes present in the tree structure are available.
   * @param id The id of the node to get.
   */
  public native function getNodeById(id:String):ext.data.NodeInterface;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-nodeParam" rel="Ext.data.TreeStore-cfg-nodeParam" class="docClass">nodeParam</a>.
   */
  public native function getNodeParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-parentIdProperty" rel="Ext.data.TreeStore-cfg-parentIdProperty" class="docClass">parentIdProperty</a>.
   */
  public native function getParentIdProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-root" rel="Ext.data.TreeStore-cfg-root" class="docClass">root</a>.
   */
  public native function getRoot():*;

  [Deprecated(replacement = "getRoot", since = "5.0")]
  /**
   * Returns the root node for this tree.
   */
  public native function getRootNode():ext.data.NodeInterface;

  /**
   * Returns the value of <a href="#!/api/Ext.data.TreeStore-cfg-rootVisible" rel="Ext.data.TreeStore-cfg-rootVisible" class="docClass">rootVisible</a>.
   */
  public native function getRootVisible():Boolean;

  public native function isVisible(node:Object):Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-clearOnLoad" rel="Ext.data.TreeStore-cfg-clearOnLoad" class="docClass">clearOnLoad</a>.
   * @param clearOnLoad The new value.
   */
  public native function setClearOnLoad(clearOnLoad:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootId" rel="Ext.data.TreeStore-cfg-defaultRootId" class="docClass">defaultRootId</a>.
   * @param defaultRootId The new value.
   */
  public native function setDefaultRootId(defaultRootId:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootProperty" rel="Ext.data.TreeStore-cfg-defaultRootProperty" class="docClass">defaultRootProperty</a>.
   * @param defaultRootProperty The new value.
   */
  public native function setDefaultRootProperty(defaultRootProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-defaultRootText" rel="Ext.data.TreeStore-cfg-defaultRootText" class="docClass">defaultRootText</a>.
   * @param defaultRootText The new value.
   */
  public native function setDefaultRootText(defaultRootText:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-folderSort" rel="Ext.data.TreeStore-cfg-folderSort" class="docClass">folderSort</a>.
   * @param folderSort The new value.
   */
  public native function setFolderSort(folderSort:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-nodeParam" rel="Ext.data.TreeStore-cfg-nodeParam" class="docClass">nodeParam</a>.
   * @param nodeParam The new value.
   */
  public native function setNodeParam(nodeParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-parentIdProperty" rel="Ext.data.TreeStore-cfg-parentIdProperty" class="docClass">parentIdProperty</a>.
   * @param parentIdProperty The new value.
   */
  public native function setParentIdProperty(parentIdProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-root" rel="Ext.data.TreeStore-cfg-root" class="docClass">root</a>.
   * @param root The new value.
   */
  public native function setRoot(root:*):void;

  [Deprecated(replacement = "setRoot", since = "5.0")]
  /**
   * Sets the root node for this store.  See also the <a href="#!/api/Ext.data.TreeStore-cfg-root" rel="Ext.data.TreeStore-cfg-root" class="docClass">root</a> config option.
   * @return The new root
   */
  public native function setRootNode(root:*):ext.data.NodeInterface;

  /**
   * Sets the value of <a href="#!/api/Ext.data.TreeStore-cfg-rootVisible" rel="Ext.data.TreeStore-cfg-rootVisible" class="docClass">rootVisible</a>.
   * @param rootVisible The new value.
   */
  public native function setRootVisible(rootVisible:Boolean):void;
}
}