package ext.data {

[Uses("ext.data.Store")]
[Uses("ext.data.LocalStore")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.data.ProxyStore")]
[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.XmlStore", require)]
/**
 * Small helper class to make creating <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>s from XML data easier.
 * A XmlStore will be automatically configured with a <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a>.
 *
 *
 * <p>A store configuration would be something like:
 * <pre><code>var store = new <a href="#!/api/Ext.data.XmlStore" rel="Ext.data.XmlStore" class="docClass">Ext.data.XmlStore</a>({
 *     // store configs
 *     storeId: 'myStore',
 *     url: 'sheldon.xml', // automatically configures a HttpProxy
 *     // reader configs
 *     record: 'Item', // records will have an "Item" tag
 *     idPath: 'ASIN',
 *     totalRecords: '&#64;TotalResults'
 *     fields: [
 *         // set up the fields mapping into the xml doc
 *         // The first needs mapping, the others are very basic
 *         {name: 'Author', mapping: 'ItemAttributes > Author'},
 *         'Title', 'Manufacturer', 'ProductGroup'
 *     ]
 * });
 * </code></pre></p>
 *
 *
 * <p>This store is configured to consume a returned object of the form:
 * <pre><code>&#60?xml version="1.0" encoding="UTF-8"?>
 * &#60ItemSearchResponse xmlns="http://webservices.amazon.com/AWSECommerceService/2009-05-15">
 *     &#60Items>
 *         &#60Request>
 *             &#60IsValid>True&#60/IsValid>
 *             &#60ItemSearchRequest>
 *                 &#60Author>Sidney Sheldon&#60/Author>
 *                 &#60SearchIndex>Books&#60/SearchIndex>
 *             &#60/ItemSearchRequest>
 *         &#60/Request>
 *         &#60TotalResults>203&#60/TotalResults>
 *         &#60TotalPages>21&#60/TotalPages>
 *         &#60Item>
 *             &#60ASIN>0446355453&#60/ASIN>
 *             &#60DetailPageURL>
 *                 http://www.amazon.com/
 *             &#60/DetailPageURL>
 *             &#60ItemAttributes>
 *                 &#60Author>Sidney Sheldon&#60/Author>
 *                 &#60Manufacturer>Warner Books&#60/Manufacturer>
 *                 &#60ProductGroup>Book&#60/ProductGroup>
 *                 &#60Title>Master of the Game&#60/Title>
 *             &#60/ItemAttributes>
 *         &#60/Item>
 *     &#60/Items>
 * &#60/ItemSearchResponse>
 * </code></pre>
 * An object literal of this form could also be used as the <a href="#!/api/Ext.data.XmlStore-cfg-data" rel="Ext.data.XmlStore-cfg-data" class="docClass">data</a> config option.</p>
 *
 *
 * <p><b>Note:</b> This class accepts all of the configuration options of
 * <b><a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">XmlReader</a></b>.</p>
 */
public class XmlStore extends ext.data.Store {
  /**
   * @param config @inheritDoc
   */
  public function XmlStore(config:ext.data.XmlStore = null) {
    super();
  }
}
}