package ext.data.field {

[Uses("ext.data.field.DataField")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.field.Date", require)]
/**
 * This class provides Date specific processing for fields.
 *
 * <p>In previous releases this functionality was integral to the <code>Field</code> base class.</p>
 */
public class DateDataField extends ext.data.field.DataField {
  /**
   * @param config @inheritDoc
   */
  public function DateDataField(config:ext.data.field.DateDataField = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Serves as a default for the <a href="#!/api/Ext.data.field.Date-cfg-dateReadFormat" rel="Ext.data.field.Date-cfg-dateReadFormat" class="docClass">dateReadFormat</a> and <a href="#!/api/Ext.data.field.Date-cfg-dateWriteFormat" rel="Ext.data.field.Date-cfg-dateWriteFormat" class="docClass">dateWriteFormat</a> config options. This
   * will be used in place of those other configurations if not specified.
   *
   * <p>A format string for the <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a> function, or "timestamp" if the value provided by
   * the Reader is a UNIX timestamp, or "time" if the value provided by the Reader is a javascript millisecond
   * timestamp. See <a href="#!/api/Ext.Date" rel="Ext.Date" class="docClass">Ext.Date</a>.</p>
   *
   * <p>It is quite important to note that while this config is optional, it will default to using the base
   * JavaScript Date object's <code>parse</code> function if not specified, rather than <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   * This can cause unexpected issues, especially when converting between timezones, or when converting dates that
   * do not have a timezone specified. The behavior of the native <code>Date.parse</code> is implementation-specific, and
   * depending on the value of the date string, it might return the UTC date or the local date. <strong>For this reason
   * it is strongly recommended that you always specify an explicit date format when parsing dates.</strong></p>
   * @see #getDateFormat()
   */
  public native function get dateFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dateFormat(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Used when converting received data into a Date when the <a href="#!/api/Ext.data.field.Date-property-type" rel="Ext.data.field.Date-property-type" class="docClass">type</a> is specified as <code>"date"</code>.
   * This configuration takes precedence over <a href="#!/api/Ext.data.field.Date-cfg-dateFormat" rel="Ext.data.field.Date-cfg-dateFormat" class="docClass">dateFormat</a>.
   * See <a href="#!/api/Ext.data.field.Date-cfg-dateFormat" rel="Ext.data.field.Date-cfg-dateFormat" class="docClass">dateFormat</a> for more information.
   * @see #getDateReadFormat()
   */
  public native function get dateReadFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dateReadFormat(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Provides a custom format when serializing dates with a <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Ext.data.writer.Writer</a>.
   * If this is not specified, the <a href="#!/api/Ext.data.field.Date-cfg-dateFormat" rel="Ext.data.field.Date-cfg-dateFormat" class="docClass">dateFormat</a> will be used. If no <code>dateFormat</code>
   * is specified, 'timestamp' format is used.
   *
   * <p>See the <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Ext.data.writer.Writer</a> docs for more information on writing dates.</p>
   *
   * <p><strong>Note</strong> It is not possible to use the standard date serialization pathway or <a href="#!/api/Ext-property-USE_NATIVE_JSON" rel="Ext-property-USE_NATIVE_JSON" class="docClass">native browser JSON production</a>
   * to use a <a href="#!/api/Ext.data.writer.Json" rel="Ext.data.writer.Json" class="docClass">JsonWriter</a> to send Microsoft formated
   * "JSON" dates.</p>
   *
   * <p>To use a <a href="#!/api/Ext.data.writer.Json" rel="Ext.data.writer.Json" class="docClass">JsonWriter</a> to write dates in a JSON packet in
   * the form <code>"\/Date(1357372800000)\/"</code> configure the field like this:</p>
   *
   * <p>   {
   *        type: 'date',
   *        dateFormat: 'MS',     // To parse incoming dates from server correctly
   *        serialize: null       // Avoid formatting or conversion by the Writer
   *    }</p>
   *
   * <p>Then override the <code><a href="#!/api/Ext.JSON" rel="Ext.JSON" class="docClass">Ext.JSON</a></code> date serialize function:</p>
   *
   * <p>   <a href="#!/api/Ext.JSON-method-encodeDate" rel="Ext.JSON-method-encodeDate" class="docClass">Ext.JSON.encodeDate</a> = function (d) {
   *        return '"' + <a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a>(d, 'MS') + '"';
   *    };</p>
   * @see #getDateWriteFormat()
   */
  public native function get dateWriteFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dateWriteFormat(value:String):void;

  [ExtConfig]
  /**
   * Used to manually set strict date parsing on a per-field basis. If no <code>useStrict</code>
   * is specified, will use value of Ext.Date.useStrict to determine how to
   * process dates.
   * @since 6.2.0
   */
  public native function get useStrict():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useStrict(value:Boolean):void;

  /**
   * Gets the dateFormat for this field. See <a href="#!/api/Ext.data.field.Date-cfg-dateFormat" rel="Ext.data.field.Date-cfg-dateFormat" class="docClass">dateFormat</a>.
   * @return dateFormat
   */
  public native function getDateFormat():String;

  /**
   * Gets the dateReadFormat for this field. See <a href="#!/api/Ext.data.field.Date-cfg-dateReadFormat" rel="Ext.data.field.Date-cfg-dateReadFormat" class="docClass">dateReadFormat</a>.
   * @return dateReadFormat
   */
  public native function getDateReadFormat():String;

  /**
   * Gets the dateWriteFormat for this field. See <a href="#!/api/Ext.data.field.Date-cfg-dateWriteFormat" rel="Ext.data.field.Date-cfg-dateWriteFormat" class="docClass">dateWriteFormat</a>.
   * @return dateWriteFormat
   */
  public native function getDateWriteFormat():String;
}
}