package ext.data.identifier {

[Uses("ext.data.identifier.GeneratorIdentifier")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.identifier.Sequential", require)]
/**
 * This class is a sequential id generator. A simple use of this class would be like so:
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: 'sequential'
 * });
 * // assign id's of 1, 2, 3, etc.
 * </code></pre>
 *
 * <p>An example of a configured generator would be:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: {
 *         type: 'sequential',
 *         prefix: 'ID_',
 *         seed: 1000,
 *         increment: 10
 *     }
 * });
 * // assign id's of ID_1000, ID_1010, ID_1020, etc.
 * </code></pre>
 */
public class SequentialIdentifier extends ext.data.identifier.GeneratorIdentifier {
  /**
   * @param config @inheritDoc
   */
  public function SequentialIdentifier(config:ext.data.identifier.SequentialIdentifier = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number by which to adjust the <code>seed</code> after for the next sequential id.
   * @default 1
   * @see #getIncrement()
   * @see #setIncrement()
   */
  public native function get increment():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set increment(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The string to place in front of the sequential number for each generated id.
   * @see #getPrefix()
   * @see #setPrefix()
   */
  public native function get prefix():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set prefix(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number at which to start generating sequential id's.
   * @default 1
   * @see #getSeed()
   * @see #setSeed()
   */
  public native function get seed():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set seed(value:Number):void;

  /**
   * Generates and returns the next id.
   * @return The next id. If a <a href="#!/api/Ext.data.identifier.Sequential-cfg-prefix" rel="Ext.data.identifier.Sequential-cfg-prefix" class="docClass">prefix</a> was specified, returns
   * a String, otherwise returns a Number.
   */
  override public native function generate():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.identifier.Sequential-cfg-increment" rel="Ext.data.identifier.Sequential-cfg-increment" class="docClass">increment</a>.
   */
  public native function getIncrement():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.identifier.Sequential-cfg-prefix" rel="Ext.data.identifier.Sequential-cfg-prefix" class="docClass">prefix</a>.
   */
  public native function getPrefix():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.identifier.Sequential-cfg-seed" rel="Ext.data.identifier.Sequential-cfg-seed" class="docClass">seed</a>.
   */
  public native function getSeed():Number;

  /**
   * Sets the value of <a href="#!/api/Ext.data.identifier.Sequential-cfg-increment" rel="Ext.data.identifier.Sequential-cfg-increment" class="docClass">increment</a>.
   * @param increment The new value.
   */
  public native function setIncrement(increment:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.identifier.Sequential-cfg-prefix" rel="Ext.data.identifier.Sequential-cfg-prefix" class="docClass">prefix</a>.
   * @param prefix The new value.
   */
  public native function setPrefix(prefix:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.identifier.Sequential-cfg-seed" rel="Ext.data.identifier.Sequential-cfg-seed" class="docClass">seed</a>.
   * @param seed The new value.
   */
  public native function setSeed(seed:Number):void;
}
}