package ext.data.operation {
import ext.Base;
import ext.data.Batch;
import ext.data.Request;
import ext.data.ResultSet;
import ext.data.proxy.DataProxy;

[Uses("ext.Base")]
[Native("Ext.data.operation.Operation", require)]
/**
 * Represents a read or write operation performed by a <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>.
 * Operation objects are used to enable communication between Stores and Proxies.
 * Application developers should rarely need to interact with Operation objects directly.
 *
 * <p>Several Operations can be batched together in a <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a>.</p>
 */
public class Operation extends ext.Base {
  /**
   * Creates new Operation object.
   * @param config Config object.
   */
  public function Operation(config:ext.data.operation.Operation = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The batch for this operation, if applicable
   * @see #getBatch()
   * @see #setBatch()
   */
  public native function get batch():ext.data.Batch;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set batch(value:ext.data.Batch):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Function to execute when operation completed.
   * <ul>
   *   <li><code>records</code> : Array
   * Array of records.
   *   </li>
   *   <li><code>operation</code> : ext.data.operation.Operation
   * The Operation itself.
   *   </li>
   *   <li><code>success</code> : Boolean
   * True when operation completed successfully.
   *   </li>
   * </ul>
   * @see #getCallback()
   * @see #setCallback()
   */
  public native function get callback():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set callback(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The id of the operation.
   * @see #getId()
   * @see #setId()
   */
  public native function get id():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Parameters to pass along with the request when performing the operation.
   * @see #getParams()
   * @see #setParams()
   */
  public native function get params():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set params(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The proxy for this operation
   * @see #getProxy()
   * @see #setProxy()
   */
  public native function get proxy():ext.data.proxy.DataProxy;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set proxy(value:ext.data.proxy.DataProxy):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The records associated with this operation. If this is a <code>read</code> operation, this will be
   * <code>null</code> until data is returned from the <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Ext.data.proxy.Proxy</a>.
   * @see #getRecords()
   * @see #setRecords()
   */
  public native function get records():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set records(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The request for this operation.
   * @see #getRequest()
   * @see #setRequest()
   */
  public native function get request():ext.data.Request;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set request(value:ext.data.Request):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The ResultSet for this operation.
   * @see #getResultSet()
   * @see #setResultSet()
   */
  public native function get resultSet():ext.data.ResultSet;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set resultSet(value:ext.data.ResultSet):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Scope for the <a href="#!/api/Ext.data.operation.Operation-cfg-callback" rel="Ext.data.operation.Operation-cfg-callback" class="docClass">callback</a> function.
   * @see #getScope()
   * @see #setScope()
   */
  public native function get scope():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True if this Operation is to be executed synchronously. This property is inspected by a
   * <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Batch</a> to see if a series of Operations can be executed in parallel or not.
   * @default false
   * @see #getSynchronous()
   * @see #setSynchronous()
   */
  public native function get synchronous():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set synchronous(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The url for this operation. Typically this will be provided by a proxy and not configured here.
   * @default ''
   * @see #getUrl()
   * @see #setUrl()
   */
  public native function get url():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set url(value:String):void;

  /**
   * Aborts the processing of this operation on the <a href="#!/api/Ext.data.operation.Operation-cfg-proxy" rel="Ext.data.operation.Operation-cfg-proxy" class="docClass">proxy</a>.
   * This is only valid for proxies that make asynchronous requests.
   */
  public native function abort():void;

  /**
   * Checks whether this operation should cause writing to occur.
   * @return Whether the operation should cause a write to occur.
   */
  public native function allowWrite():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-batch" rel="Ext.data.operation.Operation-cfg-batch" class="docClass">batch</a>.
   */
  public native function getBatch():ext.data.Batch;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-callback" rel="Ext.data.operation.Operation-cfg-callback" class="docClass">callback</a>.
   * <ul>
   *   <li><code>records</code> : Array
   * Array of records.
   *   </li>
   *   <li><code>operation</code> : ext.data.operation.Operation
   * The Operation itself.
   *   </li>
   *   <li><code>success</code> : Boolean
   * True when operation completed successfully.
   *   </li>
   * </ul>
   */
  public native function getCallback():Function;

  /**
   * Returns the error string or object that was set using <a href="#!/api/Ext.data.operation.Operation-method-setException" rel="Ext.data.operation.Operation-method-setException" class="docClass">setException</a>
   * @return The error object
   */
  public native function getError():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-id" rel="Ext.data.operation.Operation-cfg-id" class="docClass">id</a>.
   */
  public native function getId():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-params" rel="Ext.data.operation.Operation-cfg-params" class="docClass">params</a>.
   */
  public native function getParams():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-proxy" rel="Ext.data.operation.Operation-cfg-proxy" class="docClass">proxy</a>.
   */
  public native function getProxy():ext.data.proxy.DataProxy;

  /**
   * Returns the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a>s associated with this operation. For read
   * operations the records as set by the <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a> will be
   * returned (returns <code>null</code> if the proxy has not yet set the records).
   *
   * <p>For create, update, and destroy operations the operation's initially configured
   * records will be returned, although the proxy may modify these records' data at some
   * point after the operation is initialized.</p>
   */
  public native function getRecords():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-request" rel="Ext.data.operation.Operation-cfg-request" class="docClass">request</a>.
   */
  public native function getRequest():ext.data.Request;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-resultSet" rel="Ext.data.operation.Operation-cfg-resultSet" class="docClass">resultSet</a>.
   */
  public native function getResultSet():ext.data.ResultSet;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-scope" rel="Ext.data.operation.Operation-cfg-scope" class="docClass">scope</a>.
   */
  public native function getScope():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-synchronous" rel="Ext.data.operation.Operation-cfg-synchronous" class="docClass">synchronous</a>.
   */
  public native function getSynchronous():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Operation-cfg-url" rel="Ext.data.operation.Operation-cfg-url" class="docClass">url</a>.
   */
  public native function getUrl():String;

  /**
   * Returns true if this Operation encountered an exception (see also <a href="#!/api/Ext.data.operation.Operation-method-getError" rel="Ext.data.operation.Operation-method-getError" class="docClass">getError</a>)
   * @return True if there was an exception
   */
  public native function hasException():Boolean;

  /**
   * Returns true if the Operation has been completed
   * @return True if the Operation is complete
   */
  public native function isComplete():Boolean;

  /**
   * Returns true if the Operation has been started but has not yet completed.
   * @return True if the Operation is currently running
   */
  public native function isRunning():Boolean;

  /**
   * Returns true if the Operation has been started. Note that the Operation may have started AND completed, see
   * <a href="#!/api/Ext.data.operation.Operation-method-isRunning" rel="Ext.data.operation.Operation-method-isRunning" class="docClass">isRunning</a> to test if the Operation is currently running.
   * @return True if the Operation has started
   */
  public native function isStarted():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-batch" rel="Ext.data.operation.Operation-cfg-batch" class="docClass">batch</a>.
   * @param batch The new value.
   */
  public native function setBatch(batch:ext.data.Batch):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-callback" rel="Ext.data.operation.Operation-cfg-callback" class="docClass">callback</a>.
   * <ul>
   *   <li><code>records</code> : Array
   * Array of records.
   *   </li>
   *   <li><code>operation</code> : ext.data.operation.Operation
   * The Operation itself.
   *   </li>
   *   <li><code>success</code> : Boolean
   * True when operation completed successfully.
   *   </li>
   * </ul>
   * @param callback The new value.
   */
  public native function setCallback(callback:Function):void;

  /**
   * Marks the Operation as completed.
   */
  public native function setCompleted():void;

  /**
   * Marks the Operation as having experienced an exception. Can be supplied with an option error message/object.
   * @param error error string/object
   */
  public native function setException(error:* = undefined):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-id" rel="Ext.data.operation.Operation-cfg-id" class="docClass">id</a>.
   * @param id The new value.
   */
  public native function setId(id:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-params" rel="Ext.data.operation.Operation-cfg-params" class="docClass">params</a>.
   * @param params The new value.
   */
  public native function setParams(params:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-proxy" rel="Ext.data.operation.Operation-cfg-proxy" class="docClass">proxy</a>.
   * @param proxy The new value.
   */
  public native function setProxy(proxy:ext.data.proxy.DataProxy):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-records" rel="Ext.data.operation.Operation-cfg-records" class="docClass">records</a>.
   * @param records The new value.
   */
  public native function setRecords(records:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-request" rel="Ext.data.operation.Operation-cfg-request" class="docClass">request</a>.
   * @param request The new value.
   */
  public native function setRequest(request:ext.data.Request):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-resultSet" rel="Ext.data.operation.Operation-cfg-resultSet" class="docClass">resultSet</a>.
   * @param resultSet The new value.
   */
  public native function setResultSet(resultSet:ext.data.ResultSet):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-scope" rel="Ext.data.operation.Operation-cfg-scope" class="docClass">scope</a>.
   * @param scope The new value.
   */
  public native function setScope(scope:Object):void;

  /**
   * Marks the Operation as started.
   */
  public native function setStarted():void;

  /**
   * Marks the Operation as successful.
   * @param complete <code>true</code> to also mark this operation
   * as being complete See <a href="#!/api/Ext.data.operation.Operation-method-setCompleted" rel="Ext.data.operation.Operation-method-setCompleted" class="docClass">setCompleted</a>.
   */
  public native function setSuccessful(complete:Boolean = false):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-synchronous" rel="Ext.data.operation.Operation-cfg-synchronous" class="docClass">synchronous</a>.
   * @param synchronous The new value.
   */
  public native function setSynchronous(synchronous:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Operation-cfg-url" rel="Ext.data.operation.Operation-cfg-url" class="docClass">url</a>.
   * @param url The new value.
   */
  public native function setUrl(url:String):void;

  /**
   * Returns true if the Operation has completed and was successful
   * @return True if successful
   */
  public native function wasSuccessful():Boolean;

  /**
   * Process records in the operation after the response is successful and the result
   * set is parsed correctly. The base class implementation of this method is used by
   * "create" and "update" operations to allow the server response to update the client
   * side records.
   * @param resultSet The result set
   * @param request The request
   * @param response The response
   */
  protected native function doProcess(resultSet:ext.data.ResultSet, request:ext.data.Request, response:Object):void;
}
}