package ext.data.operation {
import ext.util.Grouper;

[Uses("ext.data.operation.Operation")]
[Uses("ext.Base")]
[Native("Ext.data.operation.Read", require)]
/**
 * Enacpsulates a read operation as performed by a <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">proxy</a>.
 *
 * <p>This class is instantiated by <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">stores</a> and <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">records</a> and should
 * not need to be instantiated in user code.</p>
 */
public class ReadOperation extends ext.data.operation.Operation {
  /**
   * @param config @inheritDoc
   */
  public function ReadOperation(config:ext.data.operation.ReadOperation = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional array of filter objects. Only applies to 'read' actions.
   * @see #getFilters()
   * @see #setFilters()
   */
  public native function get filters():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set filters(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional grouping configuration. Only applies to 'read' actions where grouping is desired.
   * @see #getGrouper()
   * @see #setGrouper()
   */
  public native function get grouper():ext.util.Grouper;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set grouper(value:ext.util.Grouper):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number of records to load. Used on 'read' actions when paging is being used.
   * @see #getLimit()
   * @see #setLimit()
   */
  public native function get limit():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set limit(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The page for this operation.
   * @see #getPage()
   * @see #setPage()
   */
  public native function get page():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set page(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional array of sorter objects. Only applies to 'read' actions.
   * @see #getSorters()
   * @see #setSorters()
   */
  public native function get sorters():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  [ArrayElementType("ext.util.Sorter")]
  /**
   * @private
   */
  public native function set sorters(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The start index (offset), used in paging when running a 'read' action.
   * @see #getStart()
   * @see #setStart()
   */
  public native function get start():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set start(value:Number):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Read-cfg-filters" rel="Ext.data.operation.Read-cfg-filters" class="docClass">filters</a>.
   */
  public native function getFilters():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Read-cfg-grouper" rel="Ext.data.operation.Read-cfg-grouper" class="docClass">grouper</a>.
   */
  public native function getGrouper():ext.util.Grouper;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Read-cfg-limit" rel="Ext.data.operation.Read-cfg-limit" class="docClass">limit</a>.
   */
  public native function getLimit():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Read-cfg-page" rel="Ext.data.operation.Read-cfg-page" class="docClass">page</a>.
   */
  public native function getPage():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Read-cfg-sorters" rel="Ext.data.operation.Read-cfg-sorters" class="docClass">sorters</a>.
   */
  public native function getSorters():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.operation.Read-cfg-start" rel="Ext.data.operation.Read-cfg-start" class="docClass">start</a>.
   */
  public native function getStart():Number;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Read-cfg-filters" rel="Ext.data.operation.Read-cfg-filters" class="docClass">filters</a>.
   * @param filters The new value.
   */
  public native function setFilters(filters:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Read-cfg-grouper" rel="Ext.data.operation.Read-cfg-grouper" class="docClass">grouper</a>.
   * @param grouper The new value.
   */
  public native function setGrouper(grouper:ext.util.Grouper):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Read-cfg-limit" rel="Ext.data.operation.Read-cfg-limit" class="docClass">limit</a>.
   * @param limit The new value.
   */
  public native function setLimit(limit:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Read-cfg-page" rel="Ext.data.operation.Read-cfg-page" class="docClass">page</a>.
   * @param page The new value.
   */
  public native function setPage(page:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Read-cfg-sorters" rel="Ext.data.operation.Read-cfg-sorters" class="docClass">sorters</a>.
   * @param sorters The new value.
   */
  public native function setSorters(sorters:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.operation.Read-cfg-start" rel="Ext.data.operation.Read-cfg-start" class="docClass">start</a>.
   * @param start The new value.
   */
  public native function setStart(start:Number):void;
}
}