package ext.data.proxy {

[Uses("ext.data.proxy.WebStorageProxy")]
[Uses("ext.data.proxy.ClientProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.SessionStorage", require)]
/**
 * Proxy which uses HTML5 session storage as its data storage/retrieval mechanism. If this proxy is used in a browser
 * where session storage is not supported, the constructor will throw an error. A session storage proxy requires a
 * unique ID which is used as a key in which all record data are stored in the session storage object.
 *
 * <p>It's important to supply this unique ID as it cannot be reliably determined otherwise. If no id is provided but the
 * attached store has a storeId, the storeId will be used. If neither option is presented the proxy will throw an error.</p>
 *
 * <p>Proxies are almost always used with a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">store</a>:</p>
 *
 * <pre><code>new <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>({
 *     proxy: {
 *         type: 'sessionstorage',
 *         id  : 'myProxyKey'
 *     }
 * });
 * </code></pre>
 *
 * <p>Alternatively you can instantiate the Proxy directly:</p>
 *
 * <pre><code>new <a href="#!/api/Ext.data.proxy.SessionStorage" rel="Ext.data.proxy.SessionStorage" class="docClass">Ext.data.proxy.SessionStorage</a>({
 *     id  : 'myOtherProxyKey'
 * });
 * </code></pre>
 *
 * <p>Note that session storage is different to local storage (see <a href="#!/api/Ext.data.proxy.LocalStorage" rel="Ext.data.proxy.LocalStorage" class="docClass">Ext.data.proxy.LocalStorage</a>) - if a browser
 * session is ended (e.g. by closing the browser) then all data in a SessionStorageProxy are lost. Browser restarts
 * don't affect the <a href="#!/api/Ext.data.proxy.LocalStorage" rel="Ext.data.proxy.LocalStorage" class="docClass">Ext.data.proxy.LocalStorage</a> - the data are preserved.</p>
 */
public class SessionStorageProxy extends ext.data.proxy.WebStorageProxy {
  /**
   * @param config @inheritDoc
   */
  public function SessionStorageProxy(config:ext.data.proxy.SessionStorageProxy = null) {
    super();
  }
}
}