package ext.data.proxy {
import ext.data.Model;

[Uses("ext.data.proxy.ClientProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.WebStorage", require)]
[ExcludeClass]
/**
 * WebStorageProxy is simply a superclass for the <a href="#!/api/Ext.data.proxy.LocalStorage" rel="Ext.data.proxy.LocalStorage" class="docClass">LocalStorage</a> and <a href="#!/api/Ext.data.proxy.SessionStorage" rel="Ext.data.proxy.SessionStorage" class="docClass">SessionStorage</a> proxies. It uses the new HTML5 key/value client-side storage objects to
 * save <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">model instances</a> for offline use.
 */
public class WebStorageProxy extends ext.data.proxy.ClientProxy {
  /**
   * Creates the proxy, throws an error if local storage is not supported in the current browser.
   * @param config Config object.
   */
  public function WebStorageProxy(config:ext.data.proxy.WebStorageProxy = null) {
    super();
  }

  /**
   * Cached map of records already retrieved by this Proxy. Ensures that the same instance is always retrieved.
   * @default {}
   */
  public native function get cache():Object;

  /**
   * @private
   */
  public native function set cache(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The unique ID used as the key in which all record data are stored in the local storage object.
   * @see #getId()
   * @see #setId()
   */
  public native function get id():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:String):void;

  /**
   * Destroys all records stored in the proxy and removes all keys and values used to support the proxy from the
   * storage object.
   */
  override public native function clear():void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.WebStorage-cfg-id" rel="Ext.data.proxy.WebStorage-cfg-id" class="docClass">id</a>.
   */
  override public native function getId():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.WebStorage-cfg-id" rel="Ext.data.proxy.WebStorage-cfg-id" class="docClass">id</a>.
   * @param id The new value.
   */
  override public native function setId(id:String):void;

  /**
   * Saves the given record in the Proxy.
   * @param record The model instance
   * @param id The id to save the record under (defaults to the value of the record's getId() function)
   */
  public native function setRecord(record:ext.data.Model, id:String = null):void;
}
}