package ext.data.schema {

[Uses("ext.data.schema.Association")]
[Uses("ext.Base")]
[Native("Ext.data.schema.ManyToOne", require)]
/**
 * <strong>This class is never created directly. It should be constructed through associations in <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.</strong>
 *
 * <p>Declares a relationship between a single entity type and multiple related entities. The relationship can
 * be declared as a keyed or keyless relationship.</p>
 *
 * <pre><code>// Keyed
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Customer', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name']
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ticket', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'title', {
 *         name: 'customerId',
 *         reference: 'Customer'
 *     }]
 * });
 *
 * // Keyless
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Customer', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name'],
 *     hasMany: 'Ticket'
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ticket', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'title']
 * });
 *
 * // Generated methods
 * var customer = new Customer();
 * customer.tickets();
 *
 * var ticket = new Ticket();
 * ticket.getCustomer();
 * ticket.setCustomer();
 * </code></pre>
 *
 * <p>By declaring a keyed relationship, extra functionality is gained that maintains
 * the key field in the model as changes are made to the association.</p>
 *
 * <p>For available configuration options, see <a href="#!/api/Ext.data.schema.Reference" rel="Ext.data.schema.Reference" class="docClass">Ext.data.schema.Reference</a>.
 * The "one" record type will have a generated <a href="#!/api/Ext.data.schema.Association-method-storeGetter" rel="Ext.data.schema.Association-method-storeGetter" class="docClass">Ext.data.schema.Association.storeGetter</a>. The "many" record type
 * will have a <a href="#!/api/Ext.data.schema.Association-method-recordGetter" rel="Ext.data.schema.Association-method-recordGetter" class="docClass">getter</a> and <a href="#!/api/Ext.data.schema.Association-method-recordSetter" rel="Ext.data.schema.Association-method-recordSetter" class="docClass">setter</a>.</p>
 */
public class ManyToOne extends ext.data.schema.Association {
  /**
   * @inheritDoc
   */
  public function ManyToOne(config:ext.data.schema.Association = null) {
    super();
  }
}
}