package ext.data.schema {

[Native("Ext.data.schema.Reference", require)]
/**
 * <strong>This is not a real JavaScript class and cannot be created.  This is for documentation purposes only.</strong>
 *
 * <p>This documentation is for:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.field.Field-cfg-reference" rel="Ext.data.field.Field-cfg-reference" class="docClass">reference config</a></li>
 * </ul>
 *
 *
 * <p>The <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">name</a> of the entity referenced by this field.
 * In most databases, this relationship is represented by a "foreign key". That is, a
 * value for such a field matches the value of the <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">id</a>
 * for an entity of this type.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.Organization', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *      ...
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.User', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      fields: [
 *          { name: 'organizationId', reference: 'Organization' }
 *      ],
 *      ...
 *  });
 * </code></pre>
 *
 * <p>If a <code>reference</code> is not nullable, set the <a href="#!/api/Ext.data.field.Field-cfg-allowBlank" rel="Ext.data.field.Field-cfg-allowBlank" class="docClass">Ext.data.field.Field.allowBlank</a> property
 * to false.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.User', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      fields: [
 *          { name: 'organizationId', reference: 'Organization', allowBlank: false }
 *      ],
 *      ...
 *  });
 * </code></pre>
 *
 * <p>If the name of the generated <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">association</a> or other aspects
 * need to be specified, the <code>reference</code> can be an object. The following usage shows
 * what would be generated by default given the above examples using the string form.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.User', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      fields: [{
 *          name: 'organizationId',
 *          reference: {
 *              type: 'Organization',
 *              association: 'UsersByOrganization',
 *              role: 'organization',
 *              inverse: 'users'
 *          }
 *      }],
 *      ...
 *  });
 * </code></pre>
 *
 * <p>Finally, a <code>reference</code> can also describe ownership between the entities. By default,
 * no ownership relationship is assumed. If, however, the User entities are owned by
 * their Organization, we could say this:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.User', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      fields: [{
 *          name: 'organizationId',
 *          reference: {
 *              parent: 'Organization' // Organization is the parent of User
 *          }
 *      }],
 *      ...
 *  });
 * </code></pre>
 */
public class Reference {
  [ExtConfig]
  /**
   * The name of the association. By default, the name of the association is the
   * capitalized <code>inverse</code> plus "By" plus the capitalized <code>role</code>.
   */
  public native function get association():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set association(value:String):void;

  [ExtConfig]
  /**
   * Set this property instead of <code>reference.type</code> to indicate that the referenced entity
   * is an owned child of this entity. That is, the <code>reference</code> entity should be deleted
   * when this entity is deleted.
   */
  public native function get child():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set child(value:String):void;

  [ExtConfig]
  /**
   * The name of the setter function (if required). Typically this is generated
   * based on the <a href="#!/api/Ext.data.schema.Reference-cfg-role" rel="Ext.data.schema.Reference-cfg-role" class="docClass">role</a>.
   */
  public native function get getterName():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set getterName(value:String):void;

  [ExtConfig]
  /**
   * The name of the inverse role (of this entity with respect to the <code>reference</code>
   * entity). By default, this is the <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">pluralized</a>
   * name of this entity, unless this <code>reference</code> is <code>unique</code>, in which case the default
   * name is the <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">singularized</a> name of this entity.
   *
   * <p>This config may also be an object containing a role, getter, or setter.</p>
   */
  public native function get inverse():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inverse(value:*):void;

  [ExtConfig]
  /**
   * Set this property instead of <code>reference.type</code> to indicate that the referenced entity
   * is the owning parent of this entity. That is, this entity should be deleted when the
   * <code>reference</code> entity is deleted.
   */
  public native function get parent():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set parent(value:String):void;

  [ExtConfig]
  /**
   * A custom <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> configuration for reading
   * nested data.
   */
  public native function get reader():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reader(value:Object):void;

  [ExtConfig]
  /**
   * The name of the role played by the referenced entity. By default, this is the field
   * name (minus its "Id" suffix if present).
   */
  public native function get role():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set role(value:String):void;

  [ExtConfig]
  /**
   * The type which this field references. This is the value set by the string form of
   * <code>reference</code>. If the referenced entity has an ownership relationship this field
   * should be omitted and <code>reference.parent</code> or <code>reference.child</code> should be specified
   * instead.
   */
  public native function get type():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set type(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> True for this reference to create a <code><a href="#!/api/Ext.data.schema.OneToOne" rel="Ext.data.schema.OneToOne" class="docClass">Ext.data.schema.OneToOne</a></code>
   * relationship. If not specified, the default is <code><a href="#!/api/Ext.data.schema.ManyToOne" rel="Ext.data.schema.ManyToOne" class="docClass">Ext.data.schema.ManyToOne</a></code>.
   */
  public native function get unique():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set unique(value:Boolean):void;
}
}