package ext.data.validator {

[Uses("ext.data.validator.Validator")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.validator.Bound", require)]
/**
 * A superclass for a validator that checks if a value is within a certain range.
 */
public class BoundValidator extends ext.data.validator.Validator {
  /**
   * @param config @inheritDoc
   */
  public function BoundValidator(config:ext.data.validator.BoundValidator = null) {
    super();
  }

  [ExtConfig]
  /**
   * The error message to return when the value is not in the specified range
   * and both the minimum and maximum are specified.
   */
  public native function get bothMessage():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bothMessage(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The error message to return when the value is empty.
   * @default 'Must be present'
   * @see #getEmptyMessage()
   * @see #setEmptyMessage()
   */
  public native function get emptyMessage():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set emptyMessage(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum length value.
   * @see #getMax()
   * @see #setMax()
   */
  public native function get max():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set max(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The error message to return when the value is more than the maximum
   * and only a maximum is specified.
   * @see #getMaxOnlyMessage()
   * @see #setMaxOnlyMessage()
   */
  public native function get maxOnlyMessage():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxOnlyMessage(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum length value.
   * @see #getMin()
   * @see #setMin()
   */
  public native function get min():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set min(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The error message to return when the value is less than the minimum
   * and only a minimum is specified.
   * @see #getMinOnlyMessage()
   * @see #setMinOnlyMessage()
   */
  public native function get minOnlyMessage():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minOnlyMessage(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.validator.Bound-cfg-emptyMessage" rel="Ext.data.validator.Bound-cfg-emptyMessage" class="docClass">emptyMessage</a>.
   */
  public native function getEmptyMessage():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.validator.Bound-cfg-max" rel="Ext.data.validator.Bound-cfg-max" class="docClass">max</a>.
   */
  public native function getMax():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.validator.Bound-cfg-maxOnlyMessage" rel="Ext.data.validator.Bound-cfg-maxOnlyMessage" class="docClass">maxOnlyMessage</a>.
   */
  public native function getMaxOnlyMessage():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.validator.Bound-cfg-min" rel="Ext.data.validator.Bound-cfg-min" class="docClass">min</a>.
   */
  public native function getMin():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.validator.Bound-cfg-minOnlyMessage" rel="Ext.data.validator.Bound-cfg-minOnlyMessage" class="docClass">minOnlyMessage</a>.
   */
  public native function getMinOnlyMessage():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.validator.Bound-cfg-emptyMessage" rel="Ext.data.validator.Bound-cfg-emptyMessage" class="docClass">emptyMessage</a>.
   * @param emptyMessage The new value.
   */
  public native function setEmptyMessage(emptyMessage:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.validator.Bound-cfg-max" rel="Ext.data.validator.Bound-cfg-max" class="docClass">max</a>.
   * @param max The new value.
   */
  public native function setMax(max:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.validator.Bound-cfg-maxOnlyMessage" rel="Ext.data.validator.Bound-cfg-maxOnlyMessage" class="docClass">maxOnlyMessage</a>.
   * @param maxOnlyMessage The new value.
   */
  public native function setMaxOnlyMessage(maxOnlyMessage:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.validator.Bound-cfg-min" rel="Ext.data.validator.Bound-cfg-min" class="docClass">min</a>.
   * @param min The new value.
   */
  public native function setMin(min:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.validator.Bound-cfg-minOnlyMessage" rel="Ext.data.validator.Bound-cfg-minOnlyMessage" class="docClass">minOnlyMessage</a>.
   * @param minOnlyMessage The new value.
   */
  public native function setMinOnlyMessage(minOnlyMessage:String):void;
}
}