package ext.data.writer {

[Uses("ext.data.writer.DataWriter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.writer.Xml", require)]
/**
 * This class is used to write <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> data to the server in an XML format.
 * The <a href="#!/api/Ext.data.writer.Xml-cfg-documentRoot" rel="Ext.data.writer.Xml-cfg-documentRoot" class="docClass">documentRoot</a> property is used to specify the root element in the XML document.
 * The <a href="#!/api/Ext.data.writer.Xml-cfg-record" rel="Ext.data.writer.Xml-cfg-record" class="docClass">record</a> option is used to specify the element name for each record that will make up the XML document.
 */
public class XmlWriter extends ext.data.writer.DataWriter {
  /**
   * @param config @inheritDoc
   */
  public function XmlWriter(config:ext.data.writer.XmlWriter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The root to be used if <a href="#!/api/Ext.data.writer.Xml-cfg-documentRoot" rel="Ext.data.writer.Xml-cfg-documentRoot" class="docClass">documentRoot</a> is empty and a root is required
   * to form a valid XML document.
   * @default 'xmlData'
   * @see #getDefaultDocumentRoot()
   * @see #setDefaultDocumentRoot()
   */
  public native function get defaultDocumentRoot():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultDocumentRoot(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the root element of the document. Defaults to <tt>'xmlData'</tt>.
   * If there is more than 1 record and the root is not specified, the default document root will still be used
   * to ensure a valid XML document is created.
   *
   * <p>If the <a href="#!/api/Ext.data.writer.Xml-cfg-record" rel="Ext.data.writer.Xml-cfg-record" class="docClass">record</a> mapping includes a root element name, eg: "SystemInfo>Operation", and
   * the selector includes the root element name, then you must configure this as <code>false</code></p>
   * @default 'xmlData'
   * @see #getDocumentRoot()
   * @see #setDocumentRoot()
   */
  public native function get documentRoot():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set documentRoot(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A header to use in the XML document (such as setting the encoding or version).
   * Defaults to <tt>''</tt>.
   * @default ''
   * @see #getHeader()
   * @see #setHeader()
   */
  public native function get header():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set header(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the node to use for each record. Defaults to
   * the owning <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>'s <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Reader</a>'s
   * <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">Ext.data.reader.Xml.record</a> setting, or <code>'record'</code>.
   * @default 'record'
   * @see #getRecord()
   * @see #setRecord()
   */
  public native function get record():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set record(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Xml-cfg-defaultDocumentRoot" rel="Ext.data.writer.Xml-cfg-defaultDocumentRoot" class="docClass">defaultDocumentRoot</a>.
   */
  public native function getDefaultDocumentRoot():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Xml-cfg-documentRoot" rel="Ext.data.writer.Xml-cfg-documentRoot" class="docClass">documentRoot</a>.
   */
  public native function getDocumentRoot():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Xml-cfg-header" rel="Ext.data.writer.Xml-cfg-header" class="docClass">header</a>.
   */
  public native function getHeader():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Xml-cfg-record" rel="Ext.data.writer.Xml-cfg-record" class="docClass">record</a>.
   */
  public native function getRecord():String;

  /**
   * Serializes an object to XML.
   * Properties will be serialized as child elements unless their first character is <code>'&#64;'</code>
   *
   * <p>For example:</p>
   *
   * <p>   myWriter.objectToElement('SystemComponent', {
   *        "&#64;SystemNumber": '10118795',
   *        "SystemInfo>SystemName": 'Phase Noise Measurement System',
   *        AssetId: 'DE3208',
   *        AgilentModel: 'E5505A',
   *        SerialNumber: 'US44101357',
   *    }, []).join('');</p>
   *
   * <p>becomes</p>
   *
   * <p>   <SystemComponent SystemNumber="10118795">
   *      <SystemInfo>
   *          <SystemName>Phase Noise Measurement System</SystemName>
   *      </SystemInfo>
   *      <AssetId>DE3208</AssetId>
   *      <AgilentModel>E5505A</AgilentModel>
   *      <SerialNumber>US44101357</SerialNumber>
   *    </SystemComponent></p>
   * @param name The element name for the object.
   * @param o The object to serialize.
   * @param output The array into which to serialize the object.
   */
  public native function objectToElement(name:String, o:Object, output:Array = null):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Xml-cfg-defaultDocumentRoot" rel="Ext.data.writer.Xml-cfg-defaultDocumentRoot" class="docClass">defaultDocumentRoot</a>.
   * @param defaultDocumentRoot The new value.
   */
  public native function setDefaultDocumentRoot(defaultDocumentRoot:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Xml-cfg-documentRoot" rel="Ext.data.writer.Xml-cfg-documentRoot" class="docClass">documentRoot</a>.
   * @param documentRoot The new value.
   */
  public native function setDocumentRoot(documentRoot:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Xml-cfg-header" rel="Ext.data.writer.Xml-cfg-header" class="docClass">header</a>.
   * @param header The new value.
   */
  public native function setHeader(header:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Xml-cfg-record" rel="Ext.data.writer.Xml-cfg-record" class="docClass">record</a>.
   * @param record The new value.
   */
  public native function setRecord(record:String):void;
}
}