package ext.dd {
import ext.Base;
import ext.util.Region;
import js.Event;
import js.HTMLElement;

[Uses("ext.Base")]
[Native]
/**
 * DragDropManager is a singleton that tracks the element interaction for
 * all DragDrop items in the window.  Generally, you will not call
 * this class directly, but it does have helper methods that could
 * be useful in your DragDrop implementations.
 * <p>Type of singleton DragDropManager.</p>
 * @see ext.dd.#DragDropManager ext.dd.DragDropManager
 */
public class SDragDropManager extends ext.Base {
  /**
   * In intersect mode, drag and drop interaction is defined by the
   * overlap of two or more drag and drop objects.
   * @default 1
   */
  public native function get INTERSECT():Number;

  /**
   * In point mode, drag and drop interaction is defined by the
   * location of the cursor during the drag/drop
   * @default 0
   */
  public native function get POINT():Number;

  /**
   * The number of pixels that the mouse needs to move after the
   * mousedown before the drag is initiated.  Default=8;
   * defaults to the same value used in the LongPress gesture so that drag cannot be
   * initiated if there is a possible pending longpress
   * @default 8
   */
  public native function get clickPixelThresh():Number;

  /**
   * @private
   */
  public native function set clickPixelThresh(value:Number):void;

  /**
   * Class to add to the <a href="#!/api/Ext.dd.DragDrop-method-getDragEl" rel="Ext.dd.DragDrop-method-getDragEl" class="docClass">dragged element</a> of a DragDrop instance.
   * @default Ext.baseCSSPrefix + 'dd-drag-current'
   */
  public native function get dragCls():String;

  /**
   * The current drag and drop mode.  Default: POINT
   * @default 0
   */
  public native function get mode():Number;

  /**
   * @private
   */
  public native function set mode(value:Number):void;

  /**
   * This config is only provided to provide old, usually unwanted drag/drop behaviour.
   *
   * <p>From ExtJS 4.1.0 onwards, when drop targets are contained in floating, absolutely positioned elements
   * such as in <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Windows</a>, which may overlap each other, <code>over</code> and <code>drop</code> events
   * are only delivered to the topmost drop target at the mouse position.</p>
   *
   * <p>If all targets below that in zIndex order should also receive notifications, set
   * <code>notifyOccluded</code> to <code>true</code>.</p>
   * @default false
   */
  public native function get notifyOccluded():Boolean;

  /**
   * @private
   */
  public native function set notifyOccluded(value:Boolean):void;

  [Deprecated(replacement = "touchAction", since = "6.2.0")]
  /**
   * <code>true</code> to invoke <code>preventDefault()</code> on all events during a drag (may be
   * mouse, touch, or pointer events depending on the platform).
   * @default true
   */
  public native function get preventDefault():Boolean;

  /**
   * @private
   */
  public native function set preventDefault(value:Boolean):void;

  [Deprecated(since = "6.2.0")]
  /**
   * <code>true</code> to invoke <code>stopPropagation()</code> on all events during a drag (may be
   * mouse, touch, or pointer events depending on the platform).
   * @default false
   */
  public native function get stopPropagation():Boolean;

  /**
   * @private
   */
  public native function set stopPropagation(value:Boolean):void;

  /**
   * Set useCache to false if you want to force object the lookup of each
   * drag and drop linked element constantly during a drag.
   * @default true
   */
  public native function get useCache():Boolean;

  /**
   * @private
   */
  public native function set useCache(value:Boolean):void;

  /**
   * Helper function for getting the best match from the list of drag
   * and drop objects returned by the drag and drop events when we are
   * in INTERSECT mode.  It returns either the first object that the
   * cursor is over, or the object that has the greatest overlap with
   * the dragged element.
   * @param dds The array of drag and drop objects
   * targeted
   * @return The best single match
   */
  public native function getBestMatch(dds:Array):ext.dd.DragDrop;

  /**
   * Returns the style property for the DOM element (i.e.,
   * document.getElById(id).style)
   * @param id the id of the elment to get
   * @return The style property of the element
   */
  public native function getCss(id:String):Object;

  /**
   * Returns the DragDrop instance for a given id
   * @param id the id of the DragDrop object
   * @return the drag drop object, null if it is not found
   */
  public native function getDDById(id:String):ext.dd.DragDrop;

  [Deprecated(message = "use Ext.lib.Ext.getDom instead\n")]
  /**
   * Returns the actual DOM element
   * @param id the id of the elment to get
   * @return The element
   */
  public native function getElement(id:String):Object;

  /**
   * Returns a Region object containing the drag and drop element's position
   * and size, including the padding configured for it
   * @param oDD the drag and drop object to get the location for.
   * @return a Region object representing the total area
   * the element occupies, including any padding
   * the instance is configured for.
   */
  public native function getLocation(oDD:ext.dd.DragDrop):ext.util.Region;

  /**
   * Returns the drag and drop instances that are in all groups the
   * passed in instance belongs to.
   * @param p_oDD the obj to get related data for
   * @param bTargetsOnly if true, only return targetable objs
   * @return the related instances
   */
  public native function getRelated(p_oDD:ext.dd.DragDrop, bTargetsOnly:Boolean):Array;

  /**
   * Utility function to determine if a given element has been
   * registered as a drag drop item.
   * @param id the element id to check
   * @return true if this element is a DragDrop item,
   * false otherwise
   */
  public native function isDragDrop(id:String):Boolean;

  /**
   * Utility function to determine if a given element has been
   * registered as a drag drop handle for the given Drag Drop object.
   * @param id the element id to check
   * @return true if this element is a DragDrop handle, false
   * otherwise
   */
  public native function isHandle(id:String):Boolean;

  /**
   * Returns true if the specified dd target is a legal target for
   * the specifice drag obj
   * @param oDD the drag obj
   * @param oTargetDD the target
   * @return true if the target is a legal target for the
   * dd obj
   */
  public native function isLegalTarget(oDD:ext.dd.DragDrop, oTargetDD:ext.dd.DragDrop):Boolean;

  /**
   * Is drag and drop locked?
   * @return True if drag and drop is locked, false otherwise.
   */
  public native function isLocked():Boolean;

  /**
   * My goal is to be able to transparently determine if an object is
   * typeof DragDrop, and the exact subclass of DragDrop.  typeof
   * returns "object", oDD.constructor.toString() always returns
   * "DragDrop" and not the name of the subclass.  So for now it just
   * evaluates a well-known variable in DragDrop.
   * @param oDD The object to evaluate
   * @return true if typeof oDD = DragDrop
   */
  public native function isTypeOfDD(oDD:Object):Boolean;

  /**
   * Lock all drag and drop functionality
   */
  public native function lock():void;

  /**
   * Refreshes the cache of the top-left and bottom-right points of the
   * drag and drop objects in the specified group(s).  This is in the
   * format that is stored in the drag and drop instance, so typical
   * usage is:
   *
   * <pre><code><a href="#!/api/Ext.dd.DragDropManager-method-refreshCache" rel="Ext.dd.DragDropManager-method-refreshCache" class="docClass">Ext.dd.DragDropManager.refreshCache</a>(ddinstance.groups);
   * </code></pre>
   *
   * <p>Alternatively:</p>
   *
   * <pre><code><a href="#!/api/Ext.dd.DragDropManager-method-refreshCache" rel="Ext.dd.DragDropManager-method-refreshCache" class="docClass">Ext.dd.DragDropManager.refreshCache</a>({group1:true, group2:true});
   * </code></pre>
   *
   * <p>TODO: this really should be an indexed array.  Alternatively this
   * method could accept both.</p>
   * @param groups an associative array of groups to refresh
   */
  public native function refreshCache(groups:Object):void;

  /**
   * Each DragDrop instance must be registered with the DragDropManager.
   * This is executed in DragDrop.init()
   * @param oDD the DragDrop object to register
   * @param sGroup the name of the group this element belongs to
   */
  public native function regDragDrop(oDD:ext.dd.DragDrop, sGroup:String):void;

  /**
   * Each DragDrop handle element must be registered.  This is done
   * automatically when executing DragDrop.setHandleElId()
   * @param sDDId the DragDrop id this element is a handle for
   * @param sHandleId the id of the element that is the drag
   * handle
   */
  public native function regHandle(sDDId:String, sHandleId:String):void;

  /**
   * Fired when either the drag pixel threshold or the mousedown hold
   * time threshold has been met.
   * @param x the X position of the original mousedown
   * @param y the Y position of the original mousedown
   */
  public native function startDrag(x:Number, y:Number):void;

  /**
   * Utility to stop event propagation and event default, if these
   * features are turned on.
   * @param e the event as returned by this.getEvent()
   */
  public native function stopEvent(e:js.Event):void;

  /**
   * Unlock all drag and drop functionality
   */
  public native function unlock():void;

  /**
   * This checks to make sure an element exists and is in the DOM.  The
   * main purpose is to handle cases where innerHTML is used to remove
   * drag and drop objects from the DOM.
   * @param el the element to check
   * @return true if the element looks usable
   */
  public native function verifyEl(el:js.HTMLElement):Boolean;
}
}