package ext.dd {
import ext.Base;
import js.Event;

[Uses("ext.Base")]
[Native]
/**
 * Provides easy access to all drag drop components that are registered on a page. Items can be retrieved either
 * directly by DOM node id, or by passing in the drag drop event that occurred and looking up the event target.
 * <p>Type of singleton Registry.</p>
 * @see ext.dd.#Registry ext.dd.Registry
 */
public class SRegistry extends ext.Base {
  /**
   * Returns the handle registered for a DOM Node by id
   * @param id The DOM node or id to look up
   * @return handle The custom handle data
   */
  public native function getHandle(id:*):Object;

  /**
   * Returns the handle that is registered for the DOM node that is the target of the event
   * @param e The event
   * @return handle The custom handle data
   */
  public native function getHandleFromEvent(e:js.Event):Object;

  /**
   * Returns a custom data object that is registered for a DOM node by id
   * @param id The DOM node or id to look up
   * @return data The custom data
   */
  public native function getTarget(id:*):Object;

  /**
   * Returns a custom data object that is registered for the DOM node that is the target of the event
   * @param e The event
   * @return data The custom data
   */
  public native function getTargetFromEvent(e:js.Event):Object;

  /**
   * Registers a drag drop element.
   * @param element The id or DOM node to register
   * @param data An custom data object that will be passed between the elements that are involved in drag
   * drop operations. You can populate this object with any arbitrary properties that your own code knows how to
   * interpret, plus there are some specific properties known to the Registry that should be populated in the data
   * object (if applicable):
   * @param {Array} data.handles Array of DOM nodes that trigger dragging for the element being registered.
   * @param {Boolean} data.isHandle True if the element passed in triggers dragging itself, else false.
   */
  public native function register(element:*, data:Object):void;

  /**
   * Unregister a drag drop element
   * @param element The id or DOM node to unregister
   */
  public native function unregister(element:*):void;
}
}