package ext.dd {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * Provides automatic scrolling of overflow regions in the page during drag operations.
 *
 * <p>The ScrollManager configs will be used as the defaults for any scroll container registered with it, but you can also
 * override most of the configs per scroll container by adding a ddScrollConfig object to the target element that
 * contains these properties: <a href="#!/api/Ext.dd.ScrollManager-property-hthresh" rel="Ext.dd.ScrollManager-property-hthresh" class="docClass">hthresh</a>, <a href="#!/api/Ext.dd.ScrollManager-property-vthresh" rel="Ext.dd.ScrollManager-property-vthresh" class="docClass">vthresh</a>, <a href="#!/api/Ext.dd.ScrollManager-property-increment" rel="Ext.dd.ScrollManager-property-increment" class="docClass">increment</a> and <a href="#!/api/Ext.dd.ScrollManager-property-frequency" rel="Ext.dd.ScrollManager-property-frequency" class="docClass">frequency</a>. Example
 * usage:</p>
 *
 * <pre><code>var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('scroll-ct');
 * el.ddScrollConfig = {
 *     vthresh: 50,
 *     hthresh: -1,
 *     frequency: 100,
 *     increment: 200
 * };
 * <a href="#!/api/Ext.dd.ScrollManager-method-register" rel="Ext.dd.ScrollManager-method-register" class="docClass">Ext.dd.ScrollManager.register</a>(el);
 * </code></pre>
 *
 * <p>Note: This class is designed to be used in "Point Mode</p>
 * <p>Type of singleton ScrollManager.</p>
 * @see ext.dd.#ScrollManager ext.dd.ScrollManager
 */
public class SScrollManager extends ext.Base {
  /**
   * The animation duration in seconds - MUST BE less than <a href="#!/api/Ext.dd.ScrollManager-property-frequency" rel="Ext.dd.ScrollManager-property-frequency" class="docClass">Ext.dd.ScrollManager.frequency</a>!
   * @default 0.4
   */
  public native function get animDuration():Number;

  /**
   * @private
   */
  public native function set animDuration(value:Number):void;

  /**
   * True to animate the scroll
   * @default true
   */
  public native function get animated():Boolean;

  /**
   * @private
   */
  public native function set animated(value:Boolean):void;

  /**
   * The named drag drop <a href="#!/api/Ext.dd.DragSource-cfg-ddGroup" rel="Ext.dd.DragSource-cfg-ddGroup" class="docClass">group</a> to which this container belongs. If a ddGroup is
   * specified, then container scrolling will only occur when a dragged object is in the same ddGroup.
   */
  public native function get ddGroup():String;

  /**
   * @private
   */
  public native function set ddGroup(value:String):void;

  /**
   * The frequency of scrolls in milliseconds
   * @default 500
   */
  public native function get frequency():Number;

  /**
   * @private
   */
  public native function set frequency(value:Number):void;

  /**
   * The number of pixels from the right or left edge of a container the pointer needs to be to trigger scrolling
   */
  public native function get hthresh():Object;

  /**
   * @private
   */
  public native function set hthresh(value:Object):void;

  /**
   * The number of pixels to scroll in each scroll increment
   * @default 100
   */
  public native function get increment():Number;

  /**
   * @private
   */
  public native function set increment(value:Number):void;

  /**
   * The number of pixels from the top or bottom edge of a container the pointer needs to be to trigger scrolling
   */
  public native function get vthresh():Object;

  /**
   * @private
   */
  public native function set vthresh(value:Object):void;

  /**
   * Manually trigger a cache refresh.
   */
  public native function refreshCache():void;

  /**
   * Registers new overflow element(s) to auto scroll
   * @param el The id of or the element to be scrolled or an array of either
   */
  public native function register(el:*):void;

  /**
   * Unregisters overflow element(s) so they are no longer scrolled
   * @param el The id of or the element to be removed or an array of either
   */
  public native function unregister(el:*):void;
}
}