package ext.dd {
import ext.Component;
import ext.dom.Element;

[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.dd.StatusProxy", require)]
/**
 * A specialized floating Component that supports a drop status icon and auto-repair.
 * This is the default drag proxy used by all Ext.dd components.
 */
public class StatusProxy extends ext.Component {
  /**
   * Creates new StatusProxy.
   * @param config Config object.
   */
  public function StatusProxy(config:ext.dd.StatusProxy = null) {
    super();
  }

  [ExtConfig]
  /**
   * The CSS class to apply to the status element when drop is allowed.
   * @default Ext.baseCSSPrefix + 'dd-drop-ok'
   */
  public native function get dropAllowed():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropAllowed(value:String):void;

  [ExtConfig]
  /**
   * The CSS class to apply to the status element when drop is not allowed.
   * @default Ext.baseCSSPrefix + 'dd-drop-nodrop'
   */
  public native function get dropNotAllowed():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropNotAllowed(value:String):void;

  /**
   * Hides the proxy
   * @param animateTarget <strong>only valid for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components
   * such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s or <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">ToolTip</a>s, or regular Components which have
   * been configured with <code>floating: true</code>.</strong>. The target to which the Component should animate while hiding.
   * @default null
   * @param callback A callback function to call after the Component is hidden.
   * @param scope The scope (<code>this</code> reference) in which the callback is executed.
   * Defaults to this Component.
   */
  override public native function hide(animateTarget:Object = null, callback:Function = null, scope:Object = null):void;

  /**
   * Updates the contents of the ghost element
   * @param htmlOrData If this component has been configured with a
   * template via the tpl config then it will use this argument as data to populate the
   * template. If this component was not configured with a template, the components
   * content area will be updated via <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> update.
   * @param loadScripts Only legitimate when using the <code>html</code>
   * configuration. Causes embedded script tags to be executed. Inline source will be executed
   * with this Component as the scope (<code>this</code> reference).
   * @default false
   * @param callback Only legitimate when using the <code>html</code> configuration.
   * Callback to execute when scripts have finished loading.
   * @param scriptScope (Default `this`) The scope (<code>this</code> reference) in which to
   * execute <em>inline</em> script elements content. Scripts with a <code>src</code> attribute cannot
   * be executed with this scope.
   * @default `this`
   */
  override public native function update(htmlOrData:*, loadScripts:Boolean = false, callback:Function = null, scriptScope:Object = null):void;

  /**
   * Returns the ghost element
   * @return el
   */
  public native function getGhost():ext.dom.Element;

  /**
   * Causes the proxy to return to its position of origin via an animation.
   * Should be called after an invalid drop operation by the item being dragged.
   * @param xy The XY position of the element ([x, y])
   * @param callback The function to call after the repair is complete.
   * @param scope The scope (<code>this</code> reference) in which the callback function is executed.
   * Defaults to the browser window.
   */
  public native function repair(xy:Array, callback:Function, scope:Object):void;

  /**
   * Resets the status indicator to the default dropNotAllowed value
   * @param clearGhost True to also remove all content from the ghost, false to preserve it
   */
  public native function reset(clearGhost:Boolean):void;

  /**
   * Updates the proxy's visual element to indicate the status of whether or not drop is allowed
   * over the current target element.
   * @param cssClass The css class for the new drop status indicator image
   */
  public native function setStatus(cssClass:String):void;

  /**
   * Stops the repair animation if it's currently running
   */
  public native function stop():void;

  /**
   * Force the Element to sync its shadow and shim positions
   */
  public native function sync():void;
}
}