package ext.dom {
import ext.Component;
import ext.ElementLoader;
import ext.dd.DD;
import ext.dd.DDProxy;
import ext.dd.DDTarget;
import ext.dd.IDDScrollConfig;
import ext.fx.Anim;
import ext.mixin.Observable;
import ext.util.IPositionable;
import ext.util.KeyMap;
import ext.util.Positionable;
import ext.util.Region;
import js.HTMLElement;

[Uses("ext.mixin.Observable")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Native("Ext.dom.Element", require)]
/**
 * Where supported. Fires when an element is activated, for instance, through a mouse click or a keypress.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_ACTIVATE
 */
[Event(name = 'onDOMActivate', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when an attribute has been modified.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_ATTR_MODIFIED
 */
[Event(name = 'onDOMAttrModified', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when the character data has been modified.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_CHARACTER_DATA_MODIFIED
 */
[Event(name = 'onDOMCharacterDataModified', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Similar to HTML focus event, but can be applied to any focusable element.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_FOCUS_IN
 */
[Event(name = 'onDOMFocusIn', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Similar to HTML blur event, but can be applied to any focusable element.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_FOCUS_OUT
 */
[Event(name = 'onDOMFocusOut', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when a node has been added as a child of another node.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_NODE_INSERTED
 */
[Event(name = 'onDOMNodeInserted', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when a node is being inserted into a document.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_NODE_INSERTED_INTO_DOCUMENT
 */
[Event(name = 'onDOMNodeInsertedIntoDocument', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when a descendant node of the element is removed.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_NODE_REMOVED
 */
[Event(name = 'onDOMNodeRemoved', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when a node is being removed from a document.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_NODE_REMOVED_FROM_DOCUMENT
 */
[Event(name = 'onDOMNodeRemovedFromDocument', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Where supported. Fires when the subtree is modified.
 * @eventType ext.dom.events.Element_e_tEvent.DOM_SUBTREE_MODIFIED
 */
[Event(name = 'onDOMSubtreeModified', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when an object/image is stopped from loading before completely loaded.
 * @eventType ext.dom.events.Element_e_tEvent.ABORT
 */
[Event(name = 'onAbort', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when an element loses focus either via the pointing device or by tabbing navigation.
 * @eventType ext.dom.events.Element_e_tEvent.BLUR
 */
[Event(name = 'onBlur', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a control loses the input focus and its value has been modified since gaining focus.
 * @eventType ext.dom.events.Element_e_tEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a mouse click is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.CLICK
 */
[Event(name = 'onClick', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a right click is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.CONTEXT_MENU
 */
[Event(name = 'onContextMenu', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a mouse double click is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.DBLCLICK
 */
[Event(name = 'onDblclick', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when there is a double tap.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.DOUBLE_TAP
 */
[Event(name = 'onDoubleTap', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when an object/image/frame cannot be loaded properly.
 * @eventType ext.dom.events.Element_e_tEvent.ERROR
 */
[Event(name = 'onError', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when an element receives focus either via the pointing device or by tab navigation.
 * @eventType ext.dom.events.Element_e_tEvent.FOCUS
 */
[Event(name = 'onFocus', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when focus is moved <em>within</em> an element.
 * @eventType ext.dom.events.Element_e_tEvent.FOCUS_MOVE
 */
[Event(name = 'onFocusMove', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a keydown is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.KEY_DOWN
 */
[Event(name = 'onKeyDown', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a keypress is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.KEYPRESS
 */
[Event(name = 'onKeypress', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a keyup is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.KEY_UP
 */
[Event(name = 'onKeyUp', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when the user agent finishes loading all content within the element. Only supported by window, frames,
 * objects and images.
 * @eventType ext.dom.events.Element_e_tEvent.LOAD
 */
[Event(name = 'onLoad', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when you touch and hold still for more than 1 second.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.LONG_PRESS
 */
[Event(name = 'onLongPress', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when a mousedown is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_DOWN
 */
[Event(name = 'onMouseDown', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when the mouse enters the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_ENTER
 */
[Event(name = 'onMouseEnter', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when the mouse leaves the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_LEAVE
 */
[Event(name = 'onMouseLeave', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a mousemove is detected with the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_MOVE
 */
[Event(name = 'onMouseMove', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a mouseout is detected with the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_OUT
 */
[Event(name = 'onMouseOut', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a mouseover is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_OVER
 */
[Event(name = 'onMouseOver', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a mouseup is detected within the element.
 * @eventType ext.dom.events.Element_e_tEvent.MOUSE_UP
 */
[Event(name = 'onMouseUp', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires whenever this Element actually becomes visible (painted) on the screen. This is useful when you need to
 * perform 'read' operations on the DOM element, i.e: calculating natural sizes and positioning.
 *
 * <p><strong>Note:</strong> This event is not available to be used with event delegation. Instead <code>painted</code> only fires if you explicitly
 * add at least one listener to it, for performance reasons.</p>
 * @eventType ext.dom.events.ElementEvent.PAINTED
 */
[Event(name = 'onPainted', type = 'ext.dom.events.ElementEvent')]
/**
 * Fires continuously when there is pinching (the touch must move for this to be fired).
 * @eventType ext.dom.events.Element_event_node_optionsEvent.PINCH
 */
[Event(name = 'onPinch', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when a pinch has ended.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.PINCH_END
 */
[Event(name = 'onPinchEnd', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fired once when a pinch has started.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.PINCH_START
 */
[Event(name = 'onPinchStart', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when a form is reset.
 * @eventType ext.dom.events.Element_e_tEvent.RESET
 */
[Event(name = 'onReset', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Important note: For the best performance on mobile devices, use this only when you absolutely need to monitor
 * a Element's size.
 *
 * <p><strong>Note:</strong> This event is not available to be used with event delegation. Instead <code>resize</code> only fires if you explicitly
 * add at least one listener to it, for performance reasons.</p>
 * @eventType ext.dom.events.ElementEvent.RESIZE
 */
[Event(name = 'onResize', type = 'ext.dom.events.ElementEvent')]
/**
 * Fires continuously when there is rotation (the touch must move for this to be fired).
 * When listening to this, ensure you know about the <a href="#!/api/Ext.event.Event-property-angle" rel="Ext.event.Event-property-angle" class="docClass">Ext.event.Event.angle</a> and <a href="#!/api/Ext.event.Event-property-rotation" rel="Ext.event.Event-property-rotation" class="docClass">Ext.event.Event.rotation</a>
 * properties in the <code>event</code> object.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.ROTATE
 */
[Event(name = 'onRotate', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when a rotation event has ended.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.ROTATE_END
 */
[Event(name = 'onRotateEnd', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fired once when a rotation has started.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.ROTATE_START
 */
[Event(name = 'onRotateStart', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when a document view is scrolled.
 * @eventType ext.dom.events.Element_e_tEvent.SCROLL
 */
[Event(name = 'onScroll', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when a user selects some text in a text field, including input and textarea.
 * @eventType ext.dom.events.Element_e_tEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when there is a single tap.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.SINGLE_TAP
 */
[Event(name = 'onSingleTap', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when a form is submitted.
 * @eventType ext.dom.events.Element_e_tEvent.SUBMIT
 */
[Event(name = 'onSubmit', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Fires when there is a swipe
 * When listening to this, ensure you know about the <a href="#!/api/Ext.event.Event-property-direction" rel="Ext.event.Event-property-direction" class="docClass">Ext.event.Event.direction</a> property in the <code>event</code> object.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.SWIPE
 */
[Event(name = 'onSwipe', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when you touch and hold still for more than 1 second.
 * @eventType ext.dom.events.Element_event_node_optionsEvent.TAP_HOLD
 */
[Event(name = 'onTapHold', type = 'ext.dom.events.Element_event_node_optionsEvent')]
/**
 * Fires when the user agent removes all content from a window or frame. For elements, it fires when the target
 * element or any of its content has been removed.
 * @eventType ext.dom.events.Element_e_tEvent.UNLOAD
 */
[Event(name = 'onUnload', type = 'ext.dom.events.Element_e_tEvent')]
/**
 * Encapsulates a DOM element, adding simple DOM manipulation facilities, normalizing for browser differences.
 *
 * <p><strong>Note:</strong> The events included in this Class are the ones we've found to be the most commonly used. Many events are
 * not listed here due to the expedient rate of change across browsers. For a more comprehensive list, please visit the
 * following resources:</p>
 *
 * <ul>
 * <li><a href="https://developer.mozilla.org/en-US/docs/Web/Events">Mozilla Event Reference Guide</a></li>
 * <li><a href="http://www.w3.org/TR/pointerevents/">W3 Pointer Events</a></li>
 * <li><a href="http://www.w3.org/TR/touch-events/">W3 Touch Events</a></li>
 * <li><a href="http://www.w3.org/TR/DOM-Level-2-Events/">W3 DOM 2 Events</a></li>
 * <li><a href="http://www.w3.org/TR/DOM-Level-3-Events/">W3 DOM 3 Events</a></li>
 * </ul>
 *
 *
 * <h2>Usage</h2>
 *
 * <pre><code>// by id
 * var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>("my-div");
 *
 * // by DOM element reference
 * var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(myDivElement);
 * </code></pre>
 *
 * <h2>Selecting Descendant Elements</h2>
 *
 * <p><a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances can be used to select descendant nodes using CSS selectors.
 * There are 3 methods that can be used for this purpose, each with a slightly different
 * twist:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.dom.Element-method-query" rel="Ext.dom.Element-method-query" class="docClass">query</a></li>
 * <li><a href="#!/api/Ext.dom.Element-method-selectNode" rel="Ext.dom.Element-method-selectNode" class="docClass">selectNode</a></li>
 * <li><a href="#!/api/Ext.dom.Element-method-select" rel="Ext.dom.Element-method-select" class="docClass">select</a></li>
 * </ul>
 *
 *
 * <p>These methods can accept any valid CSS selector since they all use
 * <a href="http://www.w3.org/TR/css3-selectors/">querySelectorAll</a> under the hood. The primary
 * difference between these three methods is their return type:</p>
 *
 * <p>To get an array of HTMLElement instances matching the selector '.foo' use the query
 * method:</p>
 *
 * <pre><code>element.query('.foo');
 * </code></pre>
 *
 * <p>This can easily be transformed into an array of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances by setting
 * the <code>asDom</code> parameter to <code>false</code>:</p>
 *
 * <pre><code>element.query('.foo', false);
 * </code></pre>
 *
 * <p>If the desired result is only the first matching HTMLElement use the selectNode method:</p>
 *
 * <pre><code>element.selectNode('.foo');
 * </code></pre>
 *
 * <p>Once again, the dom node can be wrapped in an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> by setting the <code>asDom</code>
 * parameter to <code>false</code>:</p>
 *
 * <pre><code>element.selectNode('.foo', false);
 * </code></pre>
 *
 * <p>The <code>select</code> method is used when the desired return type is a <a href="#!/api/Ext.dom.CompositeElementLite" rel="Ext.dom.CompositeElementLite" class="docClass">CompositeElementLite</a> or a <a href="#!/api/Ext.dom.CompositeElement" rel="Ext.dom.CompositeElement" class="docClass">CompositeElement</a>.  These are collections of elements that can be operated on as a
 * group using any of the methods of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.  The only difference between the two
 * is that CompositeElementLite is a collection of HTMLElement instances, while
 * CompositeElement is a collection of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances.  To retrieve a
 * CompositeElementLite that represents a collection of HTMLElements for selector '.foo':</p>
 *
 * <pre><code>element.select('.foo');
 * </code></pre>
 *
 * <p>For a <a href="#!/api/Ext.dom.CompositeElement" rel="Ext.dom.CompositeElement" class="docClass">CompositeElement</a> simply pass <code>true</code> as the
 * <code>composite</code> parameter:</p>
 *
 * <pre><code>element.select('.foo', true);
 * </code></pre>
 *
 * <p>The query selection methods can be used even if you don't have a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> to
 * start with For example to select an array of all HTMLElements in the document that match the
 * selector '.foo', simply wrap the document object in an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instance using
 * <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>(document).query('.foo');
 * </code></pre>
 *
 * <h1>Animations</h1>
 *
 * <p>When an element is manipulated, by default there is no animation.</p>
 *
 * <pre><code>var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>("my-div");
 *
 * // no animation
 * el.setWidth(100);
 * </code></pre>
 *
 * <p>specified as boolean (true) for default animation effects.</p>
 *
 * <pre><code>// default animation
 * el.setWidth(100, true);
 * </code></pre>
 *
 * <p>To configure the effects, an object literal with animation options to use as the Element animation configuration
 * object can also be specified. Note that the supported Element animation configuration options are a subset of the
 * <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> animation options specific to Fx effects. The supported Element animation configuration options
 * are:</p>
 *
 * <pre><code>Option    Default   Description
 * --------- --------  ---------------------------------------------
 * duration  350       The duration of the animation in milliseconds
 * easing    easeOut   The easing method
 * callback  none      A function to execute when the anim completes
 * scope     this      The scope (this) of the callback function
 * </code></pre>
 *
 * <p>Usage:</p>
 *
 * <pre><code>// Element animation options object
 * var opt = {
 *     duration: 1000,
 *     easing: 'elasticIn',
 *     callback: this.foo,
 *     scope: this
 * };
 * // animation with some options set
 * el.setWidth(100, opt);
 * </code></pre>
 *
 * <p>The Element animation object being used for the animation will be set on the options object as "anim", which allows
 * you to stop or manipulate the animation. Here is an example:</p>
 *
 * <pre><code>// using the "anim" property to get the Anim object
 * if(opt.anim.isAnimated()){
 *     opt.anim.stop();
 * }
 * </code></pre>
 */
public class Element extends ext.mixin.Observable implements ext.util.IPositionable {
  /**
   * Visibility mode constant for use with <a href="#!/api/Ext.dom.Element-method-setVisibilityMode" rel="Ext.dom.Element-method-setVisibilityMode" class="docClass">setVisibilityMode</a>.
   * Use CSS <code>clip</code> property to reduce element's dimensions to 0px by 0px, effectively
   * making it hidden while not being truly invisible. This is useful when an element
   * needs to be published to the Assistive Technologies such as screen readers.
   * @default 4
   */
  public static native function get CLIP():Number;

  /**
   * Visibility mode constant for use with <a href="#!/api/Ext.dom.Element-method-setVisibilityMode" rel="Ext.dom.Element-method-setVisibilityMode" class="docClass">setVisibilityMode</a>.
   * Use the CSS 'display' property to hide the element.
   * @default 2
   */
  public static native function get DISPLAY():Number;

  /**
   * Visibility mode constant for use with <a href="#!/api/Ext.dom.Element-method-setVisibilityMode" rel="Ext.dom.Element-method-setVisibilityMode" class="docClass">setVisibilityMode</a>.
   * Use CSS absolute positioning and top/left offsets to hide the element.
   * @default 3
   */
  public static native function get OFFSETS():Number;

  /**
   * Visibility mode constant for use with <a href="#!/api/Ext.dom.Element-method-setVisibilityMode" rel="Ext.dom.Element-method-setVisibilityMode" class="docClass">setVisibilityMode</a>.
   * Use the CSS 'visibility' property to hide the element.
   *
   * <p>Note that in this mode, <a href="#!/api/Ext.dom.Element-method-isVisible" rel="Ext.dom.Element-method-isVisible" class="docClass">isVisible</a> may return true
   * for an element even though it actually has a parent element that is hidden. For this
   * reason, and in most cases, using the <a href="#!/api/Ext.dom.Element-static-property-OFFSETS" rel="Ext.dom.Element-static-property-OFFSETS" class="docClass">OFFSETS</a> mode is a better choice.</p>
   * @default 1
   */
  public static native function get VISIBILITY():Number;

  /**
   * Gets the globally shared flyweight Element, with the passed node as the active
   * element. Do not store a reference to this element - the dom node can be overwritten
   * by other code. <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a> is alias for <a href="#!/api/Ext.dom.Element-static-method-fly" rel="Ext.dom.Element-static-method-fly" class="docClass">fly</a>.
   *
   * <p>Use this to make one-time references to DOM elements which are not going to be
   * accessed again either by application code, or by Ext's classes. If accessing an
   * element which will be processed regularly, then <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a> will be
   * more appropriate to take advantage of the caching provided by the
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> class.</p>
   *
   * <p>If this method is called with and id or element that has already been cached by
   * a previous call to <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>() it will return the cached Element instead of the
   * flyweight instance.</p>
   * @param dom The DOM node or <code>id</code>.
   * @param named Allows for creation of named reusable flyweights to prevent
   * conflicts (e.g. internally Ext uses "_global").
   * @return The shared Element object (or <code>null</code> if no matching
   * element was found).
   */
  public static native function fly(dom:*, named:String = null):ext.dom.Element;

  /**
   * Returns the top Element that is located at the passed coordinates taking into account
   * the scroll position of the document.
   * @since 6.2.0
   * @param x The x coordinate
   * @param y The y coordinate
   * @param asDom <code>true</code> to return a DOM element.
   * @default false
   * @return The found element.
   */
  public static native function fromPagePoint(x:Number, y:Number, asDom:Boolean = false):*;

  /**
   * Returns the top Element that is located at the passed coordinates in the current viewport.
   * @param x The x coordinate
   * @param y The y coordinate
   * @param asDom <code>true</code> to return a DOM element.
   * @default false
   * @return The found element.
   */
  public static native function fromPoint(x:Number, y:Number, asDom:Boolean = false):*;

  /**
   * Retrieves <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> objects. <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a> is alias for <a href="#!/api/Ext.dom.Element-static-method-get" rel="Ext.dom.Element-static-method-get" class="docClass">get</a>.
   *
   * <p><strong>This method does not retrieve <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s.</strong> This method retrieves <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * objects which encapsulate DOM elements. To retrieve a Component by its ID, use <a href="#!/api/Ext.ComponentManager-method-get" rel="Ext.ComponentManager-method-get" class="docClass">Ext.ComponentManager.get</a>.</p>
   *
   * <p>When passing an id, it should not include the <code>#</code> character that is used for a css selector.</p>
   *
   * <pre><code>// For an element with id 'foo'
   * <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('foo'); // Correct
   * <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('#foo'); // Incorrect
   * </code></pre>
   *
   * <p>Uses simple caching to consistently return the same object. Automatically fixes if an object was recreated with
   * the same id via AJAX or DOM.</p>
   * @param el The <code>id</code> of the node, a DOM Node or an existing Element.
   * @return The Element object (or <code>null</code> if no matching element was found).
   */
  public static native function get(el:*):ext.dom.Element;

  /**
   * Returns the active element in the DOM. If the browser supports activeElement
   * on the document, this is returned. If not, the focus is tracked and the active
   * element is maintained internally.
   * @param asElement Return <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> instance instead of DOM node.
   * @return The active (focused) element in the document.
   */
  public static native function getActiveElement(asElement:Boolean):js.HTMLElement;

  /**
   * Returns the common ancestor of the two passed elements.
   * @param returnDom Pass <code>true</code> to return a DOM element. Otherwise An <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> will be returned.
   * @return The common ancestor.
   */
  public static native function getCommonAncestor(nodeA:*, nodeB:*, returnDom:Boolean):*;

  /**
   * Retrieves the document height
   * @return documentHeight
   */
  public static native function getDocumentHeight():Number;

  /**
   * Retrieves the document width
   * @return documentWidth
   */
  public static native function getDocumentWidth():Number;

  /**
   * Retrieves the current orientation of the window. This is calculated by
   * determining if the height is greater than the width.
   * @return Orientation of window: 'portrait' or 'landscape'
   */
  public static native function getOrientation():String;

  /**
   * Retrieves the viewport size of the window.
   * @return object containing width and height properties
   */
  public static native function getViewSize():Object;

  /**
   * Retrieves the viewport height of the window.
   * @return viewportHeight
   */
  public static native function getViewportHeight():Number;

  /**
   * Retrieves the viewport width of the window.
   * @return viewportWidth
   */
  public static native function getViewportWidth():Number;

  /**
   * Normalizes CSS property keys from dash delimited to camel case JavaScript Syntax.
   * For example:
   *
   * <ul>
   * <li>border-width -> borderWidth</li>
   * <li>padding-top -> paddingTop</li>
   * </ul>
   * @param prop The property to normalize
   * @return The normalized string
   */
  public static native function normalize(prop:String):String;

  /**
   * Parses a number or string representing margin sizes into an object. Supports CSS-style margin declarations
   * (e.g. 10, "10", "10 10", "10 10 10" and "10 10 10 10" are all valid options and would return the same result)
   * @param box The encoded margins
   * @return An object with margin sizes for top, right, bottom and left containing the unit
   */
  public static native function parseBox(box:*):Object;

  /**
   * Converts a CSS string into an object with a property for each style.
   *
   * <p>The sample code below would return an object with 2 properties, one
   * for background-color and one for color.</p>
   *
   * <pre><code>var css = 'background-color: red; color: blue;';
   * console.log(<a href="#!/api/Ext.dom.Element-static-method-parseStyles" rel="Ext.dom.Element-static-method-parseStyles" class="docClass">Ext.dom.Element.parseStyles</a>(css));
   * </code></pre>
   * @param styles A CSS string
   * @return styles
   */
  public static native function parseStyles(styles:String):Object;

  /**
   * Selects child nodes of a given root based on the passed CSS selector.
   * @param selector The CSS selector.
   * @param asDom <code>false</code> to return an array of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default true
   * @param root The root element of the query or id of
   * the root
   * @return An Array of elements that match
   * the selector.  If there are no matches, an empty Array is returned.
   */
  public static native function query(selector:String, asDom:Boolean = true, root:* = undefined):Array;

  /**
   * Selects elements based on the passed CSS selector to enable
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> methods to be applied to many related
   * elements in one statement through the returned
   * <a href="#!/api/Ext.dom.CompositeElementLite" rel="Ext.dom.CompositeElementLite" class="docClass">CompositeElementLite</a> object.
   * @param selector The CSS selector or an array of
   * elements
   * @param composite Return a CompositeElement as opposed to
   * a CompositeElementLite. Defaults to false.
   * @default false
   * @param root The root element of the query or id of
   * the root
   */
  public static native function select(selector:*, composite:Boolean = false, root:* = undefined):ext.dom.CompositeElementLite;

  /**
   * Serializes a DOM form into a url encoded string
   * @param form The form
   * @return The url encoded form
   */
  public static native function serializeForm(form:Object):String;

  /**
   * Parses a number or string representing margin sizes into an object. Supports CSS-style margin declarations
   * (e.g. 10, "10", "10 10", "10 10 10" and "10 10 10 10" are all valid options and would return the same result)
   * @param box The encoded margins, or an object with top, right,
   * @param units The type of units to add
   * @return An string with unitized (px if units is not specified) metrics for top, right, bottom and left
   */
  public static native function unitizeBox(box:*, units:String):String;

  /**
   * Creates new Element directly by passing an id or the HTMLElement.  This
   * constructor should not be called directly.  Always use <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get()</a>
   * or <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext#fly()</a> instead.
   *
   * <p>In older versions of Ext JS and Sencha Touch this constructor checked to see if
   * there was already an instance of this element in the cache and if so, returned
   * the same instance. As of version 5 this behavior has been removed in order to
   * avoid a redundant cache lookup since the most common path is for the Element
   * constructor to be called from <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get()</a>, which has already
   * checked for a cache entry.</p>
   *
   * <p>Correct way of creating a new <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or retrieving it from the cache):</p>
   *
   * <pre><code>var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('foo'); // by id
   *
   * var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(document.getElementById('foo')); // by DOM reference
   * </code></pre>
   *
   * <p>Incorrect way of creating a new <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a></p>
   *
   * <pre><code>var el = new <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>('foo');
   * </code></pre>
   *
   * <p>For quick and easy access to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> methods use a flyweight:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>('foo').addCls('foo-hovered');
   * </code></pre>
   *
   * <p>This simply attaches the DOM node with id='foo' to the global flyweight Element
   * instance to avoid allocating an extra <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instance.  If, however,
   * the Element instance has already been cached by a previous call to <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(),
   * then <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>() will return the cached Element instance.  For more info see
   * <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>.</p>
   */
  public function Element(idOrHtmlElement:Object) {
    super();
  }

  /**
   * A reference to the <code>Component</code> that owns this element. This is <code>null</code> if there
   * is no direct owner.
   */
  public native function get component():ext.Component;

  /**
   * @private
   */
  public native function set component(value:ext.Component):void;

  /**
   * The DOM element
   */
  public native function get dom():js.HTMLElement;

  /**
   * @private
   */
  public native function set dom(value:js.HTMLElement):void;

  /**
   * Empties this element. Removes all child nodes.
   */
  public native function get empty():Object;

  /**
   * @private
   */
  public native function set empty(value:Object):void;

  /**
   * The DOM element ID
   */
  public native function get id():String;

  /**
   * @private
   */
  public native function set id(value:String):void;

  /**
   * The individual DD scroll config of this Element. If not set, the default
   * settings of <code>ScrollManager</code> are used.
   * <p>Ext AS introduces an interface for scroll configs to improve IDE support of setting
   * individual values. For example, to configure the scrolling behavior of an element <code>el</code>,
   * you could do the following:
   * </p>
   * <pre>
   *   el.ddScrollConfig = IDDScrollConfig({});
   *   scrollConfig.animate = true;
   *   scrollConfig.animDuration = 1.5;
   *   ext.dd.ScrollManager.register(el);
   * </pre>
   * Note that we cast an empty Object to the interface, which would fail in real ActionScript,
   * but does the job of convincing the IDE that the Object can be assigned to property <code>ddScrollConfig</code>
   * and saves us from providing a boiler-plate implementation of the interface.
   *
   * @see ext.dd.ScrollManager
   */
  public native function get ddScrollConfig():ext.dd.IDDScrollConfig;

  /**
   * @private
   */
  public native function set ddScrollConfig(value:ext.dd.IDDScrollConfig):void;

  /**
   * The shadow created by {&#64;link #enableShadow()}.
   *
   * @see #enableShadow()
   */
  public native function get shadow():ext.dom.Shadow;

  /**
   * Destroys this element by removing it from the cache, removing its DOM reference,
   * and removing all of its event listeners.
   */
  override public native function destroy(...params):void;

  /**
   * Starts a custom animation on this Element.
   *
   * <p>The following properties may be specified in <code>from</code>, <code>to</code>, and <code>keyframe</code> objects:</p>
   *
   * <ul>
   * <li><code>x</code> - The page X position in pixels.</li>
   * <li><code>y</code> - The page Y position in pixels</li>
   * <li><code>left</code> - The element's CSS <code>left</code> value. Units must be supplied.</li>
   * <li><code>top</code> - The element's CSS <code>top</code> value. Units must be supplied.</li>
   * <li><code>width</code> - The element's CSS <code>width</code> value. Units must be supplied.</li>
   * <li><code>height</code> - The element's CSS <code>height</code> value. Units must be supplied.</li>
   * <li><code>scrollLeft</code> - The element's <code>scrollLeft</code> value.</li>
   * <li><code>scrollTop</code> - The element's <code>scrollTop</code> value.</li>
   * <li><code>opacity</code> - The element's <code>opacity</code> value (between <code>0</code> and <code>1</code>).</li>
   * </ul>
   *
   *
   * <p><strong>Be aware</strong> that animating an Element which is being used by an Ext Component
   * without in some way informing the Component about the changed element state will
   * result in incorrect Component behaviour. This is because the Component will be
   * using the old state of the element. To avoid this problem, it is now possible
   * to directly animate certain properties of Components.</p>
   * @param config Configuration for <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a>.
   * Note that the <a href="#!/api/Ext.fx.Anim-cfg-to" rel="Ext.fx.Anim-cfg-to" class="docClass">to</a> config is required.
   * @return The new animation.
   */
  public native function addAnimation(config:Object):ext.fx.Anim;

  /**
   * Adds the given CSS class(es) to this Element.
   * @param names The CSS classes to add separated by space,
   * or an array of classes
   * @param prefix Prefix to prepend to each class. The separator <code>-</code> will be
   * appended to the prefix.
   * @param suffix Suffix to append to each class. The separator <code>-</code> will be
   * prepended to the suffix.
   * @return this
   */
  public native function addCls(names:*, prefix:String = null, suffix:String = null):ext.dom.Element;

  /**
   * Sets up event handlers to add and remove a css class when the mouse is down and then up on this element (a click effect)
   * @param className The class to add
   * @param testFn A test function to execute before adding the class. The passed parameter
   * will be the Element instance. If this functions returns false, the class will not be added.
   * @param scope The scope to execute the testFn in.
   * @return this
   */
  public native function addClsOnClick(className:String, testFn:Function = null, scope:Object = null):ext.dom.Element;

  /**
   * Sets up event handlers to add and remove a css class when this element has the focus
   * @param className The class to add
   * @param testFn A test function to execute before adding the class. The passed parameter
   * will be the Element instance. If this functions returns false, the class will not be added.
   * @param scope The scope to execute the testFn in.
   * @return this
   */
  public native function addClsOnFocus(className:String, testFn:Function = null, scope:Object = null):ext.dom.Element;

  /**
   * Sets up event handlers to add and remove a css class when the mouse is over this element
   * @param className The class to add
   * @param testFn A test function to execute before adding the class. The passed parameter
   * will be the Element instance. If this functions returns false, the class will not be added.
   * @param scope The scope to execute the testFn in.
   * @return this
   */
  public native function addClsOnOver(className:String, testFn:Function = null, scope:Object = null):ext.dom.Element;

  /**
   * Convenience method for constructing a KeyMap
   * @param key Either a string with the keys to listen for, the numeric key code,
   * array of key codes or an object with the following options:
   * @param key.key
   * @param {Boolean} key.shift
   * @param {Boolean} key.ctrl
   * @param {Boolean} key.alt
   * @param fn The function to call
   * @param scope The scope (<code>this</code> reference) in which the specified function is executed. Defaults to this Element.
   * @return The KeyMap created
   */
  public native function addKeyListener(key:*, fn:Function, scope:Object = null):ext.util.KeyMap;

  /**
   * Creates a KeyMap for this element
   * @param config The KeyMap config. See <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a> for more details
   * @return The KeyMap created
   */
  public native function addKeyMap(config:Object):ext.util.KeyMap;

  /**
   * Aligns the element with another element relative to the specified anchor points. If
   * the other element is the document it aligns it to the viewport. The position
   * parameter is optional, and can be specified in any one of the following formats:
   *
   * <ul>
   * <li><strong>Blank</strong>: Defaults to aligning the element's top-left corner to the target's
   * bottom-left corner ("tl-bl").</li>
   * <li><strong>One anchor (deprecated)</strong>: The passed anchor position is used as the target
   * element's anchor point.  The element being aligned will position its top-left
   * corner (tl) to that point. <em>This method has been deprecated in favor of the newer
   * two anchor syntax below</em>.</li>
   * <li><strong>Two anchors</strong>: If two values from the table below are passed separated by a dash,
   * the first value is used as the element's anchor point, and the second value is
   * used as the target's anchor point.</li>
   * </ul>
   *
   *
   * <p>In addition to the anchor points, the position parameter also supports the "?"
   * character. If "?" is passed at the end of the position string, the element will
   * attempt to align as specified, but the position will be adjusted to constrain to
   * the viewport if necessary. Note that the element being aligned might be swapped to
   * align to a different position than that specified in order to enforce the viewport
   * constraints. Following are all of the supported anchor positions:</p>
   *
   * <pre>Value  Description
   * -----  -----------------------------
   * tl     The top left corner (default)
   * t      The center of the top edge
   * tr     The top right corner
   * l      The center of the left edge
   * c      In the center of the element
   * r      The center of the right edge
   * bl     The bottom left corner
   * b      The center of the bottom edge
   * br     The bottom right corner
   * </pre>
   *
   *
   * <p>Example Usage:</p>
   *
   * <pre><code>// align el to other-el using the default positioning
   * // ("tl-bl", non-constrained)
   * el.alignTo("other-el");
   *
   * // align the top left corner of el with the top right corner of other-el
   * // (constrained to viewport)
   * el.alignTo("other-el", "tr?");
   *
   * // align the bottom right corner of el with the center left edge of other-el
   * el.alignTo("other-el", "br-l?");
   *
   * // align the center of el with the bottom left corner of other-el and
   * // adjust the x position by -6 pixels (and the y position by 0)
   * el.alignTo("other-el", "c-bl", [-6, 0]);
   * </code></pre>
   *
   * <p><strong>Overridden in Ext.overrides.util.Positionable.</strong></p>
   * @param element The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param position The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * Element animation config object
   */
  public native function alignTo(element:*, position:String = "tl-bl?", offsets:Array = null):void;

  /**
   * Anchors an element to another element and realigns it when the window is resized.
   * @param anchorToEl The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param alignment The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * @param animate true for the default animation or a standard
   * Element animation config object
   * @param monitorScroll True to monitor body scroll and
   * reposition. If this parameter is a number, it is used as the buffer delay in
   * milliseconds.
   * @default 50
   * @param callback The function to call after the animation finishes
   * @return this
   */
  public native function anchorTo(anchorToEl:*, alignment:String = "tl-bl?", offsets:Array = null, animate:* = undefined, monitorScroll:* = 50, callback:Function = null):ext.util.Positionable;

  /**
   * Calls <code><a href="#!/api/Ext.dom.Element-method-addAnimation" rel="Ext.dom.Element-method-addAnimation" class="docClass">addAnimation</a></code> and returns this Element (for call chaining). For
   * details, see <code><a href="#!/api/Ext.dom.Element-method-addAnimation" rel="Ext.dom.Element-method-addAnimation" class="docClass">addAnimation</a></code>.
   * @param config Configuration for <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a>.
   * Note that the <a href="#!/api/Ext.fx.Anim-cfg-to" rel="Ext.fx.Anim-cfg-to" class="docClass">to</a> config is required.
   * @return this
   */
  public native function animate(config:Object):ext.dom.Element;

  /**
   * Appends the passed element(s) to this element
   * @param el The id or element to insert
   * or a DomHelper config
   * @param returnDom True to return the raw DOM element instead
   * of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The inserted <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or
   * HTMLElement if <em>returnDom</em> is <em>true</em>).
   */
  public native function appendChild(el:*, returnDom:Boolean = false):ext.dom.Element;

  /**
   * Appends this element to the passed element.
   * @param el The new parent element.
   * The id of the node, a DOM Node or an existing Element.
   * @return This element.
   */
  public native function appendTo(el:*):ext.dom.Element;

  /**
   * More flexible version of <a href="#!/api/Ext.dom.Element-method-setStyle" rel="Ext.dom.Element-method-setStyle" class="docClass">setStyle</a> for setting style properties.
   *
   * <p>Styles in object form should be a valid DOM element style property.<br/>
   * <a href="http://www.w3schools.com/jsref/dom_obj_style.asp">Valid style property names</a>
   * (<em>along with the supported CSS version for each</em>)</p>
   *
   * <pre><code>// &lt;div id="my-el"&gt;Phineas Flynn&lt;/div&gt;
   *
   * var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('my-el');
   *
   * el.applyStyles('color: white;');
   *
   * el.applyStyles({
   *     fontWeight: 'bold',
   *     backgroundColor: 'gray',
   *     padding: '10px'
   * });
   *
   * el.applyStyles(function () {
   *     if (name.initialConfig.html === 'Phineas Flynn') {
   *         return 'font-style: italic;';
   *         // OR return { fontStyle: 'italic' };
   *     }
   * });
   * </code></pre>
   * @param styles A style specification string, e.g. "width:100px", or object in the form <code>{width:"100px"}</code>, or
   * a function which returns such a specification.
   * @return this
   */
  public native function applyStyles(styles:*):ext.dom.Element;

  /**
   * Tries to blur the element. Any exceptions are caught and ignored.
   * @return this
   */
  public native function blur():ext.dom.Element;

  /**
   * Wraps the specified element with a special 9 element markup/CSS block that renders by default as
   * a gray container with a gradient background, rounded corners and a 4-way shadow.
   *
   * <p>This special markup is used throughout Ext when box wrapping elements (<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>,
   * <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> when <a href="#!/api/Ext.panel.Panel-cfg-frame" rel="Ext.panel.Panel-cfg-frame" class="docClass">frame=true</a>, <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>).
   * The markup is of this form:</p>
   *
   * <pre><code>&lt;div class="{0}-tl"&gt;&lt;div class="{0}-tr"&gt;&lt;div class="{0}-tc"&gt;&lt;/div&gt;&lt;/div&gt;&lt;/div&gt;
   * &lt;div class="{0}-ml"&gt;&lt;div class="{0}-mr"&gt;&lt;div class="{0}-mc"&gt;&lt;/div&gt;&lt;/div&gt;&lt;/div&gt;
   * &lt;div class="{0}-bl"&gt;&lt;div class="{0}-br"&gt;&lt;div class="{0}-bc"&gt;&lt;/div&gt;&lt;/div&gt;&lt;/div&gt;
   * </code></pre>
   *
   * <p>Example usage:</p>
   *
   * <pre><code>// Basic box wrap
   * <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>("foo").boxWrap();
   *
   * // You can also add a custom class and use CSS inheritance rules to customize the box look.
   * // 'x-box-blue' is a built-in alternative -- look at the related CSS definitions as an example
   * // for how to create a custom box wrap style.
   * <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>("foo").boxWrap().addCls("x-box-blue");
   * </code></pre>
   * @param cls A base CSS class to apply to the containing wrapper element.
   * Note that there are a number of CSS rules that are dependent on this name to make the overall effect work,
   * so if you supply an alternate base class, make sure you also supply all of the necessary rules.
   * @default 'x-box'
   * @return The outermost wrapping element of the created box structure.
   */
  public native function boxWrap(cls:String = 'x-box'):ext.dom.Element;

  /**
   * When an element is moved around in the DOM, or is hidden using <code>display:none</code>, it loses layout, and therefore
   * all scroll positions of all descendant elements are lost.
   *
   * <p>This function caches them, and returns a function, which when run will restore the cached positions.
   * In the following example, the Panel is moved from one Container to another which will cause it to lose all scroll positions:</p>
   *
   * <pre><code>var restoreScroll = myPanel.el.cacheScrollValues();
   * myOtherContainer.add(myPanel);
   * restoreScroll();
   * </code></pre>
   * @return A function which will restore all descendant elements of this Element to their scroll
   * positions recorded when this function was executed. Be aware that the returned function is a closure which has
   * captured the scope of <code>cacheScrollValues</code>, so take care to dereference it as soon as not needed - if is it is a <code>var</code>
   * it will drop out of scope, and the reference will be freed.
   */
  public native function cacheScrollValues():Function;

  /**
   * Centers the Element in either the viewport, or another Element.
   * @param centerIn element in
   * which to center the element.
   * @return This element
   */
  public native function center(centerIn:*):ext.dom.Element;

  /**
   * Selects a single <em>direct</em> child based on the passed CSS selector (the selector should not contain an id).
   * @param selector The CSS selector.
   * @param returnDom <code>true</code> to return the DOM node instead of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * @default false
   * @return The child <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or DOM node if <code>returnDom</code> is <code>true</code>)
   */
  public native function child(selector:String, returnDom:Boolean = false):*;

  /**
   * Removes Empty, or whitespace filled text nodes. Combines adjacent text nodes.
   * @param forceReclean By default the element keeps track if it has been cleaned already
   * so you can call this over and over. However, if you update the element and need to force a re-clean, you
   * can pass true.
   * @default false
   */
  public native function clean(forceReclean:Boolean = false):void;

  /**
   * Clears any opacity settings from this element. Required in some cases for IE.
   * @return this
   */
  public native function clearOpacity():ext.dom.Element;

  /**
   * Clears positioning back to the default when the document was loaded.
   * @param value The value to use for the left, right, top, bottom.
   * You could use 'auto'.
   * @default ''
   * @return this
   */
  public native function clearPositioning(value:String = ''):ext.dom.Element;

  /**
   * Store the current overflow setting and clip overflow on the element - use <a href="#!/api/Ext.dom.Element-method-unclip" rel="Ext.dom.Element-method-unclip" class="docClass">unclip</a> to remove
   * @return this
   */
  public native function clip():ext.dom.Element;

  /**
   * Clone this element.
   * @param deep <code>true</code> if the children of the node should also be cloned.
   * @default false
   * @param returnDom <code>true</code> to return the DOM node instead of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * @default false
   * @return The newly cloned <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or DOM node if <code>returnDom</code> is <code>true</code>).
   */
  public native function clone(deep:Boolean = false, returnDom:Boolean = false):*;

  /**
   * Returns <code>true</code> if this element is an ancestor of the passed element, or is
   * the element.
   * @param element The dom element,
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>, or id (string) of the dom element to check.
   * @return True if this element is an ancestor of el or the el itself, else false
   */
  public native function contains(element:*):Boolean;

  /**
   * Creates the passed DomHelper config and appends it to this element or optionally
   * inserts it before the passed child element.
   * @param config DomHelper element config object.  If no tag is specified
   * (e.g., {tag:'input'}) then a div will be automatically generated with the specified
   * attributes.
   * @param insertBefore a child element of this element
   * @param returnDom true to return the dom node instead of creating
   * an Element
   * @default false
   * @return The new child element (or HTMLElement if
   * <em>returnDom</em> is <em>true</em>)
   */
  public native function createChild(config:Object, insertBefore:js.HTMLElement = null, returnDom:Boolean = false):*;

  /**
   * Creates a proxy element of this element
   * @param config The class name of the proxy element or a DomHelper config object
   * @param renderTo The element or element id to render the proxy to. Defaults to: document.body.
   * @param matchBox True to align and size the proxy to this element now.
   * @default false
   * @return The new proxy element
   */
  public native function createProxy(config:*, renderTo:* = undefined, matchBox:Boolean = false):ext.dom.Element;

  /**
   * Translates an element using CSS 3 in 2D.
   */
  public native function cssTranslate():void;

  /**
   * Disables the shadow element created by {&#64;link #enableShadow()}.
   * @see #enableShadow()
   */
  public native function disableShadow():void;

  /**
   * Selects a single child at any depth below this element based on the passed CSS selector (the selector should not contain an id).
   * @param selector The CSS selector
   * @param returnDom <code>true</code> to return the DOM node instead of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The child <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or DOM node if <code>returnDom</code> is <code>true</code>)
   */
  public native function down(selector:String, returnDom:Boolean = false):*;

  /**
   * Convenience method for setVisibilityMode(Element.DISPLAY).
   * @param display What to set display to when visible
   * @return this
   */
  public native function enableDisplayMode(display:String = null):ext.dom.Element;

  /**
   * Enables a shadow element that will always display behind this element
   * @param options Configuration options for the shadow:
   * @param {Number} [options.offset=4] Number of pixels to offset the shadow
   * @param {String} [options.mode='sides'] The shadow display mode.  Supports the following options:
   *     - `'sides'`: Shadow displays on both sides and bottom only
   *     - `'frame'`: Shadow displays equally on all four sides
   *     - `'drop'`: Traditional bottom-right drop shadow
   *     - `'bottom'`: Shadow is offset to the bottom
   *
   * @param {Boolean} [options.animate=false] `true` to animate the shadow while
   * the element is animating.  By default the shadow will be hidden during animation.
   * @param isVisible (private) Components pass isVisible to avoid the extra dom read to determine
   * whether or not this element is visible.
   */
  public native function enableShadow(options:Object = null, isVisible:* = undefined):void;

  /**
   * Fade an element in (from transparent to opaque). The ending opacity can be specified using the <code>opacity</code>
   * config option. Usage:
   *
   * <pre><code>// default: fade in from opacity 0 to 100%
   * el.fadeIn();
   *
   * // custom: fade in from opacity 0 to 75% over 2 seconds
   * el.fadeIn({ opacity: .75, duration: 2000});
   *
   * // common config options shown with default values
   * el.fadeIn({
   *     opacity: 1, //can be any value between 0 and 1 (e.g. .5)
   *     easing: 'easeOut',
   *     duration: 500
   * });
   * </code></pre>
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function fadeIn(options:Object = null):ext.dom.Element;

  /**
   * Fade an element out (from opaque to transparent). The ending opacity can be specified using the <code>opacity</code>
   * config option. Note that IE may require <code>useDisplay:true</code> in order to redisplay correctly.
   * Usage:
   *
   * <pre><code>// default: fade out from the element's current opacity to 0
   * el.fadeOut();
   *
   * // custom: fade out from the element's current opacity to 25% over 2 seconds
   * el.fadeOut({ opacity: .25, duration: 2000});
   *
   * // common config options shown with default values
   * el.fadeOut({
   *     opacity: 0, //can be any value between 0 and 1 (e.g. .5)
   *     easing: 'easeOut',
   *     duration: 500,
   *     remove: false,
   *     useDisplay: false
   * });
   * </code></pre>
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function fadeOut(options:Object = null):ext.dom.Element;

  /**
   * Looks at this node and then at parent nodes for a match of the passed simple selector.
   * @param selector The simple selector to test. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param limit The max depth to search as a number or an element which causes the upward traversal to stop
   * and is <strong>not</strong> considered for inclusion as the result. (defaults to 50 || document.documentElement)
   * @param returnEl True to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> object instead of DOM node
   * @default false
   * @return The matching DOM node (or
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> if <em>returnEl</em> is <em>true</em>).  Or null if no match was found.
   */
  public native function findParent(selector:String, limit:* = undefined, returnEl:Boolean = false):*;

  /**
   * Looks at parent nodes for a match of the passed simple selector.
   * @param selector The simple selector to test. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param limit The max depth to search as a number or an element which causes the upward traversal to stop
   * and is <strong>not</strong> considered for inclusion as the result. (defaults to 50 || document.documentElement)
   * @param returnEl True to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> object instead of DOM node
   * @default false
   * @return The matching DOM node (or
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> if <em>returnEl</em> is <em>true</em>).  Or null if no match was found.
   */
  public native function findParentNode(selector:String, limit:* = undefined, returnEl:Boolean = false):*;

  /**
   * Gets the first child, skipping text nodes
   * @param selector Find the next sibling that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom <code>true</code> to return a raw DOM node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The first child or null
   */
  public native function first(selector:String = null, returnDom:Boolean = false):*;

  /**
   * Try to focus the element either immediately or after a timeout
   * if <code>defer</code> argument is specified.
   * @param defer Milliseconds to defer the focus
   * @return this
   */
  public native function focus(defer:Number = NaN):ext.dom.Element;

  [Deprecated(replacement = "isFocusable", since = "5.0.0")]
  /**
   * Checks whether this element can be focused.
   * @return True if the element is focusable
   */
  public native function focusable():Boolean;

  /**
   * Shows a ripple of exploding, attenuating borders to draw attention to an Element. Usage:
   *
   * <pre><code>// default: a single light blue ripple
   * el.frame();
   *
   * // custom: 3 red ripples lasting 3 seconds total
   * el.frame("#ff0000", 3, { duration: 3000 });
   *
   * // common config options shown with default values
   * el.frame("#C3DAF9", 1, {
   *     duration: 1000 // duration of each individual ripple.
   *     // Note: Easing is not configurable and will be ignored if included
   * });
   * </code></pre>
   * @param color The hex color value for the border.
   * @default '#C3DAF9'
   * @param count The number of ripples to display.
   * @default 1
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function frame(color:String = '#C3DAF9', count:Number = 1, options:Object = null):ext.dom.Element;

  /**
   * Gets the x,y coordinates to align this element with another element. See
   * <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignTo</a> for more info on the supported position values.
   * @param element The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param position The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * @return [x, y]
   */
  public native function getAlignToXY(element:*, position:String = "tl-bl?", offsets:Array = null):Array;

  /**
   * Gets the x,y coordinates specified by the anchor position on the element.
   * @param anchor The specified anchor position.
   * See <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignTo</a> for details on supported anchor positions.
   * @default 'tl'
   * @param local True to get the local (element top/left-relative) anchor
   * position instead of page coordinates
   * @param size An object containing the size to use for calculating anchor
   * position {width: (target width), height: (target height)} (defaults to the
   * element's current size)
   * @return [x, y] An array containing the element's x and y coordinates
   */
  public native function getAnchorXY(anchor:String = 'tl', local:Boolean = false, size:Object = null):Array;

  /**
   * Returns the value of an attribute from the element's underlying DOM node.
   * @param name The attribute name.
   * @param namespace The namespace in which to look for the attribute.
   * @return The attribute value.
   */
  public native function getAttribute(name:String, namespace:String = null):String;

  [Deprecated(replacement = "getAttribute", since = "5.0.0")]
  /**
   * Returns the value of a namespaced attribute from the element's underlying DOM node.
   * @param namespace The namespace in which to look for the attribute
   * @param name The attribute name
   * @return The attribute value
   */
  public native function getAttributeNS(namespace:String, name:String):String;

  /**
   * Returns an object containing a map of all attributes of this element's DOM node.
   * @return Key/value pairs of attribute names and their values.
   */
  public native function getAttributes():Object;

  /**
   * Gets the width of the border(s) for the specified side(s)
   * @param side Can be t, l, r, b or any combination of those to add
   * multiple values. For example, passing <code>'lr'</code> would get the border <strong>l</strong>eft
   * width + the border <strong>r</strong>ight width.
   * @return The width of the sides passed added together
   */
  public native function getBorderWidth(side:String):Number;

  /**
   * Gets the bottom Y coordinate of the element (element Y position + element height)
   * @param local True to get the local css position instead of page
   * coordinate
   */
  public native function getBottom(local:Boolean = false):Number;

  /**
   * Return an object defining the area of this Element which can be passed to
   * <a href="#!/api/Ext.util.Positionable-method-setBox" rel="Ext.util.Positionable-method-setBox" class="docClass">setBox</a> to set another Element's size/location to match this element.
   * @param contentBox If true a box for the content of the element is
   * returned.
   * @param local If true the element's left and top relative to its
   * <code>offsetParent</code> are returned instead of page x/y.
   * @return An object in the format
   * <ul>
   *   <li><code>x</code> : Number
   * The element's X position.
   *   </li>
   *   <li><code>y</code> : Number
   * The element's Y position.
   *   </li>
   *   <li><code>width</code> : Number
   * The element's width.
   *   </li>
   *   <li><code>height</code> : Number
   * The element's height.
   *   </li>
   *   <li><code>bottom</code> : Number
   * The element's lower bound.
   *   </li>
   *   <li><code>right</code> : Number
   * The element's rightmost bound.
   * <p>The returned object may also be addressed as an Array where index 0 contains the X
   * position and index 1 contains the Y position. The result may also be used for
   * <a href="#!/api/Ext.util.Positionable-method-setXY" rel="Ext.util.Positionable-method-setXY" class="docClass">setXY</a></p>
   *   </li>
   * </ul>
   */
  public native function getBox(contentBox:Boolean = false, local:Boolean = false):Object;

  /**
   * Returns a child element of this element given its <code>id</code>.
   * @param id The id of the desired child element.
   * @param asDom True to return the DOM element, false to return a
   * wrapped Element object.
   * @default false
   * @return The child element (or HTMLElement if
   * <em>asDom</em> is <em>true</em>).  Or null if no match was found.
   */
  public native function getById(id:String, asDom:Boolean = false):*;

  [Deprecated(replacement = "getAlignToXY", since = "5.0.0")]
  /**
   * Calculates the x, y to center this element on the screen
   * @return The x, y values [x, y]
   */
  public native function getCenterXY():Array;

  /**
   * Returns a region object that defines the client area of this element.
   *
   * <p>That is, the area <em>within</em> any scrollbars.</p>
   * @return A Region containing "top, left, bottom, right" properties.
   */
  public native function getClientRegion():ext.util.Region;

  [Deprecated(replacement = "getHeight", since = "5.0.0")]
  /**
   * Returns either the offsetHeight or the height of this element based on CSS height adjusted by padding or borders
   * when needed to simulate offsetHeight when offsets aren't available. This may not work on display:none elements
   * if a height has not been set using CSS.
   */
  public native function getComputedHeight():Number;

  [Deprecated(replacement = "getWidth", since = "5.0.0")]
  /**
   * Returns either the offsetWidth or the width of this element based on CSS width adjusted by padding or borders
   * when needed to simulate offsetWidth when offsets aren't available. This may not work on display:none elements
   * if a width has not been set using CSS.
   */
  public native function getComputedWidth():Number;

  /**
   * Returns the content region of this element for purposes of constraining or clipping floating
   * children.  That is the region within the borders and scrollbars, but not within the padding.
   */
  public native function getConstrainRegion():ext.util.Region;

  /**
   * Returns the <code>[X, Y]</code> vector by which this Positionable's element must be translated to make a best
   * attempt to constrain within the passed constraint. Returns <code>false</code> if the element
   * does not need to be moved.
   *
   * <p>Priority is given to constraining the top and left within the constraint.</p>
   *
   * <p>The constraint may either be an existing element into which the element is to be
   * constrained, or a <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> into which this element is to be
   * constrained.</p>
   *
   * <p>By default, any extra shadow around the element is <strong>not</strong> included in the constrain calculations - the edges
   * of the element are used as the element bounds. To constrain the shadow within the constrain region, set the
   * <code>constrainShadow</code> property on this element to <code>true</code>.</p>
   * @param constrainTo The
   * Positionable, HTMLElement, element id, or Region into which the element is to be
   * constrained.
   * @param proposedPosition A proposed <code>[X, Y]</code> position to test for validity
   * and to produce a vector for instead of using the element's current position
   * @param proposedSize A proposed <code>[width, height]</code> size to constrain
   * instead of using the element's current size
   * @return <strong>If</strong> the element <em>needs</em> to be translated, an <code>[X, Y]</code>
   * vector by which this element must be translated. Otherwise, <code>false</code>.
   */
  public native function getConstrainVector(constrainTo:* = undefined, proposedPosition:Array = null, proposedSize:Array = null):*;

  [Deprecated(replacement = "getHtml", since = "5.0.0")]
  /**
   * Returns the <code>innerHTML</code> of an Element or an empty string if the element's
   * dom no longer exists.
   */
  public native function getHTML():String;

  /**
   * Returns the offset height of the element.
   * @param contentHeight <code>true</code> to get the height minus borders and padding.
   * @return The element's height.
   */
  public native function getHeight(contentHeight:Boolean = false):Number;

  /**
   * Returns the <code>innerHTML</code> of an Element or an empty string if the element's
   * dom no longer exists.
   */
  public native function getHtml():String;

  /**
   * Gets the left X coordinate
   * @param local True to get the local css position instead of
   * page coordinate
   */
  public native function getLeft(local:Boolean = false):Number;

  /**
   * Gets this element's <a href="#!/api/Ext.ElementLoader" rel="Ext.ElementLoader" class="docClass">ElementLoader</a>
   * @return The loader
   */
  public native function getLoader():ext.ElementLoader;

  /**
   * Returns the x coordinate of this element reletive to its <code>offsetParent</code>.
   * @return The local x coordinate
   */
  public native function getLocalX():Number;

  /**
   * Returns the x and y coordinates of this element relative to its <code>offsetParent</code>.
   * @return The local XY position of the element
   */
  public native function getLocalXY():Array;

  /**
   * Returns the y coordinate of this element reletive to its <code>offsetParent</code>.
   * @return The local y coordinate
   */
  public native function getLocalY():Number;

  /**
   * Returns an object with properties top, left, right and bottom representing the margins of this element unless sides is passed,
   * then it returns the calculated width of the sides (see <a href="#!/api/Ext.dom.Element-method-getPadding" rel="Ext.dom.Element-method-getPadding" class="docClass">getPadding</a>).
   * @param sides Any combination of 'l', 'r', 't', 'b' to get the sum of those sides.
   */
  public native function getMargin(sides:String = null):*;

  /**
   * Returns the offsets of this element from the passed element. The element must both
   * be part of the DOM tree and not have display:none to have page coordinates.
   * @param offsetsTo The Positionable,
   * HTMLElement, or element id to get get the offsets from.
   * @return The XY page offsets (e.g. <code>[100, -200]</code>)
   */
  public native function getOffsetsTo(offsetsTo:*):Array;

  /**
   * Gets the width of the padding(s) for the specified side(s).
   * @param side Can be t, l, r, b or any combination of those to add
   * multiple values. For example, passing <code>'lr'</code> would get the padding <strong>l</strong>eft +
   * the padding <strong>r</strong>ight.
   * @return The padding of the sides passed added together.
   */
  public native function getPadding(side:String):Number;

  [Deprecated(replacement = "getBox", since = "5.0.0")]
  /**
   * Returns an object defining the area of this Element which can be passed to
   * <a href="#!/api/Ext.util.Positionable-method-setBox" rel="Ext.util.Positionable-method-setBox" class="docClass">Ext.util.Positionable.setBox</a> to set another Element's size/location to match this element.
   * @param asRegion If true an <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Ext.util.Region</a> will be returned
   * @return box An object in the following format:
   * <pre><code>{
   *     left: &lt;Element's X position&gt;,
   *     top: &lt;Element's Y position&gt;,
   *     width: &lt;Element's width&gt;,
   *     height: &lt;Element's height&gt;,
   *     bottom: &lt;Element's lower bound&gt;,
   *     right: &lt;Element's rightmost bound&gt;
   * }
   * </code></pre>
   * <p>The returned object may also be addressed as an Array where index 0 contains
   * the X position and index 1 contains the Y position. So the result may also be
   * used for <a href="#!/api/Ext.dom.Element-method-setXY" rel="Ext.dom.Element-method-setXY" class="docClass">setXY</a></p>
   */
  public native function getPageBox(asRegion:Boolean = false):*;

  /**
   * Gets an object with all CSS positioning properties. Useful along with
   * <code>setPostioning</code> to get snapshot before performing an update and then restoring
   * the element.
   * @param autoPx true to return pixel values for "auto" styles.
   * @default false
   */
  public native function getPositioning(autoPx:Boolean = false):Object;

  /**
   * Returns a region object that defines the area of this element.
   * @param contentBox If true a box for the content of the element is
   * returned.
   * @return A Region containing "top, left, bottom, right" properties.
   */
  public native function getRegion(contentBox:Boolean = false):ext.util.Region;

  /**
   * Gets the right X coordinate of the element (element X position + element width)
   * @param local True to get the local css position instead of page
   * coordinates
   */
  public native function getRight(local:Boolean = false):Number;

  /**
   * Returns the current scroll position of the element.
   * @return An object containing the scroll position in the format
   * <code>{left: (scrollLeft), top: (scrollTop)}</code>
   */
  public native function getScroll():Object;

  /**
   * Gets the left scroll position
   * @return The left scroll position
   */
  public native function getScrollLeft():Number;

  /**
   * Gets the top scroll position
   * @return The top scroll position
   */
  public native function getScrollTop():Number;

  /**
   * Returns the size of the element.
   * @param contentSize <code>true</code> to get the width/size minus borders and padding.
   * @return An object containing the element's size:
   * <ul>
   *   <li><code>width</code> : Number</li>
   *   <li><code>height</code> : Number</li>
   * </ul>
   */
  public native function getSize(contentSize:Boolean = false):Object;

  /**
   * Returns a named style property based on computed/currentStyle (primary) and
   * inline-style if primary is not available.
   * @param property The style property (or multiple property names
   * in an array) whose value is returned.
   * @param inline if <code>true</code> only inline styles will be returned.
   * @default false
   * @return The current value of the style property for this element
   * (or a hash of named style values if multiple property arguments are requested).
   */
  public native function getStyle(property:*, inline:Boolean = false):*;

  [Deprecated(replacement = "getSize", since = "5.0.0")]
  /**
   * Returns the dimensions of the element available to lay content out in.
   *
   * <p>getStyleSize utilizes prefers style sizing if present, otherwise it chooses the larger of offsetHeight/clientHeight and
   * offsetWidth/clientWidth. To obtain the size excluding scrollbars, use getViewSize.</p>
   *
   * <p>Sizing of the document body is handled at the adapter level which handles special cases for IE and strict modes, etc.</p>
   * @return Object describing width and height.
   * <ul>
   *   <li><code>width</code> : Number</li>
   *   <li><code>height</code> : Number</li>
   * </ul>
   */
  public native function getStyleSize():Object;

  /**
   * Returns the width in pixels of the passed text, or the width of the text in this Element.
   * @param text The text to measure. Defaults to the innerHTML of the element.
   * @param min The minumum value to return.
   * @param max The maximum value to return.
   * @return The text width in pixels.
   */
  public native function getTextWidth(text:String, min:Number = NaN, max:Number = NaN):Number;

  /**
   * Gets the top Y coordinate
   * @param local True to get the local css position instead of page
   * coordinates
   */
  public native function getTop(local:Boolean = false):Number;

  /**
   * Returns this element's touch action.  (see <a href="#!/api/Ext.dom.Element-method-setTouchAction" rel="Ext.dom.Element-method-setTouchAction" class="docClass">setTouchAction</a>)
   *
   * <p>The returned object is shared and should not be mutated.</p>
   */
  public native function getTouchAction():Object;

  /**
   * Returns the value of the <code>value</code> attribute.
   * @param asNumber <code>true</code> to parse the value as a number.
   */
  public native function getValue(asNumber:Boolean):*;

  /**
   * Returns the <strong>content</strong> region of this element. That is the region within the borders
   * and padding.
   * @return A Region containing "top, left, bottom, right" member data.
   */
  public native function getViewRegion():ext.util.Region;

  /**
   * Returns the dimensions of the element available to lay content out in.  For
   * most elements this is the clientHeight/clientWidth.  If the element is
   * the document/document.body the window's innerHeight/innerWidth is returned
   *
   * <p>If the element (or any ancestor element) has CSS style <code>display: none</code>, the
   * dimensions will be zero.</p>
   * @return Object describing width and height.
   * <ul>
   *   <li><code>width</code> : Number</li>
   *   <li><code>height</code> : Number</li>
   * </ul>
   */
  public native function getViewSize():Object;

  /**
   * Returns the offset width of the element.
   * @param contentWidth <code>true</code> to get the width minus borders and padding.
   * @return The element's width.
   */
  public native function getWidth(contentWidth:Boolean = false):Number;

  /**
   * Gets element X position in page coordinates
   */
  public native function getX():Number;

  /**
   * Gets element X and Y positions in page coordinates
   * @return [x, y]
   */
  public native function getXY():Array;

  /**
   * Gets element Y position in page coordinates
   */
  public native function getY():Number;

  /**
   * Returns this element's z-index
   */
  public native function getZIndex():Number;

  /**
   * Slides the element while fading it out of view. An anchor point can be optionally passed to set the ending point
   * of the effect. Usage:
   *
   * <pre><code>// default: slide the element downward while fading out
   * el.ghost();
   *
   * // custom: slide the element out to the right with a 2-second duration
   * el.ghost('r', { duration: 2000 });
   *
   * // common config options shown with default values
   * el.ghost('b', {
   *     easing: 'easeOut',
   *     duration: 500
   * });
   * </code></pre>
   * @param anchor One of the valid <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> anchor positions (defaults to bottom: 'b')
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function ghost(anchor:String = null, options:Object = null):ext.dom.Element;

  /**
   * Checks if the specified CSS class exists on this element's DOM node.
   * @param name The CSS class to check for.
   * @return <code>true</code> if the class exists, else <code>false</code>.
   */
  public native function hasCls(name:String):Boolean;

  /**
   * Hide this element - Uses display mode to determine whether to use "display",
   * "visibility", "offsets", or "clip". See <a href="#!/api/Ext.dom.Element-method-setVisible" rel="Ext.dom.Element-method-setVisible" class="docClass">setVisible</a>.
   * @param animate true for the default animation or a standard
   * Element animation config object
   * @return this
   */
  public native function hide(animate:* = undefined):ext.dom.Element;

  /**
   * Highlights the Element by setting a color (applies to the background-color by default, but can be changed using
   * the "attr" config option) and then fading back to the original color. If no original color is available, you
   * should provide the "endColor" config option which will be cleared after the animation. Usage:
   *
   * <pre><code>// default: highlight background to yellow
   * el.highlight();
   *
   * // custom: highlight foreground text to blue for 2 seconds
   * el.highlight("0000ff", { attr: 'color', duration: 2000 });
   *
   * // common config options shown with default values
   * el.highlight("ffff9c", {
   *     attr: "backgroundColor", //can be any valid CSS property (attribute) that supports a color value
   *     endColor: (current color) or "ffffff",
   *     easing: 'easeIn',
   *     duration: 1000
   * });
   * </code></pre>
   * @param color The highlight color. Should be a 6 char hex color without the leading #
   * (defaults to yellow: 'ffff9c')
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function highlight(color:String = null, options:Object = null):ext.dom.Element;

  /**
   * Sets up event handlers to call the passed functions when the mouse is moved into and out of the Element.
   * @param overFn The function to call when the mouse enters the Element.
   * @param outFn The function to call when the mouse leaves the Element.
   * @param scope The scope (<code>this</code> reference) in which the functions are executed. Defaults
   * to the Element's DOM element.
   * @param options Options for the listener. See <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">the
   * options parameter</a>.
   * @return this
   */
  public native function hover(overFn:Function, outFn:Function, scope:Object = null, options:Object = null):ext.dom.Element;

  /**
   * Initializes a <a href="#!/api/Ext.dd.DD" rel="Ext.dd.DD" class="docClass">Ext.dd.DD</a> drag drop object for this element.
   * @param group The group the DD object is member of
   * @param config The DD config object
   * @param overrides An object containing methods to override/implement on the DD object
   * @return The DD object
   */
  public native function initDD(group:String, config:Object, overrides:Object):ext.dd.DD;

  /**
   * Initializes a <a href="#!/api/Ext.dd.DDProxy" rel="Ext.dd.DDProxy" class="docClass">Ext.dd.DDProxy</a> object for this element.
   * @param group The group the DDProxy object is member of
   * @param config The DDProxy config object
   * @param overrides An object containing methods to override/implement on the DDProxy object
   * @return The DDProxy object
   */
  public native function initDDProxy(group:String, config:Object, overrides:Object):ext.dd.DDProxy;

  /**
   * Initializes a <a href="#!/api/Ext.dd.DDTarget" rel="Ext.dd.DDTarget" class="docClass">Ext.dd.DDTarget</a> object for this element.
   * @param group The group the DDTarget object is member of
   * @param config The DDTarget config object
   * @param overrides An object containing methods to override/implement on the DDTarget object
   * @return The DDTarget object
   */
  public native function initDDTarget(group:String, config:Object, overrides:Object):ext.dd.DDTarget;

  /**
   * Inserts this element after the passed element in the DOM.
   * @param el The element to insert after.
   * The <code>id</code> of the node, a DOM Node or an existing Element.
   * @return This element.
   */
  public native function insertAfter(el:*):ext.dom.Element;

  /**
   * Inserts this element before the passed element in the DOM.
   * @param el The element before which this element will be inserted.
   * The id of the node, a DOM Node or an existing Element.
   * @return This element.
   */
  public native function insertBefore(el:*):ext.dom.Element;

  /**
   * Inserts (or creates) an element as the first child of this element
   * @param el The id or element to insert
   * or a DomHelper config to create and insert
   * @param returnDom True to return the raw DOM element instead
   * of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The new child element (or HTMLElement if
   * <em>returnDom</em> is <em>true</em>).
   */
  public native function insertFirst(el:*, returnDom:Boolean = false):*;

  /**
   * Inserts an html fragment into this element
   * @param where Where to insert the html in relation to this element - beforeBegin, afterBegin, beforeEnd, afterEnd.
   * See <a href="#!/api/Ext.dom.Helper-method-insertHtml" rel="Ext.dom.Helper-method-insertHtml" class="docClass">Ext.dom.Helper.insertHtml</a> for details.
   * @param html The HTML fragment
   * @param returnEl True to return an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The inserted node (or nearest related if more than 1 inserted)
   */
  public native function insertHtml(where:String, html:String, returnEl:Boolean = false):*;

  /**
   * Inserts (or creates) the passed element (or DomHelper config) as a sibling of this element
   * @param el The id, element to insert or a DomHelper config
   * to create and insert <em>or</em> an array of any of those.
   * @param where 'before' or 'after'
   * @default 'before'
   * @param returnDom True to return the raw DOM element instead of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The inserted <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or
   * HTMLElement if <em>returnDom</em> is <em>true</em>). If an array is passed, the last
   * inserted element is returned.
   */
  public native function insertSibling(el:*, where:String = 'before', returnDom:Boolean = false):*;

  /**
   * Returns <code>true</code> if this element matches the passed simple selector
   * (e.g. 'div.some-class' or 'span:first-child').
   * @param selector The simple selector to test or a function which is passed
   * candidate nodes, and should return <code>true</code> for nodes which match.
   * @return <code>true</code> if this element matches the selector, else <code>false</code>.
   */
  public native function matches(selector:*):Boolean;

  /**
   * Returns <code>true</code> if this element is an ancestor of the passed element
   * @param el The element or id of the element
   * to search for in this elements descendants.
   */
  public native function isAncestor(el:*):Boolean;

  [Deprecated(since = "5.0.0")]
  /**
   * Returns true if this element uses the border-box-sizing model.  This method is
   * deprecated as of version 5.0 because border-box sizing is forced upon all elements
   * via a style sheet rule, and the browsers that do not support border-box (IE6/7 strict
   * mode) are no longer supported.
   */
  public native function isBorderBox():Boolean;

  /**
   * Determines if this element is a descendant of the passed in Element.
   */
  public native function isDescendent():void;

  [Deprecated(message = "use element.isStyle(&#39;display&#39;, &#39;none&#39;);\n", since = "5.0.0")]
  /**
   * Returns true if display is not "none"
   */
  public native function isDisplayed():Boolean;

  /**
   * Checks whether this element can be focused programmatically or by clicking.
   * To check if an element is in the document tab flow, use <a href="#!/api/Ext.dom.Element-method-isTabbable" rel="Ext.dom.Element-method-isTabbable" class="docClass">isTabbable</a>.
   * @return True if the element is focusable
   */
  public native function isFocusable():Boolean;

  /**
   * Returns <code>true</code> if this Element is an input field, or is editable in any way.
   * @return <code>true</code> if this Element is an input field, or is editable in any way.
   */
  public native function isInputField():Boolean;

  /**
   * Returns true if this element is masked. Also re-centers any displayed message
   * within the mask.
   * @param deep Go up the DOM hierarchy to determine if any parent
   * element is masked.
   */
  public native function isMasked(deep:Boolean = false):Boolean;

  /**
   * Returns true if this element is scrollable.
   */
  public native function isScrollable():Boolean;

  /**
   * Checks if the current value of a style is equal to a given value.
   * @param style property whose value is returned.
   * @param value to check against.
   * @return <code>true</code> for when the current value equals the given value.
   */
  public native function isStyle(style:String, value:String):Boolean;

  /**
   * Checks whether this element participates in the sequential focus navigation,
   * and can be reached by using Tab key.
   * @param includeHidden pass <code>true</code> if hidden, or unattached elements should be returned.
   * @default false
   * @return True if the element is tabbable.
   */
  public native function isTabbable(includeHidden:Boolean = false):Boolean;

  [Deprecated(since = "5.0.0")]
  /**
   * Returns <code>true</code> if the value of the given property is visually transparent. This
   * may be due to a 'transparent' style value or an rgba value with 0 in the alpha
   * component.
   * @param prop The style property whose value is to be tested.
   * @return <code>true</code> if the style property is visually transparent.
   */
  public native function isTransparent(prop:String):Boolean;

  /**
   * Checks whether the element is currently visible using both visibility and display properties.
   * @param deep True to walk the dom and see if parent elements are hidden.
   * If false, the function only checks the visibility of the element itself and it may return
   * <code>true</code> even though a parent is not visible.
   * @default false
   * @return <code>true</code> if the element is currently visible, else <code>false</code>
   */
  public native function isVisible(deep:Boolean = false):Boolean;

  /**
   * Gets the last child, skipping text nodes
   * @param selector Find the previous sibling that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom <code>true</code> to return a raw DOM node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The last child <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or
   * HTMLElement if <em>returnDom</em> is <em>true</em>).  Or null if no match is found.
   */
  public native function last(selector:String = null, returnDom:Boolean = false):*;

  /**
   * Direct access to the <a href="#!/api/Ext.ElementLoader" rel="Ext.ElementLoader" class="docClass">Ext.ElementLoader</a> <a href="#!/api/Ext.ElementLoader-method-load" rel="Ext.ElementLoader-method-load" class="docClass">Ext.ElementLoader.load</a> method.
   * The method takes the same object parameter as <a href="#!/api/Ext.ElementLoader-method-load" rel="Ext.ElementLoader-method-load" class="docClass">Ext.ElementLoader.load</a>
   * @param options a options object for <a href="#!/api/Ext.ElementLoader" rel="Ext.ElementLoader" class="docClass">Ext.ElementLoader</a> <a href="#!/api/Ext.ElementLoader-method-load" rel="Ext.ElementLoader-method-load" class="docClass">Ext.ElementLoader.load</a>
   * @return this
   */
  public native function load(options:*):ext.dom.Element;

  /**
   * Puts a mask over this element to disable user interaction.
   * This method can only be applied to elements which accept child nodes. Use
   * <a href="#!/api/Ext.dom.Element-method-unmask" rel="Ext.dom.Element-method-unmask" class="docClass">unmask</a> to remove the mask.
   * @param msg A message to display in the mask
   * @param msgCls A css class to apply to the msg element
   * @return The mask element
   */
  public native function mask(msg:String = null, msgCls:String = null):ext.dom.Element;

  /**
   * Monitors this Element for the mouse leaving. Calls the function after the specified delay only if
   * the mouse was not moved back into the Element within the delay. If the mouse <em>was</em> moved
   * back in, the function is not called.
   * @param delay The delay <strong>in milliseconds</strong> to wait for possible mouse re-entry before calling the handler function.
   * @param handler The function to call if the mouse remains outside of this Element for the specified time.
   * @param scope The scope (<code>this</code> reference) in which the handler function executes. Defaults to this Element.
   * @return The listeners object which was added to this element so that monitoring can be stopped. Example usage:
   * <pre><code>// Hide the menu if the mouse moves out for 250ms or more
   * this.mouseLeaveMonitor = this.menuEl.monitorMouseLeave(250, this.hideMenu, this);
   * ...
   * // Remove mouseleave monitor on menu destroy
   * this.mouseLeaveMonitor.destroy();
   * </code></pre>
   */
  public native function monitorMouseLeave(delay:Number, handler:Function, scope:Object = null):Object;

  /**
   * Move the element relative to its current position.
   * @param direction Possible values are:
   * <ul>
   * <li><code>"l"</code> (or <code>"left"</code>)</li>
   * <li><code>"r"</code> (or <code>"right"</code>)</li>
   * <li><code>"t"</code> (or <code>"top"</code>, or <code>"up"</code>)</li>
   * <li><code>"b"</code> (or <code>"bottom"</code>, or <code>"down"</code>)</li>
   * </ul>
   * @param distance How far to move the element in pixels
   * @param animate true for the default animation or a standard
   * Element animation config object
   */
  public native function move(direction:String, distance:Number, animate:* = undefined):void;

  [Deprecated(replacement = "setXY", since = "4.2.0")]
  /**
   * Sets the position of the element in page coordinates.
   * @param x X value for new position (coordinates are page-based)
   * @param y Y value for new position (coordinates are page-based)
   * @param animate True for the default animation, or a standard
   * Element animation config object
   * @return this
   */
  public native function moveTo(x:Number, y:Number, animate:* = undefined):ext.dom.Element;

  /**
   * Gets the next sibling, skipping text nodes
   * @param selector Find the next sibling that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom <code>true</code> to return a raw dom node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The next sibling <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> (or
   * HTMLElement if <em>asDom</em> is <em>true</em>).  Or null if no match is found.
   */
  public native function next(selector:String = null, returnDom:Boolean = false):*;

  /**
   * Gets the parent node for this element, optionally chaining up trying to match a selector
   * @param selector Find a parent node that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom True to return a raw dom node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The parent node (<a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or
   * HTMLElement if <em>returnDom</em> is <em>true</em>).  Or null if no match is found.
   */
  public native function parent(selector:String = null, returnDom:Boolean = false):*;

  [Deprecated(replacement = "animate", since = "4.0")]
  /**
   * Creates a pause before any subsequent queued effects begin. If there are no effects queued after the pause it will
   * have no effect. Usage:
   *
   * <pre><code>el.pause(1);
   * </code></pre>
   * @param seconds The length of time to pause (in seconds)
   * @return The Element
   */
  public native function pause(seconds:Number):ext.dom.Element;

  /**
   * Initializes positioning on this element. If a desired position is not passed,
   * it will make the the element positioned relative IF it is not already positioned.
   * @param pos Positioning to use "relative", "absolute" or "fixed"
   * @param zIndex The zIndex to apply
   * @param x Set the page X position
   * @param y Set the page Y position
   */
  public native function position(pos:String = null, zIndex:Number = NaN, x:Number = NaN, y:Number = NaN):void;

  /**
   * Gets the previous sibling, skipping text nodes
   * @param selector Find the previous sibling that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom <code>true</code> to return a raw DOM node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The previous sibling (<a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or
   * HTMLElement if <em>returnDom</em> is <em>true</em>).  Or null if no match is found.
   */
  public native function prev(selector:String = null, returnDom:Boolean = false):*;

  /**
   * Fades the element out while slowly expanding it in all directions. When the effect is completed, the element will
   * be hidden (visibility = 'hidden') but block elements will still take up space in the document. Usage:
   *
   * <pre><code>// default
   * el.puff();
   *
   * // common config options shown with default values
   * el.puff({
   *     easing: 'easeOut',
   *     duration: 500,
   *     useDisplay: false
   * });
   * </code></pre>
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function puff(options:Object = null):ext.dom.Element;

  [Deprecated(replacement = "clearListeners", since = "5.0.0")]
  /**
   * Removes all listeners for this object including the managed listeners
   */
  public native function purgeAllListeners():ext.dom.Element;

  /**
   * Selects child nodes based on the passed CSS selector.
   * Delegates to document.querySelectorAll. More information can be found at
   * <a href="http://www.w3.org/TR/css3-selectors/">http://www.w3.org/TR/css3-selectors/</a>
   *
   * <p>All selectors, attribute filters and pseudos below can be combined infinitely
   * in any order. For example <code>div.foo:nth-child(odd)[&#64;foo=bar].bar:first</code> would be
   * a perfectly valid selector.</p>
   *
   * <h2>Element Selectors:</h2>
   *
   * <ul>
   * <li>* any element</li>
   * <li>E an element with the tag E</li>
   * <li>E F All descendant elements of E that have the tag F</li>
   * <li>E > F or E/F all direct children elements of E that have the tag F</li>
   * <li>E + F all elements with the tag F that are immediately preceded by an element with the tag E</li>
   * <li>E ~ F all elements with the tag F that are preceded by a sibling element with the tag E</li>
   * </ul>
   *
   *
   * <h2>Attribute Selectors:</h2>
   *
   * <p>The use of  and quotes are optional. For example, div[&#64;foo='bar'] is also a valid attribute selector.</p>
   *
   * <ul>
   * <li>E[foo] has an attribute "foo"</li>
   * <li>E[foo=bar] has an attribute "foo" that equals "bar"</li>
   * <li>E[foo^=bar] has an attribute "foo" that starts with "bar"</li>
   * <li>E[foo$=bar] has an attribute "foo" that ends with "bar"</li>
   * <li>E[foo*=bar] has an attribute "foo" that contains the substring "bar"</li>
   * <li>E[foo%=2] has an attribute "foo" that is evenly divisible by 2</li>
   * <li>E[foo!=bar] has an attribute "foo" that does not equal "bar"</li>
   * </ul>
   *
   *
   * <h2>Pseudo Classes:</h2>
   *
   * <ul>
   * <li>E:first-child E is the first child of its parent</li>
   * <li>E:last-child E is the last child of its parent</li>
   * <li>E:nth-child(n) E is the nth child of its parent (1 based as per the spec)</li>
   * <li>E:nth-child(odd) E is an odd child of its parent</li>
   * <li>E:nth-child(even) E is an even child of its parent</li>
   * <li>E:only-child E is the only child of its parent</li>
   * <li>E:checked E is an element that is has a checked attribute that is true (e.g. a radio or checkbox)</li>
   * <li>E:first the first E in the resultset</li>
   * <li>E:last the last E in the resultset</li>
   * <li>E:nth(n) the nth E in the resultset (1 based)</li>
   * <li>E:odd shortcut for :nth-child(odd)</li>
   * <li>E:even shortcut for :nth-child(even)</li>
   * <li>E:not(S) an E element that does not match simple selector S</li>
   * <li>E:has(S) an E element that has a descendant that matches simple selector S</li>
   * <li>E:next(S) an E element whose next sibling matches simple selector S</li>
   * <li>E:prev(S) an E element whose previous sibling matches simple selector S</li>
   * <li>E:any(S1|S2|S2) an E element which matches any of the simple selectors S1, S2 or S3//\</li>
   * </ul>
   *
   *
   * <h2>CSS Value Selectors:</h2>
   *
   * <ul>
   * <li>E{display=none} CSS value "display" that equals "none"</li>
   * <li>E{display^=none} CSS value "display" that starts with "none"</li>
   * <li>E{display$=none} CSS value "display" that ends with "none"</li>
   * <li>E{display*=none} CSS value "display" that contains the substring "none"</li>
   * <li>E{display%=2} CSS value "display" that is evenly divisible by 2</li>
   * <li>E{display!=none} CSS value "display" that does not equal "none"</li>
   * </ul>
   * @param selector The CSS selector.
   * @param asDom <code>false</code> to return an array of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default true
   * @return An Array of elements (
   * HTMLElement or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> if <em>asDom</em> is <em>false</em>) that match the selector.<br/>
   * If there are no matches, an empty Array is returned.
   */
  public native function query(selector:String, asDom:Boolean = true):Array;

  /**
   * Adds one or more CSS classes to this element and removes the same class(es) from all siblings.
   * @param className The CSS class to add, or an array of classes.
   * @return this
   */
  public native function radioCls(className:*):ext.dom.Element;

  [Deprecated(replacement = "destroy", since = "5.0.0")]
  /**
   * Destroys this element by removing it from the cache, removing its DOM reference,
   * and removing all of its event listeners.
   */
  public native function remove():void;

  [Deprecated(replacement = "clearListeners", since = "5.0.0")]
  /**
   * Removes all listeners for this object including the managed listeners
   */
  public native function removeAllListeners():ext.dom.Element;

  /**
   * Remove any anchor to this element. See <a href="#!/api/Ext.util.Positionable-method-anchorTo" rel="Ext.util.Positionable-method-anchorTo" class="docClass">anchorTo</a>.
   * @return this
   */
  public native function removeAnchor():ext.util.Positionable;

  /**
   * Removes the given CSS class(es) from this Element.
   * @param names The CSS classes to remove separated by space,
   * or an array of classes
   * @param prefix Prefix to prepend to each class. The separator <code>-</code> will be
   * appended to the prefix.
   * @param suffix Suffix to append to each class. The separator <code>-</code> will be
   * prepended to the suffix.
   * return {<a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>} this
   */
  public native function removeCls(names:*, prefix:String = null, suffix:String = null):void;

  /**
   * Forces the browser to repaint this element.
   * @return this
   */
  public native function repaint():ext.dom.Element;

  /**
   * Replaces the passed element with this element
   * @param el The element to replace.
   * The id of the node, a DOM Node or an existing Element.
   * @param destroy <code>false</code> to prevent destruction of the replaced
   * element
   * @default true
   * @return This element
   */
  public native function replace(el:*, destroy:Boolean = true):ext.dom.Element;

  /**
   * Replaces a CSS class on the element with another.
   * If the old name does not exist, the new name will simply be added.
   * @param oldName The CSS class to replace.
   * @param newName The replacement CSS class.
   * @param prefix Prefix to prepend to each class to be replaced.
   * @default ''
   * @param suffix Suffix to append to each class to be replaced.
   * @default ''
   * @return this
   */
  public native function replaceCls(oldName:String, newName:String, prefix:String = '', suffix:String = ''):ext.dom.Element;

  /**
   * Replaces this element with the passed element
   * @param el The new element (id of the
   * node, a DOM Node or an existing Element) or a DomHelper config of an element to create
   * @return This element
   */
  public native function replaceWith(el:*):ext.dom.Element;

  [Deprecated(replacement = "animate", since = "4.0")]
  /**
   * Animates the transition of an element's dimensions from a starting height/width to an ending height/width. This
   * method is a convenience implementation of <a href="#!/api/Ext.dom.Element-method-shift" rel="Ext.dom.Element-method-shift" class="docClass">shift</a>. Usage:
   *
   * <pre><code>// change height and width to 100x100 pixels
   * el.scale(100, 100);
   *
   * // common config options shown with default values.  The height and width will default to
   * // the element's existing values if passed as null.
   * el.scale(
   *     [element's width],
   *     [element's height], {
   *         easing: 'easeOut',
   *         duration: 350
   *     }
   * );
   * </code></pre>
   * @param width The new width (pass undefined to keep the original width)
   * @param height The new height (pass undefined to keep the original height)
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function scale(width:Number, height:Number, options:Object = null):ext.dom.Element;

  /**
   * Scrolls this element the specified direction. Does bounds checking to make sure the scroll is
   * within this element's scrollable range.
   * @param direction Possible values are:
   * <ul>
   * <li><code>"l"</code> (or <code>"left"</code>)</li>
   * <li><code>"r"</code> (or <code>"right"</code>)</li>
   * <li><code>"t"</code> (or <code>"top"</code>, or <code>"up"</code>)</li>
   * <li><code>"b"</code> (or <code>"bottom"</code>, or <code>"down"</code>)</li>
   * </ul>
   * @param distance How far to scroll the element in pixels
   * @param animate true for the default animation or a standard Element
   * animation config object
   * @return Returns true if a scroll was triggered or false if the element
   * was scrolled as far as it could go.
   */
  public native function scroll(direction:String, distance:Number, animate:* = undefined):Boolean;

  /**
   * Scrolls this element by the passed delta values, optionally animating.
   *
   * <p>All of the following are equivalent:</p>
   *
   * <pre><code> el.scrollBy(10, 10, true);
   *  el.scrollBy([10, 10], true);
   *  el.scrollBy({ x: 10, y: 10 }, true);
   * </code></pre>
   * @param deltaX Either the x delta, an Array specifying x and y deltas or
   * an object with "x" and "y" properties.
   * @param deltaY Either the y delta, or an animate flag or config object.
   * @param animate Animate flag/config object if the delta values were passed separately.
   * @return this
   */
  public native function scrollBy(deltaX:*, deltaY:*, animate:*):ext.dom.Element;

  /**
   * Scrolls this element into view within the passed container.
   *
   * <pre><code>  <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *       storeId:'simpsonsStore',
   *       fields:['name', 'email', 'phone'],
   *       data:{'items':[
   *           { 'name': 'Lisa',  "email":"lisa&#64;simpsons.com",  "phone":"555-111-1224"  },
   *           { 'name': 'Bart',  "email":"bart&#64;simpsons.com",  "phone":"555-222-1234" },
   *           { 'name': 'Homer', "email":"homer&#64;simpsons.com",  "phone":"555-222-1244"  },
   *           { 'name': 'Marge', "email":"marge&#64;simpsons.com", "phone":"555-222-1254"  },
   *           { 'name': 'Milhouse', "email":"milhouse&#64;simpsons.com",  "phone":"555-222-1244"  },
   *           { 'name': 'Willy', "email":"willy&#64;simpsons.com", "phone":"555-222-1254"  },
   *           { 'name': 'Skinner', "email":"skinner&#64;simpsons.com",  "phone":"555-222-1244"  },
   *           { 'name': 'Hank (last row)', "email":"hank&#64;simpsons.com", "phone":"555-222-1254"  }
   *       ]},
   *       proxy: {
   *           type: 'memory',
   *           reader: {
   *               type: 'json',
   *               rootProperty: 'items'
   *           }
   *       }
   *   });
   *
   *   var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *       title: 'Simpsons',
   *       store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('simpsonsStore'),
   *       columns: [
   *           { text: 'Name',  dataIndex: 'name', width: 125 },
   *           { text: 'Email', dataIndex: 'email', flex: 1 },
   *           { text: 'Phone', dataIndex: 'phone' }
   *       ],
   *       height: 190,
   *       width: 400,
   *       renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *       tbar: [{
   *           text: 'Scroll row 7 into view',
   *           handler: function () {
   *               var view = grid.getView();
   *
   *               <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(view.getRow(7)).scrollIntoView(view.getEl(), null, true);
   *           }
   *       }]
   *   });
   * </code></pre>
   * @param container (Default document.body) The container element
   * to scroll.  Should be a string (id), dom node, or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * @default document.body
   * @param hscroll False to disable horizontal scroll.
   * @default true
   * @param animate true for the default animation or a standard Element
   * animation config object
   * @param highlight true to <a href="#!/api/Ext.dom.Element-method-highlight" rel="Ext.dom.Element-method-highlight" class="docClass">highlight</a> the element when it is in view.
   * @default false
   * @return this
   */
  public native function scrollIntoView(container:* = undefined, hscroll:Boolean = true, animate:* = undefined, highlight:Boolean = false):ext.dom.Element;

  /**
   * Scrolls this element the specified scroll point. It does NOT do bounds checking so
   * if you scroll to a weird value it will try to do it. For auto bounds checking, use <a href="#!/api/Ext.dom.Element-event-scroll" rel="Ext.dom.Element-event-scroll" class="docClass">scroll</a>.
   * @param side Either "left" for scrollLeft values or "top" for scrollTop values.
   * @param value The new scroll value
   * @param animate true for the default animation or a standard Element
   * animation config object
   * @return this
   */
  public native function scrollTo(side:String, value:Number, animate:* = undefined):ext.dom.Element;

  /**
   * Selects descendant elements of this element based on the passed CSS selector to
   * enable <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> methods to be applied to many related
   * elements in one statement through the returned
   * <a href="#!/api/Ext.dom.CompositeElementLite" rel="Ext.dom.CompositeElementLite" class="docClass">CompositeElementLite</a> object.
   * @param selector The CSS selector or an array of elements
   * @param composite Return a CompositeElement as opposed to a
   * CompositeElementLite. Defaults to false.
   */
  public native function select(selector:*, composite:Boolean = false):ext.dom.CompositeElementLite;

  /**
   * Selects a single descendant element of this element using a CSS selector
   * (see <a href="#!/api/Ext.dom.Element-method-query" rel="Ext.dom.Element-method-query" class="docClass">query</a>).
   * @param selector The selector query
   * @param asDom <code>false</code> to return an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default true
   * @return The DOM element (or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> if
   * <em>asDom</em> is <em>false</em>) which matched the selector.
   */
  public native function selectNode(selector:String, asDom:Boolean = true):*;

  /**
   * Enable text selection for this element (normalized across browsers)
   * @return this
   */
  public native function selectable():ext.dom.Element;

  /**
   * Sets the passed attributes as attributes of this element (a <code>style</code> attribute
   * can be a string, object or function).
   *
   * <p>Example component (though any <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> would suffice):</p>
   *
   * <pre><code>var cmp = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'component',
   *     html: 'test',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
   * });
   * </code></pre>
   *
   * <p>Once the component is rendered, you can fetch a reference to its outer
   * element to use <code>set</code>:</p>
   *
   * <pre><code>cmp.el.set({
   *     foo: 'bar'
   * });
   * </code></pre>
   *
   * <p>This sets an attribute on the element of <strong>foo="bar"</strong>:</p>
   *
   * <pre><code>&lt;div class="x-component x-component-default x-border-box" id="component-1009" foo="bar"&gt;test&lt;/div&gt;
   * </code></pre>
   *
   * <p>To remove the attribute pass a value of <strong>undefined</strong>:</p>
   *
   * <pre><code>cmp.el.set({
   *     foo: undefined
   * });
   * </code></pre>
   *
   * <p><strong>Note:</strong></p>
   *
   * <ul>
   * <li>You cannot remove an attribute by passing <code>undefined</code> when the
   * <code>expandos</code> param is set to <strong>false</strong>.</li>
   * <li>Passing an attribute of <code>style</code> results in the request being handed off to
   * <a href="#!/api/Ext.dom.Element-method-applyStyles" rel="Ext.dom.Element-method-applyStyles" class="docClass">applyStyles</a>.</li>
   * <li>Passing an attribute of <code>cls</code> results in the element's dom's
   * <a href="http://www.w3schools.com/jsref/prop_html_classname.asp">className</a> property
   * being set directly.  For additional flexibility when setting / removing
   * classes see:
   *
   * <ul>
   * <li><a href="#!/api/Ext.dom.Element-method-addCls" rel="Ext.dom.Element-method-addCls" class="docClass">addCls</a></li>
   * <li><a href="#!/api/Ext.dom.Element-method-removeCls" rel="Ext.dom.Element-method-removeCls" class="docClass">removeCls</a></li>
   * <li><a href="#!/api/Ext.dom.Element-method-replaceCls" rel="Ext.dom.Element-method-replaceCls" class="docClass">replaceCls</a></li>
   * <li><a href="#!/api/Ext.dom.Element-method-setCls" rel="Ext.dom.Element-method-setCls" class="docClass">setCls</a></li>
   * <li><a href="#!/api/Ext.dom.Element-method-toggleCls" rel="Ext.dom.Element-method-toggleCls" class="docClass">toggleCls</a></li>
   * </ul>
   * </li>
   * </ul>
   * @param attributes The object with the attributes.
   * @param useSet <code>false</code> to override the default <code>setAttribute</code>
   * to use <a href="http://help.dottoro.com/ljvovanq.php">expandos</a>.
   * @default true
   * @return this
   */
  public native function set(attributes:Object, useSet:Boolean = true):ext.dom.Element;

  /**
   * Sets the element's CSS bottom style.
   * @param bottom Number of pixels or CSS string value to set as
   * the bottom CSS property value
   * @return this
   */
  public native function setBottom(bottom:*):ext.dom.Element;

  [Deprecated(replacement = "setBox", since = "4.2.0")]
  /**
   * Sets the element's position and size in one shot. If animation is true then
   * width, height, x and y will be animated concurrently.
   * @param x X value for new position (coordinates are page-based)
   * @param y Y value for new position (coordinates are page-based)
   * @param width The new width. This may be one of:
   * <ul>
   * <li>A Number specifying the new width in pixels</li>
   * <li>A String used to set the CSS width style. Animation may <strong>not</strong> be used.</li>
   * </ul>
   * @param height The new height. This may be one of:
   * <ul>
   * <li>A Number specifying the new height in pixels</li>
   * <li>A String used to set the CSS height style. Animation may <strong>not</strong> be used.</li>
   * </ul>
   * @param animate true for the default animation or
   * a standard Element animation config object
   * @return this
   */
  public native function setBounds(x:Number, y:Number, width:*, height:*, animate:* = undefined):ext.dom.Element;

  /**
   * Sets the element's box. If animate is true then x, y, width, and height will be
   * animated concurrently.
   * @param box The box to fill {x, y, width, height}
   * @param animate true for the default animation or a standard
   * Element animation config object
   */
  public native function setBox(box:Object, animate:* = undefined):void;

  /**
   * Sets the specified CSS class on this element's DOM node.
   * @param className The CSS class to set on this element.
   */
  public native function setCls(className:*):void;

  /**
   * Sets the CSS display property. Uses originalDisplay if the specified value is a
   * boolean true.
   * @param value Boolean value to display the element using its
   * default display, or a string to set the display directly.
   * @return this
   */
  public native function setDisplayed(value:*):ext.dom.Element;

  [Deprecated(replacement = "setHtml", since = "5.0.0")]
  /**
   * Updates the innerHTML of this element, optionally searching for and processing scripts.
   * @param html The new HTML
   * @param loadScripts Pass <code>true</code> to look for and process scripts.
   * @param callback For async script loading you can be notified when the update completes.
   * @param scope (Default `this`) The scope (<code>this</code> reference) in which to execute the callback.
   * <p>Also used as the scope for any <em>inline</em> script source if the <code>loadScripts</code> parameter is <code>true</code>.
   * Scripts with a <code>src</code> attribute cannot be executed in this scope.</p>
   * <p>Defaults to this Element.</p>
   * @default `this`
   * @return this
   */
  public native function setHTML(html:String, loadScripts:Boolean = false, callback:Function = null, scope:Object = null):ext.dom.Element;

  /**
   * Set the height of this Element.
   *
   * <pre><code>// change the height to 200px and animate with default configuration
   * <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>('elementId').setHeight(200, true);
   *
   * // change the height to 150px and animate with a custom configuration
   * <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>('elId').setHeight(150, {
   *     duration : 500, // animation will have a duration of .5 seconds
   *     // will change the content to "finished"
   *     callback: function(){ this.setHtml("finished"); }
   * });
   * </code></pre>
   * @param height The new height. This may be one of:
   * <ul>
   * <li>A Number specifying the new height in pixels.</li>
   * <li>A String used to set the CSS height style. Animation may <strong>not</strong> be used.</li>
   * </ul>
   * @param animate a standard Element animation config object or <code>true</code> for
   * the default animation (<code>{duration: 350, easing: 'ease-in'}</code>)
   * @return this
   */
  public native function setHeight(height:*, animate:* = undefined):ext.dom.Element;

  /**
   * Updates the innerHTML of this element, optionally searching for and processing scripts.
   * @param html The new HTML
   * @param loadScripts Pass <code>true</code> to look for and process scripts.
   * @param callback For async script loading you can be notified when the update completes.
   * @param scope (Default `this`) The scope (<code>this</code> reference) in which to execute the callback.
   * <p>Also used as the scope for any <em>inline</em> script source if the <code>loadScripts</code> parameter is <code>true</code>.
   * Scripts with a <code>src</code> attribute cannot be executed in this scope.</p>
   * <p>Defaults to this Element.</p>
   * @default `this`
   * @return this
   */
  public native function setHtml(html:String, loadScripts:Boolean = false, callback:Function = null, scope:Object = null):ext.dom.Element;

  /**
   * Sets the element's left position directly using CSS style
   * (instead of <a href="#!/api/Ext.dom.Element-method-setX" rel="Ext.dom.Element-method-setX" class="docClass">setX</a>).
   * @param left Number of pixels or CSS string value to
   * set as the left CSS property value
   * @return this
   */
  public native function setLeft(left:*):ext.dom.Element;

  [Deprecated(replacement = "setLocalXY", since = "4.2.0")]
  /**
   * Sets the element's left and top positions directly using CSS style
   * @param left Number of pixels or CSS string value to
   * set as the left CSS property value
   * @param top Number of pixels or CSS string value to
   * set as the top CSS property value
   * @return this
   */
  public native function setLeftTop(left:String, top:String):ext.dom.Element;

  /**
   * Sets the local x coordinate of this element using CSS style. When used on an
   * absolute positioned element this method is symmetrical with <a href="#!/api/Ext.util.Positionable-method-getLocalX" rel="Ext.util.Positionable-method-getLocalX" class="docClass">getLocalX</a>, but
   * may not be symmetrical when used on a relatively positioned element.
   * @param x The x coordinate. A value of <code>null</code> sets the left style to 'auto'.
   */
  public native function setLocalX(x:Number):void;

  /**
   * Sets the local x and y coordinates of this element using CSS style. When used on an
   * absolute positioned element this method is symmetrical with <a href="#!/api/Ext.util.Positionable-method-getLocalXY" rel="Ext.util.Positionable-method-getLocalXY" class="docClass">getLocalXY</a>, but
   * may not be symmetrical when used on a relatively positioned element.
   * @param x The x coordinate or an array containing [x, y]. A value of
   * <code>null</code> sets the left style to 'auto'
   * @param y The y coordinate, required if x is not an array. A value of
   * <code>null</code> sets the top style to 'auto'
   */
  public native function setLocalXY(x:*, y:Number = NaN):void;

  /**
   * Sets the local y coordinate of this element using CSS style. When used on an
   * absolute positioned element this method is symmetrical with <a href="#!/api/Ext.util.Positionable-method-getLocalY" rel="Ext.util.Positionable-method-getLocalY" class="docClass">getLocalY</a>, but
   * may not be symmetrical when used on a relatively positioned element.
   * @param y The y coordinate. A value of <code>null</code> sets the top style to 'auto'.
   */
  public native function setLocalY(y:Number):void;

  [Deprecated(replacement = "setXY", since = "4.2.0")]
  /**
   * Sets the position of the element in page coordinates.
   * @param x X value for new position
   * @param y Y value for new position
   * @param animate True for the default animation, or a standard
   * Element animation config object
   * @return this
   */
  public native function setLocation(x:Number, y:Number, animate:* = undefined):ext.dom.Element;

  /**
   * Set the maximum height of this Element.
   * @param height The new maximum height.
   * @return this
   */
  public native function setMaxHeight(height:*):ext.dom.Element;

  /**
   * Set the maximum width of this Element.
   * @param width The new maximum width.
   * @return this
   */
  public native function setMaxWidth(width:*):ext.dom.Element;

  /**
   * Set the minimum height of this Element.
   * @param height The new minimum height.
   * @return this
   */
  public native function setMinHeight(height:*):ext.dom.Element;

  /**
   * Set the minimum width of this Element.
   * @param width The new minimum width.
   * @return this
   */
  public native function setMinWidth(width:*):ext.dom.Element;

  /**
   * Set the opacity of the element
   * @param opacity The new opacity. 0 = transparent, .5 = 50% visible, 1 = fully visible, etc
   * @param animate a standard Element animation config object or <code>true</code> for
   * the default animation (<code>{duration: 350, easing: 'ease-in'}</code>)
   * @return this
   */
  public native function setOpacity(opacity:Number, animate:* = undefined):ext.dom.Element;

  /**
   * Set positioning with an object returned by <code>getPositioning</code>.
   * @return this
   */
  public native function setPositioning(posCfg:Object):ext.dom.Element;

  /**
   * Sets the element's CSS right style.
   * @param right Number of pixels or CSS string value to
   * set as the right CSS property value
   * @return this
   */
  public native function setRight(right:*):ext.dom.Element;

  /**
   * Sets the left scroll position
   * @param left The left scroll position
   * @return this
   */
  public native function setScrollLeft(left:Number):ext.dom.Element;

  /**
   * Sets the top scroll position
   * @param top The top scroll position
   * @return this
   */
  public native function setScrollTop(top:Number):ext.dom.Element;

  /**
   * Set the size of this Element. If animation is true, both width and height will be animated concurrently.
   * @param width The new width. This may be one of:
   * <ul>
   * <li>A Number specifying the new width in pixels.</li>
   * <li>A String used to set the CSS width style. Animation may <strong>not</strong> be used.</li>
   * <li>A size object in the format <code>{width: widthValue, height: heightValue}</code>.</li>
   * </ul>
   * @param height The new height. This may be one of:
   * <ul>
   * <li>A Number specifying the new height in  pixels.</li>
   * <li>A String used to set the CSS height style. Animation may <strong>not</strong> be used.</li>
   * </ul>
   * @param animate a standard Element animation config object or <code>true</code> for
   * the default animation (<code>{duration: 350, easing: 'ease-in'}</code>)
   * @return this
   */
  public native function setSize(width:*, height:*, animate:* = undefined):ext.dom.Element;

  /**
   * Wrapper for setting style properties, also takes single object parameter of
   * multiple styles.
   *
   * <p>Styles should be a valid DOM element style property.<br/>
   * <a href="http://www.w3schools.com/jsref/dom_obj_style.asp">Valid style property names</a>
   * (<em>along with the supported CSS version for each</em>)</p>
   *
   * <pre><code>// &lt;div id="my-el"&gt;Phineas Flynn&lt;/div&gt;
   *
   * var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('my-el');
   *
   * // two-param syntax
   * el.setStyle('color', 'white');
   *
   * // single-param syntax
   * el.setStyle({
   *     fontWeight: 'bold',
   *     backgroundColor: 'gray',
   *     padding: '10px'
   * });
   * </code></pre>
   * @param property The style property to be set, or an object of
   * multiple styles.
   * @param value The value to apply to the given property, or null if
   * an object was passed.
   * @return this
   */
  public native function setStyle(property:*, value:String = null):ext.dom.Element;

  /**
   * Sets the element's top position directly using CSS style
   * (instead of <a href="#!/api/Ext.dom.Element-method-setY" rel="Ext.dom.Element-method-setY" class="docClass">setY</a>).
   * @param top Number of pixels or CSS string value to
   * set as the top CSS property value
   * @return this
   */
  public native function setTop(top:*):ext.dom.Element;

  /**
   * Sets the element's top and left positions directly using CSS style.
   */
  public native function setTopLeft():void;

  /**
   * Sets the CSS touch-action
   * property on this element and emulates its behavior on browsers where touch-action
   * is not supported.
   * @param touchAction An object with touch-action names as the keys, and
   * boolean values to enable or disable specific touch actions. Accepted keys are:
   * <ul>
   * <li><code>panX</code></li>
   * <li><code>panY</code></li>
   * <li><code>pinchZoom</code></li>
   * <li><code>doubleTapZoom</code></li>
   * </ul>
   * <p>All touch actions are enabled (<code>true</code>) by default, so it is usually only necessary
   * to specify which touch actions to disable.  For example, the following disables
   * only vertical scrolling and double-tap-zoom on an element</p>
   * <pre><code>element.setTouchAction({
   *     panY: false,
   *     doubleTapZoom: false
   * });
   * </code></pre>
   * @return this
   */
  public native function setTouchAction(touchAction:Object):ext.dom.Element;

  /**
   * Use this to change the visibility mode between <a href="#!/api/Ext.dom.Element-static-property-VISIBILITY" rel="Ext.dom.Element-static-property-VISIBILITY" class="docClass">VISIBILITY</a>,
   * <a href="#!/api/Ext.dom.Element-static-property-DISPLAY" rel="Ext.dom.Element-static-property-DISPLAY" class="docClass">DISPLAY</a>, <a href="#!/api/Ext.dom.Element-static-property-OFFSETS" rel="Ext.dom.Element-static-property-OFFSETS" class="docClass">OFFSETS</a>, or <a href="#!/api/Ext.dom.Element-static-property-CLIP" rel="Ext.dom.Element-static-property-CLIP" class="docClass">CLIP</a>.
   * @param mode The method by which the element will be <a href="#!/api/Ext.dom.Element-method-hide" rel="Ext.dom.Element-method-hide" class="docClass">hidden</a> (you can
   * also use the <a href="#!/api/Ext.dom.Element-method-setVisible" rel="Ext.dom.Element-method-setVisible" class="docClass">setVisible</a> or <a href="#!/api/Ext.dom.Element-method-toggle" rel="Ext.dom.Element-method-toggle" class="docClass">toggle</a> method to toggle element
   * visibility).
   * @return this
   */
  public native function setVisibilityMode(mode:Number):ext.dom.Element;

  /**
   * Sets the visibility of the element (see details). If the visibilityMode is set
   * to Element.DISPLAY, it will use the display property to hide the element,
   * otherwise it uses visibility. The default is to hide and show using the
   * visibility property.
   * @param visible Whether the element is visible
   * @param animate True for the default animation,
   * or a standard Element animation config object.
   * @return this
   */
  public native function setVisible(visible:Boolean, animate:* = undefined):ext.dom.Element;

  /**
   * Set the width of this Element.
   *
   * <pre><code>// change the width to 200px and animate with default configuration
   * <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>('elementId').setWidth(200, true);
   *
   * // change the width to 150px and animate with a custom configuration
   * <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>('elId').setWidth(150, {
   *     duration : 500, // animation will have a duration of .5 seconds
   *     // will change the content to "finished"
   *     callback: function(){ this.setHtml("finished"); }
   * });
   * </code></pre>
   * @param width The new width. This may be one of:
   * <ul>
   * <li>A Number specifying the new width in pixels.</li>
   * <li>A String used to set the CSS width style. Animation may <strong>not</strong> be used.</li>
   * </ul>
   * @param animate a standard Element animation config object or <code>true</code> for
   * the default animation (<code>{duration: 350, easing: 'ease-in'}</code>)
   * @return this
   */
  public native function setWidth(width:*, animate:* = undefined):ext.dom.Element;

  /**
   * Sets this Element's page-level x coordinate
   * @param x The X position
   * @param animate True for the default animation, or a standard
   * Element animation config object
   */
  public native function setX(x:Number, animate:* = undefined):void;

  /**
   * Sets this Element's page-level x and y coordinates
   * @param pos Contains X &amp; Y [x, y] values for new position (coordinates
   * are page-based)
   * @param animate True for the default animation, or a standard
   * Element animation config object
   */
  public native function setXY(pos:Array, animate:* = undefined):void;

  /**
   * Sets this Element's page-level y coordinate
   * @param y The Y position
   * @param animate True for the default animation, or a standard
   * Element animation config object
   */
  public native function setY(y:Number, animate:* = undefined):void;

  /**
   * Sets the z-index of this Element and synchronizes the z-index of shadow and/or
   * shim if present.
   * @param zindex The new z-index to set
   * @return this
   */
  public native function setZIndex(zindex:Number):ext.dom.Element;

  [Deprecated(replacement = "animate", since = "4.0")]
  /**
   * Animates the transition of any combination of an element's dimensions, xy position and/or opacity. Any of these
   * properties not specified in the config object will not be changed. This effect requires that at least one new
   * dimension, position or opacity setting must be passed in on the config object in order for the function to have
   * any effect. Usage:
   *
   * <pre><code>// slide the element horizontally to x position 200 while changing the height and opacity
   * el.shift({ x: 200, height: 50, opacity: .8 });
   *
   * // common config options shown with default values.
   * el.shift({
   *     width: [element's width],
   *     height: [element's height],
   *     x: [element's x position],
   *     y: [element's y position],
   *     opacity: [element's opacity],
   *     easing: 'easeOut',
   *     duration: 350
   * });
   * </code></pre>
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function shift(options:Object):ext.dom.Element;

  /**
   * Show this element - Uses display mode to determine whether to use "display",
   * "visibility", "offsets", or "clip". See <a href="#!/api/Ext.dom.Element-method-setVisible" rel="Ext.dom.Element-method-setVisible" class="docClass">setVisible</a>.
   * @param animate true for the default animation or a standard
   * Element animation config object.
   * @return this
   */
  public native function show(animate:* = undefined):ext.dom.Element;

  /**
   * Slides the element into view. An anchor point can be optionally passed to set the point of origin for the slide
   * effect. This function automatically handles wrapping the element with a fixed-size container if needed. See the
   * <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> class overview for valid anchor point options. Usage:
   *
   * <pre><code>// default: slide the element in from the top
   * el.slideIn();
   *
   * // custom: slide the element in from the right with a 2-second duration
   * el.slideIn('r', { duration: 2000 });
   *
   * // common config options shown with default values
   * el.slideIn('t', {
   *     easing: 'easeOut',
   *     duration: 500
   * });
   * </code></pre>
   * @param anchor One of the valid <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> anchor positions (defaults to top: 't')
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @param {Boolean} options.preserveScroll Set to true if preservation of any descendant elements'
   * <code>scrollTop</code> values is required. By default the DOM wrapping operation performed by <code>slideIn</code> and
   * <code>slideOut</code> causes the browser to lose all scroll positions.
   * @return The Element
   */
  public native function slideIn(anchor:String = null, options:Object = null):ext.dom.Element;

  /**
   * Slides the element out of view. An anchor point can be optionally passed to set the end point for the slide
   * effect. When the effect is completed, the element will be hidden (visibility = 'hidden') but block elements will
   * still take up space in the document. The element must be removed from the DOM using the 'remove' config option if
   * desired. This function automatically handles wrapping the element with a fixed-size container if needed. See the
   * <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> class overview for valid anchor point options. Usage:
   *
   * <pre><code>// default: slide the element out to the top
   * el.slideOut();
   *
   * // custom: slide the element out to the right with a 2-second duration
   * el.slideOut('r', { duration: 2000 });
   *
   * // common config options shown with default values
   * el.slideOut('t', {
   *     easing: 'easeOut',
   *     duration: 500,
   *     remove: false,
   *     useDisplay: false
   * });
   * </code></pre>
   * @param anchor One of the valid <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> anchor positions (defaults to top: 't')
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function slideOut(anchor:String = null, options:Object = null):ext.dom.Element;

  /**
   * Stops the specified event(s) from bubbling and optionally prevents the default action
   *
   * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     fields: ['name', 'email'],
   *     data: [{
   *         'name': 'Finn',
   *         "email": "finn&#64;adventuretime.com"
   *     }]
   * });
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title: 'Land of Ooo',
   *     store: store,
   *     columns: [{
   *         text: 'Name',
   *         dataIndex: 'name'
   *     }, {
   *         text: 'Email &lt;img style="vertical-align:middle;" src="{some-help-image-src}" /&gt;',
   *         dataIndex: 'email',
   *         flex: 1,
   *         listeners: {
   *             render: function(col) {
   *                 // Swallow the click event when the click occurs on the
   *                 // help icon - preventing the sorting of data by that
   *                 // column and instead performing an action specific to
   *                 // the help icon
   *                 var img = col.getEl().down('img');
   *                 img.swallowEvent(['click', 'mousedown'], true);
   *                 col.on('click', function() {
   *                     // logic to show a help dialog
   *                     console.log('image click handler');
   *                 }, col);
   *             }
   *         }
   *     }],
   *     height: 200,
   *     width: 400,
   *     renderTo: document.body
   * });
   * </code></pre>
   * @param eventName an event / array of events to stop from bubbling
   * @param preventDefault true to prevent the default action too
   * @return this
   */
  public native function swallowEvent(eventName:*, preventDefault:Boolean = false):ext.dom.Element;

  /**
   * Blinks the element as if it was clicked and then collapses on its center (similar to switching off a television).
   * When the effect is completed, the element will be hidden (visibility = 'hidden') but block elements will still
   * take up space in the document. The element must be removed from the DOM using the 'remove' config option if
   * desired. Usage:
   *
   * <pre><code>// default
   * el.switchOff();
   *
   * // all config options shown with default values
   * el.switchOff({
   *     easing: 'easeIn',
   *     duration: .3,
   *     remove: false,
   *     useDisplay: false
   * });
   * </code></pre>
   * @param options Object literal with any of the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> config options
   * @return The Element
   */
  public native function switchOff(options:Object = null):ext.dom.Element;

  /**
   * Toggles the element's visibility, depending on visibility mode.
   * @param animate True for the default animation, or a standard Element animation config object
   * @return this
   */
  public native function toggle(animate:* = undefined):ext.dom.Element;

  /**
   * Toggles the specified CSS class on this element (removes it if it already exists, otherwise adds it).
   * @param className The CSS class to toggle.
   * @param state If specified as <code>true</code>, causes the class to be added. If specified as <code>false</code>, causes
   * the class to be removed.
   * @return this
   */
  public native function toggleCls(className:String, state:Boolean = false):ext.dom.Element;

  /**
   * Translates the passed page coordinates into left/top css values for the element
   * @param x The page x or an array containing [x, y]
   * @param y The page y, required if x is not an array
   * @return An object with left and top properties. e.g.
   * {left: (value), top: (value)}
   */
  public native function translatePoints(x:*, y:Number = NaN):Object;

  /**
   * Return clipping (overflow) to original clipping before <a href="#!/api/Ext.dom.Element-method-clip" rel="Ext.dom.Element-method-clip" class="docClass">clip</a> was called
   * @return this
   */
  public native function unclip():ext.dom.Element;

  /**
   * Hides a previously applied mask.
   */
  public native function unmask():void;

  /**
   * Disables text selection for this element (normalized across browsers)
   * @return this
   */
  public native function unselectable():ext.dom.Element;

  /**
   * .
   * Walks up the dom looking for a parent node that matches the passed simple selector (e.g. 'div.some-class' or 'span:first-child').
   * This is a shortcut for findParentNode() that always returns an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * @param selector The simple selector to test. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param limit The max depth to search as a number or an element that causes the upward
   * traversal to stop and is <strong>not</strong> considered for inclusion as the result.
   * (defaults to 50 || document.documentElement)
   * @param returnDom True to return the DOM node instead of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @return The matching DOM node (or HTMLElement if
   * <em>returnDom</em> is <em>true</em>).  Or null if no match was found.
   */
  public native function up(selector:String, limit:* = undefined, returnDom:Boolean = false):*;

  [Deprecated(replacement = "setHtml", since = "5.0.0")]
  /**
   * Updates the innerHTML of this element, optionally searching for and processing scripts.
   * @param html The new HTML
   * @param loadScripts Pass <code>true</code> to look for and process scripts.
   * @param callback For async script loading you can be notified when the update completes.
   * @param scope (Default `this`) The scope (<code>this</code> reference) in which to execute the callback.
   * <p>Also used as the scope for any <em>inline</em> script source if the <code>loadScripts</code> parameter is <code>true</code>.
   * Scripts with a <code>src</code> attribute cannot be executed in this scope.</p>
   * <p>Defaults to this Element.</p>
   * @default `this`
   * @return this
   */
  public native function update(html:String, loadScripts:Boolean = false, callback:Function = null, scope:Object = null):ext.dom.Element;

  /**
   * Updates the <em>text</em> value of this element.
   * Replaces the content of this element with a <em>single text node</em> containing the passed text.
   * @param text The text to display in this Element.
   */
  public native function updateText(text:String):void;

  /**
   * Creates and wraps this element with another element
   * @param config DomHelper element config object for the wrapper element or null for an empty div
   * @param returnDom True to return the raw DOM element instead of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   * @param selector A CSS selector to select a descendant node within the created element to use as the wrapping element.
   * @return The newly created wrapper element
   */
  public native function wrap(config:Object = null, returnDom:Boolean = false, selector:String = null):*;
}
}