package ext.dom {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.dom.Underlay", require)]
[ExcludeClass]
/**
 * A class that provides an underlay element which displays behind an absolutely positioned
 * target element and tracks its size and position. Abstract base class for
 * <a href="#!/api/Ext.dom.Shadow" rel="Ext.dom.Shadow" class="docClass">Ext.dom.Shadow</a> and <a href="#!/api/Ext.dom.Shim" rel="Ext.dom.Shim" class="docClass">Ext.dom.Shim</a>
 */
public class Underlay extends ext.Base {
  /**
   * This Underlay's current element.
   */
  public native function get el():ext.dom.Element;

  [ExtConfig]
  /**
   * The target element
   */
  public native function get target():ext.dom.Element;

  [ExtConfig]
  /**
   * @private
   */
  public native function set target(value:ext.dom.Element):void;

  [ExtConfig]
  /**
   * The CSS z-index to use for this underlay.  Defaults to the z-index of <a href="#!/api/Ext.dom.Underlay-cfg-target" rel="Ext.dom.Underlay-cfg-target" class="docClass">target</a>.
   * @see #setZIndex()
   */
  public native function get zIndex():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set zIndex(value:Number):void;

  /**
   * Hides the underlay
   */
  public native function hide():void;

  /**
   * Aligns the underlay to its target element
   * @param x The x position of the target element.  If not provided, the
   * x position will be read from the DOM.
   * @param y The y position of the target element.  If not provided, the
   * y position will be read from the DOM.
   * @param width The width of the target element.  If not provided, the
   * width will be read from the DOM.
   * @param height The height of the target element.  If not provided, the
   * height will be read from the DOM.
   */
  public native function realign(x:Number = NaN, y:Number = NaN, width:Number = NaN, height:Number = NaN):void;

  /**
   * Adjust the z-index of this underlay
   * @param zIndex The new z-index
   */
  public native function setZIndex(zIndex:Number):void;

  /**
   * Shows the underlay
   */
  public native function show():void;

  /**
   * Called before the underlay is shown, immediately after its element is retrieved
   * from the pool
   */
  protected native function beforeShow():void;

  /**
   * Returns the dom element that this underlay should be inserted before.
   * Defaults to the target element
   */
  protected native function getInsertionTarget():ext.dom.Element;

  protected native function getPool():ext.Base;
}
}