package ext.drag.proxy {
import ext.Base;
import ext.dom.Element;
import ext.drag.DragInfo;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.drag.proxy.None", require)]
/**
 * A base class for drag proxies that are shown to represent the
 * dragged item during a drag.
 *
 * <p>Default implementations are:
 * - <a href="#!/api/Ext.drag.proxy.Original" rel="Ext.drag.proxy.Original" class="docClass">Ext.drag.proxy.Original</a>: Moves the original element.
 * - <a href="#!/api/Ext.drag.proxy.Placeholder" rel="Ext.drag.proxy.Placeholder" class="docClass">Ext.drag.proxy.Placeholder</a>: Creates a new element each drag.</p>
 *
 * <p>This implementation does not provide a proxy element, so it can be used
 * for cursor tracking only.</p>
 */
public class NoneDragProxy extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Perform any cleanup required. This is called as the drag ends.
   */
  protected native function cleanup():void;

  /**
   * Get the proxy element for the drag source. This is called as
   * the drag starts. This element may be cached on the instance and
   * reused.
   * @param info Drag info
   * @return The element.
   */
  protected native function getElement(info:ext.drag.DragInfo):ext.dom.Element;

  /**
   * Called when the target changes for the active drag. This may
   * mean the target is now <code>null</code>.
   * @param info Drag info
   */
  protected native function update(info:ext.drag.DragInfo):void;
}
}