package ext.env {
import ext.Version;

[Native("Ext.env.Browser")]
[ExcludeClass]
/**
 * Provides information about browser.
 *
 * <p>Should not be manually instantiated unless for unit-testing.
 * Access the global instance stored in <a href="#!/api/Ext.browser" rel="Ext.browser" class="docClass">Ext.browser</a> instead.</p>
 */
public class Browser {
  /**
   * The full name of the current browser's engine.
   * Possible values are:
   *
   * <ul>
   * <li>WebKit</li>
   * <li>Gecko</li>
   * <li>Presto</li>
   * <li>Trident</li>
   * <li>Other</li>
   * </ul>
   */
  public native function get engineName():String;

  /**
   * Refer to <a href="#!/api/Ext.Version" rel="Ext.Version" class="docClass">Ext.Version</a>.
   */
  public native function get engineVersion():ext.Version;

  /**
   * <code>true</code> if page is running over SSL.
   */
  public native function get isSecure():Boolean;

  /**
   * @private
   */
  public native function set isSecure(value:Boolean):void;

  /**
   * <code>true</code> if browser is using strict mode.
   */
  public native function get isStrict():Boolean;

  /**
   * @private
   */
  public native function set isStrict(value:Boolean):void;

  /**
   * The full name of the current browser.
   * Possible values are:
   *
   * <ul>
   * <li>IE</li>
   * <li>Firefox</li>
   * <li>Safari</li>
   * <li>Chrome</li>
   * <li>Opera</li>
   * <li>Other</li>
   * </ul>
   */
  public native function get name():String;

  /**
   * Browser User Agent string.
   */
  public native function get userAgent():String;

  /**
   * @private
   */
  public native function set userAgent(value:String):void;

  /**
   * Refer to <a href="#!/api/Ext.Version" rel="Ext.Version" class="docClass">Ext.Version</a>.
   */
  public native function get version():ext.Version;

  /**
   * A "hybrid" property, can be either accessed as a method call, for example:
   *
   * <pre><code>if (<a href="#!/api/Ext.browser-method-is" rel="Ext.browser-method-is" class="docClass">Ext.browser.is</a>('IE')) {
   *     // ...
   * }
   * </code></pre>
   *
   * <p>Or as an object with Boolean properties, for example:</p>
   *
   * <pre><code>if (Ext.browser.is.IE) {
   *     // ...
   * }
   * </code></pre>
   *
   * <p>Versions can be conveniently checked as well. For example:</p>
   *
   * <pre><code>if (Ext.browser.is.IE10) {
   *     // Equivalent to (Ext.browser.is.IE &amp;&amp; Ext.browser.version.equals(10))
   * }
   * </code></pre>
   *
   * <p><strong>Note:</strong> Only <a href="#!/api/Ext.Version-method-getMajor" rel="Ext.Version-method-getMajor" class="docClass">major component</a>  and <a href="#!/api/Ext.Version-method-getShortVersion" rel="Ext.Version-method-getShortVersion" class="docClass">simplified</a>
   * value of the version are available via direct property checking.</p>
   *
   * <p>Supported values are:</p>
   *
   * <ul>
   * <li>IE</li>
   * <li>Firefox</li>
   * <li>Safari</li>
   * <li>Chrome</li>
   * <li>Opera</li>
   * <li>WebKit</li>
   * <li>Gecko</li>
   * <li>Presto</li>
   * <li>Trident</li>
   * <li>WebView</li>
   * <li>Other</li>
   * </ul>
   * @param name The OS name to check.
   */
  public native function matches(name:String):Boolean;
}
}