package ext.event {
import ext.Base;
import ext.util.Point;
import js.Event;
import js.HTMLElement;

[Uses("ext.Base")]
[Native("Ext.event.Event", require)]
/**
 * Just as <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> wraps around a native DOM node, <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a> wraps the browser's native
 * event-object normalizing cross-browser differences such as mechanisms to stop event-propagation along with a method
 * to prevent default actions from taking place.
 *
 * <p>Here is a simple example of how you use it:</p>
 *
 * <pre class='inline-example preview'><code>var container = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.Container</a>', {
 *     layout: 'fit',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         id: 'logger',
 *         styleHtmlContent: true,
 *         html: 'Click somewhere!',
 *         padding: 5
 *     }]
 * });
 *
 * container.getEl().on({
 *     click: function(e, node) {
 *         var string = '';
 *
 *         string += 'You clicked at: &lt;strong&gt;{ x: ' + e.pageX + ', y: ' + e.pageY + ' }&lt;/strong&gt; &lt;i&gt;(e.pageX &amp; e.pageY)&lt;/i&gt;';
 *         string += '&lt;hr /&gt;';
 *         string += 'The HTMLElement you clicked has the className of: &lt;strong&gt;' + e.target.className + '&lt;/strong&gt; &lt;i&gt;(e.target)&lt;/i&gt;';
 *         string += '&lt;hr /&gt;';
 *         string += 'The HTMLElement which has the listener has a className of: &lt;strong&gt;' + e.currentTarget.className + '&lt;/strong&gt; &lt;i&gt;(e.currentTarget)&lt;/i&gt;';
 *
 *         <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('logger').setHtml(string);
 *     }
 * });
 * </code></pre>
 *
 * <h2>Recognizers</h2>
 *
 * <p>Ext JS includes many default event recognizers to know when a user interacts with the application.</p>
 *
 * <p>For a full list of default recognizers, and more information, please view the <a href="#!/api/Ext.event.gesture.Recognizer" rel="Ext.event.gesture.Recognizer" class="docClass">Ext.event.gesture.Recognizer</a> documentation.</p>
 *
 * <p>This class also provides a set of constants for use with key events.  These are useful
 * for determining if a specific key was pressed, and are available both on instances,
 * and as static properties of the class.  The following two statements are equivalent:</p>
 *
 * <pre><code>if (e.getKey() === <a href="#!/api/Ext.event.Event-property-TAB" rel="Ext.event.Event-property-TAB" class="docClass">Ext.event.Event.TAB</a>) {
 *     // tab key was pressed
 * }
 *
 * if (e.getKey() === e.TAB) {
 *     // tab key was pressed
 * }
 * </code></pre>
 */
public class Event extends ext.Base {
  /**
   * Key constant
   * @default 65
   */
  public static native function get A():Number;

  /**
   * Key constant
   * @default 18
   */
  public static native function get ALT():Number;

  /**
   * Key constant
   * @default 66
   */
  public static native function get B():Number;

  /**
   * Key constant
   * @default 8
   */
  public static native function get BACKSPACE():Number;

  /**
   * Key constant
   * @default 67
   */
  public static native function get C():Number;

  /**
   * Key constant
   * @default 20
   */
  public static native function get CAPS_LOCK():Number;

  /**
   * Key constant
   * @default 93
   */
  public static native function get CONTEXT_MENU():Number;

  /**
   * Key constant
   * @default 17
   */
  public static native function get CTRL():Number;

  /**
   * Key constant
   * @default 68
   */
  public static native function get D():Number;

  /**
   * Key constant
   * @default 46
   */
  public static native function get DELETE():Number;

  /**
   * Key constant
   * @default 40
   */
  public static native function get DOWN():Number;

  /**
   * Key constant
   * @default 69
   */
  public static native function get E():Number;

  /**
   * Key constant
   * @default 56
   */
  public static native function get EIGHT():Number;

  /**
   * Key constant
   * @default 35
   */
  public static native function get END():Number;

  /**
   * Key constant
   * @default 13
   */
  public static native function get ENTER():Number;

  /**
   * Key constant
   * @default 27
   */
  public static native function get ESC():Number;

  /**
   * Key constant
   * @default 70
   */
  public static native function get F():Number;

  /**
   * Key constant
   * @default 112
   */
  public static native function get F1():Number;

  /**
   * Key constant
   * @default 121
   */
  public static native function get F10():Number;

  /**
   * Key constant
   * @default 122
   */
  public static native function get F11():Number;

  /**
   * Key constant
   * @default 123
   */
  public static native function get F12():Number;

  /**
   * Key constant
   * @default 113
   */
  public static native function get F2():Number;

  /**
   * Key constant
   * @default 114
   */
  public static native function get F3():Number;

  /**
   * Key constant
   * @default 115
   */
  public static native function get F4():Number;

  /**
   * Key constant
   * @default 116
   */
  public static native function get F5():Number;

  /**
   * Key constant
   * @default 117
   */
  public static native function get F6():Number;

  /**
   * Key constant
   * @default 118
   */
  public static native function get F7():Number;

  /**
   * Key constant
   * @default 119
   */
  public static native function get F8():Number;

  /**
   * Key constant
   * @default 120
   */
  public static native function get F9():Number;

  /**
   * Key constant
   * @default 53
   */
  public static native function get FIVE():Number;

  /**
   * Key constant
   * @default 52
   */
  public static native function get FOUR():Number;

  /**
   * Key constant
   * @default 71
   */
  public static native function get G():Number;

  /**
   * Key constant
   * @default 72
   */
  public static native function get H():Number;

  /**
   * Key constant
   * @default 36
   */
  public static native function get HOME():Number;

  /**
   * Key constant
   * @default 73
   */
  public static native function get I():Number;

  /**
   * Key constant
   * @default 45
   */
  public static native function get INSERT():Number;

  /**
   * Key constant
   * @default 74
   */
  public static native function get J():Number;

  /**
   * Key constant
   * @default 75
   */
  public static native function get K():Number;

  /**
   * Key constant
   * @default 76
   */
  public static native function get L():Number;

  /**
   * Key constant
   * @default 37
   */
  public static native function get LEFT():Number;

  /**
   * Key constant
   * @default 77
   */
  public static native function get M():Number;

  /**
   * Key constant
   * @default 78
   */
  public static native function get N():Number;

  /**
   * Key constant
   * @default 57
   */
  public static native function get NINE():Number;

  /**
   * Key constant
   * @default 12
   */
  public static native function get NUM_CENTER():Number;

  /**
   * Key constant
   * @default 111
   */
  public static native function get NUM_DIVISION():Number;

  /**
   * Key constant
   * @default 104
   */
  public static native function get NUM_EIGHT():Number;

  /**
   * Key constant
   * @default 101
   */
  public static native function get NUM_FIVE():Number;

  /**
   * Key constant
   * @default 100
   */
  public static native function get NUM_FOUR():Number;

  /**
   * Key constant
   * @default 109
   */
  public static native function get NUM_MINUS():Number;

  /**
   * Key constant
   * @default 106
   */
  public static native function get NUM_MULTIPLY():Number;

  /**
   * Key constant
   * @default 105
   */
  public static native function get NUM_NINE():Number;

  /**
   * Key constant
   * @default 97
   */
  public static native function get NUM_ONE():Number;

  /**
   * Key constant
   * @default 110
   */
  public static native function get NUM_PERIOD():Number;

  /**
   * Key constant
   * @default 107
   */
  public static native function get NUM_PLUS():Number;

  /**
   * Key constant
   * @default 103
   */
  public static native function get NUM_SEVEN():Number;

  /**
   * Key constant
   * @default 102
   */
  public static native function get NUM_SIX():Number;

  /**
   * Key constant
   * @default 99
   */
  public static native function get NUM_THREE():Number;

  /**
   * Key constant
   * @default 98
   */
  public static native function get NUM_TWO():Number;

  /**
   * Key constant
   * @default 96
   */
  public static native function get NUM_ZERO():Number;

  /**
   * Key constant
   * @default 79
   */
  public static native function get O():Number;

  /**
   * Key constant
   * @default 49
   */
  public static native function get ONE():Number;

  /**
   * Key constant
   * @default 80
   */
  public static native function get P():Number;

  /**
   * Key constant
   * @default 34
   */
  public static native function get PAGE_DOWN():Number;

  /**
   * Key constant
   * @default 33
   */
  public static native function get PAGE_UP():Number;

  /**
   * Key constant
   * @default 19
   */
  public static native function get PAUSE():Number;

  /**
   * Key constant
   * @default 44
   */
  public static native function get PRINT_SCREEN():Number;

  /**
   * Key constant
   * @default 81
   */
  public static native function get Q():Number;

  /**
   * Key constant
   * @default 82
   */
  public static native function get R():Number;

  /**
   * Key constant
   * @default 13
   */
  public static native function get RETURN():Number;

  /**
   * Key constant
   * @default 39
   */
  public static native function get RIGHT():Number;

  /**
   * Key constant
   * @default 83
   */
  public static native function get S():Number;

  /**
   * Key constant
   * @default 55
   */
  public static native function get SEVEN():Number;

  /**
   * Key constant
   * @default 16
   */
  public static native function get SHIFT():Number;

  /**
   * Key constant
   * @default 54
   */
  public static native function get SIX():Number;

  /**
   * Key constant
   * @default 32
   */
  public static native function get SPACE():Number;

  /**
   * Key constant
   * @default 84
   */
  public static native function get T():Number;

  /**
   * Key constant
   * @default 9
   */
  public static native function get TAB():Number;

  /**
   * Key constant
   * @default 51
   */
  public static native function get THREE():Number;

  /**
   * Key constant
   * @default 50
   */
  public static native function get TWO():Number;

  /**
   * Key constant
   * @default 85
   */
  public static native function get U():Number;

  /**
   * Key constant
   * @default 38
   */
  public static native function get UP():Number;

  /**
   * Key constant
   * @default 86
   */
  public static native function get V():Number;

  /**
   * Key constant
   * @default 87
   */
  public static native function get W():Number;

  /**
   * The mouse wheel delta scaling factor. This value depends on browser version and OS and
   * attempts to produce a similar scrolling experience across all platforms and browsers.
   *
   * <p>To change this value:</p>
   *
   * <pre><code> Ext.event.Event.prototype.WHEEL_SCALE = 72;
   * </code></pre>
   */
  public static native function get WHEEL_SCALE():Number;

  /**
   * Key constant
   * @default 88
   */
  public static native function get X():Number;

  /**
   * Key constant
   * @default 89
   */
  public static native function get Y():Number;

  /**
   * Key constant
   * @default 90
   */
  public static native function get Z():Number;

  /**
   * Key constant
   * @default 48
   */
  public static native function get ZERO():Number;

  /**
   * Create an Ext IEventObject browser event wrapper from the
   * given browser event.
   * @param browserEvent the browser event to wrap
   */
  public function Event(browserEvent:js.Event) {
    super();
  }

  /**
   * True if the alt key was down during the event.
   */
  public native function get altKey():Boolean;

  /**
   * @private
   */
  public native function set altKey(value:Boolean):void;

  /**
   * The angle of the rotation.
   *
   * <p><strong>This is only available when the event type is <code>rotate</code></strong></p>
   */
  public native function get angle():Number;

  /**
   * @private
   */
  public native function set angle(value:Number):void;

  /**
   * The raw browser event which this object wraps.
   */
  public native function get browserEvent():js.Event;

  /**
   * @private
   */
  public native function set browserEvent(value:js.Event):void;

  /**
   * Indicates which mouse button caused the event for mouse events, for example
   * <code>mousedown</code>, <code>click</code>, <code>mouseup</code>:
   * - <code>0</code> for left button.
   * - <code>1</code> for middle button.
   * - <code>2</code> for right button.
   *
   * <p><em>Note</em>: In IE8 &amp; IE9, the <code>click</code> event does not provide the button.</p>
   */
  public native function get button():Number;

  /**
   * @private
   */
  public native function set button(value:Number):void;

  /**
   * True if the control key was down during the event.
   * In Mac this will also be true when meta key was down.
   */
  public native function get ctrlKey():Boolean;

  /**
   * @private
   */
  public native function set ctrlKey(value:Boolean):void;

  /**
   * Refers to the element the event handler was attached to, vs the <code>target</code>, which is
   * the actual element that fired the event.  For example, if the event bubbles, the
   * <code>target</code> element may be a descendant of the <code>currentTarget</code>, as the event may
   * have been triggered on the <code>target</code> and then bubbled up to the <code>currentTarget</code>
   * where it was handled.
   */
  public native function get currentTarget():js.HTMLElement;

  /**
   * @private
   */
  public native function set currentTarget(value:js.HTMLElement):void;

  /**
   * Indicates whether or not <a href="#!/api/Ext.event.Event-method-preventDefault" rel="Ext.event.Event-method-preventDefault" class="docClass">preventDefault()</a> was called on the event.
   * @default false
   */
  public native function get defaultPrevented():Boolean;

  /**
   * @private
   */
  public native function set defaultPrevented(value:Boolean):void;

  [Deprecated(replacement = "currentTarget", since = "5.0.0")]
  /**
   * Same as <code>currentTarget</code>
   */
  public native function get delegatedTarget():js.HTMLElement;

  /**
   * @private
   */
  public native function set delegatedTarget(value:js.HTMLElement):void;

  /**
   * The direction of the swipe. Available options are:
   *
   * <ul>
   * <li>up</li>
   * <li>down</li>
   * <li>left</li>
   * <li>right</li>
   * </ul>
   *
   *
   * <p><strong>This is only available when the event type is <code>swipe</code></strong></p>
   */
  public native function get direction():Number;

  /**
   * @private
   */
  public native function set direction(value:Number):void;

  /**
   * The distance of the event.
   *
   * <p><strong>This is only available when the event type is <code>swipe</code> and <code>pinch</code>.</strong></p>
   */
  public native function get distance():Number;

  /**
   * @private
   */
  public native function set distance(value:Number):void;

  /**
   * The duration of the swipe.
   *
   * <p><strong>This is only available when the event type is <code>swipe</code></strong></p>
   */
  public native function get duration():Number;

  /**
   * @private
   */
  public native function set duration(value:Number):void;

  /**
   * The browsers x coordinate of the event.
   * Note: this only works in browsers that support pageX on the native browser event
   * object (pageX is not natively supported in IE9 and earlier).  In Ext JS, for a
   * cross browser normalized x-coordinate use <a href="#!/api/Ext.event.Event-method-getX" rel="Ext.event.Event-method-getX" class="docClass">getX</a>
   */
  public native function get pageX():Number;

  /**
   * @private
   */
  public native function set pageX(value:Number):void;

  /**
   * The browsers y coordinate of the event.
   * Note: this only works in browsers that support pageY on the native browser event
   * object (pageY is not natively supported in IE9 and earlier).  In Ext JS, for a
   * cross browser normalized y-coordinate use <a href="#!/api/Ext.event.Event-method-getY" rel="Ext.event.Event-method-getY" class="docClass">getY</a>
   */
  public native function get pageY():Number;

  /**
   * @private
   */
  public native function set pageY(value:Number):void;

  /**
   * The pointer type for this event. May be empty if the event was
   * not triggered by a pointer. Current available types are:
   * - <code>mouse</code>
   * - <code>touch</code>
   * - <code>pen</code>
   */
  public native function get pointerType():String;

  /**
   * @private
   */
  public native function set pointerType(value:String):void;

  /**
   * A amount of rotation, since the start of the event.
   *
   * <p><strong>This is only available when the event type is <code>rotate</code></strong></p>
   */
  public native function get rotation():Number;

  /**
   * @private
   */
  public native function set rotation(value:Number):void;

  /**
   * The scape of a pinch event.
   *
   * <p><strong>This is only available when the event type is <code>pinch</code></strong></p>
   */
  public native function get scale():Number;

  /**
   * @private
   */
  public native function set scale(value:Number):void;

  /**
   * True if the shift key was down during the event.
   */
  public native function get shiftKey():Boolean;

  /**
   * @private
   */
  public native function set shiftKey(value:Boolean):void;

  /**
   * The element that fired this event.  For the element whose handlers are currently
   * being processed, i.e. the element that the event handler was attached to, use
   * <code>currentTarget</code>
   */
  public native function get target():js.HTMLElement;

  /**
   * @private
   */
  public native function set target(value:js.HTMLElement):void;

  /**
   * The event type (copied from the browser event).
   */
  public native function get type():String;

  /**
   * @private
   */
  public native function set type(value:String):void;

  /**
   * Claims this event as the currently active gesture.  Once a gesture is claimed
   * no other gestures will fire events until after the current gesture has completed.
   * For example, if <code>claimGesture()</code> is invoked on a dragstart or drag event, no
   * swipestart or swipe events will be fired until the drag gesture completes, even if
   * the gesture also meets the required duration and distance requirements to be recognized
   * as a swipe.
   *
   * <p>If <code>claimGesture()</code> is invoked on a mouse, touch, or pointer event, it will disable
   * all gesture events until termination of the current gesture is indicated by a
   * mouseup, touchend, or pointerup event.</p>
   */
  public native function claimGesture():ext.event.Event;

  [Deprecated(since = "5.0.0")]
  /**
   * Clones this event.
   * @return The cloned copy
   */
  public native function clone():ext.event.Event;

  /**
   * Gets the character code for the event.
   */
  public native function getCharCode():Number;

  /**
   * Returns a normalized keyCode for the event.
   * @return The key code
   */
  public native function getKey():Number;

  /**
   * Returns the name of the keyCode for the event.
   * @return The key name
   */
  public native function getKeyName():String;

  [Deprecated(replacement = "getX", since = "4.0")]
  /**
   * Gets the x coordinate of the event.
   */
  public native function getPageX():Number;

  [Deprecated(replacement = "getY", since = "4.0")]
  /**
   * Gets the y coordinate of the event.
   */
  public native function getPageY():Number;

  /**
   * Returns a point object that consists of the object coordinates.
   * @return point
   */
  public native function getPoint():ext.util.Point;

  /**
   * Gets the related target.
   * @param selector A simple selector to filter the target or look for an
   * ancestor of the target. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple
   * selectors.
   * @param maxDepth The max depth to search as a number or
   * element (defaults to 10 || document.body).
   * @param returnEl <code>true</code> to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> object instead of DOM
   * node.
   */
  public native function getRelatedTarget(selector:String = null, maxDepth:* = undefined, returnEl:Boolean = false):js.HTMLElement;

  /**
   * Gets the target for the event.
   * @param selector A simple selector to filter the target or look
   * for an ancestor of the target
   * @param maxDepth (Default 10||document.body) The max depth to
   * search as a number or element (defaults to 10 || document.body)
   * @default 10||document.body
   * @param returnEl <code>true</code> to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> object instead
   * of DOM node.
   */
  public native function getTarget(selector:String = null, maxDepth:* = 10, returnEl:Boolean = false):js.HTMLElement;

  /**
   * Returns the time of the event.
   */
  public native function getTime():Date;

  /**
   * Normalizes mouse wheel y-delta across browsers. To get x-delta information, use
   * <a href="#!/api/Ext.event.Event-method-getWheelDeltas" rel="Ext.event.Event-method-getWheelDeltas" class="docClass">getWheelDeltas</a> instead.
   * @return The mouse wheel y-delta
   */
  public native function getWheelDelta():Number;

  /**
   * Returns the mouse wheel deltas for this event.
   * @return An object with "x" and "y" properties holding the mouse wheel deltas.
   */
  public native function getWheelDeltas():Object;

  /**
   * Gets the x coordinate of the event.
   */
  public native function getX():Number;

  /**
   * Gets the X and Y coordinates of the event.
   * @return The xy values like [x, y]
   */
  public native function getXY():Array;

  /**
   * Gets the y coordinate of the event.
   */
  public native function getY():Number;

  /**
   * Returns true if the control, meta, shift or alt key was pressed during this event.
   */
  public native function hasModifier():Boolean;

  /**
   * Injects a DOM event using the data in this object and (optionally) a new target.
   * This is a low-level technique and not likely to be used by application code. The
   * currently supported event types are:
   *
   * <p><b>HTMLEvents</b></p>
   *
   *
   * <ul>
   * <li>load</li>
   * <li>unload</li>
   * <li>select</li>
   * <li>change</li>
   * <li>submit</li>
   * <li>reset</li>
   * <li>resize</li>
   * <li>scroll</li>
   * </ul>
   *
   *
   * <p><b>MouseEvents</b></p>
   *
   *
   * <ul>
   * <li>click</li>
   * <li>dblclick</li>
   * <li>mousedown</li>
   * <li>mouseup</li>
   * <li>mouseover</li>
   * <li>mousemove</li>
   * <li>mouseout</li>
   * </ul>
   *
   *
   * <p><b>UIEvents</b></p>
   *
   *
   * <ul>
   * <li>focusin</li>
   * <li>focusout</li>
   * <li>activate</li>
   * <li>focus</li>
   * <li>blur</li>
   * </ul>
   * @param target If specified, the target for the event. This
   * is likely to be used when relaying a DOM event. If not specified, <a href="#!/api/Ext.event.Event-method-getTarget" rel="Ext.event.Event-method-getTarget" class="docClass">getTarget</a>
   * is used to determine the target.
   */
  public native function injectEvent(target:* = undefined):void;

  /**
   * Checks if the key pressed was a "navigation" key. A navigation key is defined by
   * these keys:
   *
   * <ul>
   * <li>Page Up</li>
   * <li>Page Down</li>
   * <li>End</li>
   * <li>Home</li>
   * <li>Left</li>
   * <li>Up</li>
   * <li>Right</li>
   * <li>Down</li>
   * <li>Return</li>
   * <li>Tab</li>
   * <li>Esc</li>
   * </ul>
   * @param scrollableOnly Only check navigation keys that can cause
   * element scrolling by their default action.
   * @return <code>true</code> if the press is a navigation keypress
   */
  public native function isNavKeyPress(scrollableOnly:Boolean = false):Boolean;

  /**
   * Checks if the key pressed was a "special" key. A special key is defined as one of
   * these keys:
   *
   * <ul>
   * <li>Page Up</li>
   * <li>Page Down</li>
   * <li>End</li>
   * <li>Home</li>
   * <li>Left arrow</li>
   * <li>Up arrow</li>
   * <li>Right arrow</li>
   * <li>Down arrow</li>
   * <li>Return</li>
   * <li>Tab</li>
   * <li>Esc</li>
   * <li>Backspace</li>
   * <li>Delete</li>
   * <li>Shift</li>
   * <li>Ctrl</li>
   * <li>Alt</li>
   * <li>Pause</li>
   * <li>Caps Lock</li>
   * <li>Print Screen</li>
   * <li>Insert</li>
   * </ul>
   * @return <code>true</code> if the key for this event is special
   */
  public native function isSpecialKey():Boolean;

  /**
   * Prevents the browsers default handling of the event.
   *
   * <p><strong>From override Ext.overrides.event.Event:</strong> call to produce method</p>
   * @return this
   */
  public native function preventDefault():ext.event.Event;

  /**
   * Initiates a drag gesture in response to this event
   *
   * <p>Only available when <code>type</code> is <code>'longpress'</code>.  When invoked a dragstart event
   * will be immediately fired at the coordinates of the longpress event.  Thereafter
   * drag events will fire in response to movement on the screen without regard
   * to the distance moved.</p>
   */
  public native function startDrag():void;

  /**
   * Stop the event (<code><a href="#!/api/Ext.event.Event-method-preventDefault" rel="Ext.event.Event-method-preventDefault" class="docClass">preventDefault</a></code> and <code><a href="#!/api/Ext.event.Event-method-stopPropagation" rel="Ext.event.Event-method-stopPropagation" class="docClass">stopPropagation</a></code>).
   * @return this
   */
  public native function stopEvent():ext.event.Event;

  /**
   * Cancels bubbling of the event.
   *
   * <p><strong>Overridden in Ext.overrides.event.Event.</strong></p>
   * @return this
   */
  public native function stopPropagation():ext.event.Event;

  /**
   * Returns true if the target of this event is a child of <code>el</code>.  Unless the allowEl
   * parameter is set, it will return false if if the target is <code>el</code>.
   * Example usage:
   *
   * <pre><code>// Handle click on any child of an element
   * <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().on('click', function(e){
   *     if(e.within('some-el')){
   *         alert('Clicked on a child of some-el!');
   *     }
   * });
   *
   * // Handle click directly on an element, ignoring clicks on child nodes
   * <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().on('click', function(e,t){
   *     if((t.id == 'some-el') &amp;&amp; !e.within(t, true)){
   *         alert('Clicked directly on some-el!');
   *     }
   * });
   * </code></pre>
   * @param el The id, DOM element or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> to check
   * @param related <code>true</code> to test if the related target is within el instead
   * of the target
   * @param allowEl <code>true</code> to also check if the passed element is the target
   * or related target
   */
  public native function within(el:*, related:Boolean = false, allowEl:Boolean = false):Boolean;
}
}