package ext.form {
import ext.Mixin;
import ext.form.field.Field;

[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Native("Ext.form.FieldAncestor", require)]
[ExtConfig]
/**
 * Fires when the active error message is changed for any one of the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> instances
 * within this container.
 * @eventType ext.form.events.FieldAncestor_field_errorEvent.FIELD_ERROR_CHANGE
 */
[Event(name = 'onFieldErrorChange', type = 'ext.form.events.FieldAncestor_field_errorEvent')]
/**
 * Fires when the validity state of any one of the <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> instances within this
 * container changes.
 * @eventType ext.form.events.FieldAncestor_field_isValidEvent.FIELD_VALIDITY_CHANGE
 */
[Event(name = 'onFieldValidityChange', type = 'ext.form.events.FieldAncestor_field_isValidEvent')]
/**
 * A mixin for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> components that are likely to have form fields in their
 * items subtree. Adds the following capabilities:
 *
 * <ul>
 * <li>Methods for handling the addition and removal of <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> and <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a>
 * instances at any depth within the container.</li>
 * <li>Events (<a href="#!/api/Ext.form.FieldAncestor-event-fieldvaliditychange" rel="Ext.form.FieldAncestor-event-fieldvaliditychange" class="docClass">fieldvaliditychange</a> and <a href="#!/api/Ext.form.FieldAncestor-event-fielderrorchange" rel="Ext.form.FieldAncestor-event-fielderrorchange" class="docClass">fielderrorchange</a>) for handling changes to the state
 * of individual fields at the container level.</li>
 * <li>Automatic application of <a href="#!/api/Ext.form.FieldAncestor-cfg-fieldDefaults" rel="Ext.form.FieldAncestor-cfg-fieldDefaults" class="docClass">fieldDefaults</a> config properties to each field added within the
 * container, to facilitate uniform configuration of all fields.</li>
 * </ul>
 *
 *
 * <p>This mixin is primarily for internal use by <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a> and <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>,
 * and should not normally need to be used directly.</p>
 */
public class FieldAncestor extends ext.Mixin implements ext.form.IFieldAncestor {
  /**
   * @param config @inheritDoc
   */
  public function FieldAncestor(config:ext.form.FieldAncestor = null) {
    super();
  }

  [ExtConfig]
  /**
   * If specified, the properties in this object are used as default config values for each <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a>
   * instance (e.g. <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> or <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>) that is added as a descendant of
   * this container. Corresponding values specified in an individual field's own configuration, or from the <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults config</a> of its parent container, will take precedence. See the
   * documentation for <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> to see what config options may be specified in the fieldDefaults.
   *
   * <p>Example:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
   *     fieldDefaults: {
   *         labelAlign: 'left',
   *         labelWidth: 100
   *     },
   *     items: [{
   *         xtype: 'fieldset',
   *         defaults: {
   *             labelAlign: 'top'
   *         },
   *         items: [{
   *             name: 'field1'
   *         }, {
   *             name: 'field2'
   *         }]
   *     }, {
   *         xtype: 'fieldset',
   *         items: [{
   *             name: 'field3',
   *             labelWidth: 150
   *         }, {
   *             name: 'field4'
   *         }]
   *     }]
   * });
   * </code></pre>
   *
   * <p>In this example, field1 and field2 will get labelAlign:'top' (from the fieldset's defaults) and labelWidth:100
   * (from fieldDefaults), field3 and field4 will both get labelAlign:'left' (from fieldDefaults and field3 will use
   * the labelWidth:150 from its own config.</p>
   */
  public native function get fieldDefaults():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldDefaults(value:Object):void;

  /**
   * Initializes the FieldAncestor's state; this must be called from the initComponent method of any components
   * importing this mixin.
   */
  protected native function initFieldAncestor():void;

  /**
   * Fired when the error message of any field within the container changes.
   * @param field The sub-field whose active error changed
   * @param error The new active error message
   */
  protected native function onFieldErrorChange(field:ext.form.Labelable, error:String):void;

  /**
   * Fired when the validity of any field within the container changes.
   * @param field The sub-field whose validity changed
   * @param valid The new validity state
   */
  protected native function onFieldValidityChange(field:ext.form.field.Field, valid:Boolean):void;
}
}