package ext.form {
import ext.Component;
import ext.container.Container;
import ext.dom.Element;
import ext.form.field.Field;

[Uses("ext.container.Container")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.FieldAncestor")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.form.FieldContainer", require)]
/**
 * Fires when the active error message is changed via <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a>.
 * @eventType ext.form.events.FieldContainer_errorEvent.ERROR_CHANGE
 */
[Event(name = 'onErrorChange', type = 'ext.form.events.FieldContainer_errorEvent')]
/**
 * Fires when the active error message is changed for any one of the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> instances
 * within this container.
 * @eventType ext.form.events.FieldContainer_field_errorEvent.FIELD_ERROR_CHANGE
 */
[Event(name = 'onFieldErrorChange', type = 'ext.form.events.FieldContainer_field_errorEvent')]
/**
 * Fires when the validity state of any one of the <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> instances within this
 * container changes.
 * @eventType ext.form.events.FieldContainer_field_isValidEvent.FIELD_VALIDITY_CHANGE
 */
[Event(name = 'onFieldValidityChange', type = 'ext.form.events.FieldContainer_field_isValidEvent')]
/**
 * FieldContainer is a derivation of <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> that implements the
 * <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Labelable</a> mixin. This allows it to be configured so that it is rendered with
 * a <a href="#!/api/Ext.form.FieldContainer-cfg-fieldLabel" rel="Ext.form.FieldContainer-cfg-fieldLabel" class="docClass">field label</a> and optional <a href="#!/api/Ext.form.FieldContainer-cfg-msgTarget" rel="Ext.form.FieldContainer-cfg-msgTarget" class="docClass">error message</a> around its sub-items.
 * This is useful for arranging a group of fields or other components within a single item in a form, so
 * that it lines up nicely with other fields. A common use is for grouping a set of related fields under
 * a single label in a form.
 *
 * <p>The container's configured <a href="#!/api/Ext.form.FieldContainer-cfg-items" rel="Ext.form.FieldContainer-cfg-items" class="docClass">items</a> will be layed out within the field body area according to the
 * configured <a href="#!/api/Ext.form.FieldContainer-cfg-layout" rel="Ext.form.FieldContainer-cfg-layout" class="docClass">layout</a> type. The default layout is <code>'autocontainer'</code>.</p>
 *
 * <p>Like regular fields, FieldContainer can inherit its decoration configuration from the
 * <a href="#!/api/Ext.form.Panel-cfg-fieldDefaults" rel="Ext.form.Panel-cfg-fieldDefaults" class="docClass">fieldDefaults</a> of an enclosing FormPanel. In addition,
 * FieldContainer itself can pass <a href="#!/api/Ext.form.FieldContainer-cfg-fieldDefaults" rel="Ext.form.FieldContainer-cfg-fieldDefaults" class="docClass">fieldDefaults</a> to any <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">fields</a>
 * it may itself contain.</p>
 *
 * <p>If you are grouping a set of <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">Checkbox</a> or <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">Radio</a>
 * fields in a single labeled container, consider using a <a href="#!/api/Ext.form.CheckboxGroup" rel="Ext.form.CheckboxGroup" class="docClass">Ext.form.CheckboxGroup</a>
 * or <a href="#!/api/Ext.form.RadioGroup" rel="Ext.form.RadioGroup" class="docClass">Ext.form.RadioGroup</a> instead as they are specialized for handling those types.</p>
 *
 * <h1>Example</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'FieldContainer Example',
 *     width: 550,
 *     bodyPadding: 10,
 *
 *     items: [{
 *         xtype: 'fieldcontainer',
 *         fieldLabel: 'Last Three Jobs',
 *         labelWidth: 100,
 *
 *         // The body area will contain three text fields, arranged
 *         // horizontally, separated by draggable splitters.
 *         layout: 'hbox',
 *         items: [{
 *             xtype: 'textfield',
 *             flex: 1
 *         }, {
 *             xtype: 'splitter'
 *         }, {
 *             xtype: 'textfield',
 *             flex: 1
 *         }, {
 *             xtype: 'splitter'
 *         }, {
 *             xtype: 'textfield',
 *             flex: 1
 *         }]
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <h1>Usage of fieldDefaults</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'FieldContainer Example',
 *     width: 350,
 *     bodyPadding: 10,
 *
 *     items: [{
 *         xtype: 'fieldcontainer',
 *         fieldLabel: 'Your Name',
 *         labelWidth: 75,
 *         defaultType: 'textfield',
 *
 *         // Arrange fields vertically, stretched to full width
 *         layout: 'anchor',
 *         defaults: {
 *             layout: '100%'
 *         },
 *
 *         // These config values will be applied to both sub-fields, except
 *         // for Last Name which will use its own msgTarget.
 *         fieldDefaults: {
 *             msgTarget: 'under',
 *             labelAlign: 'top'
 *         },
 *
 *         items: [{
 *             fieldLabel: 'First Name',
 *             name: 'firstName'
 *         }, {
 *             fieldLabel: 'Last Name',
 *             name: 'lastName',
 *             msgTarget: 'under'
 *         }]
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class FieldContainer extends ext.container.Container implements ext.form.ILabelable, ext.form.IFieldAncestor {
  /**
   * @param config @inheritDoc
   */
  public function FieldContainer(config:ext.form.FieldContainer = null) {
    super();
  }

  /**
   * The div Element wrapping the component's contents. Only available after the component has been rendered.
   */
  public native function get bodyEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set bodyEl(value:ext.dom.Element):void;

  /**
   * The div Element that will contain the component's error message(s). Note that depending on the configured
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">msgTarget</a>, this element may be hidden in favor of some other form of presentation, but will always
   * be present in the DOM for use by assistive technologies.
   */
  public native function get errorEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set errorEl(value:ext.dom.Element):void;

  /**
   * Flag denoting that this object is labelable as a field. Always true.
   * @default true
   */
  public native function get isFieldLabelable():Boolean;

  /**
   * @private
   */
  public native function set isFieldLabelable(value:Boolean):void;

  /**
   * The label Element for this component. Only available after the component has been rendered.
   */
  public native function get labelEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set labelEl(value:ext.dom.Element):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If specified, then the component will be displayed with this value as its active error when first rendered. Use
   * <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a> or <a href="#!/api/Ext.form.Labelable-method-unsetActiveError" rel="Ext.form.Labelable-method-unsetActiveError" class="docClass">unsetActiveError</a> to change it after component creation.
   * @see #getActiveError()
   * @see #setActiveError()
   */
  public native function get activeError():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activeError(value:String):void;

  [ExtConfig]
  /**
   * The template used to format the Array of error messages passed to <a href="#!/api/Ext.form.Labelable-method-setActiveErrors" rel="Ext.form.Labelable-method-setActiveErrors" class="docClass">setActiveErrors</a> into a single HTML
   * string. if the <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">msgTarget</a> is title, it defaults to a list separated by new lines. Otherwise, it
   * renders each message as an item in an unordered list.
   */
  public native function get activeErrorsTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeErrorsTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * at the end of the input containing element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterBodyEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterBodyEl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the label text. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the label element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the <a href="#!/api/Ext.form.Labelable-method-getSubTplMarkup" rel="Ext.form.Labelable-method-getSubTplMarkup" class="docClass">subTpl markup</a>. If an <code>XTemplate</code> is used, the
   * component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get afterSubTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterSubTpl(value:*):void;

  [ExtConfig]
  /**
   * Localized announcement text for validation errors. This text
   * will be used by Assistive Technologies such as screen readers to alert the users when
   * field validation fails.
   *
   * <p>This config is used with Ext.String.format. '{0}' will be replaced with the actual
   * error message(s), '{1}' will be replaced with field label.</p>
   * @default 'Input error. {0}.'
   */
  public native function get ariaErrorText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaErrorText(value:String):void;

  [ExtConfig]
  /**
   * Optional text description for this object. This text will be
   * announced to Assistive Technology users when the object is focused.
   */
  public native function get ariaHelp():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaHelp(value:String):void;

  [ExtConfig]
  /**
   * Whether to adjust the component's body width to make room for 'side'
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">error messages</a>.
   * @default true
   */
  public native function get autoFitErrors():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoFitErrors(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the body content element.
   * @default Ext.baseCSSPrefix + 'form-item-body'
   */
  public native function get baseBodyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseBodyCls(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * at the beginning of the input containing element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeBodyEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeBodyEl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the label text. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the label element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the <a href="#!/api/Ext.form.Labelable-method-getSubTplMarkup" rel="Ext.form.Labelable-method-getSubTplMarkup" class="docClass">subTpl markup</a>. If an <code>XTemplate</code> is used, the
   * component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get beforeSubTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeSubTpl(value:*):void;

  [ExtConfig]
  /**
   * If set to true, the field container will automatically combine and display the validation errors from
   * all the fields it contains as a single error on the container, according to the configured
   * <a href="#!/api/Ext.form.FieldContainer-cfg-msgTarget" rel="Ext.form.FieldContainer-cfg-msgTarget" class="docClass">msgTarget</a>. Defaults to false.
   * @default false
   */
  public native function get combineErrors():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set combineErrors(value:Boolean):void;

  [ExtConfig]
  /**
   * If set to true, and there is no defined <a href="#!/api/Ext.form.FieldContainer-cfg-fieldLabel" rel="Ext.form.FieldContainer-cfg-fieldLabel" class="docClass">fieldLabel</a>, the field container will automatically
   * generate its label by combining the labels of all the fields it contains. Defaults to false.
   * @default false
   */
  public native function get combineLabels():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set combineLabels(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the error message element.
   * @default Ext.baseCSSPrefix + 'form-error-msg'
   */
  public native function get errorMsgCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set errorMsgCls(value:String):void;

  [ExtConfig]
  /**
   * An extra CSS class to be applied to the body content element in addition to <a href="#!/api/Ext.form.Labelable-cfg-baseBodyCls" rel="Ext.form.Labelable-cfg-baseBodyCls" class="docClass">baseBodyCls</a>.
   * @default ''
   */
  public native function get fieldBodyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldBodyCls(value:String):void;

  [ExtConfig]
  /**
   * If specified, the properties in this object are used as default config values for each <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a>
   * instance (e.g. <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> or <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>) that is added as a descendant of
   * this container. Corresponding values specified in an individual field's own configuration, or from the <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults config</a> of its parent container, will take precedence. See the
   * documentation for <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> to see what config options may be specified in the fieldDefaults.
   *
   * <p>Example:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
   *     fieldDefaults: {
   *         labelAlign: 'left',
   *         labelWidth: 100
   *     },
   *     items: [{
   *         xtype: 'fieldset',
   *         defaults: {
   *             labelAlign: 'top'
   *         },
   *         items: [{
   *             name: 'field1'
   *         }, {
   *             name: 'field2'
   *         }]
   *     }, {
   *         xtype: 'fieldset',
   *         items: [{
   *             name: 'field3',
   *             labelWidth: 150
   *         }, {
   *             name: 'field4'
   *         }]
   *     }]
   * });
   * </code></pre>
   *
   * <p>In this example, field1 and field2 will get labelAlign:'top' (from the fieldset's defaults) and labelWidth:100
   * (from fieldDefaults), field3 and field4 will both get labelAlign:'left' (from fieldDefaults and field3 will use
   * the labelWidth:150 from its own config.</p>
   * @see #setFieldDefaults()
   */
  public native function get fieldDefaults():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fieldDefaults(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The label for the field. It gets appended with the <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>, and its position and sizing is
   * determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> configs.
   * @see #getFieldLabel()
   * @see #setFieldLabel()
   */
  public native function get fieldLabel():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fieldLabel(value:String):void;

  [ExtConfig]
  /**
   * A CSS class to be applied to the outermost element to denote that it is participating in the form field layout.
   * @default Ext.baseCSSPrefix + 'form-item'
   */
  public native function get formItemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formItemCls(value:String):void;

  [ExtConfig]
  /**
   * When set to true, the label element (<a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>) will be automatically
   * hidden if the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> is empty. Setting this to false will cause the empty label element to be
   * rendered and space to be reserved for it; this is useful if you want a field without a label to line up with
   * other labeled fields in the same form.
   *
   * <p>If you wish to unconditionall hide the label even if a non-empty fieldLabel is configured, then set the
   * <a href="#!/api/Ext.form.Labelable-cfg-hideLabel" rel="Ext.form.Labelable-cfg-hideLabel" class="docClass">hideLabel</a> config to true.</p>
   * @default true
   */
  public native function get hideEmptyLabel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideEmptyLabel(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true to completely hide the label element (<a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>). Also see
   * <a href="#!/api/Ext.form.Labelable-cfg-hideEmptyLabel" rel="Ext.form.Labelable-cfg-hideEmptyLabel" class="docClass">hideEmptyLabel</a>, which controls whether space will be reserved for an empty fieldLabel.
   * @default false
   */
  public native function get hideLabel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideLabel(value:Boolean):void;

  [ExtConfig]
  /**
   * Controls the position and alignment of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a>. Valid values are:
   *
   * <ul>
   * <li>"left" (the default) - The label is positioned to the left of the field, with its text aligned to the left.
   * Its width is determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> config.</li>
   * <li>"top" - The label is positioned above the field.</li>
   * <li>"right" - The label is positioned to the left of the field, with its text aligned to the right.
   * Its width is determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> config.</li>
   * </ul>
   * @default 'left'
   */
  public native function get labelAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelAlign(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * inside the label element (as attributes). If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get labelAttrTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelAttrTpl(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the label element. This (single) CSS class is used to formulate the renderSelector
   * and drives the field layout where it is concatenated with a hyphen ('-') and <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a>. To add
   * additional classes, use <a href="#!/api/Ext.form.Labelable-cfg-labelClsExtra" rel="Ext.form.Labelable-cfg-labelClsExtra" class="docClass">labelClsExtra</a>.
   * @default Ext.baseCSSPrefix + 'form-item-label'
   */
  public native function get labelCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelCls(value:String):void;

  [ExtConfig]
  /**
   * An optional string of one or more additional CSS classes to add to the label element. Defaults to empty.
   */
  public native function get labelClsExtra():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelClsExtra(value:String):void;

  [ExtConfig]
  /**
   * The string to use when joining the labels of individual sub-fields, when <a href="#!/api/Ext.form.FieldContainer-cfg-combineLabels" rel="Ext.form.FieldContainer-cfg-combineLabels" class="docClass">combineLabels</a> is
   * set to true. Defaults to ', '.
   * @default ', '
   */
  public native function get labelConnector():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelConnector(value:String):void;

  [ExtConfig]
  /**
   * The amount of space in pixels between the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and the field body.
   * This defaults to <code>5</code> for compatibility with Ext JS 4, however, as of Ext JS 5
   * the space between the label and the body can optionally be determined by the theme
   * using the $form-label-horizontal-spacing (for side-aligned labels) and
   * $form-label-vertical-spacing (for top-aligned labels) SASS variables.
   * In order for the stylesheet values as to take effect, you must use a labelPad value
   * of <code>null</code>.
   * @default 5
   */
  public native function get labelPad():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelPad(value:Number):void;

  [ExtConfig]
  /**
   * Character(s) to be inserted at the end of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">label text</a>.
   *
   * <p>Set to empty string to hide the separator completely.</p>
   * @default ':'
   */
  public native function get labelSeparator():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelSeparator(value:String):void;

  [ExtConfig]
  /**
   * A CSS style specification string to apply directly to this field's label.
   */
  public native function get labelStyle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelStyle(value:String):void;

  [ExtConfig]
  /**
   * The width of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> in pixels. Only applicable if <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a>
   * is set to "left" or "right".
   * @default 100
   */
  public native function get labelWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelWidth(value:Number):void;

  [ExtConfig]
  /**
   * The location where the error message text should display. Must be one of the following values:
   *
   * <ul>
   * <li><p><code>qtip</code> Display a quick tip containing the message when the user hovers over the field.
   * This is the default.</p>
   *
   * <p><strong><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a> must have been called for this setting to work.</strong></p></li>
   * <li><p><code>title</code> Display the message in a default browser title attribute popup.</p></li>
   * <li><code>under</code> Add a block div beneath the field containing the error message.</li>
   * <li><code>side</code> Add an error icon to the right of the field, displaying the message in a popup on hover.</li>
   * <li><code>none</code> Don't display any error message. This might be useful if you are implementing custom error display.</li>
   * <li><code>[element id]</code> Add the error message directly to the innerHTML of the specified element.</li>
   * </ul>
   * @default 'qtip'
   */
  public native function get msgTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set msgTarget(value:String):void;

  [ExtConfig]
  /**
   * true to disable displaying any <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">error message</a> set on this object.
   * @default false
   */
  public native function get preventMark():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventMark(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get invalidCls():String;

  /**
   * @inheritDoc
   */
  public native function set invalidCls(value:String):void;

  /**
   * Called when a <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> instance is added to the container's subtree.
   */
  override protected native function onAdd(component:ext.Component, position:Number):void;

  /**
   * Called when a <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> instance is removed from the container's subtree.
   */
  override protected native function onRemove(component:ext.Component, autoDestroy:Boolean):void;

  /**
   * Gets the active error message for this component, if any. This does not trigger validation on its own, it merely
   * returns any message that the component may already hold.
   * @return The active error message on the component; if there is no error, an empty string is returned.
   */
  public native function getActiveError():String;

  /**
   * Gets an Array of any active error messages currently applied to the field. This does not trigger validation on
   * its own, it merely returns any messages that the component may already hold.
   * @return The active error messages on the component; if there are no errors, an empty Array is
   * returned.
   */
  public native function getActiveErrors():Array;

  /**
   * Takes an Array of invalid <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> objects and builds a combined list of error
   * messages from them. Defaults to prepending each message by the field name and a colon. This
   * can be overridden to provide custom combined error message handling, for instance changing
   * the format of each message or sorting the array (it is sorted in order of appearance by default).
   * @param invalidFields An Array of the sub-fields which are currently invalid.
   * @return The combined list of error messages
   */
  public native function getCombinedErrors(invalidFields:Array):Array;

  /**
   * Returns the combined field label if <a href="#!/api/Ext.form.FieldContainer-cfg-combineLabels" rel="Ext.form.FieldContainer-cfg-combineLabels" class="docClass">combineLabels</a> is set to true and if there is no
   * set <a href="#!/api/Ext.form.FieldContainer-cfg-fieldLabel" rel="Ext.form.FieldContainer-cfg-fieldLabel" class="docClass">fieldLabel</a>. Otherwise returns the fieldLabel like normal. You can also override
   * this method to provide a custom generated label.
   * @return The label, or empty string if none.
   */
  public native function getFieldLabel():String;

  /**
   * Get the input id, if any, for this component. This is used as the "for" attribute on the label element.
   * Implementing subclasses may also use this as e.g. the id for their own input element.
   * @return The input id
   */
  public native function getInputId():String;

  /**
   * Tells whether the field currently has an active error message. This does not trigger validation on its own, it
   * merely looks for any message that the component may already hold.
   */
  public native function hasActiveError():Boolean;

  /**
   * Checks if the field has a visible label
   * @return True if the field has a visible label
   */
  public native function hasVisibleLabel():Boolean;

  /**
   * Performs initialization of this mixin. Component classes using this mixin should call this method during their
   * own initialization.
   */
  public native function initLabelable():void;

  /**
   * Sets the active error message to the given string. This replaces the entire error message contents with the given
   * string. Also see <a href="#!/api/Ext.form.Labelable-method-setActiveErrors" rel="Ext.form.Labelable-method-setActiveErrors" class="docClass">setActiveErrors</a> which accepts an Array of messages and formats them according to the
   * <a href="#!/api/Ext.form.Labelable-cfg-activeErrorsTpl" rel="Ext.form.Labelable-cfg-activeErrorsTpl" class="docClass">activeErrorsTpl</a>. Note that this only updates the error message element's text and attributes, you'll
   * have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> instead.
   * @param msg The error message
   */
  public native function setActiveError(msg:String):void;

  /**
   * Set the active error message to an Array of error messages. The messages are formatted into a single message
   * string using the <a href="#!/api/Ext.form.Labelable-cfg-activeErrorsTpl" rel="Ext.form.Labelable-cfg-activeErrorsTpl" class="docClass">activeErrorsTpl</a>. Also see <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a> which allows setting the entire error
   * contents with a single string. Note that this only updates the error message element's text and attributes,
   * you'll have to call doComponentLayout to actually update the field's layout to match. If the field extends
   * <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> instead.
   * @param errors The error messages
   */
  public native function setActiveErrors(errors:Array):void;

  /**
   * Applies a set of default configuration values to this Labelable instance. For each of the properties in the given
   * object, check if this component hasOwnProperty that config; if not then it's inheriting a default value from its
   * prototype and we should apply the default value.
   * @param defaults The defaults to apply to the object.
   */
  public native function setFieldDefaults(defaults:Object):void;

  /**
   * Set the label of this field.
   * @param label The new label. The <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a> will be automatically appended to the label
   * string.
   */
  public native function setFieldLabel(label:String):void;

  /**
   * Returns the trimmed label by slicing off the label separator character. Can be overridden.
   * @return The trimmed field label, or empty string if not defined
   */
  public native function trimLabelSeparator():String;

  /**
   * Clears the active error message(s). Note that this only clears the error message element's text and attributes,
   * you'll have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-clearInvalid" rel="Ext.form.field.Base-method-clearInvalid" class="docClass">clearInvalid</a> instead.
   */
  public native function unsetActiveError():void;

  /**
   * Generates the arguments for the field decorations <a href="#!/api/Ext.form.Labelable-cfg-labelableRenderTpl" rel="Ext.form.Labelable-cfg-labelableRenderTpl" class="docClass">rendering template</a>.
   * @param data optional object to use as the base data object.  If provided,
   * this method will add properties to the base object instead of creating a new one.
   * @return The template arguments
   */
  protected native function getLabelableRenderData(data:Object):Object;

  /**
   * Initializes the FieldAncestor's state; this must be called from the initComponent method of any components
   * importing this mixin.
   */
  protected native function initFieldAncestor():void;

  /**
   * Fired when the validity of any field within the container changes.
   * @param field The sub-field whose validity changed
   * @param valid The new validity state
   */
  protected native function onFieldValidityChange(field:ext.form.field.Field, valid:Boolean):void;
}
}