package ext.form {
import ext.Component;
import ext.container.Container;
import ext.form.field.Checkbox;
import ext.form.field.Field;
import ext.panel.Tool;

[Uses("ext.container.Container")]
[Uses("ext.form.FieldAncestor")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.form.FieldSet", require)]
/**
 * Fires before this FieldSet is collapsed. Return false to prevent the collapse.
 * @eventType ext.form.events.FieldSetEvent.BEFORE_COLLAPSE
 */
[Event(name = 'onBeforeCollapse', type = 'ext.form.events.FieldSetEvent')]
/**
 * Fires before this FieldSet is expanded. Return false to prevent the expand.
 * @eventType ext.form.events.FieldSetEvent.BEFORE_EXPAND
 */
[Event(name = 'onBeforeExpand', type = 'ext.form.events.FieldSetEvent')]
/**
 * Fires after this FieldSet has collapsed.
 * @eventType ext.form.events.FieldSetEvent.COLLAPSE
 */
[Event(name = 'onCollapse', type = 'ext.form.events.FieldSetEvent')]
/**
 * Fires after this FieldSet has expanded.
 * @eventType ext.form.events.FieldSetEvent.EXPAND
 */
[Event(name = 'onExpand', type = 'ext.form.events.FieldSetEvent')]
/**
 * Fires when the active error message is changed for any one of the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> instances
 * within this container.
 * @eventType ext.form.events.FieldSet_field_errorEvent.FIELD_ERROR_CHANGE
 */
[Event(name = 'onFieldErrorChange', type = 'ext.form.events.FieldSet_field_errorEvent')]
/**
 * Fires when the validity state of any one of the <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> instances within this
 * container changes.
 * @eventType ext.form.events.FieldSet_field_isValidEvent.FIELD_VALIDITY_CHANGE
 */
[Event(name = 'onFieldValidityChange', type = 'ext.form.events.FieldSet_field_isValidEvent')]
/**
 * A container for grouping sets of fields, rendered as a HTML <code>fieldset</code> element. The <a href="#!/api/Ext.form.FieldSet-cfg-title" rel="Ext.form.FieldSet-cfg-title" class="docClass">title</a>
 * config will be rendered as the fieldset's <code>legend</code>.
 *
 * <p>While FieldSets commonly contain simple groups of fields, they are general <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Containers</a>
 * and may therefore contain any type of components in their <a href="#!/api/Ext.form.FieldSet-cfg-items" rel="Ext.form.FieldSet-cfg-items" class="docClass">items</a>, including other nested containers.
 * The default <a href="#!/api/Ext.form.FieldSet-cfg-layout" rel="Ext.form.FieldSet-cfg-layout" class="docClass">layout</a> for the FieldSet's items is <code>'anchor'</code>, but it can be configured to use any other
 * layout type.</p>
 *
 * <p>FieldSets may also be collapsed if configured to do so; this can be done in two ways:</p>
 *
 * <ol>
 * <li>Set the <a href="#!/api/Ext.form.FieldSet-cfg-collapsible" rel="Ext.form.FieldSet-cfg-collapsible" class="docClass">collapsible</a> config to true; this will result in a collapse button being rendered next to
 * the <a href="#!/api/Ext.form.FieldSet-cfg-title" rel="Ext.form.FieldSet-cfg-title" class="docClass">legend title</a>, or:</li>
 * <li>Set the <a href="#!/api/Ext.form.FieldSet-cfg-checkboxToggle" rel="Ext.form.FieldSet-cfg-checkboxToggle" class="docClass">checkboxToggle</a> config to true; this is similar to using <a href="#!/api/Ext.form.FieldSet-cfg-collapsible" rel="Ext.form.FieldSet-cfg-collapsible" class="docClass">collapsible</a> but renders
 * a <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">checkbox</a> in place of the toggle button. The fieldset will be expanded when the
 * checkbox is checked and collapsed when it is unchecked. The checkbox will also be included in the
 * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit parameters</a> using <a href="#!/api/Ext.form.FieldSet-cfg-checkbox" rel="Ext.form.FieldSet-cfg-checkbox" class="docClass">checkbox</a>.</li>
 * </ol>
 *
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Simple Form with FieldSets',
 *     labelWidth: 75, // label settings here cascade unless overridden
 *     url: 'save-form.php',
 *     frame: true,
 *     bodyStyle: 'padding:5px 5px 0',
 *     width: 550,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     layout: 'column', // arrange fieldsets side by side
 *     items: [{
 *         // Fieldset in Column 1 - collapsible via toggle button
 *         xtype:'fieldset',
 *         columnWidth: 0.5,
 *         title: 'Fieldset 1',
 *         collapsible: true,
 *         defaultType: 'textfield',
 *         defaults: {anchor: '100%'},
 *         layout: 'anchor',
 *         items :[{
 *             fieldLabel: 'Field 1',
 *             name: 'field1'
 *         }, {
 *             fieldLabel: 'Field 2',
 *             name: 'field2'
 *         }]
 *     }, {
 *         // Fieldset in Column 2 - collapsible via checkbox, collapsed by default, contains a panel
 *         xtype:'fieldset',
 *         title: 'Show Panel', // title or checkboxToggle creates fieldset header
 *         columnWidth: 0.5,
 *         checkboxToggle: true,
 *         collapsed: true, // fieldset initially collapsed
 *         layout:'anchor',
 *         items :[{
 *             xtype: 'panel',
 *             anchor: '100%',
 *             title: 'Panel inside a fieldset',
 *             frame: true,
 *             height: 52
 *         }]
 *     }]
 * });
 * </code></pre>
 */
public class FieldSet extends ext.container.Container implements ext.form.IFieldAncestor {
  /**
   * @param config @inheritDoc
   */
  public function FieldSet(config:ext.form.FieldSet = null) {
    super();
  }

  [ExtConfig]
  /**
   * The base CSS class applied to the fieldset.
   * @default 'x-fieldset'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">Ext.container.Container.layout</a> for the fieldset's immediate child items.
   * @default 'anchor'
   * @see #setLayout()
   */
  override public native function get layout():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set layout(value:*):void;

  /**
   * Refers to the <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">Ext.form.field.Checkbox</a> component that is added next to the title in the legend. Only
   * populated if the fieldset is configured with <a href="#!/api/Ext.form.FieldSet-cfg-checkboxToggle" rel="Ext.form.FieldSet-cfg-checkboxToggle" class="docClass">checkboxToggle</a>:true.
   */
  public native function get checkboxCmp():ext.form.field.Checkbox;

  /**
   * @private
   */
  public native function set checkboxCmp(value:ext.form.field.Checkbox):void;

  /**
   * The component for the fieldset's legend. Will only be defined if the configuration requires a legend to be
   * created, by setting the <a href="#!/api/Ext.form.FieldSet-cfg-title" rel="Ext.form.FieldSet-cfg-title" class="docClass">title</a> or <a href="#!/api/Ext.form.FieldSet-cfg-checkboxToggle" rel="Ext.form.FieldSet-cfg-checkboxToggle" class="docClass">checkboxToggle</a> options.
   */
  public native function get legend():ext.Component;

  /**
   * @private
   */
  public native function set legend(value:ext.Component):void;

  /**
   * Refers to the <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">Ext.panel.Tool</a> component that is added as the collapse/expand button next to the title in
   * the legend. Only populated if the fieldset is configured with <a href="#!/api/Ext.form.FieldSet-cfg-collapsible" rel="Ext.form.FieldSet-cfg-collapsible" class="docClass">collapsible</a>:true.
   */
  public native function get toggleCmp():ext.panel.Tool;

  /**
   * @private
   */
  public native function set toggleCmp(value:ext.panel.Tool):void;

  [ExtConfig]
  /**
   * A configuration for the generated checkbox that is adjacent to the <a href="#!/api/Ext.form.FieldSet-cfg-title" rel="Ext.form.FieldSet-cfg-title" class="docClass">title</a> in the header.
   * This config is only effective when <a href="#!/api/Ext.form.FieldSet-cfg-checkboxToggle" rel="Ext.form.FieldSet-cfg-checkboxToggle" class="docClass">checkboxToggle</a> is true
   * @since 6.2.0
   */
  public native function get checkbox():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkbox(value:Object):void;

  [Deprecated(replacement = "checkbox", since = "6.2.0")]
  [ExtConfig]
  public native function get checkboxName():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkboxName(value:String):void;

  [ExtConfig]
  /**
   * Set to true to render a checkbox into the fieldset frame just in front of the legend to expand/collapse the
   * fieldset when the checkbox is toggled.. This checkbox will be included in form submits using
   * the <a href="#!/api/Ext.form.FieldSet-cfg-checkbox" rel="Ext.form.FieldSet-cfg-checkbox" class="docClass">checkbox</a> configuration.
   * @default false
   */
  public native function get checkboxToggle():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkboxToggle(value:Boolean):void;

  [ExtConfig]
  /**
   * The ui to use for the fieldset's checkbox.
   * @default 'default'
   */
  public native function get checkboxUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkboxUI(value:String):void;

  [ExtConfig]
  /**
   * Set to true to render the fieldset as collapsed by default. If <a href="#!/api/Ext.form.FieldSet-cfg-checkboxToggle" rel="Ext.form.FieldSet-cfg-checkboxToggle" class="docClass">checkboxToggle</a> is specified, the checkbox
   * will also be unchecked by default.
   * @default false
   * @see #setCollapsed()
   */
  public native function get collapsed():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set collapsed(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true to make the fieldset collapsible and have the expand/collapse toggle button automatically rendered
   * into the legend element, false to keep the fieldset statically sized with no collapse button.
   * Another option is to configure <a href="#!/api/Ext.form.FieldSet-cfg-checkboxToggle" rel="Ext.form.FieldSet-cfg-checkboxToggle" class="docClass">checkboxToggle</a>. Use the <a href="#!/api/Ext.form.FieldSet-cfg-collapsed" rel="Ext.form.FieldSet-cfg-collapsed" class="docClass">collapsed</a> config to collapse the
   * fieldset by default.
   * @default false
   */
  public native function get collapsible():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapsible(value:Boolean):void;

  [ExtConfig]
  /**
   * Fieldset description to be announced by screen readers.
   * @default '{0} field set'
   */
  public native function get descriptionText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set descriptionText(value:String):void;

  [ExtConfig]
  /**
   * Text to be announced by screen readers when toggle tool
   * or checkbox is focused.
   * @default 'Expand field set'
   */
  public native function get expandText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandText(value:String):void;

  [ExtConfig]
  /**
   * If specified, the properties in this object are used as default config values for each <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a>
   * instance (e.g. <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> or <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>) that is added as a descendant of
   * this container. Corresponding values specified in an individual field's own configuration, or from the <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults config</a> of its parent container, will take precedence. See the
   * documentation for <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> to see what config options may be specified in the fieldDefaults.
   *
   * <p>Example:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
   *     fieldDefaults: {
   *         labelAlign: 'left',
   *         labelWidth: 100
   *     },
   *     items: [{
   *         xtype: 'fieldset',
   *         defaults: {
   *             labelAlign: 'top'
   *         },
   *         items: [{
   *             name: 'field1'
   *         }, {
   *             name: 'field2'
   *         }]
   *     }, {
   *         xtype: 'fieldset',
   *         items: [{
   *             name: 'field3',
   *             labelWidth: 150
   *         }, {
   *             name: 'field4'
   *         }]
   *     }]
   * });
   * </code></pre>
   *
   * <p>In this example, field1 and field2 will get labelAlign:'top' (from the fieldset's defaults) and labelWidth:100
   * (from fieldDefaults), field3 and field4 will both get labelAlign:'left' (from fieldDefaults and field3 will use
   * the labelWidth:150 from its own config.</p>
   */
  public native function get fieldDefaults():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldDefaults(value:Object):void;

  [ExtConfig]
  /**
   * A title to be displayed in the fieldset's legend. May contain HTML markup.
   * @see #setTitle()
   */
  public native function get title():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set title(value:String):void;

  [ExtConfig]
  /**
   * Set to true will add a listener to the titleCmp property for the click event which will execute the
   * <a href="#!/api/Ext.form.FieldSet-method-toggle" rel="Ext.form.FieldSet-method-toggle" class="docClass">toggle</a> method. This option is only used when the <a href="#!/api/Ext.form.FieldSet-cfg-collapsible" rel="Ext.form.FieldSet-cfg-collapsible" class="docClass">collapsible</a> property is set to true.
   * @default true
   */
  public native function get toggleOnTitleClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set toggleOnTitleClick(value:Boolean):void;

  /**
   * Collapses the fieldset.
   * @return this
   */
  public native function collapse():ext.form.FieldSet;

  /**
   * Expands the fieldset.
   * @return this
   */
  public native function expand():ext.form.FieldSet;

  /**
   * Set the collapsed state of the fieldset.
   * @since 6.2.0
   * @param collapsed The collapsed state.
   */
  public native function setCollapsed(collapsed:Boolean):void;

  /**
   * Sets the title of this fieldset.
   * @param title The new title.
   * @return this
   */
  public native function setTitle(title:String):ext.form.FieldSet;

  /**
   * Toggle the fieldset's collapsed state to the opposite of what it is currently.
   */
  public native function toggle():void;

  /**
   * Creates the checkbox component. This is only called internally, but could be overridden in subclasses to
   * customize the checkbox's configuration or even return an entirely different component type.
   */
  protected native function createCheckboxCmp():ext.Component;

  /**
   * Creates the legend title component. This is only called internally, but could be overridden in subclasses to
   * customize the title component. If <a href="#!/api/Ext.form.FieldSet-cfg-toggleOnTitleClick" rel="Ext.form.FieldSet-cfg-toggleOnTitleClick" class="docClass">toggleOnTitleClick</a> is set to true, a listener for the click event
   * will toggle the collapsed state of the FieldSet.
   */
  protected native function createTitleCmp():ext.Component;

  /**
   * Creates the toggle button component. This is only called internally, but could be overridden in subclasses to
   * customize the toggle component.
   */
  protected native function createToggleCmp():ext.Component;

  /**
   * Initializes the FieldAncestor's state; this must be called from the initComponent method of any components
   * importing this mixin.
   */
  protected native function initFieldAncestor():void;

  /**
   * Fired when the error message of any field within the container changes.
   * @param field The sub-field whose active error changed
   * @param error The new active error message
   */
  protected native function onFieldErrorChange(field:ext.form.Labelable, error:String):void;

  /**
   * Fired when the validity of any field within the container changes.
   * @param field The sub-field whose validity changed
   * @param valid The new validity state
   */
  protected native function onFieldValidityChange(field:ext.form.field.Field, valid:Boolean):void;
}
}