package ext.form {
import ext.Mixin;
import ext.dom.Element;

[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Native("Ext.form.Labelable", require)]
[ExtConfig]
/**
 * Fires when the active error message is changed via <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a>.
 * @eventType ext.form.events.Labelable_errorEvent.ERROR_CHANGE
 */
[Event(name = 'onErrorChange', type = 'ext.form.events.Labelable_errorEvent')]
/**
 * A mixin which allows a component to be configured and decorated with a label and/or error message as is
 * common for form fields. This is used by e.g. <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> and <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>
 * to let them be managed by the Field layout.
 *
 * <p>NOTE: This mixin is mainly for internal library use and most users should not need to use it directly. It
 * is more likely you will want to use one of the component classes that import this mixin, such as
 * <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> or <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>.</p>
 *
 * <p>Use of this mixin does not make a component a field in the logical sense, meaning it does not provide any
 * logic or state related to values or validation; that is handled by the related <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a>
 * mixin. These two mixins may be used separately (for example <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a> is Labelable but not a
 * Field), or in combination (for example <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> implements both and has logic for connecting the
 * two.)</p>
 *
 * <p>Component classes which use this mixin should use the Field layout
 * or a derivation thereof to properly size and position the label and message according to the component config.
 * They must also call the <a href="#!/api/Ext.form.Labelable-method-initLabelable" rel="Ext.form.Labelable-method-initLabelable" class="docClass">initLabelable</a> method during component initialization to ensure the mixin gets
 * set up correctly.</p>
 */
public class Labelable extends ext.Mixin implements ext.form.ILabelable {
  /**
   * @param config @inheritDoc
   */
  public function Labelable(config:ext.form.Labelable = null) {
    super();
  }

  /**
   * The div Element wrapping the component's contents. Only available after the component has been rendered.
   */
  public native function get bodyEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set bodyEl(value:ext.dom.Element):void;

  /**
   * The div Element that will contain the component's error message(s). Note that depending on the configured
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">msgTarget</a>, this element may be hidden in favor of some other form of presentation, but will always
   * be present in the DOM for use by assistive technologies.
   */
  public native function get errorEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set errorEl(value:ext.dom.Element):void;

  /**
   * Flag denoting that this object is labelable as a field. Always true.
   * @default true
   */
  public native function get isFieldLabelable():Boolean;

  /**
   * @private
   */
  public native function set isFieldLabelable(value:Boolean):void;

  /**
   * The label Element for this component. Only available after the component has been rendered.
   */
  public native function get labelEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set labelEl(value:ext.dom.Element):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If specified, then the component will be displayed with this value as its active error when first rendered. Use
   * <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a> or <a href="#!/api/Ext.form.Labelable-method-unsetActiveError" rel="Ext.form.Labelable-method-unsetActiveError" class="docClass">unsetActiveError</a> to change it after component creation.
   * @see #getActiveError()
   * @see #setActiveError()
   */
  public native function get activeError():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activeError(value:String):void;

  [ExtConfig]
  /**
   * The template used to format the Array of error messages passed to <a href="#!/api/Ext.form.Labelable-method-setActiveErrors" rel="Ext.form.Labelable-method-setActiveErrors" class="docClass">setActiveErrors</a> into a single HTML
   * string. if the <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">msgTarget</a> is title, it defaults to a list separated by new lines. Otherwise, it
   * renders each message as an item in an unordered list.
   */
  public native function get activeErrorsTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeErrorsTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * at the end of the input containing element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterBodyEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterBodyEl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the label text. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the label element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the <a href="#!/api/Ext.form.Labelable-method-getSubTplMarkup" rel="Ext.form.Labelable-method-getSubTplMarkup" class="docClass">subTpl markup</a>. If an <code>XTemplate</code> is used, the
   * component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get afterSubTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterSubTpl(value:*):void;

  [ExtConfig]
  /**
   * Localized announcement text for validation errors. This text
   * will be used by Assistive Technologies such as screen readers to alert the users when
   * field validation fails.
   *
   * <p>This config is used with Ext.String.format. '{0}' will be replaced with the actual
   * error message(s), '{1}' will be replaced with field label.</p>
   * @default 'Input error. {0}.'
   */
  public native function get ariaErrorText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaErrorText(value:String):void;

  [ExtConfig]
  /**
   * Optional text description for this object. This text will be
   * announced to Assistive Technology users when the object is focused.
   */
  public native function get ariaHelp():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaHelp(value:String):void;

  [ExtConfig]
  /**
   * Whether to adjust the component's body width to make room for 'side'
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">error messages</a>.
   * @default true
   */
  public native function get autoFitErrors():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoFitErrors(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the body content element.
   * @default Ext.baseCSSPrefix + 'form-item-body'
   */
  public native function get baseBodyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseBodyCls(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * at the beginning of the input containing element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeBodyEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeBodyEl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the label text. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the label element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the <a href="#!/api/Ext.form.Labelable-method-getSubTplMarkup" rel="Ext.form.Labelable-method-getSubTplMarkup" class="docClass">subTpl markup</a>. If an <code>XTemplate</code> is used, the
   * component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get beforeSubTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeSubTpl(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the error message element.
   * @default Ext.baseCSSPrefix + 'form-error-msg'
   */
  public native function get errorMsgCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set errorMsgCls(value:String):void;

  [ExtConfig]
  /**
   * An extra CSS class to be applied to the body content element in addition to <a href="#!/api/Ext.form.Labelable-cfg-baseBodyCls" rel="Ext.form.Labelable-cfg-baseBodyCls" class="docClass">baseBodyCls</a>.
   * @default ''
   */
  public native function get fieldBodyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldBodyCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The label for the field. It gets appended with the <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>, and its position and sizing is
   * determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> configs.
   * @see #getFieldLabel()
   * @see #setFieldLabel()
   */
  public native function get fieldLabel():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fieldLabel(value:String):void;

  [ExtConfig]
  /**
   * A CSS class to be applied to the outermost element to denote that it is participating in the form field layout.
   * @default Ext.baseCSSPrefix + 'form-item'
   */
  public native function get formItemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formItemCls(value:String):void;

  [ExtConfig]
  /**
   * When set to true, the label element (<a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>) will be automatically
   * hidden if the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> is empty. Setting this to false will cause the empty label element to be
   * rendered and space to be reserved for it; this is useful if you want a field without a label to line up with
   * other labeled fields in the same form.
   *
   * <p>If you wish to unconditionall hide the label even if a non-empty fieldLabel is configured, then set the
   * <a href="#!/api/Ext.form.Labelable-cfg-hideLabel" rel="Ext.form.Labelable-cfg-hideLabel" class="docClass">hideLabel</a> config to true.</p>
   * @default true
   */
  public native function get hideEmptyLabel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideEmptyLabel(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true to completely hide the label element (<a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>). Also see
   * <a href="#!/api/Ext.form.Labelable-cfg-hideEmptyLabel" rel="Ext.form.Labelable-cfg-hideEmptyLabel" class="docClass">hideEmptyLabel</a>, which controls whether space will be reserved for an empty fieldLabel.
   * @default false
   */
  public native function get hideLabel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideLabel(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to use when marking the component invalid.
   * @default Ext.baseCSSPrefix + 'form-invalid'
   */
  public native function get invalidCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set invalidCls(value:String):void;

  [ExtConfig]
  /**
   * Controls the position and alignment of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a>. Valid values are:
   *
   * <ul>
   * <li>"left" (the default) - The label is positioned to the left of the field, with its text aligned to the left.
   * Its width is determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> config.</li>
   * <li>"top" - The label is positioned above the field.</li>
   * <li>"right" - The label is positioned to the left of the field, with its text aligned to the right.
   * Its width is determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> config.</li>
   * </ul>
   * @default 'left'
   */
  public native function get labelAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelAlign(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * inside the label element (as attributes). If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get labelAttrTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelAttrTpl(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the label element. This (single) CSS class is used to formulate the renderSelector
   * and drives the field layout where it is concatenated with a hyphen ('-') and <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a>. To add
   * additional classes, use <a href="#!/api/Ext.form.Labelable-cfg-labelClsExtra" rel="Ext.form.Labelable-cfg-labelClsExtra" class="docClass">labelClsExtra</a>.
   * @default Ext.baseCSSPrefix + 'form-item-label'
   */
  public native function get labelCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelCls(value:String):void;

  [ExtConfig]
  /**
   * An optional string of one or more additional CSS classes to add to the label element. Defaults to empty.
   */
  public native function get labelClsExtra():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelClsExtra(value:String):void;

  [ExtConfig]
  /**
   * The amount of space in pixels between the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and the field body.
   * This defaults to <code>5</code> for compatibility with Ext JS 4, however, as of Ext JS 5
   * the space between the label and the body can optionally be determined by the theme
   * using the $form-label-horizontal-spacing (for side-aligned labels) and
   * $form-label-vertical-spacing (for top-aligned labels) SASS variables.
   * In order for the stylesheet values as to take effect, you must use a labelPad value
   * of <code>null</code>.
   * @default 5
   */
  public native function get labelPad():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelPad(value:Number):void;

  [ExtConfig]
  /**
   * Character(s) to be inserted at the end of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">label text</a>.
   *
   * <p>Set to empty string to hide the separator completely.</p>
   * @default ':'
   */
  public native function get labelSeparator():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelSeparator(value:String):void;

  [ExtConfig]
  /**
   * A CSS style specification string to apply directly to this field's label.
   */
  public native function get labelStyle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelStyle(value:String):void;

  [ExtConfig]
  /**
   * The width of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> in pixels. Only applicable if <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a>
   * is set to "left" or "right".
   * @default 100
   */
  public native function get labelWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelWidth(value:Number):void;

  [ExtConfig]
  /**
   * The location where the error message text should display. Must be one of the following values:
   *
   * <ul>
   * <li><p><code>qtip</code> Display a quick tip containing the message when the user hovers over the field.
   * This is the default.</p>
   *
   * <p><strong><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a> must have been called for this setting to work.</strong></p></li>
   * <li><p><code>title</code> Display the message in a default browser title attribute popup.</p></li>
   * <li><code>under</code> Add a block div beneath the field containing the error message.</li>
   * <li><code>side</code> Add an error icon to the right of the field, displaying the message in a popup on hover.</li>
   * <li><code>none</code> Don't display any error message. This might be useful if you are implementing custom error display.</li>
   * <li><code>[element id]</code> Add the error message directly to the innerHTML of the specified element.</li>
   * </ul>
   * @default 'qtip'
   */
  public native function get msgTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set msgTarget(value:String):void;

  [ExtConfig]
  /**
   * true to disable displaying any <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">error message</a> set on this object.
   * @default false
   */
  public native function get preventMark():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventMark(value:Boolean):void;

  /**
   * Gets the active error message for this component, if any. This does not trigger validation on its own, it merely
   * returns any message that the component may already hold.
   * @return The active error message on the component; if there is no error, an empty string is returned.
   */
  public native function getActiveError():String;

  /**
   * Gets an Array of any active error messages currently applied to the field. This does not trigger validation on
   * its own, it merely returns any messages that the component may already hold.
   * @return The active error messages on the component; if there are no errors, an empty Array is
   * returned.
   */
  public native function getActiveErrors():Array;

  /**
   * Returns the label for the field. Defaults to simply returning the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> config. Can be overridden
   * to provide a custom generated label.
   * @return The configured field label, or empty string if not defined
   */
  public native function getFieldLabel():String;

  /**
   * Get the input id, if any, for this component. This is used as the "for" attribute on the label element.
   * Implementing subclasses may also use this as e.g. the id for their own input element.
   * @return The input id
   */
  public native function getInputId():String;

  /**
   * Tells whether the field currently has an active error message. This does not trigger validation on its own, it
   * merely looks for any message that the component may already hold.
   */
  public native function hasActiveError():Boolean;

  /**
   * Checks if the field has a visible label
   * @return True if the field has a visible label
   */
  public native function hasVisibleLabel():Boolean;

  /**
   * Performs initialization of this mixin. Component classes using this mixin should call this method during their
   * own initialization.
   */
  public native function initLabelable():void;

  /**
   * Sets the active error message to the given string. This replaces the entire error message contents with the given
   * string. Also see <a href="#!/api/Ext.form.Labelable-method-setActiveErrors" rel="Ext.form.Labelable-method-setActiveErrors" class="docClass">setActiveErrors</a> which accepts an Array of messages and formats them according to the
   * <a href="#!/api/Ext.form.Labelable-cfg-activeErrorsTpl" rel="Ext.form.Labelable-cfg-activeErrorsTpl" class="docClass">activeErrorsTpl</a>. Note that this only updates the error message element's text and attributes, you'll
   * have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> instead.
   * @param msg The error message
   */
  public native function setActiveError(msg:String):void;

  /**
   * Set the active error message to an Array of error messages. The messages are formatted into a single message
   * string using the <a href="#!/api/Ext.form.Labelable-cfg-activeErrorsTpl" rel="Ext.form.Labelable-cfg-activeErrorsTpl" class="docClass">activeErrorsTpl</a>. Also see <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a> which allows setting the entire error
   * contents with a single string. Note that this only updates the error message element's text and attributes,
   * you'll have to call doComponentLayout to actually update the field's layout to match. If the field extends
   * <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> instead.
   * @param errors The error messages
   */
  public native function setActiveErrors(errors:Array):void;

  /**
   * Applies a set of default configuration values to this Labelable instance. For each of the properties in the given
   * object, check if this component hasOwnProperty that config; if not then it's inheriting a default value from its
   * prototype and we should apply the default value.
   * @param defaults The defaults to apply to the object.
   */
  public native function setFieldDefaults(defaults:Object):void;

  /**
   * Set the label of this field.
   * @param label The new label. The <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a> will be automatically appended to the label
   * string.
   */
  public native function setFieldLabel(label:String):void;

  /**
   * Returns the trimmed label by slicing off the label separator character. Can be overridden.
   * @return The trimmed field label, or empty string if not defined
   */
  public native function trimLabelSeparator():String;

  /**
   * Clears the active error message(s). Note that this only clears the error message element's text and attributes,
   * you'll have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-clearInvalid" rel="Ext.form.field.Base-method-clearInvalid" class="docClass">clearInvalid</a> instead.
   */
  public native function unsetActiveError():void;

  /**
   * Generates the arguments for the field decorations <a href="#!/api/Ext.form.Labelable-cfg-labelableRenderTpl" rel="Ext.form.Labelable-cfg-labelableRenderTpl" class="docClass">rendering template</a>.
   * @param data optional object to use as the base data object.  If provided,
   * this method will add properties to the base object instead of creating a new one.
   * @return The template arguments
   */
  protected native function getLabelableRenderData(data:Object):Object;

  /**
   * Gets the markup to be inserted into the outer template's bodyEl. Defaults to empty string, should be implemented
   * by classes including this mixin as needed.
   * @return The markup to be inserted
   */
  protected native function getSubTplMarkup():String;
}
}