package ext.form.action {

[Uses("ext.form.action.SubmitAction")]
[Uses("ext.form.action.DirectAction")]
[Uses("ext.form.action.Action")]
[Uses("ext.Base")]
[Uses("ext.Mixin")]
[Native("Ext.form.action.DirectSubmit", require)]
/**
 * Provides Ext Direct support for submitting form data.
 *
 * <p>This example illustrates usage of Ext Direct to submit a form.</p>
 *
 * <pre><code>var myFormPanel = new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
 *     // configs for FormPanel
 *     title: 'Basic Information',
 *     renderTo: document.body,
 *     width: 300, height: 160,
 *     padding: 10,
 *     buttons:[{
 *         text: 'Submit',
 *         handler: function(){
 *             myFormPanel.getForm().submit({
 *                 params: {
 *                     foo: 'bar',
 *                     uid: 34
 *                 }
 *             });
 *         }
 *     }],
 *
 *     // configs apply to child items
 *     defaults: {anchor: '100%'},
 *     defaultType: 'textfield',
 *     items: [{
 *         fieldLabel: 'Name',
 *         name: 'name'
 *     },{
 *         fieldLabel: 'Email',
 *         name: 'email'
 *     },{
 *         fieldLabel: 'Company',
 *         name: 'company'
 *     }],
 *
 *     // configs for BasicForm
 *     api: {
 *         // The server-side method to call for load() requests
 *         load: 'Profile.getBasicInfo',
 *         // The server-side must mark the submit handler as a 'formHandler'
 *         submit: 'Profile.updateBasicInfo'
 *     },
 *     // specify the order for the passed params
 *     paramOrder: ['uid', 'foo']
 * });
 * </code></pre>
 *
 * <p>Before using DirectLoad action, make sure you set up Ext Direct remoting provider.
 * See <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> for more information.</p>
 *
 * <p>For corresponding load action, see <a href="#!/api/Ext.form.action.DirectLoad" rel="Ext.form.action.DirectLoad" class="docClass">Ext.form.action.DirectLoad</a>.</p>
 */
public class DirectSubmitAction extends ext.form.action.SubmitAction implements ext.form.action.IDirectAction {
  /**
   * @param config @inheritDoc
   */
  public function DirectSubmitAction(config:ext.form.action.DirectSubmitAction = null) {
    super();
  }
}
}