package ext.form.field {

[Uses("ext.form.field.Picker")]
[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Date", require)]
/**
 * Provides a date input field with a <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">date picker</a> dropdown and automatic date
 * validation.
 *
 * <p>This field recognizes and uses the JavaScript Date object as its main <a href="#!/api/Ext.form.field.Date-cfg-value" rel="Ext.form.field.Date-cfg-value" class="docClass">value</a> type. In addition,
 * it recognizes string values which are parsed according to the <a href="#!/api/Ext.form.field.Date-cfg-format" rel="Ext.form.field.Date-cfg-format" class="docClass">format</a> and/or <a href="#!/api/Ext.form.field.Date-cfg-altFormats" rel="Ext.form.field.Date-cfg-altFormats" class="docClass">altFormats</a>
 * configs. These may be reconfigured to use date formats appropriate for the user's locale.</p>
 *
 * <p>The field may be limited to a certain range of dates by using the <a href="#!/api/Ext.form.field.Date-cfg-minValue" rel="Ext.form.field.Date-cfg-minValue" class="docClass">minValue</a>, <a href="#!/api/Ext.form.field.Date-cfg-maxValue" rel="Ext.form.field.Date-cfg-maxValue" class="docClass">maxValue</a>,
 * <a href="#!/api/Ext.form.field.Date-cfg-disabledDays" rel="Ext.form.field.Date-cfg-disabledDays" class="docClass">disabledDays</a>, and <a href="#!/api/Ext.form.field.Date-cfg-disabledDates" rel="Ext.form.field.Date-cfg-disabledDates" class="docClass">disabledDates</a> config parameters. These configurations will be used both
 * in the field's validation, and in the date picker dropdown by preventing invalid dates from being selected.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     width: 300,
 *     bodyPadding: 10,
 *     title: 'Dates',
 *     items: [{
 *         xtype: 'datefield',
 *         anchor: '100%',
 *         fieldLabel: 'From',
 *         name: 'from_date',
 *         maxValue: new Date()  // limited to the current date or prior
 *     }, {
 *         xtype: 'datefield',
 *         anchor: '100%',
 *         fieldLabel: 'To',
 *         name: 'to_date',
 *         value: new Date()  // defaults to today
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Date Formats Examples</h1>
 *
 * <p>This example shows a couple of different date format parsing scenarios. Both use custom date format
 * configurations; the first one matches the configured <code>format</code> while the second matches the <code>altFormats</code>.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     width: 300,
 *     bodyPadding: 10,
 *     title: 'Dates',
 *     items: [{
 *         xtype: 'datefield',
 *         anchor: '100%',
 *         fieldLabel: 'Date',
 *         name: 'date',
 *         // The value matches the format; will be parsed and displayed using that format.
 *         format: 'm d Y',
 *         value: '02 04 1978'
 *     }, {
 *         xtype: 'datefield',
 *         anchor: '100%',
 *         fieldLabel: 'Date',
 *         name: 'date',
 *         // The value does not match the format, but does match an altFormat; will be parsed
 *         // using the altFormat and displayed using the format.
 *         format: 'm d Y',
 *         altFormats: 'm,d,Y|m.d.Y',
 *         value: '02.04.1978'
 *     }]
 * });
 * </code></pre>
 */
public class DateField extends ext.form.field.Picker {
  /**
   * @param config @inheritDoc
   */
  public function DateField(config:ext.form.field.DateField = null) {
    super();
  }

  [ExtConfig]
  /**
   * The format text to be announced by screen readers
   * when the field is focused.
   * @default 'Expected date format {0}.'
   */
  override public native function get formatText():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set formatText(value:String):void;

  [ExtConfig]
  /**
   * The error text to display when the date in the field is invalid.
   * @default "{0} is not a valid date - it must be in the format {1}"
   */
  override public native function get invalidText():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set invalidText(value:String):void;

  [ExtConfig]
  /**
   * An additional CSS class used to style the trigger button. The trigger will always get the class 'x-form-trigger'
   * and triggerCls will be <strong>appended</strong> if specified (default class displays a calendar icon).
   * @default 'x-form-date-trigger'
   */
  override public native function get triggerCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set triggerCls(value:String):void;

  [ExtConfig]
  /**
   * Multiple date formats separated by "|" to try when parsing a user input value and it does not match the defined
   * format.
   * @default "m/d/Y|n/j/Y|n/j/y|m/j/y|n/d/y|m/j/Y|n/d/Y|m-d-y|m-d-Y|m/d|m-d|md|mdy|mdY|d|Y-m-d|n-j|n/j"
   */
  public native function get altFormats():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set altFormats(value:String):void;

  [ExtConfig]
  /**
   * The text that Assistive Technologies such as screen readers
   * will announce when the date falls on a disabled date.
   * @default "This date cannot be selected"
   */
  public native function get ariaDisabledDatesText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDisabledDatesText(value:String):void;

  [ExtConfig]
  /**
   * The text that Assistive Technologies such as screen readers
   * will announce when the date falls on a disabled day of week.
   * @default "This day of week is disabled"
   */
  public native function get ariaDisabledDaysText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDisabledDaysText(value:String):void;

  [ExtConfig]
  /**
   * This date format will be used to format ARIA attributes in the field and its Picker,
   * to provide Assistive Technologies such as screen readers with user friendly text.
   * The format must be valid <a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a>.
   * @default "M j Y"
   */
  public native function get ariaFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaFormat(value:String):void;

  [ExtConfig]
  /**
   * The text that Assistive Technologies such as screen readers
   * will announce when the date in the cell is after <a href="#!/api/Ext.form.field.Date-cfg-maxValue" rel="Ext.form.field.Date-cfg-maxValue" class="docClass">maxValue</a>. The date substituted
   * for {0} will be formatted as per <a href="#!/api/Ext.form.field.Date-cfg-ariaFormat" rel="Ext.form.field.Date-cfg-ariaFormat" class="docClass">ariaFormat</a>.
   * @default "The date must be equal to or before {0}"
   */
  public native function get ariaMaxText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaMaxText(value:String):void;

  [ExtConfig]
  /**
   * The text that Assistive Technologies such as screen readers
   * will announce when the date in the cell is before <a href="#!/api/Ext.form.field.Date-cfg-minValue" rel="Ext.form.field.Date-cfg-minValue" class="docClass">minValue</a>. The date substituted
   * for {0} will be formatted as per <a href="#!/api/Ext.form.field.Date-cfg-ariaFormat" rel="Ext.form.field.Date-cfg-ariaFormat" class="docClass">ariaFormat</a>.
   * @default "The date must be equal to or after {0}"
   */
  public native function get ariaMinText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaMinText(value:String):void;

  [ExtConfig]
  /**
   * An array of "dates" to disable, as strings. These strings will be used to build a dynamic regular expression so
   * they are very powerful. Some examples:
   *
   * <pre><code>// disable these exact dates:
   * disabledDates: ["03/08/2003", "09/16/2003"]
   * // disable these days for every year:
   * disabledDates: ["03/08", "09/16"]
   * // only match the beginning (useful if you are using short years):
   * disabledDates: ["^03/08"]
   * // disable every day in March 2006:
   * disabledDates: ["03/../2006"]
   * // disable every day in every March:
   * disabledDates: ["^03"]
   * </code></pre>
   *
   * <p>Note that the format of the dates included in the array should exactly match the <a href="#!/api/Ext.form.field.Date-cfg-format" rel="Ext.form.field.Date-cfg-format" class="docClass">format</a> config. In order
   * to support regular expressions, if you are using a <a href="#!/api/Ext.form.field.Date-cfg-format" rel="Ext.form.field.Date-cfg-format" class="docClass">date format</a> that has "." in it, you will have
   * to escape the dot when restricting dates. For example: <code>["03\\.08\\.03"]</code>.</p>
   * @see #setDisabledDates()
   */
  public native function get disabledDates():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabledDates(value:Array):void;

  [ExtConfig]
  /**
   * The tooltip text to display when the date falls on a disabled date.
   * @default "Disabled"
   */
  public native function get disabledDatesText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledDatesText(value:String):void;

  [ExtConfig]
  /**
   * An array of days to disable, 0 based. Some examples:
   *
   * <pre><code>// disable Sunday and Saturday:
   * disabledDays:  [0, 6]
   * // disable weekdays:
   * disabledDays: [1,2,3,4,5]
   * </code></pre>
   * @see #setDisabledDays()
   */
  public native function get disabledDays():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabledDays(value:Array):void;

  [ExtConfig]
  /**
   * The tooltip to display when the date falls on a disabled day of week.
   * @default "Disabled"
   */
  public native function get disabledDaysText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledDaysText(value:String):void;

  [ExtConfig]
  /**
   * The default date format string which can be overriden for localization support. The format must be valid
   * according to <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   * @default "m/d/Y"
   */
  public native function get format():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set format(value:String):void;

  [ExtConfig]
  /**
   * The error text to display when the date in the cell is after <a href="#!/api/Ext.form.field.Date-cfg-maxValue" rel="Ext.form.field.Date-cfg-maxValue" class="docClass">maxValue</a>.
   * @default "The date in this field must be equal to or before {0}"
   */
  public native function get maxText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxText(value:String):void;

  [ExtConfig]
  /**
   * The maximum allowed date. Can be either a Javascript date object or a string date in a valid format.
   * @see #setMaxValue()
   */
  public native function get maxValue():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxValue(value:*):void;

  [ExtConfig]
  /**
   * The error text to display when the date in the cell is before <a href="#!/api/Ext.form.field.Date-cfg-minValue" rel="Ext.form.field.Date-cfg-minValue" class="docClass">minValue</a>.
   * @default "The date in this field must be equal to or after {0}"
   */
  public native function get minText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minText(value:String):void;

  [ExtConfig]
  /**
   * The minimum allowed date. Can be either a Javascript date object or a string date in a valid format.
   * @see #setMinValue()
   */
  public native function get minValue():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minValue(value:*):void;

  [ExtConfig]
  /**
   * false to hide the footer area of the Date picker containing the Today button and disable the keyboard handler for
   * spacebar that selects the current date.
   * @default true
   */
  public native function get showToday():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showToday(value:Boolean):void;

  [ExtConfig]
  /**
   * Day index at which the week should begin, 0-based.
   *
   * <p>Defaults to <code>0</code> (Sunday).</p>
   * @default undefined
   */
  public native function get startDay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set startDay(value:Number):void;

  [ExtConfig]
  /**
   * The date format string which will be submitted to the server. The format must be valid according to
   * <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   *
   * <p>Defaults to <a href="#!/api/Ext.form.field.Date-cfg-format" rel="Ext.form.field.Date-cfg-format" class="docClass">format</a>.</p>
   */
  public native function get submitFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitFormat(value:String):void;

  [ExtConfig]
  /**
   * True to enforce strict date parsing to prevent the default Javascript "date rollover".
   * Defaults to the useStrict parameter set on <a href="#!/api/Ext.Date" rel="Ext.Date" class="docClass">Ext.Date</a>
   * See <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   */
  public native function get useStrict():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useStrict(value:Boolean):void;

  /**
   * Runs all of Date's validations and returns an array of any errors. Note that this first runs Text's validations,
   * so the returned array is an amalgamation of all field errors. The additional validation checks are testing that
   * the date format is valid, that the chosen date is within the min and max date constraints set, that the date
   * chosen is not in the disabledDates regex and that the day chosed is not one of the disabledDays.
   * @param value The value to get errors for (defaults to the current field value)
   * @return All validation errors for this field
   */
  override public native function getErrors(value:*):Array;

  /**
   * Returns the current data value of the field. The type of value returned is particular to the type of the
   * particular field (e.g. a Date object for <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a>), as the result of calling <a href="#!/api/Ext.form.field.Date-method-rawToValue" rel="Ext.form.field.Date-method-rawToValue" class="docClass">rawToValue</a> on
   * the field's <a href="#!/api/Ext.form.field.Date-method-processRawValue" rel="Ext.form.field.Date-method-processRawValue" class="docClass">processed</a> String value. To return the raw String value, see <a href="#!/api/Ext.form.field.Date-method-getRawValue" rel="Ext.form.field.Date-method-getRawValue" class="docClass">getRawValue</a>.
   * @return value The field value
   */
  override public native function getValue():*;

  /**
   * Sets the value of the date field. You can pass a date object or any string that can be parsed into a valid date,
   * using <a href="#!/api/Ext.form.field.Date-cfg-format" rel="Ext.form.field.Date-cfg-format" class="docClass">format</a> as the date format, according to the same rules as <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a> (the default
   * format used is "m/d/Y").
   *
   * <p>Usage:</p>
   *
   * <pre><code>//All of these calls set the same date value (May 4, 2006)
   *
   * //Pass a date object:
   * var dt = new Date('5/4/2006');
   * dateField.me = this,
   * setValue(dt);
   *
   * //Pass a date string (default format):
   * dateField.setValue('05/04/2006');
   *
   * //Pass a date string (custom format):
   * dateField.format = 'Y-m-d';
   * dateField.setValue('2006-05-04');
   * </code></pre>
   * @param value The value to set
   * @return this
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * Attempts to parse a given string value using a given <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">date format</a>.
   * @param value The value to attempt to parse
   * @param format A valid date format (see <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>)
   * @return The parsed Date object, or null if the value could not be successfully parsed.
   */
  public native function safeParse(value:String, format:String):Date;

  /**
   * Replaces any existing disabled dates with new values and refreshes the Date picker.
   * @param disabledDates An array of date strings (see the <a href="#!/api/Ext.form.field.Date-cfg-disabledDates" rel="Ext.form.field.Date-cfg-disabledDates" class="docClass">disabledDates</a> config for details on
   * supported values) used to disable a pattern of dates.
   */
  public native function setDisabledDates(disabledDates:Array):void;

  /**
   * Replaces any existing disabled days (by index, 0-6) with new values and refreshes the Date picker.
   * @param disabledDays An array of disabled day indexes. See the <a href="#!/api/Ext.form.field.Date-cfg-disabledDays" rel="Ext.form.field.Date-cfg-disabledDays" class="docClass">disabledDays</a> config for details on
   * supported values.
   */
  public native function setDisabledDays(disabledDays:Array):void;

  /**
   * Replaces any existing <a href="#!/api/Ext.form.field.Date-cfg-maxValue" rel="Ext.form.field.Date-cfg-maxValue" class="docClass">maxValue</a> with the new value and refreshes the Date picker.
   * @param value The maximum date that can be selected
   */
  public native function setMaxValue(value:Date):void;

  /**
   * Replaces any existing <a href="#!/api/Ext.form.field.Date-cfg-minValue" rel="Ext.form.field.Date-cfg-minValue" class="docClass">minValue</a> with the new value and refreshes the Date picker.
   * @param value The minimum date that can be selected
   */
  public native function setMinValue(value:Date):void;
}
}