package ext.form.field {

[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Display", require)]
/**
 * A display-only text field which is not validated and not submitted. This is useful for when you want to display a
 * value from a form's <a href="#!/api/Ext.form.Basic-method-load" rel="Ext.form.Basic-method-load" class="docClass">loaded data</a> but do not want to allow the user to edit or submit that
 * value. The value can be optionally <a href="#!/api/Ext.form.field.Display-cfg-htmlEncode" rel="Ext.form.field.Display-cfg-htmlEncode" class="docClass">HTML encoded</a> if it contains HTML markup that you do not want
 * to be rendered.
 *
 * <p>If you have more complex content, or need to include components within the displayed content, also consider using a
 * <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a> instead.</p>
 *
 * <p>Example:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     width: 175,
 *     height: 150,
 *     bodyPadding: 10,
 *     title: 'Final Score',
 *     items: [{
 *         xtype: 'displayfield',
 *         fieldLabel: 'Home',
 *         name: 'home_score',
 *         value: '10'
 *     }, {
 *         xtype: 'displayfield',
 *         fieldLabel: 'Visitor',
 *         name: 'visitor_score',
 *         value: '11'
 *     }],
 *     buttons: [{
 *         text: 'Update'
 *     }]
 * });
 * </code></pre>
 */
public class DisplayField extends ext.form.field.BaseField {
  /**
   * @param config @inheritDoc
   */
  public function DisplayField(config:ext.form.field.DisplayField = null) {
    super();
  }

  [ExtConfig]
  /**
   * The default CSS class for the field.
   * @default "x-form-display-field"
   */
  override public native function get fieldCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set fieldCls(value:String):void;

  [ExtConfig]
  /**
   * True to escape HTML in text when rendering it.
   * @default false
   */
  public native function get htmlEncode():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set htmlEncode(value:Boolean):void;

  [ExtConfig]
  /**
   * A function to transform the raw value for display in the field.
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     renderTo: document.body,
   *     width: 175,
   *     bodyPadding: 10,
   *     title: 'Final Score',
   *     items: [{
   *         xtype: 'displayfield',
   *         fieldLabel: 'Grade',
   *         name: 'final_grade',
   *         value: 68,
   *         renderer: function (value, field) {
   *             var color = (value &lt; 70) ? 'red' : 'black';
   *             return '&lt;span style="color:' + color + ';"&gt;' + value + '&lt;/span&gt;';
   *         }
   *     }]
   * });
   * </code></pre>
   */
  public native function get renderer():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderer(value:Function):void;

  [ExtConfig]
  /**
   * The scope to execute the <a href="#!/api/Ext.form.field.Display-cfg-renderer" rel="Ext.form.field.Display-cfg-renderer" class="docClass">renderer</a> function. Defaults to this.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;
}
}