package ext.form.field {
import ext.form.FieldContainer;
import ext.toolbar.Toolbar;
import js.HTMLElement;

[Uses("ext.form.FieldContainer")]
[Uses("ext.form.field.Field")]
[Uses("ext.Base")]
[Uses("ext.form.FieldAncestor")]
[Uses("ext.Mixin")]
[Uses("ext.form.Labelable")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.form.field.HtmlEditor", require)]
/**
 * Fires when the editor is first receives the focus. Any insertion must wait until after this event.
 * @eventType ext.form.field.events.HtmlEditorEvent.ACTIVATE
 */
[Event(name = 'onActivate', type = 'ext.form.field.events.HtmlEditorEvent')]
/**
 * Fires before the iframe editor is updated with content from the textarea. Return false to cancel the
 * push.
 * @eventType ext.form.field.events.HtmlEditor_htmlEvent.BEFORE_PUSH
 */
[Event(name = 'onBeforePush', type = 'ext.form.field.events.HtmlEditor_htmlEvent')]
/**
 * Fires before the textarea is updated with content from the editor iframe. Return false to cancel the
 * sync.
 * @eventType ext.form.field.events.HtmlEditor_htmlEvent.BEFORE_SYNC
 */
[Event(name = 'onBeforeSync', type = 'ext.form.field.events.HtmlEditor_htmlEvent')]
/**
 * Fires when the value of a field is changed. The value of a field is
 * checked for changes when the field's <a href="#!/api/Ext.form.field.Field-method-setValue" rel="Ext.form.field.Field-method-setValue" class="docClass">setValue</a> method
 * is called and when any of the events listed in
 * <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">checkChangeEvents</a> are fired.
 * @eventType ext.form.field.events.HtmlEditor_newValue_oldValueEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.form.field.events.HtmlEditor_newValue_oldValueEvent')]
/**
 * Fires when a change in the field's <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state is detected.
 * @eventType ext.form.field.events.HtmlEditor_isDirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.form.field.events.HtmlEditor_isDirtyEvent')]
/**
 * Fires when the editor switches edit modes
 * @eventType ext.form.field.events.HtmlEditor_sourceEditEvent.EDIT_MODE_CHANGE
 */
[Event(name = 'onEditModeChange', type = 'ext.form.field.events.HtmlEditor_sourceEditEvent')]
/**
 * Fires when the editor is fully initialized (including the iframe)
 * @eventType ext.form.field.events.HtmlEditorEvent.INITIALIZE
 */
[Event(name = 'onInitialize', type = 'ext.form.field.events.HtmlEditorEvent')]
/**
 * Fires when the iframe editor is updated with content from the textarea.
 * @eventType ext.form.field.events.HtmlEditor_htmlEvent.PUSH
 */
[Event(name = 'onPush', type = 'ext.form.field.events.HtmlEditor_htmlEvent')]
/**
 * Fires when the textarea is updated with content from the editor iframe.
 * @eventType ext.form.field.events.HtmlEditor_htmlEvent.SYNC
 */
[Event(name = 'onSync', type = 'ext.form.field.events.HtmlEditor_htmlEvent')]
/**
 * Fires when a change in the field's validity is detected.
 * @eventType ext.form.field.events.HtmlEditor_isValidEvent.VALIDITY_CHANGE
 */
[Event(name = 'onValidityChange', type = 'ext.form.field.events.HtmlEditor_isValidEvent')]
/**
 * Provides a lightweight HTML Editor component. Some toolbar features are not supported by Safari and will be
 * automatically hidden when needed. These are noted in the config options where appropriate.
 *
 * <p>The editor's toolbar buttons have tooltips defined in the <a href="#!/api/Ext.form.field.HtmlEditor-property-buttonTips" rel="Ext.form.field.HtmlEditor-property-buttonTips" class="docClass">buttonTips</a> property, but they are not
 * enabled by default unless the global <a href="#!/api/Ext.tip.QuickTipManager" rel="Ext.tip.QuickTipManager" class="docClass">Ext.tip.QuickTipManager</a> singleton is
 * <a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">initialized</a>.</p>
 *
 * <p>An Editor is a sensitive component that can't be used in all spots standard fields can be used. Putting an
 * Editor within any element that has display set to 'none' can cause problems in Safari and Firefox due to their
 * default iframe reloading bugs.</p>
 *
 * <h1>Example usage</h1>
 *
 * <p>Simple example rendered with default options:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();  // enable tooltips
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.HtmlEditor" rel="Ext.form.field.HtmlEditor" class="docClass">Ext.form.HtmlEditor</a>', {
 *     width: 580,
 *     height: 250,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>Passed via xtype into a container and with custom options:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();  // enable tooltips
 * new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
 *     title: 'HTML Editor',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     width: 550,
 *     height: 250,
 *     frame: true,
 *     layout: 'fit',
 *     items: {
 *         xtype: 'htmleditor',
 *         enableColors: false,
 *         enableAlignments: false
 *     }
 * });
 * </code></pre>
 *
 * <h1>Reflow issues</h1>
 *
 * <p>In some browsers, a layout reflow will cause the underlying editor iframe to be reset. This
 * is most commonly seen when using the editor in collapsed panels with animation. In these cases
 * it is best to avoid animation. More information can be found here: https://bugzilla.mozilla.org/show_bug.cgi?id=90268</p>
 */
public class HtmlEditor extends ext.form.FieldContainer implements ext.form.field.IField {
  /**
   * @param config @inheritDoc
   */
  public function HtmlEditor(config:ext.form.field.HtmlEditor = null) {
    super();
  }

  [ExtConfig]
  /**
   * True to disable the field. Disabled Fields will not be <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a>.
   * @default false
   * @see #setDisabled()
   */
  override public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set disabled(value:Boolean):void;

  /**
   * Object collection of toolbar tooltips for the buttons in the editor. The key is the command id associated with
   * that button and the value is a valid QuickTips object. For example:
   *
   * <pre><code>{
   *     bold: {
   *         title: 'Bold (Ctrl+B)',
   *         text: 'Make the selected text bold.',
   *         cls: 'x-html-editor-tip'
   *     },
   *     italic: {
   *         title: 'Italic (Ctrl+I)',
   *         text: 'Make the selected text italic.',
   *         cls: 'x-html-editor-tip'
   *     }
   *     // ...
   * }
   * </code></pre>
   * @default {bold: {title: 'Bold (Ctrl+B)', text: 'Make the selected text bold.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, italic: {title: 'Italic (Ctrl+I)', text: 'Make the selected text italic.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, underline: {title: 'Underline (Ctrl+U)', text: 'Underline the selected text.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, increasefontsize: {title: 'Grow Text', text: 'Increase the font size.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, decreasefontsize: {title: 'Shrink Text', text: 'Decrease the font size.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, backcolor: {title: 'Text Highlight Color', text: 'Change the background color of the selected text.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, forecolor: {title: 'Font Color', text: 'Change the color of the selected text.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, justifyleft: {title: 'Align Text Left', text: 'Align text to the left.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, justifycenter: {title: 'Center Text', text: 'Center text in the editor.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, justifyright: {title: 'Align Text Right', text: 'Align text to the right.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, insertunorderedlist: {title: 'Bullet List', text: 'Start a bulleted list.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, insertorderedlist: {title: 'Numbered List', text: 'Start a numbered list.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, createlink: {title: 'Hyperlink', text: 'Make the selected text a hyperlink.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}, sourceedit: {title: 'Source Edit', text: 'Switch to source editing mode.', cls: Ext.baseCSSPrefix + 'html-editor-tip'}}
   */
  public native function get buttonTips():Object;

  /**
   * @private
   */
  public native function set buttonTips(value:Object):void;

  /**
   * Flag denoting that this component is a Field. Always true.
   * @default true
   */
  public native function get isFormField():Boolean;

  /**
   * @private
   */
  public native function set isFormField(value:Boolean):void;

  /**
   * The original value of the field as configured in the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a> configuration, or as loaded by the last
   * form load operation if the form's <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> setting is <code>true</code>.
   */
  public native function get originalValue():*;

  /**
   * @private
   */
  public native function set originalValue(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the iframe element. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get afterIFrameTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterIFrameTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the textarea element. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get afterTextAreaTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterTextAreaTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the iframe element. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get beforeIFrameTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeIFrameTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the textarea element. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get beforeTextAreaTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeTextAreaTpl(value:*):void;

  [ExtConfig]
  /**
   * A config object to apply to the toolbar's <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">buttons</a> to affect how they operate, eg:
   *
   * <p>   buttonDefaults: {
   *        tooltip: {
   *            align: 't-b',
   *            anchor: true
   *        }
   *    }</p>
   * @since 6.2.0
   */
  public native function get buttonDefaults():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttonDefaults(value:Object):void;

  [ExtConfig]
  /**
   * The default text for the create link prompt
   * @default 'Please enter the URL for the link:'
   */
  public native function get createLinkText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set createLinkText(value:String):void;

  [ExtConfig]
  /**
   * A default <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">ui</a> to use for the HtmlEditor's toolbar
   * <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">buttons</a>.
   * @default 'default-toolbar'
   */
  public native function get defaultButtonUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultButtonUI(value:String):void;

  [ExtConfig]
  /**
   * The default value for the create link prompt
   * @default 'http://'
   */
  public native function get defaultLinkValue():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultLinkValue(value:String):void;

  [ExtConfig]
  /**
   * A default value to be put into the editor to resolve focus issues.
   *
   * <p>Defaults to (Non-breaking space) in Opera,
   * (Zero-width space) in all other browsers.</p>
   */
  public native function get defaultValue():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultValue(value:String):void;

  [ExtConfig]
  /**
   * Enable the left, center, right alignment buttons
   * @default true
   */
  public native function get enableAlignments():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableAlignments(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable the fore/highlight color buttons
   * @default true
   */
  public native function get enableColors():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableColors(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable font selection. Not available in Safari 2.
   * @default true
   */
  public native function get enableFont():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFont(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable the increase/decrease font size buttons
   * @default true
   */
  public native function get enableFontSize():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFontSize(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable the bold, italic and underline buttons
   * @default true
   */
  public native function get enableFormat():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFormat(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable the create link button. Not available in Safari 2.
   * @default true
   */
  public native function get enableLinks():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableLinks(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable the bullet and numbered list buttons. Not available in Safari 2.
   * @default true
   */
  public native function get enableLists():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableLists(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable the switch to source edit button. Not available in Safari 2.
   * @default true
   */
  public native function get enableSourceEdit():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableSourceEdit(value:Boolean):void;

  [ExtConfig]
  /**
   * An array of available font families
   * @default ['Arial', 'Courier New', 'Tahoma', 'Times New Roman', 'Verdana']
   */
  public native function get fontFamilies():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fontFamilies(value:Array):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * inside the iframe element (as attributes). If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get iframeAttrTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set iframeAttrTpl(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the field. By default this is used as the parameter name when including the
   * <a href="#!/api/Ext.form.field.Field-method-getSubmitData" rel="Ext.form.field.Field-method-getSubmitData" class="docClass">field value</a> in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>. To prevent the field from
   * being included in the form submit, set <a href="#!/api/Ext.form.field.Field-cfg-submitValue" rel="Ext.form.field.Field-cfg-submitValue" class="docClass">submitValue</a> to false.
   * @see #getName()
   */
  public native function get name():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  /**
   * Setting this to false will prevent the field from being <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a> even when it is
   * not disabled.
   * @default true
   */
  public native function get submitValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitValue(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether this field should be validated immediately whenever a change in its value is detected.
   * If the validation results in a change in the field's validity, a <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will be
   * fired. This allows the field to show feedback about the validity of its contents immediately as the user is
   * typing.
   *
   * <p>When set to false, feedback will not be immediate. However the form will still be validated before submitting if
   * the clientValidation option to <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">Ext.form.Basic.doAction</a> is enabled, or if the field or form are validated
   * manually.</p>
   *
   * <p>See also <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">Ext.form.field.Base.checkChangeEvents</a> for controlling how changes to the field's value are
   * detected.</p>
   * @default true
   */
  public native function get validateOnChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validateOnChange(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This property, when a <code>String</code>, contributes its value to the error state of this
   * instance as reported by <code>getErrors</code>.
   * @see #getValidation()
   * @see #setValidation()
   */
  public native function get validation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set validation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A value to initialize this field with.
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Object):void;

  [ExtConfig]
  /**
   * The event name(s) to use to publish the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a>
   * <a href="#!/api/Ext.form.field.Base-cfg-bind" rel="Ext.form.field.Base-cfg-bind" class="docClass">Ext.form.field.Base.bind</a> for this field.
   * @default 'change'
   * @since 5.0.1
   */
  public native function get valuePublishEvent():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valuePublishEvent(value:*):void;

  /**
   * A utility for grouping a set of modifications which may trigger value changes into a single transaction, to
   * prevent excessive firing of <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change</a> events. This is useful for instance if the field has sub-fields which
   * are being updated as a group; you don't want the container field to check its own changed state for each subfield
   * change.
   * @param fn The function to call with change checks suspended.
   */
  public native function batchChanges(fn:Function):void;

  /**
   * Checks whether the value of the field has changed since the last time it was checked.
   * If the value has changed, it:
   *
   * <ol>
   * <li>Fires the <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change event</a>,</li>
   * <li>Performs validation if the <a href="#!/api/Ext.form.field.Field-cfg-validateOnChange" rel="Ext.form.field.Field-cfg-validateOnChange" class="docClass">validateOnChange</a> config is enabled, firing the
   * <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange event</a> if the validity has changed, and</li>
   * <li>Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">dirty state</a> of the field and fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange event</a>
   * if it has changed.</li>
   * </ol>
   */
  public native function checkChange():void;

  /**
   * Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state of the field and if it has changed since the last time it was checked,
   * fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange</a> event.
   */
  public native function checkDirty():void;

  /**
   * If you need/want custom HTML cleanup, this is the method you should override.
   * @param html The HTML to be cleaned
   * @return The cleaned HTML
   */
  public native function cleanHtml(html:String):String;

  /**
   * Cleans up values initialized by this Field mixin on the current instance.
   * Components using this mixin should call this method before being destroyed.
   */
  public native function cleanupField():void;

  /**
   * Executes a Midas editor command directly on the editor document. For visual commands, you should use
   * <a href="#!/api/Ext.form.field.HtmlEditor-method-relayCmd" rel="Ext.form.field.HtmlEditor-method-relayCmd" class="docClass">relayCmd</a> instead. <strong>This should only be called after the editor is initialized.</strong>
   * @param cmd The Midas command
   * @param value The value to pass to the command
   * @default null
   */
  public native function execCmd(cmd:String, value:* = null):void;

  /**
   * Only relevant if the instance's <a href="#!/api/Ext.form.field.Field-method-isFileUpload" rel="Ext.form.field.Field-method-isFileUpload" class="docClass">isFileUpload</a> method returns true. Returns a reference to the file input
   * DOM element holding the user's selected file. The input will be appended into the submission form and will not be
   * returned, so this method should also create a replacement.
   */
  public native function extractFileInput():js.HTMLElement;

  /**
   * Called when the editor initializes the iframe with HTML contents. Override this method if you
   * want to change the initialization markup of the iframe (e.g. to add stylesheets).
   *
   * <p><strong>Note:</strong> IE8-Standards has unwanted scroller behavior, so the default meta tag forces IE7 compatibility.
   * Also note that forcing IE7 mode works when the page is loaded normally, but if you are using IE's Web
   * Developer Tools to manually set the document mode, that will take precedence and override what this
   * code sets by default. This can be confusing when developing, but is not a user-facing issue.</p>
   */
  public native function getDocMarkup():String;

  /**
   * Runs this field's validators and returns an array of error messages for any validation failures. This is called
   * internally during validation and would not usually need to be used manually.
   *
   * <p>Each subclass should override or augment the return value to provide their own errors.</p>
   * @param value The value to get errors for (defaults to the current field value)
   * @return All error messages for this field; an empty Array if none.
   */
  public native function getErrors(value:*):Array;

  /**
   * Returns the value(s) that should be saved to the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> instance for this field, when <a href="#!/api/Ext.form.Basic-method-updateRecord" rel="Ext.form.Basic-method-updateRecord" class="docClass">Ext.form.Basic.updateRecord</a> is called. Typically this will be an object with a single name-value pair, the name
   * being this field's <a href="#!/api/Ext.form.field.Field-method-getName" rel="Ext.form.field.Field-method-getName" class="docClass">name</a> and the value being its current data value. More advanced field
   * implementations may return more than one name-value pair. The returned values will be saved to the corresponding
   * field names in the Model.
   *
   * <p>Note that the values returned from this method are not guaranteed to have been successfully <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validated</a>.</p>
   * @return A mapping of submit parameter names to values; each value should be a string, or an array of
   * strings if that particular name has multiple values. It can also return null if there are no parameters to be
   * submitted.
   */
  public native function getModelData(includeEmptyText:Object, isSubmitting:Object):Object;

  /**
   * Returns the <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a> attribute of the field. This is used as the parameter name
   * when including the field value in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>.
   * @return name The field <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a>
   */
  public native function getName():String;

  /**
   * Returns the parameter(s) that would be included in a standard form submit for this field. Typically this will be
   * an object with a single name-value pair, the name being this field's <a href="#!/api/Ext.form.field.Field-method-getName" rel="Ext.form.field.Field-method-getName" class="docClass">name</a> and the value being
   * its current stringified value. More advanced field implementations may return more than one name-value pair.
   *
   * <p>Note that the values returned from this method are not guaranteed to have been successfully <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validated</a>.</p>
   * @return A mapping of submit parameter names to values; each value should be a string, or an array of
   * strings if that particular name has multiple values. It can also return null if there are no parameters to be
   * submitted.
   */
  public native function getSubmitData():Object;

  /**
   * Returns the editor's toolbar. <strong>This is only available after the editor has been rendered.</strong>
   */
  public native function getToolbar():ext.toolbar.Toolbar;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   */
  public native function getValidation():*;

  /**
   * Initializes this Field mixin on the current instance. Components using this mixin should call this method during
   * their own initialization process.
   */
  public native function initField():void;

  /**
   * Initializes the field's value based on the initial config.
   */
  public native function initValue():void;

  /**
   * Inserts the passed text at the current cursor position.
   * <strong>Note:</strong> the editor must be initialized and activated to insert text.
   */
  public native function insertAtCursor(text:String):void;

  /**
   * Returns true if the value of this Field has been changed from its <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a>.
   * Will always return false if the field is disabled.
   *
   * <p>Note that if the owning <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">form</a> was configured with
   * <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> then the <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a> is updated when
   * the values are loaded by <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a>.</p>
   * @return True if this field has been changed from its original value (and is not disabled),
   * false otherwise.
   */
  public native function isDirty():Boolean;

  /**
   * Returns whether this Field is a file upload field; if it returns true, forms will use special techniques for
   * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitting the form</a> via AJAX. See <a href="#!/api/Ext.form.Basic-method-hasUpload" rel="Ext.form.Basic-method-hasUpload" class="docClass">Ext.form.Basic.hasUpload</a> for details. If
   * this returns true, the <a href="#!/api/Ext.form.field.Field-method-extractFileInput" rel="Ext.form.field.Field-method-extractFileInput" class="docClass">extractFileInput</a> method must also be implemented to return the corresponding file
   * input element.
   */
  public native function isFileUpload():Boolean;

  /**
   * Returns whether or not the field value is currently valid by <a href="#!/api/Ext.form.field.Field-method-getErrors" rel="Ext.form.field.Field-method-getErrors" class="docClass">validating</a> the field's current
   * value. The <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will not be fired; use <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> instead if you want the event
   * to fire. <strong>Note</strong>: <a href="#!/api/Ext.form.field.Field-cfg-disabled" rel="Ext.form.field.Field-cfg-disabled" class="docClass">disabled</a> fields are always treated as valid.
   *
   * <p>Implementations are encouraged to ensure that this method does not have side-effects such as triggering error
   * message display.</p>
   * @return True if the value is valid, else false
   */
  public native function isValid():Boolean;

  /**
   * Display one or more error messages associated with this field, using
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">Ext.form.Labelable.msgTarget</a> to determine how to display the messages and
   * applying <a href="#!/api/Ext.form.Labelable-cfg-invalidCls" rel="Ext.form.Labelable-cfg-invalidCls" class="docClass">Ext.form.Labelable.invalidCls</a> to the field's UI element.
   *
   * <pre><code>var formPanel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     title: 'Contact Info',
   *     width: 300,
   *     bodyPadding: 10,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items: [{
   *         xtype: 'textfield',
   *         name: 'name',
   *         id: 'nameId',
   *         fieldLabel: 'Name'
   *     }],
   *     bbar: [{
   *         text: 'Mark both fields invalid',
   *         handler: function() {
   *             var nameField = formPanel.getForm().findField('name');
   *             nameField.markInvalid('Name invalid message');
   *
   *             // multiple error string syntax
   *             // nameField.markInvalid(['First message', 'Second message']);
   *         }
   *     }]
   * });
   * </code></pre>
   *
   * <p><strong>Note</strong>: this method does not cause the Field's <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> or
   * <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a> methods to return <code>false</code> if the value does <em>pass</em> validation.
   * So simply marking a Field as invalid will not prevent submission of forms
   * submitted with the <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   * @param errors The validation message(s) to display.
   */
  public native function markInvalid(errors:*):void;

  /**
   * Pushes the value of the textarea into the iframe editor.
   */
  public native function pushValue():void;

  /**
   * Executes a Midas editor command on the editor document and performs necessary focus and toolbar updates.
   * <strong>This should only be called after the editor is initialized.</strong>
   * @param cmd The Midas command
   * @param value The value to pass to the command
   * @default null
   */
  public native function relayCmd(cmd:String, value:* = null):void;

  /**
   * Resets the current field value to the originally loaded value and clears any validation messages. See <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>
   */
  public native function reset():void;

  /**
   * Resets the field's <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a> property so it matches the current <a href="#!/api/Ext.form.field.Field-method-getValue" rel="Ext.form.field.Field-method-getValue" class="docClass">value</a>. This is
   * called by <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a> if the form's
   * <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> property is set to true.
   */
  public native function resetOriginalValue():void;

  /**
   * Sets the read only state of this field.
   * @param readOnly Whether the field should be read only.
   */
  public native function setReadOnly(readOnly:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   * @param validation The new value.
   */
  public native function setValidation(validation:*):void;

  /**
   * Syncs the contents of the editor iframe with the textarea.
   */
  public native function syncValue():void;

  /**
   * Toggles the editor between standard and source edit mode.
   * @param sourceEditMode True for source edit, false for standard
   */
  public native function toggleSourceEdit(sourceEditMode:Boolean = false):void;

  /**
   * Triggers a toolbar update by reading the markup state of the current selection in the editor.
   */
  public native function updateToolbar():void;

  /**
   * @inheritDoc
   */
  public native function clearInvalid():void;

  /**
   * @inheritDoc
   */
  public native function getValue():*;

  /**
   * @inheritDoc
   */
  public native function isEqual(value1:Object, value2:Object):Boolean;

  /**
   * @inheritDoc
   */
  public native function setValue(value:*):ext.form.field.IField;

  /**
   * @inheritDoc
   */
  public native function validate():Boolean;

  /**
   * Template method before a field is reset.
   */
  protected native function beforeReset():void;

  /**
   * Called when the editor creates its toolbar. Override this method if you need to
   * add custom toolbar buttons.
   */
  protected native function createToolbar(editor:ext.form.field.HtmlEditor):void;
}
}