package ext.form.field {
import ext.data.Model;

[Uses("ext.form.field.ComboBox")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.form.field.Picker")]
[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Tag", require)]
/**
 * Fires when the <strong><a href="#!/api/Ext.form.field.Tag-method-autoSize" rel="Ext.form.field.Tag-method-autoSize" class="docClass">autoSize</a></strong> function is triggered and the field is resized according to the
 * <a href="#!/api/Ext.form.field.Tag-cfg-grow" rel="Ext.form.field.Tag-cfg-grow" class="docClass">grow</a>/<a href="#!/api/Ext.form.field.Tag-cfg-growMin" rel="Ext.form.field.Tag-cfg-growMin" class="docClass">growMin</a>/<a href="#!/api/Ext.form.field.Tag-cfg-growMax" rel="Ext.form.field.Tag-cfg-growMax" class="docClass">growMax</a> configs as a result. This event provides a hook for the
 * developer to apply additional logic at runtime to resize the field if needed.
 * @eventType ext.form.field.events.Tag_heightEvent.AUTO_SIZE
 */
[Event(name = 'onAutoSize', type = 'ext.form.field.events.Tag_heightEvent')]
/**
 * <code>tagfield</code> provides a combobox that removes the hassle of dealing with long and unruly select
 * options. The selected list is visually maintained in the value display area instead of
 * within the picker itself. Users may easily add or remove <code>tags</code> from the
 * display value area.
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *       fields: ['id','show'],
 *       data: [
 *           {id: 0, show: 'Battlestar Galactica'},
 *           {id: 1, show: 'Doctor Who'},
 *           {id: 2, show: 'Farscape'},
 *           {id: 3, show: 'Firefly'},
 *           {id: 4, show: 'Star Trek'},
 *           {id: 5, show: 'Star Wars: Christmas Special'}
 *       ]
 *    });
 *
 *   <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *       renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *       title: 'Sci-Fi Television',
 *       height: 200,
 *       width: 500,
 *       items: [{
 *           xtype: 'tagfield',
 *           fieldLabel: 'Select a Show',
 *           store: shows,
 *           displayField: 'show',
 *           valueField: 'id',
 *           queryMode: 'local',
 *           filterPickList: true
 *       }]
 *   });
 * </code></pre>
 *
 * <h3>History</h3>
 *
 * <p>Inspired by the SuperBoxSelect component for ExtJS 3,
 * which in turn was inspired by the BoxSelect component for ExtJS 2.</p>
 *
 * <p>Various contributions and suggestions made by many members of the ExtJS community which can be seen
 * in the <a href="http://www.sencha.com/forum/showthread.php?134751-Ext.ux.form.field.BoxSelect">user extension forum post</a>.</p>
 *
 * <p>By: kvee_iv http://www.sencha.com/forum/member.php?29437-kveeiv</p>
 */
public class Tag extends ext.form.field.ComboBox {
  /**
   * @param config @inheritDoc
   */
  public function Tag(config:ext.form.field.Tag = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The character(s) used to separate new values to be added when <a href="#!/api/Ext.form.field.Tag-cfg-createNewOnEnter" rel="Ext.form.field.Tag-cfg-createNewOnEnter" class="docClass">createNewOnEnter</a>
   * or <a href="#!/api/Ext.form.field.Tag-cfg-createNewOnBlur" rel="Ext.form.field.Tag-cfg-createNewOnBlur" class="docClass">createNewOnBlur</a> are set.
   * <code><a href="#!/api/Ext.form.field.Tag-cfg-multiSelect" rel="Ext.form.field.Tag-cfg-multiSelect" class="docClass">multiSelect</a> = true</code>.
   * @default ','
   * @see #getDelimiter()
   * @see #setDelimiter()
   */
  override public native function get delimiter():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set delimiter(value:String):void;

  [ExtConfig]
  /**
   * When <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> is <code>false</code>, new records can be created by the user as they
   * are typed. These records are <strong>not</strong> added to the combo's store. Multiple new values
   * may be added by separating them with the <a href="#!/api/Ext.form.field.Tag-cfg-delimiter" rel="Ext.form.field.Tag-cfg-delimiter" class="docClass">delimiter</a>, and can be further configured using the
   * <a href="#!/api/Ext.form.field.Tag-cfg-createNewOnEnter" rel="Ext.form.field.Tag-cfg-createNewOnEnter" class="docClass">createNewOnEnter</a> and <a href="#!/api/Ext.form.field.Tag-cfg-createNewOnBlur" rel="Ext.form.field.Tag-cfg-createNewOnBlur" class="docClass">createNewOnBlur</a> configuration options.
   *
   * <p>This functionality is primarily useful for things such as an email address.</p>
   * @default true
   */
  override public native function get forceSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set forceSelection(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> if this field should automatically grow and shrink vertically to its content.
   * Note that this overrides the natural trigger grow functionality, which is used to size
   * the field horizontally.
   * @default true
   */
  override public native function get grow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set grow(value:Boolean):void;

  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.Tag-cfg-grow" rel="Ext.form.field.Tag-cfg-grow" class="docClass">grow</a> is <code>false</code>
   *
   * <p>The maximum height to allow when <a href="#!/api/Ext.form.field.Tag-cfg-grow" rel="Ext.form.field.Tag-cfg-grow" class="docClass">grow</a> is <code>true</code>, or <code>false</code> to allow for
   * natural vertical growth based on the current selected values. See also <a href="#!/api/Ext.form.field.Tag-cfg-growMin" rel="Ext.form.field.Tag-cfg-growMin" class="docClass">growMin</a>.</p>
   * @default false
   */
  override public native function get growMax():Number;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set growMax(value:Number):void;

  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.Tag-cfg-grow" rel="Ext.form.field.Tag-cfg-grow" class="docClass">grow</a> is <code>false</code>
   *
   * <p>The minimum height to allow when <a href="#!/api/Ext.form.field.Tag-cfg-grow" rel="Ext.form.field.Tag-cfg-grow" class="docClass">grow</a> is <code>true</code>, or <code>false</code> to allow for
   * natural vertical growth based on the current selected values. See also <a href="#!/api/Ext.form.field.Tag-cfg-growMax" rel="Ext.form.field.Tag-cfg-growMax" class="docClass">growMax</a>.</p>
   * @default false
   */
  override public native function get growMin():Number;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set growMin(value:Number):void;

  [ExtConfig]
  /**
   * If set to <code>true</code>, allows the combo field to hold more than one value at a time, and allows selecting multiple
   * items from the dropdown list. The combo's text field will show all selected values using the template
   * defined by <a href="#!/api/Ext.form.field.Tag-cfg-labelTpl" rel="Ext.form.field.Tag-cfg-labelTpl" class="docClass">labelTpl</a>.
   * @default true
   */
  override public native function get multiSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set multiSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * Label to be announced to screen reader users
   * when they use Up and Down arrow keys to navigate the list of available values.
   * @default 'Available values'
   */
  public native function get ariaAvailableListLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaAvailableListLabel(value:String):void;

  [ExtConfig]
  /**
   * Template text for announcing deselected values to
   * screen reader users. '{0}' will be replaced with the list of values removed from
   * selected list.
   * @default '{0} removed from selection.'
   */
  public native function get ariaDeselectedText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDeselectedText(value:String):void;

  [ExtConfig]
  /**
   * The text to be announced by screen readers when input element is
   * focused. This text is used when this component is configured not to allow creating
   * new values; when <a href="#!/api/Ext.form.field.Tag-cfg-createNewOnEnter" rel="Ext.form.field.Tag-cfg-createNewOnEnter" class="docClass">createNewOnEnter</a> is set to <code>true</code>, <a href="#!/api/Ext.form.field.Tag-cfg-ariaHelpTextEditable" rel="Ext.form.field.Tag-cfg-ariaHelpTextEditable" class="docClass">ariaHelpTextEditable</a>
   * will be used instead.
   * @default 'Use Up and Down arrows to view available values, Enter to select. ' + 'Use Left and Right arrows to view selected values, Delete key to deselect.'
   */
  public native function get ariaHelpText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaHelpText(value:String):void;

  [ExtConfig]
  /**
   * The text to be announced by screen readers when
   * input element is focused. This text is used when <a href="#!/api/Ext.form.field.Tag-cfg-createNewOnEnter" rel="Ext.form.field.Tag-cfg-createNewOnEnter" class="docClass">createNewOnEnter</a> is set to <code>true</code>;
   * see also <a href="#!/api/Ext.form.field.Tag-cfg-ariaHelpText" rel="Ext.form.field.Tag-cfg-ariaHelpText" class="docClass">ariaHelpText</a>.
   * @default 'Use Up and Down arrows to view available values, Enter to select. ' + 'Type and press Enter to create a new value. ' + 'Use Left and Right arrows to view selected values, Delete key to deselect.'
   */
  public native function get ariaHelpTextEditable():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaHelpTextEditable(value:String):void;

  [ExtConfig]
  /**
   * Text to announce to screen reader users when no
   * values are currently selected. This text is used when Tag field is focused.
   * @default 'No value selected.'
   */
  public native function get ariaNoneSelectedText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaNoneSelectedText(value:String):void;

  [ExtConfig]
  /**
   * Label to be announced to screen reader users
   * when they use Left and Right arrow keys to navigate the list of currently selected values.
   * @default 'Selected values'
   */
  public native function get ariaSelectedListLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaSelectedListLabel(value:String):void;

  [ExtConfig]
  /**
   * Template text for announcing selected values to screen
   * reader users. '{0}' will be replaced with the list of selected values.
   * @default 'Selected {0}.'
   */
  public native function get ariaSelectedText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaSelectedText(value:String):void;

  [ExtConfig]
  /**
   * Set to <code>false</code> to disable clearing selected
   * values with Backspace key. This mode is recommended for accessible applications.
   * @default true
   */
  public native function get clearOnBackspace():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clearOnBackspace(value:Boolean):void;

  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> is <code>true</code>.
   *
   * <p>With this set to <code>true</code>, the creation described in
   * <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> will also be triggered when the field loses focus.</p>
   *
   * <p>Please note that this behavior is also affected by the configuration options
   * <a href="#!/api/Ext.form.field.Tag-cfg-autoSelect" rel="Ext.form.field.Tag-cfg-autoSelect" class="docClass">autoSelect</a> and <a href="#!/api/Ext.form.field.Tag-cfg-selectOnTab" rel="Ext.form.field.Tag-cfg-selectOnTab" class="docClass">selectOnTab</a>. If those are true and an existing
   * item would have been selected as a result, the partial text the user has entered will
   * be discarded and the existing item will be added to the selection.</p>
   *
   * <p>Setting this option to <code>true</code> is not recommended for accessible applications.</p>
   * @default false
   */
  public native function get createNewOnBlur():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set createNewOnBlur(value:Boolean):void;

  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> is <code>true</code>.
   *
   * <p>With this set to <code>true</code>, the creation described in
   * <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> will also be triggered by the 'enter' key.</p>
   * @default false
   */
  public native function get createNewOnEnter():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set createNewOnEnter(value:Boolean):void;

  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.Tag-cfg-multiSelect" rel="Ext.form.field.Tag-cfg-multiSelect" class="docClass">multiSelect</a> is <code>false</code>.
   *
   * <p>Controls the formatting of the form submit value of the field as returned by <a href="#!/api/Ext.form.field.Tag-method-getSubmitValue" rel="Ext.form.field.Tag-method-getSubmitValue" class="docClass">getSubmitValue</a></p>
   *
   * <ul>
   * <li><code>true</code> for the field value to submit as a json encoded array in a single GET/POST variable</li>
   * <li><code>false</code> for the field to submit as an array of GET/POST variables</li>
   * </ul>
   * @default false
   */
  public native function get encodeSubmitValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set encodeSubmitValue(value:Boolean):void;

  [ExtConfig]
  /**
   * True to hide the currently selected values from the drop down list.
   *
   * <p>Setting this option to <code>true</code> is not recommended for accessible applications.</p>
   *
   * <ul>
   * <li><code>true</code> to hide currently selected values from the drop down pick list</li>
   * <li><code>false</code> to keep the item in the pick list as a selected item</li>
   * </ul>
   * @default false
   */
  public native function get filterPickList():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set filterPickList(value:Boolean):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a> to use for the inner
   * markup of the labeled items. Defaults to the configured <a href="#!/api/Ext.form.field.Tag-cfg-displayField" rel="Ext.form.field.Tag-cfg-displayField" class="docClass">displayField</a>
   */
  public native function get labelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelTpl(value:*):void;

  [ExtConfig]
  /**
   * <ul>
   * <li><code>true</code> to have each selected value fill to the width of the form field</li>
   * <li>`false to have each selected value size to its displayed contents</li>
   * </ul>
   * @default false
   */
  public native function get stacked():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stacked(value:Boolean):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a> to use for the tip of the labeled items.
   * @since 5.1.1
   */
  public native function get tipTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tipTpl(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to activate the trigger when clicking in empty space in the field. Note that the
   * subsequent behavior of this is controlled by the field's <a href="#!/api/Ext.form.field.Tag-cfg-triggerAction" rel="Ext.form.field.Tag-cfg-triggerAction" class="docClass">triggerAction</a>.
   * This behavior is similar to that of a basic ComboBox with <a href="#!/api/Ext.form.field.Tag-cfg-editable" rel="Ext.form.field.Tag-cfg-editable" class="docClass">editable</a> <code>false</code>.
   * @default true
   */
  public native function get triggerOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set triggerOnClick(value:Boolean):void;

  [ExtConfig]
  /**
   * The name of the parameter used to load unknown records into the store. If left unspecified, <a href="#!/api/Ext.form.field.Tag-cfg-valueField" rel="Ext.form.field.Tag-cfg-valueField" class="docClass">valueField</a>
   * will be used.
   */
  public native function get valueParam():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valueParam(value:String):void;

  /**
   * Track height change to fire <a href="#!/api/Ext.form.field.Tag-event-autosize" rel="Ext.form.field.Tag-event-autosize" class="docClass">autosize</a> event, when applicable.
   * @param width The width that was set
   * @param height The height that was set
   * @param oldWidth The old width, or <code>undefined</code> if this was the initial layout.
   * @param oldHeight The old height, or <code>undefined</code> if this was the initial layout.
   */
  override protected native function afterComponentLayout(width:Number, height:Number, oldWidth:*, oldHeight:*):void;

  /**
   * Initiate auto-sizing for height based on <a href="#!/api/Ext.form.field.Tag-cfg-grow" rel="Ext.form.field.Tag-cfg-grow" class="docClass">grow</a>, if applicable.
   */
  override public native function autoSize():void;

  /**
   * Intercept calls to getRawValue to pretend there is no inputEl for rawValue handling,
   * so that we can use inputEl for user input of just the current value.
   * @return value The raw String value of the field
   */
  override public native function getRawValue():String;

  /**
   * Overridden to optionally allow for submitting the field as a json encoded array.
   */
  override public native function getSubmitData():Object;

  /**
   * Overridden to be more accepting of varied value types
   * @param value1 The first value to compare
   * @param value2 The second value to compare
   */
  override public native function isEqual(value1:Object, value2:Object):Boolean;

  /**
   * Sets the specified value(s) into the field. The following value formats are recognized:
   *
   * <ul>
   * <li><p>Single Values</p>
   *
   * <ul>
   * <li>A string associated to this field's configured <a href="#!/api/Ext.form.field.Tag-cfg-valueField" rel="Ext.form.field.Tag-cfg-valueField" class="docClass">valueField</a></li>
   * <li>A record containing at least this field's configured <a href="#!/api/Ext.form.field.Tag-cfg-valueField" rel="Ext.form.field.Tag-cfg-valueField" class="docClass">valueField</a> and <a href="#!/api/Ext.form.field.Tag-cfg-displayField" rel="Ext.form.field.Tag-cfg-displayField" class="docClass">displayField</a></li>
   * </ul>
   * </li>
   * <li><p>Multiple Values</p>
   *
   * <ul>
   * <li>If <a href="#!/api/Ext.form.field.Tag-cfg-multiSelect" rel="Ext.form.field.Tag-cfg-multiSelect" class="docClass">multiSelect</a> is <code>true</code>, a string containing multiple strings as
   * specified in the Single Values section above, concatenated in to one string
   * with each entry separated by this field's configured <a href="#!/api/Ext.form.field.Tag-cfg-delimiter" rel="Ext.form.field.Tag-cfg-delimiter" class="docClass">delimiter</a></li>
   * <li>An array of strings as specified in the Single Values section above</li>
   * <li>An array of records as specified in the Single Values section above</li>
   * </ul>
   * </li>
   * </ul>
   *
   *
   * <p>In any of the string formats above, the following occurs if an associated record cannot be found:</p>
   *
   * <ol>
   * <li>If <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> is <code>false</code>, a new record of the <a href="#!/api/Ext.form.field.Tag-cfg-store" rel="Ext.form.field.Tag-cfg-store" class="docClass">store</a>'s configured model type
   * will be created using the given value as the <a href="#!/api/Ext.form.field.Tag-cfg-displayField" rel="Ext.form.field.Tag-cfg-displayField" class="docClass">displayField</a> and <a href="#!/api/Ext.form.field.Tag-cfg-valueField" rel="Ext.form.field.Tag-cfg-valueField" class="docClass">valueField</a>.
   * This record will be added to the current value, but it will <strong>not</strong> be added to the store.</li>
   * <li>If <a href="#!/api/Ext.form.field.Tag-cfg-forceSelection" rel="Ext.form.field.Tag-cfg-forceSelection" class="docClass">forceSelection</a> is <code>true</code> and <a href="#!/api/Ext.form.field.Tag-cfg-queryMode" rel="Ext.form.field.Tag-cfg-queryMode" class="docClass">queryMode</a> is <code>remote</code>, the list of unknown
   * values will be submitted as a call to the <a href="#!/api/Ext.form.field.Tag-cfg-store" rel="Ext.form.field.Tag-cfg-store" class="docClass">store</a>'s load as a parameter named by
   * the <a href="#!/api/Ext.form.field.Tag-cfg-valueParam" rel="Ext.form.field.Tag-cfg-valueParam" class="docClass">valueParam</a> with values separated by the configured <a href="#!/api/Ext.form.field.Tag-cfg-delimiter" rel="Ext.form.field.Tag-cfg-delimiter" class="docClass">delimiter</a>.
   * <strong> This process will cause setValue to asynchronously process. </strong> This will only be attempted
   * once. Any unknown values that the server does not return records for will be removed.</li>
   * <li>Otherwise, unknown values will be removed.</li>
   * </ol>
   * @param value The value to set
   * @return this, or <code>false</code> if asynchronously querying for unknown values
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * Overridden to handle partial-input selections more directly
   */
  public native function assertValue():void;

  /**
   * Returns the record from valueStore for the labeled item node
   */
  public native function getRecordByListItemNode(itemEl:Object):ext.data.Model;

  /**
   * Returns the records for the field's current value
   * @return The records for the field's current value
   */
  public native function getValueRecords():Array;

  /**
   * Intercept calls to onBlur to remove focusCls, because the base field
   * classes assume this should be applied to inputEl
   */
  public native function onBlur():void;

  /**
   * Intercept calls to onFocus to add focusCls, because the base field
   * classes assume this should be applied to inputEl
   */
  public native function onFocus():void;

  /**
   * Removal of labelled item by node reference
   */
  public native function removeByListItemNode(itemEl:Object):void;

  /**
   * Removes a value or values from the current value of the field
   * @param value The value or values to remove from the current value, see <a href="#!/api/Ext.form.field.Tag-method-setValue" rel="Ext.form.field.Tag-method-setValue" class="docClass">setValue</a>
   */
  public native function removeValue(value:*):void;

  /**
   * Intercept calls to renderActiveError to add invalidCls, because the base
   * field classes assume this should be applied to inputEl
   */
  public native function renderActiveError():void;

  /**
   * Toggle of labeled item selection by node reference
   */
  public native function toggleSelectionByListItemNode(itemEl:Object, keepExisting:Object):void;

  /**
   * Delegation control for selecting and removing labeled items or triggering list collapse/expansion
   */
  protected native function onItemListClick(e:Object):void;

  /**
   * Handles keyDown processing of key-based selection of labeled items.
   * Supported keyboard controls:
   *
   * <ul>
   * <li><p>If pick list is expanded</p>
   *
   * <ul>
   * <li><code>CTRL-A</code> will select all the items in the pick list</li>
   * </ul>
   * </li>
   * <li><p>If the cursor is at the beginning of the input field and there are values present</p>
   *
   * <ul>
   * <li><code>CTRL-A</code> will highlight all the currently selected values</li>
   * <li><code>BACKSPACE</code> and <code>DELETE</code> will remove any currently highlighted selected values</li>
   * <li><code>RIGHT</code> and <code>LEFT</code> will move the current highlight in the appropriate direction</li>
   * <li><code>SHIFT-RIGHT</code> and <code>SHIFT-LEFT</code> will add to the current highlight in the appropriate direction</li>
   * </ul>
   * </li>
   * </ul>
   */
  protected native function onKeyDown(e:Object):void;

  /**
   * Handles auto-selection and creation of labeled items based on this field's
   * delimiter, as well as the keyUp processing of key-based selection of labeled items.
   */
  protected native function onKeyUp(e:Object, t:Object):void;

  /**
   * Overridden to get and set the DOM value directly for type-ahead suggestion (bypassing get/setRawValue)
   */
  protected native function onTypeAhead():void;
}
}