package ext.form.trigger {
import ext.Base;
import ext.dom.Element;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.form.trigger.Trigger", require)]
/**
 * Base class for <a href="#!/api/Ext.form.field.Text-cfg-triggers" rel="Ext.form.field.Text-cfg-triggers" class="docClass">Text Field triggers</a>
 */
public class Trigger extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * @param config @inheritDoc
   */
  public function Trigger(config:ext.form.trigger.Trigger = null) {
    super();
  }

  /**
   * An optional template for rendering child elements inside the trigger element.
   * Useful for creating more complex triggers such as <a href="#!/api/Ext.form.trigger.Spinner" rel="Ext.form.trigger.Spinner" class="docClass">Ext.form.trigger.Spinner</a>.
   */
  public native function get bodyTpl():*;

  /**
   * @private
   */
  public native function set bodyTpl(value:*):void;

  [ExtConfig]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   */
  public native function get cls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cls(value:String):void;

  [ExtConfig]
  /**
   * An additional CSS class (or classes) to be added to the trigger's element. Can
   * be a single class name (e.g. 'foo') or a space-separated list of class names
   * (e.g. 'foo bar').
   */
  public native function get extraCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set extraCls(value:String):void;

  [ExtConfig]
  /**
   * Function to run when trigger is clicked or tapped.
   * &#64;controllable
   * @default undefined
   */
  public native function get handler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to initially render the trigger hidden.
   * @default false
   */
  public native function get hidden():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hidden(value:Boolean):void;

  [ExtConfig]
  /**
   * Set 'false' to prevent trigger from being hidden even though the related field is set <a href="#!/api/Ext.form.field.Text-cfg-readOnly" rel="Ext.form.field.Text-cfg-readOnly" class="docClass">readOnly</a>
   * @default true
   */
  public native function get hideOnReadOnly():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideOnReadOnly(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to attach a <a href="#!/api/Ext.util.ClickRepeater" rel="Ext.util.ClickRepeater" class="docClass">click repeater</a> to the trigger
   * @default false
   */
  public native function get repeatClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set repeatClick(value:Boolean):void;

  [ExtConfig]
  /**
   * Execution context for the <a href="#!/api/Ext.form.trigger.Trigger-cfg-handler" rel="Ext.form.trigger.Trigger-cfg-handler" class="docClass">handler</a> function.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * The triggers tooltip text. This text is available when using <code><a href="#!/api/Ext.tip.QuickTipManager" rel="Ext.tip.QuickTipManager" class="docClass">Ext.QuickTips</a></code>.
   * @since 6.2.0
   */
  public native function get tooltip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tooltip(value:String):void;

  [ExtConfig]
  /**
   * An optional weighting to change the ordering of the items. The default weight is
   * <code>0</code>.  Triggers are sorted by weight in ascending order before being rendered.<br/>
   * The value may be a negative value in order to position custom triggers ahead of
   * default triggers like that of ComboBox.
   * @default 0
   */
  public native function get weight():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set weight(value:Number):void;

  [ExtConfig]
  /**
   * The trigger's width, in pixels. Typically this is not needed
   * as the trigger width is normally determined by the style sheet, (see
   * extjs-text-field or
   * Ext.form.field.Text.$extjs-text-field-ui).
   */
  public native function get width():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set width(value:Number):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Get the element for this trigger.
   * @return The element for this trigger, <code>null</code> if not rendered.
   */
  public native function getEl():ext.dom.Element;

  /**
   * Hides the trigger
   */
  public native function hide():void;

  /**
   * Returns <code>true</code> if this Trigger is visible.
   * @since 5.0.0
   * @return <code>true</code> if this trigger is visible, <code>false</code> otherwise.
   */
  public native function isVisible():Boolean;

  /**
   * Shows the trigger
   */
  public native function show():void;

  /**
   * Called when this trigger's field is rendered
   */
  protected native function afterFieldRender():void;

  /**
   * Allows addition of data to the render data object for the <a href="#!/api/Ext.form.trigger.Trigger-property-bodyTpl" rel="Ext.form.trigger.Trigger-property-bodyTpl" class="docClass">bodyTpl</a>.
   */
  protected native function getBodyRenderData():Object;

  /**
   * Returns the element that should receive the "state" classes - <a href="#!/api/Ext.form.trigger.Trigger-property-focusCls" rel="Ext.form.trigger.Trigger-property-focusCls" class="docClass">focusCls</a>,
   * <a href="#!/api/Ext.form.trigger.Trigger-property-overCls" rel="Ext.form.trigger.Trigger-property-overCls" class="docClass">overCls</a>, and <a href="#!/api/Ext.form.trigger.Trigger-property-clickCls" rel="Ext.form.trigger.Trigger-property-clickCls" class="docClass">clickCls</a>.
   */
  protected native function getStateEl():ext.dom.Element;

  /**
   * Handles a click on the trigger's element
   */
  protected native function onClick():void;

  /**
   * Called when this trigger's field is blurred
   */
  protected native function onFieldBlur():void;

  /**
   * Called when this trigger's field is focused
   */
  protected native function onFieldFocus():void;

  /**
   * Called when this trigger's field is rendered
   */
  protected native function onFieldRender():void;
}
}