package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Action", require)]
/**
 * A Grid header type which renders an icon, or a series of icons in a grid cell, and offers a scoped click
 * handler for each icon.
 *
 * <pre class='inline-example '><code>// Init the singleton.  Any tag-based quick tips will start working.
 * <a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId:'employeeStore',
 *     fields:['firstname', 'lastname', 'seniority', 'dep', 'hired'],
 *     data:[
 *         {firstname:"Michael", lastname:"Scott"},
 *         {firstname:"Dwight", lastname:"Schrute"},
 *         {firstname:"Jim", lastname:"Halpert"},
 *         {firstname:"Kevin", lastname:"Malone"},
 *         {firstname:"Angela", lastname:"Martin"}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Action Column Demo',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('employeeStore'),
 *     columns: [
 *         {text: 'First Name',  dataIndex:'firstname'},
 *         {text: 'Last Name',  dataIndex:'lastname'},
 *         {
 *             xtype:'actioncolumn',
 *             width:50,
 *             items: [{
 *                 icon: 'extjs-build/examples/shared/icons/fam/cog_edit.png',  // Use a URL in the icon config
 *                 tooltip: 'Edit',
 *                 handler: function(grid, rowIndex, colIndex) {
 *                     var rec = grid.getStore().getAt(rowIndex);
 *                     alert("Edit " + rec.get('firstname'));
 *                 }
 *             },{
 *                 icon: 'extjs-build/examples/restful/images/delete.png',
 *                 tooltip: 'Delete',
 *                 handler: function(grid, rowIndex, colIndex) {
 *                     var rec = grid.getStore().getAt(rowIndex);
 *                     alert("Terminate " + rec.get('firstname'));
 *                 }
 *             }]
 *         }
 *     ],
 *     width: 250,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>The action column can be at any index in the columns array, and a grid can have any number of
 * action columns.</p>
 */
public class ActionColumn extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function ActionColumn(config:ext.grid.column.ActionColumn = null) {
    super();
  }

  [ExtConfig]
  /**
   * If true, the action will not respond to click events, and will be displayed semi-opaque.
   *
   * <p>This Column may also be disabled on a row by row basis by configuring a <a href="#!/api/Ext.grid.column.Action-cfg-isDisabled" rel="Ext.grid.column.Action-cfg-isDisabled" class="docClass">isDisabled</a> method.</p>
   * @see #setDisabled()
   */
  override public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * An array of action items copied from the configured <a href="#!/api/Ext.grid.column.Action-cfg-items" rel="Ext.grid.column.Action-cfg-items" class="docClass">items</a> configuration. Each will have
   * an <code>enable</code> and <code>disable</code> method added which will enable and disable the associated action, and
   * update the displayed icon accordingly.
   */
  override public native function get items():Array;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set items(value:Array):void;

  [ExtConfig]
  /**
   * Text to display in this column's menu item if no <a href="#!/api/Ext.grid.column.Action-cfg-text" rel="Ext.grid.column.Action-cfg-text" class="docClass">text</a> was specified as a header.
   * @default <i>Actions</i>
   */
  override public native function get menuText():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set menuText(value:String):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> reference) in which the <code><a href="#!/api/Ext.grid.column.Action-cfg-handler" rel="Ext.grid.column.Action-cfg-handler" class="docClass">handler</a></code>,
   * <code><a href="#!/api/Ext.grid.column.Action-cfg-getClass" rel="Ext.grid.column.Action-cfg-getClass" class="docClass">getClass</a></code>, <code><a href="#!/api/Ext.grid.column.Action-cfg-isDisabled" rel="Ext.grid.column.Action-cfg-isDisabled" class="docClass">isDisabled</a></code> and <code><a href="#!/api/Ext.grid.column.Action-cfg-getTip" rel="Ext.grid.column.Action-cfg-getTip" class="docClass">getTip</a></code> functions
   * are executed.
   * Defaults to this Column.
   */
  override public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * A tooltip message to be displayed on hover. <a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager</a> must
   * have been initialized.
   *
   * <p>The tooltip may also be determined on a row by row basis by configuring a <a href="#!/api/Ext.grid.column.Action-cfg-getTip" rel="Ext.grid.column.Action-cfg-getTip" class="docClass">getTip</a> method.</p>
   */
  override public native function get tooltip():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tooltip(value:String):void;

  [ExtConfig]
  /**
   * The alt text to use for the image element.
   * @default ''
   */
  public native function get altText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set altText(value:String):void;

  [ExtConfig]
  /**
   * A function which returns the CSS class to apply to the icon image.
   *
   * <p>For information on using the icons provided in the SDK see <a href="#!/api/Ext.grid.column.Action-cfg-iconCls" rel="Ext.grid.column.Action-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * <ul>
   *   <li><code>v</code> : Object
   * The value of the column's configured field (if any).
   *   </li>
   *   <li><code>metadata</code> : Object
   * An object in which you may set the following attributes:
   * <ul>
   *   <li><code>css</code> : String
   * A CSS class name to add to the cell's TD element.
   *   </li>
   *   <li><code>attr</code> : String
   * An HTML attribute definition string to apply to the data container
   * element <em>within</em> the table cell (e.g. 'style="color:red;"').
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>r</code> : ext.data.Model
   * The Record providing the data.
   *   </li>
   *   <li><code>rowIndex</code> : Number
   * The row index.
   *   </li>
   *   <li><code>colIndex</code> : Number
   * The column index.
   *   </li>
   *   <li><code>store</code> : ext.data.Store
   * The Store which is providing the data Model.
   *   </li>
   * </ul>
   */
  public native function get getClass():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set getClass(value:Function):void;

  [ExtConfig]
  /**
   * A function which returns the tooltip string for any row.
   *
   * <p><em>Note</em>: Outside of an Ext.application() use of this config requires
   * <a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a> to be called.</p>
   *
   * <pre><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     storeId: 'employeeStore',
   *     fields: ['firstname', 'grade'],
   *     data: [{
   *         firstname: "Michael",
   *         grade: 50
   *     }, {
   *         firstname: "Dwight",
   *         grade: 100
   *     }]
   * });
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title: 'Action Column Demo',
   *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('employeeStore'),
   *     columns: [{
   *         text: 'First Name',
   *         dataIndex: 'firstname'
   *     }, {
   *         text: 'Last Name',
   *         dataIndex: 'grade'
   *     }, {
   *         xtype: 'actioncolumn',
   *         width: 50,
   *         icon: 'sample/icons/action-icons.png',
   *         getTip: function(value, metadata, record, row, col, store) {
   *             var avg = store.average('grade'),
   *                 grade = record.get('grade');
   *
   *             if (grade &lt; avg) {
   *                 metadata.tdCls = "below-average";
   *             }
   *
   *             return grade &gt; 70 ? 'Pass' : 'Fail';
   *         },
   *         handler: function(grid, rowIndex, colIndex) {
   *             var rec = grid.getStore().getAt(rowIndex);
   *             alert("Edit " + rec.get('firstname'));
   *         }
   *     }],
   *     width: 250,
   *     renderTo: document.body
   * });
   * </code></pre>
   */
  public native function get getTip():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set getTip(value:Function):void;

  [ExtConfig]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @since 6.2.0
   */
  public native function get glyph():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  /**
   * A function called when the icon is clicked.
   * <ul>
   *   <li><code>view</code> : ext.view.TableView
   * The owning TableView.
   *   </li>
   *   <li><code>rowIndex</code> : Number
   * The row index clicked on.
   *   </li>
   *   <li><code>colIndex</code> : Number
   * The column index clicked on.
   *   </li>
   *   <li><code>item</code> : Object
   * The clicked item (or this Column if multiple <a href="#!/api/Ext.grid.column.Action-cfg-items" rel="Ext.grid.column.Action-cfg-items" class="docClass">items</a> were not configured).
   *   </li>
   *   <li><code>e</code> : js.Event
   * The click event.
   *   </li>
   *   <li><code>record</code> : ext.data.Model
   * The Record underlying the clicked row.
   *   </li>
   *   <li><code>row</code> : js.HTMLElement
   * The table row clicked upon.
   * &#64;controllable
   *   </li>
   * </ul>
   */
  public native function get handler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.grid.column.Action-cfg-iconCls" rel="Ext.grid.column.Action-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * @default Ext#BLANK_IMAGE_URL
   */
  public native function get icon():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   */
  public native function get iconCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig("isDisabled")]
  /**
   * A function which determines whether the action item for any row is disabled and returns <code>true</code> or <code>false</code>.
   * <ul>
   *   <li><code>view</code> : ext.view.TableView
   * The owning TableView.
   *   </li>
   *   <li><code>rowIndex</code> : Number
   * The row index.
   *   </li>
   *   <li><code>colIndex</code> : Number
   * The column index.
   *   </li>
   *   <li><code>item</code> : Object
   * The clicked item (or this Column if multiple <a href="#!/api/Ext.grid.column.Action-cfg-items" rel="Ext.grid.column.Action-cfg-items" class="docClass">items</a> were not configured).
   *   </li>
   *   <li><code>record</code> : ext.data.Model
   * The Record underlying the row.
   *   </li>
   * </ul>
   */
  public native function get isDisabled_():Function;

  [ExtConfig("isDisabled")]
  /**
   * @private
   */
  public native function set isDisabled_(value:Function):void;

  [ExtConfig]
  /**
   * Default ARIA role for each action item.
   * @default "button"
   */
  public native function get itemAriaRole():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemAriaRole(value:String):void;

  [ExtConfig]
  /**
   * Default tabIndex attribute value for each action item.
   * @default 0
   */
  public native function get itemTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Prevent grid selection upon click.
   * Beware that if you allow for the selection to happen then the selection model will steal focus from
   * any possible floating window (like a message box) raised in the handler. This will prevent closing the
   * window when pressing the Escape button since it will no longer contain a focused component.
   * @default true
   */
  public native function get stopSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stopSelection(value:Boolean):void;

  /**
   * Disables this ActionColumn's action at the specified index.
   * @param silent
   * @default false
   */
  public native function disableAction(index:*, silent:Boolean = false):void;

  /**
   * Enables this ActionColumn's action at the specified index.
   * @param silent
   * @default false
   */
  public native function enableAction(index:*, silent:Boolean = false):void;
}
}