package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Boolean", require)]
/**
 * A Column definition class which renders boolean data fields.  See the <a href="#!/api/Ext.grid.column.Column-cfg-xtype" rel="Ext.grid.column.Column-cfg-xtype" class="docClass">xtype</a>
 * config option of <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">Ext.grid.column.Column</a> for more details.
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *    fields: [
 *        {name: 'framework', type: 'string'},
 *        {name: 'rocks', type: 'boolean'}
 *    ],
 *    data: [
 *        { framework: 'Ext JS 5', rocks: true },
 *        { framework: 'Ext GWT', rocks: true },
 *        { framework: 'Other Guys', rocks: false }
 *    ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Boolean Column Demo',
 *     store: store,
 *     columns: [
 *         { text: 'Framework',  dataIndex: 'framework', flex: 1 },
 *         {
 *             xtype: 'booleancolumn',
 *             text: 'Rocks',
 *             trueText: 'Yes',
 *             falseText: 'No',
 *             dataIndex: 'rocks'
 *         }
 *     ],
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class BooleanColumn extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function BooleanColumn(config:ext.grid.column.BooleanColumn = null) {
    super();
  }

  [ExtConfig]
  /**
   * This flag indicates that the renderer produces HTML.
   *
   * <p>If this column is going to be updated rapidly, and the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">Ext.grid.column.Column.renderer</a> or <a href="#!/api/Ext.grid.column.Boolean-cfg-updater" rel="Ext.grid.column.Boolean-cfg-updater" class="docClass">updater</a> only produces
   * text, then to avoid the expense of HTML parsing and element production during the
   * update, this property may be configured as <code>false</code>.</p>
   * @default false
   */
  override public native function get producesHTML():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set producesHTML(value:Boolean):void;

  [ExtConfig]
  /**
   * The string returned by the renderer when the column value is falsey (but not undefined).
   * @default 'false'
   */
  public native function get falseText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set falseText(value:String):void;

  [ExtConfig]
  /**
   * The string returned by the renderer when the column value is not falsey.
   * @default 'true'
   */
  public native function get trueText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set trueText(value:String):void;

  [ExtConfig]
  /**
   * The string returned by the renderer when the column value is undefined.
   * @default '&amp;#160;'
   */
  public native function get undefinedText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set undefinedText(value:String):void;
}
}