package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Template", require)]
/**
 * A Column definition class which renders a value by processing a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>'s
 * <a href="#!/api/Ext.data.Model-method-getData" rel="Ext.data.Model-method-getData" class="docClass">data</a> using a <a href="#!/api/Ext.grid.column.Template-cfg-tpl" rel="Ext.grid.column.Template-cfg-tpl" class="docClass">configured</a>
 * <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a>.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId:'employeeStore',
 *     fields:['firstname', 'lastname', 'seniority', 'department'],
 *     groupField: 'department',
 *     data:[
 *         { firstname: "Michael", lastname: "Scott",   seniority: 7, department: "Management" },
 *         { firstname: "Dwight",  lastname: "Schrute", seniority: 2, department: "Sales" },
 *         { firstname: "Jim",     lastname: "Halpert", seniority: 3, department: "Sales" },
 *         { firstname: "Kevin",   lastname: "Malone",  seniority: 4, department: "Accounting" },
 *         { firstname: "Angela",  lastname: "Martin",  seniority: 5, department: "Accounting" }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Column Template Demo',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('employeeStore'),
 *     columns: [
 *         { text: 'Full Name',       xtype: 'templatecolumn', tpl: '{firstname} {lastname}', flex:1 },
 *         { text: 'Department (Yrs)', xtype: 'templatecolumn', tpl: '{department} ({seniority})' }
 *     ],
 *     height: 200,
 *     width: 300,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class TemplateColumn extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function TemplateColumn(config:ext.grid.column.TemplateColumn = null) {
    super();
  }

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a>, or an XTemplate <em>definition string</em> to use to process a
   * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>'s data object to produce a cell's rendered value.
   */
  override public native function get tpl():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tpl(value:*):void;
}
}