package ext.grid.feature {
import ext.data.Model;

[Uses("ext.grid.feature.Feature")]
[Uses("ext.grid.feature.AbstractSummary")]
[Uses("ext.util.Observable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.grid.feature.Grouping", require)]
/**
 * @eventType ext.grid.feature.events.Grouping_view_node_group_eEvent.GROUP_CLICK
 */
[Event(name = 'onGroupClick', type = 'ext.grid.feature.events.Grouping_view_node_group_eEvent')]
/**
 * @eventType ext.grid.feature.events.Grouping_view_node_groupEvent.GROUP_COLLAPSE
 */
[Event(name = 'onGroupCollapse', type = 'ext.grid.feature.events.Grouping_view_node_groupEvent')]
/**
 * @eventType ext.grid.feature.events.Grouping_view_node_group_eEvent.GROUP_CONTEXT_MENU
 */
[Event(name = 'onGroupContextMenu', type = 'ext.grid.feature.events.Grouping_view_node_group_eEvent')]
/**
 * @eventType ext.grid.feature.events.Grouping_view_node_group_eEvent.GROUP_DBLCLICK
 */
[Event(name = 'onGroupDblclick', type = 'ext.grid.feature.events.Grouping_view_node_group_eEvent')]
/**
 * @eventType ext.grid.feature.events.Grouping_view_node_groupEvent.GROUP_EXPAND
 */
[Event(name = 'onGroupExpand', type = 'ext.grid.feature.events.Grouping_view_node_groupEvent')]
/**
 * This feature allows to display the grid rows aggregated into groups as specified by the <a href="#!/api/Ext.data.Store-cfg-grouper" rel="Ext.data.Store-cfg-grouper" class="docClass">grouper</a>
 *
 * <p>underneath. The groups can also be expanded and collapsed.</p>
 *
 * <h2>Extra Events</h2>
 *
 * <p>This feature adds several extra events that will be fired on the grid to interact with the groups:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.grid.feature.Grouping-event-groupclick" rel="Ext.grid.feature.Grouping-event-groupclick" class="docClass">groupclick</a></li>
 * <li><a href="#!/api/Ext.grid.feature.Grouping-event-groupdblclick" rel="Ext.grid.feature.Grouping-event-groupdblclick" class="docClass">groupdblclick</a></li>
 * <li><a href="#!/api/Ext.grid.feature.Grouping-event-groupcontextmenu" rel="Ext.grid.feature.Grouping-event-groupcontextmenu" class="docClass">groupcontextmenu</a></li>
 * <li><a href="#!/api/Ext.grid.feature.Grouping-event-groupexpand" rel="Ext.grid.feature.Grouping-event-groupexpand" class="docClass">groupexpand</a></li>
 * <li><a href="#!/api/Ext.grid.feature.Grouping-event-groupcollapse" rel="Ext.grid.feature.Grouping-event-groupcollapse" class="docClass">groupcollapse</a></li>
 * </ul>
 *
 *
 * <h2>Menu Augmentation</h2>
 *
 * <p>This feature adds extra options to the grid column menu to provide the user with functionality to modify the grouping.
 * This can be disabled by setting the <a href="#!/api/Ext.grid.feature.Grouping-cfg-enableGroupingMenu" rel="Ext.grid.feature.Grouping-cfg-enableGroupingMenu" class="docClass">enableGroupingMenu</a> option. The option to disallow grouping from being turned off
 * by the user is <a href="#!/api/Ext.grid.feature.Grouping-cfg-enableNoGroups" rel="Ext.grid.feature.Grouping-cfg-enableNoGroups" class="docClass">enableNoGroups</a>.</p>
 *
 * <h2>Controlling Group Text</h2>
 *
 * <p>The <a href="#!/api/Ext.grid.feature.Grouping-cfg-groupHeaderTpl" rel="Ext.grid.feature.Grouping-cfg-groupHeaderTpl" class="docClass">groupHeaderTpl</a> is used to control the rendered title for each group. It can modified to customized
 * the default display.</p>
 *
 * <h2>Groupers</h2>
 *
 * <p>By default, this feature expects that the data field that is mapped to by the
 * <a href="#!/api/Ext.data.AbstractStore-cfg-groupField" rel="Ext.data.AbstractStore-cfg-groupField" class="docClass">Ext.data.AbstractStore.groupField</a> config is a simple data type such as a
 * String or a Boolean. However, if you intend to group by a data field that is a
 * complex data type such as an Object or Array, it is necessary to define one or more
 * <a href="#!/api/Ext.util.Grouper" rel="Ext.util.Grouper" class="docClass">groupers</a> on the feature that it can then use to lookup
 * internal group information when grouping by different fields.</p>
 *
 * <pre class='inline-example '><code>var feature = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.feature.Grouping" rel="Ext.grid.feature.Grouping" class="docClass">Ext.grid.feature.Grouping</a>', {
 *     startCollapsed: true,
 *     groupers: [{
 *         property: 'asset',
 *         groupFn: function (val) {
 *             return val.data.name;
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <h2>Example Usage</h2>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name', 'seniority', 'department'],
 *     groupField: 'department',
 *     data: [
 *         { name: 'Michael Scott', seniority: 7, department: 'Management' },
 *         { name: 'Dwight Schrute', seniority: 2, department: 'Sales' },
 *         { name: 'Jim Halpert', seniority: 3, department: 'Sales' },
 *         { name: 'Kevin Malone', seniority: 4, department: 'Accounting' },
 *         { name: 'Angela Martin', seniority: 5, department: 'Accounting' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Employees',
 *     store: store,
 *     columns: [
 *         { text: 'Name', dataIndex: 'name' },
 *         { text: 'Seniority', dataIndex: 'seniority' }
 *     ],
 *     features: [{ftype:'grouping'}],
 *     width: 200,
 *     height: 275,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p><strong>Note:</strong> To use grouping with a grid that has <a href="#!/api/Ext.grid.column.Column-cfg-locked" rel="Ext.grid.column.Column-cfg-locked" class="docClass">locked columns</a>, you need to supply
 * the grouping feature as a config object - so the grid can create two instances of the grouping feature.</p>
 */
public class Grouping extends ext.grid.feature.Feature implements ext.grid.feature.IAbstractSummary {
  /**
   * @param config @inheritDoc
   */
  public function Grouping(config:ext.grid.feature.Grouping = null) {
    super();
  }

  [ExtConfig]
  /**
   * Set to <code>false</code> to disable collapsing groups from the UI.
   *
   * <p>This is set to <code>false</code> when the associated <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">store</a> is
   * a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">BufferedStore</a>.</p>
   * @default true
   */
  public native function get collapsible():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapsible(value:Boolean):void;

  [ExtConfig]
  /**
   * Number of pixels to indent per grouping level
   * @default 17
   */
  public native function get depthToIndent():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set depthToIndent(value:Number):void;

  [ExtConfig]
  /**
   * True to enable the grouping control in the header menu.
   * @default true
   */
  public native function get enableGroupingMenu():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableGroupingMenu(value:Boolean):void;

  [ExtConfig]
  /**
   * True to allow the user to turn off grouping.
   * @default true
   */
  public native function get enableNoGroups():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableNoGroups(value:Boolean):void;

  [ExtConfig]
  /**
   * Text displayed in the grid header menu for grouping by header.
   * @default "Group by this field"
   */
  public native function get groupByText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set groupByText(value:String):void;

  [ExtConfig]
  /**
   * A string Template snippet, an array of strings (optionally followed by an object containing Template methods) to be used to construct a Template, or a Template instance.
   *
   * <ul>
   * <li><p>Example 1 (Template snippet):</p>
   *
   * <pre><code>groupHeaderTpl: 'Group: {name}'
   * </code></pre></li>
   * <li><p>Example 2 (Array):</p>
   *
   * <pre><code>groupHeaderTpl: [
   *     'Group: ',
   *     '&lt;div&gt;{name:this.formatName}&lt;/div&gt;',
   *     {
   *         formatName: function(name) {
   *             return <a href="#!/api/Ext.String-method-trim" rel="Ext.String-method-trim" class="docClass">Ext.String.trim</a>(name);
   *         }
   *     }
   * ]
   * </code></pre></li>
   * <li><p>Example 3 (Template Instance):</p>
   *
   * <pre><code>groupHeaderTpl: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>',
   *     'Group: ',
   *     '&lt;div&gt;{name:this.formatName}&lt;/div&gt;',
   *     {
   *         formatName: function(name) {
   *             return <a href="#!/api/Ext.String-method-trim" rel="Ext.String-method-trim" class="docClass">Ext.String.trim</a>(name);
   *         }
   *     }
   * )
   * </code></pre></li>
   * </ul>
   * @default '{columnName}: {name}'
   * <ul>
   *   <li><code>groupField</code> : String
   * The field name being grouped by.
   *   </li>
   *   <li><code>columnName</code> : String
   * The column header associated with the field being grouped by <em>if there is a column for the field</em>, falls back to the groupField name.
   *   </li>
   *   <li><code>groupValue</code>
   * The value of the <a href="#!/api/Ext.data.Store-cfg-groupField" rel="Ext.data.Store-cfg-groupField" class="docClass">groupField</a> for the group header being rendered.
   *   </li>
   *   <li><code>renderedGroupValue</code> : String
   * The rendered value of the <a href="#!/api/Ext.data.Store-cfg-groupField" rel="Ext.data.Store-cfg-groupField" class="docClass">groupField</a> for the group header being rendered, as produced by the column renderer.
   *   </li>
   *   <li><code>name</code> : String
   * An alias for renderedGroupValue
   *   </li>
   *   <li><code>rows</code> : Array
   * Deprecated - use children instead. An array containing the child records for the group being rendered. <em>Not available if the store is a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">BufferedStore</a></em>
   *   </li>
   *   <li><code>children</code> : Array
   * An array containing the child records for the group being rendered. <em>Not available if the store is a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">BufferedStore</a></em>
   *   </li>
   * </ul>
   */
  public native function get groupHeaderTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set groupHeaderTpl(value:*):void;

  [ExtConfig]
  /**
   * These are grouper objects defined for the feature. If the group names are derived
   * from complex data types, it is necessary to convert them as a store would.
   *
   * <p>However, since only one grouper can be defined on the store at a time and
   * this feature clears the current grouper when a new one is added, it is
   * necessary to define a cache of groupers that the feature can lookup as needed.</p>
   *
   * <p>Expected grouper object properties are <code>property</code> and <code>groupFn</code>.</p>
   * @default null
   */
  public native function get groupers():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set groupers(value:Array):void;

  [ExtConfig]
  /**
   * True to hide the header that is currently grouped.
   * @default false
   */
  public native function get hideGroupedHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideGroupedHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * The name of the property which contains the Array of summary objects.
   * It allows to use server-side calculated summaries.
   * @default undefined
   */
  public native function get remoteRoot():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set remoteRoot(value:String):void;

  [ExtConfig]
  /**
   * Text displayed in the grid header for enabling/disabling grouping.
   * @default "Show in groups"
   */
  public native function get showGroupsText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showGroupsText(value:String):void;

  [ExtConfig]
  /**
   * True to start all groups collapsed.
   * @default false
   */
  public native function get startCollapsed():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set startCollapsed(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get showSummaryRow():Boolean;

  /**
   * @inheritDoc
   */
  public native function set showSummaryRow(value:Boolean):void;

  /**
   * Collapse a group
   * @param groupName The group name.
   * @param options . Pass when the group should be scrolled into view.
   * This contains flags for postprocessing the group's header row after
   * collapsing. See <a href="#!/api/Ext.panel.Table-method-ensureVisible" rel="Ext.panel.Table-method-ensureVisible" class="docClass">Ext.panel.Table.ensureVisible</a> for details.
   */
  public native function collapse(groupName:String, options:Object = null):void;

  /**
   * Collapse all groups
   */
  public native function collapseAll():void;

  /**
   * Expand a group
   * @param groupName The group name.
   * @param options . Pass when the group should be scrolled into view.
   * This contains flags for postprocessing the group's first row after
   * expansion. See <a href="#!/api/Ext.panel.Table-method-ensureVisible" rel="Ext.panel.Table-method-ensureVisible" class="docClass">Ext.panel.Table.ensureVisible</a> for details. <em>note:</em>
   * a boolean may be passed to indicate whether to focus the target group after expand.
   */
  public native function expand(groupName:String, options:Object = null):void;

  /**
   * Expand all groups
   */
  public native function expandAll():void;

  /**
   * Returns the group data object for the group to which the passed record belongs <strong>if the Store is grouped</strong>.
   * @param record The record for which to return group information.
   * @return A single group data block as returned from <a href="#!/api/Ext.data.Store-method-getGroups" rel="Ext.data.Store-method-getGroups" class="docClass">Store.getGroups</a>. Returns
   * <code>undefined</code> if the Store is not grouped.
   */
  public native function getRecordGroup(record:ext.data.Model):Object;

  /**
   * Returns <code>true</code> if the named group is expanded.
   * @param groupName The group name. This is the value of
   * the <a href="#!/api/Ext.data.Store-cfg-groupField" rel="Ext.data.Store-cfg-groupField" class="docClass">groupField</a>.
   * @return <code>true</code> if the group defined by that value is expanded.
   */
  public native function isExpanded(groupName:String):Boolean;

  /**
   * Toggle whether or not to show the summary row.
   * @param visible True to show the summary row
   */
  public native function toggleSummaryRow(visible:Boolean):void;
}
}