package ext.grid.filters.filter {

[Uses("ext.grid.filters.filter.SingleFilter")]
[Uses("ext.grid.filters.filter.BaseFilter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.grid.filters.filter.Boolean", require)]
/**
 * The boolean grid filter allows you to create a filter selection that limits results
 * to values matching true or false.  The filter can be set programmatically or via
 * user input with a configurable <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">radio field</a> in the filter section
 * of the column header.
 *
 * <p>Boolean filters use unique radio group IDs, so you may utilize more than one.</p>
 *
 * <p>Example Boolean Filter Usage:</p>
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['id','show', 'visible'],
 *     data: [
 *         {id: 0, show: 'Battlestar Galactica', visible: true},
 *         {id: 1, show: 'Doctor Who', visible: true},
 *         {id: 2, show: 'Farscape', visible: false},
 *         {id: 3, show: 'Firefly', visible: true},
 *         {id: 4, show: 'Star Trek', visible: true},
 *         {id: 5, show: 'Star Wars: Christmas Special', visible: false}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     title: 'Sci-Fi Television',
 *     height: 250,
 *     width: 375,
 *     store: shows,
 *     plugins: 'gridfilters',
 *     columns: [{
 *         dataIndex: 'id',
 *         text: 'ID',
 *         width: 50
 *     },{
 *         dataIndex: 'show',
 *         text: 'Show',
 *         flex: 1
 *     },{
 *         dataIndex: 'visible',
 *         text: 'Visibility',
 *         width: 125,
 *         filter: {
 *             type: 'boolean',
 *             value: 'true',
 *             yesText: 'True',
 *             noText: 'False'
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class BooleanFilter extends ext.grid.filters.filter.SingleFilter {
  /**
   * @param config @inheritDoc
   */
  public function BooleanFilter(config:ext.grid.filters.filter.BooleanFilter = null) {
    super();
  }

  [ExtConfig]
  /**
   * Set this to null if you do not want either option to be checked by default. Defaults to false.
   * @default false
   */
  public native function get defaultValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultValue(value:Boolean):void;

  [ExtConfig]
  /**
   * Defaults to 'No'.
   * @default 'No'
   */
  public native function get noText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set noText(value:String):void;

  [ExtConfig]
  /**
   * Defaults to 'Yes'.
   * @default 'Yes'
   */
  public native function get yesText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set yesText(value:String):void;
}
}