package ext.grid.filters.filter {

[Uses("ext.grid.filters.filter.SingleFilter")]
[Uses("ext.grid.filters.filter.BaseFilter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.grid.filters.filter.String", require)]
/**
 * The string grid filter allows you to create a filter selection that limits results
 * to values matching a particular string.  The filter can be set programmatically or via
 * user input with a configurable <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">text field</a> in the filter section
 * of the column header.
 *
 * <p>Example String Filter Usage:</p>
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['id','show'],
 *     data: [
 *         {id: 0, show: 'Battlestar Galactica'},
 *         {id: 1, show: 'Doctor Who'},
 *         {id: 2, show: 'Farscape'},
 *         {id: 3, show: 'Firefly'},
 *         {id: 4, show: 'Star Trek'},
 *         {id: 5, show: 'Star Wars: Christmas Special'}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     title: 'Sci-Fi Television',
 *     height: 250,
 *     width: 250,
 *     store: shows,
 *     plugins: 'gridfilters',
 *     columns: [{
 *         dataIndex: 'id',
 *         text: 'ID',
 *         width: 50
 *     },{
 *         dataIndex: 'show',
 *         text: 'Show',
 *         flex: 1,
 *         filter: {
 *             // required configs
 *             type: 'string',
 *             // optional configs
 *             value: 'star',  // setting a value makes the filter active.
 *             itemDefaults: {
 *                 // any <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a> configs accepted
 *             }
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class StringFilter extends ext.grid.filters.filter.SingleFilter {
  /**
   * @param config @inheritDoc
   */
  public function StringFilter(config:ext.grid.filters.filter.StringFilter = null) {
    super();
  }

  [ExtConfig]
  /**
   * The empty text to show for each field.
   * @default 'Enter Filter Text...'
   */
  public native function get emptyText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set emptyText(value:String):void;
}
}