package ext.grid.header {
import ext.container.Container;
import ext.grid.column.Column;
import ext.util.IFocusableContainer;

[Uses("ext.container.Container")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.grid.header.Container", require)]
/**
 * @eventType ext.grid.header.events.HeaderContainer_columnEvent.COLUMN_HIDE
 */
[Event(name = 'onColumnHide', type = 'ext.grid.header.events.HeaderContainer_columnEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_column_fromIdx_toIdxEvent.COLUMN_MOVE
 */
[Event(name = 'onColumnMove', type = 'ext.grid.header.events.HeaderContainer_column_fromIdx_toIdxEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_column_widthEvent.COLUMN_RESIZE
 */
[Event(name = 'onColumnResize', type = 'ext.grid.header.events.HeaderContainer_column_widthEvent')]
/**
 * Fired after the columns change in any way, when a column has been hidden or shown, or when a column
 * is added to or removed from this header container.
 * @eventType ext.grid.header.events.HeaderContainerEvent.COLUMNS_CHANGED
 */
[Event(name = 'onColumnsChanged', type = 'ext.grid.header.events.HeaderContainerEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_columnEvent.COLUMNS_HOW
 */
[Event(name = 'onColumnsHow', type = 'ext.grid.header.events.HeaderContainer_columnEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_column_e_tEvent.HEADER_CLICK
 */
[Event(name = 'onHeaderClick', type = 'ext.grid.header.events.HeaderContainer_column_e_tEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_column_e_tEvent.HEADER_CONTEXT_MENU
 */
[Event(name = 'onHeaderContextMenu', type = 'ext.grid.header.events.HeaderContainer_column_e_tEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_column_e_tEvent.HEADER_TRIGGER_CLICK
 */
[Event(name = 'onHeaderTriggerClick', type = 'ext.grid.header.events.HeaderContainer_column_e_tEvent')]
/**
 * Fired immediately after the column header menu is created.
 * @eventType ext.grid.header.events.HeaderContainer_menuEvent.MENU_CREATE
 */
[Event(name = 'onMenuCreate', type = 'ext.grid.header.events.HeaderContainer_menuEvent')]
/**
 * @eventType ext.grid.header.events.HeaderContainer_column_directionEvent.SORT_CHANGE
 */
[Event(name = 'onSortChange', type = 'ext.grid.header.events.HeaderContainer_column_directionEvent')]
/**
 * Headercontainer is a docked container (<em><code>top</code> or <code>bottom</code> only</em>) that holds the
 * headers (<a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">grid columns</a>) of a
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">grid</a> or <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">tree</a>.  The headercontainer
 * handles resizing, moving, and hiding columns.  As columns are hidden, moved or
 * resized, the headercontainer triggers changes within the grid or tree's
 * <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">view</a>.  You will not generally need to instantiate this class
 * directly.
 *
 * <p>You may use the
 * <a href="#!/api/Ext.panel.Table-method-getHeaderContainer" rel="Ext.panel.Table-method-getHeaderContainer" class="docClass">getHeaderContainer()</a>
 * accessor method to access the tree or grid's headercontainer.</p>
 *
 * <p>Grids and trees also have an alias to the two more useful headercontainer methods:</p>
 *
 * <ul>
 * <li><strong><a href="#!/api/Ext.panel.Table-method-getColumns" rel="Ext.panel.Table-method-getColumns" class="docClass">getColumns</a></strong> - aliases
 * <a href="#!/api/Ext.grid.header.Container-method-getGridColumns" rel="Ext.grid.header.Container-method-getGridColumns" class="docClass">getGridColumns</a></li>
 * <li><strong><a href="#!/api/Ext.panel.Table-method-getVisibleColumns" rel="Ext.panel.Table-method-getVisibleColumns" class="docClass">getVisibleColumns</a></strong> - aliases
 * <a href="#!/api/Ext.grid.header.Container-method-getVisibleGridColumns" rel="Ext.grid.header.Container-method-getVisibleGridColumns" class="docClass">getVisibleGridColumns</a></li>
 * </ul>
 */
public class HeaderContainer extends ext.container.Container implements ext.util.IFocusableContainer {
  /**
   * @param config @inheritDoc
   */
  public function HeaderContainer(config:ext.grid.header.HeaderContainer = null) {
    super();
  }

  [ExtConfig]
  /**
   * HeaderContainer overrides the default weight of 0 for all docked items to 100.
   * This is so that it has more priority over things like toolbars.
   * @default 100
   * @see #setWeight()
   */
  override public native function get weight():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set weight(value:Number):void;

  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  public native function get focusableContainerEl():*;

  /**
   * @private
   */
  public native function set focusableContainerEl(value:*):void;

  /**
   * True if this HeaderContainer is in fact a group header which contains sub headers.
   */
  public native function get isGroupHeader():Boolean;

  /**
   * @private
   */
  public native function set isGroupHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  /**
   * Width of the header if no width or flex is specified.
   * @default 100
   */
  public native function get defaultWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultWidth(value:Number):void;

  [ExtConfig]
  /**
   * False to disable column hiding within this grid.
   * @default true
   */
  public native function get enableColumnHide():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableColumnHide(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to constrain column dragging so that a column cannot be dragged into or out of this column.
   *
   * <p><strong>Note that this config is only valid for column headers which contain child column headers, eg:</strong>
   *     {
   *         sealed: true
   *         text: 'ExtJS',
   *         columns: [{
   *             text: '3.0.4',
   *             dataIndex: 'ext304'
   *         }, {
   *             text: '4.1.0',
   *             dataIndex: 'ext410'
   *         }
   *     }</p>
   * @default false
   */
  public native function get sealed():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set sealed(value:Boolean):void;

  [ExtConfig]
  /**
   * Provides the default sortable state for all Headers within this HeaderContainer.
   * Also turns on or off the menus in the HeaderContainer. Note that the menu is
   * shared across every header and therefore turning it off will remove the menu
   * items for every header.
   * @default true
   */
  public native function get sortable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set sortable(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get enableFocusableContainer():Boolean;

  /**
   * @inheritDoc
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  /**
   * Returns the number of <b>grid columns</b> descended from this HeaderContainer.
   * Group Columns are HeaderContainers. All grid columns are returned, including hidden ones.
   */
  public native function getColumnCount():uint;

  /**
   * Returns an array of menu CheckItems corresponding to all immediate children
   * of the passed Container which have been configured as hideable.
   */
  public native function getColumnMenu(headerContainer:Object):Array;

  /**
   * Returns an array of all columns which exist in the grid's View, visible or not.
   * This goes down to the leaf column header level, and does not return <strong>grouped</strong>
   * headers which contain sub headers.
   *
   * <p>It includes hidden headers even though they are not rendered. This is for
   * collection of menu items for the column hide/show menu.</p>
   *
   * <p>Headers which have a hidden ancestor have a <code>hiddenAncestor: true</code> property
   * injected so that descendants are known to be hidden without interrogating that
   * header's ownerCt axis for a hidden ancestor.</p>
   *
   * <p>See also <a href="#!/api/Ext.grid.header.Container-method-getVisibleGridColumns" rel="Ext.grid.header.Container-method-getVisibleGridColumns" class="docClass">getVisibleGridColumns</a></p>
   * @return columns An array of columns.  Returns an
   * empty array if no columns are found.
   */
  public native function getGridColumns():Array;

  /**
   * Get a leaf level header by index regardless of what the nesting
   * structure is.
   * @param index The column index for which to retrieve the column.
   */
  public native function getHeaderAtIndex(index:Number):ext.grid.column.Column;

  /**
   * Returns the index of a leaf level header regardless of what the nesting
   * structure is.
   *
   * <p>If a group header is passed, the index of the first leaf level header within it is returned.</p>
   * @param header The header to find the index of
   * @return The index of the specified column header
   */
  public native function getHeaderIndex(header:ext.grid.column.Column):Number;

  /**
   * Returns an array of menu items to be placed into the shared menu
   * across all headers in this header container.
   * @return menuItems
   */
  public native function getMenuItems():Array;

  /**
   * Gets the full width of all columns that are visible for setting width of tables.
   */
  public native function getTableWidth():uint;

  /**
   * Returns an array of the <strong>visible</strong> columns in the grid. This goes down to the
   * lowest column header level, and does not return <strong>grouped</strong> headers which contain
   * sub headers.
   *
   * <p>See also <a href="#!/api/Ext.grid.header.Container-method-getGridColumns" rel="Ext.grid.header.Container-method-getGridColumns" class="docClass">getGridColumns</a></p>
   * @return columns An array of visible columns.  Returns
   * an empty array if no visible columns are found.
   */
  public native function getVisibleGridColumns():Array;

  /**
   * When passed a column index, returns the closet <em>visible</em> column to that. If the column at the passed index is visible,
   * that is returned. If it is hidden, either the next visible, or the previous visible column is returned.
   * @param index Position at which to find the closest visible column.
   */
  public native function getVisibleHeaderClosestToIndex(index:Number):ext.grid.column.Column;
}
}