package ext.grid.plugin {
import ext.grid.ViewDropZone;
import ext.plugin.AbstractPlugin;
import ext.view.ViewDragZone;

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Base")]
[Native("Ext.grid.plugin.DragDrop", require)]
/**
 * <strong>This event is fired through the <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a> and its owning
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Grid</a>. You can add listeners to the grid or grid <a href="#!/api/Ext.grid.Panel-cfg-viewConfig" rel="Ext.grid.Panel-cfg-viewConfig" class="docClass">view config</a> object</strong>
 *
 * <p>Fired when a drop gesture has been triggered by a mouseup event in a valid drop
 * position in the grid view.</p>
 *
 * <p>Returning <code>false</code> to this event signals that the drop gesture was invalid and
 * animates the drag proxy back to the point from which the drag began.</p>
 *
 * <p>The dropHandlers parameter can be used to defer the processing of this event. For
 * example, you can force the handler to wait for the result of a message box
 * confirmation or an asynchronous server call (<em>see the details of the dropHandlers
 * property for more information</em>).</p>
 *
 * <pre><code>grid.on('beforedrop', function(node, data, overModel, dropPosition, dropHandlers) {
 *     // Defer the handling
 *     dropHandlers.wait = true;
 *     <a href="#!/api/Ext.MessageBox-method-confirm" rel="Ext.MessageBox-method-confirm" class="docClass">Ext.MessageBox.confirm</a>('Drop', 'Are you sure', function(btn){
 *         if (btn === 'yes') {
 *             dropHandlers.processDrop();
 *         } else {
 *             dropHandlers.cancelDrop();
 *         }
 *     });
 * });
 * </code></pre>
 *
 * <p>Any other return value continues with the data transfer operation unless the wait
 * property is set.</p>
 * @eventType ext.grid.plugin.events.GridViewDragDropPlugin_node_data_overModel_dropPosition_dropHandlersEvent.BEFORE_DROP
 */
[Event(name = 'onBeforeDrop', type = 'ext.grid.plugin.events.GridViewDragDropPlugin_node_data_overModel_dropPosition_dropHandlersEvent')]
/**
 * <strong>This event is fired through the <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a> and its owning
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Grid</a>. You can add listeners to the grid or grid <a href="#!/api/Ext.grid.Panel-cfg-viewConfig" rel="Ext.grid.Panel-cfg-viewConfig" class="docClass">view config</a> object</strong>
 *
 * <p>Fired when a drop operation has been completed and the data has been moved or
 * copied.</p>
 * @eventType ext.grid.plugin.events.GridViewDragDropPlugin_node_data_overModel_dropPositionEvent.DROP
 */
[Event(name = 'onDrop', type = 'ext.grid.plugin.events.GridViewDragDropPlugin_node_data_overModel_dropPositionEvent')]
/**
 * This plugin provides drag and drop functionality for a <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a>.
 *
 * <p>A specialized instance of <a href="#!/api/Ext.dd.DragZone" rel="Ext.dd.DragZone" class="docClass">DragZone</a> and <a href="#!/api/Ext.dd.DropZone" rel="Ext.dd.DropZone" class="docClass">DropZone</a> are attached to the grid view.  The DropZone will participate in drops
 * from DragZones having the same <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup" rel="Ext.grid.plugin.DragDrop-cfg-ddGroup" class="docClass">ddGroup</a> including drops from within the same
 * grid.</p>
 *
 * <p>Note that where touch gestures are available, the <code>longpress</code> gesture will initiate
 * the drag in order that the <code>touchstart</code> may still be used to initiate a scroll.</p>
 *
 * <p>On platforms which implement the <a href="https://www.w3.org/TR/pointerevents/">Pointer Events standard</a> (IE),
 * the <code>touchstart</code> event is usually claimed by the platform, however, this plugin
 * uses the <code>longpress</code> event to trigger drags, so <code>touchstart</code> will not initiate a scroll.
 * On these platforms, a two finger drag gesture will scroll the content, or a single
 * finger drag on an empty area of the view will scroll the content.</p>
 *
 * <p>During the drop operation a data object is passed to a participating DropZone's drop
 * handlers.  The drag data object has the following properties:</p>
 *
 * <ul>
 * <li><p><strong>copy:</strong> Boolean <br> The value of <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-copy" rel="Ext.grid.plugin.DragDrop-cfg-copy" class="docClass">copy</a>.  Or <code>true</code> if
 * <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-allowCopy" rel="Ext.grid.plugin.DragDrop-cfg-allowCopy" class="docClass">allowCopy</a> is true <strong>and</strong> the control key was pressed as the drag operation
 * began.</p></li>
 * <li><p><strong>view:</strong> <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a> <br> The source grid view from which the
 * drag originated</p></li>
 * <li><p><strong>ddel:</strong> HTMLElement <br> The drag proxy element which moves with the cursor</p></li>
 * <li><p><strong>item:</strong> HTMLElement <br> The grid view node upon which the mousedown event was
 * registered</p></li>
 * <li><p><strong>records:</strong> Array <br> An Array of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>s
 * representing the selected data being dragged from the source grid view</p></li>
 * </ul>
 *
 *
 * <p>By adding this plugin to a view, two new events will be fired from the client
 * grid view as well as its owning Grid: <code><a href="#!/api/Ext.grid.plugin.DragDrop-event-beforedrop" rel="Ext.grid.plugin.DragDrop-event-beforedrop" class="docClass">beforedrop</a></code> and <code><a href="#!/api/Ext.grid.plugin.DragDrop-event-drop" rel="Ext.grid.plugin.DragDrop-event-drop" class="docClass">drop</a></code>.</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name'],
 *     data: [
 *         ["Lisa"],
 *         ["Bart"],
 *         ["Homer"],
 *         ["Marge"]
 *     ],
 *     proxy: {
 *         type: 'memory',
 *         reader: 'array'
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     store: store,
 *     enableLocking: true,
 *     columns: [{
 *         header: 'Name',
 *         dataIndex: 'name',
 *         flex: true
 *     }],
 *     viewConfig: {
 *         plugins: {
 *             ptype: 'gridviewdragdrop',
 *             dragText: 'Drag and drop to reorganize'
 *         }
 *     },
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class GridViewDragDropPlugin extends ext.plugin.AbstractPlugin {
  /**
   * @param config @inheritDoc
   */
  public function GridViewDragDropPlugin(config:ext.grid.plugin.GridViewDragDropPlugin = null) {
    super();
  }

  [ExtConfig]
  /**
   * Set as <code>true</code> to allow the user to hold down the control key at the start of the
   * drag operation and copy the dragged records between grids.  Otherwise, dragged
   * records will be moved.
   *
   * <p><strong>Note:</strong> This only applies to records dragged between two different grids with
   * unique stores.</p>
   *
   * <p>See <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-copy" rel="Ext.grid.plugin.DragDrop-cfg-copy" class="docClass">copy</a> to enable the copying of all dragged records.</p>
   * @default false
   */
  public native function get allowCopy():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowCopy(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to register this container with the Scrollmanager for auto scrolling during drag operations.
   * A <a href="#!/api/Ext.dd.ScrollManager" rel="Ext.dd.ScrollManager" class="docClass">Ext.dd.ScrollManager</a> configuration may also be passed.
   * @default false
   */
  public native function get containerScroll():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set containerScroll(value:*):void;

  [ExtConfig]
  /**
   * Set as <code>true</code> to copy the records from the source grid to the destination drop
   * grid.  Otherwise, dragged records will be moved.
   *
   * <p><strong>Note:</strong> This only applies to records dragged between two different grids with
   * unique stores.</p>
   *
   * <p>See <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-allowCopy" rel="Ext.grid.plugin.DragDrop-cfg-allowCopy" class="docClass">allowCopy</a> to allow only control-drag operations to copy records.</p>
   * @default false
   */
  public native function get copy():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set copy(value:Boolean):void;

  [ExtConfig]
  /**
   * A named drag drop group to which this object belongs. If a group is specified, then both the DragZones and
   * DropZone used by this plugin will only interact with other drag drop objects in the same group.
   * @default gridDD
   */
  public native function get ddGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ddGroup(value:String):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup" rel="Ext.grid.plugin.DragDrop-cfg-ddGroup" class="docClass">ddGroup</a> to which the DragZone will belong.
   *
   * <p>This defines which other DropZones the DragZone will interact with. Drag/DropZones only interact with other
   * Drag/DropZones which are members of the same <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup" rel="Ext.grid.plugin.DragDrop-cfg-ddGroup" class="docClass">ddGroup</a>.</p>
   */
  public native function get dragGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dragGroup(value:String):void;

  [ExtConfig]
  /**
   * The text to show while dragging.
   *
   * <p>Two placeholders can be used in the text:</p>
   *
   * <ul>
   * <li><code>{0}</code> The number of selected items.</li>
   * <li><code>{1}</code> 's' when more than 1 items (only useful for English).</li>
   * </ul>
   * @default '{0} selected row{1}'
   */
  public native function get dragText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dragText(value:String):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.view.DragZone" rel="Ext.view.DragZone" class="docClass">DragZone</a> which handles mousedown and dragging of records from the grid.
   */
  public native function get dragZone():ext.view.ViewDragZone;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dragZone(value:ext.view.ViewDragZone):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup" rel="Ext.grid.plugin.DragDrop-cfg-ddGroup" class="docClass">ddGroup</a> to which the DropZone will belong.
   *
   * <p>This defines which other DragZones the DropZone will interact with. Drag/DropZones only interact with other
   * Drag/DropZones which are members of the same <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup" rel="Ext.grid.plugin.DragDrop-cfg-ddGroup" class="docClass">ddGroup</a>.</p>
   */
  public native function get dropGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropGroup(value:String):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.grid.ViewDropZone" rel="Ext.grid.ViewDropZone" class="docClass">DropZone</a> which handles mouseover and dropping records in any grid which shares the same <a href="#!/api/Ext.grid.plugin.DragDrop-cfg-dropGroup" rel="Ext.grid.plugin.DragDrop-cfg-dropGroup" class="docClass">dropGroup</a>.
   */
  public native function get dropZone():ext.grid.ViewDropZone;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropZone(value:ext.grid.ViewDropZone):void;

  [ExtConfig]
  /**
   * <code>false</code> to disallow dragging items from the View.
   * @default true
   */
  public native function get enableDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableDrag(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>false</code> to disallow the View from accepting drop gestures.
   * @default true
   */
  public native function get enableDrop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableDrop(value:Boolean):void;
}
}