package ext.grid.plugin.events {
import ext.grid.plugin.GridEditingPlugin;
import net.jangaroo.ext.FlExtEvent;

[Uses("net.jangaroo.ext.FlExtEvent")]
public class GridEditingPlugin_contextEvent extends net.jangaroo.ext.FlExtEvent {
  public static const __PARAMETER_SEQUENCE__:Array;

  /**
   * "Fires before editing is triggered. Return false from event handler to stop the editing.
   * @see ext.grid.plugin.GridEditingPlugin
   * @eventType onBeforeEdit
   */
  public static const BEFORE_EDIT:String = "onBeforeEdit";

  /**
   * "Fires when the user started editing but then cancelled the edit.
   * @see ext.grid.plugin.GridEditingPlugin
   * @eventType onCancelEdit
   */
  public static const CANCEL_EDIT:String = "onCancelEdit";

  /**
   * "Fires after editing. Usage example:
   *
   * <pre><code>grid.on('edit', function(editor, e) {
   *     // commit the changes right after editing finished
   *     e.record.commit();
   * });
   * </code></pre>
   * @see ext.grid.plugin.GridEditingPlugin
   * @eventType onEdit
   */
  public static const EDIT:String = "onEdit";

  /**
   * "Fires after editing, but before the value is set in the record. Return false from event handler to
   * cancel the change.
   *
   * <p>Usage example showing how to remove the red triangle (dirty record indicator) from some records (not all). By
   * observing the grid's validateedit event, it can be cancelled if the edit occurs on a targeted row (for example)
   * and then setting the field's new value in the Record directly:</p>
   *
   * <pre><code>grid.on('validateedit', function (editor, context) {
   *     var myTargetRow = 6;
   *
   *     if (context.rowIdx === myTargetRow) {
   *         context.record.data[context.field] = context.value;
   *     }
   * });
   * </code></pre>
   * @see ext.grid.plugin.GridEditingPlugin
   * @eventType onValidateEdit
   */
  public static const VALIDATE_EDIT:String = "onValidateEdit";

  public function GridEditingPlugin_contextEvent(type:String, arguments:Array) {
    super(null, null);
  }

  public native function get editor():ext.grid.plugin.GridEditingPlugin;

  /**
   * The editing context with the following properties:
   * <ul>
   *   <li><code>grid</code> : ext.grid.GridPanel
   * The owning grid Panel.
   *   </li>
   *   <li><code>record</code> : ext.data.Model
   * The record being edited.
   *   </li>
   *   <li><code>field</code> : String
   * The name of the field being edited.
   *   </li>
   *   <li><code>value</code>
   * The field's current value.
   *   </li>
   *   <li><code>row</code> : js.HTMLElement
   * The grid row element.
   *   </li>
   *   <li><code>column</code> : ext.grid.column.Column
   * The Column being edited.
   *   </li>
   *   <li><code>rowIdx</code> : Number
   * The index of the row being edited.
   *   </li>
   *   <li><code>colIdx</code> : Number
   * The index of the column being edited.
   *   </li>
   *   <li><code>cancel</code> : Boolean
   * Set this to <code>true</code> to cancel the edit or return false from your handler.
   *   </li>
   *   <li><code>originalValue</code>
   * Alias for value (only when using <a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">CellEditing</a>).
   *   </li>
   * </ul>
   */
  public native function get context():Object;
}
}