package ext.grid.selection {
import ext.data.Model;

[Uses("ext.grid.selection.Selection")]
[Uses("ext.Base")]
[Native("Ext.grid.selection.Rows", require)]
/**
 * A class which encapsulates a range of rows defining a selection in a grid.
 */
public class RowsSelection extends ext.grid.selection.Selection {
  /**
   * This property indicates the this selection represents selected rows.
   * @default true
   */
  public native function get isRows():Boolean;

  /**
   * This method is called to indicate the start of multiple changes to the selected row set.
   *
   * <p>Internally this method increments a counter that is decremented by <code><a href="#!/api/Ext.grid.selection.Rows-method-endUpdate" rel="Ext.grid.selection.Rows-method-endUpdate" class="docClass">endUpdate</a></code>. It
   * is important, therefore, that if you call <code>beginUpdate</code> directly you match that
   * call with a call to <code>endUpdate</code> or you will prevent the collection from updating
   * properly.</p>
   */
  public native function beginUpdate():void;

  /**
   * Returns <code>true</code> if the passed <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> is selected.
   * @param record The record to test.
   * @return <code>true</code> if the passed <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> is selected.
   */
  public native function contains(record:ext.data.Model):Boolean;

  /**
   * This method is called after modifications are complete on a selected row set. For details
   * see <code><a href="#!/api/Ext.grid.selection.Rows-method-beginUpdate" rel="Ext.grid.selection.Rows-method-beginUpdate" class="docClass">beginUpdate</a></code>.
   */
  public native function endUpdate():void;

  /**
   * Returns the number of records selected
   * @return The number of records selected.
   */
  public native function getCount():Number;

  /**
   * @return The row index of the first row in the range or zero if no range.
   */
  public native function getFirstRowIndex():Number;

  /**
   * @return The row index of the last row in the range or -1 if no range.
   */
  public native function getLastRowIndex():Number;

  /**
   * Returns the records selected.
   * @return The records selected.
   */
  public native function getRecords():Array;
}
}