package ext.grid.selection {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.grid.selection.Selection", require)]
/**
 * Base class for selections which may be of three subtypes:
 *
 * <ul>
 * <li><a href="#!/api/Ext.grid.selection.Cells" rel="Ext.grid.selection.Cells" class="docClass">Cells</a> A rectangular range of cells defined by a start
 * record/column and an end record/column.</li>
 * <li><a href="#!/api/Ext.grid.selection.Rows" rel="Ext.grid.selection.Rows" class="docClass">Rows</a> An array of records.</li>
 * <li><a href="#!/api/Ext.grid.selection.Columns" rel="Ext.grid.selection.Columns" class="docClass">Columns</a> An array of columns in which all records
 * are included.</li>
 * </ul>
 */
public class Selection extends ext.Base {
  /**
   * Clones this selection object.
   * @return A clone of this instance.
   */
  public native function clone():ext.grid.selection.Selection;

  /**
   * Calls the passed function for each selected cell from top left to bottom right
   * iterating over columns within each row.
   * @param fn The function to call. If this returns <code>false</code>, the iteration is
   * halted with no further calls.
   * @param {ext.grid.CellContext} fn.context The CellContext representing the current cell.
   * @param {Number} fn.columnIndex The column index of the current cell.
   * @param {Number} fn.rowIndex The row index of the current cell.
   * @param scope The context (<code>this</code> reference) in which <code>fn</code> is executed.
   * Defaults to this Selection object.
   */
  public native function eachCell(fn:Function, scope:Object = null):void;

  /**
   * Calls the passed function for each selected column from left to right.
   * @param fn The function to call. If this returns false, the iteration is
   * halted with no further calls.
   * @param {ext.grid.column.Column} fn.column The current column.
   * @param {Number} fn.columnIndex The index of the current column. <em>Note that in a
   * locked grid, this is relative to the outermost grid encompassing both sides</em>.
   * @param scope The context (<code>this</code> reference) in which <code>fn</code> is executed.
   * Defaults to this Selection object.
   */
  public native function eachColumn(fn:Function, scope:Object = null):void;

  /**
   * Calls the passed function for each selected <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a>.
   * @param fn The function to call. If this returns <code>false</code>, the iteration is
   * halted with no further calls.
   * @param {ext.data.Model} fn.record The current record.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * Defaults to this Selection object.
   */
  public native function eachRow(fn:Function, scope:Object = null):void;
}
}