package ext.grid.selection {
import ext.data.Model;
import ext.panel.TablePanel;
import ext.plugin.AbstractPlugin;

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Base")]
[Native("Ext.grid.selection.Replicator", require)]
/**
 * A plugin for use in grids which use the <a href="#!/api/Ext.grid.selection.SpreadsheetModel" rel="Ext.grid.selection.SpreadsheetModel" class="docClass">spreadsheet</a> selection model,
 * with <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-extensible" rel="Ext.grid.selection.SpreadsheetModel-cfg-extensible" class="docClass">extensible</a> configured as <code>true</code> or <code>"y"</code>, meaning that
 * the selection may be extended up or down using a draggable extension handle.
 *
 * <p>This plugin propagates values from the selection into the extension area.</p>
 *
 * <p>If just <em>one</em> row is selected, the values in that row are replicated unchanged into the extension area.</p>
 *
 * <p>If more than one row is selected, the two rows closest to the selected block are taken to provide a numeric
 * difference, and that difference is used to calculate the sequence of values all the way into the extension area.</p>
 */
public class SelectionReplicator extends ext.plugin.AbstractPlugin {
  /**
   * @param config @inheritDoc
   */
  public function SelectionReplicator(config:ext.grid.selection.SelectionReplicator = null) {
    super();
  }

  /**
   * An array of the columns encompassed by the selection block. This is gathered before <a href="#!/api/Ext.grid.selection.Replicator-method-replicateSelection" rel="Ext.grid.selection.Replicator-method-replicateSelection" class="docClass">replicateSelection</a>
   * is called, so is available to subclasses which implement their own <a href="#!/api/Ext.grid.selection.Replicator-method-replicateSelection" rel="Ext.grid.selection.Replicator-method-replicateSelection" class="docClass">replicateSelection</a> method.
   */
  public native function get columns():Array;

  /**
   * @private
   */
  public native function set columns(value:Array):void;

  /**
   * A utility method, which, when passed a record, uses the <a href="#!/api/Ext.grid.selection.Replicator-property-columns" rel="Ext.grid.selection.Replicator-property-columns" class="docClass">columns</a> property to extract the values
   * of that record which are encompassed by the selection.
   *
   * <p>Note that columns with no <a href="#!/api/Ext.grid.column.Column-cfg-dataIndex" rel="Ext.grid.column.Column-cfg-dataIndex" class="docClass">dataIndex</a> cannot yield a value.</p>
   * @param record The record from which to read values.
   * @return The values of the fields used by the selected column range for the passed record.
   */
  public native function getColumnValues(record:ext.data.Model):Array;

  /**
   * This is the method which is called when the <a href="#!/api/Ext.grid.selection.SpreadsheetModel" rel="Ext.grid.selection.SpreadsheetModel" class="docClass">spreadsheet</a> selection model's
   * extender handle is dragged and released.
   *
   * <p>It is passed contextual information about the selection and the extension area.</p>
   *
   * <p>Subclass authors may override it to gain access to the event and perform their own data replication.</p>
   *
   * <p>By default, the selection is extended to encompass the selection area. Returning <code>false</code> from this method
   * vetoes that.</p>
   * @param ownerGrid The owning grid.
   * @param sel An object describing the contiguous selected area.
   * @param extension An object describing the type and size of extension.
   * @param {String} extension.type <code>"rows"</code> or <code>"columns"</code>
   * @param {ext.grid.CellContext} extension.start The start (top left) cell of the extension area.
   * @param {ext.grid.CellContext} extension.end The end (bottom right) cell of the extension area.
   * @param {Number} [extension.columns] The number of columns extended (-ve means on the left side).
   * @param {Number} [extension.rows] The number of rows extended (-ve means on the top side).
   */
  public native function replicateSelection(ownerGrid:ext.panel.TablePanel, sel:ext.grid.selection.Selection, extension:Object):void;
}
}