package ext.layout {
import ext.Base;
import ext.Component;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.layout.Layout", require)]
[ExtConfig]
/**
 * This class is the base for all layout types: component and container.
 */
public class Layout extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * @param config @inheritDoc
   */
  public function Layout(config:ext.layout.Layout = null) {
    super();
  }

  /**
   * Used only during a layout run, this value indicates that a
   * layout has finished its calculations. This flag is set to true prior to the call to
   * <a href="#!/api/Ext.layout.Layout-method-calculate" rel="Ext.layout.Layout-method-calculate" class="docClass">calculate</a> and should be set to false if this layout has more work to do.
   */
  public native function get done():Boolean;

  /**
   * @private
   */
  public native function set done(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Layout, or subclass thereof.
   * @default true
   */
  public native function get isLayout():Boolean;

  [ExtConfig]
  /**
   * An object that contains as keys the names of the properties that can be animated
   * by child items as a consequence of a layout. This config is used internally by the
   * <a href="#!/api/Ext.layout.container.Accordion" rel="Ext.layout.container.Accordion" class="docClass">accordion</a> layout to cause the child panels
   * to animate to their proper size and position after a collapse/expand event.
   * @since 4.1.0
   */
  public native function get animatePolicy():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animatePolicy(value:Object):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Destroys this layout. This method removes a <code>targetCls</code> from the <code>target</code>
   * element and calls <code>onDestroy</code>.
   *
   * <p>A derived class can override either this method or <code>onDestroy</code> but in all
   * cases must call the base class versions of these methods to allow the base class to
   * perform its cleanup.</p>
   *
   * <p>This method (or <code>onDestroy</code>) are overridden by subclasses most often to purge
   * event handlers or remove unmanged DOM nodes.</p>
   */
  override public native function destroy(...params):void;

  /**
   * Called before any calculation cycles to prepare for layout.
   *
   * <p>This is a write phase and DOM reads should be strictly avoided when overridding
   * this method.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function beginLayout(ownerContext:ext.Base):void;

  /**
   * Called before any calculation cycles to reset DOM values and prepare for calculation.
   *
   * <p>This is a write phase and DOM reads should be strictly avoided when overridding
   * this method.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function beginLayoutCycle(ownerContext:ext.Base):void;

  /**
   * Called to perform the calculations for this layout. This method will be called at
   * least once and may be called repeatedly if the <a href="#!/api/Ext.layout.Layout-property-done" rel="Ext.layout.Layout-property-done" class="docClass">done</a> property is cleared
   * before return to indicate that this layout is not yet done. The <a href="#!/api/Ext.layout.Layout-property-done" rel="Ext.layout.Layout-property-done" class="docClass">done</a> property
   * is always set to <code>true</code> before entering this method.
   *
   * <p>This is a read phase and DOM writes should be strictly avoided in derived classes.
   * Instead, DOM writes need to be written to <a href="#!/api/Ext.layout.ContextItem" rel="Ext.layout.ContextItem" class="docClass">Ext.layout.ContextItem</a> objects to
   *  be flushed at the next opportunity.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function calculate(ownerContext:ext.Base):void;

  /**
   * This method (if implemented) is called at the end of the cycle in which this layout
   * completes (by not setting <a href="#!/api/Ext.layout.Layout-property-done" rel="Ext.layout.Layout-property-done" class="docClass">done</a> to <code>false</code> in <a href="#!/api/Ext.layout.Layout-method-calculate" rel="Ext.layout.Layout-method-calculate" class="docClass">calculate</a>). It is
   * possible for the layout to complete and yet become invalid before the end of the cycle,
   * in which case, this method will not be called. It is also possible for this method to
   * be called and then later the layout becomes invalidated. This will result in
   * <a href="#!/api/Ext.layout.Layout-method-calculate" rel="Ext.layout.Layout-method-calculate" class="docClass">calculate</a> being called again, followed by another call to this method.
   *
   * <p>This is a read phase and DOM writes should be strictly avoided in derived classes.
   * Instead, DOM writes need to be written to <a href="#!/api/Ext.layout.ContextItem" rel="Ext.layout.ContextItem" class="docClass">Ext.layout.ContextItem</a> objects to
   * be flushed at the next opportunity.</p>
   *
   * <p>This method need not be implemented by derived classes and, in fact, should only be
   * implemented when needed.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function completeLayout(ownerContext:ext.Base):void;

  /**
   * This method (if implemented) is called after all layouts have completed. In most
   * ways this is similar to <a href="#!/api/Ext.layout.Layout-method-completeLayout" rel="Ext.layout.Layout-method-completeLayout" class="docClass">completeLayout</a>. This call can cause this (or any
   * layout) to be become invalid (see <a href="#!/api/Ext.layout.Context-method-invalidate" rel="Ext.layout.Context-method-invalidate" class="docClass">Ext.layout.Context.invalidate</a>), but this
   * is best avoided. This method is intended to be where final reads are made and so it
   * is best to avoid invalidating layouts at this point whenever possible. Even so, this
   * method can be used to perform final checks that may require all other layouts to be
   * complete and then invalidate some results.
   *
   * <p>This is a read phase and DOM writes should be strictly avoided in derived classes.
   * Instead, DOM writes need to be written to <a href="#!/api/Ext.layout.ContextItem" rel="Ext.layout.ContextItem" class="docClass">Ext.layout.ContextItem</a> objects to
   * be flushed at the next opportunity.</p>
   *
   * <p>This method need not be implemented by derived classes and, in fact, should only be
   * implemented when needed.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function finalizeLayout(ownerContext:ext.Base):void;

  /**
   * This method is called after all layouts are complete and their calculations flushed
   * to the DOM. No further layouts will be run and this method is only called once per
   * layout run. The base component layout caches <code>lastComponentSize</code>.
   *
   * <p>This is a write phase and DOM reads should be avoided if possible when overridding
   * this method.</p>
   *
   * <p>This method need not be implemented by derived classes and, in fact, should only be
   * implemented when needed.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function finishedLayout(ownerContext:ext.Base):void;

  /**
   * This method (if implemented) is called after all layouts are finished, and all have
   * a <code>lastComponentSize</code> cached. No further layouts will be run and this method is only
   * called once per layout run. It is the bookend to <a href="#!/api/Ext.layout.Layout-method-beginLayout" rel="Ext.layout.Layout-method-beginLayout" class="docClass">beginLayout</a>.
   *
   * <p>This is a write phase and DOM reads should be avoided if possible when overridding
   * this method. This is the catch-all tail method to a layout and so the rules are more
   * relaxed. Even so, for performance reasons, it is best to avoid reading the DOM. If
   * a read is necessary, consider implementing a <a href="#!/api/Ext.layout.Layout-method-finalizeLayout" rel="Ext.layout.Layout-method-finalizeLayout" class="docClass">finalizeLayout</a> method to do the
   * required reads.</p>
   *
   * <p>This method need not be implemented by derived classes and, in fact, should only be
   * implemented when needed.</p>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  public native function notifyOwner(ownerContext:ext.Base):void;

  /**
   * This method is called when a child item changes in some way. By default this calls
   * <a href="#!/api/Ext.Component-method-updateLayout" rel="Ext.Component-method-updateLayout" class="docClass">Ext.Component.updateLayout</a> on this layout's owner.
   * @param child The child item that has changed.
   * @return True if this layout has handled the content change.
   */
  public native function onContentChange(child:ext.Component):Boolean;

  /**
   * Removes layout's itemCls and owning Container's itemCls.
   * Clears the managed dimensions flags
   */
  protected native function afterRemove(item:Object):void;

  /**
   * Called before an item is rendered to allow the layout to configure the item.
   * @param item The item to be configured
   */
  protected native function configureItem(item:ext.Component):void;

  /**
   * Returns an object describing how this layout manages the size of the given component.
   * This method must be implemented by any layout that manages components.
   * @return An object describing the sizing done by the layout
   * for this item.
   */
  protected native function getItemSizePolicy(item:ext.Component):ext.layout.SizePolicy;

  /**
   * Returns the set of items to layout (empty by default).
   */
  protected native function getLayoutItems():Array;

  /**
   * A one-time initialization method called just before rendering.
   */
  protected native function initLayout():void;

  /**
   * Validates item is in the proper place in the dom.
   */
  protected native function isValidParent(item:Object, target:Object, position:Object):void;

  /**
   * Iterates over all passed items, ensuring they are rendered.  If the items are already rendered,
   * also determines if the items are in the proper place in the dom.
   */
  protected native function renderItems(items:Object, target:Object):void;
}
}