package ext.layout.component {

[Uses("ext.layout.component.ComponentLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.layout.component.Auto", require)]
/**
 * The class is the default component layout for <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> when no explicit
 * <code><a href="#!/api/Ext.Component-cfg-componentLayout" rel="Ext.Component-cfg-componentLayout" class="docClass">componentLayout</a></code> is configured.
 *
 * <p>This class uses template methods to perform the individual aspects of measurement,
 * calculation and publication of results. The methods called depend on the component's
 * <a href="#!/api/Ext.Component-method-getSizeModel" rel="Ext.Component-method-getSizeModel" class="docClass">size model</a>.</p>
 *
 * <h2>configured / calculated</h2>
 *
 * <p>In either of these size models, the dimension of the outer element is of a known size.
 * The size is found in the <code>ownerContext</code> (the <a href="#!/api/Ext.layout.ContextItem" rel="Ext.layout.ContextItem" class="docClass">Ext.layout.ContextItem</a> for the owner
 * component) as either "width" or "height". This value, if available, is passed to the
 * <code>publishInnerWidth</code> or <code>publishInnerHeight</code> method, respectively.</p>
 *
 * <h2>shrinkWrap</h2>
 *
 * <p>When a dimension uses the <code>shrinkWrap</code> size model, that means the content is measured,
 * then the outer (owner) size is calculated and published.</p>
 *
 * <p>For example, for a shrinkWrap width, the following sequence of calls are made:</p>
 *
 * <ul>
 * <li><code><a href="#!/api/Ext.layout.component.Component-method-measureContentWidth" rel="Ext.layout.component.Component-method-measureContentWidth" class="docClass">Ext.layout.component.Component.measureContentWidth</a></code></li>
 * <li><code>publishOwnerWidth</code>
 *
 * <ul>
 * <li><code>calculateOwnerWidthFromContentWidth</code></li>
 * <li><code>publishInnerWidth</code> (in the event of hitting a min/maxWidth constraint)</li>
 * </ul>
 * </li>
 * </ul>
 *
 *
 * <h2>natural</h2>
 *
 * <p>When a dimension uses the <code>natural</code> size model, the measurement is made on the outer
 * (owner) element. This size is then used to determine the content area in much the same
 * way as if the outer element had a <code>configured</code> or <code>calculated</code> size model.</p>
 *
 * <ul>
 * <li><code><a href="#!/api/Ext.layout.component.Component-method-measureOwnerWidth" rel="Ext.layout.component.Component-method-measureOwnerWidth" class="docClass">Ext.layout.component.Component.measureOwnerWidth</a></code></li>
 * <li><code>publishInnerWidth</code></li>
 * </ul>
 */
public class AutoComponentLayout extends ext.layout.component.ComponentLayout {
  /**
   * @param config @inheritDoc
   */
  public function AutoComponentLayout(config:ext.layout.component.AutoComponentLayout = null) {
    super();
  }

  [ExtConfig]
  /**
   * When publishing height of an auto Component, it is usually not written to the DOM.
   * Setting this to <code>true</code> overrides this behaviour.
   * @default false
   */
  public native function get setHeightInDom():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set setHeightInDom(value:Boolean):void;

  [ExtConfig]
  /**
   * When publishing width of an auto Component, it is usually not written to the DOM.
   * Setting this to <code>true</code> overrides this behaviour.
   * @default false
   */
  public native function get setWidthInDom():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set setWidthInDom(value:Boolean):void;
}
}