package ext.layout.container {

[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Box", require)]
/**
 * Base Class for HBoxLayout and VBoxLayout Classes. Generally it should not need to be used directly.
 */
public class BoxLayout extends ext.layout.container.ContainerLayout {
  /**
   * @param config @inheritDoc
   */
  public function BoxLayout(config:ext.layout.container.BoxLayout = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Controls how the child items of the container are aligned. The value is used to
   * position items "perpendicularly". That is, for horizontal boxes (where <code>vertical</code>
   * is <code>false</code>), then this will position items vertically. Otherwise, this will position
   * items horizontally. The acceptable values for this property are best explained in
   * context with the value of <code>vertical</code>.
   *
   * <p>If <code>vertical</code> is <code>false</code> then this layout is behaving as an <code>hbox</code> and this config
   * operates as follows:</p>
   *
   * <ul>
   * <li><strong>begin</strong> : Child items are aligned vertically at the top of the container.</li>
   * <li><strong>middle</strong> : Child items are vertically centered in the container.</li>
   * <li><strong>end</strong> : Child items are aligned vertically at the bottom of the container.</li>
   * <li><strong>stretch</strong> : Child items are stretched vertically to fill the height of the container.</li>
   * <li><strong>stretchmax</strong> : Child items are stretched vertically to the height of the largest item.</li>
   * </ul>
   *
   *
   * <p>If <code>vertical</code> is <code>true</code> then this layout is behaving as an <code>vbox</code> and this config
   * operates as follows:</p>
   *
   * <ul>
   * <li><strong>begin</strong> : Child items are aligned horizontally at the left side of the container.</li>
   * <li><strong>middle</strong> : Child items are horizontally centered in the container.</li>
   * <li><strong>end</strong> : Child items are aligned horizontally at the right of the container.</li>
   * <li><strong>stretch</strong> : Child items are stretched horizontally to fill the width of the container.</li>
   * <li><strong>stretchmax</strong> : Child items are stretched horizontally to the size of the largest item.</li>
   * </ul>
   *
   *
   * <p>For backwards compatibility, the following values are also recognized:</p>
   *
   * <ul>
   * <li><strong>left</strong> : Same as <strong>begin</strong>.</li>
   * <li><strong>top</strong> : Same as <strong>begin</strong>.</li>
   * <li><strong>center</strong> : Same as <strong>middle</strong>.</li>
   * <li><strong>right</strong> : Same as <strong>end</strong>.</li>
   * <li><strong>bottom</strong> : Same as <strong>end</strong>.</li>
   * </ul>
   * @default "begin"
   * @see #getAlign()
   * @see #setAlign()
   */
  public native function get align():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set align(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The Math method
   * to use for rounding fractional pixels when <code><a href="#!/api/Ext.layout.container.Box-cfg-align" rel="Ext.layout.container.Box-cfg-align" class="docClass">align</a>:middle</code> is used.<br/>
   * The possible values are:
   *
   * <ul>
   * <li><a href="https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/round">round</a></li>
   * <li><a href="https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/floor">floor</a></li>
   * <li><a href="https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/ceil">ceil</a></li>
   * </ul>
   * @default 'round'
   * @see #getAlignRoundingMethod()
   * @see #setAlignRoundingMethod()
   */
  public native function get alignRoundingMethod():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set alignRoundingMethod(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Limits the size of <a href="#!/api/Ext.layout.container.Box-cfg-align" rel="Ext.layout.container.Box-cfg-align" class="docClass">aligned</a> components to the size of the container
   * under certain circumstances. Firstly, the container's height (for <code>hbox</code>) or width
   * (for <code>vbox</code>) must not be determined by the size of the child components. Secondly,
   * the child components must have <a href="#!/api/Ext.Component-cfg-shrinkWrap" rel="Ext.Component-cfg-shrinkWrap" class="docClass">shrinkwrap</a>
   * enabled for this dimension.
   * @default false
   * @see #getConstrainAlign()
   * @see #setConstrainAlign()
   */
  public native function get constrainAlign():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set constrainAlign(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This flag can be set to <code>false</code> to ignore the <code>split</code> config on box items. This is
   * set to <code>false</code> by <code><a href="#!/api/Ext.layout.container.Accordion" rel="Ext.layout.container.Accordion" class="docClass">Ext.layout.container.Accordion</a></code>.
   * @default true
   * @see #getEnableSplitters()
   * @see #setEnableSplitters()
   */
  public native function get enableSplitters():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set enableSplitters(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An overflow handler or config object for an overflow handler.  This is typically
   * specified as one of the following strings:
   *
   * <ul>
   * <li><code>scroller</code> - Scroller buttons are rendered before and after the content.</li>
   * <li><code>menu</code> - Overflowing items are rendered into a menu, and a button is rendered
   *  after the items, which shows the menu when clicked.</li>
   * </ul>
   *
   *
   * <p>NOTE: This config is currently only supported when box layout is used by the
   * following components:</p>
   *
   * <ul>
   * <li><a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a></li>
   * <li><a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a></li>
   * <li><a href="#!/api/Ext.toolbar.Breadcrumb" rel="Ext.toolbar.Breadcrumb" class="docClass">Ext.toolbar.Breadcrumb</a></li>
   * <li><a href="#!/api/Ext.tab.Bar" rel="Ext.tab.Bar" class="docClass">Ext.tab.Bar</a></li>
   * </ul>
   *
   *
   * <p>Components where <code>overflowHandler</code> is not supported should use
   * <code><a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a>:true</code> if they have overflowing
   * content.</p>
   * @see #getOverflowHandler()
   * @see #setOverflowHandler()
   */
  public native function get overflowHandler():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set overflowHandler(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Controls how the child items of the container are packed together. Acceptable
   * configuration values for this property are:
   *
   * <ul>
   * <li><strong>start</strong> - child items are packed together at <strong>left</strong> (HBox) or <strong>top</strong>
   * (VBox) side of container (<strong>default</strong>)</li>
   * <li><strong>center</strong> - child items are packed together at <strong>mid-width</strong> (HBox) or
   * <strong>mid-height</strong> (VBox) of container</li>
   * <li><strong>end</strong> - child items are packed together at <strong>right</strong> (HBox) or <strong>bottom</strong>
   * (VBox) side of container</li>
   * </ul>
   * @default 'start'
   * @see #getPack()
   * @see #setPack()
   */
  public native function get pack():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set pack(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Sets the padding to be applied to all child items managed by this layout.
   *
   * <p>This property must be specified as a string containing space-separated, numeric
   * padding values. The order of the sides associated with each value matches the
   * way CSS processes padding values:</p>
   *
   * <ul>
   * <li>If there is only one value, it applies to all sides.</li>
   * <li>If there are two values, the top and bottom borders are set to the first
   * value and the right and left are set to the second.</li>
   * <li>If there are three values, the top is set to the first value, the left and
   * right are set to the second, and the bottom is set to the third.</li>
   * <li>If there are four values, they apply to the top, right, bottom, and left,
   * respectively.</li>
   * </ul>
   * @default 0
   * @see #getPadding()
   * @see #setPadding()
   */
  public native function get padding():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set padding(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Allows stretchMax calculation to take into account the max perpendicular size
   * (height for HBox layout and width for VBox layout) of another Box layout when
   * calculating its maximum perpendicular child size.
   *
   * <p>If specified as a string, this may be either a known Container ID, or a
   * ComponentQuery selector which is rooted at this layout's Container (ie, to find
   * a sibling, use <code>"^&gt;#siblingItemId</code>).</p>
   * @see #getStretchMaxPartner()
   * @see #setStretchMaxPartner()
   */
  public native function get stretchMaxPartner():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set stretchMaxPartner(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to <code>true</code> to switch the layout to <code>vbox</code>.
   * @default false
   * @see #getVertical()
   * @see #setVertical()
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-align" rel="Ext.layout.container.Box-cfg-align" class="docClass">align</a>.
   */
  public native function getAlign():String;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-alignRoundingMethod" rel="Ext.layout.container.Box-cfg-alignRoundingMethod" class="docClass">alignRoundingMethod</a>.
   */
  public native function getAlignRoundingMethod():String;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-constrainAlign" rel="Ext.layout.container.Box-cfg-constrainAlign" class="docClass">constrainAlign</a>.
   */
  public native function getConstrainAlign():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-enableSplitters" rel="Ext.layout.container.Box-cfg-enableSplitters" class="docClass">enableSplitters</a>.
   */
  public native function getEnableSplitters():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-overflowHandler" rel="Ext.layout.container.Box-cfg-overflowHandler" class="docClass">overflowHandler</a>.
   */
  public native function getOverflowHandler():*;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-pack" rel="Ext.layout.container.Box-cfg-pack" class="docClass">pack</a>.
   */
  public native function getPack():String;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-padding" rel="Ext.layout.container.Box-cfg-padding" class="docClass">padding</a>.
   */
  public native function getPadding():String;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-stretchMaxPartner" rel="Ext.layout.container.Box-cfg-stretchMaxPartner" class="docClass">stretchMaxPartner</a>.
   */
  public native function getStretchMaxPartner():*;

  /**
   * Returns the value of <a href="#!/api/Ext.layout.container.Box-cfg-vertical" rel="Ext.layout.container.Box-cfg-vertical" class="docClass">vertical</a>.
   */
  public native function getVertical():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-align" rel="Ext.layout.container.Box-cfg-align" class="docClass">align</a>.
   * @param align The new value.
   */
  public native function setAlign(align:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-alignRoundingMethod" rel="Ext.layout.container.Box-cfg-alignRoundingMethod" class="docClass">alignRoundingMethod</a>.
   * @param alignRoundingMethod The new value.
   */
  public native function setAlignRoundingMethod(alignRoundingMethod:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-constrainAlign" rel="Ext.layout.container.Box-cfg-constrainAlign" class="docClass">constrainAlign</a>.
   * @param constrainAlign The new value.
   */
  public native function setConstrainAlign(constrainAlign:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-enableSplitters" rel="Ext.layout.container.Box-cfg-enableSplitters" class="docClass">enableSplitters</a>.
   * @param enableSplitters The new value.
   */
  public native function setEnableSplitters(enableSplitters:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-overflowHandler" rel="Ext.layout.container.Box-cfg-overflowHandler" class="docClass">overflowHandler</a>.
   * @param overflowHandler The new value.
   */
  public native function setOverflowHandler(overflowHandler:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-pack" rel="Ext.layout.container.Box-cfg-pack" class="docClass">pack</a>.
   * @param pack The new value.
   */
  public native function setPack(pack:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-padding" rel="Ext.layout.container.Box-cfg-padding" class="docClass">padding</a>.
   * @param padding The new value.
   */
  public native function setPadding(padding:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-stretchMaxPartner" rel="Ext.layout.container.Box-cfg-stretchMaxPartner" class="docClass">stretchMaxPartner</a>.
   * @param stretchMaxPartner The new value.
   */
  public native function setStretchMaxPartner(stretchMaxPartner:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.layout.container.Box-cfg-vertical" rel="Ext.layout.container.Box-cfg-vertical" class="docClass">vertical</a>.
   * @param vertical The new value.
   */
  public native function setVertical(vertical:Boolean):void;

  /**
   * This method is called to (re)cache our understanding of flexes. This happens during beginLayoutCycle and may need to
   * be called again if the flexes are changed during the layout (e.g., like ColumnLayout).
   */
  protected native function cacheFlexes(ownerContext:Object):void;
}
}