package ext.layout.container {
import ext.Base;
import ext.Component;
import ext.dom.Element;
import ext.layout.Layout;
import ext.util.IElementContainer;

[Uses("ext.layout.Layout")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.layout.container.Container", require)]
/**
 * This class is intended to be extended or created via the <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>
 * configuration property.  See <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">Ext.container.Container.layout</a> for additional details.
 */
public class ContainerLayout extends ext.layout.Layout implements ext.util.IElementContainer {
  /**
   * @param config @inheritDoc
   */
  public function ContainerLayout(config:ext.layout.container.ContainerLayout = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The canonical form of <code>childEls</code> is an object keyed by child's property name
   * with values that are objects with the following properties.
   *
   * <ul>
   * <li><code>itemId</code> - The id to combine with the Component's id that is the id of the
   * child element.</li>
   * <li><code>id</code> - The id of the child element.</li>
   * <li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This
   * should be set on things like the generated content for an <code><a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">Ext.view.View</a></code>.</li>
   * <li><code>select</code>: A selector that will be passed to <a href="#!/api/Ext.dom.Element-method-select" rel="Ext.dom.Element-method-select" class="docClass">Ext.dom.Element.select</a>.</li>
   * <li><code>selectNode</code>: A selector that will be passed to <a href="#!/api/Ext.dom.Element-method-selectNode" rel="Ext.dom.Element-method-selectNode" class="docClass">Ext.dom.Element.selectNode</a>.</li>
   * </ul>
   *
   *
   * <p>For example:</p>
   *
   * <pre><code> childEls: {
   *      button: true,
   *      buttonText: 'text',
   *      buttonImage: {
   *          itemId: 'image'
   *      }
   *  }
   * </code></pre>
   *
   * <p>The above is translated into the following complete form:</p>
   *
   * <pre><code> childEls: {
   *      button: {
   *          name: 'button',
   *          itemId: 'button'
   *      },
   *      buttonText: {
   *          name: 'buttonText',
   *          itemId: 'text'
   *      },
   *      buttonImage: {
   *          name: 'buttonImage',
   *          itemId: 'image'
   *      }
   *  }
   * </code></pre>
   *
   * <p>The above can be provided as an array like so:</p>
   *
   * <pre><code> childEls: [
   *      'button',
   *      { name: 'buttonText', itemId: 'text' },
   *      { name: 'buttonImage', itemId: 'image' }
   *  }
   * </code></pre>
   *
   * <p>For example, a Component which renders a title and body text:</p>
   *
   * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     renderTpl: [
   *         '&lt;h1 id="{id}-title" data-ref="title"&gt;{title}&lt;/h1&gt;',
   *         '&lt;p&gt;{msg}&lt;/p&gt;',
   *     ],
   *     renderData: {
   *         title: "Error",
   *         msg: "Something went wrong"
   *     },
   *     childEls: ["title"],
   *     listeners: {
   *         afterrender: function(cmp){
   *             // After rendering the component will have a title property
   *             cmp.title.setStyle({color: "red"});
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p><strong>Note:</strong> <code>childEl</code>s in the <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a>
   * must be referenced in a <strong>data-ref</strong> attribute.  Notice in the above example
   * that the "title" <code>childEl</code> is set in the <code>renderTpl</code> using
   * <strong>data-ref="title"</strong>.</p>
   *
   * <p>When using <code>select</code>, the property will be an instance of <a href="#!/api/Ext.dom.CompositeElement" rel="Ext.dom.CompositeElement" class="docClass">Ext.CompositeElement</a>.
   * In all other cases, the property will be an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or <code>null</code>
   * if not found.</p>
   *
   * <p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.
   * The following issues should be considered:</p>
   *
   * <ul>
   * <li>Performance: using selectors can be 10x slower than id lookup.</li>
   * <li>Over-selecting: selectors are applied after the DOM elements for all children
   * have been rendered, so selectors can match elements from child components
   * (including nested versions of the same component) accidentally.</li>
   * </ul>
   *
   *
   * <p>This above issues are most important when using <code>select</code> since it returns multiple
   * elements.</p>
   * @see #getChildEls()
   * @see #setChildEls()
   */
  public native function get childEls():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set childEls(value:*):void;

  [ExtConfig]
  /**
   * An optional extra CSS class that will be added to the container. This can be useful for
   * adding customized styles to the container or any of its children using standard CSS
   * rules. See <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.<a href="#!/api/Ext.Component-cfg-componentCls" rel="Ext.Component-cfg-componentCls" class="docClass">componentCls</a> also.
   */
  public native function get itemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemCls(value:String):void;

  /**
   * In addition to work done by our base classes, containers benefit from some extra
   * cached data. The following properties are added to the ownerContext:
   *
   * <ul>
   * <li>visibleItems: the result of <a href="#!/api/Ext.layout.container.Container-method-getVisibleItems" rel="Ext.layout.container.Container-method-getVisibleItems" class="docClass">getVisibleItems</a></li>
   * <li>childItems: the ContextItem[] for each visible item</li>
   * <li>targetContext: the ContextItem for the <a href="#!/api/Ext.layout.container.Container-method-getTarget" rel="Ext.layout.container.Container-method-getTarget" class="docClass">getTarget</a> element</li>
   * </ul>
   * @param ownerContext The context item for the layout's owner
   * component.
   */
  override public native function beginLayout(ownerContext:ext.Base):void;

  /**
   * Adds layout's itemCls and owning Container's itemCls
   * @param item The item to be configured
   */
  override protected native function configureItem(item:ext.Component):void;

  [ArrayElementType("ext.Component")]
  /**
   * Returns an array of child components either for a render phase (Performed in the beforeLayout
   * method of the layout's base class), or the layout phase (onLayout).
   * @return of child components
   */
  override protected native function getLayoutItems():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a>.
   */
  public native function getChildEls():*;

  /**
   * Returns the element into which extra functional DOM elements can be inserted. Defaults to the owner Component's encapsulating element.
   *
   * <p>May be overridden in Component layout managers which implement a <a href="#!/api/Ext.layout.container.Container-method-getRenderTarget" rel="Ext.layout.container.Container-method-getRenderTarget" class="docClass">component render target</a> which must only
   * contain child components.</p>
   */
  public native function getElementTarget():ext.dom.Element;

  /**
   * Returns the element into which rendering must take place. Defaults to the owner Container's
   * target element.
   *
   * <p>May be overridden in layout managers which implement an inner element.</p>
   */
  public native function getRenderTarget():ext.dom.Element;

  /**
   * Returns the owner component's resize element.
   */
  public native function getTarget():ext.dom.Element;

  /**
   * Returns all items that are both rendered and visible
   * @return All matching items
   */
  public native function getVisibleItems():Array;

  /**
   * Sets the value of <a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a>.
   * @param childEls The new value.
   */
  public native function setChildEls(childEls:*):void;

  /**
   * Returns the container size (that of the target). Only the fixed-sized dimensions can
   * be returned because the shrinkWrap dimensions are based on the contentWidth/Height
   * as determined by the container layout.
   * @param ownerContext The owner's context item.
   * @param inDom True if the container size must be in the DOM.
   * @default false
   * @return The size
   * <ul>
   *   <li><code>width</code> : Number
   * The width
   *   </li>
   *   <li><code>height</code> : Number
   * The height
   *   </li>
   * </ul>
   */
  protected native function getContainerSize(ownerContext:ext.Base, inDom:Boolean = false):Object;

  /**
   * Returns all items that are rendered
   * @return All matching items
   */
  protected native function getRenderedItems():Array;
}
}