package ext.layout.container {

[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Fit", require)]
/**
 * This is a base class for layouts that contain a single item that automatically expands to fill the layout's
 * container. This class is intended to be extended or created via the layout:'fit'
 * <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">Ext.container.Container.layout</a> config, and should generally not need to be created directly via the new keyword.
 *
 * <p>Fit layout does not have any direct config options (other than inherited ones). To fit a panel to a container using
 * Fit layout, simply set <code>layout: 'fit'</code> on the container and add a single panel to it.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Fit Layout',
 *     width: 300,
 *     height: 150,
 *     layout:'fit',
 *     items: {
 *         title: 'Inner Panel',
 *         html: 'This is the inner panel content',
 *         bodyPadding: 20,
 *         border: false
 *     },
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>If the container has multiple items, all of the items will all be equally sized. This is usually not
 * desired, so to avoid this, place only a <strong>single</strong> item in the container. This sizing of all items
 * can be used to provide a background <a href="#!/api/Ext.Img" rel="Ext.Img" class="docClass">image</a> that is "behind" another item
 * such as a <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">dataview</a> if you also absolutely position the items.</p>
 */
public class FitLayout extends ext.layout.container.ContainerLayout {
  /**
   * @param config @inheritDoc
   */
  public function FitLayout(config:ext.layout.container.FitLayout = null) {
    super();
  }

  /**
   * An optional extra CSS class that will be added to the container. This can be useful for
   * adding customized styles to the container or any of its children using standard CSS
   * rules. See <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.<a href="#!/api/Ext.Component-cfg-componentCls" rel="Ext.Component-cfg-componentCls" class="docClass">componentCls</a> also.
   * @default Ext.baseCSSPrefix + 'fit-item'
   */
  override public native function get itemCls():String;

  /**
   * @private
   */
  override public native function set itemCls(value:String):void;
}
}