package ext.layout.container {

[Uses("ext.layout.container.AutoLayout")]
[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Form", require)]
/**
 * This is a layout that will render form Fields, one under the other all stretched to the Container width.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>', {
 *     width: 500,
 *     height: 300,
 *     title: "FormLayout Panel",
 *     layout: 'form',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     bodyPadding: 5,
 *     defaultType: 'textfield',
 *     items: [{
 *        fieldLabel: 'First Name',
 *         name: 'first',
 *         allowBlank:false
 *     },{
 *         fieldLabel: 'Last Name',
 *         name: 'last'
 *     },{
 *         fieldLabel: 'Company',
 *         name: 'company'
 *     }, {
 *         fieldLabel: 'Email',
 *         name: 'email',
 *         vtype:'email'
 *     }, {
 *         fieldLabel: 'DOB',
 *         name: 'dob',
 *         xtype: 'datefield'
 *     }, {
 *         fieldLabel: 'Age',
 *         name: 'age',
 *         xtype: 'numberfield',
 *         minValue: 0,
 *         maxValue: 100
 *     }, {
 *         xtype: 'timefield',
 *         fieldLabel: 'Time',
 *         name: 'time',
 *         minValue: '8:00am',
 *         maxValue: '6:00pm'
 *     }]
 * });
 * </code></pre>
 */
public class FormLayout extends ext.layout.container.AutoLayout {
  /**
   * @param config @inheritDoc
   */
  public function FormLayout(config:ext.layout.container.FormLayout = null) {
    super();
  }

  [ExtConfig]
  /**
   * The amount of space, in pixels, to use between the items. Defaults to the value
   * inherited from the theme's stylesheet as configured by
   * $form-item-margin-bottom.
   */
  public native function get itemSpacing():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemSpacing(value:Number):void;

  [ExtConfig]
  /**
   * The width of the labels. This can be either a number in pixels, or a valid CSS
   * "width" style, e.g. <code>'100px'</code>, or <code>'30%'</code>.  When configured, all labels will assume
   * this width, and any <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> specified
   * on the items will be ignored.
   *
   * <p>The default behavior of this layout when no no labelWidth is specified is to size
   * the labels to the text-width of the label with the longest text.</p>
   */
  public native function get labelWidth():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelWidth(value:*):void;
}
}