package ext.layout.container {

[Uses("ext.layout.container.BoxLayout")]
[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.HBox", require)]
/**
 * A layout that arranges items horizontally across a
 * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>. This layout optionally divides available
 * horizontal space between child <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a>
 * containing a numeric <a href="#!/api/Ext.Component-cfg-flex" rel="Ext.Component-cfg-flex" class="docClass">flex</a> configuration.
 *
 * <p>This layout may be used to set the heights or vertical position of child items
 * by configuring it with the <a href="#!/api/Ext.layout.container.HBox-cfg-align" rel="Ext.layout.container.HBox-cfg-align" class="docClass">align</a> option.  The horizontal position of the
 * child items may be set using the <a href="#!/api/Ext.layout.container.HBox-cfg-pack" rel="Ext.layout.container.HBox-cfg-pack" class="docClass">pack</a> config.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>', {
 *     width: 500,
 *     height: 300,
 *     title: "HBoxLayout Panel",
 *     layout: {
 *         type: 'hbox',
 *         align: 'stretch'
 *     },
 *     renderTo: document.body,
 *     items: [{
 *         xtype: 'panel',
 *         title: 'Inner Panel One',
 *         flex: 2
 *     },{
 *         xtype: 'panel',
 *         title: 'Inner Panel Two',
 *         flex: 1
 *     },{
 *         xtype: 'panel',
 *         title: 'Inner Panel Three',
 *         flex: 1
 *     }]
 * });
 * </code></pre>
 *
 * <p>The following example may be used to view the outcomes when combining the <code>align</code> and
 * <code>pack</code> configs:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
 *     xtype: 'panel',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     height: 400,
 *     width: 520,
 *     defaultListenerScope: true,
 *     layout: 'hbox',
 *     defaultType: 'button',
 *     items: [{
 *         text: 'One'
 *     }, {
 *         text: 'Two'
 *     }, {
 *         text: 'Three'
 *     }],
 *     dockedItems: [{
 *         xtype: 'toolbar',
 *         dock: 'top',
 *         items: [{
 *             xtype: 'buttongroup',
 *             title: 'align',
 *             layout: 'fit',
 *             items: [{
 *                 xtype: 'segmentedbutton',
 *                 margin: 10,
 *                 allowDepress: true,
 *                 defaults: {
 *                     configType: 'align'  // custom config used in this example
 *                 },
 *                 items: [{
 *                     text: 'begin'
 *                 }, {
 *                     text: 'middle'
 *                 }, {
 *                     text: 'end'
 *                 }, {
 *                     text: 'stretch'
 *                 }, {
 *                     text: 'stretchmax'
 *                 }],
 *                 listeners: {
 *                     toggle: 'onToggle'
 *                 }
 *             }]
 *         }, '-&gt;', {
 *             xtype: 'buttongroup',
 *             title: 'pack',
 *                layout: 'fit',
 *             items: [{
 *                 xtype: 'segmentedbutton',
 *                 margin: 10,
 *                 allowDepress: true,
 *                 defaults: {
 *                     configType: 'pack'  // custom config used in this example
 *                 },
 *                 items: [{
 *                     text: 'start'
 *                 }, {
 *                     text: 'center'
 *                 }, {
 *                     text: 'end'
 *                 }],
 *                 listeners: {
 *                     toggle: 'onToggle'
 *                 }
 *             }]
 *         }]
 *     }],
 *
 *     onToggle: function (group, button, isPressed) {
 *         var cfg = {};
 *
 *         cfg[button.configType] = isPressed ? button.getText() : null;
 *         this.setLayout(cfg);
 *     }
 * });
 * </code></pre>
 */
public class HBoxLayout extends ext.layout.container.BoxLayout {
  /**
   * @param config @inheritDoc
   */
  public function HBoxLayout(config:ext.layout.container.HBoxLayout = null) {
    super();
  }
}
}