package ext.list {
import ext.Widget;
import ext.data.TreeModel;

[Uses("ext.Widget")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.list.Tree", require)]
/**
 * A lightweight component to display data in a simple tree structure.
 */
public class Tree extends ext.Widget {
  /**
   * @param config @inheritDoc
   */
  public function Tree(config:ext.list.Tree = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default configuration for the widgets created for tree items.
   * @default {xtype: 'treelistitem'}
   * <ul>
   *   <li><code>xtype</code> : String (optional)
   * The type of item to create. By default, items are <code><a href="#!/api/Ext.list.TreeItem" rel="Ext.list.TreeItem" class="docClass">treelistitem</a></code>
   * instances. This can be customized but this <code>xtype</code> must reference a class that
   * ultimately derives from the <code><a href="#!/api/Ext.list.AbstractTreeItem" rel="Ext.list.AbstractTreeItem" class="docClass">Ext.list.AbstractTreeItem</a></code> base class.
   * @default "treelistitem"
   *   </li>
   * </ul>
   * @see #getDefaults()
   * @see #setDefaults()
   */
  public native function get defaults():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaults(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to expand only on the click of the expander element. Setting this to
   * <code>false</code> will allow expansion on click of any part of the element.
   * @default true
   * @see #getExpanderOnly()
   * @see #setExpanderOnly()
   */
  public native function get expanderOnly():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set expanderOnly(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to select the node when clicking the expander.
   * @default false
   * @see #getSelectOnExpander()
   * @see #setSelectOnExpander()
   */
  public native function get selectOnExpander():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selectOnExpander(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The current selected node.
   * @see #getSelection()
   * @see #setSelection()
   */
  public native function get selection():ext.data.TreeModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selection(value:ext.data.TreeModel):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if only 1 node per branch may be expanded.
   * @default false
   * @see #getSingleExpand()
   * @see #setSingleExpand()
   */
  public native function get singleExpand():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set singleExpand(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The data source to which this component is bound.
   * @see #getStore()
   * @see #setStore()
   */
  public native function get store():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set store(value:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.list.Tree-cfg-defaults" rel="Ext.list.Tree-cfg-defaults" class="docClass">defaults</a>.
   * <ul>
   *   <li><code>xtype</code> : String (optional)
   * The type of item to create. By default, items are <code><a href="#!/api/Ext.list.TreeItem" rel="Ext.list.TreeItem" class="docClass">treelistitem</a></code>
   * instances. This can be customized but this <code>xtype</code> must reference a class that
   * ultimately derives from the <code><a href="#!/api/Ext.list.AbstractTreeItem" rel="Ext.list.AbstractTreeItem" class="docClass">Ext.list.AbstractTreeItem</a></code> base class.
   * @default "treelistitem"
   *   </li>
   * </ul>
   */
  public native function getDefaults():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.list.Tree-cfg-expanderOnly" rel="Ext.list.Tree-cfg-expanderOnly" class="docClass">expanderOnly</a>.
   */
  public native function getExpanderOnly():Boolean;

  /**
   * Get a child <a href="#!/api/Ext.list.AbstractTreeItem" rel="Ext.list.AbstractTreeItem" class="docClass">item</a> by node.
   * @param node The node.
   * @return The item. <code>null</code> if not found.
   */
  public native function getItem(node:ext.data.TreeModel):ext.list.AbstractTreeItem;

  /**
   * This method is called to populate and return a config object for new nodes. This
   * can be overridden by derived classes to manipulate properties or <code>xtype</code> of the
   * returned object. Upon return, the object is passed to <code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></code> and the
   * reference is stored as part of this tree.
   *
   * <p>The base class implementation will apply any configured <code><a href="#!/api/Ext.list.Tree-cfg-defaults" rel="Ext.list.Tree-cfg-defaults" class="docClass">defaults</a></code> to the
   * object it returns.</p>
   * @param node The node backing the item.
   * @param parent The parent item. This is never <code>null</code> but
   * may be an instance of <code><a href="#!/api/Ext.list.RootTreeItem" rel="Ext.list.RootTreeItem" class="docClass">Ext.list.RootTreeItem</a></code>.
   * @return The config object to pass to <code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></code> for the item.
   */
  public native function getItemConfig(node:ext.data.TreeModel, parent:ext.list.AbstractTreeItem):Object;

  /**
   * Returns the value of <a href="#!/api/Ext.list.Tree-cfg-selectOnExpander" rel="Ext.list.Tree-cfg-selectOnExpander" class="docClass">selectOnExpander</a>.
   */
  public native function getSelectOnExpander():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.Tree-cfg-selection" rel="Ext.list.Tree-cfg-selection" class="docClass">selection</a>.
   */
  public native function getSelection():ext.data.TreeModel;

  /**
   * Returns the value of <a href="#!/api/Ext.list.Tree-cfg-singleExpand" rel="Ext.list.Tree-cfg-singleExpand" class="docClass">singleExpand</a>.
   */
  public native function getSingleExpand():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.Tree-cfg-store" rel="Ext.list.Tree-cfg-store" class="docClass">store</a>.
   */
  public native function getStore():*;

  /**
   * Sets the value of <a href="#!/api/Ext.list.Tree-cfg-defaults" rel="Ext.list.Tree-cfg-defaults" class="docClass">defaults</a>.
   * <ul>
   *   <li><code>xtype</code> : String (optional)
   * The type of item to create. By default, items are <code><a href="#!/api/Ext.list.TreeItem" rel="Ext.list.TreeItem" class="docClass">treelistitem</a></code>
   * instances. This can be customized but this <code>xtype</code> must reference a class that
   * ultimately derives from the <code><a href="#!/api/Ext.list.AbstractTreeItem" rel="Ext.list.AbstractTreeItem" class="docClass">Ext.list.AbstractTreeItem</a></code> base class.
   * @default "treelistitem"
   *   </li>
   * </ul>
   * @param defaults The new value.
   */
  public native function setDefaults(defaults:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.Tree-cfg-expanderOnly" rel="Ext.list.Tree-cfg-expanderOnly" class="docClass">expanderOnly</a>.
   * @param expanderOnly The new value.
   */
  public native function setExpanderOnly(expanderOnly:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.Tree-cfg-selectOnExpander" rel="Ext.list.Tree-cfg-selectOnExpander" class="docClass">selectOnExpander</a>.
   * @param selectOnExpander The new value.
   */
  public native function setSelectOnExpander(selectOnExpander:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.Tree-cfg-selection" rel="Ext.list.Tree-cfg-selection" class="docClass">selection</a>.
   * @param selection The new value.
   */
  public native function setSelection(selection:ext.data.TreeModel):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.Tree-cfg-singleExpand" rel="Ext.list.Tree-cfg-singleExpand" class="docClass">singleExpand</a>.
   * @param singleExpand The new value.
   */
  public native function setSingleExpand(singleExpand:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.Tree-cfg-store" rel="Ext.list.Tree-cfg-store" class="docClass">store</a>.
   * @param store The new value.
   */
  public native function setStore(store:*):void;
}
}