package ext.menu {

[Uses("ext.menu.Item")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.menu.CheckItem", require)]
/**
 * Fires before a change event. Return false to cancel.
 * @eventType ext.menu.events.CheckItem_checkedEvent.BEFORE_CHECK_CHANGE
 */
[Event(name = 'onBeforeCheckChange', type = 'ext.menu.events.CheckItem_checkedEvent')]
/**
 * Fires after a change event.
 * @eventType ext.menu.events.CheckItem_checkedEvent.CHECK_CHANGE
 */
[Event(name = 'onCheckChange', type = 'ext.menu.events.CheckItem_checkedEvent')]
/**
 * A menu item that contains a togglable checkbox by default, but that can also be a part of a radio group.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     width: 100,
 *     height: 110,
 *     floating: false,  // usually you want this set to True (default)
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),  // usually rendered by it's containing component
 *     items: [{
 *         xtype: 'menucheckitem',
 *         text: 'select all'
 *     },{
 *         xtype: 'menucheckitem',
 *         text: 'select specific'
 *     },{
 *         iconCls: 'add16',
 *         text: 'icon item'
 *     },{
 *         text: 'regular item'
 *     }]
 * });
 * </code></pre>
 */
public class CheckItem extends ext.menu.Item {
  /**
   * @param config @inheritDoc
   */
  public function CheckItem(config:ext.menu.CheckItem = null) {
    super();
  }

  [ExtConfig]
  /**
   * Whether to not to hide the owning menu when this item is clicked.
   * Defaults to <code>false</code> for checkbox items, and to <code>true</code> for radio group items.
   * @default false
   */
  override public native function get hideOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set hideOnClick(value:Boolean):void;

  [ExtConfig]
  /**
   * True to prevent the checked item from being toggled. Any submenu will still be accessible.
   * @default false
   */
  public native function get checkChangeDisabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkChangeDisabled(value:Boolean):void;

  [ExtConfig]
  /**
   * Alternative for the <a href="#!/api/Ext.menu.CheckItem-event-checkchange" rel="Ext.menu.CheckItem-event-checkchange" class="docClass">checkchange</a> event.  Gets called with the same parameters.
   * &#64;controllable
   */
  public native function get checkHandler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkHandler(value:*):void;

  [ExtConfig]
  /**
   * True to render the menuitem initially checked.
   * @default false
   * @see #setChecked()
   */
  public native function get checked():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set checked(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class used by <a href="#!/api/Ext.menu.CheckItem-cfg-cls" rel="Ext.menu.CheckItem-cfg-cls" class="docClass">cls</a> to show the checked state.
   * Defaults to <code><a href="#!/api/Ext-property-baseCSSPrefix" rel="Ext-property-baseCSSPrefix" class="docClass">Ext.baseCSSPrefix</a> + 'menu-item-checked'</code>.
   * @default Ext.baseCSSPrefix + 'menu-item-checked'
   */
  public native function get checkedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkedCls(value:String):void;

  [ExtConfig]
  /**
   * Name of a radio group that the item belongs.
   *
   * <p>Specifying this option will turn check item into a radio item.</p>
   *
   * <p>Note that the group name must be globally unique.</p>
   */
  public native function get group():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set group(value:String):void;

  [ExtConfig]
  /**
   * The CSS class applied to this item's icon image to denote being a part of a radio group.
   * Defaults to <code>Ext.baseCSSClass + 'menu-group-icon'</code>.
   * Any specified <a href="#!/api/Ext.menu.CheckItem-cfg-iconCls" rel="Ext.menu.CheckItem-cfg-iconCls" class="docClass">iconCls</a> overrides this.
   * @default Ext.baseCSSPrefix + 'menu-group-icon'
   */
  public native function get groupCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set groupCls(value:String):void;

  [ExtConfig]
  /**
   * Scope for the <a href="#!/api/Ext.menu.CheckItem-cfg-checkHandler" rel="Ext.menu.CheckItem-cfg-checkHandler" class="docClass">checkHandler</a> callback.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * Text to be announced by screen readers when a check item
   * submenu is focused.
   * @default '{0} submenu'
   */
  public native function get submenuText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submenuText(value:String):void;

  [ExtConfig]
  /**
   * The CSS class used by <a href="#!/api/Ext.menu.CheckItem-cfg-cls" rel="Ext.menu.CheckItem-cfg-cls" class="docClass">cls</a> to show the unchecked state.
   * Defaults to <code><a href="#!/api/Ext-property-baseCSSPrefix" rel="Ext-property-baseCSSPrefix" class="docClass">Ext.baseCSSPrefix</a> + 'menu-item-unchecked'</code>.
   * @default Ext.baseCSSPrefix + 'menu-item-unchecked'
   */
  public native function get uncheckedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set uncheckedCls(value:String):void;

  /**
   * Disables just the checkbox functionality of this menu Item. If this menu item has a submenu, that submenu
   * will still be accessible
   */
  public native function disableCheckChange():void;

  /**
   * Re-enables the checkbox functionality of this menu item after having been
   * disabled by <a href="#!/api/Ext.menu.CheckItem-method-disableCheckChange" rel="Ext.menu.CheckItem-method-disableCheckChange" class="docClass">disableCheckChange</a>
   */
  public native function enableCheckChange():void;

  /**
   * Sets the checked state of the item
   * @param checked True to check, false to un-check
   * @param suppressEvents True to prevent firing the checkchange events.
   * @default false
   */
  public native function setChecked(checked:Boolean, suppressEvents:Boolean = false):void;
}
}