package ext.menu {

[Uses("ext.menu.Item")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.menu.Separator", require)]
/**
 * Adds a separator bar to a menu, used to divide logical groups of menu items. Generally you will
 * add one of these by using "-" in your call to add() or in your items config rather than creating one directly.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     width: 100,
 *     height: 100,
 *     floating: false,  // usually you want this set to True (default)
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),  // usually rendered by it's containing component
 *     items: [{
 *         text: 'icon item',
 *         iconCls: 'add16'
 *     },{
 *         xtype: 'menuseparator'
 *     },{
 *        text: 'separator above'
 *     },{
 *        text: 'regular item'
 *     }]
 * });
 * </code></pre>
 */
public class Separator extends ext.menu.Item {
  /**
   * @param config @inheritDoc
   */
  public function Separator(config:ext.menu.Separator = null) {
    super();
  }

  [ExtConfig]
  /**
   * The CSS class used by the separator item to show the incised line.
   * @default Ext.baseCSSPrefix + 'menu-item-separator'
   */
  public native function get separatorCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set separatorCls(value:String):void;
}
}