package ext.mixin {
import ext.Mixin;

[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Native("Ext.mixin.Accessible", require)]
[ExcludeClass]
/**
 * This mixin defines certain config options, properties, and APIs to be used
 * by Components that implement accessible traits according to WAI-ARIA 1.0 specification.
 */
public class Accessible extends ext.Mixin implements ext.mixin.IAccessible {
  /**
   * @param config @inheritDoc
   */
  public function Accessible(config:ext.mixin.Accessible = null) {
    super();
  }

  /**
   * ARIA role for this Component, defaults to no role.
   * With no role, no other ARIA attributes are set.
   */
  public native function get ariaRole():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing ARIA attributes to be set
   * on this Component's ARIA element. Use this to set the attributes that cannot be
   * determined by the Component's state, such as <code>aria-live</code>, <code>aria-flowto</code>, etc.
   *
   * <p><strong>Note</strong> that this config is only meaningful at the Component rendering time,
   * and setting it after that will do nothing.</p>
   * @default {$value: null, lazy: true}
   * @see #getAriaAttributes()
   * @see #setAriaAttributes()
   */
  public native function get ariaAttributes():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set ariaAttributes(value:Object):void;

  [ExtConfig]
  /**
   * DOM selector for a child element that is to be used
   * as description for this Component, set in <code>aria-describedby</code> attribute.
   * The selector works the same way as <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a>.
   */
  public native function get ariaDescribedBy():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDescribedBy(value:String):void;

  [ExtConfig]
  /**
   * ARIA label for this Component. It is best to use
   * <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> option instead, because screen readers prefer
   * <code>aria-labelledby</code> attribute to <code>aria-label</code>. <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabel" rel="Ext.mixin.Accessible-cfg-ariaLabel" class="docClass">ariaLabel</a> and
   * <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> config options are mutually exclusive.
   */
  public native function get ariaLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaLabel(value:String):void;

  [ExtConfig]
  /**
   * DOM selector for a child element that is to be used
   * as label for this Component, set in <code>aria-labelledby</code> attribute.
   * If the selector is by <code>#id</code>, the label element can be any existing element,
   * not necessarily a child of the main Component element.
   *
   * <p><a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> and <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabel" rel="Ext.mixin.Accessible-cfg-ariaLabel" class="docClass">ariaLabel</a> config options are
   * mutually exclusive, and <code>ariaLabelledBy</code> has the higher precedence.</p>
   */
  public native function get ariaLabelledBy():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaLabelledBy(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Accessible-cfg-ariaAttributes" rel="Ext.mixin.Accessible-cfg-ariaAttributes" class="docClass">ariaAttributes</a>.
   */
  public native function getAriaAttributes():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Accessible-cfg-ariaAttributes" rel="Ext.mixin.Accessible-cfg-ariaAttributes" class="docClass">ariaAttributes</a>.
   * @param ariaAttributes The new value.
   */
  public native function setAriaAttributes(ariaAttributes:Object):void;
}
}