package ext.mixin {
import ext.Component;
import ext.Mixin;

[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Native("Ext.mixin.Container", require)]
[ExcludeClass]
/**
 * Common methods for both classic &amp; modern containers
 */
public class Container extends ext.Mixin implements ext.mixin.IContainer {
  /**
   * <code>true</code> in this class to identify an object as an instantiated Container, or subclass thereof.
   * @default true
   */
  public native function get isContainer():Boolean;

  /**
   * @private
   */
  public native function set isContainer(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <code>true</code>, this container will be marked as being a point in the hierarchy where
   * references to items with a specified <code>reference</code> config will be held. The container
   * will automatically become a referenceHolder if a controller is specified.
   *
   * <p>See the introductory docs for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for more information
   * about references &amp; reference holders.</p>
   * @default false
   * @see #getReferenceHolder()
   * @see #setReferenceHolder()
   */
  public native function get referenceHolder():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set referenceHolder(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Container-cfg-referenceHolder" rel="Ext.mixin.Container-cfg-referenceHolder" class="docClass">referenceHolder</a>.
   */
  public native function getReferenceHolder():Boolean;

  /**
   * Returns an object holding the descendants of this view keyed by their
   * <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code>. This object should not be held
   * past the scope of the function calling this method. It will not be valid if items
   * are added or removed from this or any sub-container.
   *
   * <p>The intended usage is shown here (assume there are 3 components with reference
   * values of "foo", "bar" and "baz" at some level below this container):</p>
   *
   * <pre><code> onClick: function () {
   *      var refs = this.getReferences();
   *
   *      // using "refs" we can access any descendant by its "reference"
   *
   *      refs.foo.getValue() + refs.bar.getValue() + refs.baz.getValue();
   *  }
   * </code></pre>
   *
   * <p>If <code>this</code> component has a <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code> assigned
   * to it, that is <strong>not</strong> included in this object. That reference is understood to
   * belong to the ancestor container configured as the <code>referenceHolder</code>.</p>
   * @since 5.0.0
   * @return An object with each child reference. This will be <code>null</code> if this
   * container has no descendants with a <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code>
   * specified.
   */
  public native function getReferences():Object;

  /**
   * Gets a reference to the component with the specified reference value.
   *
   * <p>The method is a short-hand for the <a href="#!/api/Ext.mixin.Container-method-lookupReference" rel="Ext.mixin.Container-method-lookupReference" class="docClass">lookupReference</a> method.</p>
   * @since 6.0.1
   * @param key The name of the reference to lookup.
   * @return The referenced component or <code>null</code> if it is not found.
   */
  public native function lookup(key:String):ext.Component;

  /**
   * Gets a reference to the component with the specified reference value.
   *
   * <p>The <a href="#!/api/Ext.mixin.Container-method-lookup" rel="Ext.mixin.Container-method-lookup" class="docClass">lookup</a> method is a short-hand version of this method.</p>
   * @since 5.0
   * @param key The name of the reference to lookup.
   * @return The referenced component or <code>null</code> if it is not found.
   */
  public native function lookupReference(key:String):ext.Component;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Container-cfg-referenceHolder" rel="Ext.mixin.Container-cfg-referenceHolder" class="docClass">referenceHolder</a>.
   * @param referenceHolder The new value.
   */
  public native function setReferenceHolder(referenceHolder:Boolean):void;
}
}