package ext.mixin {
import ext.IMixin;

[Uses("ext.Mixin")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.Base")]
[Mixin("ext.mixin.Accessible")]
/**
 * This mixin defines certain config options, properties, and APIs to be used
 * by Components that implement accessible traits according to WAI-ARIA 1.0 specification.
 * @see ext.mixin.Accessible
 */
public interface IAccessible extends ext.IMixin {
  /**
   * ARIA role for this Component, defaults to no role.
   * With no role, no other ARIA attributes are set.
   */
  function get ariaRole():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing ARIA attributes to be set
   * on this Component's ARIA element. Use this to set the attributes that cannot be
   * determined by the Component's state, such as <code>aria-live</code>, <code>aria-flowto</code>, etc.
   *
   * <p><strong>Note</strong> that this config is only meaningful at the Component rendering time,
   * and setting it after that will do nothing.</p>
   * @default {$value: null, lazy: true}
   * @see #getAriaAttributes()
   * @see #setAriaAttributes()
   */
  function get ariaAttributes():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  function set ariaAttributes(value:Object):void;

  [ExtConfig]
  /**
   * DOM selector for a child element that is to be used
   * as description for this Component, set in <code>aria-describedby</code> attribute.
   * The selector works the same way as <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a>.
   */
  function get ariaDescribedBy():String;

  [ExtConfig]
  function set ariaDescribedBy(value:String):void;

  [ExtConfig]
  /**
   * ARIA label for this Component. It is best to use
   * <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> option instead, because screen readers prefer
   * <code>aria-labelledby</code> attribute to <code>aria-label</code>. <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabel" rel="Ext.mixin.Accessible-cfg-ariaLabel" class="docClass">ariaLabel</a> and
   * <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> config options are mutually exclusive.
   */
  function get ariaLabel():String;

  [ExtConfig]
  function set ariaLabel(value:String):void;

  [ExtConfig]
  /**
   * DOM selector for a child element that is to be used
   * as label for this Component, set in <code>aria-labelledby</code> attribute.
   * If the selector is by <code>#id</code>, the label element can be any existing element,
   * not necessarily a child of the main Component element.
   *
   * <p><a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> and <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabel" rel="Ext.mixin.Accessible-cfg-ariaLabel" class="docClass">ariaLabel</a> config options are
   * mutually exclusive, and <code>ariaLabelledBy</code> has the higher precedence.</p>
   */
  function get ariaLabelledBy():String;

  [ExtConfig]
  function set ariaLabelledBy(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Accessible-cfg-ariaAttributes" rel="Ext.mixin.Accessible-cfg-ariaAttributes" class="docClass">ariaAttributes</a>.
   */
  function getAriaAttributes():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Accessible-cfg-ariaAttributes" rel="Ext.mixin.Accessible-cfg-ariaAttributes" class="docClass">ariaAttributes</a>.
   * @param ariaAttributes The new value.
   */
  function setAriaAttributes(ariaAttributes:Object):void;
}
}