package ext.mixin {

[Uses("ext.mixin.Dirty")]
[Mixin("ext.mixin.Dirty")]
/**
 * This mixin provides a <code>dirty</code> config that tracks the modified state of an object. If
 * the class using this mixin is <a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">observable</a>, changes to the
 * <code>dirty</code> config will fire the <code>dirtychange</code> event.
 * @see ext.mixin.Dirty
 */
public interface IDirty {
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config property describes the modified state of this object. In most
   * cases this config's value is maintained by the object and should be considered
   * readonly. The class implementor should be the only one to call the setter.
   * @see #getDirty()
   * @see #setDirty()
   */
  function get dirty():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  function set dirty(value:Boolean):void;

  [ExtConfig]
  /**
   * This config property indicates that the <code>dirty</code> state of this object should be
   * ignored. Because this capability is mixed in at a class level, this config can
   * be helpful when some instances do not participate in dirty state tracking.
   *
   * <p>This option should be set at construction time. When set to <code>true</code>, the object
   * will always have <code>dirty</code> value of <code>false</code>.</p>
   * @default false
   */
  function get ignoreDirty():Boolean;

  [ExtConfig]
  function set ignoreDirty(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a>.
   */
  function getDirty():Boolean;

  /**
   * Returns <code>true</code> if this object is <code>dirty</code>.
   */
  function isDirty():void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a>.
   * @param dirty The new value.
   */
  function setDirty(dirty:Boolean):void;
}
}