package ext.panel {
import ext.Component;
import ext.container.Container;
import ext.container.IDockingContainer;
import ext.dd.DragSource;
import ext.dom.Element;
import ext.toolbar.Toolbar;

[Uses("ext.container.Container")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.panel.Panel", require)]
/**
 * Fires before the user closes the panel. Return false from any listener to stop the close event being
 * fired
 * @eventType ext.panel.events.PanelEvent.BEFORE_CLOSE
 */
[Event(name = 'onBeforeClose', type = 'ext.panel.events.PanelEvent')]
/**
 * Fires before this panel is collapsed. Return false to prevent the collapse.
 * @eventType ext.panel.events.Panel_direction_animateEvent.BEFORE_COLLAPSE
 */
[Event(name = 'onBeforeCollapse', type = 'ext.panel.events.Panel_direction_animateEvent')]
/**
 * Fires before this panel is expanded. Return false to prevent the expand.
 * @eventType ext.panel.events.Panel_animateEvent.BEFORE_EXPAND
 */
[Event(name = 'onBeforeExpand', type = 'ext.panel.events.Panel_animateEvent')]
/**
 * Fires when the user closes the panel.
 * @eventType ext.panel.events.PanelEvent.CLOSE
 */
[Event(name = 'onClose', type = 'ext.panel.events.PanelEvent')]
/**
 * Fires after this Panel has collapsed.
 * @eventType ext.panel.events.PanelEvent.COLLAPSE
 */
[Event(name = 'onCollapse', type = 'ext.panel.events.PanelEvent')]
/**
 * Fires when any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> is added or inserted as a docked item.
 * @eventType ext.panel.events.Panel_component_indexEvent.DOCKED_ADD
 */
[Event(name = 'onDockedAdd', type = 'ext.panel.events.Panel_component_indexEvent')]
/**
 * Fires when any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> is removed from the docked items.
 * @eventType ext.panel.events.Panel_componentEvent.DOCKED_REMOVE
 */
[Event(name = 'onDockedRemove', type = 'ext.panel.events.Panel_componentEvent')]
/**
 * Fires after this Panel has expanded.
 * @eventType ext.panel.events.PanelEvent.EXPAND
 */
[Event(name = 'onExpand', type = 'ext.panel.events.PanelEvent')]
/**
 * Fires after a collapsed Panel has been "floated" by clicking on
 * it's header. Only applicable when the Panel is an item in a
 * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">Border Layout</a>.
 * @eventType ext.panel.events.PanelEvent.FLOAT
 */
[Event(name = 'onFloat', type = 'ext.panel.events.PanelEvent')]
/**
 * Fired when the Panel glyph has been changed by the <a href="#!/api/Ext.panel.Panel-method-setGlyph" rel="Ext.panel.Panel-method-setGlyph" class="docClass">setGlyph</a> method.
 * @eventType ext.panel.events.Panel_newGlyph_oldGlyphEvent.GLYPH_CHANGE
 */
[Event(name = 'onGlyphChange', type = 'ext.panel.events.Panel_newGlyph_oldGlyphEvent')]
/**
 * Fires after the Panel icon has been set or changed.
 * @eventType ext.panel.events.Panel_newIcon_oldIconEvent.ICON_CHANGE
 */
[Event(name = 'onIconChange', type = 'ext.panel.events.Panel_newIcon_oldIconEvent')]
/**
 * Fires after the Panel iconCls has been set or changed.
 * @eventType ext.panel.events.Panel_newIconCls_oldIconClsEvent.ICON_CLS_CHANGE
 */
[Event(name = 'onIconClsChange', type = 'ext.panel.events.Panel_newIconCls_oldIconClsEvent')]
/**
 * Fires after the Panel title has been set or changed.
 * @eventType ext.panel.events.Panel_newTitle_oldTitleEvent.TITLE_CHANGE
 */
[Event(name = 'onTitleChange', type = 'ext.panel.events.Panel_newTitle_oldTitleEvent')]
/**
 * Fires after a "floated" Panel has returned to it's collapsed state
 * as a result of the mouse leaving the Panel. Only applicable when
 * the Panel is an item in a
 * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">Border Layout</a>.
 * @eventType ext.panel.events.PanelEvent.UNFLOAT
 */
[Event(name = 'onUnfloat', type = 'ext.panel.events.PanelEvent')]
/**
 * Panel is a container that has specific functionality and structural components that make it the perfect building
 * block for application-oriented user interfaces.
 *
 * <p>Panels are, by virtue of their inheritance from <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>, capable of being configured with a
 * <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>, and containing child Components.</p>
 *
 * <p>When either specifying child <a href="#!/api/Ext.panel.Panel-cfg-items" rel="Ext.panel.Panel-cfg-items" class="docClass">items</a> of a Panel, or dynamically <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">adding</a>
 * Components to a Panel, remember to consider how you wish the Panel to arrange those child elements, and whether those
 * child elements need to be sized using one of Ext's built-in <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code>
 * schemes. By default, Panels use the <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">Auto</a> scheme. This simply renders child
 * components, appending them one after the other inside the Container, and <strong>does not apply any sizing</strong> at all.</p>
 *
 * <p><p><img src="" alt="Panel components" width="" height=""/></p></p>
 *
 * <p>A Panel may also contain <a href="#!/api/Ext.panel.Panel-cfg-bbar" rel="Ext.panel.Panel-cfg-bbar" class="docClass">bottom</a> and <a href="#!/api/Ext.panel.Panel-cfg-tbar" rel="Ext.panel.Panel-cfg-tbar" class="docClass">top</a> toolbars, along with separate <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">header</a>, <a href="#!/api/Ext.panel.Panel-cfg-fbar" rel="Ext.panel.Panel-cfg-fbar" class="docClass">footer</a> and body sections.</p>
 *
 * <p>Panel also provides built-in <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible, expandable</a> and <a href="#!/api/Ext.panel.Panel-cfg-closable" rel="Ext.panel.Panel-cfg-closable" class="docClass">closable</a> behavior. Panels can
 * be easily dropped into any <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> or layout, and the layout and rendering pipeline
 * is <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">completely managed by the framework</a>.</p>
 *
 * <p><strong>Note:</strong> By default, the <code><a href="#!/api/Ext.panel.Panel-cfg-closable" rel="Ext.panel.Panel-cfg-closable" class="docClass">close</a></code> header tool <em>destroys</em> the Panel resulting in removal of the
 * Panel and the destruction of any descendant Components. This makes the Panel object, and all its descendants
 * <strong>unusable</strong>. To enable the close tool to simply <em>hide</em> a Panel for later re-use, configure the Panel with
 * <code><a href="#!/api/Ext.panel.Panel-cfg-closeAction" rel="Ext.panel.Panel-cfg-closeAction" class="docClass">closeAction</a>: 'hide'</code>.</p>
 *
 * <p>Usually, Panels are used as constituents within an application, in which case, they would be used as child items of
 * Containers, and would themselves use Ext.Components as child <a href="#!/api/Ext.panel.Panel-cfg-items" rel="Ext.panel.Panel-cfg-items" class="docClass">items</a>. However to illustrate simply rendering a
 * Panel into the document, here's how to do it:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Hello',
 *     width: 200,
 *     html: '&lt;p&gt;World!&lt;/p&gt;',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>A more realistic scenario is a Panel created to house input fields which will not be rendered, but used as a
 * constituent part of a Container:</p>
 *
 * <pre class='inline-example '><code>var filterPanel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     bodyPadding: 5,  // Don't want content to crunch against the borders
 *     width: 300,
 *     title: 'Filters',
 *     items: [{
 *         xtype: 'datefield',
 *         fieldLabel: 'Start date'
 *     }, {
 *         xtype: 'datefield',
 *         fieldLabel: 'End date'
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>Note that the Panel above is configured to render into the document and assigned a size. In a real world scenario,
 * the Panel will often be added inside a Container which will use a <a href="#!/api/Ext.panel.Panel-cfg-layout" rel="Ext.panel.Panel-cfg-layout" class="docClass">layout</a> to render, size and position its
 * child Components.</p>
 *
 * <p>Panels will often use specific <a href="#!/api/Ext.panel.Panel-cfg-layout" rel="Ext.panel.Panel-cfg-layout" class="docClass">layout</a>s to provide an application with shape and structure by containing and
 * arranging child Components:</p>
 *
 * <pre class='inline-example '><code>var resultsPanel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Results',
 *     width: 600,
 *     height: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     layout: {
 *         type: 'vbox',       // Arrange child items vertically
 *         align: 'stretch',    // Each takes up full width
 *         padding: 5
 *     },
 *     items: [{               // Results grid specified as a config object with an xtype of 'grid'
 *         xtype: 'grid',
 *         columns: [{header: 'Column One'}],            // One header just for show. There's no data,
 *         store: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.ArrayStore" rel="Ext.data.ArrayStore" class="docClass">Ext.data.ArrayStore</a>', {}), // A dummy empty data store
 *         flex: 1                                       // Use 1/3 of Container's height (hint to Box layout)
 *     }, {
 *         xtype: 'splitter'   // A splitter between the two child items
 *     }, {                    // Details Panel specified as a config object (no xtype defaults to 'panel').
 *         title: 'Details',
 *         bodyPadding: 5,
 *         items: [{
 *             fieldLabel: 'Data item',
 *             xtype: 'textfield'
 *         }], // An array of form fields
 *         flex: 2             // Use 2/3 of Container's height (hint to Box layout)
 *     }]
 * });
 * </code></pre>
 *
 * <p>The example illustrates one possible method of displaying search results. The Panel contains a grid with the
 * resulting data arranged in rows. Each selected row may be displayed in detail in the Panel below. The <a href="#!/api/Ext.layout.container.VBox" rel="Ext.layout.container.VBox" class="docClass">vbox</a> layout is used to arrange the two vertically. It is configured to stretch child items
 * horizontally to full width. Child items may either be configured with a numeric height, or with a <code>flex</code> value to
 * distribute available space proportionately.</p>
 *
 * <p>This Panel itself may be a child item of, for example, a <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a> which
 * will size its child items to fit within its content area.</p>
 *
 * <p>Using these techniques, as long as the <strong>layout</strong> is chosen and configured correctly, an application may have any
 * level of nested containment, all dynamically sized according to configuration, the user's preference and available
 * browser size.</p>
 */
public class Panel extends ext.container.Container implements ext.container.IDockingContainer {
  /**
   * @param config @inheritDoc
   */
  public function Panel(config:ext.panel.Panel = null) {
    super();
  }

  [ExtConfig]
  /**
   * Specify as <code>false</code> to render the Panel with zero width borders.
   *
   * <p>Leaving the value as <code>true</code> uses the selected theme's $panel-border-width</p>
   *
   * <p>Defaults to <code>false</code> when using or extending Neptune.</p>
   *
   * <p><strong>Note:</strong> is ignored when <a href="#!/api/Ext.panel.Panel-cfg-frame" rel="Ext.panel.Panel-cfg-frame" class="docClass">frame</a> is set to <strong>true</strong>.</p>
   * @default true
   * @see #setBorder()
   */
  override public native function get border():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set border(value:*):void;

  [ExtConfig]
  /**
   * True to constrain the panel within its containing element, false to allow it to fall outside of its containing
   * element. By default floating components such as Windows will be rendered to <code>document.body</code>. To render and constrain the window within
   * another element specify <a href="#!/api/Ext.panel.Panel-cfg-renderTo" rel="Ext.panel.Panel-cfg-renderTo" class="docClass">renderTo</a>. Optionally the header only can be constrained
   * using <a href="#!/api/Ext.panel.Panel-cfg-constrainHeader" rel="Ext.panel.Panel-cfg-constrainHeader" class="docClass">constrainHeader</a>.
   * @default false
   */
  override public native function get constrain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set constrain(value:Boolean):void;

  [ExtConfig]
  /**
   * True to apply a frame to the panel.
   *
   * <p><strong>Note:</strong> <code>frame: true</code> overrides <a href="#!/api/Ext.panel.Panel-cfg-border" rel="Ext.panel.Panel-cfg-border" class="docClass">border:false</a></p>
   * @default false
   */
  override public native function get frame():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set frame(value:Boolean):void;

  [ExtConfig]
  /**
   * The name of the element property in this Panel to mask when masked by a LoadMask.
   *
   * <p>Defaults to <code>"el"</code> to indicate that any LoadMask should be rendered into this Panel's encapsulating element.</p>
   *
   * <p>This could be configured to be <code>"body"</code> so that only the body is masked and toolbars and the header are still mouse-accessible.</p>
   * @default "el"
   */
  override public native function get maskElement():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set maskElement(value:String):void;

  /**
   * The Panel's body <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> which may be used to contain HTML content.
   * The content may be specified in the <a href="#!/api/Ext.panel.Panel-cfg-html" rel="Ext.panel.Panel-cfg-html" class="docClass">html</a> config, or it may be loaded using the
   * <a href="#!/api/Ext.panel.Panel-cfg-loader" rel="Ext.panel.Panel-cfg-loader" class="docClass">loader</a> config. Read-only.
   *
   * <p>If this is used to load visible HTML elements in either way, then
   * the Panel may not be used as a Layout for hosting nested Panels.</p>
   *
   * <p>If this Panel is intended to be used as the host of a Layout (See <a href="#!/api/Ext.panel.Panel-cfg-layout" rel="Ext.panel.Panel-cfg-layout" class="docClass">layout</a>
   * then the body Element must not be loaded or changed - it is under the control
   * of the Panel's Layout.</p>
   */
  public native function get body():ext.dom.Element;

  /**
   * Only present if this Panel has been configured with <a href="#!/api/Ext.panel.Panel-cfg-draggable" rel="Ext.panel.Panel-cfg-draggable" class="docClass">draggable</a> <code>true</code>.
   *
   * <h2>Simple dragging</h2>
   *
   * <p>If this Panel is configured <a href="#!/api/Ext.panel.Panel-cfg-simpleDrag" rel="Ext.panel.Panel-cfg-simpleDrag" class="docClass">simpleDrag</a> <code>true</code> (the default is <code>false</code>), this property
   * will reference an instance of <a href="#!/api/Ext.util.ComponentDragger" rel="Ext.util.ComponentDragger" class="docClass">Ext.util.ComponentDragger</a> (A subclass of
   * <a href="#!/api/Ext.dd.DragTracker" rel="Ext.dd.DragTracker" class="docClass">DragTracker</a>) which handles moving the Panel's DOM Element,
   * and constraining according to the <a href="#!/api/Ext.panel.Panel-cfg-constrain" rel="Ext.panel.Panel-cfg-constrain" class="docClass">constrain</a> and <a href="#!/api/Ext.panel.Panel-cfg-constrainHeader" rel="Ext.panel.Panel-cfg-constrainHeader" class="docClass">constrainHeader</a> .</p>
   *
   * <p>This object fires various events during its lifecycle and during a drag operation.</p>
   *
   * <h2>Complex dragging interacting with other DragDrop instances</h2>
   *
   * <p>By default, this property in a <a href="#!/api/Ext.panel.Panel-cfg-draggable" rel="Ext.panel.Panel-cfg-draggable" class="docClass">draggable</a> Panel will contain an instance of <a href="#!/api/Ext.dd.DragSource" rel="Ext.dd.DragSource" class="docClass">Ext.dd.DragSource</a> which handles dragging the Panel.</p>
   *
   * <p>The developer must provide implementations of the abstract methods of <a href="#!/api/Ext.dd.DragSource" rel="Ext.dd.DragSource" class="docClass">Ext.dd.DragSource</a> in order to
   * supply behaviour for each stage of the drag/drop process.</p>
   *
   * <p>See also <a href="#!/api/Ext.panel.Panel-cfg-draggable" rel="Ext.panel.Panel-cfg-draggable" class="docClass">draggable</a>.</p>
   */
  public native function get dd():ext.dd.DragSource;

  /**
   * @private
   */
  public native function set dd(value:ext.dd.DragSource):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Panel, or subclass thereof.
   * @default true
   */
  public native function get isPanel():Boolean;

  /**
   * @private
   */
  public native function set isPanel(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to animate the transition when the panel is collapsed, <code>false</code> to skip the animation (defaults to <code>true</code>
   * if the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> class is available, otherwise <code>false</code>). May also be specified as the animation
   * duration in milliseconds.
   */
  public native function get animCollapse():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animCollapse(value:*):void;

  [ExtConfig]
  /**
   * Convenience config. Short for 'Bottom Bar'.
   *
   * <pre><code>bbar: [
   *   { xtype: 'button', text: 'Button 1' }
   * ]
   * </code></pre>
   *
   * <p>is equivalent to</p>
   *
   * <pre><code>dockedItems: [{
   *     xtype: 'toolbar',
   *     dock: 'bottom',
   *     items: [
   *         { xtype: 'button', text: 'Button 1' }
   *     ]
   * }]
   * </code></pre>
   */
  public native function get bbar():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bbar(value:*):void;

  [ExtConfig]
  /**
   * A shortcut to add or remove the border on the body of a panel. In the classic theme
   * this only applies to a panel which has the <a href="#!/api/Ext.panel.Panel-cfg-frame" rel="Ext.panel.Panel-cfg-frame" class="docClass">frame</a> configuration set to <code>true</code>.
   * @since 2.3.0
   */
  public native function get bodyBorder():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bodyBorder(value:Boolean):void;

  [ExtConfig]
  /**
   * A CSS class, space-delimited string of classes, or array of classes to be applied to the panel's body element.
   * The following examples are all valid:
   *
   * <pre><code>bodyCls: 'foo'
   * bodyCls: 'foo bar'
   * bodyCls: ['foo', 'bar']
   * </code></pre>
   */
  public native function get bodyCls():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bodyCls(value:*):void;

  [ExtConfig]
  /**
   * A shortcut for setting a padding style on the body element. The value can either be
   * a number to be applied to all sides, or a normal css string describing padding.
   * @default undefined
   */
  public native function get bodyPadding():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bodyPadding(value:*):void;

  [ExtConfig]
  /**
   * Custom CSS styles to be applied to the panel's body element, which can be supplied as a valid CSS style string,
   * an object containing style property name/value pairs or a function that returns such a string or object.
   * For example, these two formats are interpreted to be equivalent:
   *
   * <pre><code>bodyStyle: 'background:#ffc; padding:10px;'
   *
   * bodyStyle: {
   *     background: '#ffc',
   *     padding: '10px'
   * }
   * </code></pre>
   * @since 2.3.0
   */
  public native function get bodyStyle():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bodyStyle(value:*):void;

  [ExtConfig]
  /**
   * The alignment of any buttons added to this panel. Valid values are 'right', 'left' and 'center' (defaults to
   * 'right' for buttons/fbar, 'left' for other toolbar types).
   *
   * <p><strong>NOTE:</strong> The preferred way to specify toolbars is to use the dockedItems config. Instead of buttonAlign you
   * would add the layout: { pack: 'start' | 'center' | 'end' } option to the dockedItem config.</p>
   */
  public native function get buttonAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttonAlign(value:String):void;

  [ExtConfig]
  /**
   * Convenience config used for adding buttons docked to the bottom of the panel. This is a
   * synonym for the <a href="#!/api/Ext.panel.Panel-cfg-fbar" rel="Ext.panel.Panel-cfg-fbar" class="docClass">fbar</a> config.
   *
   * <pre><code>buttons: [
   *   { text: 'Button 1' }
   * ]
   * </code></pre>
   *
   * <p>is equivalent to</p>
   *
   * <pre><code>dockedItems: [{
   *     xtype: 'toolbar',
   *     dock: 'bottom',
   *     ui: 'footer',
   *     defaults: {
   *         minWidth: 200
   *     },
   *     items: [
   *         { xtype: 'component', flex: 1 },
   *         { xtype: 'button', text: 'Button 1' }
   *     ]
   * }]
   * </code></pre>
   *
   * <p>The <a href="#!/api/Ext.panel.Panel-cfg-minButtonWidth" rel="Ext.panel.Panel-cfg-minButtonWidth" class="docClass">minButtonWidth</a> is used as the default <a href="#!/api/Ext.button.Button-cfg-minWidth" rel="Ext.button.Button-cfg-minWidth" class="docClass">minWidth</a> for
   * each of the buttons in the buttons toolbar.</p>
   */
  public native function get buttons():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttons(value:Array):void;

  [ExtConfig]
  /**
   * True to display the 'close' tool button and allow the user to close the window, false to hide the button and
   * disallow closing the window.
   *
   * <p>By default, when close is requested by clicking the close button in the header, the <a href="#!/api/Ext.panel.Panel-method-close" rel="Ext.panel.Panel-method-close" class="docClass">close</a> method will be
   * called. This will <em><a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a></em> the Panel and its content meaning that it may not be
   * reused.</p>
   *
   * <p>To make closing a Panel <em>hide</em> the Panel so that it may be reused, set <a href="#!/api/Ext.panel.Panel-cfg-closeAction" rel="Ext.panel.Panel-cfg-closeAction" class="docClass">closeAction</a> to 'hide'.</p>
   * @default false
   */
  public native function get closable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set closable(value:Boolean):void;

  [ExtConfig]
  /**
   * The action to take when the close header tool is clicked:
   *
   * <ul>
   * <li><p><strong><code>'<a href="#!/api/Ext.panel.Panel-method-destroy" rel="Ext.panel.Panel-method-destroy" class="docClass">destroy</a>'</code></strong> :</p>
   *
   * <p><a href="#!/api/Ext.panel.Panel-method-remove" rel="Ext.panel.Panel-method-remove" class="docClass">remove</a> the window from the DOM and <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a> it and all descendant
   * Components. The window will <strong>not</strong> be available to be redisplayed via the <a href="#!/api/Ext.panel.Panel-method-show" rel="Ext.panel.Panel-method-show" class="docClass">show</a> method.</p></li>
   * <li><p><strong><code>'<a href="#!/api/Ext.panel.Panel-method-hide" rel="Ext.panel.Panel-method-hide" class="docClass">hide</a>'</code></strong> :</p>
   *
   * <p><a href="#!/api/Ext.panel.Panel-method-hide" rel="Ext.panel.Panel-method-hide" class="docClass">hide</a> the window by setting visibility to hidden and applying negative offsets. The window will be
   * available to be redisplayed via the <a href="#!/api/Ext.panel.Panel-method-show" rel="Ext.panel.Panel-method-show" class="docClass">show</a> method.</p></li>
   * </ul>
   *
   *
   * <p><strong>Note:</strong> This behavior has changed! setting <em>does</em> affect the <a href="#!/api/Ext.panel.Panel-method-close" rel="Ext.panel.Panel-method-close" class="docClass">close</a> method which will invoke the
   * appropriate closeAction.</p>
   * @default 'destroy'
   */
  public native function get closeAction():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set closeAction(value:String):void;

  [ExtConfig]
  /**
   * Text to be announced by screen readers when the
   * <strong>close</strong> <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">tool</a> is focused.  Will also be set as the close
   * tool's <a href="#!/api/Ext.panel.Tool-cfg-tooltip" rel="Ext.panel.Tool-cfg-tooltip" class="docClass">tooltip</a> text.
   *
   * <p><strong>Note:</strong> Applicable when the panel is <a href="#!/api/Ext.panel.Panel-cfg-closable" rel="Ext.panel.Panel-cfg-closable" class="docClass">closable</a>: true</p>
   * @default 'Close panel'
   */
  public native function get closeToolText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set closeToolText(value:String):void;

  [ExtConfig]
  /**
   * The direction to collapse the Panel when the toggle button is clicked.
   *
   * <p>Defaults to the <a href="#!/api/Ext.panel.Panel-cfg-headerPosition" rel="Ext.panel.Panel-cfg-headerPosition" class="docClass">headerPosition</a></p>
   *
   * <p><strong>Important: This config is <em>ignored</em> for <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> Panels which are direct child items of a <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>.</strong></p>
   *
   * <p>Specify as <code>'top'</code>, <code>'bottom'</code>, <code>'left'</code> or <code>'right'</code>.</p>
   */
  public native function get collapseDirection():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseDirection(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to make sure the collapse/expand toggle button always renders first (to the left of) any other tools in
   * the panel's title bar, <code>false</code> to render it last.
   * @default true
   */
  public native function get collapseFirst():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseFirst(value:Boolean):void;

  [ExtConfig]
  /**
   * <strong>Important: this config is only effective for <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> Panels which are direct child items of a
   * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>.</strong>
   *
   * <p>When <em>not</em> a direct child item of a <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>, then the Panel's header
   * remains visible, and the body is collapsed to zero dimensions. If the Panel has no header, then a new header
   * (orientated correctly depending on the <a href="#!/api/Ext.panel.Panel-cfg-collapseDirection" rel="Ext.panel.Panel-cfg-collapseDirection" class="docClass">collapseDirection</a>) will be inserted to show a the title and a re-
   * expand tool.</p>
   *
   * <p>When a child item of a <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>, this config has three possible values:</p>
   *
   * <ul>
   * <li><p><code>undefined</code> - When collapsed, a placeholder <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">Header</a> is injected into the layout to
   * represent the Panel and to provide a UI with a Tool to allow the user to re-expand the Panel.</p></li>
   * <li><p><code>"header"</code> - The Panel collapses to leave its header visible as when not inside a
   * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>.</p></li>
   * <li><p><code>"mini"</code> - The Panel collapses without a visible header.</p></li>
   * </ul>
   */
  public native function get collapseMode():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseMode(value:String):void;

  [ExtConfig]
  /**
   * Text to be announced by screen readers when
   * <strong>collapse</strong> <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">tool</a> is focused.  Will also be set as the
   * collapse tool's <a href="#!/api/Ext.panel.Tool-cfg-tooltip" rel="Ext.panel.Tool-cfg-tooltip" class="docClass">tooltip</a> text.
   *
   * <p><strong>Note:</strong> Applicable when the panel is <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a>: true</p>
   * @default 'Collapse panel'
   */
  public native function get collapseToolText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseToolText(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to render the panel collapsed, <code>false</code> to render it expanded.
   * @default false
   * @see #getCollapsed()
   * @see #setCollapsed()
   */
  public native function get collapsed():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set collapsed(value:Boolean):void;

  [ExtConfig]
  /**
   * A CSS class to add to the panel's element after it has been collapsed.
   * @default 'collapsed'
   */
  public native function get collapsedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapsedCls(value:String):void;

  [ExtConfig]
  /**
   * True to make the panel collapsible and have an expand/collapse toggle Tool added into the header tool button
   * area. False to keep the panel sized either statically, or by an owning layout manager, with no toggle Tool.
   * When a panel is used in a <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>, the <a href="#!/api/Ext.panel.Panel-cfg-floatable" rel="Ext.panel.Panel-cfg-floatable" class="docClass">floatable</a> option
   * can influence the behavior of collapsing.
   * See <a href="#!/api/Ext.panel.Panel-cfg-collapseMode" rel="Ext.panel.Panel-cfg-collapseMode" class="docClass">collapseMode</a> and <a href="#!/api/Ext.panel.Panel-cfg-collapseDirection" rel="Ext.panel.Panel-cfg-collapseDirection" class="docClass">collapseDirection</a>
   */
  public native function get collapsible():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapsible(value:Boolean):void;

  [ExtConfig]
  /**
   * True to constrain the panel header within its containing element (allowing the panel body to fall outside of
   * its containing element) or false to allow the header to fall outside its containing element.
   * Optionally the entire panel can be constrained using <a href="#!/api/Ext.panel.Panel-cfg-constrain" rel="Ext.panel.Panel-cfg-constrain" class="docClass">constrain</a>.
   * @default false
   */
  public native function get constrainHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * Reference name of the component to act as the default
   * button for this Panel. Default button is activated by pressing Enter key while focus
   * is contained within the Panel's <a href="#!/api/Ext.panel.Panel-cfg-defaultButtonTarget" rel="Ext.panel.Panel-cfg-defaultButtonTarget" class="docClass">defaultButtonTarget</a>.
   *
   * <p>The most obvious use for <code>defaultButton</code> is submitting a form:</p>
   *
   * <pre><code> var loginWindow = new <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>({
   *      autoShow: true,
   *      width: 300,
   *      layout: 'form',
   *      title: 'Enter login information',
   *      referenceHolder: true,
   *      defaultFocus: 'textfield',
   *      defaultButton: 'okButton',
   *
   *      items: [{
   *          xtype: 'textfield',
   *          fieldLabel: 'User name'
   *      }, {
   *          xtype: 'textfield',
   *          fieldLabel: 'Password'
   *      }],
   *
   *      buttons: [{
   *          reference: 'okButton',
   *          text: 'Login',
   *          handler: function() {
   *              <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Submit', 'Your login is being processed');
   *          }
   *      }]
   *  });
   * </code></pre>
   */
  public native function get defaultButton():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultButton(value:String):void;

  [ExtConfig]
  /**
   * Name of the element that will be the target of
   * <a href="#!/api/Ext.panel.Panel-cfg-defaultButton" rel="Ext.panel.Panel-cfg-defaultButton" class="docClass">defaultButton</a> keydown listener. The default element is Panel body, which
   * means that pressing Enter key while focus is on docked items will not fire <code>defaultButton</code>
   * action.
   *
   * <p>If you want <code>defaultButton</code> action to fire in docked items, set this config to <code>"el"</code>.</p>
   */
  public native function get defaultButtonTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultButtonTarget(value:String):void;

  [ExtConfig]
  /**
   * This object holds the default weights applied to dockedItems that have no weight. These start with a
   * weight of 1, to allow negative weights to insert before top items and are odd numbers
   * so that even weights can be used to get between different dock orders.
   *
   * <p>To make default docking order match border layout, do this:</p>
   *
   * <pre><code> Ext.panel.Panel.prototype.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };
   * </code></pre>
   *
   * <p>Changing these defaults as above or individually on this object will effect all Panels.
   * To change the defaults on a single panel, you should replace the entire object:</p>
   *
   * <pre><code> initComponent: function () {
   *      // NOTE: Don't change members of defaultDockWeights since the object is shared.
   *      this.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };
   *
   *      this.callParent();
   *  }
   * </code></pre>
   *
   * <p>To change only one of the default values, you do this:</p>
   *
   * <pre><code> initComponent: function () {
   *      // NOTE: Don't change members of defaultDockWeights since the object is shared.
   *      this.defaultDockWeights = <a href="#!/api/Ext-method-applyIf" rel="Ext-method-applyIf" class="docClass">Ext.applyIf</a>({ top: 10 }, this.defaultDockWeights);
   *
   *      this.callParent();
   *  }
   * </code></pre>
   * @default {top: {render: 1, visual: 1}, left: {render: 3, visual: 5}, right: {render: 5, visual: 7}, bottom: {render: 7, visual: 3}}
   */
  public native function get defaultDockWeights():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultDockWeights(value:Object):void;

  [ExtConfig]
  [ArrayElementType("ext.Component")]
  /**
   * A component or series of components to be added as docked items to this panel. The docked items can be docked to
   * either the top, right, left or bottom of a panel. This is typically used for things like toolbars or tab bars:
   *
   * <pre><code>var panel = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *     dockedItems: [{
   *         xtype: 'toolbar',
   *         dock: 'top',
   *         items: [{
   *             text: 'Docked to the top'
   *         }]
   *     }]
   * });
   * </code></pre>
   */
  public native function get dockedItems():Array;

  [ExtConfig]
  [ArrayElementType("ext.Component")]
  /**
   * @private
   */
  public native function set dockedItems(value:Array):void;

  [ExtConfig]
  /**
   * Text to be announced by screen readers when
   * <strong>expand</strong> <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">tool</a> is focused.  Will also be set as the
   * expand tool's <a href="#!/api/Ext.panel.Tool-cfg-tooltip" rel="Ext.panel.Tool-cfg-tooltip" class="docClass">tooltip</a> text.
   *
   * <p><strong>Note:</strong> Applicable when the panel is <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a>: true</p>
   * @default 'Expand panel'
   */
  public native function get expandToolText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandToolText(value:String):void;

  [ExtConfig]
  /**
   * Convenience config used for adding items to the bottom of the panel. Short for Footer Bar.
   *
   * <pre><code>fbar: [
   *   { type: 'button', text: 'Button 1' }
   * ]
   * </code></pre>
   *
   * <p>is equivalent to</p>
   *
   * <pre><code>dockedItems: [{
   *     xtype: 'toolbar',
   *     dock: 'bottom',
   *     ui: 'footer',
   *     defaults: {
   *         minWidth: 200
   *     },
   *     items: [
   *         { xtype: 'component', flex: 1 },
   *         { xtype: 'button', text: 'Button 1' }
   *     ]
   * }]
   * </code></pre>
   *
   * <p>The <a href="#!/api/Ext.panel.Panel-cfg-minButtonWidth" rel="Ext.panel.Panel-cfg-minButtonWidth" class="docClass">minButtonWidth</a> is used as the default <a href="#!/api/Ext.button.Button-cfg-minWidth" rel="Ext.button.Button-cfg-minWidth" class="docClass">minWidth</a> for
   * each of the buttons in the fbar.</p>
   */
  public native function get fbar():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fbar(value:*):void;

  [ExtConfig]
  /**
   * <strong>Important: This config is only effective for <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> Panels which are direct child items of a
   * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>.</strong>
   *
   * <p>true to allow clicking a collapsed Panel's <a href="#!/api/Ext.panel.Panel-cfg-placeholder" rel="Ext.panel.Panel-cfg-placeholder" class="docClass">placeholder</a> to display the Panel floated above the layout,
   * false to force the user to fully expand a collapsed region by clicking the expand button to see it again.</p>
   * @default true
   */
  public native function get floatable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set floatable(value:Boolean):void;

  [ExtConfig]
  /**
   * True to apply a frame to the panel panels header (if 'frame' is true).
   * @default true
   */
  public native function get frameHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set frameHeader(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  /**
   * Pass as <code>false</code> to prevent a Header from being created and shown.
   *
   * <p>Pass as a config object (optionally containing an <code>xtype</code>) to custom-configure this Panel's header.</p>
   *
   * <p>See <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">Ext.panel.Header</a> for all the options that may be specified here.</p>
   *
   * <p>A <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">panel header</a> is a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> which contains the Panel's <a href="#!/api/Ext.panel.Panel-cfg-title" rel="Ext.panel.Panel-cfg-title" class="docClass">title</a> and <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">tools</a>.
   * You may also configure the Panel's <code>header</code> option with its own child items which go <em>before</em> the <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">tools</a></p>
   *
   * <p>By default the panel <a href="#!/api/Ext.panel.Panel-cfg-title" rel="Ext.panel.Panel-cfg-title" class="docClass">title</a> is inserted after items configured in this config, but before any tools.
   * To insert the title at any point in the full array, specify the <a href="#!/api/Ext.panel.Header-cfg-titlePosition" rel="Ext.panel.Header-cfg-titlePosition" class="docClass">titlePosition</a> config:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *     title: 'Test',
   *     tools: [{
   *         type: 'refresh'
   *     }, {
   *         type: 'help'
   *     }],
   *     titlePosition: 2 // Title will come AFTER the two tools
   *     ...
   * });
   * </code></pre>
   */
  public native function get header():ext.panel.PanelHeader;

  [ExtConfig]
  /**
   * @private
   */
  public native function set header(value:ext.panel.PanelHeader):void;

  [ExtConfig]
  /**
   * Optional CSS class to apply to the header element on mouseover
   */
  public native function get headerOverCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headerOverCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specify as <code>'top'</code>, <code>'bottom'</code>, <code>'left'</code> or <code>'right'</code>.
   * @default 'top'
   * @see #getHeaderPosition()
   * @see #setHeaderPosition()
   */
  public native function get headerPosition():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set headerPosition(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to hide the expand/collapse toggle button when <code><a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> == true</code>, <code>false</code> to display it.
   * @default false
   */
  public native function get hideCollapseTool():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideCollapseTool(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.panel.Panel-cfg-iconCls" rel="Ext.panel.Panel-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * @see #getIcon()
   */
  public native function get icon():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The side of the title to render the icon.
   * @see #getIconAlign()
   * @see #setIconAlign()
   */
  public native function get iconAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconAlign(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  /**
   * Convenience config. Short for 'Left Bar' (left-docked, vertical toolbar).
   *
   * <pre><code>lbar: [
   *   { xtype: 'button', text: 'Button 1' }
   * ]
   * </code></pre>
   *
   * <p>is equivalent to</p>
   *
   * <pre><code>dockedItems: [{
   *     xtype: 'toolbar',
   *     dock: 'left',
   *     items: [
   *         { xtype: 'button', text: 'Button 1' }
   *     ]
   * }]
   * </code></pre>
   */
  public native function get lbar():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lbar(value:*):void;

  [ExtConfig]
  /**
   * When true, the dock component layout writes
   * height information to the panel's DOM elements based on its shrink wrap height
   * calculation. This ensures that the browser respects the calculated height.
   * When false, the dock component layout will not write heights on the panel or its
   * body element. In some simple layout cases, not writing the heights to the DOM may
   * be desired because this allows the browser to respond to direct DOM manipulations
   * (like animations).
   * @default true
   */
  public native function get manageHeight():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set manageHeight(value:Boolean):void;

  [ExtConfig]
  /**
   * Minimum width of all footer toolbar buttons in pixels. If set, this will be used as the default
   * value for the <a href="#!/api/Ext.button.Button-cfg-minWidth" rel="Ext.button.Button-cfg-minWidth" class="docClass">Ext.button.Button.minWidth</a> config of each Button added to the <strong>footer toolbar</strong> via the
   * <a href="#!/api/Ext.panel.Panel-cfg-fbar" rel="Ext.panel.Panel-cfg-fbar" class="docClass">fbar</a> or <a href="#!/api/Ext.panel.Panel-cfg-buttons" rel="Ext.panel.Panel-cfg-buttons" class="docClass">buttons</a> configurations. It will be ignored for buttons that have a minWidth configured
   * some other way, e.g. in their own config object or via the <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults</a> of
   * their parent container.
   * @default 75
   */
  public native function get minButtonWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minButtonWidth(value:Number):void;

  [ExtConfig]
  /**
   * True to overlap the header in a panel over the framing of the panel itself. This is needed when frame:true (and
   * is done automatically for you). Otherwise it is undefined. If you manually add rounded corners to a panel header
   * which does not have frame:true, this will need to be set to true.
   */
  public native function get overlapHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overlapHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * <strong>Important: This config is only effective for <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> Panels which are direct child items of a
   * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a> when not using the <code>'header'</code> <a href="#!/api/Ext.panel.Panel-cfg-collapseMode" rel="Ext.panel.Panel-cfg-collapseMode" class="docClass">collapseMode</a>.</strong>
   *
   * <p><strong>Optional.</strong> A Component (or config object for a Component) to show in place of this Panel when this Panel is
   * collapsed by a <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>. Defaults to a generated <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">Header</a> containing a <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">Tool</a> to re-expand the Panel.</p>
   */
  public native function get placeholder():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set placeholder(value:*):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.dom.Element-method-setVisibilityMode" rel="Ext.dom.Element-method-setVisibilityMode" class="docClass">mode</a> for hiding collapsed panels when
   * using <a href="#!/api/Ext.panel.Panel-cfg-collapseMode" rel="Ext.panel.Panel-cfg-collapseMode" class="docClass">collapseMode</a> "placeholder".
   * @default Ext.Element.VISIBILITY
   */
  public native function get placeholderCollapseHideMode():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set placeholderCollapseHideMode(value:Number):void;

  [Deprecated(replacement = "header", since = "4.1.0")]
  [ExtConfig]
  /**
   * @default false
   */
  public native function get preventHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * Convenience config. Short for 'Right Bar' (right-docked, vertical toolbar).
   *
   * <pre><code>rbar: [
   *   { xtype: 'button', text: 'Button 1' }
   * ]
   * </code></pre>
   *
   * <p>is equivalent to</p>
   *
   * <pre><code>dockedItems: [{
   *     xtype: 'toolbar',
   *     dock: 'right',
   *     items: [
   *         { xtype: 'button', text: 'Button 1' }
   *     ]
   * }]
   * </code></pre>
   */
  public native function get rbar():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rbar(value:*):void;

  [ExtConfig]
  /**
   * Allows for this panel to include the <a href="#!/api/Ext.panel.Panel-cfg-dockedItems" rel="Ext.panel.Panel-cfg-dockedItems" class="docClass">dockedItems</a> when trying to determine
   * the overall size of the panel. This option is only applicable when this panel is
   * also shrink wrapping in the same dimensions. See <a href="#!/api/Ext.panel.Panel-cfg-shrinkWrap" rel="Ext.panel.Panel-cfg-shrinkWrap" class="docClass">Ext.Panel.shrinkWrap</a> for
   * an explanation of the configuration options.
   * @default false
   */
  public native function get shrinkWrapDock():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set shrinkWrapDock(value:*):void;

  [ExtConfig]
  /**
   * When <a href="#!/api/Ext.panel.Panel-cfg-draggable" rel="Ext.panel.Panel-cfg-draggable" class="docClass">draggable</a> is <code>true</code>, Specify this as <code>true</code> to  cause the <code>draggable</code> config
   * to work the same as it does in <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>. This Panel
   * just becomes movable. No DragDrop instances receive any notifications.
   * For example:
   *
   * <pre class='inline-example '><code>var win = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('widget.window', {
   *     height: 300,
   *     width: 300,
   *     title: 'Constraining Window',
   *     closable: false,
   *     items: {
   *         title: "Floating Panel",
   *         width: 100,
   *         height: 100,
   *         floating: true,
   *         draggable: true,
   *         constrain: true,
   *         simpleDrag: true
   *     }
   * });
   * win.show();
   * // Floating components begin life hidden
   * win.child('[title=Floating Panel]').show();
   * </code></pre>
   * @default false
   */
  public native function get simpleDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set simpleDrag(value:Boolean):void;

  [ExtConfig]
  /**
   * Convenience config. Short for 'Top Bar'.
   *
   * <pre><code>tbar: [
   *   { xtype: 'button', text: 'Button 1' }
   * ]
   * </code></pre>
   *
   * <p>is equivalent to</p>
   *
   * <pre><code>dockedItems: [{
   *     xtype: 'toolbar',
   *     dock: 'top',
   *     items: [
   *         { xtype: 'button', text: 'Button 1' }
   *     ]
   * }]
   * </code></pre>
   */
  public native function get tbar():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tbar(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The title text or config object for the <a href="#!/api/Ext.panel.Title" rel="Ext.panel.Title" class="docClass">Title</a> component.
   * @see #getTitle()
   * @see #setTitle()
   */
  public native function get title():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set title(value:*):void;

  [ExtConfig("title")]
  /**
   * Alias for <code>title</code> to allow specifying the title as a <code>PanelTitle</code> config object.
   * @see #title
   */
  public native function set titleConfig(value:ext.panel.PanelTitle):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The alignment of the title text within the available space between the
   * icon and the tools.
   * @see #getTitleAlign()
   * @see #setTitleAlign()
   */
  public native function get titleAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set titleAlign(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to allow expanding and collapsing the panel (when <code><a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> = true</code>) by clicking anywhere in
   * the header bar, <code>false</code>) to allow it only by clicking to tool button). When a panel is used in a
   * <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>, the <a href="#!/api/Ext.panel.Panel-cfg-floatable" rel="Ext.panel.Panel-cfg-floatable" class="docClass">floatable</a> option can influence the behavior of collapsing.
   */
  public native function get titleCollapse():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set titleCollapse(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The rotation of the header's title text.  Can be one of the following values:
   *
   * <ul>
   * <li><code>'default'</code> - use the default rotation, depending on the dock position of the header</li>
   * <li><code>0</code> - no rotation</li>
   * <li><code>1</code> - rotate 90deg clockwise</li>
   * <li><code>2</code> - rotate 90deg counter-clockwise</li>
   * </ul>
   *
   *
   * <p>The default behavior of this config depends on the dock position of the header:</p>
   *
   * <ul>
   * <li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>
   * <li><code>'right'</code> - <code>1</code></li>
   * <li><code>'left'</code> - <code>1</code></li>
   * </ul>
   * @see #getTitleRotation()
   * @see #setTitleRotation()
   */
  public native function get titleRotation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set titleRotation(value:*):void;

  [ExtConfig]
  /**
   * An array of <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">Ext.panel.Tool</a> configs/instances to be added to the header tool area. The tools are stored as
   * child components of the header container. They can be accessed using <a href="#!/api/Ext.panel.Panel-method-down" rel="Ext.panel.Panel-method-down" class="docClass">down</a> and {<a href="#!/api/Ext.panel.Panel-method-query" rel="Ext.panel.Panel-method-query" class="docClass">query</a>}, as well as the
   * other component methods. The toggle tool is automatically created if <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> is set to true.
   *
   * <p>Note that, apart from the toggle tool which is provided when a panel is collapsible, these tools only provide the
   * visual button. Any required functionality must be provided by adding handlers that implement the necessary
   * behavior.</p>
   *
   * <p>Example usage:</p>
   *
   * <pre><code>tools:[{
   *     type:'refresh',
   *     tooltip: 'Refresh form Data',
   *     // hidden:true,
   *     handler: function(event, toolEl, panelHeader) {
   *         // refresh logic
   *     }
   * },
   * {
   *     type:'help',
   *     tooltip: 'Get Help',
   *     callback: function(panel, tool, event) {
   *         // show help here
   *     }
   * }]
   * </code></pre>
   *
   * <p>The difference between <code>handler</code> and <code>callback</code> is the signature. For details on
   * the distinction, see <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">Ext.panel.Tool</a>.</p>
   */
  public native function get tools():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tools(value:*):void;

  /**
   * Attempts a default component lookup (see <a href="#!/api/Ext.container.Container-method-getComponent" rel="Ext.container.Container-method-getComponent" class="docClass">Ext.container.Container.getComponent</a>). If the component is not found in the normal
   * items, the dockedItems are searched and the matched component (if any) returned (see <a href="#!/api/Ext.panel.Panel-method-getDockedComponent" rel="Ext.panel.Panel-method-getDockedComponent" class="docClass">getDockedComponent</a>). Note that docked
   * items will only be matched by component id or itemId -- if you pass a numeric index only non-docked child components will be searched.
   * @since 2.3.0
   * @param comp The component id, itemId or position to find
   * @return The component (if found)
   */
  override public native function getComponent(comp:*):ext.Component;

  [Deprecated(message = "Use &lt;code>getDockedItems(...)&lt;/code>, &lt;code>down(...)&lt;/code> or &lt;code>query(...)&lt;/code> with selector &lt;code>&#39;toolbar[dock=\"top\"]&#39;&lt;/code> instead.\n\n", since = "6.0")]
  /**
   * Returns the <a href="Ext.Toolbar.html">toolbar</a> from the top (<code><a href="output/Ext.Panel.html#Ext.Panel-tbar">tbar</a></code>) section of the panel.
   *
   * @return The toolbar
   */
  public native function getTopToolbar():ext.toolbar.Toolbar;

  [Deprecated(message = "Use &lt;code>getDockedItems(...)&lt;/code>, &lt;code>down(...)&lt;/code> or &lt;code>query(...)&lt;/code> with selector &lt;code>&#39;toolbar[dock=\"bottom\"][ui!=\"footer\"]&#39;&lt;/code> instead. Use selector &lt;code>&#39;toolbar[dock=\"bottom\"]&#39;&lt;/code> for not filtering out footer toolbars. Adapt &lt;code>ui&lt;/code> parameter if your footer UI style has a different name.\n\n", since = "6.0")]
  /**
   * Returns the <a href="Ext.Toolbar.html">toolbar</a> from the bottom (<code><a href="output/Ext.Panel.html#Ext.Panel-bbar">bbar</a></code>) section of the panel.
   *
   * @return The toolbar
   */
  public native function getBottomToolbar():ext.toolbar.Toolbar;

  [Deprecated(message = "Use &lt;code>getDockedItems(...)&lt;/code>, &lt;code>down(...)&lt;/code> or &lt;code>query(...)&lt;/code> with selector &lt;code>&#39;toolbar[dock=\"bottom\"][ui=\"footer\"]&#39;&lt;/code> instead. Adapt &lt;code>ui&lt;/code> parameter if your footer UI style has a different name.\n\n", since = "6.0")]
  /**
   * Returns the <a href="Ext.Toolbar.html">toolbar</a> from the footer (<code><a href="output/Ext.Panel.html#Ext.Panel-fbar">fbar</a></code>) section of the panel.
   *
   * @return The toolbar
   */
  public native function getFooterToolbar():ext.toolbar.Toolbar;

  /**
   * Adds a CSS class to the body element. If not rendered, the class will
   * be added when the panel is rendered.
   * @param cls The class to add
   * @return this
   */
  public native function addBodyCls(cls:*):ext.panel.Panel;

  /**
   * Adds docked item(s) to the container.
   * @param items The Component or array of components to add. The components
   * must include a 'dock' parameter on each component to indicate where it should be docked
   * ('top', 'right', 'bottom', 'left').
   * @param pos The index at which the Component will be added
   * @return The added components.
   */
  public native function addDocked(items:*, pos:Number = NaN):Array;

  /**
   * Add tools to this panel
   * @param tools The tools to add.
   * <p>By default the tools will be accessible via keyboard, with the exception
   * of automatically added collapse/expand and close tools.</p>
   * <p>If you implement keyboard equivalents of your tools' actions elsewhere
   * and do not want the tools to participate in keyboard navigation, you can
   * make them presentational instead:</p>
   * <pre><code> panel.addTool({
   *      type: 'mytool',
   *      focusable: false,
   *      ariaRole: 'presentation',
   *      ...
   *  });
   * </code></pre>
   */
  public native function addTool(tools:*):void;

  /**
   * Closes the Panel. By default, this method, removes it from the DOM, <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a>s the
   * Panel object and all its descendant Components. The <a href="#!/api/Ext.panel.Panel-event-beforeclose" rel="Ext.panel.Panel-event-beforeclose" class="docClass">beforeclose</a> event is fired before the
   * close happens and will cancel the close action if it returns false.
   *
   * <p><strong>Note:</strong> This method is also affected by the <a href="#!/api/Ext.panel.Panel-cfg-closeAction" rel="Ext.panel.Panel-cfg-closeAction" class="docClass">closeAction</a> setting. For more explicit control use
   * <a href="#!/api/Ext.panel.Panel-method-destroy" rel="Ext.panel.Panel-method-destroy" class="docClass">destroy</a> and <a href="#!/api/Ext.panel.Panel-method-hide" rel="Ext.panel.Panel-method-hide" class="docClass">hide</a> methods.</p>
   */
  public native function close():void;

  /**
   * Collapses the panel body so that the body becomes hidden. Docked Components parallel to the border towards which
   * the collapse takes place will remain visible. Fires the <a href="#!/api/Ext.panel.Panel-event-beforecollapse" rel="Ext.panel.Panel-event-beforecollapse" class="docClass">beforecollapse</a> event which will cancel the
   * collapse action if it returns false.
   * @param direction The direction to collapse towards. Must be one of
   * <ul>
   * <li><a href="#!/api/Ext.Component-static-property-DIRECTION_TOP" rel="Ext.Component-static-property-DIRECTION_TOP" class="docClass">Ext.Component.DIRECTION_TOP</a></li>
   * <li><a href="#!/api/Ext.Component-static-property-DIRECTION_RIGHT" rel="Ext.Component-static-property-DIRECTION_RIGHT" class="docClass">Ext.Component.DIRECTION_RIGHT</a></li>
   * <li><a href="#!/api/Ext.Component-static-property-DIRECTION_BOTTOM" rel="Ext.Component-static-property-DIRECTION_BOTTOM" class="docClass">Ext.Component.DIRECTION_BOTTOM</a></li>
   * <li><a href="#!/api/Ext.Component-static-property-DIRECTION_LEFT" rel="Ext.Component-static-property-DIRECTION_LEFT" class="docClass">Ext.Component.DIRECTION_LEFT</a></li>
   * </ul>
   * <p>Defaults to <a href="#!/api/Ext.panel.Panel-cfg-collapseDirection" rel="Ext.panel.Panel-cfg-collapseDirection" class="docClass">collapseDirection</a>.</p>
   * @param animate True to animate the transition, else false
   * (defaults to the value of the <a href="#!/api/Ext.panel.Panel-cfg-animCollapse" rel="Ext.panel.Panel-cfg-animCollapse" class="docClass">animCollapse</a> panel config). May
   * also be specified as the animation duration in milliseconds.
   * @return this
   */
  public native function collapse(direction:String = null, animate:* = undefined):ext.panel.Panel;

  /**
   * converts a collapsdDir into an anchor argument for Element.slideIn
   * overridden in rtl mode to switch "l" and "r"
   */
  public native function convertCollapseDir(collapseDir:Object):void;

  /**
   * Expands the panel body so that it becomes visible.  Fires the <a href="#!/api/Ext.panel.Panel-event-beforeexpand" rel="Ext.panel.Panel-event-beforeexpand" class="docClass">beforeexpand</a> event which will
   * cancel the expand action if it returns false.
   * @param animate True to animate the transition, else false
   * (defaults to the value of the <a href="#!/api/Ext.panel.Panel-cfg-animCollapse" rel="Ext.panel.Panel-cfg-animCollapse" class="docClass">animCollapse</a> panel config).  May
   * also be specified as the animation duration in milliseconds.
   * @return this
   */
  public native function expand(animate:Boolean = false):ext.panel.Panel;

  /**
   * Returns the current collapsed state of the panel.
   * @return False when not collapsed, otherwise the value of <a href="#!/api/Ext.panel.Panel-cfg-collapseDirection" rel="Ext.panel.Panel-cfg-collapseDirection" class="docClass">collapseDirection</a>.
   */
  public native function getCollapsed():*;

  /**
   * Finds a docked component by id, itemId or position. Also see <a href="#!/api/Ext.container.DockingContainer-method-getDockedItems" rel="Ext.container.DockingContainer-method-getDockedItems" class="docClass">getDockedItems</a>
   * @param comp The id, itemId or position of the docked component (see <a href="#!/api/Ext.container.Container-method-getComponent" rel="Ext.container.Container-method-getComponent" class="docClass">getComponent</a> for details)
   * @return The docked component (if found)
   */
  public native function getDockedComponent(comp:*):ext.Component;

  /**
   * Retrieves an array of all currently docked Components.
   *
   * <p>For example to find a toolbar that has been docked at top:</p>
   *
   * <pre><code>panel.getDockedItems('toolbar[dock="top"]');
   * </code></pre>
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector string to filter the returned items.
   * @param beforeBody An optional flag to limit the set of items to only those
   *  before the body (true) or after the body (false). All components are returned if this flag is not set.
   * @return The array of docked components meeting the specified criteria.
   */
  public native function getDockedItems(selector:String, beforeBody:Boolean = undefined):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-glyph" rel="Ext.panel.Panel-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function getGlyph():Object;

  /**
   * Gets the <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">Header</a> for this panel.
   */
  public native function getHeader():ext.panel.PanelHeader;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-headerPosition" rel="Ext.panel.Panel-cfg-headerPosition" class="docClass">headerPosition</a>.
   */
  public native function getHeaderPosition():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-icon" rel="Ext.panel.Panel-cfg-icon" class="docClass">icon</a>.
   */
  public native function getIcon():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-iconAlign" rel="Ext.panel.Panel-cfg-iconAlign" class="docClass">iconAlign</a>.
   */
  public native function getIconAlign():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-iconCls" rel="Ext.panel.Panel-cfg-iconCls" class="docClass">iconCls</a>.
   */
  public native function getIconCls():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-title" rel="Ext.panel.Panel-cfg-title" class="docClass">title</a>.
   */
  public native function getTitle():*;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-titleAlign" rel="Ext.panel.Panel-cfg-titleAlign" class="docClass">titleAlign</a>.
   */
  public native function getTitleAlign():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Panel-cfg-titleRotation" rel="Ext.panel.Panel-cfg-titleRotation" class="docClass">titleRotation</a>.
   */
  public native function getTitleRotation():Object;

  /**
   * Inserts docked item(s) to the panel at the indicated position.
   * @param pos The index at which the Component will be inserted
   * @param items The Component or array of components to add. The components
   * must include a 'dock' paramater on each component to indicate where it should be docked ('top', 'right',
   * 'bottom', 'left').
   */
  public native function insertDocked(pos:Number, items:*):void;

  /**
   * Removes a CSS class from the body element.
   * @param cls The class to remove
   * @return this
   */
  public native function removeBodyCls(cls:*):ext.panel.Panel;

  /**
   * Removes the docked item from the panel.
   * @param item The Component to remove.
   * @param autoDestroy Destroy the component after removal.
   */
  public native function removeDocked(item:ext.Component, autoDestroy:Boolean = false):void;

  /**
   * Sets the body style according to the passed parameters.
   * @param style A full style specification string, or object, or the name of a style property to set.
   * @param value If the first param was a style property name, the style property value.
   * @return this
   */
  public native function setBodyStyle(style:*, value:String):ext.panel.Panel;

  /**
   * Collapses or expands the panel.
   * @param collapsed <code>true</code> to collapse the panel, <code>false</code> to expand it.
   */
  public native function setCollapsed(collapsed:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Panel-cfg-headerPosition" rel="Ext.panel.Panel-cfg-headerPosition" class="docClass">headerPosition</a>.
   * @param headerPosition The new value.
   */
  public native function setHeaderPosition(headerPosition:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Panel-cfg-iconAlign" rel="Ext.panel.Panel-cfg-iconAlign" class="docClass">iconAlign</a>.
   * @param iconAlign The new value.
   */
  public native function setIconAlign(iconAlign:Object):void;

  /**
   * Sets the title of this panel.
   * @param title The new title
   */
  public native function setTitle(title:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Panel-cfg-titleAlign" rel="Ext.panel.Panel-cfg-titleAlign" class="docClass">titleAlign</a>.
   * @param titleAlign The new value.
   */
  public native function setTitleAlign(titleAlign:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Panel-cfg-titleRotation" rel="Ext.panel.Panel-cfg-titleRotation" class="docClass">titleRotation</a>.
   * @param titleRotation The new value.
   */
  public native function setTitleRotation(titleRotation:Object):void;

  /**
   * Shortcut for performing an <a href="#!/api/Ext.panel.Panel-method-expand" rel="Ext.panel.Panel-method-expand" class="docClass">expand</a> or <a href="#!/api/Ext.panel.Panel-method-collapse" rel="Ext.panel.Panel-method-collapse" class="docClass">collapse</a> based on the current state of the panel.
   * @return this
   */
  public native function toggleCollapse():ext.panel.Panel;

  /**
   * Template method to be implemented in subclasses to add their tools after the collapsible tool.
   */
  protected native function addTools():void;

  /**
   * Invoked after the Panel is Collapsed.
   */
  protected native function afterCollapse(animated:Boolean):void;

  /**
   * Invoked after the Panel is Expanded.
   */
  protected native function afterExpand(animated:Boolean):void;

  /**
   * Invoked after a docked item is added to the Panel.
   */
  protected native function onDockedAdd(component:ext.Component):void;

  /**
   * Invoked after a docked item is removed from the Panel.
   */
  protected native function onDockedRemove(component:ext.Component):void;
}
}