package ext.picker {
import ext.Component;

[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.picker.Date", require)]
/**
 * Fires when a date is selected
 * @eventType ext.picker.events.DatePicker_dateEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.picker.events.DatePicker_dateEvent')]
/**
 * A date picker. This class is used by the <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a> field to allow browsing and selection of valid
 * dates in a popup next to the field, but may also be used with other components.
 *
 * <p>Typically you will need to implement a handler function to be notified when the user chooses a date from the picker;
 * you can register the handler using the <a href="#!/api/Ext.picker.Date-event-select" rel="Ext.picker.Date-event-select" class="docClass">select</a> event, or by implementing the <a href="#!/api/Ext.picker.Date-cfg-handler" rel="Ext.picker.Date-cfg-handler" class="docClass">handler</a> method.</p>
 *
 * <p>By default the user will be allowed to pick any date; this can be changed by using the <a href="#!/api/Ext.picker.Date-cfg-minDate" rel="Ext.picker.Date-cfg-minDate" class="docClass">minDate</a>,
 * <a href="#!/api/Ext.picker.Date-cfg-maxDate" rel="Ext.picker.Date-cfg-maxDate" class="docClass">maxDate</a>, <a href="#!/api/Ext.picker.Date-cfg-disabledDays" rel="Ext.picker.Date-cfg-disabledDays" class="docClass">disabledDays</a>, <a href="#!/api/Ext.picker.Date-cfg-disabledDatesRE" rel="Ext.picker.Date-cfg-disabledDatesRE" class="docClass">disabledDatesRE</a>, and/or <a href="#!/api/Ext.picker.Date-cfg-disabledDates" rel="Ext.picker.Date-cfg-disabledDates" class="docClass">disabledDates</a> configs.</p>
 *
 * <p>All the string values documented below may be overridden by including an Ext locale file in your page.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Choose a future date:',
 *     width: 200,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'datepicker',
 *         minDate: new Date(),
 *         handler: function(picker, date) {
 *             // do something with the selected date
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class DatePicker extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function DatePicker(config:ext.picker.DatePicker = null) {
    super();
  }

  [ExtConfig]
  /**
   * The base CSS class to apply to this components element.
   * @default 'x-datepicker'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  /**
   * The text that Assistive Technologies such as screen readers
   * will announce when the date falls on a disabled date.
   * @default "This date is disabled"
   */
  public native function get ariaDisabledDatesText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDisabledDatesText(value:String):void;

  [ExtConfig]
  /**
   * The text that Assistive Technologies such as screen readers
   * will announce when the date falls on a disabled day of week.
   * @default "This day of week is disabled"
   */
  public native function get ariaDisabledDaysText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDisabledDaysText(value:String):void;

  [ExtConfig]
  /**
   * The text that will be announced by Assistive Technologies
   * such as screen readers when user is navigating to the cell which date is later than
   * <a href="#!/api/Ext.picker.Date-cfg-maxDate" rel="Ext.picker.Date-cfg-maxDate" class="docClass">maxDate</a>.
   * @default "This date is after the maximum date"
   */
  public native function get ariaMaxText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaMaxText(value:String):void;

  [ExtConfig]
  /**
   * The text that will be announced by Assistive Technologies
   * such as screen readers when user is navigating to the cell which date is less than
   * <a href="#!/api/Ext.picker.Date-cfg-minDate" rel="Ext.picker.Date-cfg-minDate" class="docClass">minDate</a>.
   * @default "This date is before the minimum date"
   */
  public native function get ariaMinText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaMinText(value:String):void;

  [ExtConfig]
  /**
   * The text to display for the aria title
   * @default 'Date Picker: {0}'
   */
  public native function get ariaTitle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaTitle(value:String):void;

  [ExtConfig]
  /**
   * The date format to display for the current value in the <a href="#!/api/Ext.picker.Date-cfg-ariaTitle" rel="Ext.picker.Date-cfg-ariaTitle" class="docClass">ariaTitle</a>
   * @default 'F d'
   */
  public native function get ariaTitleDateFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaTitleDateFormat(value:String):void;

  [Deprecated(replacement = "ext.SDateUtil")]
  [ExtConfig]
  /**
   * An array of textual day names which can be overriden for localization support (defaults to <a href="#!/api/Ext.Date-property-dayNames" rel="Ext.Date-property-dayNames" class="docClass">Ext.Date.dayNames</a>)
   */
  public native function get dayNames():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dayNames(value:Array):void;

  [ExtConfig]
  /**
   * True to disable animations when showing the month picker.
   * @default false
   */
  public native function get disableAnim():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disableAnim(value:Boolean):void;

  [ExtConfig]
  /**
   * The class to apply to disabled cells.
   * @default 'x-datepicker-disabled'
   */
  public native function get disabledCellCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledCellCls(value:String):void;

  [ExtConfig]
  /**
   * An array of 'dates' to disable, as strings. These strings will be used to build a dynamic regular expression so
   * they are very powerful. Some examples:
   *
   * <ul>
   * <li>['03/08/2003', '09/16/2003'] would disable those exact dates</li>
   * <li>['03/08', '09/16'] would disable those days for every year</li>
   * <li>['^03/08'] would only match the beginning (useful if you are using short years)</li>
   * <li>['03/../2006'] would disable every day in March 2006</li>
   * <li>['^03'] would disable every day in every March</li>
   * </ul>
   *
   *
   * <p>Note that the format of the dates included in the array should exactly match the <a href="#!/api/Ext.picker.Date-cfg-format" rel="Ext.picker.Date-cfg-format" class="docClass">format</a> config. In order
   * to support regular expressions, if you are using a date format that has '.' in it, you will have to escape the
   * dot when restricting dates. For example: ['03\.08\.03'].</p>
   * @see #setDisabledDates()
   */
  public native function get disabledDates():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabledDates(value:Array):void;

  [ExtConfig]
  /**
   * JavaScript regular expression used to disable a pattern of dates. The <a href="#!/api/Ext.picker.Date-cfg-disabledDates" rel="Ext.picker.Date-cfg-disabledDates" class="docClass">disabledDates</a>
   * config will generate this regex internally, but if you specify disabledDatesRE it will take precedence over the
   * disabledDates value.
   * @default null
   */
  public native function get disabledDatesRE():RegExp;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledDatesRE(value:RegExp):void;

  [ExtConfig]
  /**
   * The tooltip text to display when the date falls on a disabled date.
   * @default 'Disabled'
   */
  public native function get disabledDatesText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledDatesText(value:String):void;

  [ExtConfig]
  /**
   * An array of days to disable, 0-based. For example, [0, 6] disables Sunday and Saturday.
   * @default null
   * @see #setDisabledDays()
   */
  public native function get disabledDays():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabledDays(value:Array):void;

  [ExtConfig]
  /**
   * The tooltip to display when the date falls on a disabled day.
   * @default 'Disabled'
   */
  public native function get disabledDaysText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledDaysText(value:String):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.button.Button-cfg-ui" rel="Ext.button.Button-cfg-ui" class="docClass">Ext.button.Button.ui</a> to use for the date picker's footer buttons.
   * @default 'default'
   */
  public native function get footerButtonUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set footerButtonUI(value:String):void;

  [ExtConfig]
  /**
   * The default date format string which can be overriden for localization support. The format must be valid
   * according to <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a> (defaults to <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a>).
   */
  public native function get format():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set format(value:String):void;

  [ExtConfig]
  /**
   * Optional. A function that will handle the select event of this picker. The handler is passed the following
   * parameters:
   *
   * <ul>
   * <li><code>picker</code> : <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Ext.picker.Date</a></li>
   * </ul>
   *
   *
   * <p>This Date picker.</p>
   *
   * <ul>
   * <li><code>date</code> : Date</li>
   * </ul>
   *
   *
   * <p>The selected date.</p>
   */
  public native function get handler():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handler(value:Function):void;

  [ExtConfig]
  /**
   * Specifies optional custom key event handlers for the <a href="#!/api/Ext.util.KeyNav" rel="Ext.util.KeyNav" class="docClass">Ext.util.KeyNav</a> attached to this date picker. Must
   * conform to the config format recognized by the <a href="#!/api/Ext.util.KeyNav" rel="Ext.util.KeyNav" class="docClass">Ext.util.KeyNav</a> constructor. Handlers specified in this
   * object will replace default handlers of the same name.
   */
  public native function get keyNavConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyNavConfig(value:Object):void;

  [ExtConfig]
  /**
   * The format for displaying a date in a longer format.
   * @default 'F d, Y'
   */
  public native function get longDayFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set longDayFormat(value:String):void;

  [ExtConfig]
  /**
   * Maximum allowable date (JavaScript date object)
   * @default null
   * @see #setMaxDate()
   */
  public native function get maxDate():Date;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxDate(value:Date):void;

  [ExtConfig]
  /**
   * The error text to display if the maxDate validation fails.
   * @default 'This date is after the maximum date'
   */
  public native function get maxText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxText(value:String):void;

  [ExtConfig]
  /**
   * Minimum allowable date (JavaScript date object)
   * @default null
   * @see #setMinDate()
   */
  public native function get minDate():Date;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minDate(value:Date):void;

  [ExtConfig]
  /**
   * The error text to display if the minDate validation fails.
   * @default 'This date is before the minimum date'
   */
  public native function get minText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minText(value:String):void;

  [Deprecated(replacement = "ext.SDateUtil")]
  [ExtConfig]
  /**
   * An array of textual month names which can be overriden for localization support (defaults to <a href="#!/api/Ext.Date-property-monthNames" rel="Ext.Date-property-monthNames" class="docClass">Ext.Date.monthNames</a>)
   */
  public native function get monthNames():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set monthNames(value:Array):void;

  [ExtConfig]
  /**
   * The date format for the header month
   * @default 'F Y'
   */
  public native function get monthYearFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set monthYearFormat(value:String):void;

  [ExtConfig]
  /**
   * The header month selector tooltip
   * @default 'Choose a month (Control+Up/Down to move years)'
   */
  public native function get monthYearText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set monthYearText(value:String):void;

  [ExtConfig]
  /**
   * The next month navigation button tooltip
   * @default 'Next Month (Control+Right)'
   */
  public native function get nextText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nextText(value:String):void;

  [ExtConfig]
  /**
   * The previous month navigation button tooltip
   * @default 'Previous Month (Control+Left)'
   */
  public native function get prevText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set prevText(value:String):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> reference) in which the <code><a href="#!/api/Ext.picker.Date-cfg-handler" rel="Ext.picker.Date-cfg-handler" class="docClass">handler</a></code> function will be called.
   *
   * <p>Defaults to this DatePicker instance.</p>
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * The class to apply to the selected cell.
   * @default 'x-datepicker-selected'
   */
  public native function get selectedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectedCls(value:String):void;

  [ExtConfig]
  /**
   * False to hide the footer area containing the Today button and disable the keyboard handler for spacebar that
   * selects the current date.
   * @default true
   */
  public native function get showToday():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showToday(value:Boolean):void;

  [ExtConfig]
  /**
   * Day index at which the week should begin, 0-based.
   *
   * <p>Defaults to <code>0</code> (Sunday).</p>
   * @default undefined
   */
  public native function get startDay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set startDay(value:Number):void;

  [ExtConfig]
  /**
   * The text to display on the button that selects the current date
   * @default 'Today'
   */
  public native function get todayText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set todayText(value:String):void;

  [ExtConfig]
  /**
   * A string used to format the message for displaying in a tooltip over the button that selects the current date.
   * The <code>{0}</code> token in string is replaced by today's date.
   * @default '{0} (Spacebar)'
   */
  public native function get todayTip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set todayTip(value:String):void;

  /**
   * Gets a single character to represent the day of the week
   * @return The character
   */
  public native function getDayInitial(value:Object):String;

  /**
   * Gets the current selected value of the date field
   * @return The selected date
   */
  public native function getValue():Date;

  /**
   * Hides the month picker, if it's visible.
   * @param animate Indicates whether to animate this action. If the animate
   * parameter is not specified, the behavior will use <a href="#!/api/Ext.picker.Date-cfg-disableAnim" rel="Ext.picker.Date-cfg-disableAnim" class="docClass">disableAnim</a> to determine
   * whether to animate or not.
   * @return this
   */
  public native function hideMonthPicker(animate:Boolean = false):ext.picker.DatePicker;

  /**
   * Sets the current value to today.
   * @return this
   */
  public native function selectToday():ext.picker.DatePicker;

  /**
   * Replaces any existing disabled dates with new values and refreshes the DatePicker.
   * @param disabledDates An array of date strings (see the <a href="#!/api/Ext.picker.Date-cfg-disabledDates" rel="Ext.picker.Date-cfg-disabledDates" class="docClass">disabledDates</a> config for
   * details on supported values), or a JavaScript regular expression used to disable a pattern of dates.
   * @return this
   */
  public native function setDisabledDates(disabledDates:*):void;

  /**
   * Replaces any existing disabled days (by index, 0-6) with new values and refreshes the DatePicker.
   * @param disabledDays An array of disabled day indexes. See the <a href="#!/api/Ext.picker.Date-cfg-disabledDays" rel="Ext.picker.Date-cfg-disabledDays" class="docClass">disabledDays</a> config for details
   * on supported values.
   * @return this
   */
  public native function setDisabledDays(disabledDays:Array):void;

  /**
   * Replaces any existing <a href="#!/api/Ext.picker.Date-cfg-maxDate" rel="Ext.picker.Date-cfg-maxDate" class="docClass">maxDate</a> with the new value and refreshes the DatePicker.
   * @param value The maximum date that can be selected
   * @return this
   */
  public native function setMaxDate(value:Date):void;

  /**
   * Replaces any existing <a href="#!/api/Ext.picker.Date-cfg-minDate" rel="Ext.picker.Date-cfg-minDate" class="docClass">minDate</a> with the new value and refreshes the DatePicker.
   * @param value The minimum date that can be selected
   * @return this
   */
  public native function setMinDate(value:Date):void;

  /**
   * Sets the value of the date field
   * @param value The date to set
   * @return this
   */
  public native function setValue(value:Date):void;

  /**
   * Show the month picker
   * @param animate Indicates whether to animate this action. If the animate
   * parameter is not specified, the behavior will use <a href="#!/api/Ext.picker.Date-cfg-disableAnim" rel="Ext.picker.Date-cfg-disableAnim" class="docClass">disableAnim</a> to determine
   * whether to animate or not.
   * @return this
   */
  public native function showMonthPicker(animate:Boolean = false):ext.picker.DatePicker;

  /**
   * Show the next month.
   * @return this
   */
  public native function showNextMonth(e:Object):ext.picker.DatePicker;

  /**
   * Show the next year.
   * @return this
   */
  public native function showNextYear():ext.picker.DatePicker;

  /**
   * Show the previous month.
   * @return this
   */
  public native function showPrevMonth(e:Object):ext.picker.DatePicker;

  /**
   * Show the previous year.
   * @return this
   */
  public native function showPrevYear():ext.picker.DatePicker;
}
}