package ext.plugin {

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Base")]
[Native("Ext.plugin.MouseEnter", require)]
/**
 * This plugin calls a callback whenever the mouse enters or leaves descendant
 * elements of its host component identified by a delegate
 * query selector string.
 *
 * <p>This is useful for components which render arbitrary and transient child elements
 * such as DataViews and Charts. It allows notification of mousenter events from
 * child nodes witghout having to add  listeners to each child element.</p>
 */
public class MouseEnterPlugin extends ext.plugin.AbstractPlugin {
  /**
   * @inheritDoc
   */
  public function MouseEnterPlugin(config:ext.plugin.AbstractPlugin = null) {
    super();
  }

  [ExtConfig]
  /**
   * A query selector string to identify descendant elements
   * which trigger a call to the handler.
   */
  public native function get delegate():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set delegate(value:String):void;

  [ExtConfig]
  /**
   * The element, or component element reference
   * name to which to add the mouse listener.
   * @default "el"
   */
  public native function get element():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set element(value:*):void;

  [ExtConfig]
  /**
   * A callback to invoke when a the mouse enters a
   * descendant delegate.
   * <ul>
   *   <li><code>e</code> : ext.event.Event
   * The <code>mouseover</code> event which triggered the mouse enter.
   *   </li>
   *   <li><code>target</code> : ext.dom.Element
   * The delegate element into which the mouse just entered.
   *   </li>
   * </ul>
   */
  public native function get handler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * A callback to invoke when a the mouse leaves a
   * descendant delegate.
   */
  public native function get leaveHandler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set leaveHandler(value:*):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> pointer) in which to execute the callback(s).
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;
}
}