package ext.resizer {
import ext.Component;

[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.resizer.Splitter", require)]
/**
 * This class functions <strong>between siblings of a <a href="#!/api/Ext.layout.container.VBox" rel="Ext.layout.container.VBox" class="docClass">VBox</a> or <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">HBox</a>
 * layout</strong> to resize both immediate siblings.
 *
 * <p>A Splitter will preserve the flex ratio of any flexed siblings it is required to resize. It does this by setting the <code>flex</code> property of <em>all</em> flexed siblings
 * to equal their pixel size. The actual numerical <code>flex</code> property in the Components will change, but the <strong>ratio</strong> to the total flex value will be preserved.</p>
 *
 * <p>A Splitter may be configured to show a centered mini-collapse tool orientated to collapse the <a href="#!/api/Ext.resizer.Splitter-cfg-collapseTarget" rel="Ext.resizer.Splitter-cfg-collapseTarget" class="docClass">collapseTarget</a>.
 * The Splitter will then call that sibling Panel's <a href="#!/api/Ext.panel.Panel-method-collapse" rel="Ext.panel.Panel-method-collapse" class="docClass">collapse</a> or <a href="#!/api/Ext.panel.Panel-method-expand" rel="Ext.panel.Panel-method-expand" class="docClass">expand</a> method
 * to perform the appropriate operation (depending on the sibling collapse state). To create the mini-collapse tool but take care
 * of collapsing yourself, configure the splitter with <code><a href="#!/api/Ext.resizer.Splitter-cfg-performCollapse" rel="Ext.resizer.Splitter-cfg-performCollapse" class="docClass">performCollapse</a>: false</code>.</p>
 */
public class Splitter extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function Splitter(config:ext.resizer.Splitter = null) {
    super();
  }

  /**
   * Orientation of this Splitter. <code>'vertical'</code> when used in an hbox layout, <code>'horizontal'</code>
   * when used in a vbox layout.
   */
  public native function get orientation():String;

  /**
   * @private
   */
  public native function set orientation(value:String):void;

  [ExtConfig]
  /**
   * True to enable dblclick to toggle expand and collapse on the <a href="#!/api/Ext.resizer.Splitter-cfg-collapseTarget" rel="Ext.resizer.Splitter-cfg-collapseTarget" class="docClass">collapseTarget</a> Panel.
   * @default true
   */
  public native function get collapseOnDblClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseOnDblClick(value:Boolean):void;

  [ExtConfig]
  /**
   * A string describing the relative position of the immediate sibling Panel to collapse. May be 'prev' or 'next'.
   *
   * <p>Or the immediate sibling Panel to collapse.</p>
   *
   * <p>The orientation of the mini-collapse tool will be inferred from this setting.</p>
   *
   * <p><strong>Note that only Panels may be collapsed.</strong></p>
   * @default 'next'
   */
  public native function get collapseTarget():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseTarget(value:*):void;

  [ExtConfig]
  /**
   * A class to add to the splitter when it is collapsed. See <a href="#!/api/Ext.resizer.Splitter-cfg-collapsible" rel="Ext.resizer.Splitter-cfg-collapsible" class="docClass">collapsible</a>.
   */
  public native function get collapsedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapsedCls(value:String):void;

  [ExtConfig]
  /**
   * True to show a mini-collapse tool in the Splitter to toggle expand and collapse on the <a href="#!/api/Ext.resizer.Splitter-cfg-collapseTarget" rel="Ext.resizer.Splitter-cfg-collapseTarget" class="docClass">collapseTarget</a> Panel.
   * Defaults to the <a href="#!/api/Ext.panel.Panel-cfg-collapsible" rel="Ext.panel.Panel-cfg-collapsible" class="docClass">collapsible</a> setting of the Panel.
   */
  public native function get collapsible():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapsible(value:Boolean):void;

  [ExtConfig]
  /**
   * Provides a default maximum width or height for the two components
   * that the splitter is between.
   * @default 1000
   */
  public native function get defaultSplitMax():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultSplitMax(value:Number):void;

  [ExtConfig]
  /**
   * Provides a default minimum width or height for the two components
   * that the splitter is between.
   * @default 40
   */
  public native function get defaultSplitMin():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultSplitMin(value:Number):void;

  [ExtConfig]
  /**
   * Set to false to prevent this Splitter's mini-collapse tool from managing the collapse
   * state of the <a href="#!/api/Ext.resizer.Splitter-cfg-collapseTarget" rel="Ext.resizer.Splitter-cfg-collapseTarget" class="docClass">collapseTarget</a>.
   */
  public native function get performCollapse():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set performCollapse(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The size of the splitter. This becomes the height for vertical splitters and
   * width for horizontal splitters.
   * @default 5
   * @see #getSize()
   */
  public native function get size():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set size(value:Number):void;

  [ExtConfig]
  /**
   * Any configuration options to be passed to the underlying <a href="#!/api/Ext.resizer.SplitterTracker" rel="Ext.resizer.SplitterTracker" class="docClass">Ext.resizer.SplitterTracker</a>.
   */
  public native function get tracker():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tracker(value:Object):void;

  /**
   * Returns the config object (with an <code>xclass</code> property) for the splitter tracker. This
   * is overridden by <a href="#!/api/Ext.resizer.BorderSplitter" rel="Ext.resizer.BorderSplitter" class="docClass">BorderSplitter</a> to create a
   * <a href="#!/api/Ext.resizer.BorderSplitterTracker" rel="Ext.resizer.BorderSplitterTracker" class="docClass">BorderSplitterTracker</a>.
   */
  protected native function getTrackerConfig():ext.resizer.SplitterTracker;
}
}