package ext.scroll {
import ext.Evented;
import ext.mixin.IFactoryable;

[Uses("ext.Evented")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Observable")]
[Native("Ext.scroll.Scroller", require)]
/**
 * Fires whenever the Scroller is refreshed.
 * @eventType ext.scroll.events.ScrollerEvent.REFRESH
 */
[Event(name = 'onRefresh', type = 'ext.scroll.events.ScrollerEvent')]
/**
 * Fires whenever the Scroller is scrolled.
 * @eventType ext.scroll.events.Scroller_x_yEvent.SCROLL
 */
[Event(name = 'onScroll', type = 'ext.scroll.events.Scroller_x_yEvent')]
/**
 * Fires whenever the scrolling is ended.
 * @eventType ext.scroll.events.Scroller_x_yEvent.SCROLL_END
 */
[Event(name = 'onScrollEnd', type = 'ext.scroll.events.Scroller_x_yEvent')]
/**
 * Fires whenever the scrolling is started.
 * @eventType ext.scroll.events.Scroller_x_yEvent.SCROLL_START
 */
[Event(name = 'onScrollStart', type = 'ext.scroll.events.Scroller_x_yEvent')]
/**
 * <a href="#!/api/Ext.scroll.Scroller" rel="Ext.scroll.Scroller" class="docClass">Ext.scroll.Scroller</a> allows any element to have scrollable content, both on desktop and
 * touch-screen devices, and defines a set of useful methods for manipulating the scroll
 * position and controlling the scrolling behavior.
 */
public class Scroller extends ext.Evented implements ext.mixin.IFactoryable {
  /**
   * Creates and returns an appropriate Scroller instance for the current device.
   * @param config Configuration options for the Scroller
   */
  public static native function create(config:Object):ext.scroll.Scroller;

  /**
   * @param config @inheritDoc
   */
  public function Scroller(config:ext.scroll.Scroller = null) {
    super();
  }

  [Deprecated(replacement = "y", since = "5.1.0")]
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @default 'auto'
   * @see #getDirection()
   * @see #setDirection()
   */
  public native function get direction():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set direction(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The element to make scrollable.
   * @see #getElement()
   * @see #setElement()
   */
  public native function get element():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set element(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to enable scrolling of "right-to-left" content.  This is typically
   * configured automatically by an <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> based on its inherited
   * <a href="#!/api/Ext.Component-cfg-rtl" rel="Ext.Component-cfg-rtl" class="docClass">rtl</a> state
   * @default false
   */
  public native function get rtl():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rtl(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <ul>
   * <li><code>true</code> or <code>'auto'</code> to enable horizontal auto-scrolling. In auto-scrolling mode
   * scrolling is only enabled when the <a href="#!/api/Ext.scroll.Scroller-cfg-element" rel="Ext.scroll.Scroller-cfg-element" class="docClass">element</a> has overflowing content.</li>
   * <li><code>false</code> to disable horizontal scrolling</li>
   * <li><code>'scroll'</code> to always enable horizontal scrolling regardless of content size.</li>
   * </ul>
   * @default true
   * @see #getX()
   * @see #setX()
   */
  public native function get x():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set x(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <ul>
   * <li><code>true</code> or <code>'auto'</code> to enable vertical auto-scrolling. In auto-scrolling mode
   * scrolling is only enabled when the <a href="#!/api/Ext.scroll.Scroller-cfg-element" rel="Ext.scroll.Scroller-cfg-element" class="docClass">element</a> has overflowing content.</li>
   * <li><code>false</code> to disable vertical scrolling</li>
   * <li><code>'scroll'</code> to always enable vertical scrolling regardless of content size.</li>
   * </ul>
   * @default true
   * @see #getY()
   * @see #setY()
   */
  public native function get y():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set y(value:*):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Adds a "partner" scroller.  Partner scrollers reflect each other's scroll position
   * at all times - if either scroller is scrolled, the scroll position of its partner
   * will be be automatically synchronized.
   *
   * <p>A scroller may have multiple partners.</p>
   * @param axis The axis to synchronize (<code>'x'</code>, '<code>y</code>', or '<code>both</code>')
   * @default 'both'
   */
  public native function addPartner(partner:ext.scroll.Scroller, axis:String = 'both'):void;

  /**
   * Gets the <code>clientWidth</code> and <code>clientHeight</code> of the <a href="#!/api/Ext.scroll.Scroller-cfg-element" rel="Ext.scroll.Scroller-cfg-element" class="docClass">element</a> for this scroller.
   * @return An object with <code>x</code> and <code>y</code> properties.
   */
  public native function getClientSize():Object;

  [Deprecated(replacement = "y", since = "5.1.0")]
  /**
   * Returns the value of <a href="#!/api/Ext.scroll.Scroller-cfg-direction" rel="Ext.scroll.Scroller-cfg-direction" class="docClass">direction</a>.
   */
  public native function getDirection():String;

  /**
   * Returns the value of <a href="#!/api/Ext.scroll.Scroller-cfg-element" rel="Ext.scroll.Scroller-cfg-element" class="docClass">element</a>.
   */
  public native function getElement():*;

  /**
   * Returns the maximum scroll position for this scroller
   * @return position
   */
  public native function getMaxPosition():Object;

  /**
   * Returns the maximum scroll position for this scroller for scrolling that is initiated
   * by the user via mouse or touch.  This differs from getMaxPosition in that getMaxPosition
   * returns the true maximum scroll position regardless of which axes are enabled for
   * user scrolling.
   * @return position
   */
  public native function getMaxUserPosition():Object;

  /**
   * Returns the current scroll position
   * @return An object with <code>x</code> and <code>y</code> properties.
   */
  public native function getPosition():Object;

  /**
   * Returns the amount of space consumed by scrollbars in the DOM
   * @return size An object containing the scrollbar sizes.
   */
  public native function getScrollbarSize():Object;

  [Deprecated(since = "5.0")]
  /**
   * Returns this scroller.
   *
   * <p>In Sencha Touch 2, access to a Component's Scroller was provided via
   * a Ext.scroll.View class that was returned from the Component's getScrollable()
   * method:</p>
   *
   * <pre><code>component.getScrollable().getScroller();
   * </code></pre>
   *
   * <p>in 5.0 all the functionality of Ext.scroll.View has been rolled into
   * <a href="#!/api/Ext.scroll.Scroller" rel="Ext.scroll.Scroller" class="docClass">Ext.scroll.Scroller</a>, and Ext.scroll.View has been removed.  Component's
   * getScrollable() method now returns a <a href="#!/api/Ext.scroll.Scroller" rel="Ext.scroll.Scroller" class="docClass">Ext.scroll.Scroller</a>.  This method is
   * provided for compatibility.</p>
   * @return this
   */
  public native function getScroller():ext.scroll.Scroller;

  /**
   * Returns the size of the scrollable content
   * @return size
   */
  public native function getSize():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.scroll.Scroller-cfg-x" rel="Ext.scroll.Scroller-cfg-x" class="docClass">x</a>.
   */
  public native function getX():*;

  /**
   * Returns the value of <a href="#!/api/Ext.scroll.Scroller-cfg-y" rel="Ext.scroll.Scroller-cfg-y" class="docClass">y</a>.
   */
  public native function getY():*;

  /**
   * Determines if the passed element is within the visible x and y scroll viewport.
   * @param el The dom node, <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>, or
   * id (string) of the dom element that is to be verified to be in view
   * @return Which ranges the element is in.
   * <ul>
   *   <li><code>x</code> : Boolean
   * <code>true</code> if the passed element is within the x visible range.
   *   </li>
   *   <li><code>y</code> : Boolean
   * <code>true</code> if the passed element is within the y visible range.
   *   </li>
   * </ul>
   */
  public native function isInView(el:*):Object;

  /**
   * Refreshes the scroller size and maxPosition.
   * @param immediate <code>true</code> to refresh immediately. By default refreshes
   * are deferred until the next <a href="#!/api/Ext.GlobalEvents-event-idle" rel="Ext.GlobalEvents-event-idle" class="docClass">idle</a> event to
   * ensure any pending writes have been flushed to the dom and any reflows have
   * taken place.
   * @return this
   */
  public native function refresh(immediate:Boolean):ext.scroll.Scroller;

  /**
   * Scrolls by the passed delta values, optionally animating.
   *
   * <p>All of the following are equivalent:</p>
   *
   * <pre><code> scroller.scrollBy(10, 10, true);
   *  scroller.scrollBy([10, 10], true);
   *  scroller.scrollBy({ x: 10, y: 10 }, true);
   * </code></pre>
   *
   * <p>A null value for either <code>x</code> or <code>y</code> will result in no scrolling on the given axis,
   * for example:</p>
   *
   * <pre><code>scroller.scrollBy(null, 10);
   * </code></pre>
   *
   * <p>will scroll by 10 on the y axis and leave the x axis at its current scroll position</p>
   * @param deltaX Either the x delta, an Array specifying x
   * and y deltas or an object with "x" and "y" properties.
   * @param deltaY Either the y delta, or an animate flag or
   * config object.
   * @param animate Animate flag/config object if the delta values were
   * passed separately.
   */
  public native function scrollBy(deltaX:*, deltaY:*, animate:*):void;

  /**
   * Scrolls to the given position.
   *
   * <p>All of the following are equivalent:</p>
   *
   * <pre><code> scroller.scrollTo(10, 10, true);
   *  scroller.scrollTo([10, 10], true);
   *  scroller.scrollTo({ x: 10, y: 10 }, true);
   * </code></pre>
   *
   * <p>A null value for either <code>x</code> or <code>y</code> will result in no scrolling on the given axis,
   * for example:</p>
   *
   * <pre><code>scroller.scrollTo(null, 10);
   * </code></pre>
   *
   * <p>will scroll to 10 on the y axis and leave the x axis at its current scroll position</p>
   *
   * <p>A negative value for either <code>x</code> or <code>y</code> represents an offset from the maximum scroll
   * position on the given axis:</p>
   *
   * <pre><code>// scrolls to 10px from the maximum x scroll position and 20px from maximum y
   * scroller.scrollTo(-10, -20);
   * </code></pre>
   *
   * <p>A value of Infinity on either axis will scroll to the maximum scroll position on
   * that axis:</p>
   *
   * <pre><code>// scrolls to the maximum position on both axes
   * scroller.scrollTo(Infinity, Infinity);
   * </code></pre>
   * @param x The scroll position on the x axis.
   * @param y The scroll position on the y axis.
   * @param animation Whether or not to animate the scrolling to the new position.
   * @return this
   */
  public native function scrollTo(x:Number, y:Number, animation:* = undefined):ext.scroll.Scroller;

  [Deprecated(replacement = "y", since = "5.1.0")]
  /**
   * Sets the value of <a href="#!/api/Ext.scroll.Scroller-cfg-direction" rel="Ext.scroll.Scroller-cfg-direction" class="docClass">direction</a>.
   * @param direction The new value.
   */
  public native function setDirection(direction:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.scroll.Scroller-cfg-element" rel="Ext.scroll.Scroller-cfg-element" class="docClass">element</a>.
   * @param element The new value.
   */
  public native function setElement(element:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.scroll.Scroller-cfg-x" rel="Ext.scroll.Scroller-cfg-x" class="docClass">x</a>.
   * @param x The new value.
   */
  public native function setX(x:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.scroll.Scroller-cfg-y" rel="Ext.scroll.Scroller-cfg-y" class="docClass">y</a>.
   * @param y The new value.
   */
  public native function setY(y:*):void;
}
}