package ext.selection {

[Uses("ext.selection.DataViewSelectionModel")]
[Uses("ext.selection.SelectionModel")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.util.StoreHolder")]
[Native("Ext.selection.RowModel", require)]
/**
 * Fired before a record is deselected. If any listener returns false, the
 * deselection is cancelled.
 * @eventType ext.selection.events.RowSelectionModel_record_indexEvent.BEFORE_DESELECT
 */
[Event(name = 'onBeforeDeselect', type = 'ext.selection.events.RowSelectionModel_record_indexEvent')]
/**
 * Fired before a record is selected. If any listener returns false, the
 * selection is cancelled.
 * @eventType ext.selection.events.RowSelectionModel_record_indexEvent.BEFORE_SELECT
 */
[Event(name = 'onBeforeSelect', type = 'ext.selection.events.RowSelectionModel_record_indexEvent')]
/**
 * Fired after a record is deselected
 * @eventType ext.selection.events.RowSelectionModel_record_indexEvent.DESELECT
 */
[Event(name = 'onDeselect', type = 'ext.selection.events.RowSelectionModel_record_indexEvent')]
/**
 * Fired after a record is selected
 * @eventType ext.selection.events.RowSelectionModel_record_indexEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.selection.events.RowSelectionModel_record_indexEvent')]
/**
 * RowModel Selection Model implements row based navigation for
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">grid panels</a> via user input. RowModel is the default grid selection
 * model and, generally, will not need to be specified.
 *
 * <p>By utilizing the selModel config as an object, you may also set configurations for:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.selection.RowModel-cfg-mode" rel="Ext.selection.RowModel-cfg-mode" class="docClass">mode</a> - Specifies whether user may select multiple rows or single rows</li>
 * <li><a href="#!/api/Ext.selection.RowModel-cfg-allowDeselect" rel="Ext.selection.RowModel-cfg-allowDeselect" class="docClass">allowDeselect</a> - Specifies whether user may deselect records when in SINGLE mode</li>
 * <li><a href="#!/api/Ext.selection.RowModel-cfg-ignoreRightMouseSelection" rel="Ext.selection.RowModel-cfg-ignoreRightMouseSelection" class="docClass">ignoreRightMouseSelection</a> - Specifies whether user may ignore right clicks
 * for selection purposes</li>
 * </ul>
 *
 *
 * <p>In the example below, we've enabled MULTI mode. This means that multiple rows can be selected.</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name', 'email', 'phone'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com', phone: '555-111-1224' },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com', phone: '555-222-1234' },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244' },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: store,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     selModel: {
 *        selType: 'rowmodel', // rowmodel is the default selection model
 *        mode: 'MULTI' // Allows selection of multiple rows
 *     },
 *     columns: [
 *         { text: 'Name',  dataIndex: 'name'  },
 *         { text: 'Email', dataIndex: 'email', flex: 1 },
 *         { text: 'Phone', dataIndex: 'phone' }
 *     ]
 * });
 * </code></pre>
 */
public class RowSelectionModel extends ext.selection.DataViewSelectionModel {
  /**
   * @param config @inheritDoc
   */
  public function RowSelectionModel(config:ext.selection.RowSelectionModel = null) {
    super();
  }

  [ExtConfig]
  /**
   * Turns on/off keyboard navigation within the grid.
   * @default true
   */
  override public native function get enableKeyNav():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set enableKeyNav(value:Boolean):void;

  [Deprecated(replacement = "getNavigationModel", since = "5.0.1")]
  /**
   * Returns position of the first selected cell in the selection in the format {row: row, column: column}
   */
  public native function getCurrentPosition():Object;

  /**
   * Selects the record immediately following the currently selected record.
   * @param keepExisting True to retain existing selections
   * @param suppressEvent Set to false to not fire a select event
   * @return <code>true</code> if there is a next record, else <code>false</code>
   */
  public native function selectNext(keepExisting:Boolean = false, suppressEvent:Boolean = false):Boolean;

  /**
   * Selects the record that precedes the currently selected record.
   * @param keepExisting True to retain existing selections
   * @param suppressEvent Set to false to not fire a select event
   * @return <code>true</code> if there is a previous record, else <code>false</code>
   */
  public native function selectPrevious(keepExisting:Boolean = false, suppressEvent:Boolean = false):Boolean;
}
}