package ext.selection {
import ext.data.AbstractStore;
import ext.data.Store;
import ext.mixin.IFactoryable;
import ext.mixin.Observable;
import ext.util.IStoreHolder;
import ext.view.DataView;

[Uses("ext.mixin.Observable")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.selection.Model", require)]
/**
 * Fired when a row is focused
 * @eventType ext.selection.events.SelectionModel_oldFocused_newFocusedEvent.FOCUS_CHANGE
 */
[Event(name = 'onFocusChange', type = 'ext.selection.events.SelectionModel_oldFocused_newFocusedEvent')]
/**
 * Fired after a selection change has occurred
 * @eventType ext.selection.events.SelectionModel_selectedEvent.SELECTION_CHANGE
 */
[Event(name = 'onSelectionChange', type = 'ext.selection.events.SelectionModel_selectedEvent')]
/**
 * Tracks what records are currently selected in a data-bound component.
 *
 * <p>This is an abstract class and is not meant to be directly used. Data-bound UI widgets such as
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Grid</a> and <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Tree</a> should subclass <a href="#!/api/Ext.selection.Model" rel="Ext.selection.Model" class="docClass">Ext.selection.Model</a>
 * and provide a way to binding to the component.</p>
 *
 * <p>The abstract method <code>onSelectChange</code> should be implemented in these
 * subclasses to update the UI widget.</p>
 */
public class SelectionModel extends ext.mixin.Observable implements ext.util.IStoreHolder, ext.mixin.IFactoryable {
  /**
   * @param config @inheritDoc
   */
  public function SelectionModel(config:ext.selection.SelectionModel = null) {
    super();
  }

  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @private
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated <a href="#!/api/Ext.selection.Model" rel="Ext.selection.Model" class="docClass">selection model</a>, or subclass thereof.
   * @default true
   */
  public native function get isSelectionModel():Boolean;

  /**
   * @private
   */
  public native function set isSelectionModel(value:Boolean):void;

  [ExtConfig]
  /**
   * Allow users to deselect a record in a DataView, List or Grid.
   * Only applicable when the <a href="#!/api/Ext.selection.Model-cfg-mode" rel="Ext.selection.Model-cfg-mode" class="docClass">mode</a> is 'SINGLE'.
   */
  public native function get allowDeselect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowDeselect(value:Boolean):void;

  [ExtConfig]
  /**
   * True to ignore selections that are made when using the right mouse button if there are
   * records that are already selected. If no records are selected, selection will continue
   * as normal
   * @default false
   */
  public native function get ignoreRightMouseSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreRightMouseSelection(value:Boolean):void;

  [ExtConfig]
  /**
   * Mode of selection.  Valid values are:
   *
   * <ul>
   * <li><strong>"SINGLE"</strong> - Only allows selecting one item at a time.  Use <a href="#!/api/Ext.selection.Model-cfg-allowDeselect" rel="Ext.selection.Model-cfg-allowDeselect" class="docClass">allowDeselect</a> to allow
   * deselecting that item.  Also see <a href="#!/api/Ext.selection.Model-cfg-toggleOnClick" rel="Ext.selection.Model-cfg-toggleOnClick" class="docClass">toggleOnClick</a>. This is the default.</li>
   * <li><strong>"SIMPLE"</strong> - Allows simple selection of multiple items one-by-one. Each click in grid will either
   * select or deselect an item.</li>
   * <li><strong>"MULTI"</strong> - Allows complex selection of multiple items using Ctrl and Shift keys.</li>
   * </ul>
   */
  public native function get mode():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set mode(value:String):void;

  [ExtConfig]
  /**
   * Remove records from the selection when they are removed from the store.
   *
   * <p><strong>Important:</strong> When using <a href="#!/api/Ext.toolbar.Paging" rel="Ext.toolbar.Paging" class="docClass">paging</a> or a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Ext.data.BufferedStore</a>,
   * records which are cached in the Store's <a href="#!/api/Ext.data.Store-property-data" rel="Ext.data.Store-property-data" class="docClass">data collection</a> may be removed from the Store when pages change,
   * or when rows are scrolled out of view. For this reason <code>pruneRemoved</code> should be set to <code>false</code> when using a buffered Store.</p>
   *
   * <p>Also, when previously pruned pages are returned to the cache, the records objects in the page will be
   * <em>new instances</em>, and will not match the instances in the selection model's collection. For this reason,
   * you MUST ensure that the Model definition's <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">idProperty</a> references a unique
   * key because in this situation, records in the Store have their <strong>IDs</strong> compared to records in the SelectionModel
   * in order to re-select a record which is scrolled back into view.</p>
   * @default true
   */
  public native function get pruneRemoved():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pruneRemoved(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to toggle the selection state of an item when clicked.
   * Only applicable when the <a href="#!/api/Ext.selection.Model-cfg-mode" rel="Ext.selection.Model-cfg-mode" class="docClass">mode</a> is 'SINGLE'.
   * Only applicable when the <a href="#!/api/Ext.selection.Model-cfg-allowDeselect" rel="Ext.selection.Model-cfg-allowDeselect" class="docClass">allowDeselect</a> is 'true'.
   * @default true
   */
  public native function get toggleOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set toggleOnClick(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  public native function bindComponent():void;

  /**
   * Binds a store to this instance.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Deselects a record instance by record instance or index.
   * @param records An array of records or an index
   * @param suppressEvent True to not fire a deselect event
   * @default false
   */
  public native function deselect(records:*, suppressEvent:Boolean = false):void;

  /**
   * Deselects all records in the view.
   * @param suppressEvent True to suppress any deselect events
   */
  public native function deselectAll(suppressEvent:Boolean = false):void;

  /**
   * Deselects a range of rows if the selection model <a href="#!/api/Ext.selection.Model-method-isLocked" rel="Ext.selection.Model-method-isLocked" class="docClass">is not locked</a>.
   * @param startRow The record or index of the first row in the range
   * @param endRow The record or index of the last row in the range
   */
  public native function deselectRange(startRow:Number, endRow:Number):void;

  /**
   * Returns the count of selected records.
   * @return The number of selected records
   */
  public native function getCount():Number;

  [ArrayElementType("ext.data.Model")]
  /**
   * Returns an array of the currently selected records.
   * @return The selected records
   */
  public native function getSelection():Array;

  /**
   * Returns the current selectionMode.
   * @return The selectionMode: 'SINGLE', 'MULTI' or 'SIMPLE'.
   */
  public native function getSelectionMode():String;

  /**
   * Returns true if there are any a selected records.
   */
  public native function hasSelection():Boolean;

  /**
   * Returns true if the selections are locked.
   */
  public native function isLocked():Boolean;

  /**
   * Returns true if the specified row is selected.
   * @param from The start of the range to check.
   * @param to The end of the range to check.
   */
  public native function isRangeSelected(from:*, to:*):Boolean;

  /**
   * Returns true if the specified row is selected.
   * @param record The record or index of the record to check
   */
  public native function isSelected(record:*):Boolean;

  public native function onEditorKey():void;

  public native function onSelectChange(record:Object, isSelected:Object, suppressEvent:Object, commitFn:Object):void;

  /**
   * Selects a record instance by record instance or index.
   * @param records An array of records or an index
   * @param keepExisting True to retain existing selections
   * @default false
   * @param suppressEvent True to not fire a select event
   * @default false
   */
  public native function select(records:*, keepExisting:Boolean = false, suppressEvent:Boolean = false):void;

  /**
   * Selects all records in the view.
   * @param suppressEvent True to suppress any select events
   */
  public native function selectAll(suppressEvent:Boolean):void;

  /**
   * Selects a range of rows if the selection model <a href="#!/api/Ext.selection.Model-method-isLocked" rel="Ext.selection.Model-method-isLocked" class="docClass">is not locked</a>.
   * All rows in between startRow and endRow are also selected.
   * @param startRow The record or index of the first row in the range
   * @param endRow The record or index of the last row in the range
   * @param keepExisting True to retain existing selections
   */
  public native function selectRange(startRow:Number, endRow:Number, keepExisting:Boolean = false):void;

  /**
   * Locks the current selection and disables any changes from happening to the selection.
   * @param locked True to lock, false to unlock.
   */
  public native function setLocked(locked:Boolean):void;

  /**
   * Sets the current selectionMode.
   * @param selMode 'SINGLE', 'MULTI' or 'SIMPLE'.
   */
  public native function setSelectionMode(selMode:String):void;

  /**
   * @inheritDoc
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * @inheritDoc
   */
  public native function setStore(store:Object):void;

  /**
   * Allows multiple views to be controlled by one selection model.
   * Called by AbstractView's beforeRender method.
   * @param view The View passes itself
   */
  protected native function beforeViewRender(view:ext.view.DataView):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Called by the owning grid's <a href="#!/api/Ext.grid.header.Container" rel="Ext.grid.header.Container" class="docClass">header container</a>
   * when a column header is activated by the UI (clicked, or receives a <code>SPACE</code> or <code>ENTER</code> key event).
   */
  protected native function onHeaderClick():void;

  /**
   * Template method, it is called when an existing store is unbound
   * from the current instance.
   * @param store The store being unbound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onUnbindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}