package ext.slider {
import ext.form.field.BaseField;
import ext.form.field.IField;

[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.slider.Multi", require)]
/**
 * Fires before the slider value is changed. By returning false from an event handler, you can cancel the
 * event and prevent the slider from changing.
 * @eventType ext.slider.events.MultiSlider_newValue_oldValueEvent.BEFORE_CHANGE
 */
[Event(name = 'onBeforeChange', type = 'ext.slider.events.MultiSlider_newValue_oldValueEvent')]
/**
 * Fires when the slider value is changed.
 * @eventType ext.slider.events.MultiSlider_newValue_thumbEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.slider.events.MultiSlider_newValue_thumbEvent')]
/**
 * Fires when the slider value is changed by the user and any drag operations have completed.
 * @eventType ext.slider.events.MultiSlider_newValue_thumbEvent.CHANGE_COMPLETE
 */
[Event(name = 'onChangeComplete', type = 'ext.slider.events.MultiSlider_newValue_thumbEvent')]
/**
 * Fires continuously during the drag operation while the mouse is moving.
 * @eventType ext.slider.events.MultiSlider_eEvent.DRAG
 */
[Event(name = 'onDrag', type = 'ext.slider.events.MultiSlider_eEvent')]
/**
 * Fires after the drag operation has completed.
 * @eventType ext.slider.events.MultiSlider_eEvent.DRAG_END
 */
[Event(name = 'onDragEnd', type = 'ext.slider.events.MultiSlider_eEvent')]
/**
 * Fires after a drag operation has started.
 * @eventType ext.slider.events.MultiSlider_eEvent.DRAG_START
 */
[Event(name = 'onDragStart', type = 'ext.slider.events.MultiSlider_eEvent')]
/**
 * Slider which supports vertical or horizontal orientation, keyboard adjustments, configurable snapping, axis clicking
 * and animation. Can be added as an item to any container.
 *
 * <p>Sliders can be created with more than one thumb handle by passing an array of values instead of a single one:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.slider.Multi" rel="Ext.slider.Multi" class="docClass">Ext.slider.Multi</a>', {
 *     width: 200,
 *     values: [25, 50, 75],
 *     increment: 5,
 *     minValue: 0,
 *     maxValue: 100,
 *
 *     // this defaults to true, setting to false allows the thumbs to pass each other
 *     constrainThumbs: false,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class MultiSlider extends ext.form.field.BaseField {
  /**
   * @param config @inheritDoc
   */
  public function MultiSlider(config:ext.slider.MultiSlider = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A value with which to initialize the slider. Setting this will only result in the creation
   * of a single slider thumb; if you want multiple thumbs then use the <a href="#!/api/Ext.slider.Multi-cfg-values" rel="Ext.slider.Multi-cfg-values" class="docClass">values</a> config instead.
   *
   * <p>Defaults to <a href="#!/api/Ext.slider.Multi-cfg-minValue" rel="Ext.slider.Multi-cfg-minValue" class="docClass">minValue</a>.</p>
   * @see #getValue()
   * @see #setValue()
   */
  override public native function get value():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set value(value:Object):void;

  /**
   * True while the thumb is in a drag operation
   * @default false
   */
  public native function get dragging():Boolean;

  /**
   * @private
   */
  public native function set dragging(value:Boolean):void;

  /**
   * Array containing references to each thumb
   * @default []
   */
  public native function get thumbs():Array;

  /**
   * @private
   */
  public native function set thumbs(value:Array):void;

  [ExtConfig]
  /**
   * Turn on or off animation. May be an animation configuration object:
   *
   * <pre><code>animate: {
   *     duration: 3000,
   *     easing: 'easeIn'
   * }
   * </code></pre>
   * @default true
   */
  public native function get animated():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animated(value:*):void;

  [ExtConfig]
  /**
   * Determines whether or not clicking on the Slider axis will change the slider.
   * @default true
   */
  public native function get clickToChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clickToChange(value:Boolean):void;

  [ExtConfig]
  /**
   * True to disallow thumbs from overlapping one another.
   * @default true
   */
  public native function get constrainThumbs():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainThumbs(value:Boolean):void;

  [ExtConfig]
  /**
   * The number of decimal places to which to round the Slider's value.
   *
   * <p>To disable rounding, configure as <strong>false</strong>.</p>
   * @default 0
   */
  public native function get decimalPrecision():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set decimalPrecision(value:*):void;

  [ExtConfig]
  /**
   * How many units to change the slider when adjusting by drag and drop. Use this option to enable 'snapping'.
   * @default 0
   */
  public native function get increment():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set increment(value:Number):void;

  [ExtConfig]
  /**
   * How many units to change the Slider when adjusting with keyboard navigation. If the increment
   * config is larger, it will be used instead.
   * @default 1
   */
  public native function get keyIncrement():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyIncrement(value:Number):void;

  [ExtConfig]
  /**
   * The maximum value for the Slider.
   * @default 100
   * @see #setMaxValue()
   */
  public native function get maxValue():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxValue(value:Number):void;

  [ExtConfig]
  /**
   * The minimum value for the Slider.
   * @default 0
   * @see #setMinValue()
   */
  public native function get minValue():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minValue(value:Number):void;

  [ExtConfig]
  /**
   * How many units to change the Slider when using PageUp and PageDown keys.
   * @default 10
   */
  public native function get pageSize():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pageSize(value:Number):void;

  [ExtConfig]
  /**
   * A function used to display custom text for the slider tip or the name of the
   * method on the corresponding <code><a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">controller</a></code>.
   *
   * <p>Defaults to null, which will use the default on the plugin.</p>
   * <ul>
   *   <li><code>thumb</code> : ext.Base
   * The Thumb that the Tip is attached to
   *   </li>
   *   <li><code>return</code> : String
   * The text to display in the tip
   *   </li>
   * </ul>
   */
  public native function get tipText():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tipText(value:Function):void;

  [ExtConfig]
  /**
   * True to use an <a href="#!/api/Ext.slider.Tip" rel="Ext.slider.Tip" class="docClass">Ext.slider.Tip</a> to display tips for the value. This option may also
   * provide a configuration object for an <a href="#!/api/Ext.slider.Tip" rel="Ext.slider.Tip" class="docClass">Ext.slider.Tip</a>.
   * @default true
   */
  public native function get useTips():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useTips(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Array of Number values with which to initalize the slider. A separate slider thumb will be created for each value
   * in this array. This will take precedence over the single <a href="#!/api/Ext.slider.Multi-cfg-value" rel="Ext.slider.Multi-cfg-value" class="docClass">value</a> config.
   * @see #getValues()
   */
  public native function get values():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set values(value:Array):void;

  [ExtConfig]
  /**
   * Orient the Slider vertically rather than horizontally.
   * @default false
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to <code>true</code> to calculate snap points based on <a href="#!/api/Ext.slider.Multi-cfg-increment" rel="Ext.slider.Multi-cfg-increment" class="docClass">increment</a>s from zero as opposed to
   * from this Slider's <a href="#!/api/Ext.slider.Multi-cfg-minValue" rel="Ext.slider.Multi-cfg-minValue" class="docClass">minValue</a>.
   *
   * <p>By Default, valid snap points are calculated starting <a href="#!/api/Ext.slider.Multi-cfg-increment" rel="Ext.slider.Multi-cfg-increment" class="docClass">increment</a>s from the <a href="#!/api/Ext.slider.Multi-cfg-minValue" rel="Ext.slider.Multi-cfg-minValue" class="docClass">minValue</a></p>
   * @default false
   */
  public native function get zeroBasedSnapping():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set zeroBasedSnapping(value:Boolean):void;

  /**
   * Returns the current value of the slider
   * @return The current value of the slider at the given index, or an array of all thumb values if
   * no index is given.
   */
  override public native function getValue():*;

  /**
   * Programmatically sets the value of the Slider. Ensures that the value is constrained within the minValue and
   * maxValue.
   *
   * <p>Setting the second slider's value without animation:</p>
   *
   * <pre><code>mySlider.setValue(1, 50, false);
   * </code></pre>
   *
   * <p>Setting multiple values with animation:</p>
   *
   * <pre><code>mySlider.setValue([20, 40, 60], true);
   * </code></pre>
   * @param value The value to set
   * @return this
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * Creates a new thumb and adds it to the slider
   * @param value The initial value to set on the thumb.
   * @default 0
   * @return The thumb
   */
  public native function addThumb(value:Number = 0):void;

  /**
   * Returns an array of values - one for the location of each thumb
   * @return The set of thumb values
   */
  public native function getValues():Array;

  /**
   * Sets the maximum value for the slider instance. If the current value is more than the maximum value, the current
   * value will be changed.
   * @param val The new maximum value
   */
  public native function setMaxValue(val:Number):void;

  /**
   * Sets the minimum value for the slider instance. If the current value is less than the minimum value, the current
   * value will be changed.
   * @param val The new minimum value
   */
  public native function setMinValue(val:Number):void;

  /**
   * Synchronizes thumbs position to the proper proportion of the total component width based on the current slider
   * <a href="#!/api/Ext.slider.Multi-cfg-value" rel="Ext.slider.Multi-cfg-value" class="docClass">value</a>. This will be called automatically when the Slider is resized by a layout, but if it is rendered
   * auto width, this method can be called from another resize handler to sync the Slider if necessary.
   */
  public native function syncThumbs():void;
}
}