package ext.slider {
import ext.Base;
import ext.Component;
import ext.Plugin;
import ext.tip.Tip;

[Uses("ext.tip.Tip")]
[Uses("ext.Plugin")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.slider.Tip", require)]
/**
 * Simple plugin for using an <a href="#!/api/Ext.tip.Tip" rel="Ext.tip.Tip" class="docClass">Ext.tip.Tip</a> with a slider to show the slider value. In general this class is not created
 * directly, instead pass the <a href="#!/api/Ext.slider.Multi-cfg-useTips" rel="Ext.slider.Multi-cfg-useTips" class="docClass">Ext.slider.Multi.useTips</a> and <a href="#!/api/Ext.slider.Multi-cfg-tipText" rel="Ext.slider.Multi-cfg-tipText" class="docClass">Ext.slider.Multi.tipText</a> configuration
 * options to the slider directly.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.slider.Single" rel="Ext.slider.Single" class="docClass">Ext.slider.Single</a>', {
 *     width: 214,
 *     minValue: 0,
 *     maxValue: 100,
 *     useTips: true,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>Optionally provide your own tip text by passing tipText:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.slider.Single" rel="Ext.slider.Single" class="docClass">Ext.slider.Single</a>', {
 *     width: 214,
 *     minValue: 0,
 *     maxValue: 100,
 *     useTips: true,
 *     tipText: function(thumb){
 *         return <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a>('{0}% complete', thumb.value);
 *     },
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class SliderTip extends ext.tip.Tip implements ext.Plugin {
  /**
   * @param config @inheritDoc
   */
  public function SliderTip(config:ext.slider.SliderTip = null) {
    super();
  }

  [ExtConfig]
  /**
   * Alignment configuration for the tip to the slider. See <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">Ext.util.Positionable.alignTo</a>. Default
   * values for alignment are provided by specifying the <a href="#!/api/Ext.slider.Tip-cfg-position" rel="Ext.slider.Tip-cfg-position" class="docClass">position</a> config.
   * @default null
   */
  public native function get align():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set align(value:String):void;

  [ExtConfig]
  /**
   * Offsets for aligning the tip to the slider. See <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">Ext.util.Positionable.alignTo</a>. Default values
   * for offsets are provided by specifying the <a href="#!/api/Ext.slider.Tip-cfg-position" rel="Ext.slider.Tip-cfg-position" class="docClass">position</a> config.
   * @default null
   */
  public native function get offsets():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set offsets(value:Array):void;

  [ExtConfig]
  /**
   * Sets the position for where the tip will be displayed related to the thumb. This sets
   * defaults for <a href="#!/api/Ext.slider.Tip-cfg-align" rel="Ext.slider.Tip-cfg-align" class="docClass">align</a> and <a href="#!/api/Ext.slider.Tip-cfg-offsets" rel="Ext.slider.Tip-cfg-offsets" class="docClass">offsets</a> configurations. If <a href="#!/api/Ext.slider.Tip-cfg-align" rel="Ext.slider.Tip-cfg-align" class="docClass">align</a> or
   * <a href="#!/api/Ext.slider.Tip-cfg-offsets" rel="Ext.slider.Tip-cfg-offsets" class="docClass">offsets</a> configurations are specified, they will override the defaults defined
   * by position.
   * @default For horizontal sliders, "top", for vertical sliders, "left"
   */
  public native function get position():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set position(value:String):void;

  public native function init(client:ext.Component):void;

  /**
   * Used to create the text that appears in the Tip's body. By default this just returns the value of the Slider
   * Thumb that the Tip is attached to. Override to customize.
   * @param thumb The Thumb that the Tip is attached to
   * @return The text to display in the tip
   */
  protected native function getText(thumb:ext.Base):String;
}
}