package ext.slider {
import ext.Widget;

[Uses("ext.Widget")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.slider.Widget", require)]
/**
 * A Widget-based implementation of a slider.
 */
public class SliderWidget extends ext.Widget {
  /**
   * @param config @inheritDoc
   */
  public function SliderWidget(config:ext.slider.SliderWidget = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object is a map of config property names holding a <code>true</code> if changes to
   * that config should written back to its binding. Most commonly this is used to
   * indicate that the <code>value</code> config should be monitored and changes written back
   * to the bound value.
   * @default {value: 1}
   * @see #getTwoWayBindable()
   * @see #setTwoWayBindable()
   */
  override public native function get twoWayBindable():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set twoWayBindable(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Determines whether or not clicking on the Slider axis will change the slider.
   * @default true
   * @see #getClickToChange()
   * @see #setClickToChange()
   */
  public native function get clickToChange():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set clickToChange(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum value for any slider thumb.
   * @default 100
   * @see #getMaxValue()
   * @see #setMaxValue()
   */
  public native function get maxValue():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxValue(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum value for any slider thumb.
   * @default 0
   * @see #getMinValue()
   * @see #setMinValue()
   */
  public native function get minValue():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minValue(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This controls when the value of the slider is published to the <code>ViewModel</code>. By
   * default this is done only when the thumb is released (the change is complete). To
   * cause this to happen on every change of the thumb position, specify <code>false</code>. This
   * setting is <code>true</code> by default for improved performance on slower devices (such as
   * older browsers or tablets).
   * @default true
   * @see #getPublishOnComplete()
   * @see #setPublishOnComplete()
   */
  public native function get publishOnComplete():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set publishOnComplete(value:Boolean):void;

  [ExtConfig]
  /**
   * One more values for the position of the slider's thumb(s).
   * @default 0
   * @see #setValue()
   */
  public native function get value():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Orients the slider vertically rather than horizontally.
   * @default false
   * @see #getVertical()
   * @see #setVertical()
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.slider.Widget-cfg-clickToChange" rel="Ext.slider.Widget-cfg-clickToChange" class="docClass">clickToChange</a>.
   */
  public native function getClickToChange():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.slider.Widget-cfg-maxValue" rel="Ext.slider.Widget-cfg-maxValue" class="docClass">maxValue</a>.
   */
  public native function getMaxValue():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.slider.Widget-cfg-minValue" rel="Ext.slider.Widget-cfg-minValue" class="docClass">minValue</a>.
   */
  public native function getMinValue():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.slider.Widget-cfg-publishOnComplete" rel="Ext.slider.Widget-cfg-publishOnComplete" class="docClass">publishOnComplete</a>.
   */
  public native function getPublishOnComplete():Boolean;

  /**
   * Returns the current value of the slider
   * @param index The index of the thumb to return a value for
   * @return The current value of the slider at the given index, or an array of all thumb values if
   * no index is given.
   */
  public native function getValue(index:Number):*;

  /**
   * Returns an array of values - one for the location of each thumb
   * @return The set of thumb values
   */
  public native function getValues():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.slider.Widget-cfg-vertical" rel="Ext.slider.Widget-cfg-vertical" class="docClass">vertical</a>.
   */
  public native function getVertical():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.slider.Widget-cfg-clickToChange" rel="Ext.slider.Widget-cfg-clickToChange" class="docClass">clickToChange</a>.
   * @param clickToChange The new value.
   */
  public native function setClickToChange(clickToChange:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.slider.Widget-cfg-maxValue" rel="Ext.slider.Widget-cfg-maxValue" class="docClass">maxValue</a>.
   * @param maxValue The new value.
   */
  public native function setMaxValue(maxValue:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.slider.Widget-cfg-minValue" rel="Ext.slider.Widget-cfg-minValue" class="docClass">minValue</a>.
   * @param minValue The new value.
   */
  public native function setMinValue(minValue:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.slider.Widget-cfg-publishOnComplete" rel="Ext.slider.Widget-cfg-publishOnComplete" class="docClass">publishOnComplete</a>.
   * @param publishOnComplete The new value.
   */
  public native function setPublishOnComplete(publishOnComplete:Boolean):void;

  /**
   * Programmatically sets the value of the Slider. Ensures that the value is constrained within the minValue and
   * maxValue.
   *
   * <p>Setting a single value:
   *     // Set the second slider value, don't animate
   *     mySlider.setThumbValue(1, 50, false);</p>
   *
   * <p>Setting multiple values at once
   *     // Set 3 thumb values, animate
   *     mySlider.setThumbValue([20, 40, 60], true);</p>
   * @param index Index of the thumb to move. Alternatively, it can be an array of values to set
   * for each thumb in the slider.
   * @param value The value to set the slider to. (This will be constrained within minValue and maxValue)
   * @param animate Turn on or off animation
   * @default true
   * @return this
   */
  public native function setThumbValue(index:*, value:Number, animate:Boolean = true):ext.slider.MultiSlider;

  /**
   * Sets the value of <a href="#!/api/Ext.slider.Widget-cfg-value" rel="Ext.slider.Widget-cfg-value" class="docClass">value</a>.
   * @param value The new value.
   */
  public native function setValue(value:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.slider.Widget-cfg-vertical" rel="Ext.slider.Widget-cfg-vertical" class="docClass">vertical</a>.
   * @param vertical The new value.
   */
  public native function setVertical(vertical:Boolean):void;
}
}