package ext.sparkline {

[Uses("ext.sparkline.SparklineBarBase")]
[Uses("ext.sparkline.SparklineBase")]
[Uses("ext.Widget")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.sparkline.Bar", require)]
/**
 * Plots a bar chart of the values in the passed <a href="#!/api/Ext.sparkline.Bar-cfg-values" rel="Ext.sparkline.Bar-cfg-values" class="docClass">values</a> array.
 *
 * <p>See <a href="#!/api/Ext.sparkline.Base" rel="Ext.sparkline.Base" class="docClass">the base class</a> for a simple example.</p>
 */
public class SparklineBar extends ext.sparkline.SparklineBarBase {
  /**
   * @param config @inheritDoc
   */
  public function SparklineBar(config:ext.sparkline.SparklineBar = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The bar color for positive values.
   * @default #3366cc
   * @see #getBarColor()
   * @see #setBarColor()
   */
  public native function get barColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set barColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The pixel spacing between bars.
   * @default 1
   * @see #getBarSpacing()
   * @see #setBarSpacing()
   */
  public native function get barSpacing():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set barSpacing(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The pixel width of bars.
   * @default 4
   * @see #getBarWidth()
   * @see #setBarWidth()
   */
  public native function get barWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set barWidth(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If true then the y values supplied to plot will be clipped to fall
   * between <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeMin" rel="Ext.sparkline.Bar-cfg-chartRangeMin" class="docClass">chartRangeMin</a> and <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeMax" rel="Ext.sparkline.Bar-cfg-chartRangeMax" class="docClass">chartRangeMax</a> - By default chartRangeMin/Max just ensure that the chart
   * spans at least that range of values, but does not constrain it.
   * @default false
   * @see #getChartRangeClip()
   * @see #setChartRangeClip()
   */
  public native function get chartRangeClip():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set chartRangeClip(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.
   * @see #getChartRangeMax()
   * @see #setChartRangeMax()
   */
  public native function get chartRangeMax():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set chartRangeMax(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.
   * @see #getChartRangeMin()
   * @see #setChartRangeMin()
   */
  public native function get chartRangeMin():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set chartRangeMin(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object that uses range specifiers as keys to
   * indicate bar color values for a range of values. A range specifier is
   * specified in the form <code>[number]:[number]</code>, which indicates start and end range.
   * Omitting either means an open ended range.
   *
   * <p>For example, to render green bars on all values less than -1 and red on values
   * higher than 1, you would use:</p>
   *
   * <pre><code>colorMap: {
   *     // Open ended range, with max value -1
   *     ":-1": "green",
   *
   *     // Open ended range, with min value 1
   *     "1:": "red"
   * }
   * </code></pre>
   * @see #getColorMap()
   * @see #setColorMap()
   */
  public native function get colorMap():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set colorMap(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The bar color for negative values.
   * @default #f44
   * @see #getNegBarColor()
   * @see #setNegBarColor()
   */
  public native function get negBarColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set negBarColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The bar color for null values. Usually null values are omitted and not plotted. Setting
   * this config causes a very thin bar to be plotted with the special color in the case that null is a meaningful value in the series.
   * @see #getNullColor()
   * @see #setNullColor()
   */
  public native function get nullColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set nullColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An array of colours to use for stacked bar charts.
   * The first series will use the first value in the array, the second series will use the second, etc.
   * @default ['#3366cc', '#dc3912', '#ff9900', '#109618', '#66aa00', '#dd4477', '#0099c6', '#990099']
   * @see #getStackedBarColor()
   * @see #setStackedBarColor()
   */
  public native function get stackedBarColor():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set stackedBarColor(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Centers the Y axis at zero by default.
   * @default true
   * @see #getZeroAxis()
   * @see #setZeroAxis()
   */
  public native function get zeroAxis():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set zeroAxis(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The bar color for zero values.
   * @see #getZeroColor()
   * @see #setZeroColor()
   */
  public native function get zeroColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set zeroColor(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-barColor" rel="Ext.sparkline.Bar-cfg-barColor" class="docClass">barColor</a>.
   */
  public native function getBarColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-barSpacing" rel="Ext.sparkline.Bar-cfg-barSpacing" class="docClass">barSpacing</a>.
   */
  public native function getBarSpacing():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-barWidth" rel="Ext.sparkline.Bar-cfg-barWidth" class="docClass">barWidth</a>.
   */
  public native function getBarWidth():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeClip" rel="Ext.sparkline.Bar-cfg-chartRangeClip" class="docClass">chartRangeClip</a>.
   */
  public native function getChartRangeClip():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeMax" rel="Ext.sparkline.Bar-cfg-chartRangeMax" class="docClass">chartRangeMax</a>.
   */
  public native function getChartRangeMax():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeMin" rel="Ext.sparkline.Bar-cfg-chartRangeMin" class="docClass">chartRangeMin</a>.
   */
  public native function getChartRangeMin():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-colorMap" rel="Ext.sparkline.Bar-cfg-colorMap" class="docClass">colorMap</a>.
   */
  public native function getColorMap():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-negBarColor" rel="Ext.sparkline.Bar-cfg-negBarColor" class="docClass">negBarColor</a>.
   */
  public native function getNegBarColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-nullColor" rel="Ext.sparkline.Bar-cfg-nullColor" class="docClass">nullColor</a>.
   */
  public native function getNullColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-stackedBarColor" rel="Ext.sparkline.Bar-cfg-stackedBarColor" class="docClass">stackedBarColor</a>.
   */
  public native function getStackedBarColor():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-zeroAxis" rel="Ext.sparkline.Bar-cfg-zeroAxis" class="docClass">zeroAxis</a>.
   */
  public native function getZeroAxis():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Bar-cfg-zeroColor" rel="Ext.sparkline.Bar-cfg-zeroColor" class="docClass">zeroColor</a>.
   */
  public native function getZeroColor():String;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-barColor" rel="Ext.sparkline.Bar-cfg-barColor" class="docClass">barColor</a>.
   * @param barColor The new value.
   */
  public native function setBarColor(barColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-barSpacing" rel="Ext.sparkline.Bar-cfg-barSpacing" class="docClass">barSpacing</a>.
   * @param barSpacing The new value.
   */
  public native function setBarSpacing(barSpacing:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-barWidth" rel="Ext.sparkline.Bar-cfg-barWidth" class="docClass">barWidth</a>.
   * @param barWidth The new value.
   */
  public native function setBarWidth(barWidth:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeClip" rel="Ext.sparkline.Bar-cfg-chartRangeClip" class="docClass">chartRangeClip</a>.
   * @param chartRangeClip The new value.
   */
  public native function setChartRangeClip(chartRangeClip:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeMax" rel="Ext.sparkline.Bar-cfg-chartRangeMax" class="docClass">chartRangeMax</a>.
   * @param chartRangeMax The new value.
   */
  public native function setChartRangeMax(chartRangeMax:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-chartRangeMin" rel="Ext.sparkline.Bar-cfg-chartRangeMin" class="docClass">chartRangeMin</a>.
   * @param chartRangeMin The new value.
   */
  public native function setChartRangeMin(chartRangeMin:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-colorMap" rel="Ext.sparkline.Bar-cfg-colorMap" class="docClass">colorMap</a>.
   * @param colorMap The new value.
   */
  public native function setColorMap(colorMap:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-negBarColor" rel="Ext.sparkline.Bar-cfg-negBarColor" class="docClass">negBarColor</a>.
   * @param negBarColor The new value.
   */
  public native function setNegBarColor(negBarColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-nullColor" rel="Ext.sparkline.Bar-cfg-nullColor" class="docClass">nullColor</a>.
   * @param nullColor The new value.
   */
  public native function setNullColor(nullColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-stackedBarColor" rel="Ext.sparkline.Bar-cfg-stackedBarColor" class="docClass">stackedBarColor</a>.
   * @param stackedBarColor The new value.
   */
  public native function setStackedBarColor(stackedBarColor:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-zeroAxis" rel="Ext.sparkline.Bar-cfg-zeroAxis" class="docClass">zeroAxis</a>.
   * @param zeroAxis The new value.
   */
  public native function setZeroAxis(zeroAxis:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Bar-cfg-zeroColor" rel="Ext.sparkline.Bar-cfg-zeroColor" class="docClass">zeroColor</a>.
   * @param zeroColor The new value.
   */
  public native function setZeroColor(zeroColor:String):void;
}
}