package ext.sparkline {

[Uses("ext.sparkline.SparklineBase")]
[Uses("ext.Widget")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.sparkline.Box", require)]
/**
 * Generates a box plot graph from the provided <a href="#!/api/Ext.sparkline.Box-cfg-values" rel="Ext.sparkline.Box-cfg-values" class="docClass">values</a> array.
 *
 * <p>See <a href="http://en.wikipedia.org/wiki/Box_plot">Wikipedia Box Plots</a></p>
 *
 * <p>See <a href="#!/api/Ext.sparkline.Base" rel="Ext.sparkline.Base" class="docClass">the base class</a> for a simple example.</p>
 */
public class SparklineBox extends ext.sparkline.SparklineBase {
  /**
   * @param config @inheritDoc
   */
  public function SparklineBox(config:ext.sparkline.SparklineBox = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color of the box fill.
   * @default #cdf
   * @see #getBoxFillColor()
   * @see #setBoxFillColor()
   */
  public native function get boxFillColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set boxFillColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color of the box outline.
   * @default #000
   * @see #getBoxLineColor()
   * @see #setBoxLineColor()
   */
  public native function get boxLineColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set boxLineColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.
   * @see #getChartRangeMax()
   * @see #setChartRangeMax()
   */
  public native function get chartRangeMax():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set chartRangeMax(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.
   * @see #getChartRangeMin()
   * @see #setChartRangeMin()
   */
  public native function get chartRangeMin():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set chartRangeMin(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color of the median line.
   * @default #f00
   * @see #getMedianColor()
   * @see #setMedianColor()
   */
  public native function get medianColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set medianColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The fill color of the outlier circles.
   * @default #fff
   * @see #getOutlierFillColor()
   * @see #setOutlierFillColor()
   */
  public native function get outlierFillColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set outlierFillColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The inter-quartile range multipler used to calculate values that qualify as an outlier.
   * @default 1.5
   * @see #getOutlierIQR()
   * @see #setOutlierIQR()
   */
  public native function get outlierIQR():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set outlierIQR(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color of the outlier circles' outline.
   * @default #333
   * @see #getOutlierLineColor()
   * @see #setOutlierLineColor()
   */
  public native function get outlierLineColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set outlierLineColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * By default the points are calculated from the input values array. Set this to true to passthe pre-calculated points in the values config.
   * @default false
   * @see #getRaw()
   * @see #setRaw()
   */
  public native function get raw():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set raw(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configure as <code>false</code> to not show outlier circles.
   * @default true
   * @see #getShowOutliers()
   * @see #setShowOutliers()
   */
  public native function get showOutliers():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set showOutliers(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The radius of the outlier circles.
   * @default 1.5
   * @see #getSpotRadius()
   * @see #setSpotRadius()
   */
  public native function get spotRadius():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set spotRadius(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If set, a crosshair will be drawn at the specified value point.
   * @see #getTarget()
   * @see #setTarget()
   */
  public native function get target():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set target(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color of the crosshair drawn at the pointe specified by <a href="#!/api/Ext.sparkline.Box-cfg-target" rel="Ext.sparkline.Box-cfg-target" class="docClass">target</a>.
   * @default #4a2
   * @see #getTargetColor()
   * @see #setTargetColor()
   */
  public native function get targetColor():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set targetColor(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color of the whiskers.
   * @default #000
   * @see #getWhiskerColor()
   * @see #setWhiskerColor()
   */
  public native function get whiskerColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set whiskerColor(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-boxFillColor" rel="Ext.sparkline.Box-cfg-boxFillColor" class="docClass">boxFillColor</a>.
   */
  public native function getBoxFillColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-boxLineColor" rel="Ext.sparkline.Box-cfg-boxLineColor" class="docClass">boxLineColor</a>.
   */
  public native function getBoxLineColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-chartRangeMax" rel="Ext.sparkline.Box-cfg-chartRangeMax" class="docClass">chartRangeMax</a>.
   */
  public native function getChartRangeMax():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-chartRangeMin" rel="Ext.sparkline.Box-cfg-chartRangeMin" class="docClass">chartRangeMin</a>.
   */
  public native function getChartRangeMin():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-medianColor" rel="Ext.sparkline.Box-cfg-medianColor" class="docClass">medianColor</a>.
   */
  public native function getMedianColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-outlierFillColor" rel="Ext.sparkline.Box-cfg-outlierFillColor" class="docClass">outlierFillColor</a>.
   */
  public native function getOutlierFillColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-outlierIQR" rel="Ext.sparkline.Box-cfg-outlierIQR" class="docClass">outlierIQR</a>.
   */
  public native function getOutlierIQR():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-outlierLineColor" rel="Ext.sparkline.Box-cfg-outlierLineColor" class="docClass">outlierLineColor</a>.
   */
  public native function getOutlierLineColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-raw" rel="Ext.sparkline.Box-cfg-raw" class="docClass">raw</a>.
   */
  public native function getRaw():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-showOutliers" rel="Ext.sparkline.Box-cfg-showOutliers" class="docClass">showOutliers</a>.
   */
  public native function getShowOutliers():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-spotRadius" rel="Ext.sparkline.Box-cfg-spotRadius" class="docClass">spotRadius</a>.
   */
  public native function getSpotRadius():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-target" rel="Ext.sparkline.Box-cfg-target" class="docClass">target</a>.
   */
  public native function getTarget():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-targetColor" rel="Ext.sparkline.Box-cfg-targetColor" class="docClass">targetColor</a>.
   */
  public native function getTargetColor():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Box-cfg-whiskerColor" rel="Ext.sparkline.Box-cfg-whiskerColor" class="docClass">whiskerColor</a>.
   */
  public native function getWhiskerColor():String;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-boxFillColor" rel="Ext.sparkline.Box-cfg-boxFillColor" class="docClass">boxFillColor</a>.
   * @param boxFillColor The new value.
   */
  public native function setBoxFillColor(boxFillColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-boxLineColor" rel="Ext.sparkline.Box-cfg-boxLineColor" class="docClass">boxLineColor</a>.
   * @param boxLineColor The new value.
   */
  public native function setBoxLineColor(boxLineColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-chartRangeMax" rel="Ext.sparkline.Box-cfg-chartRangeMax" class="docClass">chartRangeMax</a>.
   * @param chartRangeMax The new value.
   */
  public native function setChartRangeMax(chartRangeMax:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-chartRangeMin" rel="Ext.sparkline.Box-cfg-chartRangeMin" class="docClass">chartRangeMin</a>.
   * @param chartRangeMin The new value.
   */
  public native function setChartRangeMin(chartRangeMin:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-medianColor" rel="Ext.sparkline.Box-cfg-medianColor" class="docClass">medianColor</a>.
   * @param medianColor The new value.
   */
  public native function setMedianColor(medianColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-outlierFillColor" rel="Ext.sparkline.Box-cfg-outlierFillColor" class="docClass">outlierFillColor</a>.
   * @param outlierFillColor The new value.
   */
  public native function setOutlierFillColor(outlierFillColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-outlierIQR" rel="Ext.sparkline.Box-cfg-outlierIQR" class="docClass">outlierIQR</a>.
   * @param outlierIQR The new value.
   */
  public native function setOutlierIQR(outlierIQR:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-outlierLineColor" rel="Ext.sparkline.Box-cfg-outlierLineColor" class="docClass">outlierLineColor</a>.
   * @param outlierLineColor The new value.
   */
  public native function setOutlierLineColor(outlierLineColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-raw" rel="Ext.sparkline.Box-cfg-raw" class="docClass">raw</a>.
   * @param raw The new value.
   */
  public native function setRaw(raw:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-showOutliers" rel="Ext.sparkline.Box-cfg-showOutliers" class="docClass">showOutliers</a>.
   * @param showOutliers The new value.
   */
  public native function setShowOutliers(showOutliers:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-spotRadius" rel="Ext.sparkline.Box-cfg-spotRadius" class="docClass">spotRadius</a>.
   * @param spotRadius The new value.
   */
  public native function setSpotRadius(spotRadius:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-target" rel="Ext.sparkline.Box-cfg-target" class="docClass">target</a>.
   * @param target The new value.
   */
  public native function setTarget(target:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-targetColor" rel="Ext.sparkline.Box-cfg-targetColor" class="docClass">targetColor</a>.
   * @param targetColor The new value.
   */
  public native function setTargetColor(targetColor:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Box-cfg-whiskerColor" rel="Ext.sparkline.Box-cfg-whiskerColor" class="docClass">whiskerColor</a>.
   * @param whiskerColor The new value.
   */
  public native function setWhiskerColor(whiskerColor:String):void;
}
}