package ext.sparkline {

[Uses("ext.sparkline.SparklineBarBase")]
[Uses("ext.sparkline.SparklineBase")]
[Uses("ext.Widget")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.sparkline.TriState", require)]
/**
 * Plots bars based upon "win"/"draw" or "lose" status of the input <a href="#!/api/Ext.sparkline.TriState-cfg-values" rel="Ext.sparkline.TriState-cfg-values" class="docClass">values</a> array. Positive values mean
 * a win, zero a draw, and negative a lose.
 *
 * <p>See <a href="#!/api/Ext.sparkline.Base" rel="Ext.sparkline.Base" class="docClass">the base class</a> for a simple example.</p>
 */
public class SparklineTriState extends ext.sparkline.SparklineBarBase {
  /**
   * @param config @inheritDoc
   */
  public function SparklineTriState(config:ext.sparkline.SparklineTriState = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The pixel spacing between each bar.
   * @default 1
   * @see #getBarSpacing()
   * @see #setBarSpacing()
   */
  public native function get barSpacing():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set barSpacing(value:Number):void;

  [ExtConfig]
  /**
   * The pixel width of each bar.
   * @default 4
   * @see #setBarWidth()
   */
  public native function get barWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set barWidth(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object that uses range specifiers as keys to
   * indicate bar color values for a range of values. A range specifier is
   * specified in the form <code>[number]:[number]</code>, which indicates start and end range.
   * Omitting either means an open ended range.
   *
   * <p>For example, to render green bars on all values less than -1 and red on values
   * higher than 1, you would use:</p>
   *
   * <pre><code>colorMap: {
   *     // Open ended range, with max value -1
   *     ":-1": "green",
   *
   *     // Open ended range, with min value 1
   *     "1:": "red"
   * }
   * </code></pre>
   * @default {}
   * @see #getColorMap()
   * @see #setColorMap()
   */
  public native function get colorMap():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set colorMap(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color for negative value bars.
   * @default #f44
   * @see #getNegBarColor()
   * @see #setNegBarColor()
   */
  public native function get negBarColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set negBarColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color for positive value bars.
   * @default #6f6
   * @see #getPosBarColor()
   * @see #setPosBarColor()
   */
  public native function get posBarColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set posBarColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The color for zero value bars.
   * @default #999
   * @see #getZeroBarColor()
   * @see #setZeroBarColor()
   */
  public native function get zeroBarColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set zeroBarColor(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.TriState-cfg-barSpacing" rel="Ext.sparkline.TriState-cfg-barSpacing" class="docClass">barSpacing</a>.
   */
  public native function getBarSpacing():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.TriState-cfg-colorMap" rel="Ext.sparkline.TriState-cfg-colorMap" class="docClass">colorMap</a>.
   */
  public native function getColorMap():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.TriState-cfg-negBarColor" rel="Ext.sparkline.TriState-cfg-negBarColor" class="docClass">negBarColor</a>.
   */
  public native function getNegBarColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.TriState-cfg-posBarColor" rel="Ext.sparkline.TriState-cfg-posBarColor" class="docClass">posBarColor</a>.
   */
  public native function getPosBarColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.TriState-cfg-zeroBarColor" rel="Ext.sparkline.TriState-cfg-zeroBarColor" class="docClass">zeroBarColor</a>.
   */
  public native function getZeroBarColor():String;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.TriState-cfg-barSpacing" rel="Ext.sparkline.TriState-cfg-barSpacing" class="docClass">barSpacing</a>.
   * @param barSpacing The new value.
   */
  public native function setBarSpacing(barSpacing:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.TriState-cfg-barWidth" rel="Ext.sparkline.TriState-cfg-barWidth" class="docClass">barWidth</a>.
   * @param barWidth The new value.
   */
  public native function setBarWidth(barWidth:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.TriState-cfg-colorMap" rel="Ext.sparkline.TriState-cfg-colorMap" class="docClass">colorMap</a>.
   * @param colorMap The new value.
   */
  public native function setColorMap(colorMap:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.TriState-cfg-negBarColor" rel="Ext.sparkline.TriState-cfg-negBarColor" class="docClass">negBarColor</a>.
   * @param negBarColor The new value.
   */
  public native function setNegBarColor(negBarColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.TriState-cfg-posBarColor" rel="Ext.sparkline.TriState-cfg-posBarColor" class="docClass">posBarColor</a>.
   * @param posBarColor The new value.
   */
  public native function setPosBarColor(posBarColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.TriState-cfg-zeroBarColor" rel="Ext.sparkline.TriState-cfg-zeroBarColor" class="docClass">zeroBarColor</a>.
   * @param zeroBarColor The new value.
   */
  public native function setZeroBarColor(zeroBarColor:String):void;
}
}