package ext.state {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.state.Provider", require)]
/**
 * Fires when a state change occurs.
 * @eventType ext.state.events.Provider_key_valueEvent.STATE_CHANGE
 */
[Event(name = 'onStateChange', type = 'ext.state.events.Provider_key_valueEvent')]
/**
 * Abstract base class for state provider implementations. The provider is responsible
 * for setting values  and extracting values to/from the underlying storage source. The
 * storage source can vary and the details should be implemented in a subclass. For example
 * a provider could use a server side database or the browser localstorage where supported.
 *
 *
 *
 *
 * <p>This class provides methods for encoding and decoding <b>typed</b> variables including
 * dates and defines the Provider interface. By default these methods put the value and the
 * type information into a delimited string that can be stored. These should be overridden in
 * a subclass if you want to change the format of the encoded value and subsequent decoding.</p>
 */
public class Provider extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function Provider(config:ext.state.Provider = null) {
    super();
  }

  [ExtConfig]
  /**
   * A string to prefix to items stored in the underlying state store.
   * Defaults to <tt>'ext-'</tt>
   * @default 'ext-'
   */
  public native function get prefix():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set prefix(value:String):void;

  /**
   * Clears a value from the state
   * @param name The key name
   */
  public native function clear(name:String):void;

  /**
   * Decodes a string previously encoded with <a href="#!/api/Ext.state.Provider-method-encodeValue" rel="Ext.state.Provider-method-encodeValue" class="docClass">encodeValue</a>.
   * @param value The value to decode
   * @return The decoded value
   */
  public native function decodeValue(value:String):*;

  /**
   * Encodes a value including type information.  Decode with <a href="#!/api/Ext.state.Provider-method-decodeValue" rel="Ext.state.Provider-method-decodeValue" class="docClass">decodeValue</a>.
   * @param value The value to encode
   * @return The encoded value
   */
  public native function encodeValue(value:*):String;

  /**
   * Returns the current value for a key
   * @param name The key name
   * @param defaultValue A default value to return if the key's value is not found
   * @return The state data
   */
  public native function get(name:String, defaultValue:*):*;

  /**
   * Sets the value for a key
   * @param name The key name
   * @param value The value to set
   */
  public native function set(name:String, value:*):void;
}
}