package ext.tab {
import ext.panel.PanelBar;
import ext.util.IFocusableContainer;

[Uses("ext.panel.PanelBar")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.tab.Bar", require)]
/**
 * Fired when the currently-active tab has changed
 * @eventType ext.tab.events.TabBar_tab_cardEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.tab.events.TabBar_tab_cardEvent')]
/**
 * TabBar is used internally by a <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">TabPanel</a> and typically should not
 * need to be created manually.
 */
public class TabBar extends ext.panel.PanelBar implements ext.util.IFocusableContainer {
  /**
   * @param config @inheritDoc
   */
  public function TabBar(config:ext.tab.TabBar = null) {
    super();
  }

  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  public native function get focusableContainerEl():*;

  /**
   * @private
   */
  public native function set focusableContainerEl(value:*):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Tab Bar, or subclass thereof.
   * @default true
   */
  public native function get isTabBar():Boolean;

  /**
   * @private
   */
  public native function set isTabBar(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to follow WAI-ARIA requirement and activate tab when it is navigated to
   * with arrow keys, or <code>false</code> to disable that behavior. When activation on focus
   * is disabled, users will have to use arrow keys to focus a tab, and then press
   * Space key to activate it.
   * @default true
   * @see #getActivateOnFocus()
   * @see #setActivateOnFocus()
   */
  public native function get activateOnFocus():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activateOnFocus(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  public native function get enableFocusableContainer():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to ensure the active tab is scrolled into view when the tab changes, the text, the
   * icon or the glyph. This is only applicable if using an overflow scroller.
   * @default true
   * @since 5.1.1
   */
  public native function get ensureActiveVisibleOnChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ensureActiveVisibleOnChange(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  [Deprecated(replacement = "maxTabWidth")]
  [ExtConfig]
  /**
   * The maximum width for a tab in this tab Bar. Defaults to the tab Panel's <a href="#!/api/Ext.tab.Panel-cfg-maxTabWidth" rel="Ext.tab.Panel-cfg-maxTabWidth" class="docClass">maxTabWidth</a> value.
   */
  public native function get maxTabWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxTabWidth(value:Number):void;

  [Deprecated(replacement = "minTabWidth")]
  [ExtConfig]
  /**
   * The minimum width for a tab in this tab Bar. Defaults to the tab Panel's <a href="#!/api/Ext.tab.Panel-cfg-minTabWidth" rel="Ext.tab.Panel-cfg-minTabWidth" class="docClass">minTabWidth</a> value.
   */
  public native function get minTabWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minTabWidth(value:Number):void;

  [ExtConfig]
  /**
   * True to not show the full background on the tabbar
   * @default false
   */
  public native function get plain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plain(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The rotation of the tabs.  Can be one of the following values:
   *
   * <ul>
   * <li><code>default</code> - use the default rotation, depending on the dock position (see below)</li>
   * <li><code>0</code> - no rotation</li>
   * <li><code>1</code> - rotate 90deg clockwise</li>
   * <li><code>2</code> - rotate 90deg counter-clockwise</li>
   * </ul>
   *
   *
   * <p>The default behavior of this config depends on the dock position:</p>
   *
   * <ul>
   * <li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>
   * <li><code>'right'</code> - <code>1</code></li>
   * <li><code>'left'</code> - <code>2</code></li>
   * </ul>
   * @default 'default'
   * @see #getTabRotation()
   * @see #setTabRotation()
   */
  public native function get tabRotation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabRotation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to stretch all tabs to the height of the tallest tab when the tabBar
   * is docked horizontally, or the width of the widest tab when the tabBar is
   * docked vertically.
   * @default true
   * @see #getTabStretchMax()
   * @see #setTabStretchMax()
   */
  public native function get tabStretchMax():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabStretchMax(value:Boolean):void;

  /**
   * Ensure the passed tab is visible if using overflow scrolling
   * @param tab The tab, item in the owning <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a> or
   * the index of the item to scroll to. Defaults to the active tab.
   */
  public native function ensureTabVisible(tab:* = undefined):void;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Bar-cfg-activateOnFocus" rel="Ext.tab.Bar-cfg-activateOnFocus" class="docClass">activateOnFocus</a>.
   */
  public native function getActivateOnFocus():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Bar-cfg-tabRotation" rel="Ext.tab.Bar-cfg-tabRotation" class="docClass">tabRotation</a>.
   */
  public native function getTabRotation():*;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Bar-cfg-tabStretchMax" rel="Ext.tab.Bar-cfg-tabStretchMax" class="docClass">tabStretchMax</a>.
   */
  public native function getTabStretchMax():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Bar-cfg-activateOnFocus" rel="Ext.tab.Bar-cfg-activateOnFocus" class="docClass">activateOnFocus</a>.
   * @param activateOnFocus The new value.
   */
  public native function setActivateOnFocus(activateOnFocus:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Bar-cfg-tabRotation" rel="Ext.tab.Bar-cfg-tabRotation" class="docClass">tabRotation</a>.
   * @param tabRotation The new value.
   */
  public native function setTabRotation(tabRotation:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Bar-cfg-tabStretchMax" rel="Ext.tab.Bar-cfg-tabStretchMax" class="docClass">tabStretchMax</a>.
   * @param tabStretchMax The new value.
   */
  public native function setTabStretchMax(tabStretchMax:Boolean):void;
}
}