package ext.tab {
import ext.Component;
import ext.panel.Panel;

[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.tab.Panel", require)]
/**
 * Fires before a tab change (activated by <a href="#!/api/Ext.tab.Panel-method-setActiveTab" rel="Ext.tab.Panel-method-setActiveTab" class="docClass">setActiveTab</a>). Return false in any listener to cancel
 * the tabchange
 * @eventType ext.tab.events.TabPanel_newCard_oldCardEvent.BEFORE_TAB_CHANGE
 */
[Event(name = 'onBeforeTabChange', type = 'ext.tab.events.TabPanel_newCard_oldCardEvent')]
/**
 * Fires when a new tab has been activated (activated by <a href="#!/api/Ext.tab.Panel-method-setActiveTab" rel="Ext.tab.Panel-method-setActiveTab" class="docClass">setActiveTab</a>).
 * @eventType ext.tab.events.TabPanel_newCard_oldCardEvent.TAB_CHANGE
 */
[Event(name = 'onTabChange', type = 'ext.tab.events.TabPanel_newCard_oldCardEvent')]
/**
 * A basic tab container. TabPanels can be used exactly like a standard <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> for
 * layout purposes, but also have special support for containing child Components
 * (<code><a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a></code>) that are managed using a
 * <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">CardLayout layout manager</a>, and displayed as separate tabs.
 *
 * <p><strong>Note:</strong> By default, a tab's close tool <em>destroys</em> the child tab Component and all its descendants.
 * This makes the child tab Component, and all its descendants <strong>unusable</strong>.  To enable re-use of a tab,
 * configure the TabPanel with <code><a href="#!/api/Ext.tab.Panel-cfg-autoDestroy" rel="Ext.tab.Panel-cfg-autoDestroy" class="docClass">autoDestroy: false</a></code>.</p>
 *
 * <h2>TabPanel's layout</h2>
 *
 * <p>TabPanels use a Dock layout to position the <a href="#!/api/Ext.tab.Bar" rel="Ext.tab.Bar" class="docClass">TabBar</a> at the top of the widget.
 * Panels added to the TabPanel will have their header hidden by default because the Tab will
 * automatically take the Panel's configured title and icon.</p>
 *
 * <p>TabPanels use their <a href="#!/api/Ext.panel.Header" rel="Ext.panel.Header" class="docClass">header</a> or <a href="#!/api/Ext.panel.Panel-cfg-fbar" rel="Ext.panel.Panel-cfg-fbar" class="docClass">footer</a>
 * element (depending on the <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a> configuration) to accommodate the tab selector buttons.
 * This means that a TabPanel will not display any configured title, and will not display any configured
 * header <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">tools</a>.</p>
 *
 * <p>To display a header, embed the TabPanel in a <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Panel</a> which uses
 * <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout: 'fit'</a></code>.</p>
 *
 * <h2>Controlling tabs</h2>
 *
 * <p>Configuration options for the <a href="#!/api/Ext.tab.Tab" rel="Ext.tab.Tab" class="docClass">Ext.tab.Tab</a> that represents the component can be passed in
 * by specifying the tabConfig option:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     width: 400,
 *     height: 400,
 *     renderTo: document.body,
 *     items: [{
 *         title: 'Foo'
 *     }, {
 *         title: 'Bar',
 *         tabConfig: {
 *             title: 'Custom Title',
 *             tooltip: 'A button tooltip'
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <h2>Vetoing Changes</h2>
 *
 * <p>User interaction when changing the tabs can be vetoed by listening to the <a href="#!/api/Ext.tab.Panel-event-beforetabchange" rel="Ext.tab.Panel-event-beforetabchange" class="docClass">beforetabchange</a> event.
 * By returning <code>false</code>, the tab change will not occur.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     width: 200,
 *     height: 200,
 *     listeners: {
 *         beforetabchange: function(tabs, newTab, oldTab) {
 *             return newTab.title != 'P2';
 *         }
 *     },
 *     items: [{
 *         title: 'P1'
 *     }, {
 *         title: 'P2'
 *     }, {
 *         title: 'P3'
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Examples</h1>
 *
 * <p>Here is a basic TabPanel rendered to the body. This also shows the useful configuration <a href="#!/api/Ext.tab.Panel-cfg-activeTab" rel="Ext.tab.Panel-cfg-activeTab" class="docClass">activeTab</a>,
 * which allows you to set the active tab on render.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     width: 300,
 *     height: 200,
 *     activeTab: 0,
 *     items: [
 *         {
 *             title: 'Tab 1',
 *             bodyPadding: 10,
 *             html : 'A simple tab'
 *         },
 *         {
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>It is easy to control the visibility of items in the tab bar. Specify hidden: true to have the
 * tab button hidden initially. Items can be subsequently hidden and show by accessing the
 * tab property on the child item.</p>
 *
 * <pre class='inline-example '><code>var tabs = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     width: 400,
 *     height: 400,
 *     renderTo: document.body,
 *     items: [{
 *         title: 'Home',
 *         html: 'Home',
 *         itemId: 'home'
 *     }, {
 *         title: 'Users',
 *         html: 'Users',
 *         itemId: 'users',
 *         hidden: true
 *     }, {
 *         title: 'Tickets',
 *         html: 'Tickets',
 *         itemId: 'tickets'
 *     }]
 * });
 *
 * <a href="#!/api/Ext-method-defer" rel="Ext-method-defer" class="docClass">Ext.defer</a>(function(){
 *     tabs.child('#home').tab.hide();
 *     var users = tabs.child('#users');
 *     users.tab.show();
 *     tabs.setActiveTab(users);
 * }, 1000);
 * </code></pre>
 *
 * <p>You can remove the background of the TabBar by setting the <a href="#!/api/Ext.tab.Panel-cfg-plain" rel="Ext.tab.Panel-cfg-plain" class="docClass">plain</a> property to <code>true</code>.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     width: 300,
 *     height: 200,
 *     activeTab: 0,
 *     plain: true,
 *     items: [
 *         {
 *             title: 'Tab 1',
 *             bodyPadding: 10,
 *             html : 'A simple tab'
 *         },
 *         {
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>Another useful configuration of TabPanel is <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a>. This allows you to change the
 * position where the tabs are displayed. The available options for this are <code>'top'</code> (default) and
 * <code>'bottom'</code>.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     width: 300,
 *     height: 200,
 *     activeTab: 0,
 *     bodyPadding: 10,
 *     tabPosition: 'bottom',
 *     items: [
 *         {
 *             title: 'Tab 1',
 *             html : 'A simple tab'
 *         },
 *         {
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>The <a href="#!/api/Ext.tab.Panel-method-setActiveTab" rel="Ext.tab.Panel-method-setActiveTab" class="docClass">setActiveTab</a> is a very useful method in TabPanel which will allow you to change the
 * current active tab. You can either give it an index or an instance of a tab. For example:</p>
 *
 * <pre class='inline-example '><code>var tabs = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     items: [
 *         {
 *             id   : 'my-tab',
 *             title: 'Tab 1',
 *             html : 'A simple tab'
 *         },
 *         {
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 * var tab = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('my-tab');
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     text    : 'Select the first tab',
 *     scope   : this,
 *     handler : function() {
 *         tabs.setActiveTab(tab);
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     text    : 'Select the second tab',
 *     scope   : this,
 *     handler : function() {
 *         tabs.setActiveTab(1);
 *     },
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>The <a href="#!/api/Ext.tab.Panel-method-getActiveTab" rel="Ext.tab.Panel-method-getActiveTab" class="docClass">getActiveTab</a> is a another useful method in TabPanel which will return the current active tab.</p>
 *
 * <pre class='inline-example '><code>var tabs = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     items: [
 *         {
 *             title: 'Tab 1',
 *             html : 'A simple tab'
 *         },
 *         {
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     text    : 'Get active tab',
 *     scope   : this,
 *     handler : function() {
 *         var tab = tabs.getActiveTab();
 *         alert('Current tab: ' + tab.title);
 *     },
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>Adding a new tab is very simple with a TabPanel. You simple call the <a href="#!/api/Ext.tab.Panel-method-add" rel="Ext.tab.Panel-method-add" class="docClass">add</a> method with an config
 * object for a panel.</p>
 *
 * <pre class='inline-example '><code>var tabs = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     items: [
 *         {
 *             title: 'Tab 1',
 *             html : 'A simple tab'
 *         },
 *         {
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     text    : 'New tab',
 *     scope   : this,
 *     handler : function() {
 *         var tab = tabs.add({
 *             // we use the tabs.items property to get the length of current items/tabs
 *             title: 'Tab ' + (tabs.items.length + 1),
 *             html : 'Another one'
 *         });
 *
 *         tabs.setActiveTab(tab);
 *     },
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>Additionally, removing a tab is very also simple with a TabPanel. You simple call the <a href="#!/api/Ext.tab.Panel-method-remove" rel="Ext.tab.Panel-method-remove" class="docClass">remove</a> method
 * with an config object for a panel.</p>
 *
 * <pre class='inline-example '><code>var tabs = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *     items: [
 *         {
 *             title: 'Tab 1',
 *             html : 'A simple tab'
 *         },
 *         {
 *             id   : 'remove-this-tab',
 *             title: 'Tab 2',
 *             html : 'Another one'
 *         }
 *     ],
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     text    : 'Remove tab',
 *     scope   : this,
 *     handler : function() {
 *         var tab = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('remove-this-tab');
 *         tabs.remove(tab);
 *     },
 *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class TabPanel extends ext.panel.Panel {
  /**
   * @param config @inheritDoc
   */
  public function TabPanel(config:ext.tab.TabPanel = null) {
    super();
  }

  [ExtConfig]
  /**
   * Doesn't apply for <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">TabPanel</a>, use <a href="#!/api/Ext.tab.Panel-cfg-activeTab" rel="Ext.tab.Panel-cfg-activeTab" class="docClass">activeTab</a> instead.
   * @see #setActiveItem()
   */
  override public native function get activeItem():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set activeItem(value:*):void;

  [ExtConfig]
  /**
   * Optional configuration object for the internal <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">card layout</a>.
   * If present, this is passed straight through to the layout's constructor
   * @see #setLayout()
   */
  override public native function get layout():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set layout(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The tab to activate initially. Either an ID, index or the tab component itself. If null, no tab will be set as active.
   * @see #getActiveTab()
   * @see #setActiveTab()
   */
  public native function get activeTab():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activeTab(value:*):void;

  [ExtConfig]
  /**
   * True by default to defer the rendering of child <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> to the browsers DOM
   * until a tab is activated. False will render all contained <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> as soon as
   * the <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">layout</a> is rendered. If there is a significant amount of content or a lot of
   * heavy controls being rendered into panels that are not displayed by default, setting this to true might improve
   * performance.
   *
   * <p>The deferredRender property is internally passed to the layout manager for TabPanels (<a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a>) as its <a href="#!/api/Ext.layout.container.Card-cfg-deferredRender" rel="Ext.layout.container.Card-cfg-deferredRender" class="docClass">Ext.layout.container.Card.deferredRender</a> configuration value.</p>
   *
   * <p><strong>Note</strong>: leaving deferredRender as true means that the content within an unactivated tab will not be available</p>
   * @default true
   */
  public native function get deferredRender():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set deferredRender(value:Boolean):void;

  [ExtConfig]
  /**
   * The class added to each child item of this TabPanel.
   * @default 'x-tabpanel-child'
   */
  public native function get itemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemCls(value:String):void;

  [ExtConfig]
  /**
   * The maximum width for each tab.
   */
  public native function get maxTabWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxTabWidth(value:Number):void;

  [ExtConfig]
  /**
   * The minimum width for a tab in the <a href="#!/api/Ext.tab.Panel-cfg-tabBar" rel="Ext.tab.Panel-cfg-tabBar" class="docClass">tabBar</a>.
   */
  public native function get minTabWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minTabWidth(value:Number):void;

  [ExtConfig]
  /**
   * True to not show the full background on the TabBar.
   * @default false
   */
  public native function get plain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plain(value:Boolean):void;

  [ExtConfig]
  /**
   * True to instruct each Panel added to the TabContainer to not render its header element.
   * This is to ensure that the title of the panel does not appear twice.
   * @default true
   */
  public native function get removePanelHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set removePanelHeader(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional configuration object for the internal <a href="#!/api/Ext.tab.Bar" rel="Ext.tab.Bar" class="docClass">Ext.tab.Bar</a>.
   * If present, this is passed straight through to the TabBar's constructor
   * @see #getTabBar()
   * @see #setTabBar()
   */
  public native function get tabBar():ext.tab.TabBar;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabBar(value:ext.tab.TabBar):void;

  [ExtConfig]
  /**
   * If specified, the <a href="#!/api/Ext.tab.Panel-cfg-tabBar" rel="Ext.tab.Panel-cfg-tabBar" class="docClass">tabBar</a> will be rendered as an item of the TabPanel's
   * Header and the specified <code>tabBarHeaderPosition</code> will be used as the Panel header's
   * <a href="#!/api/Ext.panel.Header-cfg-itemPosition" rel="Ext.panel.Header-cfg-itemPosition" class="docClass">Ext.panel.Header.itemPosition</a>.  If not specified, the <a href="#!/api/Ext.tab.Panel-cfg-tabBar" rel="Ext.tab.Panel-cfg-tabBar" class="docClass">tabBar</a>
   * will be rendered as a docked item at <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a>.
   */
  public native function get tabBarHeaderPosition():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tabBarHeaderPosition(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The position where the tab strip should be rendered. Possible values are:
   *
   * <ul>
   * <li>top</li>
   * <li>bottom</li>
   * <li>left</li>
   * <li>right</li>
   * </ul>
   * @default 'top'
   * @see #getTabPosition()
   * @see #setTabPosition()
   */
  public native function get tabPosition():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabPosition(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The rotation of the tabs.  Can be one of the following values:
   *
   * <ul>
   * <li><code>'default'</code> use the default rotation, depending on <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a> (see below)</li>
   * <li><code>0</code> - no rotation</li>
   * <li><code>1</code> - rotate 90deg clockwise</li>
   * <li><code>2</code> - rotate 90deg counter-clockwise</li>
   * </ul>
   *
   *
   * <p>The default behavior of this config depends on the <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a>:</p>
   *
   * <ul>
   * <li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>
   * <li><code>'right'</code> - <code>1</code></li>
   * <li><code>'left'</code> - <code>2</code></li>
   * </ul>
   * @default 'default'
   * @see #getTabRotation()
   * @see #setTabRotation()
   */
  public native function get tabRotation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabRotation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to stretch all tabs to the height of the tallest tab when the tabBar
   * is docked horizontally, or the width of the widest tab when the tabBar is
   * docked vertically.
   * @default true
   * @see #getTabStretchMax()
   * @see #setTabStretchMax()
   */
  public native function get tabStretchMax():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabStretchMax(value:Boolean):void;

  /**
   * Makes sure we have a Tab for each item added to the TabPanel
   */
  override protected native function onAdd(component:ext.Component, position:Number):void;

  /**
   * Returns the item that is currently active inside this TabPanel.
   * @return The currently active item.
   */
  public native function getActiveTab():ext.Component;

  /**
   * Returns the <a href="#!/api/Ext.tab.Bar" rel="Ext.tab.Bar" class="docClass">Ext.tab.Bar</a> associated with this tabPanel.
   * @return The tabBar for this tabPanel
   */
  public native function getTabBar():ext.tab.TabBar;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a>.
   */
  public native function getTabPosition():String;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Panel-cfg-tabRotation" rel="Ext.tab.Panel-cfg-tabRotation" class="docClass">tabRotation</a>.
   */
  public native function getTabRotation():*;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Panel-cfg-tabStretchMax" rel="Ext.tab.Panel-cfg-tabStretchMax" class="docClass">tabStretchMax</a>.
   */
  public native function getTabStretchMax():Boolean;

  /**
   * Makes the given card active. Makes it the visible card in the TabPanel's CardLayout and highlights the Tab.
   * @param card The card to make active. Either an ID, index or the component itself.
   * @return The resulting active child Component. The call may have been vetoed, or otherwise
   * modified by an event listener.
   */
  public native function setActiveTab(card:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Panel-cfg-tabBar" rel="Ext.tab.Panel-cfg-tabBar" class="docClass">tabBar</a>.
   * @param tabBar The new value.
   */
  public native function setTabBar(tabBar:ext.tab.TabBar):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Panel-cfg-tabPosition" rel="Ext.tab.Panel-cfg-tabPosition" class="docClass">tabPosition</a>.
   * @param tabPosition The new value.
   */
  public native function setTabPosition(tabPosition:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Panel-cfg-tabRotation" rel="Ext.tab.Panel-cfg-tabRotation" class="docClass">tabRotation</a>.
   * @param tabRotation The new value.
   */
  public native function setTabRotation(tabRotation:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Panel-cfg-tabStretchMax" rel="Ext.tab.Panel-cfg-tabStretchMax" class="docClass">tabStretchMax</a>.
   * @param tabStretchMax The new value.
   */
  public native function setTabStretchMax(tabStretchMax:Boolean):void;
}
}