package ext.tip {

[Uses("ext.tip.ToolTip")]
[Uses("ext.tip.Tip")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.tip.QuickTip", require)]
/**
 * A specialized tooltip class for tooltips that can be specified in markup and automatically managed
 * by the global <a href="#!/api/Ext.tip.QuickTipManager" rel="Ext.tip.QuickTipManager" class="docClass">Ext.tip.QuickTipManager</a> instance.  See the QuickTipManager documentation for
 * additional usage details and examples.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>(); // Instantiate the QuickTipManager
 *
 *  <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', {
 *
 *      renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *      text: 'My Button',
 *      listeners: {
 *
 *          afterrender: function(me) {
 *
 *              // Register the new tip with an element's ID
 *              <a href="#!/api/Ext.tip.QuickTipManager-method-register" rel="Ext.tip.QuickTipManager-method-register" class="docClass">Ext.tip.QuickTipManager.register</a>({
 *                  target: me.getId(), // Target button's ID
 *                  title : 'My Tooltip',  // QuickTip Header
 *                  text  : 'My Button has a QuickTip' // Tip content
 *              });
 *
 *          }
 *      }
 *  });
 * </code></pre>
 */
public class QuickTip extends ext.tip.ToolTip {
  /**
   * @param config @inheritDoc
   */
  public function QuickTip(config:ext.tip.QuickTip = null) {
    super();
  }

  [ExtConfig]
  /**
   * The target HTMLElement, <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or id to associate with this Quicktip.
   *
   * <p>Defaults to the document.</p>
   * @see #setTarget()
   */
  override public native function get target():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set target(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The title text to be used to display in the Tip header.  May be a string
   * (including HTML tags) or an <a href="#!/api/Ext.panel.Title" rel="Ext.panel.Title" class="docClass">Ext.panel.Title</a> config object.
   * @default '&amp;160;'
   * @see #getTitle()
   * @see #setTitle()
   */
  override public native function get title():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set title(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to automatically use the element's DOM title value if available.
   * @default false
   */
  public native function get interceptTitles():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set interceptTitles(value:Boolean):void;

  [ExtConfig]
  /**
   * An HTML fragment, or a <a href="#!/api/Ext.dom.Helper" rel="Ext.dom.Helper" class="docClass">DomHelper</a> specification to use as the layout element content.
   * The HTML content is added after the component is rendered, so the document will not contain this HTML at the time
   * the <a href="#!/api/Ext.tip.QuickTip-event-render" rel="Ext.tip.QuickTip-event-render" class="docClass">render</a> event is fired. This content is inserted into the body <em>before</em> any configured <a href="#!/api/Ext.tip.QuickTip-cfg-contentEl" rel="Ext.tip.QuickTip-cfg-contentEl" class="docClass">contentEl</a>
   * is appended.
   */
  public native function get text():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set text(value:*):void;

  /**
   * Hides a visible tip or cancels an impending show for a particular element.
   * @param el The element that is the target of
   * the tip or ID of the element.
   */
  public native function cancelShow(el:*):void;

  /**
   * Configures a new quick tip instance and assigns it to a target element.
   *
   * <p>For example usage, see the <a href="#!/api/Ext.tip.QuickTipManager" rel="Ext.tip.QuickTipManager" class="docClass">Ext.tip.QuickTipManager</a> class header.</p>
   * @param config The config object with the following properties:
   * @param config.target (required) The target HTMLElement, <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or
   * id to associate with this Quicktip.  See <a href="#!/api/Ext.tip.QuickTip-cfg-target" rel="Ext.tip.QuickTip-cfg-target" class="docClass">target</a>.
   * @param config.text Tip body content.  See <a href="#!/api/Ext.tip.QuickTip-cfg-text" rel="Ext.tip.QuickTip-cfg-text" class="docClass">text</a>.
   * @param config.title Tip header.  See <a href="#!/api/Ext.tip.QuickTip-cfg-title" rel="Ext.tip.QuickTip-cfg-title" class="docClass">title</a>.
   * @param config.autoHide False to prevent the tip from automatically hiding on
   * mouseleave.  See <a href="#!/api/Ext.tip.QuickTip-cfg-autoHide" rel="Ext.tip.QuickTip-cfg-autoHide" class="docClass">autoHide</a>.
   * @param config.cls An optional extra CSS class that will be added to the tip.  See
   * <a href="#!/api/Ext.tip.QuickTip-cfg-cls" rel="Ext.tip.QuickTip-cfg-cls" class="docClass">cls</a>.
   * @param config.dismissDelay Delay in milliseconds before the tooltip automatically
   * hides (overrides singleton value).  See <a href="#!/api/Ext.tip.QuickTip-cfg-dismissDelay" rel="Ext.tip.QuickTip-cfg-dismissDelay" class="docClass">dismissDelay</a>.
   * @param config.width Tip width in pixels.  See <a href="#!/api/Ext.tip.QuickTip-cfg-width" rel="Ext.tip.QuickTip-cfg-width" class="docClass">width</a>.
   */
  public native function register(config:Object):void;

  /**
   * Removes this quick tip from its element and destroys it.
   * @param el The element from which the quick tip
   * is to be removed or ID of the element.
   */
  public native function unregister(el:*):void;
}
}