package ext.tip {
import ext.panel.Panel;

[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.tip.Tip", require)]
/**
 * This is the base class for <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">Ext.tip.QuickTip</a> and <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a> that provides the basic layout and
 * positioning that all tip-based classes require. This class can be used directly for simple, statically-positioned
 * tips that are displayed programmatically, or it can be extended to provide custom tip implementations.
 */
public class Tip extends ext.panel.Panel {
  /**
   * @param config @inheritDoc
   */
  public function Tip(config:ext.tip.Tip = null) {
    super();
  }

  [ExtConfig]
  /**
   * True to render a close tool button into the tooltip header.
   * @default false
   */
  override public native function get closable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set closable(value:Boolean):void;

  [ExtConfig]
  /**
   * The action to take when the close header tool is clicked:
   *
   * <ul>
   * <li><p><strong><a href="#!/api/Ext.tip.Tip-method-destroy" rel="Ext.tip.Tip-method-destroy" class="docClass">destroy</a></strong> : <a href="#!/api/Ext.tip.Tip-method-remove" rel="Ext.tip.Tip-method-remove" class="docClass">remove</a> the window from the DOM and
   * <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a> it and all descendant Components. The
   * window will <strong>not</strong> be available to be redisplayed via the <a href="#!/api/Ext.tip.Tip-method-show" rel="Ext.tip.Tip-method-show" class="docClass">show</a> method.</p></li>
   * <li><p><strong><a href="#!/api/Ext.tip.Tip-method-hide" rel="Ext.tip.Tip-method-hide" class="docClass">hide</a></strong> : <strong>Default.</strong> <a href="#!/api/Ext.tip.Tip-method-hide" rel="Ext.tip.Tip-method-hide" class="docClass">hide</a> the window by setting visibility
   * to hidden and applying negative offsets. The window will be available to be
   * redisplayed via the <a href="#!/api/Ext.tip.Tip-method-show" rel="Ext.tip.Tip-method-show" class="docClass">show</a> method.</p></li>
   * </ul>
   *
   *
   * <p><strong>Note:</strong> This behavior has changed! setting <em>does</em> affect the <a href="#!/api/Ext.tip.Tip-method-close" rel="Ext.tip.Tip-method-close" class="docClass">close</a> method
   * which will invoke the approriate closeAction.</p>
   * @default 'hide'
   */
  override public native function get closeAction():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set closeAction(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum width of the tip in pixels.
   * @default 500
   * @see #getMaxWidth()
   * @see #setMaxWidth()
   */
  override public native function get maxWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set maxWidth(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum width of the tip in pixels.
   * @default 40
   * @see #getMinWidth()
   * @see #setMinWidth()
   */
  override public native function get minWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set minWidth(value:Number):void;

  [ExtConfig]
  /**
   * <code>true</code> or "sides" for the default effect, "frame" for 4-way shadow, and "drop"
   * for bottom-right shadow.
   * @default "sides"
   */
  override public native function get shadow():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set shadow(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Width in pixels of the tip.  Width will be ignored if it
   * exceeds the bounds of <a href="#!/api/Ext.tip.Tip-cfg-minWidth" rel="Ext.tip.Tip-cfg-minWidth" class="docClass">minWidth</a> or <a href="#!/api/Ext.tip.Tip-cfg-maxWidth" rel="Ext.tip.Tip-cfg-maxWidth" class="docClass">maxWidth</a>.
   * @default 'auto'
   * @see #getWidth()
   * @see #setWidth()
   */
  override public native function get width():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set width(value:*):void;

  [ExtConfig]
  /**
   * If <code>true</code>, then the tooltip will be automatically constrained to stay within
   * the browser viewport.
   * @default true
   */
  public native function get constrainPosition():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainPosition(value:Boolean):void;

  /**
   * Shows this tip at the specified XY position.  Example usage:
   *
   * <pre><code>// Show the tip at x:50 and y:100
   * tip.showAt([50,100]);
   * </code></pre>
   * @param x The new x position or array of <code>[x,y]</code>.
   * @param y The new y position
   * @param animate True to animate the Component into its new position. You may also pass an
   * animation configuration.
   */
  override public native function showAt(x:Array, y:Number = NaN, animate:* = undefined):void;
}
}