package ext.toolbar {
import ext.container.Container;
import ext.data.TreeModel;
import ext.data.TreeStore;
import ext.util.IFocusableContainer;

[Uses("ext.container.Container")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.toolbar.Breadcrumb", require)]
/**
 * Fires when the user changes the selected record. In contrast to the <a href="#!/api/Ext.toolbar.Breadcrumb-event-selectionchange" rel="Ext.toolbar.Breadcrumb-event-selectionchange" class="docClass">selectionchange</a> event, this does
 * <em>not</em> fire at render time, only in response to user activity.
 * @eventType ext.toolbar.events.Breadcrumb_node_prevNodeEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.toolbar.events.Breadcrumb_node_prevNodeEvent')]
/**
 * Fires when the selected node changes. At render time, this event will fire
 * indicating that the configured <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-selection" rel="Ext.toolbar.Breadcrumb-cfg-selection" class="docClass">selection</a> has been selected.
 * @eventType ext.toolbar.events.Breadcrumb_node_prevNodeEvent.SELECTION_CHANGE
 */
[Event(name = 'onSelectionChange', type = 'ext.toolbar.events.Breadcrumb_node_prevNodeEvent')]
/**
 * A toolbar that displays hierarchical data from a <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a>
 * as a trail of breadcrumb buttons.  Each button represents a node in the store.  A click
 * on a button will "select" that node in the tree.  Non-leaf nodes will display their
 * child nodes on a dropdown menu of the corresponding button in the breadcrumb trail,
 * and a click on an item in the menu will trigger selection of the corresponding child
 * node.
 *
 * <p>The selection can be set programmatically  using <a href="#!/api/Ext.toolbar.Breadcrumb-method-setSelection" rel="Ext.toolbar.Breadcrumb-method-setSelection" class="docClass">setSelection</a>, or retrieved
 * using <a href="#!/api/Ext.toolbar.Breadcrumb-method-getSelection" rel="Ext.toolbar.Breadcrumb-method-getSelection" class="docClass">getSelection</a>.</p>
 */
public class Breadcrumb extends ext.container.Container implements ext.util.IFocusableContainer {
  /**
   * @param config @inheritDoc
   */
  public function Breadcrumb(config:ext.toolbar.Breadcrumb = null) {
    super();
  }

  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  public native function get focusableContainerEl():*;

  /**
   * @private
   */
  public native function set focusableContainerEl(value:*):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Button UI to use for breadcrumb items.  Use extjs-breadcrumb-ui to
   * add special styling to the breadcrumb arrows
   * @default 'plain-toolbar'
   * @see #getButtonUI()
   * @see #setButtonUI()
   */
  public native function get buttonUI():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set buttonUI(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the field in the data model to display in the navigation items of
   * this breadcrumb toolbar
   * @default 'text'
   * @see #getDisplayField()
   * @see #setDisplayField()
   */
  public native function get displayField():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set displayField(value:String):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  public native function get enableFocusableContainer():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The overflowHandler for this Breadcrumb:
   *
   * <ul>
   * <li><code>null</code> - hidden overflow</li>
   * <li><code>'scroller'</code> to render left/right scroller buttons on either side of the breadcrumb</li>
   * <li><code>'menu'</code> to render the overflowing buttons as items of an overflow menu.</li>
   * </ul>
   * @default null
   * @see #getOverflowHandler()
   * @see #setOverflowHandler()
   */
  public native function get overflowHandler():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set overflowHandler(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The selected node, or <code>"root"</code> to select the root node
   * @default 'root'
   * @see #getSelection()
   * @see #setSelection()
   */
  public native function get selection():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selection(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Controls whether or not icons of tree nodes are displayed in the breadcrumb
   * buttons.  There are 3 possible values for this config:
   *
   * <ol>
   * <li><p>unspecified (<code>null</code>) - the default value. In this mode only icons that are
   * specified in the tree data using (<a href="#!/api/Ext.data.NodeInterface-cfg-icon" rel="Ext.data.NodeInterface-cfg-icon" class="docClass">icon</a>
   * or <a href="#!/api/Ext.data.NodeInterface-cfg-iconCls" rel="Ext.data.NodeInterface-cfg-iconCls" class="docClass">iconCls</a> will be displayed, but the
   * default "folder" and "leaf" icons will not be displayed.</p></li>
   * <li><p><code>true</code> - Icons specified in the tree data will be displayed, and the default
   * "folder" and "leaf" icons will be displayed for nodes that do not specify
   * an <code>icon</code> or <code>iconCls</code>.</p></li>
   * <li><p><code>false</code> - No icons will be displayed in the breadcrumb buttons, only text.</p></li>
   * </ol>
   * @default null
   * @see #getShowIcons()
   * @see #setShowIcons()
   */
  public native function get showIcons():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set showIcons(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Controls whether or not icons of tree nodes are displayed in the breadcrumb
   * menu items. There are 3 possible values for this config:
   *
   * <ol>
   * <li><p>unspecified (<code>null</code>) - the default value. In this mode only icons that are
   * specified in the tree data using (<a href="#!/api/Ext.data.NodeInterface-cfg-icon" rel="Ext.data.NodeInterface-cfg-icon" class="docClass">icon</a>
   * or <a href="#!/api/Ext.data.NodeInterface-cfg-iconCls" rel="Ext.data.NodeInterface-cfg-iconCls" class="docClass">iconCls</a> will be displayed, but the
   * default "folder" and "leaf" icons will not be displayed.</p></li>
   * <li><p><code>true</code> - Icons specified in the tree data will be displayed, and the default
   * "folder" and "leaf" icons will be displayed for nodes that do not specify
   * an <code>icon</code> or <code>iconCls</code>.</p></li>
   * <li><p><code>false</code> - No icons will be displayed on the breadcrumb menu items, only text.</p></li>
   * </ol>
   * @default null
   * @see #getShowMenuIcons()
   * @see #setShowMenuIcons()
   */
  public native function get showMenuIcons():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set showMenuIcons(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The TreeStore that this breadcrumb toolbar should use as its data source
   * @see #getStore()
   * @see #setStore()
   */
  public native function get store():ext.data.TreeStore;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set store(value:ext.data.TreeStore):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>false</code> to use regular <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a>s instead of <a href="#!/api/Ext.button.Split" rel="Ext.button.Split" class="docClass">Split Buttons</a>.  When <code>true</code>, a click on the body of a button
   * will navigate to the specified node, and a click on the arrow will show a menu
   * containing the the child nodes.  When <code>false</code>, the only mode of navigation is
   * the menu, since a click anywhere on the button will show the menu.
   * @default true
   * @see #getUseSplitButtons()
   * @see #setUseSplitButtons()
   */
  public native function get useSplitButtons():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set useSplitButtons(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-buttonUI" rel="Ext.toolbar.Breadcrumb-cfg-buttonUI" class="docClass">buttonUI</a>.
   */
  public native function getButtonUI():String;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-displayField" rel="Ext.toolbar.Breadcrumb-cfg-displayField" class="docClass">displayField</a>.
   */
  public native function getDisplayField():String;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-overflowHandler" rel="Ext.toolbar.Breadcrumb-cfg-overflowHandler" class="docClass">overflowHandler</a>.
   */
  public native function getOverflowHandler():String;

  /**
   * Returns the currently selected <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">node</a>.
   * @return node The selected node (or null if there is no
   * selection).
   */
  public native function getSelection():ext.data.TreeModel;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-showIcons" rel="Ext.toolbar.Breadcrumb-cfg-showIcons" class="docClass">showIcons</a>.
   */
  public native function getShowIcons():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-showMenuIcons" rel="Ext.toolbar.Breadcrumb-cfg-showMenuIcons" class="docClass">showMenuIcons</a>.
   */
  public native function getShowMenuIcons():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-store" rel="Ext.toolbar.Breadcrumb-cfg-store" class="docClass">store</a>.
   */
  public native function getStore():ext.data.TreeStore;

  /**
   * Returns the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-useSplitButtons" rel="Ext.toolbar.Breadcrumb-cfg-useSplitButtons" class="docClass">useSplitButtons</a>.
   */
  public native function getUseSplitButtons():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-buttonUI" rel="Ext.toolbar.Breadcrumb-cfg-buttonUI" class="docClass">buttonUI</a>.
   * @param buttonUI The new value.
   */
  public native function setButtonUI(buttonUI:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-displayField" rel="Ext.toolbar.Breadcrumb-cfg-displayField" class="docClass">displayField</a>.
   * @param displayField The new value.
   */
  public native function setDisplayField(displayField:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-overflowHandler" rel="Ext.toolbar.Breadcrumb-cfg-overflowHandler" class="docClass">overflowHandler</a>.
   * @param overflowHandler The new value.
   */
  public native function setOverflowHandler(overflowHandler:String):void;

  /**
   * Selects the passed <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">node</a> in the breadcrumb component.
   * @param node The node in the breadcrumb <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-store" rel="Ext.toolbar.Breadcrumb-cfg-store" class="docClass">store</a> to
   * select as the active node.
   * @return this The breadcrumb component
   */
  public native function setSelection(node:ext.data.TreeModel):ext.toolbar.Breadcrumb;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-showIcons" rel="Ext.toolbar.Breadcrumb-cfg-showIcons" class="docClass">showIcons</a>.
   * @param showIcons The new value.
   */
  public native function setShowIcons(showIcons:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-showMenuIcons" rel="Ext.toolbar.Breadcrumb-cfg-showMenuIcons" class="docClass">showMenuIcons</a>.
   * @param showMenuIcons The new value.
   */
  public native function setShowMenuIcons(showMenuIcons:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-store" rel="Ext.toolbar.Breadcrumb-cfg-store" class="docClass">store</a>.
   * @param store The new value.
   */
  public native function setStore(store:ext.data.TreeStore):void;

  /**
   * Sets the value of <a href="#!/api/Ext.toolbar.Breadcrumb-cfg-useSplitButtons" rel="Ext.toolbar.Breadcrumb-cfg-useSplitButtons" class="docClass">useSplitButtons</a>.
   * @param useSplitButtons The new value.
   */
  public native function setUseSplitButtons(useSplitButtons:Boolean):void;
}
}