package ext.toolbar {
import ext.data.AbstractStore;
import ext.data.Store;
import ext.util.IStoreHolder;

[Uses("ext.toolbar.Toolbar")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.toolbar.Paging", require)]
/**
 * Fires just before the active page is changed. Return false to prevent the active page from being changed.
 * @eventType ext.toolbar.events.PagingToolbar_pageEvent.BEFORE_CHANGE
 */
[Event(name = 'onBeforeChange', type = 'ext.toolbar.events.PagingToolbar_pageEvent')]
/**
 * Fires after the active page has been changed.
 * @eventType ext.toolbar.events.PagingToolbar_pageDataEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.toolbar.events.PagingToolbar_pageDataEvent')]
/**
 * As the number of records increases, the time required for the browser to render them increases. Paging is used to
 * reduce the amount of data exchanged with the client. Note: if there are more records/rows than can be viewed in the
 * available screen area, vertical scrollbars will be added.
 *
 * <p>Paging is typically handled on the server side (see exception below). The client sends parameters to the server side,
 * which the server needs to interpret and then respond with the appropriate data.</p>
 *
 * <p><a href="#!/api/Ext.toolbar.Paging" rel="Ext.toolbar.Paging" class="docClass">Ext.toolbar.Paging</a> is a specialized toolbar that is bound to a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> and provides automatic
 * paging control. This Component <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a>s blocks of data into the <a href="#!/api/Ext.toolbar.Paging-cfg-store" rel="Ext.toolbar.Paging-cfg-store" class="docClass">store</a> by passing
 * parameters used for paging criteria.</p>
 *
 * <p>Note: The <a href="#!/api/Ext.toolbar.Paging-cfg-store" rel="Ext.toolbar.Paging-cfg-store" class="docClass">store</a> specified must support paging as defined by <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code>. In particular, this means
 * that <code><a href="#!/api/Ext.data.ChainedStore" rel="Ext.data.ChainedStore" class="docClass">Ext.data.ChainedStore</a></code> is not supported.</p>
 *
 * <p><p><img src="" alt="Ext.toolbar.Paging component" width="" height=""/></p></p>
 *
 * <p>Paging Toolbar is typically used as one of the Grid's toolbars:</p>
 *
 * <pre><code>var itemsPerPage = 2; // set the number of items you want per page
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     id: 'simpsonsStore',
 *     autoLoad: false,
 *     fields: ['name', 'email', 'phone'],
 *     pageSize: itemsPerPage, // items per page
 *     proxy: {
 *         type: 'ajax',
 *         url: 'pagingstore.js', // url that will load data with respect to start and limit params
 *         reader: {
 *             type: 'json',
 *             rootProperty: 'items',
 *             totalProperty: 'total'
 *         }
 *     }
 * });
 *
 * // specify segment of data you want to load using params
 * store.load({
 *     params: {
 *         start: 0,
 *         limit: itemsPerPage
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     width: 400,
 *     height: 125,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *
 *     store: 'simpsonsStore',
 *
 *     columns: [{
 *         text: 'Name',
 *         dataIndex: 'name'
 *     }, {
 *         text: 'Email',
 *         dataIndex: 'email',
 *         flex: 1
 *     }, {
 *         text: 'Phone',
 *         dataIndex: 'phone'
 *     }],
 *
 *     bbar: {
 *         xtype: 'pagingtoolbar',
 *         displayInfo: true
 *     }
 * });
 * </code></pre>
 *
 * <p>To use paging, you need to set a pageSize configuration on the Store, and pass the paging requirements to
 * the server when the store is first loaded.</p>
 *
 * <pre><code>store.load({
 *     params: {
 *         // specify params for the first page load if using paging
 *         start: 0,
 *         limit: myPageSize,
 *         // other params
 *         foo:   'bar'
 *     }
 * });
 * </code></pre>
 *
 * <p>If using <a href="#!/api/Ext.data.Store-cfg-autoLoad" rel="Ext.data.Store-cfg-autoLoad" class="docClass">store's autoLoad</a> configuration:</p>
 *
 * <pre><code>var myStore = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     autoLoad: {start: 0, limit: 25},
 *     ...
 * });
 * </code></pre>
 *
 * <p>The packet sent back from the server would have this form:</p>
 *
 * <pre><code>{
 *     "success": true,
 *     "results": 2000,
 *     "items": [ // Note: this must be an Array
 *         { "id":  1, "name": "Bill", "occupation": "Gardener" },
 *         { "id":  2, "name":  "Ben", "occupation": "Horticulturalist" },
 *         ...
 *         { "id": 25, "name":  "Sue", "occupation": "Botanist" }
 *     ]
 * }
 * </code></pre>
 *
 * <h2>Paging with Local Data</h2>
 *
 * <p>Paging can also be accomplished with local data using extensions:</p>
 *
 * <ul>
 * <li><a href="http://sencha.com/forum/showthread.php?t=71532">Ext.ux.data.PagingStore</a></li>
 * <li>Paging Memory Proxy (examples/ux/PagingMemoryProxy.js)</li>
 * </ul>
 */
public class PagingToolbar extends ext.toolbar.Toolbar implements ext.util.IStoreHolder {
  /**
   * @param config @inheritDoc
   */
  public function PagingToolbar(config:ext.toolbar.PagingToolbar = null) {
    super();
  }

  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @private
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  [ExtConfig]
  /**
   * Customizable piece of the default paging text. Note that this string is formatted using
   * {0} as a token that is replaced by the number of total pages. This token should be preserved when overriding this
   * string if showing the total page count is desired.
   * @default 'of {0}'
   */
  public native function get afterPageText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterPageText(value:String):void;

  [ExtConfig]
  /**
   * The text displayed before the input item.
   * @default 'Page'
   */
  public native function get beforePageText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforePageText(value:String):void;

  [ExtConfig]
  /**
   * true to display the displayMsg
   * @default false
   */
  public native function get displayInfo():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set displayInfo(value:Boolean):void;

  [ExtConfig]
  /**
   * The paging status message to display. Note that this string is
   * formatted using the braced numbers {0}-{2} as tokens that are replaced by the values for start, end and total
   * respectively. These tokens should be preserved when overriding this string if showing those values is desired.
   * @default 'Displaying {0} - {1} of {2}'
   */
  public native function get displayMsg():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set displayMsg(value:String):void;

  [ExtConfig]
  /**
   * The message to display when no records are found.
   * @default 'No data to display'
   */
  public native function get emptyMsg():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set emptyMsg(value:String):void;

  [ExtConfig]
  /**
   * The quicktip text displayed for the first page button.
   * <strong>Note</strong>: quick tips must be initialized for the quicktip to show.
   * @default 'First Page'
   */
  public native function get firstText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set firstText(value:String):void;

  [ExtConfig]
  /**
   * The width in pixels of the input field used to display and change the current page number.
   * @default 30
   */
  public native function get inputItemWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inputItemWidth(value:Number):void;

  [ExtConfig]
  /**
   * The quicktip text displayed for the last page button.
   * <strong>Note</strong>: quick tips must be initialized for the quicktip to show.
   * @default 'Last Page'
   */
  public native function get lastText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lastText(value:String):void;

  [ExtConfig]
  /**
   * The quicktip text displayed for the next page button.
   * <strong>Note</strong>: quick tips must be initialized for the quicktip to show.
   * @default 'Next Page'
   */
  public native function get nextText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nextText(value:String):void;

  [ExtConfig]
  /**
   * true to insert any configured items <em>before</em> the paging buttons.
   * @default false
   */
  public native function get prependButtons():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set prependButtons(value:Boolean):void;

  [ExtConfig]
  /**
   * The quicktip text displayed for the previous page button.
   * <strong>Note</strong>: quick tips must be initialized for the quicktip to show.
   * @default 'Previous Page'
   */
  public native function get prevText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set prevText(value:String):void;

  [ExtConfig]
  /**
   * The quicktip text displayed for the Refresh button.
   * <strong>Note</strong>: quick tips must be initialized for the quicktip to show.
   * @default 'Refresh'
   */
  public native function get refreshText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set refreshText(value:String):void;

  [ExtConfig]
  /**
   * The data source to which the paging toolbar is bound (must be the same store instance
   * used in the grid / tree). Acceptable values for this property are:
   *
   * <ul>
   * <li>Any <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> class or subclass</li>
   * <li>An <a href="#!/api/Ext.data.Store-cfg-storeId" rel="Ext.data.Store-cfg-storeId" class="docClass">ID of a store</a></li>
   * </ul>
   *
   *
   * <p>If no <code>store</code> is provided, the <code>store</code> of the owner component (if there is an
   * owner and it has a store) is used. The owner store is bound when this component
   * is rendered.</p>
   * @see #setStore()
   */
  public native function get store():ext.data.Store;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set store(value:*):void;

  /**
   * Binds a store to this instance.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Refresh the current page, has the same effect as clicking the 'refresh' button.
   * Fires the <a href="#!/api/Ext.toolbar.Paging-event-beforechange" rel="Ext.toolbar.Paging-event-beforechange" class="docClass">beforechange</a> event. If the event returns <code>false</code>, then
   * the load will not be attempted.
   * @return <code>true</code> if the load was passed to the store.
   */
  public native function doRefresh():void;

  /**
   * Gets the current store instance.
   * @return The store, null if one does not exist.
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * Move to the first page, has the same effect as clicking the 'first' button.
   * Fires the <a href="#!/api/Ext.toolbar.Paging-event-beforechange" rel="Ext.toolbar.Paging-event-beforechange" class="docClass">beforechange</a> event. If the event returns <code>false</code>, then
   * the load will not be attempted.
   * @return <code>true</code> if the load was passed to the store.
   */
  public native function moveFirst():void;

  /**
   * Move to the last page, has the same effect as clicking the 'last' button.
   * Fires the <a href="#!/api/Ext.toolbar.Paging-event-beforechange" rel="Ext.toolbar.Paging-event-beforechange" class="docClass">beforechange</a> event. If the event returns <code>false</code>, then
   * the load will not be attempted.
   * @return <code>true</code> if the load was passed to the store.
   */
  public native function moveLast():void;

  /**
   * Move to the next page, has the same effect as clicking the 'next' button.
   * Fires the <a href="#!/api/Ext.toolbar.Paging-event-beforechange" rel="Ext.toolbar.Paging-event-beforechange" class="docClass">beforechange</a> event. If the event returns <code>false</code>, then
   * the load will not be attempted.
   * @return <code>true</code> if the load was passed to the store.
   */
  public native function moveNext():void;

  /**
   * Move to the previous page, has the same effect as clicking the 'previous' button.
   * Fires the <a href="#!/api/Ext.toolbar.Paging-event-beforechange" rel="Ext.toolbar.Paging-event-beforechange" class="docClass">beforechange</a> event. If the event returns <code>false</code>, then
   * the load will not be attempted.
   * @return <code>true</code> if the load was passed to the store.
   */
  public native function movePrevious():void;

  /**
   * Sets the store to the specified store.
   * @since 5.0.0
   */
  public native function setStore(store:Object):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Template method, it is called when an existing store is unbound
   * from the current instance.
   * @param store The store being unbound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onUnbindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}