package ext.toolbar {
import ext.container.Container;
import ext.util.IFocusableContainer;

[Uses("ext.container.Container")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Native("Ext.toolbar.Toolbar", require)]
/**
 * Fires after the overflow state has changed if this toolbar has been configured with
 * an <code><a href="#!/api/Ext.toolbar.Toolbar-cfg-overflowHandler" rel="Ext.toolbar.Toolbar-cfg-overflowHandler" class="docClass">overflowHandler</a></code>.
 * @eventType ext.toolbar.events.Toolbar_lastHiddenCount_hiddenCount_hiddenItemsEvent.OVERFLOW_CHANGE
 */
[Event(name = 'onOverflowChange', type = 'ext.toolbar.events.Toolbar_lastHiddenCount_hiddenCount_hiddenItemsEvent')]
/**
 * Basic Toolbar class. Although the <a href="#!/api/Ext.container.Container-cfg-defaultType" rel="Ext.container.Container-cfg-defaultType" class="docClass">defaultType</a> for
 * Toolbar is <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a>, Toolbar elements (child items for the Toolbar container)
 * may be virtually any type of Component. Toolbar elements can be created explicitly via their
 * constructors, or implicitly via their xtypes, and can be <a href="#!/api/Ext.toolbar.Toolbar-method-add" rel="Ext.toolbar.Toolbar-method-add" class="docClass">add</a>ed dynamically.
 *
 * <h2>Some items have shortcut strings for creation:</h2>
 *
 * <table>
 * <thead>
 * <tr>
 * <th style="text-align:left;"> Shortcut </th>
 * <th style="text-align:left;"> xtype         </th>
 * <th style="text-align:left;"> Class                         </th>
 * <th style="text-align:left;"> Description</th>
 * </tr>
 * </thead>
 * <tbody>
 * <tr>
 * <td style="text-align:left;"> '->'     </td>
 * <td style="text-align:left;"> <code>tbfill</code>      </td>
 * <td style="text-align:left;"> <a href="#!/api/Ext.toolbar.Fill" rel="Ext.toolbar.Fill" class="docClass">Ext.toolbar.Fill</a>      </td>
 * <td style="text-align:left;"> begin using the right-justified button container</td>
 * </tr>
 * <tr>
 * <td style="text-align:left;"> '-'      </td>
 * <td style="text-align:left;"> <code>tbseparator</code> </td>
 * <td style="text-align:left;"> <a href="#!/api/Ext.toolbar.Separator" rel="Ext.toolbar.Separator" class="docClass">Ext.toolbar.Separator</a> </td>
 * <td style="text-align:left;"> add a vertical separator bar between toolbar items</td>
 * </tr>
 * <tr>
 * <td style="text-align:left;"> ' '      </td>
 * <td style="text-align:left;"> <code>tbspacer</code>    </td>
 * <td style="text-align:left;"> <a href="#!/api/Ext.toolbar.Spacer" rel="Ext.toolbar.Spacer" class="docClass">Ext.toolbar.Spacer</a>    </td>
 * <td style="text-align:left;"> add horizontal space between elements</td>
 * </tr>
 * </tbody>
 * </table>
 *
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a>', {
 *     renderTo: document.body,
 *     width   : 500,
 *     items: [
 *         {
 *             // xtype: 'button', // default for Toolbars
 *             text: 'Button'
 *         },
 *         {
 *             xtype: 'splitbutton',
 *             text : 'Split Button'
 *         },
 *         // begin using the right-justified button container
 *         '-&gt;', // same as { xtype: 'tbfill' }
 *         {
 *             xtype    : 'textfield',
 *             name     : 'field1',
 *             emptyText: 'enter search term'
 *         },
 *         // add a vertical separator bar between toolbar items
 *         '-', // same as {xtype: 'tbseparator'} to create <a href="#!/api/Ext.toolbar.Separator" rel="Ext.toolbar.Separator" class="docClass">Ext.toolbar.Separator</a>
 *         'text 1', // same as {xtype: 'tbtext', text: 'text1'} to create <a href="#!/api/Ext.toolbar.TextItem" rel="Ext.toolbar.TextItem" class="docClass">Ext.toolbar.TextItem</a>
 *         { xtype: 'tbspacer' },// same as ' ' to create <a href="#!/api/Ext.toolbar.Spacer" rel="Ext.toolbar.Spacer" class="docClass">Ext.toolbar.Spacer</a>
 *         'text 2',
 *         { xtype: 'tbspacer', width: 50 }, // add a 50px space
 *         'text 3'
 *     ]
 * });
 * </code></pre>
 *
 * <p>Toolbars have <a href="#!/api/Ext.toolbar.Toolbar-method-enable" rel="Ext.toolbar.Toolbar-method-enable" class="docClass">enable</a> and <a href="#!/api/Ext.toolbar.Toolbar-method-disable" rel="Ext.toolbar.Toolbar-method-disable" class="docClass">disable</a> methods which when called, will
 * enable/disable all items within your toolbar.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a>', {
 *     renderTo: document.body,
 *     width   : 400,
 *     items: [
 *         {
 *             text: 'Button'
 *         },
 *         {
 *             xtype: 'splitbutton',
 *             text : 'Split Button'
 *         },
 *         '-&gt;',
 *         {
 *             xtype    : 'textfield',
 *             name     : 'field1',
 *             emptyText: 'enter search term'
 *         }
 *     ]
 * });
 * </code></pre>
 *
 * <p>Example</p>
 *
 * <pre class='inline-example '><code>var enableBtn = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     text    : 'Enable All Items',
 *     disabled: true,
 *     scope   : this,
 *     handler : function() {
 *         //disable the enable button and enable the disable button
 *         enableBtn.disable();
 *         disableBtn.enable();
 *
 *         //enable the toolbar
 *         toolbar.enable();
 *     }
 * });
 *
 * var disableBtn = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *     text    : 'Disable All Items',
 *     scope   : this,
 *     handler : function() {
 *         //enable the enable button and disable button
 *         disableBtn.disable();
 *         enableBtn.enable();
 *
 *         //disable the toolbar
 *         toolbar.disable();
 *     }
 * });
 *
 * var toolbar = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a>', {
 *     renderTo: document.body,
 *     width   : 400,
 *     margin  : '5 0 0 0',
 *     items   : [enableBtn, disableBtn]
 * });
 * </code></pre>
 *
 * <p>Adding items to and removing items from a toolbar is as simple as calling the <a href="#!/api/Ext.toolbar.Toolbar-method-add" rel="Ext.toolbar.Toolbar-method-add" class="docClass">add</a>
 * and <a href="#!/api/Ext.toolbar.Toolbar-method-remove" rel="Ext.toolbar.Toolbar-method-remove" class="docClass">remove</a> methods. There is also a <a href="#!/api/Ext.toolbar.Toolbar-method-removeAll" rel="Ext.toolbar.Toolbar-method-removeAll" class="docClass">removeAll</a> method
 * which remove all items within the toolbar.</p>
 *
 * <pre class='inline-example '><code>var toolbar = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a>', {
 *     renderTo: document.body,
 *     width   : 700,
 *     items: [
 *         {
 *             text: 'Example Button'
 *         }
 *     ]
 * });
 *
 * var addedItems = [];
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a>', {
 *     renderTo: document.body,
 *     width   : 700,
 *     margin  : '5 0 0 0',
 *     items   : [
 *         {
 *             text   : 'Add a button',
 *             scope  : this,
 *             handler: function() {
 *                 var text = prompt('Please enter the text for your button:');
 *                 addedItems.push(toolbar.add({
 *                     text: text
 *                 }));
 *             }
 *         },
 *         {
 *             text   : 'Add a text item',
 *             scope  : this,
 *             handler: function() {
 *                 var text = prompt('Please enter the text for your item:');
 *                 addedItems.push(toolbar.add(text));
 *             }
 *         },
 *         {
 *             text   : 'Add a toolbar separator',
 *             scope  : this,
 *             handler: function() {
 *                 addedItems.push(toolbar.add('-'));
 *             }
 *         },
 *         {
 *             text   : 'Add a toolbar spacer',
 *             scope  : this,
 *             handler: function() {
 *                 addedItems.push(toolbar.add('-&gt;'));
 *             }
 *         },
 *         '-&gt;',
 *         {
 *             text   : 'Remove last inserted item',
 *             scope  : this,
 *             handler: function() {
 *                 if (addedItems.length) {
 *                     toolbar.remove(addedItems.pop());
 *                 } else if (toolbar.items.length) {
 *                     toolbar.remove(toolbar.items.last());
 *                 } else {
 *                     alert('No items in the toolbar');
 *                 }
 *             }
 *         },
 *         {
 *             text   : 'Remove all items',
 *             scope  : this,
 *             handler: function() {
 *                 toolbar.removeAll();
 *             }
 *         }
 *     ]
 * });
 * </code></pre>
 */
public class Toolbar extends ext.container.Container implements ext.util.IFocusableContainer {
  /**
   * Creates a new Toolbar
   * @param config A config object or an array of buttons to <a href="#!/api/Ext.toolbar.Toolbar-method-add" rel="Ext.toolbar.Toolbar-method-add" class="docClass">add</a>
   */
  public function Toolbar(config:ext.toolbar.Toolbar = null) {
    super();
  }

  [ExtConfig]
  /**
   * This class assigns a default layout (<code>layout: 'hbox'</code> or <code>layout: 'vbox'</code> depending upon orientation).
   *
   * <p>Developers <em>may</em> override this configuration option if another layout is required.
   * See <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">Ext.container.Container.layout</a> for additional information.</p>
   * @see #setLayout()
   */
  override public native function get layout():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set layout(value:*):void;

  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  public native function get focusableContainerEl():*;

  /**
   * @private
   */
  public native function set focusableContainerEl(value:*):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Toolbar, or subclass thereof.
   * @default true
   */
  public native function get isToolbar():Boolean;

  /**
   * @private
   */
  public native function set isToolbar(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  /**
   * A default <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">ui</a> to use for <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a> items. This is a quick and simple
   * way to change the look of all child <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Buttons</a>.
   *
   * <p>If there is no value for defaultButtonUI, the button's <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">ui</a> value will get <code>-toolbar</code>
   * appended so the <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a> has a different look when it's a child of a <a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Toolbar</a>.
   * To prevent this and have the same look as buttons outside of a toolbar, you can provide a string value to the defaultButtonUI:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
   *     renderTo    : document.body,
   *     width       : 300,
   *     title       : 'Panel',
   *     html        : 'Some Body',
   *     dockedItems : [
   *         {
   *             xtype           : 'toolbar',
   *             dock            : 'top',
   *             defaultButtonUI : 'default',
   *             items           : [
   *                 {
   *                     text : 'Save'
   *                 },
   *                 {
   *                     text : 'Remove'
   *                 }
   *             ]
   *         }
   *     ]
   * });
   * </code></pre>
   * @default 'default-toolbar'
   */
  public native function get defaultButtonUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultButtonUI(value:String):void;

  [ExtConfig]
  /**
   * Default UI for form field items.
   * @default 'default'
   */
  public native function get defaultFieldUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultFieldUI(value:String):void;

  [ExtConfig]
  /**
   * Default UI for Buttons if the toolbar has a UI of 'footer'
   * @default 'default'
   */
  public native function get defaultFooterButtonUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultFooterButtonUI(value:String):void;

  [ExtConfig]
  /**
   * Default UI for Form Fields if the toolbar has a UI of 'footer'
   * @default 'default'
   */
  public native function get defaultFooterFieldUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultFooterFieldUI(value:String):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  public native function get enableFocusableContainer():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  [ExtConfig]
  /**
   * Configure true to make the toolbar provide a button which activates a dropdown Menu to show
   * items which overflow the Toolbar's width.  Setting this too true is the equivalent
   * of setting <code><a href="#!/api/Ext.toolbar.Toolbar-cfg-overflowHandler" rel="Ext.toolbar.Toolbar-cfg-overflowHandler" class="docClass">overflowHandler</a>:'menu'</code>.
   * @default false
   */
  public native function get enableOverflow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableOverflow(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * <ul>
   * <li><code>null</code> - hidden overflow</li>
   * <li><code>'scroller'</code> to render left/right scroller buttons on either side of the breadcrumb</li>
   * <li><code>'menu'</code> to render the overflowing buttons as items of an overflow menu.</li>
   * </ul>
   */
  public native function get overflowHandler():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overflowHandler(value:String):void;

  [ExtConfig]
  /**
   * Set to <code>true</code> to make the toolbar vertical. The layout will become a <code>vbox</code>.
   * @default false
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  /**
   * Adds element(s) to the toolbar -- this function takes a variable number of
   * arguments of mixed type and adds them to the toolbar.
   *
   * <p><strong>Note</strong>: See the notes within <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">Ext.container.Container.add</a>.</p>
   * @param args The following types of arguments are all valid:
   * <ul>
   * <li><code><a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">config</a></code>: A valid button config object</li>
   * <li><code>HTMLElement</code>: Any standard HTML element</li>
   * <li><code>Field</code>: Any form field</li>
   * <li><code>Item</code>: Any subclass of <a href="#!/api/Ext.toolbar.Item" rel="Ext.toolbar.Item" class="docClass">Ext.toolbar.Item</a></li>
   * <li><p><code>String</code>: Any generic string (gets wrapped in a <a href="#!/api/Ext.toolbar.TextItem" rel="Ext.toolbar.TextItem" class="docClass">Ext.toolbar.TextItem</a>).</p>
   * <p>Note that there are a few special strings that are treated differently as explained next:</p>
   * <ul>
   * <li><code>'-'</code>: Creates a separator element</li>
   * <li><code>' '</code>: Creates a spacer element</li>
   * <li><code>'-&gt;'</code>: Creates a fill element</li>
   * </ul>
   * </li>
   * </ul>
   * @return The Components that were added.
   */
  override public native function add(...args):*;
}
}