package ext.util {

[Uses("ext.util.Observable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.util.ClickRepeater", require)]
/**
 * Fires on a specified interval during the time the element is pressed.
 * @eventType ext.util.events.ClickRepeater_eEvent.CLICK
 */
[Event(name = 'onClick', type = 'ext.util.events.ClickRepeater_eEvent')]
/**
 * Fires when the mouse button is depressed.
 * @eventType ext.util.events.ClickRepeater_eEvent.MOUSE_DOWN
 */
[Event(name = 'onMouseDown', type = 'ext.util.events.ClickRepeater_eEvent')]
/**
 * Fires when the mouse key is released.
 * @eventType ext.util.events.ClickRepeater_eEvent.MOUSE_UP
 */
[Event(name = 'onMouseUp', type = 'ext.util.events.ClickRepeater_eEvent')]
/**
 * A wrapper class which can be applied to any element. Fires a "click" event while the
 * mouse is pressed. The interval between firings may be specified in the config but
 * defaults to 20 milliseconds.
 *
 * <p>Optionally, a CSS class may be applied to the element during the time it is pressed.</p>
 */
public class ClickRepeater extends ext.util.Observable {
  /**
   * Creates new ClickRepeater.
   * @param el The element or its ID to listen on
   * @param config Config object.
   */
  public function ClickRepeater(el:*, config:ext.util.ClickRepeater = null) {
    super();
  }

  [ExtConfig]
  /**
   * True if autorepeating should start slowly and accelerate.
   * "interval" and "delay" are ignored.
   */
  public native function get accelerate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set accelerate(value:Boolean):void;

  [ExtConfig]
  /**
   * The initial delay before the repeating event begins firing.
   * Similar to an autorepeat key delay.
   * @default 250
   */
  public native function get delay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set delay(value:Number):void;

  [ExtConfig]
  /**
   * The element to act as a button.
   */
  public native function get el():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set el(value:*):void;

  [ExtConfig]
  /**
   * The interval between firings of the "click" event (in milliseconds).
   * @default 20
   */
  public native function get interval():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set interval(value:Number):void;

  [ExtConfig]
  /**
   * A CSS class name to be applied to the element while pressed.
   */
  public native function get pressedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pressedCls(value:String):void;

  [ExtConfig]
  /**
   * True to prevent the default click event
   * @default true
   */
  public native function get preventDefault():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventDefault(value:Boolean):void;

  [ExtConfig]
  /**
   * True to stop the default click event
   * @default false
   */
  public native function get stopDefault():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stopDefault(value:Boolean):void;

  /**
   * Disables the repeater and stops events from firing.
   */
  public native function disable(force:Object):void;

  /**
   * Enables the repeater and allows events to fire.
   */
  public native function enable():void;

  /**
   * Convenience function for setting disabled/enabled by boolean.
   */
  public native function setDisabled(disabled:Boolean):void;
}
}