package ext.util {
import ext.Base;
import ext.mixin.IIdentifiable;

[Uses("ext.Base")]
[Uses("ext.mixin.Identifiable")]
[Native("Ext.util.CollectionKey", require)]
/**
 * Maintains an additional key map for an <code><a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a></code>. Instances of this class
 * are seldom created manually. Rather they are created by the <code><a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a>' when
 * given an</code>extraKeys` config.
 */
public class CollectionKey extends ext.Base implements ext.mixin.IIdentifiable {
  /**
   * @param config @inheritDoc
   */
  public function CollectionKey(config:ext.util.CollectionKey = null) {
    super();
  }

  /**
   * This property is set by <code><a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a></code> when added via <code>extraKeys</code>.
   */
  public native function get name():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function to retrieve the key of an item in the collection. This can be normal
   * function that takes an item and returns the key or it can be the name of the
   * method to call on an item to get the key.
   *
   * <p>For example:</p>
   *
   * <pre><code> new <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a>({
   *      keys: {
   *          byName: {
   *              keyFn: 'getName' // each item has a "getName" method
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>Or equivalently:</p>
   *
   * <pre><code> new <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a>({
   *      keys: {
   *          byName: {
   *              keyFn: function (item) {
   *                  return item.getName();
   *              }
   *          }
   *      }
   *  });
   * </code></pre>
   * @since 5.0.0
   * @see #getKeyFn()
   * @see #setKeyFn()
   */
  public native function get keyFn():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set keyFn(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the property on each item that is its key.
   *
   * <pre><code> new <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a>({
   *      keys: {
   *          byName: 'name'
   *      }
   *  });
   * </code></pre>
   *
   * <p>Or equivalently:</p>
   *
   * <pre><code> new <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a>({
   *      keys: {
   *          byName: {
   *              property: 'name'
   *          }
   *      }
   *  });
   *
   *  var item = collection.byName.get('fooname');
   * </code></pre>
   * @see #getProperty()
   * @see #setProperty()
   */
  public native function get property():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set property(value:String):void;

  [ExtConfig]
  /**
   * The name of the sub-object property on each item that is its key. This value
   * overrides <code><a href="#!/api/Ext.util.Collection-cfg-rootProperty" rel="Ext.util.Collection-cfg-rootProperty" class="docClass">Ext.util.Collection.rootProperty</a></code>.
   *
   * <pre><code> new <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a>({
   *      keys: {
   *          byName: {
   *              property: 'name',
   *              rootProperty: 'data'
   *          }
   *      }
   *  });
   *
   *  var item = collection.byName.get('fooname');
   * </code></pre>
   * @see #setRootProperty()
   */
  public native function get rootProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rootProperty(value:String):void;

  /**
   * Returns the item or, if not <code>unique</code> possibly array of items that have the given
   * key.
   * @param key The key that will match the <code>keyFn</code> return value or value of
   * the specified <code>property</code>.
   */
  public native function get(key:*):Object;

  /**
   * Retrieves the id of this component. Will autogenerate an id if one has not already been set.
   * @return id
   */
  public native function getId():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.CollectionKey-cfg-keyFn" rel="Ext.util.CollectionKey-cfg-keyFn" class="docClass">keyFn</a>.
   * @since 5.0.0
   */
  public native function getKeyFn():*;

  /**
   * Returns the value of <a href="#!/api/Ext.util.CollectionKey-cfg-property" rel="Ext.util.CollectionKey-cfg-property" class="docClass">property</a>.
   */
  public native function getProperty():String;

  /**
   * Returns the index of the item with the given key in the collection. If this is not
   * a <code>unique</code> result, the index of the first item in the collection with the matching
   * key.
   *
   * <p>To iterate the indices of all items with a matching (not <code>unique</code>) key:</p>
   *
   * <pre><code> for (index = collection.byName.indexOf('foo');
   *          index &gt;= 0;
   *          index = collection.byName.indexOf('foo', index)) {
   *      // process item at "index"
   *  }
   * </code></pre>
   * @param key The key that will match the <code>keyFn</code> return value or value of
   * the specified <code>property</code>.
   * @param startAt The index at which to start. Only occurrences beyond
   * this index are returned.
   * @default -1
   * @return The index of the first item with the given <code>key</code> beyond the given
   * <code>startAt</code> index or -1 if there are no such items.
   */
  public native function indexOf(key:*, startAt:Number = -1):Number;

  /**
   * Sets the value of <a href="#!/api/Ext.util.CollectionKey-cfg-keyFn" rel="Ext.util.CollectionKey-cfg-keyFn" class="docClass">keyFn</a>.
   * @since 5.0.0
   * @param keyFn The new value.
   */
  public native function setKeyFn(keyFn:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.CollectionKey-cfg-property" rel="Ext.util.CollectionKey-cfg-property" class="docClass">property</a>.
   * @param property The new value.
   */
  public native function setProperty(property:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.CollectionKey-cfg-rootProperty" rel="Ext.util.CollectionKey-cfg-rootProperty" class="docClass">rootProperty</a>.
   * @param rootProperty The new value.
   */
  public native function setRootProperty(rootProperty:String):void;

  /**
   * Change the key for an existing item in the collection. If the old key does not
   * exist this call does nothing.
   * @since 5.0.0
   * @param item The item whose key has changed.
   * @param oldKey The old key for the <code>item</code>.
   */
  public native function updateKey(item:Object, oldKey:String):void;
}
}