package ext.util {
import ext.dd.DragTracker;

[Uses("ext.dd.DragTracker")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.util.ComponentDragger", require)]
/**
 * A subclass of <a href="#!/api/Ext.dd.DragTracker" rel="Ext.dd.DragTracker" class="docClass">Ext.dd.DragTracker</a> which handles dragging any Component.
 *
 * <p>This is configured with a Component to be made draggable, and a config object for the <a href="#!/api/Ext.dd.DragTracker" rel="Ext.dd.DragTracker" class="docClass">Ext.dd.DragTracker</a>
 * class.</p>
 *
 * <p>A <a href="#!/api/Ext.util.ComponentDragger-cfg-delegate" rel="Ext.util.ComponentDragger-cfg-delegate" class="docClass">delegate</a> may be provided which may be either the element to use as the mousedown target or a
 * CSS selector to activate multiple mousedown targets.</p>
 *
 * <p>When the Component begins to be dragged, its <code>beginDrag</code> method will be called if implemented.</p>
 *
 * <p>When the drag ends, its <code>endDrag</code> method will be called if implemented.</p>
 */
public class ComponentDragger extends ext.dd.DragTracker {
  /**
   * Creates new ComponentDragger.
   * @param comp The Component to provide dragging for.
   * @param config Config object
   */
  public function ComponentDragger(comp:Object, config:ext.util.ComponentDragger = null) {
    super();
  }

  [ExtConfig]
  /**
   * A CSS selector which identifies child elements within the Component's encapsulating
   * Element which are the drag handles. This limits dragging to only begin when the matching elements are
   * mousedowned.
   *
   * <p>This may also be a specific child element within the Component's encapsulating element to use as the drag handle.</p>
   */
  override public native function get delegate():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set delegate(value:String):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to constrain the Component to within the bounds of the <a href="#!/api/Ext.util.ComponentDragger-cfg-constrainTo" rel="Ext.util.ComponentDragger-cfg-constrainTo" class="docClass">constrainTo</a> region.
   */
  public native function get constrain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrain(value:Boolean):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to constrain the drag handles within the <a href="#!/api/Ext.util.ComponentDragger-cfg-constrainTo" rel="Ext.util.ComponentDragger-cfg-constrainTo" class="docClass">constrainTo</a> region.
   */
  public native function get constrainDelegate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainDelegate(value:Boolean):void;
}
}