package ext.util {

[Uses("ext.util.SFormat")]
[Uses("ext.Base")]
[Native("Ext.util.Format", require)]
/**
 * This class is a centralized place for formatting functions. It includes
 * functions to format various different types of data, such as text, dates and numeric values.
 *
 * <h2>Localization</h2>
 *
 * <p>This class contains several options for localization. These can be set once the library has loaded,
 * all calls to the functions from that point will use the locale settings that were specified.</p>
 *
 * <p>Options include:</p>
 *
 * <ul>
 * <li>thousandSeparator</li>
 * <li>decimalSeparator</li>
 * <li>currenyPrecision</li>
 * <li>currencySign</li>
 * <li>currencyAtEnd</li>
 * </ul>
 *
 *
 * <p>This class also uses the default date format defined here: <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a>.</p>
 *
 * <h2>Using with renderers</h2>
 *
 * <p>There are two helper functions that return a new function that can be used in conjunction with
 * grid renderers:</p>
 *
 * <pre><code>columns: [{
 *     dataIndex: 'date',
 *     renderer: <a href="#!/api/Ext.util.Format-method-dateRenderer" rel="Ext.util.Format-method-dateRenderer" class="docClass">Ext.util.Format.dateRenderer</a>('Y-m-d')
 * }, {
 *     dataIndex: 'time',
 *     renderer: <a href="#!/api/Ext.util.Format-method-numberRenderer" rel="Ext.util.Format-method-numberRenderer" class="docClass">Ext.util.Format.numberRenderer</a>('0.000')
 * }]
 * </code></pre>
 *
 * <p>Functions that only take a single argument can also be passed directly:</p>
 *
 * <pre><code>columns: [{
 *     dataIndex: 'cost',
 *     renderer: <a href="#!/api/Ext.util.Format-method-usMoney" rel="Ext.util.Format-method-usMoney" class="docClass">Ext.util.Format.usMoney</a>
 * }, {
 *     dataIndex: 'productCode',
 *     renderer: <a href="#!/api/Ext.util.Format-method-uppercase" rel="Ext.util.Format-method-uppercase" class="docClass">Ext.util.Format.uppercase</a>
 * }]
 * </code></pre>
 *
 * <h2>Using with XTemplates</h2>
 *
 * <p>XTemplates can also directly use <a href="#!/api/Ext.util.Format" rel="Ext.util.Format" class="docClass">Ext.util.Format</a> functions:</p>
 *
 * <pre><code>new <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>([
 *     'Date: {startDate:date("Y-m-d")}',
 *     'Cost: {cost:usMoney}'
 * ]);
 * </code></pre>
 */
public const Format:ext.util.SFormat;
}