package ext.util {
import ext.data.Model;

[Uses("ext.util.Sorter")]
[Uses("ext.Base")]
[Native("Ext.util.Grouper", require)]
/**
 * Represents a grouping of items. The grouper works in a similar fashion as the
 * <code><a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a></code> except that groups must be able to extract a value by which all items
 * in the group can be collected. By default this is derived from the <code>property</code> config
 * but can be customized using the <code>groupFn</code> if necessary.
 *
 * <p>All items with the same group value compare as equal. If the group values do not compare
 * equally, the sort can be controlled further by setting <code>sortProperty</code> or <code>sorterFn</code>.</p>
 */
public class Grouper extends ext.util.Sorter {
  /**
   * @param config @inheritDoc
   */
  public function Grouper(config:ext.util.Grouper = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The field by which records are grouped. Groups are
   * sorted alphabetically by group value as the default. To sort groups by a different
   * property, use the <a href="#!/api/Ext.util.Grouper-cfg-sortProperty" rel="Ext.util.Grouper-cfg-sortProperty" class="docClass">sortProperty</a> configuration.
   * @see #getProperty()
   * @see #setProperty()
   */
  override public native function get property():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set property(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This function is called for each item in the collection
   * to determine the group to which it belongs. By default the <code>property</code> value is
   * used to group items.
   * <ul>
   *   <li><code>item</code> : Object
   * The current item from the collection.
   *   </li>
   *   <li><code>return</code> : String
   * The group identifier for the item.
   *   </li>
   * </ul>
   * @see #getGroupFn()
   * @see #setGroupFn()
   */
  public native function get groupFn():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set groupFn(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * You can set this configuration if you want the groups
   * to be sorted on something other then the group string returned by the <code>groupFn</code>.
   * This serves the same role as <code>property</code> on a normal <code><a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a></code>.
   * @see #getSortProperty()
   * @see #setSortProperty()
   */
  public native function get sortProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set sortProperty(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Grouper-cfg-groupFn" rel="Ext.util.Grouper-cfg-groupFn" class="docClass">groupFn</a>.
   * <ul>
   *   <li><code>item</code> : Object
   * The current item from the collection.
   *   </li>
   *   <li><code>return</code> : String
   * The group identifier for the item.
   *   </li>
   * </ul>
   */
  public native function getGroupFn():Function;

  /**
   * Returns the value for grouping to be used.
   * @param item The Model instance
   */
  public native function getGroupString(item:ext.data.Model):String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Grouper-cfg-sortProperty" rel="Ext.util.Grouper-cfg-sortProperty" class="docClass">sortProperty</a>.
   */
  public native function getSortProperty():String;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Grouper-cfg-groupFn" rel="Ext.util.Grouper-cfg-groupFn" class="docClass">groupFn</a>.
   * <ul>
   *   <li><code>item</code> : Object
   * The current item from the collection.
   *   </li>
   *   <li><code>return</code> : String
   * The group identifier for the item.
   *   </li>
   * </ul>
   * @param groupFn The new value.
   */
  public native function setGroupFn(groupFn:Function):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Grouper-cfg-sortProperty" rel="Ext.util.Grouper-cfg-sortProperty" class="docClass">sortProperty</a>.
   * @param sortProperty The new value.
   */
  public native function setSortProperty(sortProperty:String):void;
}
}