package ext.util {
import ext.data.AbstractStore;
import ext.data.Store;

[Uses("ext.util.StoreHolder")]
[Mixin("ext.util.StoreHolder")]
/**
 * This class is used as a mixin.
 *
 * <p>This class is to be used to provide basic methods for binding/unbinding stores to other
 * classes.</p>
 *
 * <p>This class is not intended for direct use but rather internally by those classes that
 * manage a Store.</p>
 * @see ext.util.StoreHolder
 */
public interface IStoreHolder {
  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  function get autoDestroyBoundStore():Boolean;

  function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * Binds a store to this instance.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Gets the current store instance.
   * @return The store, null if one does not exist.
   */
  function getStore():ext.data.AbstractStore;

  /**
   * Sets the store to the specified store.
   * @since 5.0.0
   */
  function setStore(store:Object):void;
}
}