package ext.util {

[Uses("ext.util.SInflector")]
[Uses("ext.Base")]
[Native("Ext.util.Inflector", require)]
/**
 * General purpose inflector class that <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">pluralizes</a>, <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">singularizes</a> and
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">ordinalizes</a> words. Sample usage:
 *
 * <pre><code>//turning singular words into plurals
 * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('word'); //'words'
 * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('person'); //'people'
 * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('sheep'); //'sheep'
 *
 * //turning plurals into singulars
 * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('words'); //'word'
 * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('people'); //'person'
 * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('sheep'); //'sheep'
 *
 * //ordinalizing numbers
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">Ext.util.Inflector.ordinalize</a>(11); //"11th"
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">Ext.util.Inflector.ordinalize</a>(21); //"21st"
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">Ext.util.Inflector.ordinalize</a>(1043); //"1043rd"
 * </code></pre>
 *
 * <h1>Customization</h1>
 *
 * <p>The Inflector comes with a default set of US English pluralization rules. These can be augmented with additional
 * rules if the default rules do not meet your application's requirements, or swapped out entirely for other languages.
 * Here is how we might add a rule that pluralizes "ox" to "oxen":</p>
 *
 * <pre><code><a href="#!/api/Ext.util.Inflector-method-plural" rel="Ext.util.Inflector-method-plural" class="docClass">Ext.util.Inflector.plural</a>(/^(ox)$/i, "$1en");
 * </code></pre>
 *
 * <p>Each rule consists of two items - a regular expression that matches one or more rules, and a replacement string. In
 * this case, the regular expression will only match the string "ox", and will replace that match with "oxen". Here's
 * how we could add the inverse rule:</p>
 *
 * <pre><code><a href="#!/api/Ext.util.Inflector-method-singular" rel="Ext.util.Inflector-method-singular" class="docClass">Ext.util.Inflector.singular</a>(/^(ox)en$/i, "$1");
 * </code></pre>
 *
 * <p>Note that the ox/oxen rules are present by default.</p>
 */
public const Inflector:ext.util.SInflector;
}