package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.ObjectTemplate", require)]
/**
 * This class accepts an object that serves as a template for creating new objects. Like
 * other templates (<code><a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a></code>) this creation step requires a context object to give
 * the template its values.
 *
 * <p>For example:</p>
 *
 * <pre><code> var tpl = new <a href="#!/api/Ext.util.ObjectTemplate" rel="Ext.util.ObjectTemplate" class="docClass">Ext.util.ObjectTemplate</a>({
 *      property: 'Hello {name}',
 *      data: {
 *          value: '{age}'
 *      }
 *  });
 *
 *  var obj = tpl.apply({
 *      name: 'Bill',
 *      age: 42
 *  });
 *
 *  // obj = {
 *  //     property: 'Hello Bill',
 *  //     data: {
 *  //         value: 42
 *  //     }
 *  // }
 * </code></pre>
 */
public class ObjectTemplate extends ext.Base {
  /**
   * Creates an <code>ObjectTemplate</code> given a config object or instance.
   * @since 5.0.0
   * @param template The template object.
   */
  public static native function create(template:*, options:Object = null):ext.util.ObjectTemplate;

  /**
   * Constructs the <code>ObjectTemplate</code>. The actual compilation of the object to a ready to
   * apply form happens on the first call to <code>apply</code>.
   * @since 5.0.0
   */
  public function ObjectTemplate(template:Object, options:Object = null) {
    super();
  }

  /**
   * Applies the given <code>context</code> object to this template and returns a new object with
   * the appropriate pieces replaced.
   * @since 5.0.0
   * @param context The data used to populate the template.
   */
  public native function apply(context:Object):Object;

  /**
   * Compiles the  given template into an <code>apply</code> method that is ready to run. This
   * method is used recursively to process object properties and array elements.
   * @since 5.0.0
   */
  public native function compile(template:*):Function;
}
}