package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * Utility class for setting/reading values from browser cookies.
 * Values can be written using the <a href="#!/api/Ext.util.Cookies-method-set" rel="Ext.util.Cookies-method-set" class="docClass">set</a> method.
 * Values can be read using the <a href="#!/api/Ext.util.Cookies-method-get" rel="Ext.util.Cookies-method-get" class="docClass">get</a> method.
 * A cookie can be invalidated on the client machine using the <a href="#!/api/Ext.util.Cookies-method-clear" rel="Ext.util.Cookies-method-clear" class="docClass">clear</a> method.
 * <p>Type of singleton Cookies.</p>
 * @see ext.util.#Cookies ext.util.Cookies
 */
public class SCookies extends ext.Base {
  /**
   * Removes a cookie with the provided name from the browser
   * if found by setting its expiration date to sometime in the past.
   * @param name The name of the cookie to remove
   * @param path The path for the cookie.
   * This must be included if you included a path while setting the cookie.
   */
  public native function clear(name:String, path:String = null):void;

  /**
   * Retrieves cookies that are accessible by the current page. If a cookie does not exist, <code>get()</code> returns null. The
   * following example retrieves the cookie called "valid" and stores the String value in the variable validStatus.
   *
   * <pre><code>var validStatus = <a href="#!/api/Ext.util.Cookies-method-get" rel="Ext.util.Cookies-method-get" class="docClass">Ext.util.Cookies.get</a>("valid");
   * </code></pre>
   * @param name The name of the cookie to get
   * @return Returns the cookie value for the specified name;
   * null if the cookie name does not exist.
   */
  public native function get(name:String):Object;

  /**
   * Creates a cookie with the specified name and value. Additional settings for the cookie may be optionally specified
   * (for example: expiration, access restriction, SSL).
   * @param name The name of the cookie to set.
   * @param value The value to set for the cookie.
   * @param expires Specify an expiration date the cookie is to persist until. Note that the specified Date
   * object will be converted to Greenwich Mean Time (GMT).
   * @param path Setting a path on the cookie restricts access to pages that match that path. Defaults to all
   * pages ('/').
   * @param domain Setting a domain restricts access to pages on a given domain (typically used to allow
   * cookie access across subdomains). For example, "sencha.com" will create a cookie that can be accessed from any
   * subdomain of sencha.com, including www.sencha.com, support.sencha.com, etc.
   * @param secure Specify true to indicate that the cookie should only be accessible via SSL on a page
   * using the HTTPS protocol. Defaults to false. Note that this will only work if the page calling this code uses the
   * HTTPS protocol, otherwise the cookie will be created with default options.
   */
  public native function set(name:String, value:Object, expires:Object = null, path:String = null, domain:String = null, secure:Boolean = false):void;
}
}