package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * This class is a centralized place for formatting functions. It includes
 * functions to format various different types of data, such as text, dates and numeric values.
 *
 * <h2>Localization</h2>
 *
 * <p>This class contains several options for localization. These can be set once the library has loaded,
 * all calls to the functions from that point will use the locale settings that were specified.</p>
 *
 * <p>Options include:</p>
 *
 * <ul>
 * <li>thousandSeparator</li>
 * <li>decimalSeparator</li>
 * <li>currenyPrecision</li>
 * <li>currencySign</li>
 * <li>currencyAtEnd</li>
 * </ul>
 *
 *
 * <p>This class also uses the default date format defined here: <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a>.</p>
 *
 * <h2>Using with renderers</h2>
 *
 * <p>There are two helper functions that return a new function that can be used in conjunction with
 * grid renderers:</p>
 *
 * <pre><code>columns: [{
 *     dataIndex: 'date',
 *     renderer: <a href="#!/api/Ext.util.Format-method-dateRenderer" rel="Ext.util.Format-method-dateRenderer" class="docClass">Ext.util.Format.dateRenderer</a>('Y-m-d')
 * }, {
 *     dataIndex: 'time',
 *     renderer: <a href="#!/api/Ext.util.Format-method-numberRenderer" rel="Ext.util.Format-method-numberRenderer" class="docClass">Ext.util.Format.numberRenderer</a>('0.000')
 * }]
 * </code></pre>
 *
 * <p>Functions that only take a single argument can also be passed directly:</p>
 *
 * <pre><code>columns: [{
 *     dataIndex: 'cost',
 *     renderer: <a href="#!/api/Ext.util.Format-method-usMoney" rel="Ext.util.Format-method-usMoney" class="docClass">Ext.util.Format.usMoney</a>
 * }, {
 *     dataIndex: 'productCode',
 *     renderer: <a href="#!/api/Ext.util.Format-method-uppercase" rel="Ext.util.Format-method-uppercase" class="docClass">Ext.util.Format.uppercase</a>
 * }]
 * </code></pre>
 *
 * <h2>Using with XTemplates</h2>
 *
 * <p>XTemplates can also directly use <a href="#!/api/Ext.util.Format" rel="Ext.util.Format" class="docClass">Ext.util.Format</a> functions:</p>
 *
 * <pre><code>new <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>([
 *     'Date: {startDate:date("Y-m-d")}',
 *     'Cost: {cost:usMoney}'
 * ]);
 * </code></pre>
 * <p>Type of singleton Format.</p>
 * @see ext.util.#Format ext.util.Format
 */
public class SFormat extends ext.Base {
  /**
   * This may be set to <code>true</code> to make the <a href="#!/api/Ext.util.Format-method-currency" rel="Ext.util.Format-method-currency" class="docClass">currency</a> function
   * append the currency sign to the formatted value.
   *
   * <p>This may be overridden in a locale file.</p>
   * @default false
   */
  public native function get currencyAtEnd():Boolean;

  /**
   * @private
   */
  public native function set currencyAtEnd(value:Boolean):void;

  /**
   * The number of decimal places that the <a href="#!/api/Ext.util.Format-method-currency" rel="Ext.util.Format-method-currency" class="docClass">currency</a> function displays.
   *
   * <p>This may be overridden in a locale file.</p>
   * @default 2
   */
  public native function get currencyPrecision():Number;

  /**
   * @private
   */
  public native function set currencyPrecision(value:Number):void;

  /**
   * The currency sign that the <a href="#!/api/Ext.util.Format-method-currency" rel="Ext.util.Format-method-currency" class="docClass">currency</a> function displays.
   *
   * <p>This may be overridden in a locale file.</p>
   * @default '$'
   */
  public native function get currencySign():String;

  /**
   * @private
   */
  public native function set currencySign(value:String):void;

  /**
   * True to add a space between the currency and the value
   *
   * <p>This may be overridden in a locale file.</p>
   * @default ''
   * @since 6.2.0
   */
  public native function get currencySpacer():String;

  /**
   * @private
   */
  public native function set currencySpacer(value:String):void;

  /**
   * The character that the <a href="#!/api/Ext.util.Format-method-number" rel="Ext.util.Format-method-number" class="docClass">number</a> function uses as a decimal point.
   *
   * <p>This may be overridden in a locale file.</p>
   * @default '.'
   */
  public native function get decimalSeparator():String;

  /**
   * @private
   */
  public native function set decimalSeparator(value:String):void;

  /**
   * The global default date format.
   * @default 'm/d/Y'
   */
  public native function get defaultDateFormat():String;

  /**
   * @private
   */
  public native function set defaultDateFormat(value:String):void;

  /**
   * The percent sign that the <a href="#!/api/Ext.util.Format-method-percent" rel="Ext.util.Format-method-percent" class="docClass">percent</a> function displays.
   *
   * <p>This may be overridden in a locale file.</p>
   * @default '%'
   */
  public native function get percentSign():String;

  /**
   * @private
   */
  public native function set percentSign(value:String):void;

  /**
   * The character that the <a href="#!/api/Ext.util.Format-method-number" rel="Ext.util.Format-method-number" class="docClass">number</a> function uses as a thousand separator.
   *
   * <p>This may be overridden in a locale file.</p>
   * @default ','
   */
  public native function get thousandSeparator():String;

  /**
   * @private
   */
  public native function set thousandSeparator(value:String):void;

  /**
   * Formats an object of name value properties as HTML element attribute values suitable for using when creating textual markup.
   * @param attributes An object containing the HTML attributes as properties eg: <code>{height:40, vAlign:'top'}</code>
   */
  public native function attributes(attributes:Object):void;

  /**
   * Alias for <a href="#!/api/Ext.String-method-capitalize" rel="Ext.String-method-capitalize" class="docClass">Ext.String.capitalize</a>.
   */
  public native function capitalize(string:String):String;

  /**
   * Format a number as a currency.
   * @param value The numeric value to format
   * @param sign The currency sign to use (defaults to <a href="#!/api/Ext.util.Format-property-currencySign" rel="Ext.util.Format-property-currencySign" class="docClass">currencySign</a>)
   * @param decimals The number of decimals to use for the currency
   * (defaults to <a href="#!/api/Ext.util.Format-property-currencyPrecision" rel="Ext.util.Format-property-currencyPrecision" class="docClass">currencyPrecision</a>)
   * @param end True if the currency sign should be at the end of the string
   * (defaults to <a href="#!/api/Ext.util.Format-property-currencyAtEnd" rel="Ext.util.Format-property-currencyAtEnd" class="docClass">currencyAtEnd</a>)
   * @param currencySpacer True to add a space between the currency and value
   * @return The formatted currency string
   */
  public native function currency(value:*, sign:String = null, decimals:Number = NaN, end:Boolean = false, currencySpacer:String = null):String;

  /**
   * Formats the passed date using the specified format pattern.
   * Note that this uses the native Javascript Date.parse() method and is therefore subject to its idiosyncrasies.
   * Most formats assume the local timezone unless specified. One notable exception is 'YYYY-MM-DD' (note the dashes)
   * which is typically interpreted in UTC and can cause date shifting.
   * @param value The value to format. Strings must conform to the format
   * expected by the JavaScript Date object's
   * <a href="http://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Date/parse">parse() method</a>.
   * @param format Any valid date format string. Defaults to <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a>.
   * @return The formatted date string.
   */
  public native function date(value:*, format:String = null):String;

  /**
   * Returns a date rendering function that can be reused to apply a date format multiple times efficiently.
   * @param format Any valid date format string. Defaults to <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a>.
   * @return The date formatting function
   */
  public native function dateRenderer(format:String):Function;

  /**
   * Checks a reference and converts it to the default value if it's empty.
   * @param value Reference to check
   * @param defaultValue The value to insert of it's undefined.
   * @default ""
   */
  public native function defaultValue(value:Object, defaultValue:String = ""):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-ellipsis" rel="Ext.String-method-ellipsis" class="docClass">Ext.String.ellipsis</a>.
   * @param value The string to truncate.
   * @param length The maximum length to allow before truncating.
   * @param word <code>true</code> to try to find a common word break.
   * @default false
   * @return The converted text.
   */
  public native function ellipsis(value:String, length:Number, word:Boolean = false):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-escape" rel="Ext.String-method-escape" class="docClass">Ext.String.escape</a>.
   * @param string The string to escape.
   * @return The escaped string.
   */
  public native function escape(string:String):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-escapeRegex" rel="Ext.String-method-escapeRegex" class="docClass">Ext.String.escapeRegex</a>.
   * @param string The string to escape.
   * @return The escaped string.
   */
  public native function escapeRegex(string:String):String;

  /**
   * Simple format for a file size (xxx bytes, xxx KB, xxx MB).
   * @param size The numeric value to format
   * @return The formatted file size
   */
  public native function fileSize(size:*):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a>.
   * @param string The tokenized string to be formatted.
   * @param values The values to replace tokens <code>{0}</code>, <code>{1}</code>, etc in order.
   * @return The formatted string.
   */
  public native function format(string:String, ...values):String;

  /**
   * Returns the given number as a base 16 string at least <code>digits</code> in length. If
   * the number is fewer digits, 0's are prepended as necessary. If <code>digits</code> is
   * negative, the absolute value is the <em>exact</em> number of digits to return. In this
   * case, if then number has more digits, only the least significant digits are
   * returned.
   *
   * <pre><code> expect(<a href="#!/api/Ext.util.Format-method-hex" rel="Ext.util.Format-method-hex" class="docClass">Ext.util.Format.hex</a>(0x12e4, 2)).toBe('12e4');
   *  expect(<a href="#!/api/Ext.util.Format-method-hex" rel="Ext.util.Format-method-hex" class="docClass">Ext.util.Format.hex</a>(0x12e4, -2)).toBe('e4');
   *  expect(<a href="#!/api/Ext.util.Format-method-hex" rel="Ext.util.Format-method-hex" class="docClass">Ext.util.Format.hex</a>(0x0e, 2)).toBe('0e');
   * </code></pre>
   * @param value The number to format in hex.
   */
  public native function hex(value:Number, digits:Number):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-htmlDecode" rel="Ext.String-method-htmlDecode" class="docClass">Ext.String.htmlDecode</a>.
   * @param value The string to decode.
   * @return The decoded text.
   */
  public native function htmlDecode(value:String):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-htmlEncode" rel="Ext.String-method-htmlEncode" class="docClass">Ext.String.htmlEncode</a>.
   * @param value The string to encode.
   * @return The encoded text.
   */
  public native function htmlEncode(value:String):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-leftPad" rel="Ext.String-method-leftPad" class="docClass">Ext.String.leftPad</a>.
   * @param string The original string.
   * @param size The total length of the output string.
   * @param character The character with which to pad the original string.
   * @default ' '
   * @return The padded string.
   */
  public native function leftPad(string:String, size:Number, character:String = ' '):String;

  /**
   * Compares <code>value</code> against <code>threshold</code> and returns:
   *
   * <ul>
   * <li>if <code>value</code> &lt; <code>threshold</code> then it returns <code>below</code></li>
   * <li>if <code>value</code> > <code>threshold</code> then it returns <code>above</code></li>
   * <li>if <code>value</code> = <code>threshold</code> then it returns <code>equal</code> or <code>above</code> when <code>equal</code> is missing</li>
   * </ul>
   *
   *
   * <p>The usefulness of this formatter method is in templates. For example:</p>
   *
   * <pre><code> {foo:lessThanElse(0, 'negative', 'positive')}
   *
   *  {bar:lessThanElse(200, 'lessThan200', 'greaterThan200', 'equalTo200')}
   * </code></pre>
   * @param value Value that will be checked
   * @param threshold Value to compare against
   * @param below Value to return when <code>value</code> &lt; <code>threshold</code>
   * @param above Value to return when <code>value</code> > <code>threshold</code>. If <code>value</code> = <code>threshold</code> and
   * <code>equal</code> is missing then <code>above</code> is returned.
   * @param equal Value to return when <code>value</code> = <code>threshold</code>
   */
  public native function lessThanElse(value:Number, threshold:Number, below:*, above:*, equal:*):*;

  /**
   * Converts a string to all lower case letters.
   * @param value The text to convert
   * @return The converted text
   */
  public native function lowercase(value:String):String;

  /**
   * It does simple math for use in a template, for example:
   *
   * <pre><code>var tpl = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>('{value} * 10 = {value:math("* 10")}');
   * </code></pre>
   * @return A function that operates on the passed value.
   */
  public native function math():Function;

  /**
   * Returns a non-breaking space ("NBSP") for any "blank" value.
   * @since 6.2.0
   * @param strict Pass <code>false</code> to convert all falsey values to an
   * NBSP. By default, only '', <code>null</code> and <code>undefined</code> will be converted.
   * @default true
   */
  public native function nbsp(value:*, strict:Boolean = true):*;

  /**
   * Converts newline characters to the HTML tag <code>&lt;br/&gt;</code>
   * @param v The string value to format.
   * @return The string with embedded <code>&lt;br/&gt;</code> tags in place of newlines.
   */
  public native function nl2br(v:String):String;

  /**
   * Formats the passed number according to the passed format string.
   *
   * <p>The number of digits after the decimal separator character specifies the number of
   * decimal places in the resulting string. The <em>local-specific</em> decimal character is
   * used in the result.</p>
   *
   * <p>The <em>presence</em> of a thousand separator character in the format string specifies that
   * the <em>locale-specific</em> thousand separator (if any) is inserted separating thousand groups.</p>
   *
   * <p>By default, "," is expected as the thousand separator, and "." is expected as the decimal separator.</p>
   *
   * <p>Locale-specific characters are always used in the formatted output when inserting
   * thousand and decimal separators. These can be set using the <a href="#!/api/Ext.util.Format-property-thousandSeparator" rel="Ext.util.Format-property-thousandSeparator" class="docClass">thousandSeparator</a> and
   * <a href="#!/api/Ext.util.Format-property-decimalSeparator" rel="Ext.util.Format-property-decimalSeparator" class="docClass">decimalSeparator</a> options.</p>
   *
   * <p>The format string must specify separator characters according to US/UK conventions ("," as the
   * thousand separator, and "." as the decimal separator)</p>
   *
   * <p>To allow specification of format strings according to local conventions for separator characters, add
   * the string <code>/i</code> to the end of the format string. This format depends on the <a href="#!/api/Ext.util.Format-property-thousandSeparator" rel="Ext.util.Format-property-thousandSeparator" class="docClass">thousandSeparator</a> and
   * <a href="#!/api/Ext.util.Format-property-decimalSeparator" rel="Ext.util.Format-property-decimalSeparator" class="docClass">decimalSeparator</a> options. For example, if using European style separators, then the format string
   * can be specified as <code>'0.000,00'</code>. This would be equivalent to using <code>'0,000.00'</code> when using US style formatting.</p>
   *
   * <p>Examples (123456.789):</p>
   *
   * <ul>
   * <li><code>0</code> - (123457) show only digits, no precision</li>
   * <li><code>0.00</code> - (123456.79) show only digits, 2 precision</li>
   * <li><code>0.0000</code> - (123456.7890) show only digits, 4 precision</li>
   * <li><code>0,000</code> - (123,457) show comma and digits, no precision</li>
   * <li><code>0,000.00</code> - (123,456.79) show comma and digits, 2 precision</li>
   * <li><code>0,0.00</code> - (123,456.79) shortcut method, show comma and digits, 2 precision</li>
   * <li><code>0.####</code> - (123,456.789) Allow maximum 4 decimal places, but do not right pad with zeroes</li>
   * <li><code>0.00##</code> - (123456.789) Show at least 2 decimal places, maximum 4, but do not right pad with zeroes</li>
   * </ul>
   * @param v The number to format.
   * @param formatString The way you would like to format this text.
   * @return The formatted number.
   */
  public native function number(v:Number, formatString:String):String;

  /**
   * Returns a number rendering function that can be reused to apply a number format multiple
   * times efficiently.
   * @param format Any valid number format string for <a href="#!/api/Ext.util.Format-method-number" rel="Ext.util.Format-method-number" class="docClass">number</a>
   * @return The number formatting function
   */
  public native function numberRenderer(format:String):Function;

  /**
   * Returns this result:
   *
   * <pre><code> value || orValue
   * </code></pre>
   *
   * <p>The usefulness of this formatter method is in templates. For example:</p>
   *
   * <pre><code> {foo:or("bar")}
   * </code></pre>
   * @param value The "if" value.
   */
  public native function or(value:Boolean, orValue:*):void;

  /**
   * Parses a number or string representing margin sizes into an object.
   * Supports CSS-style margin declarations (e.g. 10, "10", "10 10", "10 10 10" and
   * "10 10 10 10" are all valid options and would return the same result).
   * @param box The encoded margins
   * @return An object with margin sizes for top, right, bottom and left
   */
  public native function parseBox(box:*):Object;

  /**
   * Formats the passed number as a percentage according to the passed format string.
   * The number should be between 0 and 1 to represent 0% to 100%.
   * @param value The percentage to format.
   * @param formatString See <a href="#!/api/Ext.util.Format-method-number" rel="Ext.util.Format-method-number" class="docClass">number</a> for details.
   * @default "0"
   * @return The formatted percentage.
   */
  public native function percent(value:Number, formatString:String = "0"):String;

  /**
   * If <code>value</code> is a number, returns the argument from that index. For example
   *
   * <pre><code> var s = <a href="#!/api/Ext.util.Format-method-pick" rel="Ext.util.Format-method-pick" class="docClass">Ext.util.Format.pick</a>(2, 'zero', 'one', 'two');
   *  // s === 'two'
   * </code></pre>
   *
   * <p>Otherwise, <code>value</code> is treated in a truthy/falsey manner like so:</p>
   *
   * <pre><code> var s = <a href="#!/api/Ext.util.Format-method-pick" rel="Ext.util.Format-method-pick" class="docClass">Ext.util.Format.pick</a>(null, 'first', 'second');
   *  // s === 'first'
   *
   *  s = <a href="#!/api/Ext.util.Format-method-pick" rel="Ext.util.Format-method-pick" class="docClass">Ext.util.Format.pick</a>({}, 'first', 'second');
   *  // s === 'second'
   * </code></pre>
   *
   * <p>The usefulness of this formatter method is in templates. For example:</p>
   *
   * <pre><code> {foo:pick("F","T")}
   *
   *  {bar:pick("first","second","third")}
   * </code></pre>
   * @param value The "if" value.
   */
  public native function pick(value:Boolean, firstValue:*, secondValue:*):void;

  /**
   * Selectively return the plural form of a word based on a numeric value.
   *
   * <p>For example, the following template would result in "1 Comment".  If the
   * value of <code>count</code> was 0 or greater than 1, the result would be "x Comments".</p>
   *
   * <pre><code>var tpl = new <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>('{count:plural("Comment")}');
   *
   * tpl.apply({
   *     count: 1
   * }); // returns "1 Comment"
   * </code></pre>
   *
   * <p>Examples using the static <code>plural</code> method call:</p>
   *
   * <pre><code><a href="#!/api/Ext.util.Format-method-plural" rel="Ext.util.Format-method-plural" class="docClass">Ext.util.Format.plural</a>(2, 'Comment');
   * // returns "2 Comments"
   *
   * <a href="#!/api/Ext.util.Format-method-plural" rel="Ext.util.Format-method-plural" class="docClass">Ext.util.Format.plural</a>(4, 'person', 'people');
   * // returns "4 people"
   * </code></pre>
   * @param value The value to compare against
   * @param singular The singular form of the word
   * @param plural The plural form of the word (defaults to the
   * singular form with an "s" appended)
   * @return output The pluralized output of the passed singular form
   */
  public native function plural(value:Number, singular:String, plural:String = null):String;

  /**
   * Rounds the passed number to the required decimal precision.
   * @param value The numeric value to round.
   * @param precision The number of decimal places to which to round the
   * first parameter's value. If <code>undefined</code> the <code>value</code> is passed to <code>Math.round</code>
   * otherwise the value is returned unmodified.
   * @return The rounded value.
   */
  public native function round(value:*, precision:Number = NaN):Number;

  /**
   * Checks if <code>value</code> is a positive or negative number and returns the proper param.
   *
   * <p>The usefulness of this formatter method is in templates. For example:</p>
   *
   * <pre><code> {foo:sign("clsNegative","clsPositive")}
   * </code></pre>
   */
  public native function sign(value:Number, negative:*, positive:*, zero:*):*;

  /**
   * Strips all script tags.
   * @param value The text from which to strip script tags
   * @return The stripped text
   */
  public native function stripScripts(value:Object):String;

  /**
   * Strips all HTML tags.
   * @param value The text from which to strip tags
   * @return The stripped text
   */
  public native function stripTags(value:Object):String;

  /**
   * Returns a substring from within an original string.
   * @param value The original text
   * @param start The start index of the substring
   * @param length The length of the substring
   * @return The substring
   */
  public native function substr(value:String, start:Number, length:Number):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-toggle" rel="Ext.String-method-toggle" class="docClass">Ext.String.toggle</a>.
   * @param string The current string.
   * @param value The value to compare to the current string.
   * @param other The new value to use if the string already equals the first value passed in.
   * @return The new value.
   */
  public native function toggle(string:String, value:String, other:String):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-trim" rel="Ext.String-method-trim" class="docClass">Ext.String.trim</a>.
   * @param string The string to trim.
   * @return The trimmed string.
   */
  public native function trim(string:String):String;

  /**
   * Alias for <a href="#!/api/Ext.String-method-uncapitalize" rel="Ext.String-method-uncapitalize" class="docClass">Ext.String.uncapitalize</a>.
   */
  public native function uncapitalize(string:String):String;

  /**
   * Checks a reference and converts it to empty string if it is undefined.
   * @param value Reference to check
   * @return Empty string if converted, otherwise the original value
   */
  public native function undef(value:Object):Object;

  /**
   * Converts a string to all upper case letters.
   * @param value The text to convert
   * @return The converted text
   */
  public native function uppercase(value:String):String;

  /**
   * Formats the given value using <code>encodeURI</code>.
   * @since 6.2.0
   * @param value The value to encode.
   */
  public native function uri(value:String):String;

  /**
   * Formats the given value using <code>encodeURIComponent</code>.
   * @since 6.2.0
   * @param value The value to encode.
   */
  public native function uriCmp(value:String):String;

  /**
   * Format a number as US currency.
   * @param value The numeric value to format
   * @return The formatted currency string
   */
  public native function usMoney(value:*):String;

  /**
   * Returns the word at the given <code>index</code>. Spaces and punctuation are considered
   * as word separators by default. For example:
   *
   * <pre><code> console.log(<a href="#!/api/Ext.util.Format-method-word" rel="Ext.util.Format-method-word" class="docClass">Ext.util.Format.word</a>('Hello, my name is Bob.', 2);
   *  // == 'name'
   * </code></pre>
   * @param value The sentence to break into words.
   * @param index The 0-based word index.
   * @param sep +"} The pattern by which to separate words.
   * @default "[\W\s]+"
   * @return The requested word or empty string.
   */
  public native function word(value:String, index:Number, sep:* = "[\\W\\s]+"):String;
}
}