package ext.view {
import ext.Component;
import ext.mixin.IQueryable;
import ext.toolbar.PagingToolbar;

[Uses("ext.view.DataView")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.view.AbstractView")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.StoreHolder")]
[Native("Ext.view.BoundList", require)]
/**
 * An internally used DataView for <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">ComboBox</a>.
 */
public class BoundListView extends ext.view.DataView implements ext.mixin.IQueryable {
  /**
   * @param config @inheritDoc
   */
  public function BoundListView(config:ext.view.BoundListView = null) {
    super();
  }

  [ExtConfig]
  /**
   * A String or <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a> instance to apply to inner template.
   *
   * <p><a href="#!/api/Ext.view.BoundList" rel="Ext.view.BoundList" class="docClass">Ext.view.BoundList</a> is used for the dropdown list of
   * <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>. To customize the template you can set the tpl on
   * the combobox config object:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>', {
   *     fieldLabel   : 'State',
   *     queryMode    : 'local',
   *     displayField : 'text',
   *     valueField   : 'abbr',
   *     store        : <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('StateStore', {
   *         fields : ['abbr', 'text'],
   *         data   : [
   *             {"abbr":"AL", "name":"Alabama"},
   *             {"abbr":"AK", "name":"Alaska"},
   *             {"abbr":"AZ", "name":"Arizona"}
   *             //...
   *         ]
   *     }),
   *     // Template for the dropdown menu.
   *     // Note the use of the "x-list-plain" and "x-boundlist-item" class,
   *     // this is required to make the items selectable.
   *     tpl: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>',
   *         '&lt;ul class="x-list-plain"&gt;&lt;tpl for="."&gt;',
   *             '&lt;li role="option" class="x-boundlist-item"&gt;{abbr} - {name}&lt;/li&gt;',
   *         '&lt;/tpl&gt;&lt;/ul&gt;'
   *     ),
   * });
   * </code></pre>
   *
   * <p>Defaults to:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>',
   *     '&lt;ul&gt;&lt;tpl for="."&gt;',
   *         '&lt;li role="option" class="' + itemCls + '"&gt;' + me.getInnerTpl(me.displayField) + '&lt;/li&gt;',
   *     '&lt;/tpl&gt;&lt;/ul&gt;'
   * );
   * </code></pre>
   */
  override public native function get tpl():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tpl(value:*):void;

  /**
   * A reference to the PagingToolbar instance in this view. Only populated if <a href="#!/api/Ext.view.BoundList-cfg-pageSize" rel="Ext.view.BoundList-cfg-pageSize" class="docClass">pageSize</a> is greater
   * than zero and the BoundList has been rendered.
   */
  public native function get pagingToolbar():ext.toolbar.PagingToolbar;

  /**
   * @private
   */
  public native function set pagingToolbar(value:ext.toolbar.PagingToolbar):void;

  [ExtConfig]
  /**
   * The field from the store to show in the view.
   * @default ""
   * @see #setDisplayField()
   */
  public native function get displayField():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set displayField(value:String):void;

  [ExtConfig]
  /**
   * If greater than <code>0</code>, a <a href="#!/api/Ext.toolbar.Paging" rel="Ext.toolbar.Paging" class="docClass">Ext.toolbar.Paging</a> is displayed at the bottom of the list and store
   * queries will execute with page <a href="#!/api/Ext.data.operation.Read-cfg-start" rel="Ext.data.operation.Read-cfg-start" class="docClass">start</a> and
   * <a href="#!/api/Ext.data.operation.Read-cfg-limit" rel="Ext.data.operation.Read-cfg-limit" class="docClass">limit</a> parameters.
   * @default 0
   */
  public native function get pageSize():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pageSize(value:Number):void;

  /**
   * Retrieves the first direct child of this container which matches the passed selector or component.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector. If no selector is
   * specified, the first child will be returned.
   * @return The matching child <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function child(selector:* = undefined):ext.Component;

  /**
   * Retrieves the first descendant of this container which matches the passed selector.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. If no selector is
   * specified, the first child will be returned.
   * @return The matching descendant <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function down(selector:* = undefined):ext.Component;

  /**
   * A method that returns the inner template for displaying items in the list.
   * This method is useful to override when using a more complex display value, for example
   * inserting an icon along with the text.
   *
   * <p>The XTemplate is created with a reference to the owning form field in the <code>field</code> property to provide access
   * to context. For example to highlight the currently typed value, the getInnerTpl may be configured into a
   * ComboBox as part of the <a href="#!/api/Ext.form.field.ComboBox-cfg-listConfig" rel="Ext.form.field.ComboBox-cfg-listConfig" class="docClass">listConfig</a>:</p>
   *
   * <p>   listConfig: {
   *        getInnerTpl: function() {
   *            return '{[values.name.replace(this.field.getRawValue(), "<b>" + this.field.getRawValue() + "</b>")]}';
   *        }
   *    }</p>
   * @param displayField The <a href="#!/api/Ext.view.BoundList-cfg-displayField" rel="Ext.view.BoundList-cfg-displayField" class="docClass">displayField</a> for the BoundList.
   * @return The inner template
   */
  public native function getInnerTpl(displayField:String):String;

  /**
   * Retrieves all descendant components which match the passed selector.
   * Executes an <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a> using this container as its root.
   * @param selector Selector complying to an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector.
   * If no selector is specified all items will be returned.
   * @return Components which matched the selector
   */
  public native function query(selector:String = null):Array;

  /**
   * Retrieves all descendant components which match the passed function.
   * The function should return false for components that are to be
   * excluded from the selection.
   * @param fn The matcher function. It will be called with a single argument,
   * the component being tested.
   * @param scope The scope in which to run the function. If not specified,
   * it will default to the active component.
   * @return Components matched by the passed function
   */
  public native function queryBy(fn:Function, scope:Object = null):Array;

  /**
   * Finds a component at any level under this container matching the id/itemId.
   * This is a shorthand for calling ct.down('#' + id);
   * @param id The id to find
   * @return The matching id, null if not found
   */
  public native function queryById(id:String):ext.Component;

  /**
   * Updates the display field for this view. This will automatically trigger
   * an regeneration of the tpl so that the updated displayField can be used
   */
  public native function setDisplayField(displayField:String):void;

  /**
   * Traverses the tree rooted at this node in post-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>after</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPostOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Traverses the tree rooted at this node in pre-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>before</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPreOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;
}
}