package ext.view {
import ext.data.AbstractStore;
import ext.data.Store;
import ext.dom.Element;
import ext.mixin.IFactoryable;
import ext.mixin.Observable;
import ext.util.IStoreHolder;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Native("Ext.view.NavigationModel", require)]
[ExcludeClass]
/**
 * Fired when a key has been used to navigate around the view.
 * @eventType ext.view.events.NavigationModelView_event_keyEventEvent.NAVIGATE
 */
[Event(name = 'onNavigate', type = 'ext.view.events.NavigationModelView_event_keyEventEvent')]
/**
 * This class listens for key events fired from a <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">DataView</a>, and moves the currently focused item
 * by adding the class <a href="#!/api/Ext.view.NavigationModel-property-focusCls" rel="Ext.view.NavigationModel-property-focusCls" class="docClass">focusCls</a>.
 */
public class NavigationModelView extends ext.mixin.Observable implements ext.mixin.IFactoryable, ext.util.IStoreHolder {
  /**
   * @param config @inheritDoc
   */
  public function NavigationModelView(config:ext.view.NavigationModelView = null) {
    super();
  }

  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @private
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * If this property is specified by the target class of this mixin its properties are
   * used to configure the created <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>.
   */
  public native function get factoryConfig():Object;

  /**
   * @private
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Binds a store to this instance.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * @inheritDoc
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * @inheritDoc
   */
  public native function setStore(store:Object):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Called to focus an item in the client <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">DataView</a>.
   * The default implementation adds the <a href="#!/api/Ext.view.NavigationModel-property-focusCls" rel="Ext.view.NavigationModel-property-focusCls" class="docClass">focusCls</a> to the passed item focuses it.
   * Subclasses may choose to keep focus in another target.
   *
   * <p>For example <a href="#!/api/Ext.view.BoundListKeyNav" rel="Ext.view.BoundListKeyNav" class="docClass">Ext.view.BoundListKeyNav</a> maintains focus in the input field.</p>
   */
  protected native function focusItem(item:ext.dom.Element):void;

  /**
   * Gets the listeners to bind to a new store.
   * @param store The Store which is being bound to for which a listeners object should be returned.
   * @return The listeners to be bound to the store in object literal form. The scope
   * may be omitted, it is assumed to be the current instance.
   */
  protected native function getStoreListeners(store:ext.data.Store):Object;

  /**
   * Template method, it is called when a new store is bound
   * to the current instance.
   * @param store The store being bound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onBindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Template method, it is called when an existing store is unbound
   * from the current instance.
   * @param store The store being unbound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onUnbindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}