package ext.window {

[Uses("ext.window.Window")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.window.MessageBox", require)]
/**
 * Utility class for generating different styles of message boxes.  The singleton instance, <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.MessageBox</a>
 * alias <code><a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a></code> can also be used.
 *
 * <p>Note that a MessageBox is asynchronous.  Unlike a regular JavaScript <code>alert</code> (which will halt
 * browser execution), showing a MessageBox will not cause the code to stop.  For this reason, if you have code
 * that should only run <em>after</em> some user feedback from the MessageBox, you must use a callback function
 * (see the <code>function</code> parameter for <a href="#!/api/Ext.window.MessageBox-method-show" rel="Ext.window.MessageBox-method-show" class="docClass">show</a> for more details).</p>
 *
 * <p>Basic alert</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Status', 'Changes saved successfully.');
 * </code></pre>
 *
 * <p>Prompt for user data and process the result using a callback</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.MessageBox-method-prompt" rel="Ext.MessageBox-method-prompt" class="docClass">Ext.Msg.prompt</a>('Name', 'Please enter your name:', function(btn, text){
 *     if (btn == 'ok'){
 *         // process text value and close...
 *     }
 * });
 * </code></pre>
 *
 * <p>Show a dialog using config options</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.MessageBox-event-show" rel="Ext.MessageBox-event-show" class="docClass">Ext.Msg.show</a>({
 *     title:'Save Changes?',
 *     message: 'You are closing a tab that has unsaved changes. Would you like to save your changes?',
 *     buttons: <a href="#!/api/Ext.MessageBox-property-YESNOCANCEL" rel="Ext.MessageBox-property-YESNOCANCEL" class="docClass">Ext.Msg.YESNOCANCEL</a>,
 *     icon: <a href="#!/api/Ext.MessageBox-property-QUESTION" rel="Ext.MessageBox-property-QUESTION" class="docClass">Ext.Msg.QUESTION</a>,
 *     fn: function(btn) {
 *         if (btn === 'yes') {
 *             console.log('Yes pressed');
 *         } else if (btn === 'no') {
 *             console.log('No pressed');
 *         } else {
 *             console.log('Cancel pressed');
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <p>Showing <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a> while it's already shown will cause the visible instance to be
 * overwritten with the newly passed config.  While this may be the desired outcome, you
 * can also create a new MessageBox that can exist alongside the <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a>
 * singleton instance.</p>
 *
 * <pre class='inline-example '><code>var myMsg = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.MessageBox" rel="Ext.window.MessageBox" class="docClass">Ext.window.MessageBox</a>', {
 *     // set closeAction to 'destroy' if this instance is not
 *     // intended to be reused by the application
 *     closeAction: 'destroy'
 * }).show({
 *     title: 'Custom MessageBox Instance',
 *     message: 'I can exist along with <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a>'
 * });
 *
 * <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Overlapping', '<a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.Msg</a> instance');
 * </code></pre>
 */
public class MessageBoxWindow extends ext.window.Window {
  /**
   * Button config that displays a single Cancel button
   * @default 8
   */
  public static native function get CANCEL():Number;

  /**
   * The CSS class that provides the ERROR icon image
   * @default Ext.baseCSSPrefix + 'message-box-error'
   */
  public static native function get ERROR():String;

  /**
   * The CSS class that provides the INFO icon image
   * @default Ext.baseCSSPrefix + 'message-box-info'
   */
  public static native function get INFO():String;

  /**
   * Button config that displays a single No button
   * @default 4
   */
  public static native function get NO():Number;

  /**
   * Button config that displays a single OK button
   * @default 1
   */
  public static native function get OK():Number;

  /**
   * Button config that displays OK and Cancel buttons
   * @default 9
   */
  public static native function get OKCANCEL():Number;

  /**
   * The CSS class that provides the QUESTION icon image
   * @default Ext.baseCSSPrefix + 'message-box-question'
   */
  public static native function get QUESTION():String;

  /**
   * The CSS class that provides the WARNING icon image
   * @default Ext.baseCSSPrefix + 'message-box-warning'
   */
  public static native function get WARNING():String;

  /**
   * Button config that displays a single Yes button
   * @default 2
   */
  public static native function get YES():Number;

  /**
   * Button config that displays Yes and No buttons
   * @default 6
   */
  public static native function get YESNO():Number;

  /**
   * Button config that displays Yes, No and Cancel buttons
   * @default 14
   */
  public static native function get YESNOCANCEL():Number;

  /**
   * @param config @inheritDoc
   */
  public function MessageBoxWindow(config:ext.window.MessageBoxWindow = null) {
    super();
  }

  /**
   * An object containing the default button text strings that can be overriden for localized language support.
   * Supported properties are: ok, cancel, yes and no.  Generally you should include a locale-specific
   * resource file for handling language support across the framework.
   * Customize the default text like so:
   *
   * <pre><code>Ext.window.MessageBox.buttonText.yes = "oui"; //french
   * </code></pre>
   * @default {ok: 'OK', yes: 'Yes', no: 'No', cancel: 'Cancel'}
   */
  public native function get buttonText():Object;

  /**
   * @private
   */
  public native function set buttonText(value:Object):void;

  /**
   * The default height in pixels of the message box's multiline textarea if displayed.
   * @default 75
   */
  public native function get defaultTextHeight():Number;

  /**
   * @private
   */
  public native function set defaultTextHeight(value:Number):void;

  /**
   * The minimum width in pixels of the message box if it is a progress-style dialog.  This is useful
   * for setting a different minimum width than text-only dialogs may need.
   * @default 250
   */
  public native function get minProgressWidth():Number;

  /**
   * @private
   */
  public native function set minProgressWidth(value:Number):void;

  /**
   * The minimum width in pixels of the message box if it is a prompt dialog.  This is useful
   * for setting a different minimum width than text-only dialogs may need.
   * @default 250
   */
  public native function get minPromptWidth():Number;

  /**
   * @private
   */
  public native function set minPromptWidth(value:Number):void;

  /**
   * Displays a new message box, or reinitializes an existing message box, based on the config options passed in. All
   * display functions (e.g. prompt, alert, etc.) on MessageBox call this function internally, although those calls
   * are basic shortcuts and do not support all of the config options allowed here.
   *
   * <p>Example usage:</p>
   *
   * <pre><code><a href="#!/api/Ext.MessageBox-event-show" rel="Ext.MessageBox-event-show" class="docClass">Ext.Msg.show</a>({
   *     title: 'Address',
   *     message: 'Please enter your address:',
   *     width: 300,
   *     buttons: <a href="#!/api/Ext.MessageBox-property-OKCANCEL" rel="Ext.MessageBox-property-OKCANCEL" class="docClass">Ext.Msg.OKCANCEL</a>,
   *     multiline: true,
   *     fn: saveAddress,
   *     animateTarget: 'addAddressBtn',
   *     icon: <a href="#!/api/Ext.window.MessageBox-property-INFO" rel="Ext.window.MessageBox-property-INFO" class="docClass">Ext.window.MessageBox.INFO</a>
   * });
   * </code></pre>
   * @param animateTarget <strong>only valid for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s or <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">ToolTip</a>s, or regular Components which have been configured
   * with <code>floating: true</code>.</strong> The target from which the Component should animate from while opening.
   * @default null
   * @param callback A callback function to call after the Component is displayed.
   * Only necessary if animation was specified.
   * @param scope The scope (<code>this</code> reference) in which the callback is executed.
   * Defaults to this Component.
   */
  override public native function show(animateTarget:* = null, callback:Function = null, scope:Object = null):void;

  /**
   * Displays a standard read-only message box with an OK button (comparable to the basic JavaScript alert prompt).
   * If a callback function is passed it will be called after the user clicks the button, and the
   * id of the button that was clicked will be passed as the only parameter to the callback
   * (could also be the top-right close button, which will always report as "cancel").
   * @param title The title bar text
   * @param message The message box body text
   * @param fn The callback function invoked after the message box is closed.
   * See <a href="#!/api/Ext.window.MessageBox-method-show" rel="Ext.window.MessageBox-method-show" class="docClass">show</a> method for details.
   * @param scope (Default window) The scope (<code>this</code> reference) in which the callback is executed.
   * @default window
   * @return this
   */
  public native function alert(title:String, message:String, fn:Function = null, scope:Object = null):ext.window.MessageBoxWindow;

  /**
   * Displays a confirmation message box with Yes and No buttons (comparable to JavaScript's confirm).
   * If a callback function is passed it will be called after the user clicks either button,
   * and the id of the button that was clicked will be passed as the only parameter to the callback
   * (could also be the top-right close button, which will always report as "cancel").
   * @param title The title bar text
   * @param message The message box body text
   * @param fn The callback function invoked after the message box is closed.
   * See <a href="#!/api/Ext.window.MessageBox-method-show" rel="Ext.window.MessageBox-method-show" class="docClass">show</a> method for details.
   * @param scope (Default window) The scope (<code>this</code> reference) in which the callback is executed.
   * @default window
   * @return this
   */
  public native function confirm(title:String, message:String, fn:Function = null, scope:Object = null):ext.window.MessageBoxWindow;

  /**
   * Displays a message box with a progress bar.
   *
   * <p>You are responsible for updating the progress bar as needed via <a href="#!/api/Ext.window.MessageBox-method-updateProgress" rel="Ext.window.MessageBox-method-updateProgress" class="docClass">updateProgress</a>
   * and closing the message box when the process is complete.</p>
   * @param title The title bar text
   * @param message The message box body text
   * @param progressText The text to display inside the progress bar
   * @default ''
   * @return this
   */
  public native function progress(title:String, message:String, progressText:String = ''):ext.window.MessageBoxWindow;

  /**
   * Displays a message box with OK and Cancel buttons prompting the user to enter some text (comparable to JavaScript's prompt).
   * The prompt can be a single-line or multi-line textbox.  If a callback function is passed it will be called after the user
   * clicks either button, and the id of the button that was clicked (could also be the top-right
   * close button, which will always report as "cancel") and the text that was entered will be passed as the two parameters to the callback.
   * @param title The title bar text
   * @param message The message box body text
   * @param fn The callback function invoked after the message box is closed.
   * See <a href="#!/api/Ext.window.MessageBox-method-show" rel="Ext.window.MessageBox-method-show" class="docClass">show</a> method for details.
   * @param scope (Default window) The scope (<code>this</code> reference) in which the callback is executed.
   * @default window
   * @param multiline True to create a multiline textbox using the defaultTextHeight
   * property, or the height in pixels to create the textbox/
   * @default false
   * @param value Default value of the text input element
   * @default ''
   * @return this
   */
  public native function prompt(title:String, message:String, fn:Function = null, scope:Object = null, multiline:* = false, value:String = ''):ext.window.MessageBoxWindow;

  /**
   * Adds the specified icon to the dialog.  By default, the class 'x-messagebox-icon' is applied for default
   * styling, and the class passed in is expected to supply the background image url. Pass in empty string ('')
   * to clear any existing icon. This method must be called before the MessageBox is shown.
   * The following built-in icon classes are supported, but you can also pass in a custom class name:
   *
   * <pre><code><a href="#!/api/Ext.window.MessageBox-property-INFO" rel="Ext.window.MessageBox-property-INFO" class="docClass">Ext.window.MessageBox.INFO</a>
   * <a href="#!/api/Ext.window.MessageBox-property-WARNING" rel="Ext.window.MessageBox-property-WARNING" class="docClass">Ext.window.MessageBox.WARNING</a>
   * <a href="#!/api/Ext.window.MessageBox-property-QUESTION" rel="Ext.window.MessageBox-property-QUESTION" class="docClass">Ext.window.MessageBox.QUESTION</a>
   * <a href="#!/api/Ext.window.MessageBox-property-ERROR" rel="Ext.window.MessageBox-property-ERROR" class="docClass">Ext.window.MessageBox.ERROR</a>
   * </code></pre>
   * @param icon A CSS classname specifying the icon's background image url, or empty string to clear the icon
   * @param width The width of the icon. If not specified, the default is used
   * @param height The height of the icon. If not specified, the default is used
   * @return this
   */
  public native function setIcon(icon:String, width:Number = NaN, height:Number = NaN):ext.window.MessageBoxWindow;

  /**
   * Updates a progress-style message box's text and progress bar. Only relevant on message boxes
   * initiated via <a href="#!/api/Ext.window.MessageBox-method-progress" rel="Ext.window.MessageBox-method-progress" class="docClass">progress</a> or <a href="#!/api/Ext.window.MessageBox-method-wait" rel="Ext.window.MessageBox-method-wait" class="docClass">wait</a>,
   * or by calling <a href="#!/api/Ext.window.MessageBox-method-show" rel="Ext.window.MessageBox-method-show" class="docClass">show</a> with progress: true.
   * @param value Any number between 0 and 1 (e.g., .5)
   * @default 0
   * @param progressText The progress text to display inside the progress bar.
   * @default ''
   * @param message The message box's body text is replaced with the specified string (defaults to undefined
   * so that any existing body text will not get overwritten by default unless a new value is passed in)
   * @return this
   */
  public native function updateProgress(value:Number = 0, progressText:String = '', message:String = null):ext.window.MessageBoxWindow;

  /**
   * Displays a message box with an infinitely auto-updating progress bar.  This can be used to block user
   * interaction while waiting for a long-running process to complete that does not have defined intervals.
   * You are responsible for closing the message box when the process is complete.
   * @param message The message box body text
   * @param title The title bar text
   * @param config A <a href="#!/api/Ext.ProgressBar-method-wait" rel="Ext.ProgressBar-method-wait" class="docClass">Ext.ProgressBar.wait</a> config object
   * @return this
   */
  public native function wait(message:String, title:String = null, config:Object = null):ext.window.MessageBoxWindow;
}
}