package ext.window {
import ext.Component;
import ext.dd.DragSource;
import ext.dom.Element;
import ext.panel.Panel;

[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.window.Window", require)]
/**
 * Fires after the window has been visually activated via <a href="#!/api/Ext.window.Window-method-setActive" rel="Ext.window.Window-method-setActive" class="docClass">setActive</a>.
 * @eventType ext.window.events.WindowEvent.ACTIVATE
 */
[Event(name = 'onActivate', type = 'ext.window.events.WindowEvent')]
/**
 * Fires after the window has been visually deactivated via <a href="#!/api/Ext.window.Window-method-setActive" rel="Ext.window.Window-method-setActive" class="docClass">setActive</a>.
 * @eventType ext.window.events.WindowEvent.DEACTIVATE
 */
[Event(name = 'onDeactivate', type = 'ext.window.events.WindowEvent')]
/**
 * Fires when this Window's modal mask is clicked or tapped. Returning <code>false</code> from
 * a handler will veto the subsequent preocessing of the <a href="#!/api/Ext.window.Window-cfg-maskClickAction" rel="Ext.window.Window-cfg-maskClickAction" class="docClass">maskClickAction</a>..
 * @eventType ext.window.events.WindowEvent.MASK_CLICK
 */
[Event(name = 'onMaskClick', type = 'ext.window.events.WindowEvent')]
/**
 * Fires after the window has been maximized.
 * @eventType ext.window.events.WindowEvent.MAXIMIZE
 */
[Event(name = 'onMaximize', type = 'ext.window.events.WindowEvent')]
/**
 * Fires after the window has been minimized.
 * @eventType ext.window.events.WindowEvent.MINIMIZE
 */
[Event(name = 'onMinimize', type = 'ext.window.events.WindowEvent')]
/**
 * Fires after the window has been resized.
 * @eventType ext.window.events.Window_width_heightEvent.RESIZE
 */
[Event(name = 'onResize', type = 'ext.window.events.Window_width_heightEvent')]
/**
 * Fires after the window has been restored to its original size after being maximized.
 * @eventType ext.window.events.WindowEvent.RESTORE
 */
[Event(name = 'onRestore', type = 'ext.window.events.WindowEvent')]
/**
 * A specialized panel intended for use as an application window. Windows are floated, <a href="#!/api/Ext.window.Window-cfg-resizable" rel="Ext.window.Window-cfg-resizable" class="docClass">resizable</a>, and
 * <a href="#!/api/Ext.window.Window-cfg-draggable" rel="Ext.window.Window-cfg-draggable" class="docClass">draggable</a> by default. Windows can be <a href="#!/api/Ext.window.Window-cfg-maximizable" rel="Ext.window.Window-cfg-maximizable" class="docClass">maximized</a> to fill the viewport, restored to
 * their prior size, and can be <a href="#!/api/Ext.window.Window-method-minimize" rel="Ext.window.Window-method-minimize" class="docClass">minimize</a>d.
 *
 * <p>Windows can also be linked to a <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">Ext.ZIndexManager</a> or managed by the <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">Ext.WindowManager</a> to provide
 * grouping, activation, to front, to back and other application-specific behavior.</p>
 *
 * <p>By default, Windows will be rendered to document.body. To <a href="#!/api/Ext.window.Window-cfg-constrain" rel="Ext.window.Window-cfg-constrain" class="docClass">constrain</a> a Window to another element specify
 * <a href="#!/api/Ext.Component-cfg-renderTo" rel="Ext.Component-cfg-renderTo" class="docClass">renderTo</a>.</p>
 *
 * <p><strong>As with all <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>s, it is important to consider how you want the Window to size
 * and arrange any child Components. Choose an appropriate <a href="#!/api/Ext.window.Window-cfg-layout" rel="Ext.window.Window-cfg-layout" class="docClass">layout</a> configuration which lays out child Components
 * in the required manner.</strong></p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', {
 *     title: 'Hello',
 *     height: 200,
 *     width: 400,
 *     layout: 'fit',
 *     items: {  // Let's put an empty grid in just to illustrate fit layout
 *         xtype: 'grid',
 *         border: false,
 *         columns: [{header: 'World'}],                 // One header just for show. There's no data,
 *         store: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.ArrayStore" rel="Ext.data.ArrayStore" class="docClass">Ext.data.ArrayStore</a>', {}) // A dummy empty data store
 *     }
 * }).show();
 * </code></pre>
 */
public class Window extends ext.panel.Panel {
  /**
   * @param config @inheritDoc
   */
  public function Window(config:ext.window.Window = null) {
    super();
  }

  /**
   * If this Window is configured <a href="#!/api/Ext.window.Window-cfg-draggable" rel="Ext.window.Window-cfg-draggable" class="docClass">draggable</a>, this property will contain an instance of
   * <a href="#!/api/Ext.util.ComponentDragger" rel="Ext.util.ComponentDragger" class="docClass">Ext.util.ComponentDragger</a> (A subclass of <a href="#!/api/Ext.dd.DragTracker" rel="Ext.dd.DragTracker" class="docClass">DragTracker</a>) which handles dragging
   * the Window's DOM Element, and constraining according to the <a href="#!/api/Ext.window.Window-cfg-constrain" rel="Ext.window.Window-cfg-constrain" class="docClass">constrain</a> and <a href="#!/api/Ext.window.Window-cfg-constrainHeader" rel="Ext.window.Window-cfg-constrainHeader" class="docClass">constrainHeader</a> .
   *
   * <p>This has implementations of <code>onBeforeStart</code>, <code>onDrag</code> and <code>onEnd</code> which perform the dragging action. If
   * extra logic is needed at these points, use <a href="#!/api/Ext.Function-method-createInterceptor" rel="Ext.Function-method-createInterceptor" class="docClass">createInterceptor</a> or
   * <a href="#!/api/Ext.Function-method-createSequence" rel="Ext.Function-method-createSequence" class="docClass">createSequence</a> to augment the existing implementations.</p>
   */
  override public native function get dd():ext.dd.DragSource;

  /**
   * @private
   */
  override public native function set dd(value:ext.dd.DragSource):void;

  [ExtConfig]
  /**
   * Windows render to the body on first show.
   * @default true
   */
  override public native function get autoRender():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set autoRender(value:*):void;

  [ExtConfig]
  /**
   * The base CSS class to apply to this panel's element.
   * @default 'x-window'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  /**
   * True to display the 'close' tool button and allow the user to close the window, false to hide the button and
   * disallow closing the window.
   *
   * <p>By default, when close is requested by either clicking the close button in the header or pressing ESC when the
   * Window has focus, the <a href="#!/api/Ext.window.Window-method-close" rel="Ext.window.Window-method-close" class="docClass">close</a> method will be called. This will <em><a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a></em> the
   * Window and its content meaning that it may not be reused.</p>
   *
   * <p>To make closing a Window <em>hide</em> the Window so that it may be reused, set <a href="#!/api/Ext.window.Window-cfg-closeAction" rel="Ext.window.Window-cfg-closeAction" class="docClass">closeAction</a> to 'hide'.</p>
   * @default true
   */
  override public native function get closable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set closable(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to render the window collapsed, false to render it expanded. Note that if <a href="#!/api/Ext.window.Window-cfg-expandOnShow" rel="Ext.window.Window-cfg-expandOnShow" class="docClass">expandOnShow</a>
   * is true (the default) it will override the <code>collapsed</code> config and the window will always be
   * expanded when shown.
   * @default false
   * @see #getCollapsed()
   * @see #setCollapsed()
   */
  override public native function get collapsed():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set collapsed(value:Boolean):void;

  [ExtConfig]
  /**
   * True to constrain the window within its containing element, false to allow it to fall outside of its containing
   * element. By default the window will be rendered to <code>document.body</code>. To render and constrain the window within
   * another element specify <a href="#!/api/Ext.window.Window-cfg-renderTo" rel="Ext.window.Window-cfg-renderTo" class="docClass">renderTo</a>. Optionally the header only can be constrained
   * using <a href="#!/api/Ext.window.Window-cfg-constrainHeader" rel="Ext.window.Window-cfg-constrainHeader" class="docClass">constrainHeader</a>.
   * @default false
   */
  override public native function get constrain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set constrain(value:Boolean):void;

  [ExtConfig]
  /**
   * True to constrain the window header within its containing element (allowing the window body to fall outside of
   * its containing element) or false to allow the header to fall outside its containing element.
   * Optionally the entire window can be constrained using <a href="#!/api/Ext.window.Window-cfg-constrain" rel="Ext.window.Window-cfg-constrain" class="docClass">constrain</a>.
   * @default false
   */
  override public native function get constrainHeader():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set constrainHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies a Component to receive focus when this Window is focused.
   *
   * <p>If a String is provided, the Component will be resolved using the <a href="#!/api/Ext.window.Window-method-down" rel="Ext.window.Window-method-down" class="docClass">down</a> method which uses <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a>.
   * If the string begins with an alphanumeric value, it will first attempt to find the Component based on the <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">Ext.Component.id</a> or <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">Ext.Component.itemId</a>.
   * If a matching component is not found via id, then an attempt to do a query to find a matching component.</p>
   *
   * <p>An example of finding the Component with an id/itemId:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', {
   *     autoShow     : true,
   *     width        : 300,
   *     title        : 'Login',
   *     defaultFocus : 'username',
   *     items        : [
   *         {
   *             xtype      : 'textfield',
   *             fieldLabel : 'Username',
   *             itemId     : 'username',
   *             name       : 'username'
   *         },
   *         {
   *             xtype      : 'textfield',
   *             inputType  : 'password',
   *             fieldLabel : 'Password',
   *             itemId     : 'password',
   *             name       : 'password'
   *         }
   *     ]
   * });
   * </code></pre>
   *
   * <p>If a Number is provided, this will resolve an <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a> at that index. This is very useful if
   * the window has buttons in the <a href="#!/api/Ext.window.Window-cfg-buttons" rel="Ext.window.Window-cfg-buttons" class="docClass">buttons</a> config and you want to provide default focus to one of them.</p>
   *
   * <p>An example of this would be:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', {
   *     autoShow     : true,
   *     width        : 300,
   *     title        : 'Login',
   *     defaultFocus : 1,
   *     items        : [
   *         {
   *             xtype      : 'textfield',
   *             fieldLabel : 'Username',
   *             name       : 'username'
   *        },
   *        {
   *             xtype      : 'textfield',
   *             inputType  : 'password',
   *             fieldLabel : 'Password',
   *             name       : 'password'
   *         }
   *     ],
   *     buttons      : [
   *         {
   *             text : 'Cancel'
   *         },
   *         {
   *             text : 'Login'
   *         }
   *     ]
   * });
   * </code></pre>
   *
   * <p>In summary, defaultFocus may be one of:</p>
   *
   * <ul>
   * <li>The index of a footer Button.</li>
   * <li>The id or <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">Ext.Component.itemId</a> of a descendant Component.</li>
   * <li>A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">query</a> to find a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.</li>
   * <li>A descendant <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.</li>
   * </ul>
   */
  override public native function get defaultFocus():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set defaultFocus(value:String):void;

  [ExtConfig]
  /**
   * True to allow the window to be dragged by the header bar, false to disable dragging. Note that
   * by default the window will be centered in the viewport, so if dragging is disabled the window may need to be
   * positioned programmatically after render (e.g., <code>myWindow.setPosition(100, 100);</code>).
   * @default true
   */
  override public native function get draggable():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set draggable(value:*):void;

  [ExtConfig]
  /**
   * Specify as true to float the Component outside of the document flow using CSS absolute positioning.
   *
   * <p>Components such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s and <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Menu</a>s are floating by default.</p>
   *
   * <p>Floating Components that are programmatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a> will register
   * themselves with the global <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a></p>
   *
   * <h3>Floating Components as child items of a Container</h3>
   *
   * <p>A floating Component may be used as a child item of a Container. This just allows the floating Component to seek
   * a ZIndexManager by examining the ownerCt chain.</p>
   *
   * <p>When configured as floating, Components acquire, at render time, a <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a> which
   * manages a stack of related floating Components. The ZIndexManager sorts its stack according to
   * an incrementing access counter and the <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">alwaysOnTop</a> config when the Component's <a href="#!/api/Ext.window.Window-method-toFront" rel="Ext.window.Window-method-toFront" class="docClass">toFront</a> method is called.</p>
   *
   * <p>The ZIndexManager is found by traversing up the <a href="#!/api/Ext.window.Window-property-ownerCt" rel="Ext.window.Window-property-ownerCt" class="docClass">ownerCt</a> chain to find an ancestor which itself is
   * floating. This is so that descendant floating Components of floating <em>Containers</em> (Such as a ComboBox dropdown
   * within a Window) can have its zIndex managed relative to any siblings, but always <strong>above</strong> that floating
   * ancestor Container.</p>
   *
   * <p>If no floating ancestor is found, a floating Component registers itself with the default <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a>.</p>
   *
   * <p>Floating components <em>do not participate in the Container's layout</em>. Because of this, they are not rendered until
   * you explicitly <a href="#!/api/Ext.window.Window-method-show" rel="Ext.window.Window-method-show" class="docClass">show</a> them.</p>
   *
   * <p>After rendering, the ownerCt reference is deleted, and the <a href="#!/api/Ext.window.Window-property-floatParent" rel="Ext.window.Window-property-floatParent" class="docClass">floatParent</a> property is set to the found
   * floating ancestor Container. If no floating ancestor Container was found the <a href="#!/api/Ext.window.Window-property-floatParent" rel="Ext.window.Window-property-floatParent" class="docClass">floatParent</a> property will
   * not be set.</p>
   * @default true
   */
  override public native function get floating():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set floating(value:Boolean):void;

  [ExtConfig]
  /**
   * Render this Window hidden. If <code>true</code>, the <a href="#!/api/Ext.window.Window-method-hide" rel="Ext.window.Window-method-hide" class="docClass">hide</a> method will be called internally.
   * @default true
   * @see #setHidden()
   */
  override public native function get hidden():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set hidden(value:Boolean):void;

  [ExtConfig]
  /**
   * Windows hide using offsets in order to preserve the scroll positions of their descendants.  You may review
   * other configuration options here: <a href="#!/api/Ext.Component-cfg-hideMode" rel="Ext.Component-cfg-hideMode" class="docClass">Ext.Component.hideMode</a>.
   * @default 'offsets'
   */
  override public native function get hideMode():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set hideMode(value:String):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to allow user resizing at each edge and corner of the window, false to disable resizing.
   *
   * <p>This may also be specified as a config object to <a href="#!/api/Ext.resizer.Resizer" rel="Ext.resizer.Resizer" class="docClass">Ext.resizer.Resizer</a></p>
   * @default true
   */
  override public native function get resizable():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set resizable(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The X position of the left edge of the window on initial showing. Defaults to centering the Window within the
   * width of the Window's container <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> (The Element that the Window is rendered to).
   * @see #getX()
   * @see #setX()
   */
  override public native function get x():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set x(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The Y position of the top edge of the window on initial showing. Defaults to centering the Window within the
   * height of the Window's container <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> (The Element that the Window is rendered to).
   * @see #getY()
   * @see #setY()
   */
  override public native function get y():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set y(value:Number):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Window, or subclass thereof.
   * @default true
   */
  public native function get isWindow():Boolean;

  /**
   * @private
   */
  public native function set isWindow(value:Boolean):void;

  [ExtConfig]
  /**
   * Id, Component element, or Component from which the window should animate when
   * shown or hidden.
   *
   * <p>You may also pass true to have the Window animate when maximizing and restoring
   * using the maximize / restore tools created via the <a href="#!/api/Ext.window.Window-cfg-maximizable" rel="Ext.window.Window-cfg-maximizable" class="docClass">maximizable</a> config.</p>
   *
   * <pre><code>var btn, win;
   *
   * btn = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'button',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     text: 'Show Window',
   *     handler: function() {
   *         win.show();
   *     }
   * });
   *
   * win = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'window',
   *     title: 'Animate from the Show Window Button',
   *     height: 300,
   *     width: 400,
   *     modal: true,
   *     closeAction: 'hide',
   *     animateTarget: btn
   *     // or btn.getId()
   *     // or btn.getEl()
   *     // or true (when maximizable is true)
   * });
   * </code></pre>
   * @default null
   */
  public native function get animateTarget():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animateTarget(value:*):void;

  [ExtConfig]
  /**
   * True to always expand the window when it is displayed, false to keep it in its current state (which may be
   * <a href="#!/api/Ext.window.Window-cfg-collapsed" rel="Ext.window.Window-cfg-collapsed" class="docClass">collapsed</a>) when displayed.
   * @default true
   */
  public native function get expandOnShow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandOnShow(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to false to disable the ghost panel during dragging the window.
   * Do note that you should not set this to true, by default it is a function.
   */
  public native function get ghost():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ghost(value:*):void;

  [ExtConfig]
  /**
   * True to hide this Window's shadow when another floating item in the same z-index stack is activated.
   * @default false
   */
  public native function get hideShadowOnDeactivate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideShadowOnDeactivate(value:Boolean):void;

  [ExtConfig]
  /**
   * The method to call when the window's modal mask is clicked or tapped:
   *
   * <ul>
   * <li><p><strong><code>'<a href="#!/api/Ext.window.Window-method-focus" rel="Ext.window.Window-method-focus" class="docClass">focus</a>'</code></strong> :</p>
   *
   * <p>The default. Focus the window, which will then pass focus into its <a href="#!/api/Ext.window.Window-cfg-defaultFocus" rel="Ext.window.Window-cfg-defaultFocus" class="docClass">defaultFocus</a> delegate.</p></li>
   * <li><p><strong><code>'<a href="#!/api/Ext.window.Window-method-destroy" rel="Ext.window.Window-method-destroy" class="docClass">destroy</a>'</code></strong> :</p>
   *
   * <p>Remove  the window from the DOM and <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a> it and all descendant
   * Components. The window will <strong>not</strong> be available to be redisplayed via the <a href="#!/api/Ext.window.Window-method-show" rel="Ext.window.Window-method-show" class="docClass">show</a> method.</p></li>
   * <li><p><strong><code>'<a href="#!/api/Ext.window.Window-method-hide" rel="Ext.window.Window-method-hide" class="docClass">hide</a>'</code></strong> :</p>
   *
   * <p><a href="#!/api/Ext.window.Window-method-hide" rel="Ext.window.Window-method-hide" class="docClass">hide</a> the window by setting visibility to hidden and applying negative offsets. The window will be
   * available to be redisplayed via the <a href="#!/api/Ext.window.Window-method-show" rel="Ext.window.Window-method-show" class="docClass">show</a> method.</p></li>
   * </ul>
   * @default focus
   * @since 6.2.0
   */
  public native function get maskClickAction():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maskClickAction(value:String):void;

  [ExtConfig]
  /**
   * True to display the 'maximize' tool button and allow the user to maximize the window, false to hide the button
   * and disallow maximizing the window. Note that when a window is maximized, the tool button
   * will automatically change to a 'restore' button with the appropriate behavior already built-in that will restore
   * the window to its previous size.
   * @default false
   */
  public native function get maximizable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maximizable(value:Boolean):void;

  [ExtConfig]
  /**
   * True to initially display the window in a maximized state.
   * @default false
   */
  public native function get maximized():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maximized(value:Boolean):void;

  [ExtConfig]
  /**
   * True to display the 'minimize' tool button and allow the user to minimize the window, false to hide the button
   * and disallow minimizing the window. Note that this button provides no implementation -- the
   * behavior of minimizing a window is implementation-specific, so the minimize event must be handled and a custom
   * minimize behavior implemented for this option to be useful.
   * @default false
   */
  public native function get minimizable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minimizable(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to listen to the viewport resize event and perform any layout updating if necessary.
   * This is useful if using sizes as percentages for the window.
   */
  public native function get monitorResize():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set monitorResize(value:Boolean):void;

  [ExtConfig]
  /**
   * Allows override of the built-in processing for the escape key. Default action is to close the Window (performing
   * whatever action is specified in <a href="#!/api/Ext.window.Window-cfg-closeAction" rel="Ext.window.Window-cfg-closeAction" class="docClass">closeAction</a>. To prevent the Window closing when the escape key is
   * pressed, specify this as <a href="#!/api/Ext-property-emptyFn" rel="Ext-property-emptyFn" class="docClass">Ext.emptyFn</a>.
   */
  public native function get onEsc():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set onEsc(value:Function):void;

  [ExtConfig]
  /**
   * True to render the window body with a transparent background so that it will blend into the framing elements,
   * false to add a lighter background color to visually highlight the body element and separate it more distinctly
   * from the surrounding frame.
   * @default false
   */
  public native function get plain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plain(value:Boolean):void;

  /**
   * Gets the configured default focus item.  If a <a href="#!/api/Ext.window.Window-cfg-defaultFocus" rel="Ext.window.Window-cfg-defaultFocus" class="docClass">defaultFocus</a> is set, it will
   * receive focus when the Window's <code>focus</code> method is called, otherwise the
   * Window itself will receive focus.
   */
  override public native function getDefaultFocus():ext.Component;

  /**
   * Returns the focus holder element associated with this Window.
   * By default, this is the Window's element; this can be overridden
   * by setting <a href="#!/api/Ext.window.Window-cfg-defaultFocus" rel="Ext.window.Window-cfg-defaultFocus" class="docClass">defaultFocus</a> property.
   * @return the focus holding element or Component.
   */
  override protected native function getFocusEl():ext.dom.Element;

  /**
   * Fits the window within its current container and automatically replaces the <a href="#!/api/Ext.window.Window-cfg-maximizable" rel="Ext.window.Window-cfg-maximizable" class="docClass">'maximize' tool
   * button</a> with the 'restore' tool button. Also see <a href="#!/api/Ext.window.Window-method-toggleMaximize" rel="Ext.window.Window-method-toggleMaximize" class="docClass">toggleMaximize</a>.
   * @param animate Pass <code>true</code> to animate this Window to full size.
   * @default false
   * @return this
   */
  public native function maximize(animate:Boolean = false):ext.window.Window;

  /**
   * Placeholder method for minimizing the window. By default, this method simply fires the <a href="#!/api/Ext.window.Window-event-minimize" rel="Ext.window.Window-event-minimize" class="docClass">minimize</a> event
   * since the behavior of minimizing a window is application-specific. To implement custom minimize behavior, either
   * the minimize event can be handled or this method can be overridden.
   * @return this
   */
  public native function minimize():ext.window.Window;

  /**
   * Restores a <a href="#!/api/Ext.window.Window-cfg-maximizable" rel="Ext.window.Window-cfg-maximizable" class="docClass">maximized</a> window back to its original size and position prior to being maximized
   * and also replaces the 'restore' tool button with the 'maximize' tool button. Also see <a href="#!/api/Ext.window.Window-method-toggleMaximize" rel="Ext.window.Window-method-toggleMaximize" class="docClass">toggleMaximize</a>.
   * @param animate Pass <code>true</code> to animate the restore.
   * @default false
   * @return this
   */
  public native function restore(animate:Boolean = false):ext.window.Window;

  /**
   * A shortcut method for toggling between <a href="#!/api/Ext.window.Window-method-maximize" rel="Ext.window.Window-method-maximize" class="docClass">maximize</a> and <a href="#!/api/Ext.window.Window-method-restore" rel="Ext.window.Window-method-restore" class="docClass">restore</a> based on the current maximized
   * state of the window.
   * @return this
   */
  public native function toggleMaximize():ext.window.Window;
}
}