package ext {
import ext.app.ViewController;
import ext.app.ViewModel;
import ext.container.Container;
import ext.data.Session;
import ext.dom.Element;
import ext.event.Event;
import ext.mixin.IAccessible;
import ext.mixin.IBindable;
import ext.mixin.IComponentDelegation;
import ext.mixin.IInheritable;
import ext.mixin.IKeyboard;
import ext.mixin.Observable;
import ext.plugin.AbstractPlugin;
import ext.state.IStateful;
import ext.util.IAnimate;
import ext.util.IElementContainer;
import ext.util.IFloating;
import ext.util.IFocusable;
import ext.util.IPositionable;
import ext.util.IRenderable;
import ext.util.Positionable;
import ext.util.Region;
import js.HTMLElement;
import mx.resources.IResourceManager;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Renderable")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.Focusable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.Base")]
[Uses("ext.Mixin")]
[Native("Ext.Component", require)]
[ExtConfig]
/**
 * Fires after a Component has been visually activated.
 *
 * <p><strong>Note</strong> This event is only fired if this Component is a child of a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>
 * that uses <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> as it's layout or this Component is a floating Component.</p>
 * @eventType ext.events.ComponentEvent.ACTIVATE
 */
[Event(name = 'onActivate', type = 'ext.events.ComponentEvent')]
/**
 * Fires after a Component had been added to a Container.
 * @since 3.4.0
 * @eventType ext.events.Component_container_posEvent.ADDED
 */
[Event(name = 'onAdded', type = 'ext.events.Component_container_posEvent')]
/**
 * This event first after a component's layout has been updated by a layout that
 * included animation (e.g., a <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">panel</a> in an
 * <a href="#!/api/Ext.layout.container.Accordion" rel="Ext.layout.container.Accordion" class="docClass">accordion</a> layout).
 * @since 6.0.0
 * @eventType ext.events.ComponentEvent.AFTER_LAYOUT_ANIMATION
 */
[Event(name = 'onAfterLayoutAnimation', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the component rendering is finished.
 *
 * <p>The <code>afterrender</code> event is fired after this Component has been <a href="#!/api/Ext.Component-property-rendered" rel="Ext.Component-property-rendered" class="docClass">rendered</a>, been post-processed by any
 * <code>afterRender</code> method defined for the Component.</p>
 * @since 3.4.0
 * @eventType ext.events.ComponentEvent.AFTER_RENDER
 */
[Event(name = 'onAfterRender', type = 'ext.events.ComponentEvent')]
/**
 * Fires before a Component has been visually activated. Returning <code>false</code> from an event listener can prevent
 * the activate from occurring.
 *
 * <p><strong>Note</strong> This event is only fired if this Component is a child of a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>
 * that uses <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> as it's layout.</p>
 * @eventType ext.events.ComponentEvent.BEFORE_ACTIVATE
 */
[Event(name = 'onBeforeActivate', type = 'ext.events.ComponentEvent')]
/**
 * Fires before a Component has been visually deactivated. Returning <code>false</code> from an event listener can
 * prevent the deactivate from occurring.
 *
 * <p><strong>Note</strong> This event is only fired if this Component is a child of a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>
 * that uses <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> as it's layout.</p>
 * @eventType ext.events.ComponentEvent.BEFORE_DEACTIVATE
 */
[Event(name = 'onBeforeDeactivate', type = 'ext.events.ComponentEvent')]
/**
 * Fires before the component is <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a>ed. Return <code>false</code> from an event handler to stop the
 * <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a>.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.BEFORE_DESTROY
 */
[Event(name = 'onBeforeDestroy', type = 'ext.events.ComponentEvent')]
/**
 * Fires before the component is hidden when calling the <a href="#!/api/Ext.Component-method-hide" rel="Ext.Component-method-hide" class="docClass">hide</a> method. Return <code>false</code> from an event
 * handler to stop the hide.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.BEFORE_HIDE
 */
[Event(name = 'onBeforeHide', type = 'ext.events.ComponentEvent')]
/**
 * Fires before the component is <a href="#!/api/Ext.Component-property-rendered" rel="Ext.Component-property-rendered" class="docClass">rendered</a>. Return <code>false</code> from an event handler to stop the
 * <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">render</a>.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.BEFORE_RENDER
 */
[Event(name = 'onBeforeRender', type = 'ext.events.ComponentEvent')]
/**
 * Fires before the component is shown when calling the <a href="#!/api/Ext.Component-method-show" rel="Ext.Component-method-show" class="docClass">show</a> method. Return <code>false</code> from an event
 * handler to stop the show.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.BEFORE_SHOW
 */
[Event(name = 'onBeforeShow', type = 'ext.events.ComponentEvent')]
/**
 * Fires before the state of the object is restored. Return false from an event handler to stop the restore.
 * @eventType ext.events.Component_stateEvent.BEFORE_STATE_RESTORE
 */
[Event(name = 'onBeforeStateRestore', type = 'ext.events.Component_stateEvent')]
/**
 * Fires before the state of the object is saved to the configured state provider. Return false to stop the save.
 * @eventType ext.events.Component_stateEvent.BEFORE_STATE_SAVE
 */
[Event(name = 'onBeforeStateSave', type = 'ext.events.Component_stateEvent')]
/**
 * Fires when this Component loses focus.
 * @eventType ext.events.Component_eventEvent.BLUR
 */
[Event(name = 'onBlur', type = 'ext.events.Component_eventEvent')]
/**
 * Fires <em>one time</em> - after the component has been laid out for the first time at its initial size.
 *
 * <p>This event does not fire on components that use <a href="#!/api/Ext.Component-cfg-liquidLayout" rel="Ext.Component-cfg-liquidLayout" class="docClass">liquidLayout</a>, such as
 * <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Buttons</a> and <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Form Fields</a>.</p>
 * @eventType ext.events.Component_width_heightEvent.BOX_READY
 */
[Event(name = 'onBoxReady', type = 'ext.events.Component_width_heightEvent')]
/**
 * Fires after a Component has been visually deactivated.
 *
 * <p><strong>Note</strong> This event is only fired if this Component is a child of a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>
 * that uses <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> as it's layout or this Component is a floating Component.</p>
 * @eventType ext.events.ComponentEvent.DEACTIVATE
 */
[Event(name = 'onDeactivate', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the component is <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">destroy</a>ed.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.DESTROY
 */
[Event(name = 'onDestroy', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the component is disabled.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.DISABLE
 */
[Event(name = 'onDisable', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the component is enabled.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.ENABLE
 */
[Event(name = 'onEnable', type = 'ext.events.ComponentEvent')]
/**
 * Fires when this Component receives focus.
 * @eventType ext.events.Component_eventEvent.FOCUS
 */
[Event(name = 'onFocus', type = 'ext.events.Component_eventEvent')]
/**
 * Fires when focus enters this Component's hierarchy.
 * @eventType ext.events.Component_eventEvent.FOCUS_ENTER
 */
[Event(name = 'onFocusEnter', type = 'ext.events.Component_eventEvent')]
/**
 * Fires when focus leaves this Component's hierarchy.
 * @eventType ext.events.Component_eventEvent.FOCUS_LEAVE
 */
[Event(name = 'onFocusLeave', type = 'ext.events.Component_eventEvent')]
/**
 * Fires after the component is hidden. Fires after the component is hidden when calling the <a href="#!/api/Ext.Component-method-hide" rel="Ext.Component-method-hide" class="docClass">hide</a>
 * method.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.HIDE
 */
[Event(name = 'onHide', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the component is moved.
 * @eventType ext.events.Component_x_yEvent.MOVE
 */
[Event(name = 'onMove', type = 'ext.events.Component_x_yEvent')]
/**
 * Fires when a component is removed from an <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>
 * @since 3.4.0
 * @eventType ext.events.Component_ownerCtEvent.REMOVED
 */
[Event(name = 'onRemoved', type = 'ext.events.Component_ownerCtEvent')]
/**
 * Fires after the component markup is <a href="#!/api/Ext.Component-property-rendered" rel="Ext.Component-property-rendered" class="docClass">rendered</a>.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.RENDER
 */
[Event(name = 'onRender', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the component is resized. Note that this does <em>not</em> fire when the component is first laid out at its initial
 * size. To hook that point in the life cycle, use the <a href="#!/api/Ext.Component-event-boxready" rel="Ext.Component-event-boxready" class="docClass">boxready</a> event.
 *
 * <p>This event does not fire on components that use <a href="#!/api/Ext.Component-cfg-liquidLayout" rel="Ext.Component-cfg-liquidLayout" class="docClass">liquidLayout</a>, such as
 * <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Buttons</a> and <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Form Fields</a>.</p>
 * @eventType ext.events.Component_width_height_oldWidth_oldHeightEvent.RESIZE
 */
[Event(name = 'onResize', type = 'ext.events.Component_width_height_oldWidth_oldHeightEvent')]
/**
 * Fires after the component is shown when calling the <a href="#!/api/Ext.Component-method-show" rel="Ext.Component-method-show" class="docClass">show</a> method.
 * @since 1.1.0
 * @eventType ext.events.ComponentEvent.SHOW
 */
[Event(name = 'onShow', type = 'ext.events.ComponentEvent')]
/**
 * Fires after the state of the object is restored.
 * @eventType ext.events.Component_stateEvent.STATE_RESTORE
 */
[Event(name = 'onStateRestore', type = 'ext.events.Component_stateEvent')]
/**
 * Fires after the state of the object is saved to the configured state provider.
 * @eventType ext.events.Component_stateEvent.STATE_SAVE
 */
[Event(name = 'onStateSave', type = 'ext.events.Component_stateEvent')]
/**
 * Base class for all Ext components.
 *
 * <p>The Component base class has built-in support for basic hide/show and enable/disable
 * and size control behavior.</p>
 *
 * <h2>xtypes</h2>
 *
 * <p>Every component has a specific xtype, which is its Ext-specific type name, along with
 * methods for checking the xtype like <a href="#!/api/Ext.Component-method-getXType" rel="Ext.Component-method-getXType" class="docClass">getXType</a> and <a href="#!/api/Ext.Component-method-isXType" rel="Ext.Component-method-isXType" class="docClass">isXType</a>. See the
 * [Component Guide][../../../core_concepts/components.html] for more information on xtypes
 * and the Component hierarchy.</p>
 *
 * <h2>Finding components</h2>
 *
 * <p>All Components are registered with the <a href="#!/api/Ext.ComponentManager" rel="Ext.ComponentManager" class="docClass">Ext.ComponentManager</a> on construction so
 * that they can be referenced at any time via <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>, passing the
 * <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>.</p>
 *
 * <p>Additionally the <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> provides a CSS-selectors-like way to look
 * up components by their xtype and many other attributes.  For example the following code
 * will find all textfield components inside component with <code>id: 'myform'</code>:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('#myform textfield');
 * </code></pre>
 *
 * <h2>Extending <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></h2>
 *
 * <p>All subclasses of Component may participate in the automated Ext component
 * life cycle of creation, rendering and destruction which is provided by the
 * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> class. Components may be added to a Container
 * through the <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> config option at the time
 * the Container is created, or they may be added dynamically via the
 * <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">add</a> method.</p>
 *
 * <p>All user-developed visual widgets that are required to participate in automated
 * life cycle and size management should subclass Component.</p>
 *
 * <p>See the Creating new UI controls chapter in [Component Guide][../../../core_concepts/components.html]
 * for details on how and to either extend or augment Ext JS base classes to create custom Components.</p>
 *
 * <h2>The <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> class by itself</h2>
 *
 * <p>Usually one doesn't need to instantiate the <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> class. There are subclasses
 * which implement specialized use cases, covering most application needs. However it is
 * possible to instantiate a base Component, and it can be rendered to document, or handled
 * by layouts as the child item of a Container:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
 *     html: 'Hello world!',
 *     width: 300,
 *     height: 200,
 *     padding: 20,
 *     style: {
 *         color: '#FFFFFF',
 *         backgroundColor:'#000000'
 *     },
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>The Component above creates its encapsulating <code>div</code> upon render, and use the configured
 * HTML as content. More complex internal structure may be created using the
 * <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a> configuration, although to display database-derived mass data, it is
 * recommended that an ExtJS data-backed Component such as a <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">View</a>,
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">GridPanel</a>, or <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">TreePanel</a> be used.</p>
 *
 * <p><strong>From override Ext.layout.container.border.Region:</strong> This override provides extra, border layout specific methods for <code><a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></code>. The
 * <code><a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">Ext.layout.container.Border</a></code> class requires this override so that the added functions
 * are only included in a build when <code>border</code> layout is used.</p>
 *
 * <p><strong>From override Ext.rtl.Component:</strong> This override adds RTL support and the <code>rtl</code> config option to AbstactComponent.</p>
 */
public class Component extends ext.mixin.Observable implements ext.mixin.IInheritable, ext.util.IFloating, ext.util.IPositionable, ext.mixin.IComponentDelegation, ext.mixin.IBindable, ext.util.IAnimate, ext.util.IElementContainer, ext.util.IRenderable, ext.state.IStateful, ext.util.IFocusable, ext.mixin.IAccessible, ext.mixin.IKeyboard {
  /**
   * Cancels layout of a component.
   */
  public static native function cancelLayout(comp:ext.Component, isDestroying:* = undefined):void;

  /**
   * Performs all pending layouts that were scheduled while
   * <a href="#!/api/Ext.Component-method-suspendLayouts" rel="Ext.Component-method-suspendLayouts" class="docClass">suspendLayouts</a> was in effect.
   */
  public static native function flushLayouts():void;

  /**
   * Find the Widget or Component to which the given Element belongs.
   * @param node The element from which to start to find an owning Component.
   * @param limit The element at which to stop upward searching for an
   * owning Component, or the number of Components to traverse before giving up.
   * Defaults to the document's HTML element.
   * @param selector An optional <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector to filter the target.
   * @return Component, or null
   */
  public static native function fromElement(node:*, limit:* = undefined, selector:String = null):*;

  /**
   * Resumes layout activity in the whole framework.
   *
   * <p><a href="#!/api/Ext-method-suspendLayouts" rel="Ext-method-suspendLayouts" class="docClass">Ext.suspendLayouts</a> is alias of <a href="#!/api/Ext.Component-method-suspendLayouts" rel="Ext.Component-method-suspendLayouts" class="docClass">suspendLayouts</a>.</p>
   * @param flush <code>true</code> to perform all the pending layouts. This can also be
   * achieved by calling <a href="#!/api/Ext.Component-static-method-flushLayouts" rel="Ext.Component-static-method-flushLayouts" class="docClass">flushLayouts</a> directly.
   * @default false
   */
  public static native function resumeLayouts(flush:Boolean = false):void;

  /**
   * Stops layouts from happening in the whole framework.
   *
   * <p>It's useful to suspend the layout activity while updating multiple components and
   * containers:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-suspendLayouts" rel="Ext-method-suspendLayouts" class="docClass">Ext.suspendLayouts</a>();
   * // batch of updates...
   * <a href="#!/api/Ext-method-resumeLayouts" rel="Ext-method-resumeLayouts" class="docClass">Ext.resumeLayouts</a>(true);
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-suspendLayouts" rel="Ext-method-suspendLayouts" class="docClass">Ext.suspendLayouts</a> is alias of <a href="#!/api/Ext.Component-method-suspendLayouts" rel="Ext.Component-method-suspendLayouts" class="docClass">suspendLayouts</a>.</p>
   *
   * <p>See also <a href="#!/api/Ext-method-batchLayouts" rel="Ext-method-batchLayouts" class="docClass">Ext.batchLayouts</a> for more abstract way of doing this.</p>
   */
  public static native function suspendLayouts():void;

  /**
   * TODO: IDEA cannot cope well with a static and a non-static method of the same name in the same class...
   *
   * Updates layout of a component.
   * @param comp The component to update.
   * @param defer <code>true</code> to just queue the layout if this component.
   * @default false
   * /
   *    public static native function updateLayout(comp:Component, defer:Boolean = false):void;
   */
  public function Component(config:ext.Component = null) {
    super();
  }

  [ExtConfig("id")]
  /**
   * Alias for <code>id</code> property to make it accessible from MXML.
   * @copy #id
   * @see #id
   */
  public native function set id_(value:String):void;

  /**
   * The Action assigned to this Component.
   */
  public native function get baseAction():ext.Action;

  /**
   * @private
   */
  public native function set baseAction(value:ext.Action):void;

  /**
   * Setting this property to <code>true</code> causes the <a href="#!/api/Ext.Component-method-isLayoutRoot" rel="Ext.Component-method-isLayoutRoot" class="docClass">isLayoutRoot</a> method to return
   * <code>true</code> and stop the search for the top-most component for a layout.
   * @default false
   */
  public native function get _isLayoutRoot():Boolean;

  /**
   * @private
   */
  public native function set _isLayoutRoot(value:Boolean):void;

  /**
   * ARIA role for this Component, defaults to no role.
   * With no role, no other ARIA attributes are set.
   */
  public native function get ariaRole():String;

  /**
   * The name of the padding property that is used by the layout to manage
   * padding.  See <a href="#!/api/Ext.layout.container.Auto-property-managePadding" rel="Ext.layout.container.Auto-property-managePadding" class="docClass">managePadding</a>
   * @default 'padding'
   */
  public native function get contentPaddingProperty():String;

  /**
   * @private
   */
  public native function set contentPaddingProperty(value:String):void;

  /**
   * <strong>Only present for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components which were inserted as child items of Containers.</strong>
   *
   * <p>There are other similar relationships such as the <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> which activates a <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a>, or the
   * <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu item</a> which activated a <a href="#!/api/Ext.menu.Item-cfg-menu" rel="Ext.menu.Item-cfg-menu" class="docClass">submenu</a>, or the
   * <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">column header</a> which activated the column menu.</p>
   *
   * <p>These differences are abstracted away by the <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">up</a> method.</p>
   *
   * <p>Floating Components that are programmatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a> will not have a <code>floatParent</code>
   * property.</p>
   *
   * <p>See <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> and <a href="#!/api/Ext.Component-property-zIndexManager" rel="Ext.Component-property-zIndexManager" class="docClass">zIndexManager</a></p>
   */
  public native function get floatParent():ext.container.Container;

  /**
   * <code>true</code> for interactive Components, <code>false</code> for static Components.
   * For Containers, this property reflects interactiveness of the
   * Container itself, not its children. See <a href="#!/api/Ext.util.Focusable-method-isFocusable" rel="Ext.util.Focusable-method-isFocusable" class="docClass">isFocusable</a>.
   *
   * <p><strong>Note:</strong> Plain components are static, so not focusable.</p>
   * @default false
   */
  public native function get focusable():Boolean;

  /**
   * Indicates the width of any framing elements which were added within the encapsulating
   * element to provide graphical, rounded borders. See the <a href="#!/api/Ext.Component-cfg-frame" rel="Ext.Component-cfg-frame" class="docClass">frame</a> config. This
   * property is <code>null</code> if the component is not framed.
   *
   * <p>This is an object containing the frame width in pixels for all four sides of the
   * Component containing the following properties:</p>
   * <ul>
   *   <li><code>top</code> : Number (optional)
   * The width of the top framing element in pixels.
   * Default: 0
   *   </li>
   *   <li><code>right</code> : Number (optional)
   * The width of the right framing element in pixels.
   * Default: 0
   *   </li>
   *   <li><code>bottom</code> : Number (optional)
   * The width of the bottom framing element in pixels.
   * Default: 0
   *   </li>
   *   <li><code>left</code> : Number (optional)
   * The width of the left framing element in pixels.
   * Default: 0
   *   </li>
   *   <li><code>width</code> : Number (optional)
   * The total width of the left and right framing elements in pixels.
   * Default: 0
   *   </li>
   *   <li><code>height</code> : Number (optional)
   * The total height of the top and right bottom elements in pixels.
   * Default: 0
   *   </li>
   * </ul>
   */
  public native function get frameSize():Object;

  /**
   * The config object passed to the constructor during Component creation.
   */
  public native function get initialConfig():Object;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Component, or subclass thereof.
   * @default true
   */
  public native function get isComponent():Boolean;

  /**
   * @private
   */
  public native function set isComponent(value:Boolean):void;

  /**
   * This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means
   * that every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab
   * override this property to <code>false</code> since they want to implement custom disable logic.
   * @default true
   */
  public native function get maskOnDisable():Boolean;

  /**
   * @private
   */
  public native function set maskOnDisable(value:Boolean):void;

  /**
   * This Component's owner <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> (is set automatically
   * when this Component is added to a Container).
   *
   * <p><em>Important.</em> This is not a universal upwards navigation pointer. It indicates the Container which owns and manages
   * this Component if any. There are other similar relationships such as the <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> which activates a <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a>, or the
   * <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu item</a> which activated a <a href="#!/api/Ext.menu.Item-cfg-menu" rel="Ext.menu.Item-cfg-menu" class="docClass">submenu</a>, or the
   * <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">column header</a> which activated the column menu.</p>
   *
   * <p>These differences are abstracted away by the <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">up</a> method.</p>
   *
   * <p><strong>Note</strong>: to access items within the Container see <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">itemId</a>.</p>
   * @since 2.3.0
   */
  public native function get ownerCt():ext.container.Container;

  /**
   * Indicates whether or not the component has been rendered.
   * @default false
   * @since 1.1.0
   */
  public native function get rendered():Boolean;

  /**
   * Only present for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components after
   * they have been rendered.
   *
   * <p>A reference to the ZIndexManager which is managing this Component's z-index.</p>
   *
   * <p>The <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a> maintains a stack of floating Component z-indices, and also provides
   * a single modal mask which is insert just beneath the topmost visible modal floating Component.</p>
   *
   * <p>Floating Components may be <a href="#!/api/Ext.Component-method-toFront" rel="Ext.Component-method-toFront" class="docClass">brought to the front</a> or <a href="#!/api/Ext.Component-method-toBack" rel="Ext.Component-method-toBack" class="docClass">sent to the back</a> of the
   * z-index stack.</p>
   *
   * <p>This defaults to the global <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a> for floating Components that are
   * programatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a>.</p>
   *
   * <p>For <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components that are added to a
   * Container, the ZIndexManager is acquired from the first ancestor Container found
   * that is floating. If no floating ancestor is found, the global
   * <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a> is used.</p>
   *
   * <p>See <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> and <a href="#!/api/Ext.util.Floating-property-zIndexParent" rel="Ext.util.Floating-property-zIndexParent" class="docClass">zIndexParent</a></p>
   */
  public native function get zIndexManager():ext.ZIndexManager;

  /**
   * Only present for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> Components which were
   * inserted as child items of Containers, and which have a floating Container in
   * their containment ancestry.
   *
   * <p>For <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> Components which are child items of a
   * Container, the zIndexParent will be a floating ancestor Container which is
   * responsible for the base z-index value of all its floating descendants. It
   * provides a <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a> which provides z-indexing
   * services for all its descendant floating Components.</p>
   *
   * <p>Floating Components that are programmatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a> will not have a <code>zIndexParent</code>
   * property.</p>
   *
   * <p>For example, the dropdown <a href="#!/api/Ext.view.BoundList" rel="Ext.view.BoundList" class="docClass">BoundList</a> of a ComboBox which is in a Window will have the
   * Window as its <code>zIndexParent</code>, and will always show above that Window, wherever the Window is placed in the z-index stack.</p>
   *
   * <p>See <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> and <a href="#!/api/Ext.util.Floating-property-zIndexManager" rel="Ext.util.Floating-property-zIndexManager" class="docClass">zIndexManager</a></p>
   */
  public native function get zIndexParent():ext.container.Container;

  [ExtConfig]
  /**
   * By default, when the <a href="#!/api/Ext.Component-method-alignTo" rel="Ext.Component-method-alignTo" class="docClass">alignTo</a> method is called, a floating component will
   * scroll to keep aligned with the anchoring element if the anchoring element is part of the scroll.
   *
   * <p>If this is not necessary, and the <code>alignTo</code> is a one-off operation then set this config to <code>false</code>.</p>
   * @default true
   */
  public native function get alignOnScroll():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set alignOnScroll(value:Boolean):void;

  [ExtConfig]
  /**
   * A Component or Element by which to position this component according to the <a href="#!/api/Ext.Component-cfg-defaultAlign" rel="Ext.Component-cfg-defaultAlign" class="docClass">defaultAlign</a>.
   * Defaults to the owning Container.
   *
   * <p><em>Only applicable if this component is <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a></em></p>
   *
   * <p><em>Used upon first show</em>.</p>
   */
  public native function get alignTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set alignTarget(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A flag indicating that this component should be on the top of the z-index stack for use by the <a href="#!/api/Ext.util.Floating-property-zIndexManager" rel="Ext.util.Floating-property-zIndexManager" class="docClass">zIndexManager</a>
   * to sort its stack.
   *
   * <p>This may be a positive number to prioritize the ordering of multiple visible always on top components.</p>
   *
   * <p>This may be set to a <em>negative</em> number to prioritize a component to the <em>bottom</em> of the z-index stack.</p>
   * @default false
   * @see #getAlwaysOnTop()
   * @see #setAlwaysOnTop()
   */
  public native function get alwaysOnTop():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set alwaysOnTop(value:*):void;

  [ExtConfig]
  /**
   * This configuration option is to be applied to <strong>child <code>items</code></strong> of a container managed
   * by an <a href="#!/api/Ext.layout.container.Anchor" rel="Ext.layout.container.Anchor" class="docClass">Anchor Layout</a>.
   *
   * <p>This value is what tells the layout how an item should be anchored to the container. <code>items</code>
   * added to an AnchorLayout accept an anchoring-specific config property of <strong>anchor</strong> which is a string
   * containing two values: the horizontal anchor value and the vertical anchor value (for example, '100% 50%').
   * The following types of anchor values are supported:</p>
   *
   * <ul>
   * <li><p><strong>Percentage</strong> : Any value between 1 and 100, expressed as a percentage.</p>
   *
   * <p>The first anchor is the percentage width that the item should take up within the container, and the
   * second is the percentage height.  For example:</p>
   *
   * <pre><code>// two values specified
   * anchor: '100% 50%' // render item complete width of the container and
   *                    // 1/2 height of the container
   * // one value specified
   * anchor: '100%'     // the width value; the height will default to auto
   * </code></pre></li>
   * <li><p><strong>Offsets</strong> : Any positive or negative integer value.</p>
   *
   * <p>This is a raw adjustment where the first anchor is the offset from the right edge of the container,
   * and the second is the offset from the bottom edge. For example:</p>
   *
   * <pre><code>// two values specified
   * anchor: '-50 -100' // render item the complete width of the container
   *                    // minus 50 pixels and
   *                    // the complete height minus 100 pixels.
   * // one value specified
   * anchor: '-50'      // anchor value is assumed to be the right offset value
   *                    // bottom offset will default to 0
   * </code></pre></li>
   * <li><p><strong>Sides</strong> : Valid values are <code>right</code> (or <code>r</code>) and <code>bottom</code> (or <code>b</code>).</p>
   *
   * <p>Either the container must have a fixed size or an anchorSize config value defined at render time in
   * order for these to have any effect.</p></li>
   * <li><p><strong>Mixed</strong> :</p>
   *
   * <p>Anchor values can also be mixed as needed.  For example, to render the width offset from the container
   * right edge by 50 pixels and 75% of the container's height use:</p>
   *
   * <pre><code>anchor:   '-50 75%'
   * </code></pre></li>
   * </ul>
   */
  public native function get anchor():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set anchor(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to animate the shadow along with the component while the component is animating.
   * By default the shadow is hidden while the component is animating
   * @default false
   */
  public native function get animateShadow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animateShadow(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing ARIA attributes to be set
   * on this Component's ARIA element. Use this to set the attributes that cannot be
   * determined by the Component's state, such as <code>aria-live</code>, <code>aria-flowto</code>, etc.
   *
   * <p><strong>Note</strong> that this config is only meaningful at the Component rendering time,
   * and setting it after that will do nothing.</p>
   * @default {$value: null, lazy: true}
   * @see #getAriaAttributes()
   * @see #setAriaAttributes()
   */
  public native function get ariaAttributes():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set ariaAttributes(value:Object):void;

  [ExtConfig]
  /**
   * DOM selector for a child element that is to be used
   * as description for this Component, set in <code>aria-describedby</code> attribute.
   * The selector works the same way as <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a>.
   */
  public native function get ariaDescribedBy():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaDescribedBy(value:String):void;

  [ExtConfig]
  /**
   * ARIA label for this Component. It is best to use
   * <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> option instead, because screen readers prefer
   * <code>aria-labelledby</code> attribute to <code>aria-label</code>. <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabel" rel="Ext.mixin.Accessible-cfg-ariaLabel" class="docClass">ariaLabel</a> and
   * <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> config options are mutually exclusive.
   */
  public native function get ariaLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaLabel(value:String):void;

  [ExtConfig]
  /**
   * DOM selector for a child element that is to be used
   * as label for this Component, set in <code>aria-labelledby</code> attribute.
   * If the selector is by <code>#id</code>, the label element can be any existing element,
   * not necessarily a child of the main Component element.
   *
   * <p><a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabelledBy" rel="Ext.mixin.Accessible-cfg-ariaLabelledBy" class="docClass">ariaLabelledBy</a> and <a href="#!/api/Ext.mixin.Accessible-cfg-ariaLabel" rel="Ext.mixin.Accessible-cfg-ariaLabel" class="docClass">ariaLabel</a> config options are
   * mutually exclusive, and <code>ariaLabelledBy</code> has the higher precedence.</p>
   */
  public native function get ariaLabelledBy():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaLabelledBy(value:String):void;

  [ExtConfig]
  /**
   * A tag name or <a href="#!/api/Ext.dom.Helper" rel="Ext.dom.Helper" class="docClass">DomHelper</a> spec used to create the <a href="#!/api/Ext.Component-method-getEl" rel="Ext.Component-method-getEl" class="docClass">Element</a> which will
   * encapsulate this Component.
   *
   * <p>You do not normally need to specify this. For the base classes <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> and
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more
   * complex DOM structure specified by their own <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a>s.</p>
   *
   * <p>This is intended to allow the developer to create application-specific utility Components encapsulated by
   * different DOM elements. Example usage:</p>
   *
   * <pre><code>{
   *     xtype: 'component',
   *     autoEl: {
   *         tag: 'img',
   *         src: 'http://www.example.com/example.jpg'
   *     }
   * }, {
   *     xtype: 'component',
   *     autoEl: {
   *         tag: 'blockquote',
   *         html: 'autoEl is cool!'
   *     }
   * }, {
   *     xtype: 'container',
   *     autoEl: 'ul',
   *     cls: 'ux-unordered-list',
   *     items: {
   *         xtype: 'component',
   *         autoEl: 'li',
   *         html: 'First list item'
   *     }
   * }
   * </code></pre>
   * @since 2.3.0
   */
  public native function get autoEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoEl(value:*):void;

  [ExtConfig]
  /**
   * This config is intended mainly for non-<a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components which may or may not be shown. Instead of using
   * <a href="#!/api/Ext.Component-cfg-renderTo" rel="Ext.Component-cfg-renderTo" class="docClass">renderTo</a> in the configuration, and rendering upon construction, this allows a Component to render itself
   * upon first <em><a href="#!/api/Ext.Component-method-show" rel="Ext.Component-method-show" class="docClass">show</a></em>. If <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> is <code>true</code>, the value of this config is omitted as if it is <code>true</code>.
   *
   * <p>Specify as <code>true</code> to have this Component render to the document body upon first show.</p>
   *
   * <p>Specify as an element, or the ID of an element to have this Component render to a specific element upon first
   * show.</p>
   * @default false
   */
  public native function get autoRender():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoRender(value:*):void;

  [Deprecated(replacement = "scrollable", since = "5.1.0")]
  [ExtConfig]
  /**
   * <code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,
   * <code>false</code> to clip any overflowing content.
   *
   * <p>This should not be combined with <a href="#!/api/Ext.Component-cfg-overflowX" rel="Ext.Component-cfg-overflowX" class="docClass">overflowX</a> or  <a href="#!/api/Ext.Component-cfg-overflowY" rel="Ext.Component-cfg-overflowY" class="docClass">overflowY</a>.</p>
   * @default false
   * @see #setAutoScroll()
   */
  public native function get autoScroll():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoScroll(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to automatically show the component upon creation. This config option may only be used for
   * <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> components or components that use <a href="#!/api/Ext.Component-cfg-autoRender" rel="Ext.Component-cfg-autoRender" class="docClass">autoRender</a>.
   * @default false
   * @since 2.3.0
   */
  public native function get autoShow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoShow(value:Boolean):void;

  [ExtConfig]
  /**
   * The base CSS class to apply to this component's element. This will also be prepended to elements within this
   * component like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and
   * you want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use
   * <code>componentCls</code> to add specific styling for this component.
   * @default 'x-component'
   */
  public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Setting this config option adds or removes data bindings for other configs.
   * For example, to bind the <code>title</code> config:
   *
   * <pre><code> var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'panel',
   *      bind: {
   *          title: 'Hello {user.name}'
   *      }
   *  });
   * </code></pre>
   *
   * <p>To dynamically add bindings:</p>
   *
   * <pre><code> panel.setBind({
   *      title: 'Greetings {user.name}!'
   *  });
   * </code></pre>
   *
   * <p>To remove bindings:</p>
   *
   * <pre><code> panel.setBind({
   *      title: null
   *  });
   * </code></pre>
   *
   * <p>The bind expressions are presented to <code><a href="#!/api/Ext.app.ViewModel-method-bind" rel="Ext.app.ViewModel-method-bind" class="docClass">Ext.app.ViewModel.bind</a></code>. The
   * <code>ViewModel</code> instance is determined by <code>lookupViewModel</code>.</p>
   * @default {$value: null, lazy: true}
   * @see #getBind()
   * @see #setBind()
   */
  public native function get bind():Object;

  [ExtConfig(extractXType)]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set bind(value:Object):void;

  [ExtConfig]
  /**
   * Specifies the border size for this component. The border can be a single numeric value to apply to all sides or it can
   * be a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).
   *
   * <p>For components that have no border by default, setting this won't make the border appear by itself.
   * You also need to specify border color and style:</p>
   *
   * <pre><code>border: 5,
   * style: {
   *     borderColor: 'red',
   *     borderStyle: 'solid'
   * }
   * </code></pre>
   *
   * <p>To turn off the border, use <code>border: false</code>.</p>
   * @see #setBorder()
   */
  public native function get border():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set border(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The canonical form of <code>childEls</code> is an object keyed by child's property name
   * with values that are objects with the following properties.
   *
   * <ul>
   * <li><code>itemId</code> - The id to combine with the Component's id that is the id of the
   * child element.</li>
   * <li><code>id</code> - The id of the child element.</li>
   * <li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This
   * should be set on things like the generated content for an <code><a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">Ext.view.View</a></code>.</li>
   * <li><code>select</code>: A selector that will be passed to <a href="#!/api/Ext.dom.Element-method-select" rel="Ext.dom.Element-method-select" class="docClass">Ext.dom.Element.select</a>.</li>
   * <li><code>selectNode</code>: A selector that will be passed to <a href="#!/api/Ext.dom.Element-method-selectNode" rel="Ext.dom.Element-method-selectNode" class="docClass">Ext.dom.Element.selectNode</a>.</li>
   * </ul>
   *
   *
   * <p>For example:</p>
   *
   * <pre><code> childEls: {
   *      button: true,
   *      buttonText: 'text',
   *      buttonImage: {
   *          itemId: 'image'
   *      }
   *  }
   * </code></pre>
   *
   * <p>The above is translated into the following complete form:</p>
   *
   * <pre><code> childEls: {
   *      button: {
   *          name: 'button',
   *          itemId: 'button'
   *      },
   *      buttonText: {
   *          name: 'buttonText',
   *          itemId: 'text'
   *      },
   *      buttonImage: {
   *          name: 'buttonImage',
   *          itemId: 'image'
   *      }
   *  }
   * </code></pre>
   *
   * <p>The above can be provided as an array like so:</p>
   *
   * <pre><code> childEls: [
   *      'button',
   *      { name: 'buttonText', itemId: 'text' },
   *      { name: 'buttonImage', itemId: 'image' }
   *  }
   * </code></pre>
   *
   * <p>For example, a Component which renders a title and body text:</p>
   *
   * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     renderTpl: [
   *         '&lt;h1 id="{id}-title" data-ref="title"&gt;{title}&lt;/h1&gt;',
   *         '&lt;p&gt;{msg}&lt;/p&gt;',
   *     ],
   *     renderData: {
   *         title: "Error",
   *         msg: "Something went wrong"
   *     },
   *     childEls: ["title"],
   *     listeners: {
   *         afterrender: function(cmp){
   *             // After rendering the component will have a title property
   *             cmp.title.setStyle({color: "red"});
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p><strong>Note:</strong> <code>childEl</code>s in the <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a>
   * must be referenced in a <strong>data-ref</strong> attribute.  Notice in the above example
   * that the "title" <code>childEl</code> is set in the <code>renderTpl</code> using
   * <strong>data-ref="title"</strong>.</p>
   *
   * <p>When using <code>select</code>, the property will be an instance of <a href="#!/api/Ext.dom.CompositeElement" rel="Ext.dom.CompositeElement" class="docClass">Ext.CompositeElement</a>.
   * In all other cases, the property will be an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or <code>null</code>
   * if not found.</p>
   *
   * <p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.
   * The following issues should be considered:</p>
   *
   * <ul>
   * <li>Performance: using selectors can be 10x slower than id lookup.</li>
   * <li>Over-selecting: selectors are applied after the DOM elements for all children
   * have been rendered, so selectors can match elements from child components
   * (including nested versions of the same component) accidentally.</li>
   * </ul>
   *
   *
   * <p>This above issues are most important when using <code>select</code> since it returns multiple
   * elements.</p>
   * @default {frameTable: {frame: true}, frameTL: {frame: 'tl'}, frameTC: {frame: 'tc'}, frameTR: {frame: 'tr'}, frameML: {frame: 'ml'}, frameBody: {frame: 'mc'}, frameMR: {frame: 'mr'}, frameBL: {frame: 'bl'}, frameBC: {frame: 'bc'}, frameBR: {frame: 'br'}}
   * @see #getChildEls()
   * @see #setChildEls()
   */
  public native function get childEls():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set childEls(value:*):void;

  [ExtConfig]
  /**
   * An optional extra CSS class that will be added to this component's Element.
   * The value can be a string, a list of strings separated by spaces, or an array of strings. This can be useful
   * for adding customized styles to the component or any of its children using standard CSS rules.
   * @default ''
   * @since 1.1.0
   */
  public native function get cls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cls(value:String):void;

  [ExtConfig]
  /**
   * Defines the column width inside <a href="#!/api/Ext.layout.container.Column" rel="Ext.layout.container.Column" class="docClass">column layout</a>.
   *
   * <p>The columnWidth property is always evaluated as a percentage and must be a decimal value greater than 0 and
   * less than 1 (e.g., .25).  See the description at the top of <a href="#!/api/Ext.layout.container.Column" rel="Ext.layout.container.Column" class="docClass">column layout</a> for
   * additional usage details when combining width and columnWidth configs within the layout.</p>
   */
  public native function get columnWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set columnWidth(value:Number):void;

  [ExtConfig]
  /**
   * CSS Class to be added to a components root level element to give distinction to it via styling.
   */
  public native function get componentCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set componentCls(value:String):void;

  [ExtConfig]
  /**
   * The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout
   * manager which sizes a Component's internal structure in response to the Component being sized.
   *
   * <p>Generally, developers will not use this configuration as all provided Components which need their internal
   * elements sizing (Such as <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">input fields</a>) come with their own componentLayout managers.</p>
   *
   * <p>The <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">default layout manager</a> will be used on instances of the base <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>
   * class which simply sizes the Component's encapsulating element to the height and width specified in the
   * <a href="#!/api/Ext.Component-method-setSize" rel="Ext.Component-method-setSize" class="docClass">setSize</a> method.</p>
   * @default 'autocomponent'
   */
  public native function get componentLayout():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set componentLayout(value:*):void;

  [ExtConfig]
  /**
   * True to constrain this Components within its containing element, false to allow it to fall outside of its containing
   * element. By default this Component will be rendered to <code>document.body</code>. To render and constrain this Component within
   * another element specify <a href="#!/api/Ext.Component-cfg-renderTo" rel="Ext.Component-cfg-renderTo" class="docClass">renderTo</a>.
   * @default false
   */
  public native function get constrain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrain(value:Boolean):void;

  [ExtConfig]
  /**
   * A <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> (or an element from which a Region measurement will be read) which is used
   * to constrain the component. Only applies when the component is floating.
   */
  public native function get constrainTo():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainTo(value:*):void;

  [ExtConfig]
  /**
   * An object or a string (in TRBL order) specifying insets from the configured <a href="#!/api/Ext.Component-cfg-constrainTo" rel="Ext.Component-cfg-constrainTo" class="docClass">constrain region</a>
   * within which this component must be constrained when positioning or sizing.
   * example:
   *
   * <p>   constraintInsets: '10 10 10 10' // Constrain with 10px insets from parent</p>
   */
  public native function get constraintInsets():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constraintInsets(value:*):void;

  [ExtConfig]
  /**
   * Specify an existing HTML element, or the <code>id</code> of an existing HTML element to use as the content for this component.
   *
   * <p>This config option is used to take an existing HTML element and place it in the layout element of a new component
   * (it simply moves the specified DOM element <em>after the Component is rendered</em> to use as the content.</p>
   *
   * <p><strong>Notes:</strong></p>
   *
   * <p>The specified HTML element is appended to the layout element of the component <em>after any configured
   * <a href="#!/api/Ext.Component-cfg-html" rel="Ext.Component-cfg-html" class="docClass">HTML</a> has been inserted</em>, and so the document will not contain this element at the time
   * the <a href="#!/api/Ext.Component-event-render" rel="Ext.Component-event-render" class="docClass">render</a> event is fired.</p>
   *
   * <p>The specified HTML element used will not participate in any <strong><code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code></strong>
   * scheme that the Component may use. It is just HTML. Layouts operate on child
   * <strong><code><a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a></code></strong>.</p>
   *
   * <p>Add either the <code>x-hidden</code> or the <code>x-hidden-display</code> CSS class to prevent a brief flicker of the content before it
   * is rendered to the panel.</p>
   * @since 3.4.0
   */
  public native function get contentEl():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set contentEl(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A string alias, a configuration object or an instance of a <code>ViewController</code> for
   * this container. Sample usage:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.UserController', {
   *     alias: 'controller.user'
   * });
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserContainer', {
   *     extend: 'Ext.container.container',
   *     controller: 'user'
   * });
   * // Or
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserContainer', {
   *     extend: 'Ext.container.container',
   *     controller: {
   *         type: 'user',
   *         someConfig: true
   *     }
   * });
   *
   * // Can also instance at runtime
   * var ctrl = new MyApp.UserController();
   * var view = new UserContainer({
   *     controller: ctrl
   * });
   * </code></pre>
   * @see #getController()
   * @see #setController()
   */
  public native function get controller():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set controller(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The initial set of data to apply to the <code><a href="#!/api/Ext.Component-cfg-tpl" rel="Ext.Component-cfg-tpl" class="docClass">tpl</a></code> to update the content
   * area of the Component.
   * @since 3.4.0
   * @see #getData()
   * @see #setData()
   */
  public native function get data():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set data(value:Object):void;

  [ExtConfig]
  /**
   * The default <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.dom.Element#getAlignToXY</a> anchor position value for this component
   * relative to its <a href="#!/api/Ext.Component-cfg-alignTarget" rel="Ext.Component-cfg-alignTarget" class="docClass">alignTarget</a> (which defaults to its owning Container).
   *
   * <p><em>Only applicable if this component is <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a></em></p>
   *
   * <p><em>Used upon first show</em>.</p>
   * @default "c-c"
   */
  public native function get defaultAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultAlign(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <code>true</code>, this component will be the default scope (this pointer) for events
   * specified with string names so that the scope can be dynamically resolved. The
   * component will automatically become the defaultListenerScope if a
   * <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a> is specified.
   *
   * <p>See the introductory docs for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for some sample
   * usages.</p>
   *
   * <p><strong>NOTE</strong>: This value can only be reliably set at construction time. Setting it
   * after that time may not correctly rewire all of the potentially effected
   * listeners.</p>
   * @default false
   * @see #getDefaultListenerScope()
   * @see #setDefaultListenerScope()
   */
  public native function get defaultListenerScope():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultListenerScope(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to disable the component.
   * @default false
   * @since 2.3.0
   * @see #setDisabled()
   */
  public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * CSS class to add when the Component is disabled.
   * @default 'x-item-disabled'
   */
  public native function get disabledCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabledCls(value:String):void;

  [ExtConfig]
  /**
   * The side of the <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">panel</a> where this component is to be
   * docked when specified in the panel's
   * <a href="#!/api/Ext.panel.Panel-cfg-dockedItems" rel="Ext.panel.Panel-cfg-dockedItems" class="docClass">dockedItems</a> config.
   *
   * <p>Possible values are:</p>
   *
   * <ul>
   * <li>top</li>
   * <li>bottom</li>
   * <li>left</li>
   * <li>right</li>
   * </ul>
   * @see #setDock()
   */
  public native function get dock():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set dock(value:String):void;

  [ExtConfig]
  /**
   * Specify as true to make a <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Component draggable using the Component's encapsulating element as
   * the drag handle.
   *
   * <p>This may also be specified as a config object for the <a href="#!/api/Ext.util.ComponentDragger" rel="Ext.util.ComponentDragger" class="docClass">ComponentDragger</a> which is
   * instantiated to perform dragging.</p>
   *
   * <p>For example to create a Component which may only be dragged around using a certain internal element as the drag
   * handle, use the delegate option:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>({
   *     constrain: true,
   *     floating: true,
   *     style: {
   *         backgroundColor: '#fff',
   *         border: '1px solid black'
   *     },
   *     html: '&lt;h1 style="cursor:move"&gt;The title&lt;/h1&gt;&lt;p&gt;The content&lt;/p&gt;',
   *     draggable: {
   *         delegate: 'h1'
   *     }
   * }).show();
   * </code></pre>
   * @default false
   */
  public native function get draggable():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set draggable(value:*):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to have this Component fixed at its <code>X, Y</code> coordinates in the browser viewport, immune
   * to scrolling the document.
   * @default false
   */
  public native function get fixed():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fixed(value:Boolean):void;

  [ExtConfig]
  /**
   * Flex may be applied to <strong>child items</strong> of a box layout (<a href="#!/api/Ext.layout.container.VBox" rel="Ext.layout.container.VBox" class="docClass">vbox</a> or
   * <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">hbox</a>). Each child item with a flex property will
   * fill space (horizontally in <code>hbox</code>, vertically in <code>vbox</code>) according to that item's
   * <strong>relative</strong> flex value compared to the sum of all items with a flex value specified.
   *
   * <p>Any child items that have either a <code>flex</code> of <code>0</code> or <code>undefined</code>
   * will not be 'flexed' (the initial size will not be changed).</p>
   */
  public native function get flex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set flex(value:Number):void;

  [ExtConfig]
  /**
   * Specify as true to float the Component outside of the document flow using CSS absolute positioning.
   *
   * <p>Components such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s and <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Menu</a>s are floating by default.</p>
   *
   * <p>Floating Components that are programmatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a> will register
   * themselves with the global <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a></p>
   *
   * <h3>Floating Components as child items of a Container</h3>
   *
   * <p>A floating Component may be used as a child item of a Container. This just allows the floating Component to seek
   * a ZIndexManager by examining the ownerCt chain.</p>
   *
   * <p>When configured as floating, Components acquire, at render time, a <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a> which
   * manages a stack of related floating Components. The ZIndexManager sorts its stack according to
   * an incrementing access counter and the <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">alwaysOnTop</a> config when the Component's <a href="#!/api/Ext.Component-method-toFront" rel="Ext.Component-method-toFront" class="docClass">toFront</a> method is called.</p>
   *
   * <p>The ZIndexManager is found by traversing up the <a href="#!/api/Ext.Component-property-ownerCt" rel="Ext.Component-property-ownerCt" class="docClass">ownerCt</a> chain to find an ancestor which itself is
   * floating. This is so that descendant floating Components of floating <em>Containers</em> (Such as a ComboBox dropdown
   * within a Window) can have its zIndex managed relative to any siblings, but always <strong>above</strong> that floating
   * ancestor Container.</p>
   *
   * <p>If no floating ancestor is found, a floating Component registers itself with the default <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a>.</p>
   *
   * <p>Floating components <em>do not participate in the Container's layout</em>. Because of this, they are not rendered until
   * you explicitly <a href="#!/api/Ext.Component-method-show" rel="Ext.Component-method-show" class="docClass">show</a> them.</p>
   *
   * <p>After rendering, the ownerCt reference is deleted, and the <a href="#!/api/Ext.Component-property-floatParent" rel="Ext.Component-property-floatParent" class="docClass">floatParent</a> property is set to the found
   * floating ancestor Container. If no floating ancestor Container was found the <a href="#!/api/Ext.Component-property-floatParent" rel="Ext.Component-property-floatParent" class="docClass">floatParent</a> property will
   * not be set.</p>
   * @default false
   */
  public native function get floating():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set floating(value:Boolean):void;

  [ExtConfig]
  /**
   * CSS class that will be added to focused
   * Component, and removed when Component blurs.
   * @default 'x-focus'
   */
  public native function get focusCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set focusCls(value:String):void;

  [ExtConfig]
  /**
   * Specifies whether the floated component should be automatically <a href="#!/api/Ext.Component-method-focus" rel="Ext.Component-method-focus" class="docClass">focused</a> when
   * it is <a href="#!/api/Ext.util.Floating-method-toFront" rel="Ext.util.Floating-method-toFront" class="docClass">brought to the front</a>.
   * @default true
   */
  public native function get focusOnToFront():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set focusOnToFront(value:Boolean):void;

  [ExtConfig]
  /**
   * When inside FormPanel, any component configured with <code>formBind: true</code> will
   * be enabled/disabled depending on the validity state of the form.
   * See <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a> for more information and example.
   * @default false
   */
  public native function get formBind():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formBind(value:Boolean):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to have the Component inject framing elements within the Component at render time to provide a
   * graphical rounded frame around the Component content.
   *
   * <p>This is only necessary when running on outdated, or non standard-compliant browsers such as Microsoft's Internet
   * Explorer prior to version 9 which do not support rounded corners natively.</p>
   *
   * <p>The extra space taken up by this framing is available from the read only property <a href="#!/api/Ext.Component-property-frameSize" rel="Ext.Component-property-frameSize" class="docClass">frameSize</a>.</p>
   */
  public native function get frame():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set frame(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The height of this component. A numeric value will be interpreted as the number of
   * pixels; a string value will be treated as a CSS value with units.
   * @see #getHeight()
   * @see #setHeight()
   */
  public native function get height():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set height(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to hide the component.
   * @default false
   * @since 2.3.0
   * @see #setHidden()
   */
  public native function get hidden():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set hidden(value:Boolean):void;

  [ExtConfig]
  /**
   * A String which specifies how this Component's encapsulating DOM element will be hidden. Values may be:
   *
   * <ul>
   * <li><code>'display'</code> : The Component will be hidden using the <code>display: none</code> style.</li>
   * <li><code>'visibility'</code> : The Component will be hidden using the <code>visibility: hidden</code> style.</li>
   * <li><code>'offsets'</code> : The Component will be hidden by absolutely positioning it out of the visible area of the document.
   * This is useful when a hidden Component must maintain measurable dimensions. Hiding using <code>display</code> results in a
   * Component having zero dimensions.</li>
   * </ul>
   * @default 'display'
   * @since 1.1.0
   */
  public native function get hideMode():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideMode(value:String):void;

  [ExtConfig]
  /**
   * An HTML fragment, or a <a href="#!/api/Ext.dom.Helper" rel="Ext.dom.Helper" class="docClass">DomHelper</a> specification to use as the layout element content.
   * The HTML content is added after the component is rendered, so the document will not contain this HTML at the time
   * the <a href="#!/api/Ext.Component-event-render" rel="Ext.Component-event-render" class="docClass">render</a> event is fired. This content is inserted into the body <em>before</em> any configured <a href="#!/api/Ext.Component-cfg-contentEl" rel="Ext.Component-cfg-contentEl" class="docClass">contentEl</a>
   * is appended.
   * @default ''
   * @since 3.4.0
   */
  public native function get html():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set html(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <strong>unique</strong> id of this component instance.
   *
   * <p>Use of this config should be considered carefully as this value must be unique across
   * all existing components. Components created with an <code>id</code> may be accessed globally
   * using <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>.</p>
   *
   * <p>Instead of using assigned ids, consider a <a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a> config and a <a href="#!/api/Ext.Component-cfg-controller" rel="Ext.Component-cfg-controller" class="docClass">ViewController</a>
   * to respond to events and perform processing upon this Component.</p>
   *
   * <p>Alternatively, <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">itemId</a> and <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> can be
   * used to perform selector-based searching for Components analogous to DOM querying.
   * The <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> class contains several helpful
   * <a href="#!/api/Ext.container.Container-method-down" rel="Ext.container.Container-method-down" class="docClass">shortcut methods</a> to query its descendant
   * Components by selector.</p>
   *
   * <p>Note that this <code>id</code> will also be used as the element id for the containing HTML
   * element that is rendered to the page for this component. This allows you to write
   * id-based CSS rules to style the specific instance of this component uniquely, and
   * also to select sub-elements using this component's <code>id</code> as the parent.</p>
   *
   * <p>Defaults to an <a href="#!/api/Ext.Component-method-getId" rel="Ext.Component-method-getId" class="docClass">auto-assigned id</a>.</p>
   *
   * <p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed by
   * (optional) additional letters, underscores, digits or hyphens.</p>
   * @since 1.1.0
   * @see #getId()
   */
  public native function get id():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <strong>unique</strong> id of this component instance within its container. See also the
   * <a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a> config.
   *
   * <p>An <code>itemId</code> can be used as an alternative way to get a reference to a component when no object reference is
   * available. Instead of using an <code><a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a></code> with <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">getCmp</a>, use
   * <code>itemId</code> with <a href="#!/api/Ext.container.Container-method-getComponent" rel="Ext.container.Container-method-getComponent" class="docClass">getComponent</a> which will
   * retrieve <code>itemId</code>'s or <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>'s. Since <code>itemId</code>'s are an index to the container's
   * internal collection, the <code>itemId</code> is scoped locally to the container -- avoiding
   * potential conflicts with <a href="#!/api/Ext.ComponentManager" rel="Ext.ComponentManager" class="docClass">Ext.ComponentManager</a> which requires a <strong>unique</strong>
   * <code><a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a></code> values.</p>
   *
   * <pre><code>var c = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({ //
   *     height: 300,
   *     renderTo: document.body,
   *     layout: 'auto',
   *     items: [{
   *         itemId: 'p1',
   *         title: 'Panel 1',
   *         height: 150
   *     },{
   *         itemId: 'p2',
   *         title: 'Panel 2',
   *         height: 150
   *     }]
   * });
   *
   * p1 = c.getComponent('p1'); // not the same as <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>()
   * console.log(p1);
   * p2 = p1.ownerCt.getComponent('p2'); // reference via a sibling
   * console.log(p2);
   * </code></pre>
   *
   * <p>Also see <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>, <code><a href="#!/api/Ext.container.Container-method-query" rel="Ext.container.Container-method-query" class="docClass">Ext.container.Container.query</a></code>, <code><a href="#!/api/Ext.container.Container-method-down" rel="Ext.container.Container-method-down" class="docClass">Ext.container.Container.down</a></code> and
   * <code><a href="#!/api/Ext.container.Container-method-child" rel="Ext.container.Container-method-child" class="docClass">Ext.container.Container.child</a></code>.</p>
   *
   * <p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed by
   * (optional) additional letters, underscores, digits or hyphens.</p>
   *
   * <p><strong>Note</strong>: to access the container of an item see <a href="#!/api/Ext.Component-property-ownerCt" rel="Ext.Component-property-ownerCt" class="docClass">ownerCt</a>.</p>
   * @since 3.4.0
   * @see #getItemId()
   */
  public native function get itemId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemId(value:String):void;

  [ExtConfig]
  /**
   * An object containing handlers for keyboard events. The property names of this
   * object are the key name and any modifiers. The values of the properties are the
   * descriptors of how to handle each event.
   *
   * <p>The handler descriptor can be simply the handler function (either the
   * literal function or the method name), or it can be an object with these
   * properties:</p>
   *
   * <ul>
   * <li><code>handler</code>: The function or its name to call to handle the event.</li>
   * <li><code>scope</code>: The this pointer context (can be "this" or "controller").</li>
   * <li><code>event</code>: An optional override of the key event to which to listen.</li>
   * </ul>
   *
   *
   * <p><strong>Important:</strong> Calls to <code>setKeyMap</code> do not replace the entire <code>keyMap</code> but
   * instead update the provided mappings. That is, unless <code>null</code> is passed as the
   * value of the <code>keyMap</code> which will clear the <code>keyMap</code> of all entries.</p>
   * <ul>
   *   <li><code>scope</code> : String (optional)
   * The default scope to apply to key handlers
   * which do not specify a scope. This is processed the same way as the scope of
   * listeners. It defaults to the <code>"controller"</code>, but using <code>'this'</code>
   * means that an instance method will be used.
   *   </li>
   * </ul>
   */
  public native function get keyMap():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyMap(value:Object):void;

  [ExtConfig]
  /**
   * Enables or disables processing keys in the <code>keyMap</code>. This value starts as
   * <code>null</code> and if it is <code>null</code> when <code>initKeyMap</code> is called, it will automatically
   * be set to <code>true</code>. Since <code>initKeyMap</code> is called by <code><a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></code> at the
   * proper time, this is not something application code normally handles.
   */
  public native function get keyMapEnabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyMapEnabled(value:Boolean):void;

  [ExtConfig]
  /**
   * The name of the member that should be used to listen for keydown/keypress events.
   * This is intended to be controlled at the class level not per instance.
   * @default 'el'
   */
  public native function get keyMapTarget():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyMapTarget(value:Boolean):void;

  [ExtConfig]
  /**
   * Components that achieve their internal layout results using solely CSS with no JS
   * intervention must set this to true.  This allows the component to opt out of the
   * layout run when used inside certain container layouts such as <a href="#!/api/Ext.layout.container.Form" rel="Ext.layout.container.Form" class="docClass">Form</a> and <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">Auto</a>
   * resulting in a performance gain. The following components currently use liquid
   * layout (<code>liquidLayout: true</code>):
   *
   * <ul>
   * <li>All Form Fields (subclasses of <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a>)</li>
   * <li><a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a></li>
   * </ul>
   *
   *
   * <p>It is important to keep in mind that components using liquidLayout do not fire
   * the following events:</p>
   *
   * <ul>
   * <li><a href="#!/api/Ext.Component-event-resize" rel="Ext.Component-event-resize" class="docClass">resize</a></li>
   * <li><a href="#!/api/Ext.Component-event-boxready" rel="Ext.Component-event-boxready" class="docClass">boxready</a></li>
   * </ul>
   *
   *
   * <p>In addition liquidLayout components do not call the following template methods:</p>
   *
   * <ul>
   * <li><a href="#!/api/Ext.Component-method-afterComponentLayout" rel="Ext.Component-method-afterComponentLayout" class="docClass">afterComponentLayout</a></li>
   * <li><a href="#!/api/Ext.Component-method-onBoxReady" rel="Ext.Component-method-onBoxReady" class="docClass">onBoxReady</a></li>
   * <li><a href="#!/api/Ext.Component-method-onResize" rel="Ext.Component-method-onResize" class="docClass">onResize</a></li>
   * </ul>
   *
   *
   * <p>Any component that needs to fire these events or to have these methods called during
   * its life cycle needs to set <code>liquidLayout</code> to <code>false</code>.  The following example
   * demonstrates how to enable the resize event for a
   * <a href="#!/api/Ext.form.field.TextArea" rel="Ext.form.field.TextArea" class="docClass">TextArea Field</a>:</p>
   *
   * <pre class='inline-example '><code>var win = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *         xtype: 'window',
   *         title: 'Resize This Window!',
   *         height: 100,
   *         width: 200,
   *         layout: 'anchor',
   *         items: [{
   *             xtype: 'textarea',
   *             anchor: '0 0',
   *             liquidLayout: false // allows the textarea to fire "resize"
   *         }]
   *     }),
   *     textfield = win.items.getAt(0);
   *
   * win.show();
   *
   * textfield.on('resize', function(textfield, width, height) {
   *     <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Text Field Resized', 'width: ' + width + ', height: ' + height);
   * });
   * </code></pre>
   *
   * <p>Use caution when setting <code>liquidLayout</code> to <code>false</code> as it carries a performance penalty
   * since it means the layout system must perform expensive DOM reads to determine the
   * Component's size.</p>
   * @default false
   */
  public native function get liquidLayout():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set liquidLayout(value:Boolean):void;

  [ExtConfig]
  /**
   * True to drag the component itself.  Else a lightweight version of the component
   * will be shown (<em>using the component's ghost() method</em>).
   *
   * <p><strong>Note:</strong> This config is only relevant when used with dragging implemented via
   * <a href="#!/api/Ext.util.ComponentDragger" rel="Ext.util.ComponentDragger" class="docClass">Ext.util.ComponentDragger</a>.</p>
   * @default false
   */
  public native function get liveDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set liveDrag(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A configuration object or an instance of a <a href="#!/api/Ext.ComponentLoader" rel="Ext.ComponentLoader" class="docClass">Ext.ComponentLoader</a> to load remote content
   * for this Component.
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *     loader: {
   *         url: 'content.html',
   *         autoLoad: true
   *     },
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
   * });
   * </code></pre>
   * @see #getLoader()
   */
  public native function get loader():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loader(value:*):void;

  [ExtConfig]
  /**
   * Specifies the margin for this component. The margin can be a single numeric value to apply to all sides or it can
   * be a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).
   * @see #setMargin()
   */
  public native function get margin():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set margin(value:*):void;

  [ExtConfig]
  /**
   * Related to the <a href="#!/api/Ext.Component-cfg-childEls" rel="Ext.Component-cfg-childEls" class="docClass">childEls</a> configuration which specifies named properties which correspond to component sub-elements.
   *
   * <p>The name of the element property in this component to mask when masked by a LoadMask.</p>
   *
   * <p>Defaults to <code>null</code> to indicate that Components cannot by default contain a LoadMask, and that any LoadMask should be rendered into the document body.</p>
   *
   * <p>For example, Panels use <code>"el"</code> to indicate that the whole panel should be masked. This could be configured to be
   * <code>"body"</code> so that only the body is masked and toolbars and the header are still mouse-accessible.</p>
   * @default null
   */
  public native function get maskElement():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maskElement(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum value in pixels which this Component will set its height to.
   *
   * <p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>
   * @see #getMaxHeight()
   * @see #setMaxHeight()
   */
  public native function get maxHeight():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxHeight(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum value in pixels which this Component will set its width to.
   *
   * <p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>
   * @see #getMaxWidth()
   * @see #setMaxWidth()
   */
  public native function get maxWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxWidth(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum value in pixels which this Component will set its height to.
   *
   * <p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>
   * @see #getMinHeight()
   * @see #setMinHeight()
   */
  public native function get minHeight():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minHeight(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum value in pixels which this Component will set its width to.
   *
   * <p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>
   * @see #getMinWidth()
   * @see #setMinWidth()
   */
  public native function get minWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minWidth(value:Number):void;

  [ExtConfig]
  /**
   * True to make the floated component modal and mask everything behind it when displayed, false to display it without
   * restricting access to other UI elements.
   * @default false
   */
  public native function get modal():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set modal(value:Boolean):void;

  [ExtConfig]
  /**
   * This config enables binding to your <code><a href="#!/api/Ext.data.Model-cfg-validators" rel="Ext.data.Model-cfg-validators" class="docClass">Ext.data.Model.validators</a></code>. This
   * is only processed by form fields (e.g., <code><a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a></code>) at present, but
   * this setting is inherited and so can be set on a parent container.
   *
   * <p>When set to <code>true</code> by a component or not set by a component but inherited from
   * an ancestor container, <code><a href="#!/api/Ext.data.Validation" rel="Ext.data.Validation" class="docClass">Ext.data.Validation</a></code> records are used to automatically
   * bind validation results for any form field to which a <code>value</code> is bound.</p>
   *
   * <p>While this config can be set arbitrarily high in the component hierarchy, doing
   * so can create a lot overhead if most of your form fields do not actually rely on
   * <code>validators</code> in your data model.</p>
   *
   * <p>Using this setting for a form that is bound to an <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code> might look
   * like this:</p>
   *
   * <pre><code> {
   *      xtype: 'panel',
   *      modelValidation: true,
   *      items: [{
   *          xtype: 'textfield',
   *          bind: '{theUser.firstName}'
   *      },{
   *          xtype: 'textfield',
   *          bind: '{theUser.lastName}'
   *      },{
   *          xtype: 'textfield',
   *          bind: '{theUser.phoneNumber}'
   *      },{
   *          xtype: 'textfield',
   *          bind: '{theUser.email}'
   *      }]
   *  }
   * </code></pre>
   *
   * <p>The above is equivalent to the following manual binding of validation:</p>
   *
   * <pre><code> {
   *      xtype: 'panel',
   *      items: [{
   *          xtype: 'textfield',
   *          bind: {
   *              value:      '{theUser.firstName}'
   *              validation: '{theUser.validation.firstName}'
   *          }
   *      },{
   *          xtype: 'textfield',
   *          bind: {
   *              value:      '{theUser.lastName}'
   *              validation: '{theUser.validation.lastName}'
   *          }
   *      },{
   *          xtype: 'textfield',
   *          bind: {
   *              value:      '{theUser.phoneNumber}'
   *              validation: '{theUser.validation.phoneNumber}'
   *          }
   *      },{
   *          xtype: 'textfield',
   *          bind: {
   *              value:      '{theUser.email}'
   *              validation: '{theUser.validation.email}'
   *          }
   *      }]
   *  }
   * </code></pre>
   *
   * <p>Notice that "validation" is a pseudo-association defined for all entities. See
   * <code><a href="#!/api/Ext.data.Model-method-getValidation" rel="Ext.data.Model-method-getValidation" class="docClass">Ext.data.Model.getValidation</a></code> for further details.</p>
   */
  public native function get modelValidation():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set modelValidation(value:Boolean):void;

  [ExtConfig]
  /**
   * An optional extra CSS class that will be added to this component's Element when the mouse moves over the Element,
   * and removed when the mouse moves out. This can be useful for adding customized 'active' or 'hover' styles to the
   * component or any of its children using standard CSS rules.
   * @default ''
   * @since 2.3.0
   */
  public native function get overCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overCls(value:String):void;

  [Deprecated(replacement = "scrollable", since = "5.1.0")]
  [ExtConfig]
  /**
   * Possible values are:
   *
   * <ul>
   * <li><code>'auto'</code> to enable automatic horizontal scrollbar (Style overflow-x: 'auto').</li>
   * <li><code>'scroll'</code> to always enable horizontal scrollbar (Style overflow-x: 'scroll').</li>
   * </ul>
   *
   *
   * <p>The default is overflow-x: 'hidden'. This should not be combined with <a href="#!/api/Ext.Component-cfg-autoScroll" rel="Ext.Component-cfg-autoScroll" class="docClass">autoScroll</a>.</p>
   */
  public native function get overflowX():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overflowX(value:String):void;

  [Deprecated(replacement = "scrollable", since = "5.1.0")]
  [ExtConfig]
  /**
   * Possible values are:
   *
   * <ul>
   * <li><code>'auto'</code> to enable automatic vertical scrollbar (Style overflow-y: 'auto').</li>
   * <li><code>'scroll'</code> to always enable vertical scrollbar (Style overflow-y: 'scroll').</li>
   * </ul>
   *
   *
   * <p>The default is overflow-y: 'hidden'. This should not be combined with <a href="#!/api/Ext.Component-cfg-autoScroll" rel="Ext.Component-cfg-autoScroll" class="docClass">autoScroll</a>.</p>
   */
  public native function get overflowY():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overflowY(value:String):void;

  [ExtConfig]
  /**
   * Specifies the padding for this component. The padding can be a single numeric value to apply to all sides or it
   * can be a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).
   */
  public native function get padding():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set padding(value:*):void;

  [ExtConfig]
  /**
   * An array of plugins to be added to this component. Can also be just a single plugin instead of array.
   *
   * <p>Plugins provide custom functionality for a component. The only requirement for
   * a valid plugin is that it contain an <code>init</code> method that accepts a reference of type <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. When a component
   * is created, if any plugins are available, the component will call the init method on each plugin, passing a
   * reference to itself. Each plugin can then call methods or respond to events on the component as needed to provide
   * its functionality.</p>
   *
   * <p>Plugins can be added to component by either directly referencing the plugin instance:</p>
   *
   * <pre><code>plugins: [<a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">Ext.grid.plugin.CellEditing</a>', {clicksToEdit: 1})],
   * </code></pre>
   *
   * <p>By using config object with ptype:</p>
   *
   * <pre><code>plugins: {ptype: 'cellediting', clicksToEdit: 1},
   * </code></pre>
   *
   * <p>Or with just a ptype:</p>
   *
   * <pre><code>plugins: ['cellediting', 'gridviewdragdrop'],
   * </code></pre>
   *
   * <p>See <a href="#!/api/Ext.enums.Plugin" rel="Ext.enums.Plugin" class="docClass">Ext.enums.Plugin</a> for list of all ptypes.</p>
   * @since 2.3.0
   */
  public native function get plugins():Array;

  [ExtConfig]
  [ArrayElementType("ext.Plugin")]
  /**
   * @private
   */
  public native function set plugins(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more names of config properties that this component should publish
   * to its ViewModel. Generally speaking, only properties defined in a class config
   * block (including ancestor config blocks and mixins) are eligible for publishing
   * to the viewModel. Some components override this and publish their most useful
   * configs by default.
   *
   * <p><strong>Note:</strong> We'll discuss publishing properties <strong>not</strong> found in the config block below.</p>
   *
   * <p>Values determined to be invalid by component (often form fields and model validations)
   * will not be published to the ViewModel.</p>
   *
   * <p>This config uses the <code><a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a></code> to determine the name of the data
   * object to place in the <code>ViewModel</code>. If <code>reference</code> is not set then this config
   * is ignored.</p>
   *
   * <p>By using this config and <code><a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a></code> you can bind configs between
   * components. For example:</p>
   *
   * <pre><code> ...
   *      items: [{
   *          xtype: 'textfield',
   *          reference: 'somefield',  // component's name in the ViewModel
   *          publishes: 'value' // value is not published by default
   *      },{
   *          ...
   *      },{
   *          xtype: 'displayfield',
   *          bind: 'You have entered "{somefield.value}"'
   *      }]
   *  ...
   * </code></pre>
   *
   * <p>Classes must provide this config as an Object:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.foo.Bar', {
   *      publishes: {
   *          foo: true,
   *          bar: true
   *      }
   *  });
   * </code></pre>
   *
   * <p>This is required for the config system to properly merge values from derived
   * classes.</p>
   *
   * <p>For instances this value can be specified as a value as show above or an array
   * or object as follows:</p>
   *
   * <pre><code> {
   *      xtype: 'textfield',
   *      reference: 'somefield',
   *      publishes: [
   *          'value',
   *          'rawValue',
   *          'dirty'
   *      ]
   *  }
   *
   *  // This achieves the same result as the above array form.
   *  {
   *      xtype: 'textfield',
   *      reference: 'somefield',
   *      publishes: {
   *          value: true,
   *          rawValue: true,
   *          dirty: true
   *      }
   *  }
   * </code></pre>
   *
   * <p>In some cases, users may want to publish a property to the viewModel that is not found in a class
   * config block. In these situations, you may utilize <a href="#!/api/Ext.mixin.Bindable-method-publishState" rel="Ext.mixin.Bindable-method-publishState" class="docClass">publishState</a> if the property has a
   * setter method.  Let's use <a href="#!/api/Ext.form.Labelable-method-setFieldLabel" rel="Ext.form.Labelable-method-setFieldLabel" class="docClass">setFieldLabel</a> as an example:</p>
   *
   * <pre><code>  setFieldLabel: function(fieldLabel) {
   *       this.callParent(arguments);
   *       this.publishState('fieldLabel', fieldLabel);
   *   }
   * </code></pre>
   *
   * <p>With the above chunk of code, fieldLabel may now be published to the viewModel.</p>
   * @since 5.0.0
   * @see #getPublishes()
   * @see #setPublishes()
   */
  public native function get publishes():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set publishes(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specifies a name for this component inside its component hierarchy. This name
   * must be unique within its <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">view</a>
   * or its <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">ViewController</a>. See the documentation in
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for more information about references.
   *
   * <p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed
   * by zero or more additional letters, underscores or digits. References are case
   * sensitive.</p>
   * @see #getReference()
   * @see #setReference()
   */
  public native function get reference():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set reference(value:String):void;

  [ExtConfig]
  /**
   * Defines the region inside <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>.
   *
   * <p>Possible values:</p>
   *
   * <ul>
   * <li>north - Positions component at top.</li>
   * <li>south - Positions component at bottom.</li>
   * <li>east - Positions component at right.</li>
   * <li>west - Positions component at left.</li>
   * <li>center - Positions component at the remaining space.
   * There <strong>must</strong> be a component with <code>region: "center"</code> in every border layout.</li>
   * </ul>
   * @default undefined
   * @see #setRegion()
   */
  public native function get region():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set region(value:String):void;

  [ExtConfig]
  /**
   * The data used by <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a> in addition to the following property values of the component:
   *
   * <ul>
   * <li>id</li>
   * <li>ui</li>
   * <li>uiCls</li>
   * <li>baseCls</li>
   * <li>componentCls</li>
   * <li>frame</li>
   * </ul>
   *
   *
   * <p>See <a href="#!/api/Ext.Component-cfg-renderSelectors" rel="Ext.Component-cfg-renderSelectors" class="docClass">renderSelectors</a> and <a href="#!/api/Ext.Component-cfg-childEls" rel="Ext.Component-cfg-childEls" class="docClass">childEls</a> for usage examples.</p>
   */
  public native function get renderData():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderData(value:Object):void;

  [Deprecated(replacement = "childEls", since = "5.0")]
  [ExtConfig]
  /**
   * An object containing properties specifying CSS selectors which identify child elements
   * created by the render process.
   *
   * <p>After the Component's internal structure is rendered according to the <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a>, this object is iterated through,
   * and the found Elements are added as properties to the Component using the <code>renderSelector</code> property name.</p>
   *
   * <p>For example, a Component which renders a title and description into its element:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *      renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *      renderTpl: [
   *          '&lt;h1 class="title"&gt;{title}&lt;/h1&gt;',
   *          '&lt;p&gt;{desc}&lt;/p&gt;'
   *      ],
   *      renderData: {
   *          title: "Error",
   *          desc: "Something went wrong"
   *      },
   *      renderSelectors: {
   *          titleEl: 'h1.title',
   *          descEl: 'p'
   *      },
   *      listeners: {
   *          afterrender: function(cmp){
   *              // After rendering the component will have a titleEl and descEl properties
   *              cmp.titleEl.setStyle({color: "red"});
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>The use of <code>renderSelectors</code> is deprecated (for performance reasons). The above
   * code should be refactored into something like this:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *      renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *      renderTpl: [
   *          '&lt;h1 class="title" id="{id}-titleEl" data-ref="titleEl"&gt;{title}&lt;/h1&gt;',
   *          '&lt;p id="{id}-descEl" data-ref="descEl"&gt;{desc}&lt;/p&gt;'
   *      ],
   *      renderData: {
   *          title: "Error",
   *          desc: "Something went wrong"
   *      },
   *      childEls: [
   *          'titleEl',
   *          'descEl'
   *      ]
   *  });
   * </code></pre>
   *
   * <p>To use <code>childEls</code> yet retain the use of selectors (which remains as expensive as
   * <code>renderSelectors</code>):</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *      renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *      renderTpl: [
   *          '&lt;h1 class="title"&gt;{title}&lt;/h1&gt;',
   *          '&lt;p&gt;{desc}&lt;/p&gt;'
   *      ],
   *      renderData: {
   *          title: "Error",
   *          desc: "Something went wrong"
   *      },
   *      childEls: {
   *          titleEl: { selectNode: 'h1.title' },
   *          descEl: { selectNode: 'p' }
   *      }
   *  });
   * </code></pre>
   */
  public native function get renderSelectors():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderSelectors(value:Object):void;

  [ExtConfig]
  /**
   * Specify the <code>id</code> of the element, a DOM element or an existing Element that this component will be rendered into.
   *
   * <p><strong>Notes:</strong></p>
   *
   * <p>Do <em>not</em> use this option if the Component is to be a child item of a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>.
   * It is the responsibility of the <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>'s
   * <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout manager</a> to render and manage its child items.</p>
   *
   * <p>When using this config, a call to <code>render()</code> is not required.</p>
   *
   * <p>See also: <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">render</a>.</p>
   * @since 2.3.0
   */
  public native function get renderTo():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderTo(value:*):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a> used to create the internal structure inside this Component's encapsulating
   * <a href="#!/api/Ext.Component-method-getEl" rel="Ext.Component-method-getEl" class="docClass">Element</a>.
   *
   * <p>You do not normally need to specify this. For the base classes <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> and
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered
   * with no internal structure; they render their <a href="#!/api/Ext.Component-method-getEl" rel="Ext.Component-method-getEl" class="docClass">Element</a> empty. The more specialized
   * classes with complex DOM structures provide their own template definitions.</p>
   *
   * <p>This is intended to allow the developer to create application-specific utility Components with customized
   * internal structure.</p>
   *
   * <p>Upon rendering, any created child elements may be automatically imported into object properties using the
   * <a href="#!/api/Ext.Component-cfg-renderSelectors" rel="Ext.Component-cfg-renderSelectors" class="docClass">renderSelectors</a> and <a href="#!/api/Ext.Component-cfg-childEls" rel="Ext.Component-cfg-childEls" class="docClass">childEls</a> options.</p>
   * @default '{%this.renderContent(out,values)%}'
   */
  public native function get renderTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderTpl(value:*):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to apply a <a href="#!/api/Ext.resizer.Resizer" rel="Ext.resizer.Resizer" class="docClass">Resizer</a> to this Component after rendering.
   *
   * <p>May also be specified as a config object to be passed to the constructor of <a href="#!/api/Ext.resizer.Resizer" rel="Ext.resizer.Resizer" class="docClass">Resizer</a>
   * to override any defaults. By default the Component passes its minimum and maximum size, and uses
   * <code><a href="#!/api/Ext.resizer.Resizer-cfg-dynamic" rel="Ext.resizer.Resizer-cfg-dynamic" class="docClass">Ext.resizer.Resizer.dynamic</a>: false</code></p>
   */
  public native function get resizable():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set resizable(value:*):void;

  [ExtConfig]
  /**
   * A valid <a href="#!/api/Ext.resizer.Resizer" rel="Ext.resizer.Resizer" class="docClass">Ext.resizer.Resizer</a> handles config string. Only applies when resizable = true.
   * @default 'all'
   */
  public native function get resizeHandles():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set resizeHandles(value:String):void;

  [ExtConfig]
  /**
   * True to layout this component and its descendants in "rtl" (right-to-left) mode.
   * Can be explicitly set to false to override a true value inherited from an ancestor.
   *
   * <p><strong>Defined in override Ext.rtl.Component.</strong></p>
   */
  public native function get rtl():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rtl(value:Boolean):void;

  [ExtConfig]
  /**
   * A buffer to be applied if many state events are fired within a short period.
   * @default 100
   */
  public native function get saveDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set saveDelay(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configuration options to make this Component scrollable. Acceptable values are:
   *
   * <ul>
   * <li><code>true</code> to enable auto scrolling.</li>
   * <li><code>false</code> (or <code>null</code>) to disable scrolling - this is the default.</li>
   * <li><code>x</code> or <code>horizontal</code> to enable horizontal scrolling only</li>
   * <li><code>y</code> or <code>vertical</code> to enable vertical scrolling only</li>
   * </ul>
   *
   *
   * <p>Also accepts a configuration object for a <code><a href="#!/api/Ext.scroll.Scroller" rel="Ext.scroll.Scroller" class="docClass">Ext.scroll.Scroller</a></code> if
   * if advanced configuration is needed.</p>
   *
   * <p>The getter for this config returns the <a href="#!/api/Ext.scroll.Scroller" rel="Ext.scroll.Scroller" class="docClass">Scroller</a>
   * instance.  You can use the Scroller API to read or manipulate the scroll position:</p>
   *
   * <pre><code>// scrolls the component to 5 on the x axis and 10 on the y axis
   * component.getScrollable().scrollTo(5, 10);
   * </code></pre>
   * @see #getScrollable()
   * @see #setScrollable()
   */
  public native function get scrollable():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set scrollable(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If provided this creates a new <code>Session</code> instance for this component. If this
   * is a <code>Container</code>, this will then be inherited by all child components.
   *
   * <p>To create a new session you can specify <code>true</code>:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'viewport',
   *      session: true,
   *
   *      items: [{
   *          ...
   *      }]
   *  });
   * </code></pre>
   *
   * <p>Alternatively, a config object can be provided:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'viewport',
   *      session: {
   *          ...
   *      },
   *
   *      items: [{
   *          ...
   *      }]
   *  });
   * </code></pre>
   * @default null
   * @see #getSession()
   * @see #setSession()
   */
  public native function get session():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set session(value:*):void;

  [ExtConfig]
  /**
   * Specifies whether the floating component should be given a shadow. Set to true to automatically create an
   * <a href="#!/api/Ext.dom.Shadow" rel="Ext.dom.Shadow" class="docClass">Ext.Shadow</a>, or a string indicating the shadow's display <a href="#!/api/Ext.dom.Shadow-cfg-mode" rel="Ext.dom.Shadow-cfg-mode" class="docClass">Ext.Shadow.mode</a>. Set to false to
   * disable the shadow.
   * @default 'sides'
   */
  public native function get shadow():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set shadow(value:*):void;

  [ExtConfig]
  /**
   * Number of pixels to offset the shadow.
   */
  public native function get shadowOffset():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set shadowOffset(value:Number):void;

  [ExtConfig]
  /**
   * <code>true</code> to enable an iframe shim for this Component to keep
   * windowed objects from showing through.
   */
  public native function get shim():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set shim(value:Boolean):void;

  [ExtConfig]
  /**
   * The possible values for shrinkWrap are:
   *
   * <ul>
   * <li>0 (or <code>false</code>): Neither width nor height depend on content.</li>
   * <li>1: Width depends on content (shrink wraps), but height does not.</li>
   * <li>2: Height depends on content (shrink wraps), but width does not.</li>
   * <li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>
   * </ul>
   *
   *
   * <p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed
   * to a block-level element.</p>
   * @default 2
   */
  public native function get shrinkWrap():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set shrinkWrap(value:*):void;

  [ExtConfig]
  /**
   * The unique id for this object to use for state management purposes.
   *
   * <p>See <a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a> for an explanation of saving and restoring state.</p>
   */
  public native function get stateId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stateId(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A flag which causes the object to attempt to restore the state of
   * internal properties from a saved state on startup. The object must have
   * a <a href="#!/api/Ext.state.Stateful-cfg-stateId" rel="Ext.state.Stateful-cfg-stateId" class="docClass">stateId</a> for state to be managed.
   *
   * <p>Auto-generated ids are not guaranteed to be stable across page loads and
   * cannot be relied upon to save and restore the same state for a object.</p>
   *
   * <p>For state saving to work, the state manager's provider must have been
   * set to an implementation of <a href="#!/api/Ext.state.Provider" rel="Ext.state.Provider" class="docClass">Ext.state.Provider</a> which overrides the
   * <a href="#!/api/Ext.state.Provider-method-set" rel="Ext.state.Provider-method-set" class="docClass">set</a> and <a href="#!/api/Ext.state.Provider-method-get" rel="Ext.state.Provider-method-get" class="docClass">get</a>
   * methods to save and recall name/value pairs. A built-in implementation,
   * <a href="#!/api/Ext.state.CookieProvider" rel="Ext.state.CookieProvider" class="docClass">Ext.state.CookieProvider</a> is available.</p>
   *
   * <p>To set the state provider for the current page:</p>
   *
   * <p>   <a href="#!/api/Ext.state.Manager-method-setProvider" rel="Ext.state.Manager-method-setProvider" class="docClass">Ext.state.Manager.setProvider</a>(new <a href="#!/api/Ext.state.CookieProvider" rel="Ext.state.CookieProvider" class="docClass">Ext.state.CookieProvider</a>({
   *        expires: new Date(new Date().getTime()+(1000<em>60</em>60<em>24</em>7)), //7 days from now
   *    }));</p>
   *
   * <p>A stateful object attempts to save state when one of the events
   * listed in the <a href="#!/api/Ext.state.Stateful-cfg-stateEvents" rel="Ext.state.Stateful-cfg-stateEvents" class="docClass">stateEvents</a> configuration fires.</p>
   *
   * <p>To save state, a stateful object first serializes its state by
   * calling <em><a href="#!/api/Ext.state.Stateful-method-getState" rel="Ext.state.Stateful-method-getState" class="docClass">getState</a></em>.</p>
   *
   * <p>The Component base class implements <a href="#!/api/Ext.state.Stateful-method-getState" rel="Ext.state.Stateful-method-getState" class="docClass">getState</a> to save its width and
   * height within the state only if they were initially configured, and have
   * changed from the configured value.</p>
   *
   * <p>The Panel class saves its collapsed state in addition to that.</p>
   *
   * <p>The Grid class saves its column state and store state (sorters and filters and grouper) in addition to its superclass state.</p>
   *
   * <p>If there is more application state to be save, the developer must provide an implementation which
   * first calls the superclass method to inherit the above behaviour, and then injects new properties
   * into the returned object.</p>
   *
   * <p>The value yielded by <code>getState</code> is passed to <a href="#!/api/Ext.state.Manager-method-set" rel="Ext.state.Manager-method-set" class="docClass">Ext.state.Manager.set</a>
   * which uses the configured <a href="#!/api/Ext.state.Provider" rel="Ext.state.Provider" class="docClass">Ext.state.Provider</a> to save the object
   * keyed by the <a href="#!/api/Ext.state.Stateful-cfg-stateId" rel="Ext.state.Stateful-cfg-stateId" class="docClass">stateId</a>.</p>
   *
   * <p>During construction, a stateful object attempts to <em>restore</em> its state by calling
   * <a href="#!/api/Ext.state.Manager-method-get" rel="Ext.state.Manager-method-get" class="docClass">Ext.state.Manager.get</a> passing the <a href="#!/api/Ext.state.Stateful-cfg-stateId" rel="Ext.state.Stateful-cfg-stateId" class="docClass">stateId</a></p>
   *
   * <p>The resulting object is passed to <a href="#!/api/Ext.state.Stateful-method-applyState" rel="Ext.state.Stateful-method-applyState" class="docClass">applyState</a>*. The default implementation of
   * <a href="#!/api/Ext.state.Stateful-method-applyState" rel="Ext.state.Stateful-method-applyState" class="docClass">applyState</a> simply copies properties into the object, but a developer may
   * override this to support restoration of more complex application state.</p>
   *
   * <p>You can perform extra processing on state save and restore by attaching
   * handlers to the <a href="#!/api/Ext.state.Stateful-event-beforestaterestore" rel="Ext.state.Stateful-event-beforestaterestore" class="docClass">beforestaterestore</a>, <a href="#!/api/Ext.state.Stateful-event-staterestore" rel="Ext.state.Stateful-event-staterestore" class="docClass">staterestore</a>,
   * <a href="#!/api/Ext.state.Stateful-event-beforestatesave" rel="Ext.state.Stateful-event-beforestatesave" class="docClass">beforestatesave</a> and <a href="#!/api/Ext.state.Stateful-event-statesave" rel="Ext.state.Stateful-event-statesave" class="docClass">statesave</a> events. In some simple cases,
   * passing an object for the <code>stateful</code> config may suffice. If an object is
   * provided, the properties of that object are used to include or exclude stateful
   * properties returned by <code>getState</code>. For example:</p>
   *
   * <pre><code> stateful: {
   *      height: false, // never persist the height
   *      width: true    // always persist the width
   *  }
   * </code></pre>
   *
   * <p>The above is roughly equivalent to the following:</p>
   *
   * <pre><code> getState: function () {
   *      var state = this.callParent();
   *
   *      delete state.height;
   *      state.width = this.width;
   *
   *      return state;
   *  }
   * </code></pre>
   * @default false
   * @see #getStateful()
   * @see #setStateful()
   */
  public native function get stateful():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set stateful(value:*):void;

  [ExtConfig]
  /**
   * A custom style specification to be applied to this component's Element. Should be a valid argument to
   * <a href="#!/api/Ext.dom.Element-method-applyStyles" rel="Ext.dom.Element-method-applyStyles" class="docClass">Ext.dom.Element.applyStyles</a>.
   *
   * <pre><code>new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *     title: 'Some Title',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     width: 400, height: 300,
   *     layout: 'form',
   *     items: [{
   *         xtype: 'textarea',
   *         style: {
   *             width: '95%',
   *             marginBottom: '10px'
   *         }
   *     },
   *     new <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>({
   *         text: 'Send',
   *         minWidth: '100',
   *         style: {
   *             marginBottom: '10px'
   *         }
   *     })
   *     ]
   * });
   * </code></pre>
   * @since 1.1.0
   * @see #setStyle()
   */
  public native function get style():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set style(value:*):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute for this Focusable's
   * focusEl.
   * @see #getTabIndex()
   * @see #setTabIndex()
   */
  public native function get tabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tabIndex(value:Number):void;

  [ExtConfig]
  /**
   * True to automatically call <a href="#!/api/Ext.Component-method-toFront" rel="Ext.Component-method-toFront" class="docClass">toFront</a> when the <a href="#!/api/Ext.Component-method-show" rel="Ext.Component-method-show" class="docClass">show</a> method is called on an already visible,
   * floating component.
   * @default true
   */
  public native function get toFrontOnShow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set toFrontOnShow(value:Boolean):void;

  [ExtConfig]
  /**
   * Emulates the behavior of the CSS
   * touch-action
   * property in a cross-browser compatible manner.
   *
   * <p>Keys in this object are touch action names, and values are <code>false</code> to disable
   * a touch action or <code>true</code> to enable it.  Accepted keys are:</p>
   *
   * <ul>
   * <li><code>panX</code></li>
   * <li><code>panY</code></li>
   * <li><code>pinchZoom</code></li>
   * <li><code>doubleTapZoom</code></li>
   * </ul>
   *
   *
   * <p>All touch actions are enabled (<code>true</code>) by default, so it is usually only necessary
   * to specify which touch actions to disable.  For example, the following disables
   * only horizontal scrolling and pinch-to-zoom on the component's main element:</p>
   *
   * <pre><code>touchAction: {
   *     panX: false,
   *     pinchZoom: false
   * }
   * </code></pre>
   *
   * <p>Touch actions can be specified on child elements using the child element name,
   * for example:</p>
   *
   * <pre><code>// disables horizontal scrolling on the main element, and double-tap-zoom
   * // on the child element named "body"
   * touchAction: {
   *     panY: false
   *     body: {
   *         doubleTapZoom: false
   *     }
   * }
   * </code></pre>
   *
   * <p>The primary motivation for setting the touch-action of an element is to prevent
   * the browser's default handling of a gesture such as pinch-to-zoom, or
   * drag-to-scroll, so that the application can implement its own handling of that
   * gesture on the element.  Suppose, for example, a component has a custom drag
   * handler on its element and wishes to prevent horizontal scrolling of its container
   * while it is being dragged:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *     touchAction: {
   *         panX: false
   *     },
   *     listeners: {
   *         drag: function(e) {
   *             // implement drag logic
   *         }
   *     }
   * });
   * </code></pre>
   */
  public native function get touchAction():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set touchAction(value:Object):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>, <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a> or an array of strings to form an <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>. Used in
   * conjunction with the <code><a href="#!/api/Ext.Component-cfg-data" rel="Ext.Component-cfg-data" class="docClass">data</a></code> and <code><a href="#!/api/Ext.Component-cfg-tplWriteMode" rel="Ext.Component-cfg-tplWriteMode" class="docClass">tplWriteMode</a></code> configurations.
   * @since 3.4.0
   */
  public native function get tpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tpl(value:*):void;

  [ExtConfig]
  /**
   * The Ext.(X)Template method to use when updating the content area of the Component.
   * See <code><a href="#!/api/Ext.XTemplate-method-overwrite" rel="Ext.XTemplate-method-overwrite" class="docClass">Ext.XTemplate.overwrite</a></code> for information on default mode.
   * @default 'overwrite'
   * @since 3.4.0
   */
  public native function get tplWriteMode():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tplWriteMode(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object holds a map of <code>config</code> properties that will update their binding
   * as they are modified. For example, <code>value</code> is a key added by form fields. The
   * form of this config is the same as <code><a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a></code>.
   *
   * <p>This config is defined so that updaters are not created and added for all
   * bound properties since most cannot be modified by the end-user and hence are
   * not appropriate for two-way binding.</p>
   * @see #getTwoWayBindable()
   * @see #setTwoWayBindable()
   */
  public native function get twoWayBindable():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set twoWayBindable(value:*):void;

  [ExtConfig]
  /**
   * A UI style for a component.
   * @default 'default'
   */
  public native function get ui():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ui(value:String):void;

  [ExtConfig]
  /**
   * One or more CSS classes to add to the component's primary element. This config
   * is intended solely for use by the component instantiator (the "user"), not by
   * derived classes.
   *
   * <p>For example:</p>
   *
   * <pre><code> items: [{
   *      xtype: 'button',
   *      userCls: 'my-button'
   *  ...
   *  }]
   * </code></pre>
   */
  public native function get userCls():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set userCls(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <code>ViewModel</code> is a data provider for this component and its children. The
   * data contained in the <code>ViewModel</code> is typically used by adding <code>bind</code> configs
   * to the components that want present or edit this data.
   *
   * <p>When set, the <code>ViewModel</code> is created and links to any inherited <code>viewModel</code>
   * instance from an ancestor container as the "parent". The <code>ViewModel</code> hierarchy,
   * once established, only supports creation or destruction of children. The
   * parent of a <code>ViewModel</code> cannot be changed on the fly.</p>
   *
   * <p>If this is a root-level <code>ViewModel</code>, the data model connection is made to this
   * component's associated <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Data Session</a></code>. This is
   * determined by calling <code>getInheritedSession</code>.</p>
   * @default {$value: null, lazy: true}
   * @see #getViewModel()
   * @see #setViewModel()
   */
  public native function get viewModel():ext.app.ViewModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set viewModel(value:ext.app.ViewModel):void;

  [ExtConfig]
  /**
   * A value to control how Components are laid out in a <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">Border</a> layout or as docked items.
   *
   * <p>In a Border layout, this can control how the regions (not the center) region lay out if the west or east take full height
   * or if the north or south region take full width. Also look at the <a href="#!/api/Ext.layout.container.Border-cfg-regionWeights" rel="Ext.layout.container.Border-cfg-regionWeights" class="docClass">Ext.layout.container.Border.regionWeights</a> on the Border layout. An example to show how you can
   * take control of this is:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Viewport" rel="Ext.container.Viewport" class="docClass">Ext.container.Viewport</a>', {
   *     layout      : 'border',
   *     defaultType : 'panel',
   *     items       : [
   *         {
   *             region : 'north',
   *             title  : 'North',
   *             height : 100
   *         },
   *         {
   *             region : 'south',
   *             title  : 'South',
   *             height : 100,
   *             weight : -25
   *         },
   *         {
   *             region : 'west',
   *             title  : 'West',
   *             width  : 200,
   *             weight : 15
   *         },
   *         {
   *             region : 'east',
   *             title  : 'East',
   *             width  : 200
   *         },
   *         {
   *             region : 'center',
   *             title  : 'center'
   *         }
   *     ]
   * });
   * </code></pre>
   *
   * <p>If docked items, the weight will order how the items are laid out. Here is an example to put a <a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a> above
   * a <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>'s header:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
   *     renderTo    : document.body,
   *     width       : 300,
   *     height      : 300,
   *     title       : 'Panel',
   *     html        : 'Panel Body',
   *     dockedItems : [
   *         {
   *             xtype : 'toolbar',
   *             items : [
   *                 {
   *                     text : 'Save'
   *                 }
   *             ]
   *         },
   *         {
   *             xtype  : 'toolbar',
   *             weight : -10,
   *             items  : [
   *                 {
   *                     text : 'Remove'
   *                 }
   *             ]
   *         }
   *     ]
   * });
   * </code></pre>
   * @see #setWeight()
   */
  public native function get weight():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set weight(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The width of this component. A numeric value will be interpreted as the number of
   * pixels; a string value will be treated as a CSS value with units.
   * @see #getWidth()
   * @see #setWidth()
   */
  public native function get width():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set width(value:*):void;

  [ExtConfig]
  /**
   * <strong>Note:</strong> Only applies to <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> derived classes when used as
   * a config in <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>.
   *
   * <p>This property provides a shorter alternative to creating objects than using a full
   * class name. Using <code>xtype</code> is the most common way to define component instances,
   * especially in a container. For example, the items in a form containing text fields
   * could be created explicitly like so:</p>
   *
   * <pre><code> items: [
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *          fieldLabel: 'Foo'
   *      }),
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *          fieldLabel: 'Bar'
   *      }),
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Number" rel="Ext.form.field.Number" class="docClass">Ext.form.field.Number</a>', {
   *          fieldLabel: 'Num'
   *      })
   *  ]
   * </code></pre>
   *
   * <p>But by using <code>xtype</code>, the above becomes:</p>
   *
   * <pre><code> items: [
   *      {
   *          xtype: 'textfield',
   *          fieldLabel: 'Foo'
   *      },
   *      {
   *          xtype: 'textfield',
   *          fieldLabel: 'Bar'
   *      },
   *      {
   *          xtype: 'numberfield',
   *          fieldLabel: 'Num'
   *      }
   *  ]
   * </code></pre>
   *
   * <p>When the <code>xtype</code> is common to many items, <a href="#!/api/Ext.container.Container-cfg-defaultType" rel="Ext.container.Container-cfg-defaultType" class="docClass">Ext.container.Container.defaultType</a>
   * is another way to specify the <code>xtype</code> for all items that don't have an explicit <code>xtype</code>:</p>
   *
   * <pre><code> defaultType: 'textfield',
   *  items: [
   *      { fieldLabel: 'Foo' },
   *      { fieldLabel: 'Bar' },
   *      { fieldLabel: 'Num', xtype: 'numberfield' }
   *  ]
   * </code></pre>
   *
   * <p>Each member of the <code>items</code> array is now just a "configuration object". These objects
   * are used to create and configure component instances. A configuration object can be
   * manually used to instantiate a component using <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>:</p>
   *
   * <pre><code> var text1 = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *      fieldLabel: 'Foo'
   *  });
   *
   *  // or alternatively:
   *
   *  var text1 = <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>({
   *      xtype: 'textfield',
   *      fieldLabel: 'Foo'
   *  });
   * </code></pre>
   *
   * <p>This conversion of configuration objects into instantiated components is done when
   * a container is created as part of its {Ext.container.AbstractContainer#initComponent}
   * process. As part of the same process, the <code>items</code> array is converted from its raw
   * array form into a <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a> instance.</p>
   *
   * <p>You can define your own <code>xtype</code> on a custom <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">component</a> by specifying
   * the <code>xtype</code> property in <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>. For example:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.PressMeButton', {
   *     extend: '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>',
   *     xtype: 'pressmebutton',
   *     text: 'Press Me'
   * });
   * </code></pre>
   *
   * <p>Care should be taken when naming an <code>xtype</code> in a custom component because there is
   * a single, shared scope for all xtypes. Third part components should consider using
   * a prefix to avoid collisions.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.form.CoolButton', {
   *     extend: '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>',
   *     xtype: 'ux-coolbutton',
   *     text: 'Cool!'
   * });
   * </code></pre>
   *
   * <p>See <a href="#!/api/Ext.enums.Widget" rel="Ext.enums.Widget" class="docClass">Ext.enums.Widget</a> for list of all available xtypes.</p>
   * @since 2.3.0
   */
  public native function get xtype():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set xtype(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The current X position of the DOM element based on page coordinates.
   */
  public native function get x():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set x(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The current Y position of the DOM element based on page coordinates.
   */
  public native function get y():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set y(value:Number):void;

  /**
   * @inheritDoc
   */
  public native function get stateEvents():Array;

  /**
   * @inheritDoc
   */
  public native function set stateEvents(value:Array):void;

  /**
   * @see #getEl()
   */
  public native function get el():ext.dom.Element;

  /**
   * A reference to the object which manages all of the application's localized resources.
   * This is a singleton instance which implements the IResourceManager interface.
   */
  protected native function get resourceManager():mx.resources.IResourceManager;

  /**
   * Destroys the Component. This method <strong>must not</strong> be overridden because Component
   * destruction sequence is conditional; if a <code>beforedestroy</code> handler returns <code>false</code>
   * we must abort destruction.
   *
   * <p>To add extra functionality to destruction time in a subclass, override the
   * <a href="#!/api/Ext.Component-method-doDestroy" rel="Ext.Component-method-doDestroy" class="docClass">doDestroy</a> method.</p>
   * @since 1.1.0
   */
  override public native function destroy(...params):void;

  /**
   * Retrieves the <code>id</code> of this component. Will auto-generate an <code>id</code> if one has not already been set.
   */
  override public native function getId():String;

  [Deprecated(replacement = "addCls", since = "4.1")]
  /**
   * Adds a CSS class to the top level element representing this component.
   * @since 2.3.0
   * @param cls The CSS class name to add.
   * @return Returns the Component to allow method chaining.
   */
  public native function addClass(cls:*):ext.Component;

  /**
   * Adds a CSS class to the top level element representing this component.
   * @param cls The CSS class name to add.
   * @return Returns the Component to allow method chaining.
   */
  public native function addCls(cls:*):ext.Component;

  /**
   * Adds a <code>cls</code> to the <code>uiCls</code> array, which will also call <a href="#!/api/Ext.Component-method-addUIClsToElement" rel="Ext.Component-method-addUIClsToElement" class="docClass">addUIClsToElement</a> and adds to all elements of this
   * component.
   * @param classes A string or an array of strings to add to the <code>uiCls</code>.
   * @param skip <code>true</code> to skip adding it to the class and do it later (via the return).
   */
  public native function addClsWithUI(classes:*, skip:Boolean = false):void;

  /**
   * Add events that will trigger the state to be saved. If the first argument is an
   * array, each element of that array is the name of a state event. Otherwise, each
   * argument passed to this method is the name of a state event.
   * @param events The event name or an array of event names.
   */
  public native function addStateEvents(events:*):void;

  /**
   * Aligns the element with another element relative to the specified anchor points. If
   * the other element is the document it aligns it to the viewport. The position
   * parameter is optional, and can be specified in any one of the following formats:
   *
   * <ul>
   * <li><strong>Blank</strong>: Defaults to aligning the element's top-left corner to the target's
   * bottom-left corner ("tl-bl").</li>
   * <li><strong>One anchor (deprecated)</strong>: The passed anchor position is used as the target
   * element's anchor point.  The element being aligned will position its top-left
   * corner (tl) to that point. <em>This method has been deprecated in favor of the newer
   * two anchor syntax below</em>.</li>
   * <li><strong>Two anchors</strong>: If two values from the table below are passed separated by a dash,
   * the first value is used as the element's anchor point, and the second value is
   * used as the target's anchor point.</li>
   * </ul>
   *
   *
   * <p>In addition to the anchor points, the position parameter also supports the "?"
   * character. If "?" is passed at the end of the position string, the element will
   * attempt to align as specified, but the position will be adjusted to constrain to
   * the viewport if necessary. Note that the element being aligned might be swapped to
   * align to a different position than that specified in order to enforce the viewport
   * constraints. Following are all of the supported anchor positions:</p>
   *
   * <pre>Value  Description
   * -----  -----------------------------
   * tl     The top left corner (default)
   * t      The center of the top edge
   * tr     The top right corner
   * l      The center of the left edge
   * c      In the center of the element
   * r      The center of the right edge
   * bl     The bottom left corner
   * b      The center of the bottom edge
   * br     The bottom right corner
   * </pre>
   *
   *
   * <p>Example Usage:</p>
   *
   * <pre><code>// align el to other-el using the default positioning
   * // ("tl-bl", non-constrained)
   * el.alignTo("other-el");
   *
   * // align the top left corner of el with the top right corner of other-el
   * // (constrained to viewport)
   * el.alignTo("other-el", "tr?");
   *
   * // align the bottom right corner of el with the center left edge of other-el
   * el.alignTo("other-el", "br-l?");
   *
   * // align the center of el with the bottom left corner of other-el and
   * // adjust the x position by -6 pixels (and the y position by 0)
   * el.alignTo("other-el", "c-bl", [-6, 0]);
   * </code></pre>
   *
   * <p><strong>Overridden in Ext.overrides.util.Positionable.</strong></p>
   * @param element The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param position The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * Element animation config object
   */
  public native function alignTo(element:*, position:String = "tl-bl?", offsets:Array = null):void;

  /**
   * Anchors an element to another element and realigns it when the window is resized.
   * @param anchorToEl The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param alignment The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * @param animate true for the default animation or a standard
   * Element animation config object
   * @param monitorScroll True to monitor body scroll and
   * reposition. If this parameter is a number, it is used as the buffer delay in
   * milliseconds.
   * @default 50
   * @param callback The function to call after the animation finishes
   * @return this
   */
  public native function anchorTo(anchorToEl:*, alignment:String = "tl-bl?", offsets:Array = null, animate:* = undefined, monitorScroll:* = 50, callback:Function = null):ext.util.Positionable;

  /**
   * Applies the state to the object. This should be overridden in subclasses to do
   * more complex state operations. By default it applies the state properties onto
   * the current object.
   * @param state The state
   */
  public native function applyState(state:Object):void;

  /**
   * Bubbles up the component/container hierarchy, calling the specified function with each component. The scope
   * (<em>this</em>) of function call will be the scope provided or the current component. The arguments to the function will
   * be the args provided or the current component. If the function returns false at any point, the bubble is stopped.
   * @param fn The function to call
   * @param scope The scope of the function. Defaults to current node.
   * @param args The args to call the function with. Defaults to passing the current component.
   * @return this
   */
  public native function bubble(fn:Function, scope:Object = null, args:Array = null):ext.Component;

  /**
   * Center this Component in its container.
   * @return this
   */
  public native function center():ext.Component;

  /**
   * Clone the current component using the original config values passed into this instance by default.
   * @param overrides A new config containing any properties to override in the cloned version.
   * An id property can be passed on this object, otherwise one will be generated to avoid duplicates.
   * @return clone The cloned copy of this component
   */
  public native function cloneConfig(overrides:Object):ext.Component;

  /**
   * Disable the component.
   * @since 1.1.0
   * @param silent Passing <code>true</code> will suppress the <code>disable</code> event from being fired.
   * @default false
   */
  public native function disable(silent:Boolean = false):void;

  [Deprecated(replacement = "updateLayout", since = "4.1")]
  /**
   * This method needs to be called whenever you change something on this component that
   * requires the Component's layout to be recalculated.
   * @return this
   */
  public native function doComponentLayout():ext.Component;

  /**
   * Moves this floating Component into a constrain region.
   *
   * <p>By default, this Component is constrained to be within the container it was added to, or the element it was
   * rendered to.</p>
   *
   * <p>An alternative constraint may be passed.</p>
   * @param constrainTo The Element or <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a>
   * into which this Component is to be constrained. Defaults to the element into which this floating Component
   * was rendered.
   */
  public native function doConstrain(constrainTo:* = undefined):void;

  /**
   * Enable the component
   * @since 1.1.0
   * @param silent Passing <code>true</code> will suppress the <code>enable</code> event from being fired.
   * @default false
   */
  public native function enable(silent:Boolean = false):void;

  /**
   * Ensures that this component is attached to <code>document.body</code>. If the component was
   * rendered to <a href="#!/api/Ext-method-getDetachedBody" rel="Ext-method-getDetachedBody" class="docClass">Ext.getDetachedBody</a>, then it will be appended to <code>document.body</code>.
   * Any configured position is also restored.
   * @param runLayout True to run the component's layout.
   * @default false
   */
  public native function ensureAttachedToBody(runLayout:Boolean = false):void;

  /**
   * Find a container above this component at any level by a custom function. If the passed function returns true, the
   * container will be returned.
   *
   * <p>See also the <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">up</a> method.</p>
   * @param fn The custom function to call with the arguments (container, this component).
   * @return The first Container for which the custom function returns true
   */
  public native function findParentBy(fn:Function):ext.container.Container;

  /**
   * Find a container above this component at any level by xtype or class
   *
   * <p>See also the <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">up</a> method.</p>
   * @param xtype The xtype string for a component, or the class of the component directly
   * @param shallow False to check whether this Component is descended from the xtype (this is the default), or true to check whether this Component is directly of the specified xtype.
   * @return The first Container which matches the given xtype or class
   */
  public native function findParentByType(xtype:*, shallow:Boolean = false):ext.container.Container;

  /**
   * Retrieves plugin from this component's collection by its <code>ptype</code>.
   *
   * <pre><code>var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     store: {
   *         fields: ['name'],
   *         data: [{
   *             name: 'Scott Pilgrim'
   *         }]
   *     },
   *     columns: [{
   *         header: 'Name',
   *         dataIndex: 'name',
   *         editor: 'textfield',
   *         flex: 1
   *     }],
   *     selType: 'cellmodel',
   *     plugins: {
   *         ptype: 'cellediting',
   *         clicksToEdit: 1,
   *         id: 'myplugin'
   *     },
   *     height: 200,
   *     width: 400,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
   * });
   *
   * grid.findPlugin('cellediting');  // the cellediting plugin
   * </code></pre>
   *
   * <p><strong>Note:</strong> See also <a href="#!/api/Ext.Component-method-getPlugin" rel="Ext.Component-method-getPlugin" class="docClass">getPlugin</a></p>
   * @param ptype The Plugin's <code>ptype</code> as specified by the class's
   * <a href="#!/api/Ext.Class-cfg-alias" rel="Ext.Class-cfg-alias" class="docClass">alias</a> configuration.
   * @return plugin instance or <code>undefined</code> if not found
   */
  public native function findPlugin(ptype:String):ext.plugin.AbstractPlugin;

  /**
   * Try to focus this component.
   *
   * <p>If this component is disabled, a close relation will be targeted for focus instead
   * to keep focus localized for keyboard users.</p>
   * @param selectText If applicable, <code>true</code> to also select all the text in this component, or an array consisting of start and end (defaults to start) position of selection.
   * @param delay Delay the focus this number of milliseconds (true for 10 milliseconds).
   * @param callback Only needed if the <code>delay</code> parameter is used. A function to call upon focus.
   * @param scope Only needed if the <code>delay</code> parameter is used. The scope (<code>this</code> reference) in which to execute the callback.
   * @return The focused Component. Usually <code>this</code> Component. Some Containers may
   * delegate focus to a descendant Component (<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s can do this through their
   * <a href="#!/api/Ext.window.Window-cfg-defaultFocus" rel="Ext.window.Window-cfg-defaultFocus" class="docClass">defaultFocus</a> config option. If this component is disabled, a closely
   * related component will be focused and that will be returned.
   */
  public native function focus(selectText:* = undefined, delay:* = undefined, callback:Function = null, scope:Function = null):ext.Component;

  [Deprecated(replacement = "updateLayout", since = "4.1")]
  /**
   * Updates this component's layout. If this update affects this components <a href="#!/api/Ext.Component-property-ownerCt" rel="Ext.Component-property-ownerCt" class="docClass">ownerCt</a>,
   * that component's <code>updateLayout</code> method will be called to perform the layout instead.
   * Otherwise, just this component (and its child items) will layout.
   * @param options An object with layout options.
   * @param {Boolean} options.defer <code>true</code> if this layout should be deferred.
   * @param {Boolean} options.isRoot <code>true</code> if this layout should be the root of the layout.
   */
  public native function forceComponentLayout(options:Object = null):void;

  /**
   * Returns the current animation if this object has any effects actively running or queued, else returns false.
   * @return Anim if element has active effects, else false
   */
  public native function getActiveAnimation():*;

  /**
   * Gets the x,y coordinates to align this element with another element. See
   * <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignTo</a> for more info on the supported position values.
   * @param element The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param position The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * @return [x, y]
   */
  public native function getAlignToXY(element:*, position:String = "tl-bl?", offsets:Array = null):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">alwaysOnTop</a>.
   */
  public native function getAlwaysOnTop():*;

  /**
   * Gets the x,y coordinates specified by the anchor position on the element.
   * @param anchor The specified anchor position.
   * See <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignTo</a> for details on supported anchor positions.
   * @default 'tl'
   * @param local True to get the local (element top/left-relative) anchor
   * position instead of page coordinates
   * @param size An object containing the size to use for calculating anchor
   * position {width: (target width), height: (target height)} (defaults to the
   * element's current size)
   * @return [x, y] An array containing the element's x and y coordinates
   */
  public native function getAnchorXY(anchor:String = 'tl', local:Boolean = false, size:Object = null):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Accessible-cfg-ariaAttributes" rel="Ext.mixin.Accessible-cfg-ariaAttributes" class="docClass">ariaAttributes</a>.
   */
  public native function getAriaAttributes():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-bind" rel="Ext.mixin.Bindable-cfg-bind" class="docClass">bind</a>.
   */
  public native function getBind():Object;

  /**
   * Return an object defining the area of this Element which can be passed to
   * <a href="#!/api/Ext.util.Positionable-method-setBox" rel="Ext.util.Positionable-method-setBox" class="docClass">setBox</a> to set another Element's size/location to match this element.
   * @param contentBox If true a box for the content of the element is
   * returned.
   * @param local If true the element's left and top relative to its
   * <code>offsetParent</code> are returned instead of page x/y.
   * @return An object in the format
   * <ul>
   *   <li><code>x</code> : Number
   * The element's X position.
   *   </li>
   *   <li><code>y</code> : Number
   * The element's Y position.
   *   </li>
   *   <li><code>width</code> : Number
   * The element's width.
   *   </li>
   *   <li><code>height</code> : Number
   * The element's height.
   *   </li>
   *   <li><code>bottom</code> : Number
   * The element's lower bound.
   *   </li>
   *   <li><code>right</code> : Number
   * The element's rightmost bound.
   * <p>The returned object may also be addressed as an Array where index 0 contains the X
   * position and index 1 contains the Y position. The result may also be used for
   * <a href="#!/api/Ext.util.Positionable-method-setXY" rel="Ext.util.Positionable-method-setXY" class="docClass">setXY</a></p>
   *   </li>
   * </ul>
   */
  public native function getBox(contentBox:Boolean = false, local:Boolean = false):Object;

  /**
   * Returns the value of <a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a>.
   */
  public native function getChildEls():*;

  /**
   * Returns a region object that defines the client area of this element.
   *
   * <p>That is, the area <em>within</em> any scrollbars.</p>
   * @return A Region containing "top, left, bottom, right" properties.
   */
  public native function getClientRegion():ext.util.Region;

  /**
   * Returns the content region of this element for purposes of constraining or clipping floating
   * children.  That is the region within the borders and scrollbars, but not within the padding.
   */
  public native function getConstrainRegion():ext.util.Region;

  /**
   * Returns the <code>[X, Y]</code> vector by which this Positionable's element must be translated to make a best
   * attempt to constrain within the passed constraint. Returns <code>false</code> if the element
   * does not need to be moved.
   *
   * <p>Priority is given to constraining the top and left within the constraint.</p>
   *
   * <p>The constraint may either be an existing element into which the element is to be
   * constrained, or a <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> into which this element is to be
   * constrained.</p>
   *
   * <p>By default, any extra shadow around the element is <strong>not</strong> included in the constrain calculations - the edges
   * of the element are used as the element bounds. To constrain the shadow within the constrain region, set the
   * <code>constrainShadow</code> property on this element to <code>true</code>.</p>
   * @param constrainTo The
   * Positionable, HTMLElement, element id, or Region into which the element is to be
   * constrained.
   * @param proposedPosition A proposed <code>[X, Y]</code> position to test for validity
   * and to produce a vector for instead of using the element's current position
   * @param proposedSize A proposed <code>[width, height]</code> size to constrain
   * instead of using the element's current size
   * @return <strong>If</strong> the element <em>needs</em> to be translated, an <code>[X, Y]</code>
   * vector by which this element must be translated. Otherwise, <code>false</code>.
   */
  public native function getConstrainVector(constrainTo:* = undefined, proposedPosition:Array = null, proposedSize:Array = null):*;

  /**
   * Returns the <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a> instance associated with this
   * component via the <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a> config or <a href="#!/api/Ext.mixin.Bindable-method-setController" rel="Ext.mixin.Bindable-method-setController" class="docClass">setController</a> method.
   * @return Returns this component's ViewController or
   * null if one was not configured
   */
  public native function getController():ext.app.ViewController;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-data" rel="Ext.Component-cfg-data" class="docClass">data</a>.
   * @since 3.4.0
   */
  public native function getData():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-defaultListenerScope" rel="Ext.mixin.Bindable-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.
   */
  public native function getDefaultListenerScope():Boolean;

  /**
   * Retrieves the top level element representing this component.
   * @since 1.1.0
   */
  public native function getEl():ext.dom.Element;

  /**
   * Gets the current height of the component's underlying element.
   */
  public native function getHeight():*;

  /**
   * This method returns an object containing the inherited properties for this instance.
   * @since 5.0.0
   * @param inner Pass <code>true</code> to return <code>inheritedStateInner</code> instead
   * of the normal <code>inheritedState</code> object. This is only needed internally and should
   * not be passed by user code.
   * @default false
   * @return The <code>inheritedState</code> object containing inherited properties.
   */
  public native function getInherited(inner:Boolean = false):Object;

  /**
   * This method returns the value of a config property that may be inherited from some
   * ancestor.
   *
   * <p>In some cases, a config may be explicitly set on a component with the intent of
   * <em>only</em> being presented to its children while that component should act upon the
   * inherited value (see <code>referenceHolder</code> for example). In these cases the <code>skipThis</code>
   * parameter should be specified as <code>true</code>.</p>
   * @since 5.0.0
   * @param property The name of the config property to return.
   * @param skipThis Pass <code>true</code> if the property should be ignored if
   * found on this instance. In other words, <code>true</code> means the property must be inherited
   * and not explicitly set on this instance.
   * @default false
   * @return The value of the requested <code>property</code>.
   */
  public native function getInheritedConfig(property:String, skipThis:Boolean = false):*;

  /**
   * This function takes the position argument passed to onRender and returns a
   * DOM element that you can use in the insertBefore.
   * @param position Index, element id or element you want
   * to put this component before.
   * @return DOM element that you can use in the insertBefore
   */
  public native function getInsertPosition(position:*):js.HTMLElement;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">itemId</a> assigned to this component, or when that
   * is not set, returns the value of <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>.
   */
  public native function getItemId():String;

  /**
   * Gets the <a href="#!/api/Ext.ComponentLoader" rel="Ext.ComponentLoader" class="docClass">Ext.ComponentLoader</a> for this Component.
   * @return The loader instance, null if it doesn't exist.
   */
  public native function getLoader():ext.ComponentLoader;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-maxHeight" rel="Ext.Component-cfg-maxHeight" class="docClass">maxHeight</a>.
   */
  public native function getMaxHeight():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-maxWidth" rel="Ext.Component-cfg-maxWidth" class="docClass">maxWidth</a>.
   */
  public native function getMaxWidth():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-minHeight" rel="Ext.Component-cfg-minHeight" class="docClass">minHeight</a>.
   */
  public native function getMinHeight():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-minWidth" rel="Ext.Component-cfg-minWidth" class="docClass">minWidth</a>.
   */
  public native function getMinWidth():Number;

  /**
   * Returns the offsets of this element from the passed element. The element must both
   * be part of the DOM tree and not have display:none to have page coordinates.
   * @param offsetsTo The Positionable,
   * HTMLElement, or element id to get get the offsets from.
   * @return The XY page offsets (e.g. <code>[100, -200]</code>)
   */
  public native function getOffsetsTo(offsetsTo:*):Array;

  /**
   * Retrieves a plugin from this component's collection by its <code>id</code>.
   *
   * <pre><code>var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     store: {
   *         fields: ['name'],
   *         data: [{
   *             name: 'Scott Pilgrim'
   *         }]
   *     },
   *     columns: [{
   *         header: 'Name',
   *         dataIndex: 'name',
   *         editor: 'textfield',
   *         flex: 1
   *     }],
   *     selType: 'cellmodel',
   *     plugins: {
   *         ptype: 'cellediting',
   *         clicksToEdit: 1,
   *         id: 'myplugin'
   *     },
   *     height: 200,
   *     width: 400,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
   * });
   *
   * grid.getPlugin('myplugin');  // the cellediting plugin
   * </code></pre>
   *
   * <p><strong>Note:</strong> See also <a href="#!/api/Ext.Component-method-findPlugin" rel="Ext.Component-method-findPlugin" class="docClass">findPlugin</a>. Prior to 6.2.0 the plugin had to have a
   * <code><a href="#!/api/Ext.plugin.Abstract-cfg-pluginId" rel="Ext.plugin.Abstract-cfg-pluginId" class="docClass">pluginId</a></code> property but this can now be just
   * <code><a href="#!/api/Ext.plugin.Abstract-cfg-id" rel="Ext.plugin.Abstract-cfg-id" class="docClass">id</a></code>. Both are supported (so plugins with a
   * matching <code>pluginId</code> are still found) but <code>id</code> is preferred.</p>
   * @param id The <code>id</code> set on the plugin config object.
   * @return plugin instance or <code>null</code> if not found
   */
  public native function getPlugin(id:String):ext.plugin.AbstractPlugin;

  /**
   * Gets the current XY position of the component's underlying element.
   * @param local If true the element's left and top are returned instead of page XY.
   * @default false
   * @return The XY position of the element (e.g., [100, 200])
   */
  public native function getPosition(local:Boolean = false):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a>.
   * @since 5.0.0
   */
  public native function getPublishes():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a>.
   */
  public native function getReference():String;

  /**
   * Returns a region object that defines the area of this element.
   * @param contentBox If true a box for the content of the element is
   * returned.
   * @return A Region containing "top, left, bottom, right" properties.
   */
  public native function getRegion(contentBox:Boolean = false):ext.util.Region;

  /**
   * Returns the "x" scroll position for this component.  Only applicable for
   * <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a> components
   */
  public native function getScrollX():Number;

  /**
   * Returns the "y" scroll position for this component.  Only applicable for
   * <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a> components
   */
  public native function getScrollY():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a>.
   */
  public native function getScrollable():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a>.
   */
  public native function getSession():*;

  /**
   * Gets the current size of the component's underlying element.
   * @param contentSize true to get the width/size minus borders and padding
   * @return An object containing the element's size:
   * <ul>
   *   <li><code>width</code> : Number</li>
   *   <li><code>height</code> : Number</li>
   * </ul>
   */
  public native function getSize(contentSize:Boolean = false):Object;

  /**
   * The supplied default state gathering method for the Component class.
   *
   * <p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>
   * state.</p>
   *
   * <p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state
   * to the result if this basic state is to be saved.</p>
   *
   * <p>Note that Component state will only be saved if the Component has a <a href="#!/api/Ext.Component-cfg-stateId" rel="Ext.Component-cfg-stateId" class="docClass">stateId</a> and there as a StateProvider
   * configured for the document.</p>
   */
  public native function getState():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a>.
   */
  public native function getStateful():*;

  /**
   * Return the actual tabIndex for this Focusable.
   * @return tabIndex attribute value
   */
  public native function getTabIndex():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-twoWayBindable" rel="Ext.mixin.Bindable-cfg-twoWayBindable" class="docClass">twoWayBindable</a>.
   */
  public native function getTwoWayBindable():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a>.
   */
  public native function getViewModel():*;

  /**
   * Returns the <strong>content</strong> region of this element. That is the region within the borders
   * and padding.
   * @return A Region containing "top, left, bottom, right" member data.
   */
  public native function getViewRegion():ext.util.Region;

  /**
   * Gets the current width of the component's underlying element.
   */
  public native function getWidth():*;

  /**
   * Gets the xtype for this component as registered with <a href="#!/api/Ext.ComponentManager" rel="Ext.ComponentManager" class="docClass">Ext.ComponentManager</a>. For a list of all available
   * xtypes, see the <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> header. Example usage:
   *
   * <pre><code>var t = new <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>();
   * alert(t.getXType());  // alerts 'textfield'
   * </code></pre>
   * @return The xtype
   */
  public native function getXType():String;

  /**
   * Returns this Component's xtype hierarchy as a slash-delimited string. For a list of all available xtypes, see the
   * <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> header.
   *
   * <p><strong>If using your own subclasses, be aware that a Component must register its own xtype to participate in
   * determination of inherited xtypes.</strong></p>
   *
   * <p>Example usage:</p>
   *
   * <pre class='inline-example '><code>var t = new <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>();
   * alert(t.getXTypes());  // alerts 'component/field/textfield'
   * </code></pre>
   * @since 2.3.0
   * @return The xtype hierarchy string
   */
  public native function getXTypes():String;

  [Deprecated(replacement = "getActiveAnimation", since = "4.0")]
  /**
   * Returns the current animation if this object has any effects actively running or queued, else returns false.
   * @return Anim if element has active effects, else false
   */
  public native function hasActiveFx():*;

  /**
   * Checks if the specified CSS class exists on this element's DOM node.
   * @param className The CSS class to check for.
   * @return <code>true</code> if the class exists, else <code>false</code>.
   */
  public native function hasCls(className:String):Boolean;

  /**
   * Checks if there is currently a specified <code>uiCls</code>.
   * @param cls The <code>cls</code> to check.
   */
  public native function hasUICls(cls:String):void;

  /**
   * Hides this Component, setting it to invisible using the configured <a href="#!/api/Ext.Component-cfg-hideMode" rel="Ext.Component-cfg-hideMode" class="docClass">hideMode</a>.
   * @param animateTarget <strong>only valid for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components
   * such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s or <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">ToolTip</a>s, or regular Components which have
   * been configured with <code>floating: true</code>.</strong>. The target to which the Component should animate while hiding.
   * @default null
   * @param callback A callback function to call after the Component is hidden.
   * @param scope The scope (<code>this</code> reference) in which the callback is executed.
   * Defaults to this Component.
   */
  public native function hide(animateTarget:Object = null, callback:Function = null, scope:Object = null):void;

  /**
   * Tests whether this Component matches a <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a>
   * selector string.
   * @param selector The selector string to test against.
   * @return <code>true</code> if this Component matches the selector.
   */
  public native function matches(selector:String):Boolean;

  /**
   * Determines whether <strong>this Component</strong> is an ancestor of the passed Component.
   * This will return <code>true</code> if the passed Component is anywhere within the subtree
   * beneath this Component.
   * @param possibleDescendant The Component to test for presence
   * within this Component's subtree.
   */
  public native function isAncestor(possibleDescendant:ext.Component):void;

  /**
   * Determines whether this component is the descendant of a passed component.
   * @param ancestor A Component which may contain this Component.
   * @return <code>true</code> if the component is the descendant of the passed component, otherwise <code>false</code>.
   */
  public native function isDescendantOf(ancestor:ext.Component):Boolean;

  /**
   * Method to determine whether this Component is currently disabled.
   * @return the disabled state of this Component.
   */
  public native function isDisabled():Boolean;

  /**
   * Method to determine whether this Component is draggable.
   * @return the draggable state of this component.
   */
  public native function isDraggable():Boolean;

  /**
   * Method to determine whether this Component is droppable.
   * @return the droppable state of this component.
   */
  public native function isDroppable():Boolean;

  /**
   * Method to determine whether this Component is floating.
   * @return the floating state of this component.
   */
  public native function isFloating():Boolean;

  /**
   * Determine if this Focusable can receive focus at this time.
   *
   * <p>Note that Containers can be non-focusable themselves while delegating
   * focus treatment to a child Component; see
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">#defaultFocus</a> for more information.</p>
   * @param deep Optionally determine if the container itself
   * is focusable, or if container's focus is delegated to a child component
   * and that child is focusable.
   * @default false
   * @return True if component is focusable, false if not.
   */
  public native function isFocusable(deep:Boolean = false):Boolean;

  /**
   * Method to determine whether this Component is currently set to hidden.
   * @return the hidden state of this Component.
   */
  public native function isHidden():Boolean;

  /**
   * Returns <code>true</code> if layout is suspended for this component. This can come from direct
   * suspension of this component's layout activity (<a href="#!/api/Ext.container.Container-cfg-suspendLayout" rel="Ext.container.Container-cfg-suspendLayout" class="docClass">Ext.Container.suspendLayout</a>) or if one
   * of this component's containers is suspended.
   * @return <code>true</code> layout of this component is suspended.
   */
  public native function isLayoutSuspended():Boolean;

  /**
   * Returns masked state for this Component.
   * @param deep True to look up this Component's parent masked state.
   * @default false
   * @return True if masked, false otherwise.
   */
  public native function isMasked(deep:Boolean = false):Boolean;

  /**
   * Returns <code>true</code> if this component is visible.
   * @since 1.1.0
   * @param deep Pass <code>true</code> to interrogate the visibility status of all parent Containers to
   * determine whether this Component is truly visible to the user.
   * <p>Generally, to determine whether a Component is hidden, the no argument form is needed. For example when creating
   * dynamically laid out UIs in a hidden Container before showing them.</p>
   * @default false
   * @return <code>true</code> if this component is visible, <code>false</code> otherwise.
   */
  public native function isVisible(deep:Boolean = false):Boolean;

  /**
   * Tests whether or not this Component is of a specific xtype. This can test whether this Component is descended
   * from the xtype (default) or whether it is directly of the xtype specified (<code>shallow = true</code>).
   *
   * <p><strong>If using your own subclasses, be aware that a Component must register its own xtype to participate in
   * determination of inherited xtypes.</strong></p>
   *
   * <p>For a list of all available xtypes, see the <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> header.</p>
   *
   * <p>Example usage:</p>
   *
   * <pre class='inline-example '><code>var t = new <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>();
   * var isText = t.isXType('textfield');        // true
   * var isBoxSubclass = t.isXType('field');       // true, descended from <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a>
   * var isBoxInstance = t.isXType('field', true); // false, not a direct <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> instance
   * </code></pre>
   * @since 2.3.0
   * @param xtype The xtype to check for this Component
   * @param shallow <code>true</code> to check whether this Component is directly of the specified xtype, <code>false</code> to
   * check whether this Component is descended from the xtype.
   * @default false
   * @return <code>true</code> if this component descends from the specified xtype, <code>false</code> otherwise.
   */
  public native function isXType(xtype:String, shallow:Boolean = false):Boolean;

  /**
   * Gets the controller that controls this view. May be a controller that belongs
   * to a view higher in the hierarchy.
   * @since 5.0.1
   * @param skipThis <code>true</code> to not consider the controller directly attached
   * to this view (if it exists).
   * @default false
   * @return The controller. <code>null</code> if no controller is found.
   */
  public native function lookupController(skipThis:Boolean = false):ext.app.ViewController;

  /**
   * Returns the <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Ext.data.Session</a></code> for this instance. This property may come
   * from this instance's <code><a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a></code> or be inherited from this object's parent.
   * @since 5.0.0
   * @param skipThis Pass <code>true</code> to ignore a <code>session</code> configured on
   * this instance and only consider an inherited session.
   * @default false
   */
  public native function lookupSession(skipThis:Boolean = false):ext.data.Session;

  /**
   * Gets a named template instance for this class. See <a href="#!/api/Ext.XTemplate-static-method-getTpl" rel="Ext.XTemplate-static-method-getTpl" class="docClass">Ext.XTemplate.getTpl</a>.
   * @since 6.2.0
   * @param name The name of the property that holds the template.
   * @return The template, <code>null</code> if not found.
   */
  public native function lookupTpl(name:String):ext.XTemplate;

  /**
   * Returns the <code><a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a></code> for this instance. This property may come from this
   * this instance's <code><a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a></code> or be inherited from this object's parent.
   * @since 5.0.0
   * @param skipThis Pass <code>true</code> to ignore a <code>viewModel</code> configured on
   * this instance and only consider an inherited view model.
   * @default false
   */
  public native function lookupViewModel(skipThis:Boolean = false):ext.app.ViewModel;

  /**
   * Masks this component with a semi-opaque layer and makes the contents unavailable to clicks.
   *
   * <p>See <a href="#!/api/Ext.Component-method-unmask" rel="Ext.Component-method-unmask" class="docClass">unmask</a>.</p>
   * @param msg A message to show in the center of the mask layer.
   * @param msgCls A CSS class name to use on the message element in the center of the layer.
   */
  public native function mask(msg:String = null, msgCls:String = null):void;

  /**
   * Move the element relative to its current position.
   * @param direction Possible values are:
   * <ul>
   * <li><code>"l"</code> (or <code>"left"</code>)</li>
   * <li><code>"r"</code> (or <code>"right"</code>)</li>
   * <li><code>"t"</code> (or <code>"top"</code>, or <code>"up"</code>)</li>
   * <li><code>"b"</code> (or <code>"bottom"</code>, or <code>"down"</code>)</li>
   * </ul>
   * @param distance How far to move the element in pixels
   * @param animate true for the default animation or a standard
   * Element animation config object
   */
  public native function move(direction:String, distance:Number, animate:* = undefined):void;

  /**
   * Returns the next node in the Component tree in tree traversal order.
   *
   * <p>Note that this is not limited to siblings, and if invoked upon a node with no matching siblings, will walk the
   * tree to attempt to find a match. Contrast with <a href="#!/api/Ext.Component-method-nextSibling" rel="Ext.Component-method-nextSibling" class="docClass">nextSibling</a>.</p>
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector to filter the following nodes.
   * @return The next node (or the next node which matches the selector).
   * Returns <code>null</code> if there is no matching node.
   */
  public native function nextNode(selector:String = null):ext.Component;

  /**
   * Returns the next sibling of this Component.
   *
   * <p>Optionally selects the next sibling which matches the passed <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector.</p>
   *
   * <p>May also be referred to as <strong><code>next()</code></strong></p>
   *
   * <p>Note that this is limited to siblings, and if no siblings of the item match, <code>null</code> is returned. Contrast with
   * <a href="#!/api/Ext.Component-method-nextNode" rel="Ext.Component-method-nextNode" class="docClass">nextNode</a></p>
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector to filter the following items.
   * @return The next sibling (or the next sibling which matches the selector).
   * Returns <code>null</code> if there is no matching sibling.
   */
  public native function nextSibling(selector:String = null):ext.Component;

  /**
   * Called when this Inheritable is added to a parent
   */
  public native function onInheritedAdd(instanced:Boolean):void;

  /**
   * Called when this inheritable is removed from a parent
   * @param destroying <code>true</code> if this item will be destroyed by it's container
   */
  public native function onInheritedRemove(destroying:Boolean):void;

  /**
   * Returns the previous node in the Component tree in tree traversal order.
   *
   * <p>Note that this is not limited to siblings, and if invoked upon a node with no matching siblings, will walk the
   * tree in reverse order to attempt to find a match. Contrast with <a href="#!/api/Ext.Component-method-previousSibling" rel="Ext.Component-method-previousSibling" class="docClass">previousSibling</a>.</p>
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector to filter the preceding nodes.
   * @return The previous node (or the previous node which matches the selector).
   * Returns <code>null</code> if there is no matching node.
   */
  public native function previousNode(selector:String = null):ext.Component;

  /**
   * Returns the previous sibling of this Component.
   *
   * <p>Optionally selects the previous sibling which matches the passed <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a>
   * selector.</p>
   *
   * <p>May also be referred to as <strong><code>prev()</code></strong></p>
   *
   * <p>Note that this is limited to siblings, and if no siblings of the item match, <code>null</code> is returned. Contrast with
   * <a href="#!/api/Ext.Component-method-previousNode" rel="Ext.Component-method-previousNode" class="docClass">previousNode</a></p>
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector to filter the preceding items.
   * @return The previous sibling (or the previous sibling which matches the selector).
   * Returns <code>null</code> if there is no matching sibling.
   */
  public native function previousSibling(selector:String = null):ext.Component;

  /**
   * Called by Component#doAutoRender
   *
   * <p>Register a Container configured <code>floating: true</code> with this Component's <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a>.</p>
   *
   * <p>Components added in this way will not participate in any layout, but will be rendered
   * upon first show in the way that <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s are.</p>
   */
  public native function registerFloatingItem(cmp:Object):void;

  /**
   * Remove any anchor to this element. See <a href="#!/api/Ext.util.Positionable-method-anchorTo" rel="Ext.util.Positionable-method-anchorTo" class="docClass">anchorTo</a>.
   * @return this
   */
  public native function removeAnchor():ext.util.Positionable;

  [Deprecated(replacement = "addCls", since = "4.1")]
  /**
   * Removes a CSS class from the top level element representing this component.
   * @since 2.3.0
   * @param cls The CSS class name to remove.
   * @return Returns the Component to allow method chaining.
   */
  public native function removeClass(cls:*):ext.Component;

  /**
   * Removes a CSS class from the top level element representing this component.
   * @param cls The CSS class name to remove.
   * @return Returns the Component to allow method chaining.
   */
  public native function removeCls(cls:*):ext.Component;

  /**
   * Removes a <code>cls</code> to the <code>uiCls</code> array, which will also call <a href="#!/api/Ext.Component-method-removeUIClsFromElement" rel="Ext.Component-method-removeUIClsFromElement" class="docClass">removeUIClsFromElement</a> and removes it from all
   * elements of this component.
   * @param cls A string or an array of strings to remove to the <code>uiCls</code>.
   */
  public native function removeClsWithUI(cls:*):void;

  /**
   * Renders the Component into the passed HTML element.
   *
   * <p><strong>If you are using a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> object to house this
   * Component, then do not use the render method.</strong></p>
   *
   * <p>A Container's child Components are rendered by that Container's
   * <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a> manager when the Container is first rendered.</p>
   *
   * <p>When creating complex UIs, it is important to remember that sizing and positioning
   * of child items is the responsibility of the Container's <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>
   * manager.  If you expect child items to be sized in response to user interactions, you must
   * configure the Container with a layout manager which creates and manages the type of layout you
   * have in mind.</p>
   *
   * <p><strong>Omitting the Container's <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a> config means that a basic
   * layout manager is used which does nothing but render child components sequentially into the
   * Container. No sizing or positioning will be performed in this situation.</strong></p>
   * @param container The element this Component should be
   * rendered into. If it is being created from existing markup, this should be omitted.
   * @param position The element ID or DOM node index within the container <strong>before</strong>
   * which this component will be inserted (defaults to appending to the end of the container)
   */
  public native function render(container:* = undefined, position:* = undefined):void;

  /**
   * Conditionally saves a single property from this object to the given state object.
   * The idea is to only save state which has changed from the initial state so that
   * current software settings do not override future software settings. Only those
   * values that are user-changed state should be saved.
   * @param propName The name of the property to save.
   * @param state The state object in to which to save the property.
   * @param stateName The name to use for the property in state.
   * @return True if the property was saved, false if not.
   */
  public native function savePropToState(propName:String, state:Object, stateName:String = null):Boolean;

  /**
   * Gathers additional named properties of the instance and adds their current values
   * to the passed state object.
   * @param propNames The name (or array of names) of the property to save.
   * @param state The state object in to which to save the property values.
   * @return state
   */
  public native function savePropsToState(propNames:*, state:Object):Object;

  /**
   * Saves the state of the object to the persistence store.
   */
  public native function saveState():void;

  /**
   * Scrolls this Component by the passed delta values, optionally animating.
   *
   * <p>All of the following are equivalent:</p>
   *
   * <pre><code> comp.scrollBy(10, 10, true);
   *  comp.scrollBy([10, 10], true);
   *  comp.scrollBy({ x: 10, y: 10 }, true);
   * </code></pre>
   * @param deltaX Either the x delta, an Array specifying x and y deltas or
   * an object with "x" and "y" properties.
   * @param deltaY Either the y delta, or an animate flag or config object.
   * @param animate Animate flag/config object if the delta values were passed separately.
   */
  public native function scrollBy(deltaX:*, deltaY:*, animate:*):void;

  /**
   * Scrolls this component to the specified <code>x</code> and <code>y</code> coordinates.  Only applicable
   * for <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a> components.
   * @param animate true for the default animation or a standard Element
   * animation config object
   */
  public native function scrollTo(x:Number, y:Number, animate:* = undefined):void;

  /**
   * Ensures that all effects queued after sequenceFx is called on this object are run in sequence. This is the
   * opposite of <a href="#!/api/Ext.util.Animate-method-syncFx" rel="Ext.util.Animate-method-syncFx" class="docClass">syncFx</a>.
   * @return this
   */
  public native function sequenceFx():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">alwaysOnTop</a>.
   * @param alwaysOnTop The new value.
   */
  public native function setAlwaysOnTop(alwaysOnTop:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Accessible-cfg-ariaAttributes" rel="Ext.mixin.Accessible-cfg-ariaAttributes" class="docClass">ariaAttributes</a>.
   * @param ariaAttributes The new value.
   */
  public native function setAriaAttributes(ariaAttributes:Object):void;

  [Deprecated(replacement = "setScrollable", since = "5.0.0")]
  /**
   * Sets the overflow on the content element of the component.
   * @param scroll True to allow the Component to auto scroll.
   * @return this
   */
  public native function setAutoScroll(scroll:Boolean):ext.Component;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-bind" rel="Ext.mixin.Bindable-cfg-bind" class="docClass">bind</a>.
   * @param bind The new value.
   */
  public native function setBind(bind:Object):void;

  /**
   * @param border The border, see <a href="#!/api/Ext.Component-cfg-border" rel="Ext.Component-cfg-border" class="docClass">border</a>. If a falsey value is passed
   * the border will be removed.
   */
  public native function setBorder(border:*):void;

  /**
   * Sets the element's box. If animate is true then x, y, width, and height will be
   * animated concurrently.
   * @param box The box to fill {x, y, width, height}
   * @param animate true for the default animation or a standard
   * Element animation config object
   */
  public native function setBox(box:Object, animate:* = undefined):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a>.
   * @param childEls The new value.
   */
  public native function setChildEls(childEls:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a>.
   * @param controller The new value.
   */
  public native function setController(controller:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Component-cfg-data" rel="Ext.Component-cfg-data" class="docClass">data</a>.
   * @since 3.4.0
   * @param data The new value.
   */
  public native function setData(data:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-defaultListenerScope" rel="Ext.mixin.Bindable-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.
   * @param defaultListenerScope The new value.
   */
  public native function setDefaultListenerScope(defaultListenerScope:Boolean):void;

  /**
   * Enable or disable the component.
   * @param disabled <code>true</code> to disable.
   */
  public native function setDisabled(disabled:Boolean):void;

  /**
   * Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part
   * of the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)
   * @param dock The dock position.
   * @return this
   */
  public native function setDock(dock:Object):ext.Component;

  [Deprecated(replacement = "setDock", since = "5.0")]
  /**
   * Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part
   * of the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)
   * @param dock The dock position.
   * @return this
   */
  public native function setDocked(dock:Object):ext.Component;

  /**
   * Sets the height of the component. This method fires the <a href="#!/api/Ext.Component-event-resize" rel="Ext.Component-event-resize" class="docClass">resize</a> event.
   * @param height The new height to set. This may be one of:
   * <ul>
   * <li>A Number specifying the new height in pixels.</li>
   * <li>A String used to set the CSS height style.</li>
   * <li><code>undefined</code> to leave the height unchanged.</li>
   * <li><code>null</code> to clear the height.</li>
   * </ul>
   * @return this
   */
  public native function setHeight(height:*):ext.Component;

  /**
   * Sets the hidden state of this component. This is basically the same as
   * <code><a href="#!/api/Ext.Component-method-setVisible" rel="Ext.Component-method-setVisible" class="docClass">setVisible</a></code> but the boolean parameter has the opposite meaning.
   */
  public native function setHidden(hidden:Boolean):ext.Component;

  /**
   * This method allows you to show or hide a LoadMask on top of this component.
   *
   * <p>The mask will be rendered into the element returned by <a href="#!/api/Ext.Component-method-getMaskTarget" rel="Ext.Component-method-getMaskTarget" class="docClass">getMaskTarget</a> which for most Components is the Component's
   * element. See <a href="#!/api/Ext.Component-method-getMaskTarget" rel="Ext.Component-method-getMaskTarget" class="docClass">getMaskTarget</a> and <a href="#!/api/Ext.Component-cfg-maskElement" rel="Ext.Component-cfg-maskElement" class="docClass">maskElement</a>.</p>
   *
   * <p>Most Components will return <code>null</code> indicating that their LoadMask cannot reside inside their element, but must
   * be rendered into the document body.</p>
   *
   * <p><a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">Grid Views</a> however will direct a LoadMask to be rendered into the owning <a href="#!/api/Ext.panel.Table" rel="Ext.panel.Table" class="docClass">GridPanel</a>.</p>
   * @param load True to show the default LoadMask, a config object that will be passed to the
   * LoadMask constructor, or a message String to show. False to hide the current LoadMask.
   * @return The LoadMask instance that has just been shown.
   */
  public native function setLoading(load:*):ext.LoadMask;

  /**
   * Sets the margin on the target element.
   * @param margin The margin to set. See the <a href="#!/api/Ext.Component-cfg-margin" rel="Ext.Component-cfg-margin" class="docClass">margin</a> config.
   */
  public native function setMargin(margin:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Component-cfg-maxHeight" rel="Ext.Component-cfg-maxHeight" class="docClass">maxHeight</a>.
   * @param maxHeight The new value.
   */
  public native function setMaxHeight(maxHeight:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Component-cfg-maxWidth" rel="Ext.Component-cfg-maxWidth" class="docClass">maxWidth</a>.
   * @param maxWidth The new value.
   */
  public native function setMaxWidth(maxWidth:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Component-cfg-minHeight" rel="Ext.Component-cfg-minHeight" class="docClass">minHeight</a>.
   * @param minHeight The new value.
   */
  public native function setMinHeight(minHeight:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Component-cfg-minWidth" rel="Ext.Component-cfg-minWidth" class="docClass">minWidth</a>.
   * @param minWidth The new value.
   */
  public native function setMinWidth(minWidth:Number):void;

  [Deprecated(replacement = "setScrollable", since = "5.0.0")]
  /**
   * Sets the overflow x/y on the content element of the component. The x/y overflow
   * values can be any valid CSS overflow (e.g., 'auto' or 'scroll'). By default, the
   * value is 'hidden'.  Passing <code>undefined</code> will preserve the current value.
   * @param overflowX The overflow-x value.
   * @param overflowY The overflow-y value.
   * @return this
   */
  public native function setOverflowXY(overflowX:String, overflowY:String):ext.Component;

  /**
   * Sets the page XY position of the component. To set the left and top instead, use <a href="#!/api/Ext.Component-method-setPosition" rel="Ext.Component-method-setPosition" class="docClass">setPosition</a>.
   * This method fires the <a href="#!/api/Ext.Component-event-move" rel="Ext.Component-event-move" class="docClass">move</a> event.
   * @param x The new x position or an array of <code>[x,y]</code>.
   * @param y The new y position.
   * @param animate True to animate the Component into its new position. You may also pass an
   * animation configuration.
   * @return this
   */
  public native function setPagePosition(x:Number, y:Number = NaN, animate:* = undefined):ext.Component;

  /**
   * Sets the left and top of the component. To set the page XY position instead, use <a href="#!/api/Ext.Component-method-setPagePosition" rel="Ext.Component-method-setPagePosition" class="docClass">setPagePosition</a>. This
   * method fires the <a href="#!/api/Ext.Component-event-move" rel="Ext.Component-event-move" class="docClass">move</a> event.
   * @param x The new left, an array of <code>[x,y]</code>, or animation config object containing <code>x</code> and <code>y</code> properties.
   * @param y The new top.
   * @param animate If <code>true</code>, the Component is <em>animated</em> into its new position. You may also pass an
   * animation configuration.
   * @return this
   */
  public native function setPosition(x:Number, y:Number = NaN, animate:* = undefined):ext.Component;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a>.
   * @since 5.0.0
   * @param publishes The new value.
   */
  public native function setPublishes(publishes:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a>.
   * @param reference The new value.
   */
  public native function setReference(reference:String):void;

  /**
   * This method changes the <code>region</code> config property for this border region. This is
   * only valid if this component is in a <code>border</code> layout (<code><a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">Ext.layout.container.Border</a></code>).
   *
   * <p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>
   * @param region The new <code>region</code> value (<code>"north"</code>, <code>"south"</code>, <code>"east"</code> or
   * <code>"west"</code>).
   * @return The previous value of the <code>region</code> property.
   */
  public native function setRegion(region:String):String;

  /**
   * Sets the "x" scroll position for this component.  Only applicable for
   * <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a> components
   * @param animate true for the default animation or a standard Element
   * animation config object
   */
  public native function setScrollX(x:Number, animate:* = undefined):void;

  /**
   * Sets the "y" scroll position for this component.  Only applicable for
   * <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a> components
   * @param animate true for the default animation or a standard Element
   * animation config object
   */
  public native function setScrollY(y:Number, animate:* = undefined):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scrollable</a>.
   * @param scrollable The new value.
   */
  public native function setScrollable(scrollable:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a>.
   * @param session The new value.
   */
  public native function setSession(session:*):void;

  /**
   * Sets the width and height of this Component. This method fires the <a href="#!/api/Ext.Component-event-resize" rel="Ext.Component-event-resize" class="docClass">resize</a> event. This method can accept
   * either width and height as separate arguments, or you can pass a size object like <code>{width:10, height:20}</code>.
   * @param width The new width to set. This may be one of:
   * <ul>
   * <li>A Number specifying the new width in pixels.</li>
   * <li>A String used to set the CSS width style.</li>
   * <li>A size object in the format <code>{width: widthValue, height: heightValue}</code>.</li>
   * <li><code>undefined</code> to leave the width unchanged.</li>
   * </ul>
   * @param height The new height to set (not required if a size object is passed as the first arg).
   * This may be one of:
   * <ul>
   * <li>A Number specifying the new height in pixels.</li>
   * <li>A String used to set the CSS height style. Animation may <strong>not</strong> be used.</li>
   * <li><code>undefined</code> to leave the height unchanged.</li>
   * </ul>
   * @return this
   */
  public native function setSize(width:*, height:*):ext.Component;

  /**
   * Sets the value of <a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a>.
   * @param stateful The new value.
   */
  public native function setStateful(stateful:*):void;

  /**
   * Sets the style for this Component's primary element.
   *
   * <p>Styles should be a valid DOM element style property.
   * <a href="http://www.w3schools.com/jsref/dom_obj_style.asp">Valid style property names</a>
   * (<em>along with the supported CSS version for each</em>)</p>
   *
   * <pre><code>var name = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'component',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     html: 'Phineas Flynn'
   * });
   *
   * // two-param syntax
   * name.setStyle('color', 'white');
   *
   * // single-param syntax
   * name.setStyle({
   *     fontWeight: 'bold',
   *     backgroundColor: 'gray',
   *     padding: '10px'
   * });
   * </code></pre>
   * @param property The style property to be set, or an object of
   * multiple styles.
   * @param value The value to apply to the given property, or null if an
   * object was passed.
   * @return this
   */
  public native function setStyle(property:*, value:String = null):ext.Component;

  /**
   * Set the tabIndex property for this Focusable. If the focusEl
   * is available, set tabIndex attribute on it, too.
   * @param newTabIndex new tabIndex to set
   */
  public native function setTabIndex(newTabIndex:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-twoWayBindable" rel="Ext.mixin.Bindable-cfg-twoWayBindable" class="docClass">twoWayBindable</a>.
   * @param twoWayBindable The new value.
   */
  public native function setTwoWayBindable(twoWayBindable:*):void;

  /**
   * Sets the UI for the component. This will remove any existing UIs on the component. It will also loop through any
   * <code>uiCls</code> set on the component and rename them so they include the new UI.
   * @param ui The new UI for the component.
   */
  public native function setUI(ui:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a>.
   * @param viewModel The new value.
   */
  public native function setViewModel(viewModel:*):void;

  /**
   * Convenience function to hide or show this component by Boolean.
   * @since 1.1.0
   * @param visible <code>true</code> to show, <code>false</code> to hide.
   * @return this
   */
  public native function setVisible(visible:Boolean):ext.Component;

  /**
   * Sets the <code>weight</code> config property for this component. This is only valid if this
   * component is in a <code>border</code> layout (<code><a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">Ext.layout.container.Border</a></code>).
   *
   * <p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>
   * @param weight The new <code>weight</code> value.
   * @return The previous value of the <code>weight</code> property.
   */
  public native function setWeight(weight:Number):Number;

  /**
   * Sets the width of the component. This method fires the <a href="#!/api/Ext.Component-event-resize" rel="Ext.Component-event-resize" class="docClass">resize</a> event.
   * @param width The new width to set. This may be one of:
   * <ul>
   * <li>A Number specifying the new width in pixels.</li>
   * <li>A String used to set the CSS width style.</li>
   * <li><code>undefined</code> to leave the width unchanged.</li>
   * <li><code>null</code> to clear the width.</li>
   * </ul>
   * @return this
   */
  public native function setWidth(width:*):ext.Component;

  /**
   * Shows this Component, rendering it first if <a href="#!/api/Ext.Component-cfg-autoRender" rel="Ext.Component-cfg-autoRender" class="docClass">autoRender</a> or <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> are <code>true</code>.
   *
   * <p>After being shown, a <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Component (such as a <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>), is activated it and
   * brought to the front of its <a href="#!/api/Ext.Component-property-zIndexManager" rel="Ext.Component-property-zIndexManager" class="docClass">z-index stack</a>.</p>
   * @param animateTarget <strong>only valid for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s or <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">ToolTip</a>s, or regular Components which have been configured
   * with <code>floating: true</code>.</strong> The target from which the Component should animate from while opening.
   * @default null
   * @param callback A callback function to call after the Component is displayed.
   * Only necessary if animation was specified.
   * @param scope The scope (<code>this</code> reference) in which the callback is executed.
   * Defaults to this Component.
   */
  public native function show(animateTarget:* = null, callback:Function = null, scope:Object = null):void;

  /**
   * Displays component at specific xy position.
   * A floating component (like a menu) is positioned relative to its ownerCt if any.
   * Useful for popping up a context menu:
   *
   * <pre><code>listeners: {
   *     itemcontextmenu: function(view, record, item, index, event, options) {
   *         <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
   *             width: 100,
   *             height: 100,
   *             margin: '0 0 10 0',
   *             items: [{
   *                 text: 'regular item 1'
   *             },{
   *                 text: 'regular item 2'
   *             },{
   *                 text: 'regular item 3'
   *             }]
   *         }).showAt(event.getXY());
   *     }
   * }
   * </code></pre>
   * @param x The new x position or array of <code>[x,y]</code>.
   * @param y The new y position
   * @param animate True to animate the Component into its new position. You may also pass an
   * animation configuration.
   * @return this
   */
  public native function showAt(x:Array, y:Number = NaN, animate:* = undefined):void;

  /**
   * Shows this component by the specified <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a> or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a>.
   * Used when this component is <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a>.
   * @param component The <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> to show the component by.
   * @param position Alignment position as used by <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.util.Positionable.getAlignToXY</a>.
   * Defaults to <code><a href="#!/api/Ext.Component-cfg-defaultAlign" rel="Ext.Component-cfg-defaultAlign" class="docClass">defaultAlign</a></code>. See <a href="#!/api/Ext.Component-method-alignTo" rel="Ext.Component-method-alignTo" class="docClass">alignTo</a> for possible values.
   * @param offsets Alignment offsets as used by <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.util.Positionable.getAlignToXY</a>. See <a href="#!/api/Ext.Component-method-alignTo" rel="Ext.Component-method-alignTo" class="docClass">alignTo</a> for possible values.
   * @return this
   */
  public native function showBy(component:*, position:String = null, offsets:Array = null):void;

  /**
   * Stops any running effects and clears this object's internal effects queue if it contains any additional effects
   * that haven't started yet.
   * @return The Element
   */
  public native function stopAnimation(suppressEvent:* = undefined):ext.dom.Element;

  [Deprecated(replacement = "stopAnimation", since = "4.0")]
  /**
   * Stops any running effects and clears this object's internal effects queue if it contains any additional effects
   * that haven't started yet.
   * @return The Element
   */
  public native function stopFx():ext.dom.Element;

  /**
   * Ensures that all effects queued after syncFx is called on this object are run concurrently. This is the opposite
   * of <a href="#!/api/Ext.util.Animate-method-sequenceFx" rel="Ext.util.Animate-method-sequenceFx" class="docClass">sequenceFx</a>.
   * @return this
   */
  public native function syncFx():Object;

  /**
   * Sends this Component to the back of (lower z-index than) any other visible windows
   * @return this
   */
  public native function toBack():ext.Component;

  /**
   * Brings this floating Component to the front of any other visible, floating Components managed by the same
   * <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a>
   *
   * <p>If this Component is modal, inserts the modal mask just below this Component in the z-index stack.</p>
   * @param preventFocus Specify <code>true</code> to prevent the Component from being focused.
   * @default false
   * @return this
   */
  public native function toFront(preventFocus:Boolean = false):ext.Component;

  /**
   * Toggles the specified CSS class on this component (removes it if it already exists,
   * otherwise adds it).
   * @param className The CSS class to toggle.
   * @param state If specified as <code>true</code>, causes the class to be added. If
   * specified as <code>false</code>, causes the class to be removed.
   * @return Returns the Component to allow method chaining.
   */
  public native function toggleCls(className:String, state:Boolean = false):ext.Component;

  /**
   * Translates the passed page coordinates into left/top css values for the element
   * @param x The page x or an array containing [x, y]
   * @param y The page y, required if x is not an array
   * @return An object with left and top properties. e.g.
   * {left: (value), top: (value)}
   */
  public native function translatePoints(x:*, y:Number = NaN):Object;

  /**
   * Removes the mask applied by <a href="#!/api/Ext.Component-method-mask" rel="Ext.Component-method-mask" class="docClass">mask</a>
   */
  public native function unmask():void;

  /**
   * Navigates up the ownership hierarchy searching for an ancestor Container which matches any passed selector or component.
   *
   * <p><em>Important.</em> There is not a universal upwards navigation pointer. There are several upwards relationships
   * such as the <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> which activates a <a href="#!/api/Ext.button.Button-cfg-menu" rel="Ext.button.Button-cfg-menu" class="docClass">menu</a>, or the
   * <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu item</a> which activated a <a href="#!/api/Ext.menu.Item-cfg-menu" rel="Ext.menu.Item-cfg-menu" class="docClass">submenu</a>, or the
   * <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">column header</a> which activated the column menu.</p>
   *
   * <p>These differences are abstracted away by this method.</p>
   *
   * <p>Example:</p>
   *
   * <pre><code>var owningTabPanel = grid.up('tabpanel');
   * </code></pre>
   * @param selector The selector component or actual component to test. If not passed the immediate owner/activator is returned.
   * @param limit This may be a selector upon which to stop the upward scan, or a limit of the number of steps, or Component reference to stop on.
   * @return The matching ancestor Container (or <code>undefined</code> if no match was found).
   */
  public native function up(selector:* = undefined, limit:* = undefined):ext.container.Container;

  /**
   * Update the content area of a component.
   * @since 3.4.0
   * @param htmlOrData If this component has been configured with a
   * template via the tpl config then it will use this argument as data to populate the
   * template. If this component was not configured with a template, the components
   * content area will be updated via <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> update.
   * @param loadScripts Only legitimate when using the <code>html</code>
   * configuration. Causes embedded script tags to be executed. Inline source will be executed
   * with this Component as the scope (<code>this</code> reference).
   * @default false
   * @param callback Only legitimate when using the <code>html</code> configuration.
   * Callback to execute when scripts have finished loading.
   * @param scriptScope (Default `this`) The scope (<code>this</code> reference) in which to
   * execute <em>inline</em> script elements content. Scripts with a <code>src</code> attribute cannot
   * be executed with this scope.
   * @default `this`
   */
  public native function update(htmlOrData:*, loadScripts:Boolean = false, callback:Function = null, scriptScope:Object = null):void;

  /**
   * Sets the current box measurements of the component's underlying element.
   * @param box An object in the format {x, y, width, height}
   * @return this
   */
  public native function updateBox(box:Object):ext.Component;

  /**
   * Updates this component's layout. If this update affects this components <a href="#!/api/Ext.Component-property-ownerCt" rel="Ext.Component-property-ownerCt" class="docClass">ownerCt</a>,
   * that component's <code>updateLayout</code> method will be called to perform the layout instead.
   * Otherwise, just this component (and its child items) will layout.
   * @param options An object with layout options.
   * @param {Boolean} options.defer <code>true</code> if this layout should be deferred.
   * @param {Boolean} options.isRoot <code>true</code> if this layout should be the root of the layout.
   */
  public native function updateLayout(options:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function getLocalX():Number;

  /**
   * @inheritDoc
   */
  public native function getLocalXY():Array;

  /**
   * @inheritDoc
   */
  public native function getLocalY():Number;

  /**
   * @inheritDoc
   */
  public native function getX():Number;

  /**
   * @inheritDoc
   */
  public native function getXY():Array;

  /**
   * @inheritDoc
   */
  public native function getY():Number;

  /**
   * @inheritDoc
   */
  public native function setLocalX(x:Number):void;

  /**
   * @inheritDoc
   */
  public native function setLocalXY(x:*, y:Number = NaN):void;

  /**
   * @inheritDoc
   */
  public native function setLocalY(y:Number):void;

  /**
   * @inheritDoc
   */
  public native function setX(x:Number, animate:* = undefined):void;

  /**
   * @inheritDoc
   */
  public native function setXY(pos:Array, animate:* = undefined):void;

  /**
   * @inheritDoc
   */
  public native function setY(y:Number, animate:* = undefined):void;

  /**
   * @inheritDoc
   */
  public native function animate(config:Object):Object;

  /**
   * Adds a plugin. May be called at any time in the component's life cycle.
   */
  protected native function addPlugin(plugin:ext.Plugin):void;

  /**
   * Save a property to the given state object if it is not its default or configured
   * value.
   * @param state The state object.
   * @param propName The name of the property on this object to save.
   * @param value The value of the state property (defaults to <code>this[propName]</code>).
   * @return The state object or a new object if state was <code>null</code> and the property
   * was saved.
   */
  protected native function addPropertyToState(state:Object, propName:String, value:String = null):Object;

  /**
   * Method which adds a specified UI + <code>uiCls</code> to the components element. Can be overridden
   * to add the UI to more than just the component's element.
   * @param uiCls The UI class to add to the element.
   */
  protected native function addUIClsToElement(uiCls:String):void;

  /**
   * Called by the layout system after the Component has been laid out.
   *
   * <p>This method is not called on components that use <a href="#!/api/Ext.Component-cfg-liquidLayout" rel="Ext.Component-cfg-liquidLayout" class="docClass">liquidLayout</a>, such as
   * <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Buttons</a> and <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Form Fields</a>.</p>
   * @param width The width that was set
   * @param height The height that was set
   * @param oldWidth The old width, or <code>undefined</code> if this was the initial layout.
   * @param oldHeight The old height, or <code>undefined</code> if this was the initial layout.
   */
  protected native function afterComponentLayout(width:Number, height:Number, oldWidth:*, oldHeight:*):void;

  /**
   * Invoked after the Component has been hidden.
   *
   * <p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href="#!/api/Ext.Component-method-onHide" rel="Ext.Component-method-onHide" class="docClass">onHide</a> received.</p>
   */
  protected native function afterHide(callback:Function = null, scope:Object = null):void;

  /**
   * Allows additional behavior after rendering is complete. At this stage, the
   * <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component's</a> <a href="#!/api/Ext.Component-method-getEl" rel="Ext.Component-method-getEl" class="docClass">Element</a> will have
   * been styled according to the configuration, will have had any configured CSS
   * class names added, and will be in the configured visibility and configured enable
   * state.
   *
   * <p><strong>Note:</strong> If the Component has a <a href="#!/api/Ext.Component-cfg-controller" rel="Ext.Component-cfg-controller" class="docClass">ViewController</a>
   * and the controller has an <a href="#!/api/Ext.app.ViewController-method-afterRender" rel="Ext.app.ViewController-method-afterRender" class="docClass">afterRender</a>
   * method it will be called passing the Component as the single param.</p>
   */
  protected native function afterRender():void;

  /**
   * Template method called after a Component has been positioned.
   */
  protected native function afterSetPosition(x:Number, y:Number):void;

  /**
   * Invoked after the Component is shown (after <a href="#!/api/Ext.Component-method-onShow" rel="Ext.Component-method-onShow" class="docClass">onShow</a> is called).
   *
   * <p>Gets passed the same parameters as <a href="#!/api/Ext.Component-event-show" rel="Ext.Component-event-show" class="docClass">show</a>.</p>
   */
  protected native function afterShow(animateTarget:* = undefined, callback:Function = null, scope:Object = null):void;

  /**
   * Template method to do any pre-blur processing.
   * @param e The event object
   */
  protected native function beforeBlur(e:ext.event.Event):void;

  [Deprecated(since = "6.2.0")]
  /**
   * Invoked before the Component is destroyed.
   * This method is deprecated, override <a href="#!/api/Ext.Component-method-onDestroy" rel="Ext.Component-method-onDestroy" class="docClass">onDestroy</a> instead.
   */
  protected native function beforeDestroy():void;

  /**
   * Template method to do any pre-focus processing.
   * @param e The event object
   */
  protected native function beforeFocus(e:ext.event.Event):void;

  /**
   * Occurs before componentLayout is run. In previous releases, this method could
   * return <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or
   * higher. This method is simply a notification of the impending layout to give the
   * component a chance to adjust the DOM. Ideally, DOM reads should be avoided at this
   * time to reduce expensive document reflows.
   */
  protected native function beforeLayout():void;

  /**
   * Allows additional behavior before rendering.
   *
   * <p><strong>Note:</strong> If the Component has a <a href="#!/api/Ext.Component-cfg-controller" rel="Ext.Component-cfg-controller" class="docClass">ViewController</a>
   * and the controller has a <a href="#!/api/Ext.app.ViewController-method-beforeRender" rel="Ext.app.ViewController-method-beforeRender" class="docClass">beforeRender</a>
   * method it will be called passing the Component as the single param.</p>
   */
  protected native function beforeRender():void;

  /**
   * Invoked before the Component is shown.
   */
  protected native function beforeShow():void;

  /**
   * Cancel any deferred focus on this component
   */
  protected native function cancelFocus():void;

  /**
   * Implements an upward event bubbling policy. By default a Component bubbles events up to its <a href="#!/api/Ext.Component-method-getRefOwner" rel="Ext.Component-method-getRefOwner" class="docClass">reference owner</a>.
   *
   * <p>Component subclasses may implement a different bubbling strategy by overriding this method.</p>
   */
  protected native function getBubbleTarget():ext.Component;

  /**
   * Returns the focus styling holder element associated with this Focusable.
   * By default it is the same element as <a href="#!/api/Ext.util.Focusable-method-getFocusEl" rel="Ext.util.Focusable-method-getFocusEl" class="docClass">getFocusEl</a>.
   * @return The focus styling element.
   */
  protected native function getFocusClsEl():ext.dom.Element;

  /**
   * Returns the focus holder element associated with this Focusable. At the
   * level of the Focusable base, this function returns <code>this.el</code> (or for Widgets,
   * <code>this.element</code>).
   *
   * <p>Subclasses with embedded focusable elements (such as Window, Field and Button)
   * should override this for use by <a href="#!/api/Ext.util.Focusable-method-focus" rel="Ext.util.Focusable-method-focus" class="docClass">focus</a>
   * method.</p>
   */
  protected native function getFocusEl():ext.dom.Element;

  /**
   * Returns the element which is masked by the <a href="#!/api/Ext.Component-method-mask" rel="Ext.Component-method-mask" class="docClass">mask</a> method, or into which the <a href="#!/api/Ext.Component-method-setLoading" rel="Ext.Component-method-setLoading" class="docClass">LoadMask</a> is rendered into.
   *
   * <p>The default implementation uses the <a href="#!/api/Ext.Component-cfg-maskElement" rel="Ext.Component-cfg-maskElement" class="docClass">maskElement</a> configuration to access the Component's child element by name. By default, <a href="#!/api/Ext.Component-cfg-maskElement" rel="Ext.Component-cfg-maskElement" class="docClass">maskElement</a>
   * is <code>null</code> which means that <code>null</code> is returned from this method indicating that the mask needs to be rendered into the document because
   * component structure should not be contaminated by mask elements.</p>
   *
   * <p>Some subclasses may override this method if they have knowledge about external structures where a mask could usefully be rendered.</p>
   *
   * <p>For example a <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a> will request that its owning <a href="#!/api/Ext.panel.Table" rel="Ext.panel.Table" class="docClass">GridPanel</a> be masked. The
   * GridPanel will have its own implementation of <code>getMaskTarget</code> which will return the element dictated by its own <a href="#!/api/Ext.Component-cfg-maskElement" rel="Ext.Component-cfg-maskElement" class="docClass">maskElement</a>
   * Panels use <code>"el"</code> as their <a href="#!/api/Ext.Component-cfg-maskElement" rel="Ext.Component-cfg-maskElement" class="docClass">maskElement</a> by default, but that could be overridden to be <code>"body"</code> to leave toolbars and the header
   * mouse-accessible.</p>
   */
  protected native function getMaskTarget():ext.dom.Element;

  /**
   * Used by <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a>, and the <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">up</a>
   * method to find the owning Component in the linkage hierarchy.
   *
   * <p>By default this returns the Container which contains this Component.</p>
   *
   * <p>This may be overridden by Component authors who implement ownership hierarchies
   * which are not based upon ownerCt, such as BoundLists being owned by Fields or Menus
   * being owned by Buttons.</p>
   */
  protected native function getRefOwner():ext.Component;

  /**
   * Returns an object that describes how this component's width and height are managed.
   * All of these objects are shared and should not be modified.
   * @return The size model for this component.
   * <ul>
   *   <li><code>width</code> : ext.Base
   * The <a href="#!/api/Ext.layout.SizeModel" rel="Ext.layout.SizeModel" class="docClass">size model</a>
   * for the width.
   *   </li>
   *   <li><code>height</code> : ext.Base
   * The <a href="#!/api/Ext.layout.SizeModel" rel="Ext.layout.SizeModel" class="docClass">size model</a>
   * for the height.
   *   </li>
   * </ul>
   */
  protected native function getSizeModel(ownerCtSizeModel:Object):Object;

  /**
   * The initComponent template method is an important initialization step for a Component. It is intended to be
   * implemented by each subclass of <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> to provide any needed constructor logic. The
   * initComponent method of the class being created is called first, with each initComponent method
   * up the hierarchy to <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> being called thereafter. This makes it easy to implement and,
   * if needed, override the constructor logic of the Component at any step in the hierarchy.
   *
   * <p>The initComponent method <strong>must</strong> contain a call to <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">callParent</a> in order
   * to ensure that the parent class' initComponent method is also called.</p>
   *
   * <p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,
   * so you can simply access them with <code>this.someOption</code>.</p>
   *
   * <p>The following example demonstrates using a dynamic string for the text of a button at the time of
   * instantiation of the class.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('DynamicButtonText', {
   *     extend: '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>',
   *
   *     initComponent: function() {
   *         this.text = new Date();
   *         this.renderTo = <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>();
   *         this.callParent();
   *     }
   * });
   *
   * <a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a>(function() {
   *     <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('DynamicButtonText');
   * });
   * </code></pre>
   * @since 1.1.0
   */
  protected native function initComponent():void;

  /**
   * Initialize any events on this component
   */
  protected native function initEvents():void;

  /**
   * Template method to do any Focusable related initialization that
   * does not involve event listeners creation.
   */
  protected native function initFocusable():void;

  /**
   * Template method to do any event listener initialization for a Focusable.
   * This generally happens after the focusEl is available.
   */
  protected native function initFocusableEvents():void;

  /**
   * Called by <code>getInherited</code> to initialize the inheritedState the first time it is
   * requested.
   *
   * <p><strong>Overridden in Ext.rtl.Component.</strong></p>
   */
  protected native function initInheritedState(inheritedState:Object):void;

  /**
   * This method should be called when the instance is ready to start listening for
   * keyboard events. This is called automatically for <code><a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></code> and derived
   * classes. In Classic Toolkit, this is done after the component is rendered.
   */
  protected native function initKeyMap():void;

  /**
   * Initialized the renderData to be used when rendering the renderTpl.
   * @return Object with keys and values that are going to be applied to the renderTpl
   */
  protected native function initRenderData():Object;

  /**
   * Determines whether this Component is the root of a layout. This returns <code>true</code> if
   * this component can run its layout without assistance from or impact on its owner.
   * If this component cannot run its layout given these restrictions, <code>false</code> is returned
   * and its owner will be considered as the next candidate for the layout root.
   *
   * <p>Setting the <a href="#!/api/Ext.Component-property-_isLayoutRoot" rel="Ext.Component-property-_isLayoutRoot" class="docClass">_isLayoutRoot</a> property to <code>true</code> causes this method to always
   * return <code>true</code>. This may be useful when updating a layout of a Container which shrink
   * wraps content, and you know that it will not change size, and so can safely be the
   * topmost participant in the layout run.</p>
   */
  protected native function isLayoutRoot():void;

  /**
   * Checks if a particular binding is synchronizing the value.
   * @param name The name of the property being bound to.
   * @return <code>true</code> if the binding is syncing.
   */
  protected native function isSyncing(name:String):Boolean;

  /**
   * Method to manage awareness of when components are added to their
   * respective Container, firing an <a href="#!/api/Ext.Component-event-added" rel="Ext.Component-event-added" class="docClass">added</a> event. References are
   * established at add time rather than at render time.
   *
   * <p>Allows addition of behavior when a Component is added to a
   * Container. At this stage, the Component is in the parent
   * Container's collection of child items. After calling the
   * superclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,
   * and if configured with a ref, the <code>refOwner</code> will be set.</p>
   * @since 3.4.0
   * @param container Container which holds the component.
   * @param pos Position at which the component was added.
   * @param instanced <code>false</code> if this component was instanced by the parent
   * container. <code>true</code> if the instance already existed when it was passed to the container.
   */
  protected native function onAdded(container:ext.container.Container, pos:Number, instanced:Boolean):void;

  /**
   * Invoked when this component has first achieved size. This occurs after the
   * <a href="#!/api/Ext.Component-cfg-componentLayout" rel="Ext.Component-cfg-componentLayout" class="docClass">componentLayout</a> has completed its initial run.
   *
   * <p>This method is not called on components that use <a href="#!/api/Ext.Component-cfg-liquidLayout" rel="Ext.Component-cfg-liquidLayout" class="docClass">liquidLayout</a>, such as
   * <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Buttons</a> and <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Form Fields</a>.</p>
   *
   * <p><strong>Note:</strong> If the Component has a <a href="#!/api/Ext.Component-cfg-controller" rel="Ext.Component-cfg-controller" class="docClass">ViewController</a>
   * and the controller has a <a href="#!/api/Ext.app.ViewController-method-boxReady" rel="Ext.app.ViewController-method-boxReady" class="docClass">boxReady</a> method
   * it will be called passing the Component and its width and height.</p>
   *
   * <pre><code> boxReady: function (view, width, height) {
   *      // ...
   *  }
   * </code></pre>
   * @param width The width of this component
   * @param height The height of this component
   */
  protected native function onBoxReady(width:Number, height:Number):void;

  /**
   * Allows addition of behavior to the destroy operation.
   */
  protected native function onDestroy():void;

  /**
   * Allows addition of behavior to the disable operation.
   * After calling the superclass's <code>onDisable</code>, the Component will be disabled.
   */
  protected native function onDisable():void;

  /**
   * Allows addition of behavior to the enable operation.
   * After calling the superclass's <code>onEnable</code>, the Component will be enabled.
   */
  protected native function onEnable():void;

  /**
   * Called when focus enters this Component's hierarchy
   * @param e
   * @param {ext.event.Event} e.event The underlying DOM event.
   * @param {js.HTMLElement} e.target The element gaining focus.
   * @param {js.HTMLElement} e.relatedTarget The element losing focus.
   * @param {ext.Component} e.toComponent The Component gaining focus.
   * @param {ext.Component} e.fromComponent The Component losing focus.
   */
  protected native function onFocusEnter(e:ext.event.Event):void;

  /**
   * Called when focus exits from this Component's hierarchy
   * @param e
   * @param {ext.event.Event} e.event The underlying DOM event.
   * @param {js.HTMLElement} e.target The element gaining focus.
   * @param {js.HTMLElement} e.relatedTarget The element losing focus.
   * @param {ext.Component} e.toComponent The Component gaining focus.
   * @param {ext.Component} e.fromComponent The Component losing focus.
   */
  protected native function onFocusLeave(e:ext.event.Event):void;

  /**
   * Allows addition of behavior to the hide operation. After
   * calling the superclass's onHide, the Component will be hidden.
   *
   * <p>Gets passed the same parameters as <a href="#!/api/Ext.Component-event-hide" rel="Ext.Component-event-hide" class="docClass">hide</a>.</p>
   */
  protected native function onHide(animateTarget:* = undefined, callback:Function = null, scope:Object = null):void;

  /**
   * Called after the component is moved, this method is empty by default but can be implemented by any
   * subclass that needs to perform custom logic after a move occurs.
   * @param x The new x position.
   * @param y The new y position.
   */
  protected native function onPosition(x:Number, y:Number):void;

  /**
   * Method to manage awareness of when components are removed from their
   * respective Container, firing a <a href="#!/api/Ext.Component-event-removed" rel="Ext.Component-event-removed" class="docClass">removed</a> event. References are properly
   * cleaned up after removing a component from its owning container.
   *
   * <p>Allows addition of behavior when a Component is removed from
   * its parent Container. At this stage, the Component has been
   * removed from its parent Container's collection of child items,
   * but has not been destroyed (It will be destroyed if the parent
   * Container's <code>autoDestroy</code> is <code>true</code>, or if the remove call was
   * passed a truthy second parameter). After calling the
   * superclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not
   * be present.</p>
   * @since 3.4.0
   * @param destroying Will be passed as <code>true</code> if the Container performing the remove operation will delete this
   * Component upon remove.
   */
  protected native function onRemoved(destroying:Boolean):void;

  /**
   * Template method called when this Component's DOM structure is created.
   *
   * <p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not
   * been layed out (positioned and sized).</p>
   *
   * <p>Subclasses which override this to gain access to the structure at render time should
   * call the parent class's method before attempting to access any child elements of the Component.</p>
   * @param parentNode The parent Element in which this Component's encapsulating element is contained.
   * @param containerIdx The index within the parent Container's child collection of this Component.
   */
  protected native function onRender(parentNode:ext.dom.Element, containerIdx:Number):void;

  /**
   * Called when the component is resized.
   *
   * <p>This method is not called on components that use <a href="#!/api/Ext.Component-cfg-liquidLayout" rel="Ext.Component-cfg-liquidLayout" class="docClass">liquidLayout</a>, such as
   * <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Buttons</a> and <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Form Fields</a>.</p>
   * @param width The new width that was set
   * @param height The new height that was set
   * @param oldWidth The previous width
   * @param oldHeight The previous height
   */
  protected native function onResize(width:Number, height:Number, oldWidth:Number, oldHeight:Number):void;

  /**
   * Invoked when a scroll operation is completed via this component's <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scroller</a>.
   * @param x The current x position
   * @param y The current y position
   */
  protected native function onScrollEnd(x:Number, y:Number):void;

  /**
   * Invoked when this component is scrolled via its <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scroller</a>.
   * @param x The current x position
   * @param y The current y position
   */
  protected native function onScrollMove(x:Number, y:Number):void;

  /**
   * Invoked when a scroll is initiated on this component via its <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">scroller</a>.
   * @param x The current x position
   * @param y The current y position
   */
  protected native function onScrollStart(x:Number, y:Number):void;

  /**
   * Allows addition of behavior to the show operation. After
   * calling the superclass's onShow, the Component will be visible.
   *
   * <p>Override in subclasses where more complex behaviour is needed.</p>
   *
   * <p>Gets passed the same parameters as <a href="#!/api/Ext.Component-event-show" rel="Ext.Component-event-show" class="docClass">show</a>.</p>
   */
  protected native function onShow(animateTarget:* = undefined, callback:Function = null, scope:Object = null):void;

  /**
   * Invoked after the <a href="#!/api/Ext.Component-method-afterShow" rel="Ext.Component-method-afterShow" class="docClass">afterShow</a> method is complete.
   *
   * <p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href="#!/api/Ext.Component-method-afterShow" rel="Ext.Component-method-afterShow" class="docClass">afterShow</a> received.</p>
   */
  protected native function onShowComplete(callback:Function = null, scope:Object = null):void;

  /**
   * Template method to do any post-blur processing.
   * @param e The event object
   */
  protected native function postBlur(e:ext.event.Event):void;

  /**
   * Template method to do any post-focus processing.
   * @param e The event object
   */
  protected native function postFocus(e:ext.event.Event):void;

  /**
   * Publish this components state to the <code>ViewModel</code>. If no arguments are given (or if
   * this is the first call), the entire state is published. This state is determined by
   * the <code>publishes</code> property.
   *
   * <p>This method is called only by component authors.</p>
   * @since 5.0.0
   * @param property The name of the property to update.
   * @param value The value of <code>property</code>. Only needed if <code>property</code> is given.
   */
  protected native function publishState(property:String = null, value:Object = null):void;

  /**
   * Method which removes a specified UI + <code>uiCls</code> from the components element. The <code>cls</code>
   * which is added to the element will be: <code>this.baseCls + '-' + ui + uiCls</code>.
   * @param uiCls The UI class to remove from the element.
   */
  protected native function removeUIClsFromElement(uiCls:String):void;

  /**
   * Returns the default listener scope for a "satellite" of this component.
   * Used for resolving scope for observable objects that are not part of the normal
   * Container/Component hierarchy (for example, plugins)
   * @since 5.1.1
   * @return The listener scope
   */
  protected native function resolveSatelliteListenerScope(satellite:ext.mixin.Observable, defaultScope:Object = null):Object;
}
}